<?php

namespace Modules\Api\Entities;

use App\Traits\RoundingAttributesTrait;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Builder;

class Item extends Model
{
    use RoundingAttributesTrait;
    protected $table = 'items';
    protected $appends = ['max_items'];
    public $timestamps = false;
    protected $fillable = [];
    protected $casts = [
        'id' => 'integer',
        'item_name' => 'string',
        'img' => 'string',
        'sale_price' => 'string',
        'discount_percent' => 'integer',
        'details' => 'string',
    ];
    public function getImgAttribute($value)
    {
        return in_array($value, ['', ' ', null]) ? asset('uploads/' . $this->shop_id . '/Api/noImage.png') : asset($value);
    }
    public function getMaxItemsAttribute()
    {
        return $this->order_max_quantity;
        $store = RequestSetting::select('max_items')->first();
        return  $store->max_items;
    }
    public function scopeSelection($query)
    {
        return $query->select('id', 'item_name', 'order_max_quantity', 'sale_price', 'unit_id', 'withDiscount', 'discount_percent', 'details', 'about', 'img', 'shop_id')->with('details')->withCount('rates')->withCount('wishlists')->withCount('favorites');
    }
    public function type()
    {
        return $this->belongsTo(ItemType::class, 'sale_unit', 'id');
    }
    public function unit()
    {
        return $this->belongsTo(Unit::class, 'unit_id', 'id');
    }
    public function details()
    {
        return $this->hasMany(RequestDetails::class, 'item_id', 'id');
    }
    public function itemsUnit()
    {
        return $this->hasMany(ItemUnit::class, 'item_id', 'items.id');
    }
    public function storeItems()
    {
        return $this->belongsTo(StoreItem::class, 'id', 'item_id');
    }
    public function favorites()
    {
        return $this->hasMany(Favorite::class, 'product_id', 'id')
            ->where('client_id', auth()->guard('api')->check() ? auth()->guard('api')->id() : 0);
    }
    public function isFavorite()
    {

        $check = Favorite::where('product_id', $this->id)->where('client_id', auth()->guard('api')->check() ? auth()->guard('api')->id() : 0);
        if ($check->count() > 0) {
            return true;
        }
        return false;
    }
    public function isWishlist()
    {
        $check = Wishlist::where('product_id', $this->id)->where('client_id', auth()->guard('api')->check() ? auth()->guard('api')->id() : 0);
        if ($check->count() > 0) {
            return true;
        }
        return false;
    }
    public function wishlists()
    {
        return $this->hasMany(Wishlist::class, 'product_id', 'id')
            ->where('client_id', auth()->guard('api')->check() ? auth()->guard('api')->id() : 0);
    }
    public function rates()
    {
        return $this->hasMany(ItemRate::class, 'item_id', 'id');
    }
    public function images()
    {
        return $this->hasMany(ItemImage::class, 'item_id', 'id');
    }
    public function itemVats()
    {
        return $this->hasMany(ItemVat::class, 'item_id', 'id');
    }

    public function saleItemVats()
    {
        return $this->hasMany(ItemVat::class, 'item_id', 'id')->whereIn('item_vat.vat_use', [0, 1]);
    }
    public function billAdd()
    {
        return $this->belongsTo(BillAdd::class, 'vat_id')->withDefault(['addition_value' => 0]);
    }
    public function requestQuantity()
    {
        return $this->hasMany(RequestDetails::class, 'item_id', 'id')->join('requests', function ($k) {
            $k->on('requests.id', 'request_details.request_id');
            $k->where('requests.bill_id', '=', 0);
            $k->where('requests.client_id', '=', auth('api')->id());
            $k->where('requests.status', '!=', 0);
        })->sum('quantity');
    }
    public function maxOrder($unit_id)
    {
        $settings = RequestSetting::getSettings($this->shop_id);
        $quantity = Item::leftjoin('items_unit', function ($q) use ($unit_id) {
            $q->where('items_unit.item_id', $this->id);
            $q->where('items_unit.unit_id', $unit_id);
        })->select('items_unit.unit_value', 'items_unit.unit_price')->first();
        
        \Log::info('Request Qty : ' . $this->requestQuantity() . ' --------- Store_qty : ' . $this->storeItems()->sum('store_quant'));
        if ($quantity->unit_value == "0.00") {
            if ($settings->ignore_quantity == '1') {
                $availableQuantity = $this->storeItems()->sum('store_quant');
            } else {
                $availableQuantity = $this->storeItems()->sum('store_quant') - $this->requestQuantity();
            }
        } else {
            if ($settings->ignore_quantity == '1') {
                $availableQuantity = $this->storeItems()->sum('store_quant') / $quantity->unit_value;
            } else {
                $availableQuantity = ($this->storeItems()->sum('store_quant') / $quantity->unit_value) - $this->requestQuantity();
            }
        }
        \Log::info('availableQuantity : ' . $availableQuantity);
        return $availableQuantity;
    }

    public function ItemPrice($unit_id)
    {
        $item_price = Item::leftjoin('items_unit', function ($q) use ($unit_id) {
            $q->where('items_unit.item_id', $this->id);
            $q->where('items_unit.unit_id', $unit_id);
        })->select('items_unit.unit_price')->first();
        if ($item_price->unit_price != "0.00") {
            $sale_price = $item_price->unit_price;
        }
        else
        {
            $sale_price = $this->sale_price;
        }
        return $sale_price;
    }
    public function units()
    {
        $main =Item::join("units",function($q)
        {
            $q->where('items.id',$this->id);
            $q->where('units.id', $this->unit_id);
        })->select('units.id', 'units.name','items.sale_price','items.discount_percent')->first();
        $all_units = collect();
        $all_units->push($main);
        $sub_unit = ItemUnit::where('item_id', $this->id)
            ->join('items', function ($join) {
                $join->on('items.id', 'items_unit.item_id');
            })
            ->join('units', function ($join) {
                $join->on('units.id', 'items_unit.unit_id');
            })->select('units.id', 'units.name','items_unit.unit_price','items.discount_percent')->get();

        $sub_unit->each(function ($s_unit) use ($all_units) {
            $all_units->push($s_unit);
        });
        return $all_units;
    }
    
    public function allUnits()
    {
        return $this->belongsToMany(Unit::class, 'items_unit')->withPivot(['unit_value', 'unit_price', 'unit_barcode']);
    }
    
    public function getUnits()
    {
        $units = collect();

        $units->push((object) [
            'id' => $this->unit->id,
            'name' => $this->unit->name,
            'unit_price' => $this->sale_price,
            'discount_percent' => $this->discount_percent,
            'qty' => $this->store_items_sum_store_quant,
            'unit_value' => 1,
        ]);
        $this->allUnits->each(function ($unit) use ($units) {
            $unit_value = $unit->pivot->unit_value ?? 1;
            $unit_value = $unit_value == 0 ? 1 : $unit_value;
            $units->push((object) [
                'id'    => $unit->id,
                'name'  => $unit->name,
                'unit_price' => $unit->pivot->unit_price,
                'unit_value' => $unit->pivot->unit_value,
                'qty'   => $this->store_items_sum_store_quant / $unit_value,
                'discount_percent' => $this->discount_percent,
            ]);
        });
        
        return $units;
    }
    
    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('onlyOnline', function (Builder $builder) {
            $builder->withSum('storeItems', 'store_quant');
        });
    }
}
