<?php

namespace Modules\Api\Entities;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Notifications\Notifiable;
class Request extends Model
{
    use HasFactory;
    use Notifiable;
    protected $table="requests";
    protected $guarded = [];
    protected $casts = [
        'id' =>'integer',
        'created_at'=>'string',
        'status'=>'integer',
        'order_no'=>'integer',
        'installment_dates' => 'array',
    ];
     public function scopeSelection($query)
    {
        return $query->select('id','total','net','created_at','status','order_no','discount','fee','mobile','client_name','delivery_date')->with('totalVat')->with('details')->with('items');
    }
    public function details()
    {
        return $this->hasMany(RequestDetails::class, 'request_id', 'id');
    }
    public function totalDiscount()
    {
        return $this->hasMany(RequestDetailDiscount::class, 'request_id', 'id');
    }
    public function totalDiscountCoupon()
    {
        return $this->hasMany(RequestDetailDiscount::class, 'request_id', 'id')->where('discount_type','1');
    }
    public function totalVat()
    {
        return $this->hasMany(RequestDetailVat::class, 'request_id', 'id');
    }
    public function items()
    {
        return $this->hasManyThrough(
            Item::class,
            RequestDetails::class,
            'request_id', // Foreign key on the environments table...
            'id', // Foreign key on the deployments table...
            'id', // Local key on the projects table...
            'item_id' // Local key on the environments table...
        );
    }
    public function client(): BelongsTo
    {
        return $this->belongsTo(Client::class, 'client_id', 'id');
    }
}
