<?php

namespace Modules\Api\Http\Controllers;

use App\Notifications\ClientNotify;
use App\Notifications\LoginNotify;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Validator;
use Modules\Api\Entities\Client;
use Modules\Api\Http\Requests\ForgetpasswordRequest;
use Modules\Api\Mail\SendCodeResetPassword;
use Modules\Api\Transformers\ClientResource;
use Laravel\Passport\HasApiTokens;
use Modules\Api\Http\Requests\LoginRequest;
use App\Traits\Api\ApiTrait;
use Modules\Api\Http\Requests\RegisterRequest;
use Illuminate\Http\Request;
use Modules\Api\Http\Requests\ChangepasswordRequest;
use Modules\Api\Http\Requests\CreatepasswordRequest;
use Modules\Api\Http\Requests\ProfileRequest;
use App\Traits\HelperTrait;
use Carbon\Carbon;

class ApiAuthContollerController extends Controller
{

    use HelperTrait;
    public function login(LoginRequest $request, $shop_id)
    {
        try {
            $client = Client::where(['shop_id' => $shop_id])->where(function ($q) use ($request) {
                $q->whereEmail($request->user_name)
                    ->orWhere('user_name', $request->user_name)
                    ->orWhere('mobile1', $request->user_name);
            })->first();
            if ($client) {
                if (!Hash::check(request('password'), $client->password)) {
                    return $this->responseJson('false', trans('api.pass_wrong'), null, null, 401);
                }
                $client->update([
                    'player_id' => $request->player_id
                ]);
                $client = ClientResource::make($client);
                $data = [
                    'msg'=>trans('api.welcome')
                ];
                $client->notify(new ClientNotify($data));

                return $this->responseJson(true, trans('api.login'), $client, $client->createToken("API TOKEN")->accessToken, 200);
            } else {
                return $this->responseJson(false, trans('api.no_account'), null, null, 401);
            }
        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }
    public function register(RegisterRequest $request, $shop_id)
    {
        try {
            $client = Client::where('shop_id', $shop_id)->where(function ($q) use ($request) {
                $q->whereEmail($request->email)
                    ->orWhere('user_name', $request->user_name)
                    ->orWhere('mobile1', $request->mobile1);
            })->first();
            if ($client) {
                return $this->responseJson(false, trans('api.register_fail'), null, null, 401);
            } else {
                if ($request->file('image')) {
                    $image = $this->saveImage($request->file('image'), 'uploads/' . $shop_id . '/Api/profile');
                }
                $client = new Client([
                    'user_name' => $request['user_name'],
                    'client_name' => $request['user_name'],
                    'email' => $request['email'],
                    'password' => hash::make($request['password']),
                    'mobile1' => $request['mobile1'],
                    'shop_id' => $shop_id,
                    'player_id' => $request['player_id'],
                    'image' => $image ?? NULL,
                ]);
                $client->save();
                $client = ClientResource::make($client);
                if ($client) {
                    $data = [
                        'msg'=>trans('api.thanks')
                    ];
                    $client->notify(new ClientNotify($data));
                    return $this->responseJson(true, trans('api.register'), $client, $client->createToken("API TOKEN")->accessToken, 200);
                }
            }
        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }
    
    public function forgotPassword(Request $request, $shop_id)
    {
        try {
            $validation=Client::where(['email'=>$request->email,'shop_id'=>$shop_id])->first();
            if($validation)
            {
                Client::where('email', $request->email)->update([
                    'active_code' => null
                ]);
                $client = Client::where(['shop_id' => $shop_id, 'email' => $request->email])->first();
                $updates = $client->update([
                    'active_code' => mt_rand(1000, 9999),
                    'updated_at' => Carbon::now(),
                ]);

                // return response()->json([
                //     'status' => true,
                //     'message' => 'user code save successfully',
                //     'code' => $client->active_code
                // ], 200);
                $data = [
                    'code' => $client->active_code
                ];
                // Mail::send('api::index',['data'=>$data], function ($message) use ($request) {
                //     $message->to($request->email)->subject('Welcome!');
                // });
                
                $header = "From:info@$_SERVER[HTTP_HOST] \r\n";
                $header .= "Cc:info@$_SERVER[HTTP_HOST] \r\n";
                $header .= "MIME-Version: 1.0\r\n";
                $header .= "Content-type: text/html\r\n";
                
                mail($request->email, 'Welcome!', view('api::index',['data'=>$data])->render(), $header);
                // Mail::to($request->email)->send(new SendCodeResetPassword);
                return response(['message' => trans('passwords.sent')], 200);
            }else{
                return response(['message' => trans('api.fail_email')], 200);
            }



        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }
    public function CreatePassword(CreatepasswordRequest $request, $shop_id)
    {
        try {
            $check = Client::where(['email' => $request->email, 'active_code' => $request->active_code, 'shop_id' => $shop_id])->first();
            if ($check->updated_at > now()->addHour()) {
                $check->delete();
                return $this->responseJson(false, trans('passwords.code_is_expire'), null, null, 422);
            } elseif ($check) {
                $check->update(['password' => bcrypt($request->password)]);
                $client = ClientResource::make($check);
                return $this->responseJson(true, trans('api.pass_rest'), $client, $check->createToken("API TOKEN")->accessToken, 200);
            } else {
                return $this->responseJson(false, trans('api.active_code_exist'), null, null, 500);
            }
        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }
    public function ChangePassword(ChangepasswordRequest $request, $shop_id)
    {
        try {
            if (auth('api')->check()) {
                $check = Client::where(['id' => auth()->guard('api')->id(), 'shop_id' => $shop_id])->first();
                if ($check) {
                    if (!Hash::check(request('old_password'), $check->password)) {
                        return $this->responseJson(false, 'Password is wrong.', null, null, 401);
                    }
                    $check->update(['password' => Hash::make($request->password)]);
                    return $this->responseJson(true, 'تم تغيير كلمة المرور بنجاح', null, $check->createToken("API TOKEN")->accessToken, 200);
                }
            } else {
                return $this->responseJson(false, 'Unauthorized', null, null, 401);
            }
        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }
    public function getProfile(Request $request, $shop_id)
    {
        try {
            if (auth('api')->check()) {
                $profileData = Client::where(['id' => auth()->guard('api')->id(), 'shop_id' => $shop_id])->selection()->first();
                $profileData = ClientResource::make($profileData);
                return $this->responseJson(true, 'Profile Data', $profileData, null, 200);
            } else {
                return $this->responseJson(false, trans('api.Unauthorized'), null, null, 401);
            }
        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }
    public function updateProfile(ProfileRequest $request, $shop_id)
    {
        try {
            if (auth('api')->check()) {
                $updates = auth()->guard('api')->user();
                if ($request->filled('user_name')) {
                    $updates->client_name = $request->user_name;
                }
                if ($request->filled('email')) {
                    $updates->email = $request->email;
                }
                if ($request->filled('mobile1')) {
                    $updates->mobile1 = $request->mobile1;
                }
                if ($request->file('image')) {
                    $file_name = $this->updateImage($request->file('image'), 'uploads/' . $shop_id . '/Api/profile', $updates->image);
                    $updates->image = $file_name;
                }
                $updates->save();
                if ($updates) {

                    return $this->responseJson(true, trans('api.update_suc'), ClientResource::make($updates), null, 200);
                } else {
                    return $this->responseJson(false, null, null, null, 401);
                }
            } else {
                return $this->responseJson(false, trans('api.Unauthorized'), null, null, 401);
            }
        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }
    public function logout(Request $request, $shop_id)
    {

        try {
            if (auth('api')->check()) {
                $client = Client::where(['id' => auth()->guard('api')->id(), 'shop_id' => $shop_id])->first();
                $client->update(['player_id' => null]);
                foreach (auth()->guard('api')->user()->tokens as $accessToken) {
                    $refreshToken = \DB::table('oauth_refresh_tokens')
                        ->where('access_token_id', $accessToken->id)
                        ->update(['revoked' => TRUE]);
                    $accessToken->revoke();
                }
                return $this->responseJson(true, trans('api.logout'), null, null, 200);
            } else {
                return $this->responseJson(false, trans('api.Unauthorized'), null, null, 401);
            }
        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }

    public function Unauthorized()
    {
        return $this->responseJson(false, trans('api.Unauthorized'), null, null, 401);
    }
    
    protected function getNewSlug()
    {
        $user = auth()->guard('api')->user();
        $newSlug = rand() . 'deleted-';
        if (Client::where('tele', $newSlug . $user->tele)->first()) {
            $newSlug = $this->getNewSlug();
        }
        return $newSlug;
    }


    public function deleteAccount()
    {
        $user = auth()->guard('api')->user();

        if ($user) {
            $slug = $this->getNewSlug();
            
            $user->email = $slug . $user->email;
            $user->tele = $slug . $user->tele;
            // $user->shop_id = 0;
            $user->save();

            return response()->json([
                'status' => true,
                'message' => 'Done!',
                'logout' => true
            ]);
        }

        return response()->json([
            'status' => false,
            'message' => 'Error!'
        ]);
    }
}
