<?php

namespace Modules\Api\Http\Controllers;

use App\Traits\Api\ApiTrait;
use App\Traits\HelperTrait;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Api\Entities\BadrShop;
use Modules\Api\Entities\City;
use Modules\Api\Entities\Contact;
use Modules\Api\Entities\ContactMessage;
use Modules\Api\Http\Requests\ContactMessageRequest;
use Modules\Api\Transformers\AllStaticPageResource;
use Modules\Api\Transformers\AreaResource;
use Modules\Api\Transformers\BadrShopResource;
use Modules\BadrRequest\Entities\Page;

class ApiBadrshopController extends Controller
{
    use HelperTrait;
    public function badrShop(Request $request ,$shop_id)
    {
        try {
            $badrshop = BadrShop::where(['serial_id' => $shop_id])->selection()->first();
            $badrshop = BadrShopResource::make($badrshop);
            return $this->responseJson(true, trans('api.badrshop_data'), $badrshop, null, 200);
        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }
    public function contactUs(ContactMessageRequest $request ,$shop_id)
    {
        try{
            $contactMessages = ContactMessage::create([
                'client_id' => auth()->guard('api')->id() ? auth()->guard('api')->id() : 0,
                'ticket_reason_id'=>0,
                'ticket_type_id'=>0,
                'ticket_status_id'=>0,
                'employee_id'=>0,
                'shop_id' => $shop_id,
                'phone' =>$request->phone,
                'email' => $request->email,
                'description' => $request->message,
                'title' => $request->title,
                'name' => $request->name,
                'type'=>0,
            ]);
            $contactMessages->save();
            return $this->responseJson(true,trans('api.send_suc'),null,null,200);
        }catch(\Throwable $th) {
            return $this->responseJson(false,$th->getMessage(),null,null,500);
        }
    }
    public function allAreas(Request $request ,$shop_id)
    {
        $areas=City::where(['shop_id'=>$shop_id ,'city_up'=>'0'])->selection()->get();
        $areas=AreaResource::collection($areas);
        return $this->responseJson(true,'Area.... ',$areas,null,200);
    }
    public function getCity(Request $request ,$shop_id)
    {
        $cities=City::where(['shop_id'=>$shop_id , 'city_up'=>$request->area_id])->select('city_name')->get();
        return $this->responseJson(true,'Area.... ',$cities,null,200);
    }
    public function allStaticPages(Request $request, $shop_id)
    {
        $staticPages = Page::where('shop_id', $shop_id)->selection()->get();
        $staticPages = AllStaticPageResource::collection($staticPages);
        return $this->responseJson(
            true,
            trans('api.static_page'),
            $staticPages,
            null,
            200
        );
    }
    public function getStaticPage(Request $request, $shop_id)
    {
        $staticPage = Page::where(['shop_id' => $shop_id, 'id' => $request->id])->selection()->first();
        return $this->responseJson(
            true,
            trans('api.static_page'),
            [
                'id' => $staticPage->id,
                'title' => $staticPage->title,
                'image' => $staticPage->image,
                'text' => $staticPage->text,
                'english_text' => $staticPage->english_text,
                'created_at' => $staticPage->created_at,

            ],
            null,
            200
        );
    }
}
