<?php

namespace Modules\Api\Http\Controllers;

use App\Notifications\OrderNotify;
use App\Traits\CartTrait;
use App\Traits\HelperTrait;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Modules\Api\Entities\BillAdd;
use Modules\Api\Entities\Item;
use Modules\Api\Entities\ItemUnit;
use Modules\Api\Entities\ItemVat;
use Modules\Api\Entities\Request as EntitiesRequest;
use Modules\Api\Entities\RequestCoupon;
use Modules\Api\Entities\RequestDetailDiscount;
use Modules\Api\Entities\RequestDetails;
use Modules\Api\Entities\RequestDetailVat;
use Modules\Api\Entities\RequestSetting;
use Modules\Api\Http\Requests\CardRequest;
use Modules\Api\Transformers\OrderResource;

class ApiCartController extends Controller
{
    use HelperTrait, CartTrait;

    public function getCart(Request $request, $shop_id) // Get Current Opened Card
    {
        $order = EntitiesRequest::where(['shop_id' => $shop_id, 'id' => $request->order_no, 'client_id' => auth()->guard('api')->id(), 'cart' => '0'])->selection()->first();
        return $order
                ? $this->responseJson(true, 'List of Cart', OrderResource::make($order), null, 200)
                : $this->responseJson(false, trans('api.wrong_order'), null, null, 401);
    }

    public function deleteFromCart(Request $request, $shop_id) // Delete Specific Item From Current Opened Card
    {
        try {
            DB::beginTransaction();
                Log::info(request()->getUri());
                Log::info("Delete Item | request : " . json_encode($request->all()));
                $detail = RequestDetails::where(['request_id' => $request->order_id, 'id' => $request->id])->first();
                if (! $detail) {
                    throw new Exception("هذا الصنف غير موجود في الطلب");
                }
                RequestDetailDiscount::where(['request_id' => $request->order_id, 'detail_id' => $request->id])->delete();
                RequestDetailVat::where(['request_id' => $request->order_id, 'detail_id' => $request->id])->delete();
                $detail->delete();
                $this->updateTotals($shop_id, $request, $detail->request_id);
                $order = EntitiesRequest::where(['shop_id' => $shop_id, 'client_id' => auth()->guard('api')->id(), 'id' => $request->order_id])->selection()->first();
                $order = OrderResource::make($order);
            DB::commit();
            return $this->responseJson(true, trans('api.item_del_suc'), $order, null, 200);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }

    protected function getOrder($shop_id, $request)
    {
        \Log::info("Header : " . request()->header('Authorization'));
        $row = EntitiesRequest::where(['shop_id' => $shop_id, 'client_id' =>  auth()->guard('api')->id()])
                            ->when($request->has('order_id') && $request->order_id != 0, function($query) use($request) {
                                $query->where('id', $request->order_id);
                            }, function($query) use($request) {
                                $query->where('cart', 0);
                            })
                            ->orderBy('id', 'DESC')->first();
                            
        if (! $row) {
            $row = EntitiesRequest::create([
                'created_at' => Carbon::now(),
                'order_no' => EntitiesRequest::where('shop_id', $shop_id)->max('order_no') + 1,
                'client_id' => auth()->guard('api')->id(),
                'client_name' => auth()->guard('api')->user()->user_name,
                'email' => auth()->guard('api')->user()->email,
                'mobile' => auth()->guard('api')->user()->mobile1,
                'shop_id' => $shop_id,
                'cart' => '0',
                'status' => '1',
            ]);
        }

        return $row;
    }

    protected function checkItemDoublicate($detail, &$request)
    {
        if ($request->get('id') && $detail) {
            $rows = RequestDetails::select('id', 'quantity')->where(['shop_id' => $detail->shop_id, 'request_id' => $detail->request_id, 'item_id' => $detail->item_id, 'unit_id' => $request->unit_id])->where('id', '<>', $detail->id)->get();
            if ($rows->count() == 0) return ;

            $request->quantity += $rows->sum('quantity');
            $ids = $rows->pluck('id')->toArray();

            RequestDetailDiscount::where('request_id', $detail->request_id)->whereIn('detail_id', $ids)->delete();
            RequestDetailVat::where('request_id', $detail->request_id)->whereIn('detail_id', $ids)->delete();
            RequestDetails::where('request_id', $detail->request_id)->whereIn('id', $ids)->delete();
        }
    }
    
    protected function getActualItemQuantity($shop_id, $item_id, $unit_id)
    {
        $unit_value = DB::select("SELECT unit_value FROM `items_unit` WHERE shop_id={$shop_id} AND item_id={$item_id} AND unit_id={$unit_id} LIMIT 1;");
        
        if ( isset($unit_value[0]) ) {
            $unit_value = $unit_value[0]->unit_value;
        } else {
            $unit_value = 1;
        }
        \Log::info("----------------------------- All Item Unit Value Is : $unit_value -----------------------------");
        
        $request_quantity = request()->get('id', null) ? request()->get('quantity', 0) : 0;
        $item_quantity = DB::select("SELECT sum(store_quant) AS quantity FROM store_items WHERE shop_id={$shop_id} AND item_id={$item_id} LIMIT 1;");
        $actual_item_quantity = $actual_user_item_quantity = 0;
        if ( isset($item_quantity[0]) ) {
            $actual_item_quantity = ($request_quantity + $item_quantity[0]->quantity) / $unit_value;
        }
        \Log::info("----------------------------- All Item Quantity Is : $actual_item_quantity -----------------------------");
        $user_item_quantity = DB::select("SELECT SUM(request_details.quantity * IFNULL(items_unit.unit_value, 1)) AS quantity FROM request_details
                                    	INNER JOIN requests ON requests.id=request_details.request_id
                                        left JOIN items_unit ON items_unit.unit_id=request_details.unit_id AND items_unit.item_id=request_details.item_id
                                       	WHERE request_details.shop_id={$shop_id}
                                        	AND request_details.item_id={$item_id} AND requests.client_id=".auth('api')->id()."
                                            AND requests.status != 0 AND requests.shop_id={$shop_id} AND requests.bill_id=0 LIMIT 1;");
        if ( isset($user_item_quantity[0]) ) {
            $actual_user_item_quantity = $user_item_quantity[0]->quantity / $unit_value;
        }
        \Log::info("----------------------------- All User Item Quantity Is : $actual_user_item_quantity -----------------------------");
        return $actual_item_quantity + $request_quantity - $actual_user_item_quantity;
    }

    public function addToCart(CardRequest $request, $shop_id)
    {
        \Log::info( request()->getUri() );
        \Log::info(  json_encode(request()->all()) );
        
        $settings = RequestSetting::getSettings($shop_id);
        
        $item = Item::find($request->item_id);
        $settings->max_items = $item->max_items;
        
        $order = $this->getOrder($shop_id, $request);

        $request_id = $order->id;
        $checkitem = RequestDetails::where(['shop_id' => $shop_id])
                                    ->where('request_id', $request_id)
                                    ->when($request->get('id'), fn($query, $id) => $query->where('id', $id))
                                    ->when(! $request->get('id'), fn($query) => $query->where(['item_id' =>  $request->item_id, 'unit_id' =>  $request->unit_id]))
                                    ->first();
                                    
        $this->checkItemDoublicate($checkitem, $request);
        $item = Item::find($request->item_id);
        $unit_id = $request->unit_id ?? $item->unit_id;
        
        $all_qty = ($checkitem->quantity ?? 0) + $request->quantity;
        if ($item->order_max_quantity > 0 && $item->order_max_quantity < $all_qty) {
            return $this->responseJson(false, trans('api.max_order') . $item->order_max_quantity, null, null, 401);
        }

        $check_qty = $this->getActualItemQuantity($shop_id, $request->item_id, $request->unit_id);
        // $check_qty = $item->maxOrder($unit_id) + ($checkitem->quantity ?? 0);

        \Log::info("unit qty " . $check_qty . " |  request  qty : " . $request->quantity . " | item qty :  " . ($checkitem->quantity ?? 0));
        if ($check_qty < $request->quantity) {
            return $this->responseJson(false, trans('api.max_qty') . $check_qty, null, null, 401);
        }

        if (! $request->get('id') && $checkitem) {
            $request->merge(['id' => $checkitem->id, 'quantity' => $request->quantity + $checkitem->quantity]);
        }

        if ($checkitem) {
            if ($request->quantity != $checkitem->quantity || $request->unit_id != $checkitem->unit_id) {
                $originalPrice = $unit_id == $item->unit_id
                                    ? $item->sale_price
                                    : ItemUnit::where('item_id', $item->id)->where('unit_id', $unit_id)->select('unit_price')->value('unit_price');
                $this->calcDiscount($item, $originalPrice, $shop_id, $order->id, $request->quantity, $request);
            }
            auth()->guard('api')->user()->notify(new OrderNotify(['msg' => trans('api.cart_update')]));
        }

        $this->calcDetail($item, $request->quantity,  $order->id, $shop_id, $request);
        // $this->updateItemPrices($shop_id, $order);

        Log::info("REQuest ID : $request_id");
        $order = $this->updateTotals($shop_id, $request, $request_id);
        $order = OrderResource::make($order);
        Log::info('Response : ' . json_encode($order));
        return $this->responseJson(true, 'List Of Item Details', $order, null, 200);
    }

    protected function updateTotals($shop_id, $request, $order_id=0, $cart=0)
    {
        if ($order_id) {
            $total = DB::table('request_details')->where('shop_id', $shop_id)->where('request_id', $order_id)->sum(DB::raw('(request_details.price ) * request_details.quantity'));
            $vats = DB::table('requests_details_vats')->where('shop_id', $shop_id)->where('request_id', $order_id)->sum(DB::raw('requests_details_vats.vat_weight '));
            $discounts = DB::table('requests_details_discounts')->where('shop_id', $shop_id)->where('request_id', $order_id)->sum(DB::raw('requests_details_discounts.discount_weight'));
            $fee = $this->CalcFee($request, $total);
            DB::table('requests')->where('id', $order_id)->update([
                'total' => $total,
                'discount' => $discounts,
                'fee' => $fee,
                'net' => $total + $fee + $vats
            ]);
        }
        return EntitiesRequest::where(['shop_id' => $shop_id, 'client_id' => auth()->guard('api')->id()])
                        ->when($order_id, function($query) use($order_id) {
                            $query->where('id', $order_id);
                        })->when(!$order_id, function($query) use($cart) {
                            $query->where('cart', $cart);
                        })->selection()->first();
    }

    protected function updateItemPrices($shop_id, $request)
    {
        $checkCart = RequestDetails::where(['shop_id' => $shop_id, 'request_id' => $request?->id])->get();
        $total = 0;
        foreach($checkCart as $detail) {
            $total += ($detail->sale_price * $detail->quantity);
        }

        foreach($checkCart as $detail) {
            if ($request->discount_type == 0) {
                $request_value = (($detail->sale_price * $request->discount) / 100);
            } else {
                $request_value = 0;
                if ((float) $request->discount > 0) {
                    $request_percent = (float) $request->discount / $total;
                    $request_value = $detail->sale_price * $request_percent;
                }
            }

            $originalPrice = $detail->sale_price - $request_value;

            $item_discount = RequestDetailDiscount::where('item_id', $detail->item_id)->where('detail_id', $detail->id)->where('shop_id', $shop_id)->where('unit_id', $detail->unit_id)->first();

            if ($item_discount) {
                DB::statement("UPDATE `requests_details_discounts` SET `discount_type`='{($request->discount_type ?? 0)}', `discount_value`='{$request_value}', `discount_weight`='".($request_value * $detail->quantity)."'
                                WHERE item_id={$detail->item_id} AND detail_id={$detail->id} AND shop_id={$shop_id} AND unit_id={$detail->unit_id};");
            } else {
                RequestDetailDiscount::create([
                    'item_id' => $detail->item_id,
                    'detail_id' => $detail->id,
                    'request_id' => $request->id,
                    'shop_id' => $shop_id,
                    'discount_type' =>  $request->discount_type ?? 0,
                    'discount_value' => $request_value,
                    'discount_weight' => $request_value * $detail->quantity,
                    'unit_id' => $detail->unit_id,
                ]);
            }

            $item = Item::select('id', 'vat_state')->find($detail->item_id);

            $vat = 0;
            if (in_array($item->vat_state, [1, 2])) {
                $itemVat = ItemVat::where('item_id', $item->id)->whereIn('vat_use', [0, 1])->with('billAdd')->first();
                if ($itemVat) {
                    // here we will apply vats on original price
                    $vat = $originalPrice - ($originalPrice / (1 + $itemVat->billAdd->addition_value / 100));
                }

                if ($item->vat_state == 2) {
                    $originalPrice -= $vat;
                }
            }

            if ($vat) {
                $vatdetails = RequestDetailVat::where(['shop_id' => $shop_id, 'item_id' => $detail->item_id, 'request_id' => $request->request_id])->get();
                foreach ($vatdetails as $vatdetail) {
                    $vatdetail->update([
                        'vat_value' => BillAdd::select('addition_value')->where('shop_id', $shop_id)->where('id', $vatdetails->vat_id)->first()->addition_value,
                        'vat_weight' => $vat  * $detail->quantity,
                    ]);
                }
            }
            $detail->update([
                'price' => $originalPrice,
                'updated_at' => Carbon::now()
            ]);
        }
    }

    public function calcDetail($item, $quantity, $request_id, $shop_id, $request)
    {
        $unit_id = $request->unit_id ? $request->unit_id : $item->unit_id;
        if ($unit_id == $item->unit_id) {
            $originalPrice = $item->sale_price;
        } else {
            $unit = ItemUnit::where('item_id', $item->id)
                ->where('unit_id', $unit_id)
                ->select('unit_value', 'unit_price')->first() ?? new ItemUnit();
            $originalPrice = $unit->unit_price ?? $item->sale_price;
        }
        $vat = 0;
        if (in_array($item->vat_state, [1, 2])) {
            if ($item->vat_state == 2) {
                $vat = $originalPrice - ($originalPrice / ($item->itemVats()->first()->billAdd->addition_value / 100));
            }
        }
        $this->calcDiscount($item, $originalPrice, $shop_id, $request_id, $quantity, $request, $vat);
    }

    public function calcDiscount($item, $originalPrice, $shop_id, $request_id, $quantity, $request, $vat = 0)
    {
        Log::info("calcDiscount   =>   VAT : $vat  ");
        $discount = 0;

        // $client_discount = DB::table('clients_groups')->select('group_discount')->where(['id' => auth()->guard('api')->user()->group_id, 'shop_id' => $shop_id])->value('group_discount') ?? 0;
        // if ($client_discount > 0) {
        //     $discount += (($originalPrice * $client_discount) / 100);
        //     $originalPrice = $originalPrice - (($originalPrice * $client_discount) / 100);
        // }

        if ($item->withDiscount == 1) {
            $price = $originalPrice - $vat;
            $discount += (($price * $item->discount_percent) / 100);
            $originalPrice = $price - (($originalPrice * $item->discount_percent) / 100);
        }

        $this->calcVat($item, $originalPrice, $shop_id, $request_id, $discount, $quantity, $request);
    }

    public function calcVat($item, $originalPrice, $shop_id, $request_id, $discount, $quantity, $request)
    {
        $vat = 0;
        if (in_array($item->vat_state, [1, 2])) {
            $itemVat = ItemVat::where('item_id', $item->id)->whereIn('vat_use', [0, 1])->with('billAdd')->first();
            Log::info( "itemVat : " . json_encode($itemVat) );

            if ($item->saleItemVats->count() > 0) {
                if ($item->vat_state == 2) {
                    $vat = $originalPrice - ($originalPrice / (1 + $itemVat->billAdd->addition_value / 100));
                } elseif ($item->vat_state == 1) {
                    $vat = ($originalPrice * (1 + $itemVat->billAdd->addition_value / 100)) - $originalPrice;
                }
                Log::info("calcVat   =>   originalPrice : $originalPrice  |  addition_value : {$itemVat->billAdd->addition_value}");
                if ($item->vat_state == 2) {
                    $originalPrice -= $vat;
                }
            }
        }
        $this->calcUpdatedetails($originalPrice, $vat, $item, $quantity, $request_id, $shop_id, $request, $discount);
    }

    public function calcUpdatedetails($originalPrice, $vat, $item, $quantity, $request_id, $shop_id, $request, $discount)
    {
        $checkitem = $this->getRow($originalPrice, $item, $quantity, $request_id, $shop_id, $request);

        $discounts = RequestDetailDiscount::where(['detail_id' => $checkitem->id, 'request_id'  => $request_id])->first();
        $discount_weight = 0;
        $price = $originalPrice;
        if ($discounts) {
            $price = $originalPrice - ($discounts->discount_value * $originalPrice / 100);
            $discount_weight = $discounts->discount_value  * $quantity;
            $discounts->update([
                'discount_weight' => $discount_weight,
                'updated_at' => Carbon::now(),
            ]);
        } else if ($discount != 0) {
            RequestDetailDiscount::create([
                'item_id' => $item->id,
                'request_id' => $request_id,
                'detail_id' => $checkitem->id,
                'shop_id' => $shop_id,
                'discount_type' =>  0,
                'discount_value' => $discount,
                'discount_weight' => $discount * $quantity,
                'unit_id' => $request->unit_id ?? $item->unit_id,
            ]);
        }

        $itemVats = RequestDetailVat::where(['detail_id' => $request->id, 'request_id'  => $request_id])->get();
        if ($itemVats->count() > 0) {
            foreach ($itemVats as $itemVat) {
                $itemVat->update([
                    'vat_weight' => $vat * $quantity,
                    'updated_at' => Carbon::now(),
                ]);
            }
        } else if ($vat != 0) {
            foreach ($item->saleItemVats as $saleItemVat) {
                RequestDetailVat::create([
                    'request_id' => $request_id,
                    'detail_id' => $checkitem->id,
                    'item_id' => $saleItemVat->item_id,
                    'vat_id' => $saleItemVat->vat_id,
                    'vat_operation' => $saleItemVat->billAdd->check_addition,
                    'vat_type' => $saleItemVat->vat_type,
                    'vat_value' => BillAdd::select('addition_value')->where('shop_id', $shop_id)->where('id', $saleItemVat->vat_id)->first()->addition_value,
                    'vat_weight' => $vat * $quantity,
                    'shop_id' => $shop_id,
                    'unit_id' =>  $request->unit_id ?? $item->unit_id,
                ]);
            }
        }

        $checkitem->update([
            'quantity' => $quantity,
            'price' => $price,
            'sale_price' => $originalPrice,
            // 'sale_price' => ($request->unit_id == $item->unit_id) ? $item->sale_price : $originalPrice,
            'discount_percent' => $item->discount_percent ?? 0,
            'shop_id' => $shop_id,
            'notes' => $request->notes,
            'unit_id' => $request->unit_id ?? $item->unit_id,
            'updated_at' => Carbon::now(),
        ]);
    }

    protected function getRow($originalPrice, $item, $quantity, $request_id, $shop_id, $request)
    {
        return RequestDetails::updateOrCreate(['id' => $request->id, 'shop_id' => $shop_id],[
                'created_at' => Carbon::now(),
                'item_id' => $item->id,
                'quantity' => $quantity,
                'request_id' => $request_id,
                'price' => $originalPrice,
                'sale_price' => $originalPrice,
                // 'sale_price' => ($request->unit_id == $item->unit_id) ? $item->sale_price : $originalPrice,
                'discount_percent' => $item->discount_percent ?? 0,
                'shop_id' => $shop_id,
                'unit_id' => $item->unit_id,
                'notes' => $request->notes,
                'unit_id' => $request->unit_id ?? $item->unit_id,
        ]);
    }

    public function applyCoupon(Request $request, $shop_id)
    {
        $coupon = RequestCoupon::Where(['shop_id' => $shop_id, 'is_valid' => '1', 'coupon_code' => $request->coupon])->whereDate('expire_date', '>=', date('Y-m-d'))->selection()->first();
        if (! $coupon) {
            return $this->responseJson(false, 'Coupon Invalid', null, null, 500);
        }

        $order = EntitiesRequest::where(['shop_id' => $shop_id, 'id' => $request->request_id])->first();

        foreach ($order->details as $detail) {
            if (RequestDetailDiscount::where(['item_id' => $detail->item_id, 'detail_id' => $detail->id, 'request_id' => $request->request_id, 'shop_id' => $shop_id])->count() > 0) {
                continue;
            }

            if ($coupon->discount_type == 0) {
                $coupon_value = (($detail->price * $coupon->discount_value) / 100);
            } else {
                $coupon_percent = $coupon->discount_value / $order->total;
                $coupon_value = $detail->price * $coupon_percent;
            }

            $originalPrice = $detail->price - $coupon_value;
            RequestDetailDiscount::create([
                'item_id' => $detail->item_id,
                'detail_id' => $detail->id,
                'request_id' => $request->request_id,
                'shop_id' => $shop_id,
                'discount_type' =>  $coupon->discount_type ?? 0,
                'discount_value' => $coupon_value,
                'discount_weight' => $coupon_value * $detail->quantity,
                'unit_id' => $detail->unit_id,
            ]);

            $item = Item::select('id', 'vat_state')->find($detail->item_id);

            $vat = 0;
            if (in_array($item->vat_state, [1, 2])) {
                $itemVats = ItemVat::where('item_id', $item->id)->whereIn('vat_use', [0, 1])->with('billAdd')->limit(1)->get();
                if ($itemVats->count() > 0) {
                    // here we will apply vats on original price
                    $vat = $originalPrice - ($originalPrice / (1 + $itemVats->first()->billAdd->addition_value / 100));
                }
            }

            if ($vat) {
                $vatdetails = RequestDetailVat::where(['shop_id' => $shop_id, 'item_id' => $detail->item_id, 'request_id' => $request->request_id])->get();
                foreach ($vatdetails as $vatdetail) {
                    $vatdetail->update([
                        'vat_weight' => $vat  * $detail->quantity,
                    ]);
                }
            }
            $detail->update([
                'price' => $originalPrice,
                'updated_at' => Carbon::now()
            ]);
        }

        $total = DB::table('request_details')->where('shop_id', $shop_id)->where('request_id', $request->request_id)->sum(DB::raw('(request_details.price ) * request_details.quantity'));

        $vats = DB::table('requests_details_vats')->where('shop_id', $shop_id)->where('request_id', $request->request_id)->sum(DB::raw('requests_details_vats.vat_weight '));

        $discounts = DB::table('requests_details_discounts')->where('shop_id', $shop_id)->where('request_id', $request->request_id)->sum(DB::raw('requests_details_discounts.discount_weight'));

        DB::table('requests')->where('id', $request->request_id)->update([
            'total' => $total,
            'coupon_id' => $coupon->id,
            'discount' => $discounts,
            'net' => $total + $order->fee + $vats
        ]);
        $order = EntitiesRequest::where(['shop_id' => $shop_id, 'id' => $request->request_id])->first();
        return $this->responseJson(true, 'Coupon Apply Successfully', [
            'total_invoice' => $order->net,
            'coupon_id' => $order->coupon_id,
        ], null, 200);
    }

    public function emptyCart($shop_id)
    {
        $cart = EntitiesRequest::where(['shop_id' => $shop_id, 'client_id' =>  auth()->guard('api')->id(), 'cart' => 0])->orderBy('id', 'DESC')->first();
        if (! $cart) {
            return $this->responseJson(true, 'تم حذف السلة', [], null, 200);
        }

        \Log::info(" MAKE EMPTY FOR CURT ");
        RequestDetails::where('request_id', $cart->id)->where('shop_id', $shop_id)->delete();
        RequestDetailDiscount::where('request_id', $cart->id)->Where('shop_id', $shop_id)->delete();
        RequestDetailVat::where('request_id', $cart->id)->Where('shop_id', $shop_id)->delete();
        DB::table('component_request')->where('request_id', $cart->id)->Where('shop_id', $shop_id)->delete();
        $cart->delete();

        return $this->responseJson(true, 'تم حذف السلة', [], null, 200);
    }
}
