<?php

namespace Modules\Api\Http\Controllers;

use App\Traits\Api\ApiTrait;
use App\Traits\HelperTrait;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Validator;
use Modules\Api\Entities\Favorite;
use Modules\Api\Entities\Wishlist;
use Modules\Api\Entities\Item;
use Modules\Api\Http\Requests\FavoriteRequest;
use Modules\Api\Transformers\FavouriteResource;
use Modules\Api\Transformers\ItemResource;

class ApiFavoriteController extends Controller
{
    use HelperTrait;
    public function getFavorite(Request $request ,$shop_id)
    {
        try{
            if (auth('api')->check()) {
                $favourites = Item::where('shop_id',$shop_id)->whereHas('favorites')->selection()->paginate(10);
                $favourites=ItemResource::collection($favourites);
                return $this->responseJson(true,trans('api.fav_list'),[
                    'all_items' =>$favourites,
                ],null,200);
            }
            return $this->responseJson(false,trans('api.Unauthorized'),null,null,401);
        }catch(\Throwable $th) {
            return $this->responseJson(false,$th->getMessage(),null,null,500);
        }
    }
    public function addOrDeleteFavorite(FavoriteRequest $request ,$shop_id)
    {
        try{
            if (auth('api')->check()) {
                $findProduct_id=Favorite::where(['shop_id'=>$shop_id,'product_id'=>$request->product_id ,'client_id' =>auth()->guard('api')->id()])->select('id')->first();
                if($findProduct_id){
                    $findProduct_id->delete();
                    return $this->responseJson(true,trans('api.fav_delete'),null,null,200);
                }else
                {
                    Favorite::create([
                        'product_id' => $request->product_id,
                        'client_id' => auth()->guard('api')->id(),
                        'shop_id' =>$shop_id
                    ]);
                    return $this->responseJson(true,trans('api.fav_add') ,null,null,200);
                }
            }
            return $this->responseJson(false,trans('api.Unauthorized'),null,null,401);
        }catch(\Throwable $th) {
            return $this->responseJson(false,$th->getMessage(),null,null,500);
        }
    }
}
