<?php

namespace Modules\Api\Http\Controllers;

use App\Traits\Api\ApiTrait;
use App\Traits\HelperTrait;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Api\Entities\BadrShop;
use Modules\Api\Entities\Item;
use Modules\Api\Entities\ItemType;
use Modules\Api\Entities\Slider;
use Modules\Api\Transformers\CategoryResource;
use Modules\Api\Transformers\ItemResource;
use Modules\Api\Transformers\SliderResource;
use Modules\Api\Entities\RequestSetting;

class ApiHomeController extends Controller
{
    use HelperTrait;

    public function index(Request $request, $shop_id)
    {
        try {
            
            $settings = RequestSetting::getSettings($shop_id);
            $sliders = Slider::where('shop_id', $shop_id)->selection()->take(10)->get();
            $sliders = SliderResource::collection(($sliders));
            $best_sellars = Item::join('invoices_details', 'invoices_details.item_id', '=', 'items.id')
                ->where('items.shop_id', $shop_id)
                ->select('items.*', DB::raw('SUM(invoices_details.quantity) as SoldQuantity'))
                ->with('allUnits', 'unit')
                ->groupBy('items.id')
                ->where('online', 1)
                ->when(!$settings->ignore_quantity, function($query) {
                    $query->havingRaw('store_items_sum_store_quant > 0');
                })
                ->orderBy('SoldQuantity', 'desc')->take(10)->orderBy('items.id','DESC')->get();
                
            $best_sellars = ItemResource::collection($best_sellars);
            $allcategories = ItemType::where(['shop_id' => $shop_id, 'published' => '1'])->selection()->orderBy('id','DESC')->get();
            $allcategories = CategoryResource::collection($allcategories);
            $itemQuery = Item::where('shop_id', $shop_id)->selection()->withSum('storeItems', 'store_quant')->where('online', 1)
                                    ->when(!$settings->ignore_quantity, function($query) {
                                        $query->havingRaw('store_items_sum_store_quant > 0');
                                    })->take(10);
            $allItems = $itemQuery->orderBy('id','DESC')->get();
            $allItems = ItemResource::collection($allItems);
            $discounts = clone $itemQuery->where('withDiscount', '1')->orderBy('id','DESC')->get();
            $discounts = ItemResource::collection($discounts);
            return $this->responseJson(true, 'List Of Data', [
                'sliders' => $sliders,
                'best_seller' => $best_sellars,
                'all_categories' => $allcategories,
                'discounts' => $discounts,
                'all_items' => $allItems,
                'activate_maintenance_mode' => BadrShop::select('activate_maintenance_mode')->where(['serial_id' => $shop_id])->value('activate_maintenance_mode')
            ], null, 200);
        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }
}
