<?php

namespace Modules\Api\Http\Controllers;

use App\Traits\HelperTrait;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Api\Entities\Item;
use Modules\Api\Entities\ItemImage;
use Modules\Api\Entities\RequestSetting;
use Modules\Api\Http\Requests\ItemDetailsRequest;
use Modules\Api\Transformers\ItemResource;

class ApiItemController extends Controller
{
    use HelperTrait;

    public function search(Request $request, $shop_id)
    {
        $settings = RequestSetting::getSettings($shop_id);

        $allItems = Item::where(['items.shop_id' => $shop_id])
            ->Where(function ($query) use ($request) {
                $query->when($request->category_id, function ($query) use ($request) {
                    $query->whereIN('sale_unit', explode(',', $request->category_id));
                });
            })->when($request->request_search, function ($query) use ($request) {
                $query->where('item_name', 'like', '%' . $request->get('request_search') . '%');
            })->when($request->withDiscount, function ($query) use ($request) {
                $query->where('withDiscount', '1');
            })->when($request->sort, function ($query) use ($request) {
                $query->orderBy('sale_price', $request->sort);
            })->when($request->min, function ($query) use ($request) {
                $query->where('sale_price', '>=', $request->min);
            })->when($request->max, function ($query) use ($request) {
                $query->where('sale_price', '<=', $request->max);
            })->when(($request->sort_by == "desc_dics"), function ($query) use ($request) {
                $query->orderBy('discount_percent', 'desc');
            })->when(($request->sort_by == "asc_dics"), function ($query) use ($request) {
                $query->orderBy('discount_percent', 'asc');
            })->when($request->best_sellars, function ($query) use ($request, $shop_id) {
                $query->join('invoices_details', 'invoices_details.item_id', '=', 'items.id')
                    ->select('items.*', DB::raw('SUM(invoices_details.quantity) as SoldQuantity'))
                    ->groupBy('items.id')->get();
            })->when(!$settings->ignore_quantity, function($query) {
                $query->havingRaw('store_items_sum_store_quant > 0');
            })->where('online', 1)
            ->orderBy('id','DESC')->paginate(10);
        $allItems = ItemResource::collection($allItems);
        return $this->responseJson(true, 'List Of Item Details ', [
            'all_items' => $allItems,
        ], null, 200);
    }
    public function itemDetails(ItemDetailsRequest $request, $shop_id)
    {
        try {
            $item = Item::where(['shop_id' => $shop_id, 'id' => $request->product_id])->selection()->first();
            $allimages = ItemImage::where(['shop_id' => $shop_id, 'item_id' => $request->product_id])->select('image_link')->get()->toArray();
            $allimages = collect($allimages)->push(['image_link' => $item->img]);

            $item = ItemResource::make($item);
            return $this->responseJson(true, 'List Of Item Details ', [
                'item' => $item,
                'images' => $allimages,
            ], null, 200);
        } catch (\Throwable $th) {
            return response()->json([
                'status' => false,
                'message' => $th->getMessage()
            ], 500);
        }
    }
}
