<?php

namespace Modules\Api\Http\Controllers;

use App\Notifications\OrderNotify;
use App\Traits\Api\ApiTrait;
use App\Traits\CartTrait;
use App\Traits\HelperTrait;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Api\Transformers\ItemResource;
use Modules\Api\Transformers\OrderResource;
use Illuminate\Support\Facades\Validator;
use Modules\Api\Entities\Item;
use Modules\Api\Entities\ItemUnit;
use Modules\Api\Entities\StoreItem;
use Modules\Api\Entities\ItemVat;
use Modules\Api\Entities\Request as EntitiesRequest;
use Modules\Api\Entities\RequestCoupon;
use Modules\Api\Entities\RequestDetailDiscount;
use Modules\Api\Entities\RequestDetails;
use Modules\Api\Entities\RequestDetailVat;
use Modules\Api\Http\Requests\OrderDetailsRequest;
use Modules\Api\Http\Requests\SaveOrderRequest;
use Modules\Maintenance\Entities\Order;
use PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Sum;
use Modules\Api\Transformers\AllOrderResource;

class ApiOrderController extends Controller
{
    use HelperTrait, CartTrait;
    
    public function getOrders(Request $request, $shop_id)
    {

        $orders = EntitiesRequest::where(['shop_id' => $shop_id, 'client_id' => auth()->guard('api')->id(),'cart' => '1'])->selection()->orderBy('order_no', 'DESC')->get();
        $orders = AllOrderResource::collection($orders);
        return $this->responseJson(true, 'List Of  orders', $orders, null, 200);

    }

    public function orderDetails(OrderDetailsRequest $request, $shop_id)
    {
        try {
            $order = EntitiesRequest::where(['shop_id' => $shop_id, 'id' => $request->order_id, 'client_id' => auth()->guard('api')->id(), 'cart' => '1'])->selection()->first();
            $order = OrderResource::make($order);
            return $this->responseJson(true, 'List Of Item Details', $order, null, 200);
        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }

    public function deleteOrder(OrderDetailsRequest $request, $shop_id)
    {
        try {
            $request['shop_id'] = $shop_id;
            $order = EntitiesRequest::where(['shop_id' => $shop_id, 'client_id' => auth()->guard('api')->id()])->with('details')->find($request->order_id);
            $order->update(['status' => '0']);
            $data = [
                'msg' => trans('api.order_delete',['order'=>$order->id])
            ];
            auth()->user()->notify(new OrderNotify($data));
            return $this->responseJson(true, trans('api.order_delete',['order'=>$order->id]) , null, null, 200);
        } catch (\Throwable $th) {
            return $this->responseJson(false, $th->getMessage(), null, null, 500);
        }
    }
    
    public function saveOrder(SaveOrderRequest $request, $shop_id)
    {
        \Log::info(  json_encode(request()->all()) );

        $order = EntitiesRequest::where(['shop_id' => $shop_id, 'id' => $request->request_id])->selection('client_id')->first();
        if ($request->payment_method == 2) {
            $file_name = $this->saveImage($request->file('trans_img'), 'uploads/' . $shop_id . '/Api/payment');
        }
        
        $result = $this->checkOrderItemsQuantity($shop_id, $order);
        if (! $result['status'] ) {
            \Log::info( json_encode($result) );
            return $this->responseJson(false, $result['message'], [], null, 200);
        }
        
        DB::table('requests')->where(['id' => $request->request_id])->update([
            'cart' => 1,
            'client_name' => auth()->guard('api')->user()->client_name ?? $request->client_name,
            'email' => auth()->guard('api')->user()->email ?? $request->email,
            'mobile' => auth()->guard('api')->user()->mobile1 ?? $request->mobile,
            'shop_id' => $shop_id,
            'lat' => $request->lat,
            'lon' => $request->lon,
            'status' => '1',
            'coupon_id' => $request->coupon_id,
            'city_id' => $request->city_id,
            'address' => $request->address ?? auth()->guard('api')->user()->address,
            'payment_type' => $request->payment_method,
            'trans_img' => $file_name ?? null,

        ]);
        $client = auth()->user();
        $data = [
            'msg' => trans('api.order_save',['order'=>$order->id])
        ];
        $client->notify(new OrderNotify($data));
        $order = OrderResource::make($order);
        return $this->responseJson(true, trans('api.order_save',['order'=>$order->id]), $order, null, 200);
    }
        
    protected function checkOrderItemsQuantity($shop_id, $order)
    {
        $requests_id = EntitiesRequest::select('id')->where('shop_id', $shop_id)->where('client_id', auth()->guard('api')->id())->where('status', '<>', 0)->where('bill_id', 0)->where('cart', 1)->where('id', '<>', $order->id)->pluck('id');
        $item_quantitys = $items_name = $avilables = [];
        $all_details = RequestDetails::select('id','item_id','quantity', 'unit_id')->where('shop_id', $shop_id)->whereIn('request_id', $requests_id)->get();
        $details = RequestDetails::select('id','item_id','quantity', 'unit_id')->where('shop_id', $shop_id)->where('request_id', $order->id)->get();

        foreach($all_details as $detail) {
            if ( !in_array($detail->item_id, $details->pluck('item_id')->toArray()) ) continue ;
            $unit_value = ItemUnit::select('unit_value')->where(['shop_id' => $shop_id, 'item_id' => $detail->item_id, 'unit_id' => $detail->unit_id])->value('unit_value') ?? 1;
            if ( ! isset($item_quantitys[$detail->item_id]) ) {
                $item_quantitys[$detail->item_id] = 0;
            }

            $item_quantitys[$detail->item_id] += ( $detail->quantity * $unit_value );
        }
        
        foreach($item_quantitys as $item_id => $quantity) {
            $qty = StoreItem::where('item_id', $item_id)->where('shop_id', $shop_id)->sum('store_quant');
            if (! isset($avilables[$item_id])) {
                $avilables[ $item_id ] = $qty;
            }
            
            $avilables[ $item_id ] -= $quantity;
            if ($avilables[ $item_id ] < 0) {
                $item_name = Item::select('item_name')->where('id', $item_id)->value('item_name');
                return ['status' => false, 'message' => "الكمية '{$avilables[ $item_id ]}' هي المتاحة للصنف '$item_name'"];
            }
        }
        
        foreach($details as $detail) {
            $unit_value = ItemUnit::select('unit_value')->where(['shop_id' => $shop_id, 'item_id' => $detail->item_id, 'unit_id' => $detail->unit_id])->value('unit_value') ?? 1;
            if ( ! isset($avilables[$detail->item_id]) ) {
                $avilables[$detail->item_id] = StoreItem::where('item_id', $detail->item_id)->where('shop_id', $shop_id)->sum('store_quant');
            }
            
            $item_row = Item::select('order_max_quantity', 'item_name')->find($detail->item_id);
            if ($item_row->order_max_quantity > 0 && $item_row->order_max_quantity < $detail->quantity) {
                return ['status' => false, 'message' => "لا يمكن شراء سواء  '{$item_row->order_max_quantity}'  فقط من الصنف '{$item_row->item_name}'"];
            }
            
            $qty = $detail->quantity * $unit_value;
            $avilable = $avilables[$detail->item_id] - $qty;
            
            if ($avilable < 0) {
                $item_name = Item::select('item_name')->where('id', $detail->item_id)->value('item_name');
                return ['status' => false, 'message' => "الكمية '{$avilables[$detail->item_id ]}' هي المتاحة للصنف '$item_name'"];
            }
            $avilables[$detail->item_id] -= $qty;
        }
        
        return ['status' => true];
    }
}
