<?php

namespace Modules\Api\Http\Controllers;

use App\Traits\HelperTrait;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Api\Entities\ItemRate;
use Modules\Api\Http\Requests\ReactRequest;
use Modules\Api\Transformers\ReactResource;

class ApiReactController extends Controller
{
    use HelperTrait;

    public function addReact(ReactRequest $request, $shop_id)
    {
        try {
            ItemRate::updateOrCreate([
                'shop_id' => $shop_id,
                'item_id' => $request->item_id,
                'client_id' => auth()->guard('api')->id()
            ], [
                'rate' => $request->rate ?? 0,
                'comment' => $request->comment ?? '',
            ]);
            return $this->responseJson(true, trans('api.add_re_succ'), null, null, 200);
        } catch (\Throwable $th) {
            return response()->json([
                'status' => false,
                'message' => $th->getMessage()
            ], 500);
        }
    }
    public function allReacts(Request $request, $shop_id)
    {
        $reacts = ItemRate::where(['shop_id' => $shop_id, 'item_id' => $request->item_id])->with('client:id,client_name,image')->get();
        $totalrate = $reacts->sum('rate');
        $count = $reacts->count();
        $reacts = ReactResource::collection($reacts);
        return $this->responseJson(
            true,
            'Product Reviews',
            [
                'reacts' => $reacts,
                'total_rate' =>$count>0 ?($totalrate / $count) :0,
            ],
            null,
            200
        );
    }
}
