<?php

namespace Modules\Api\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Http\Exceptions\HttpResponseException;

use Illuminate\Contracts\Validation\Validator;

class ProfileRequest extends FormRequest
{
    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
        return [
            'user_name' => ['sometimes', 'unique:clients,user_name,' . auth()->guard('api')->id() . ',id,shop_id,' . $this->shop_id],
            'email'     => ['sometimes', 'unique:clients,email,' . auth()->guard('api')->id() . ',id,shop_id,' . $this->shop_id],
            'mobile1'   => ['sometimes', 'unique:clients,mobile1,' . auth()->guard('api')->id() . ',id,shop_id,' . $this->shop_id],
            'image' =>'sometimes|image',
        ];
    }
    
    public function attributes()
    {
        return [
            'user_name' => 'اسم المستخدم',
            'email'     => 'البريد الالكتروني',
            'mobile1'   => 'رقم الهاتف',
            'image' => 'الصورة',
        ];
    }
    
    public function failedValidation(Validator $validator)
    {
        throw new HttpResponseException(response()->json([
            'success' => false,
            'message' => 'Validation errors',
            'errors'  => $validator->errors()->all()
        ]));
    }
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }



    public function log($messages)
    {
        $file = __DIR__ . DIRECTORY_SEPARATOR  . date('Y-m-d') . '.log';


            $messages = json_encode($messages, JSON_PRETTY_PRINT);

            $messages = "========================================================\n\n" . $messages . "\n\n";


        if (!file_exists($file)) {
            touch($file, strtotime('-1 days'));
        }

        if (filesize($file) > 907200) {
            $fp = fopen($file, "r+");
            ftruncate($fp, 0);
            fclose($fp);
        }

        $myfile = fopen($file, "a+");
        fwrite($myfile, $messages);
        fclose($myfile);
    }


}
