<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Round;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/
Route::any('Unauthorized', function(){
    return response()->json([
         'status' => 401,
         'message' => 'Unauthorized'
     ]);
 })->name('Unauthorized');


Route::middleware(['checkLang', \Modules\Api\Http\Middleware\Badrshop::class])->prefix('{shop_id}')->group(function () {
    // user authentication
    Route::post('login', 'ApiAuthContollerController@login')->name('authenticate');
    Route::post('register','ApiAuthContollerController@register');
    //change password without login
    Route::post('forgot-password', 'ApiAuthContollerController@forgotPassword');
    Route::post('create-password', 'ApiAuthContollerController@CreatePassword');
    //Categories
    Route::get('all-categories', 'ApiCategoryController@allCategory');
    // contacts , aboutus   terms
    Route::get('badr-shop', 'ApiBadrshopController@badrShop');
    Route::post('contact-us', 'ApiBadrshopController@contactUs');
    Route::get('home', 'ApiHomeController@index');
    //items
    Route::post('search', 'ApiItemController@search');
    Route::post('item-details', 'ApiItemController@itemDetails');

    Route::get('test', 'ApiItemController@test');
    Route::get('all-area', 'ApiBadrshopController@allAreas');
    Route::get('get-city', 'ApiBadrshopController@getCity');
    Route::get('all-react', 'ApiReactController@allReacts');

    Route::get('all-staticPages', 'ApiBadrshopController@allStaticPages');
    Route::get('get-staticPages', 'ApiBadrshopController@getStaticPage');
    
    Route::get('settings', 'RequestSettingController');

    Route::group(['middleware' => 'auth:api'], function ($router) {
        // user profile  (login)
        Route::post('change-password', 'ApiAuthContollerController@changePassword');
        Route::get('profile', 'ApiAuthContollerController@getProfile');
        Route::post('update-profile', 'ApiAuthContollerController@updateProfile');
        // whishlists
        Route::get('get-wishlist', 'ApiWishlistcontroller@getWishlist');
        Route::post('wishlist', 'ApiWishlistcontroller@addOrDeleteWishlist');
        // favorites
        Route::get('get-favorite', 'ApiFavoriteController@getFavorite');
        Route::post('favorite', 'ApiFavoriteController@addOrDeleteFavorite');
        // orders
        Route::get('orders', 'ApiOrderController@getOrders');
        Route::post('order-details', 'ApiOrderController@orderDetails');
        Route::post('order-delete', 'ApiOrderController@deleteOrder');
        Route::post('order-save', 'ApiOrderController@saveOrder');
        Route::post('add-cart', 'ApiCartController@addToCart');
        Route::post('delete-cart', 'ApiCartController@deleteFromCart');
        Route::post('all-cart', 'ApiCartController@getCart');
        Route::post('apply-coupon', 'ApiCartController@applyCoupon');
        Route::post('cart/empty', 'ApiCartController@emptyCart');
        Route::get('notification', 'ApiNotificationController@getNotify');
        Route::post('add-react', 'ApiReactController@addReact');

        //logout
        Route::post('logout', 'ApiAuthContollerController@logout');
        Route::get('delete-account', 'ApiAuthContollerController@deleteAccount');
    });
});

