<?php

namespace Modules\Api\Transformers;

use Illuminate\Http\Resources\Json\JsonResource;
use Modules\Api\Entities\Item;

class ItemResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @param  \Illuminate\Http\Request
     * @return array
     */
    public function toArray($request)
    {
        $units = [];
        
        foreach($this->getUnits() as $unit) {
            $units[] = [
                    'id' => $unit->id,
                    'name' => $unit->name,
                    'price' => (float)$unit->unit_price,
                    'qty' => (float) $this->maxOrder($unit->id),
                    'unit_value' => (int) $unit->unit_value,
                    'discount_price' => (float) ( ($unit->unit_price ==  0) ? (round_money(($unit->sale_price) - (($unit->sale_price * $unit->discount_percent) / 100)))
                        : (round_money(($unit->unit_price) - (($unit->unit_price * $unit->discount_percent) / 100))) ),
                ];
        }
        
        return array(
            'id' => $this->id,
            'item_name' => $this->item_name,
            'details' => $this->details,
            'about' => $this->details,
            'sale_price' =>  $this->sale_price,
            'discount_percent' => $this->discount_percent ? round_money($this->discount_percent) : "0",
            'sale_discount' => round_money(($this->sale_price) - (($this->sale_price * $this->discount_percent) / 100)),
            'img' => $this->img,
            'is_favorite' => $this->isFavorite(),
            'is_wishlist' => $this->isWishlist(),
            'is_sale' => $this->withDiscount ? true : false,
            'rate' => $this->rates_count > 0 ? number_format($this->rates()->sum('rate') / $this->rates_count, 2) : "0",
            'main_unit_id'=>$this->unit_id,
            'main_unit_name'=>$this->unit->name,
            'max_order' => (float) $this->max_items,
            'qty' => (float) $this->maxOrder($this->unit_id),
            'units' => $units
        );
    }
}
