<?php

namespace Modules\BadrRequest\Entities;

use App\Traits\RoundingAttributesTrait;
use Illuminate\Database\Eloquent\Model;

class BadrRequest extends Model {
    use RoundingAttributesTrait;
	protected $table = 'requests';
	public $timestamps = true;
    protected $guarded = [];
	protected $appends = [ 'status_display' ];
    public function scopeSelection($query)
    {
        return $query->select('id','total','net','created_at','status','order_no','discount','fee','mobile','client_name','delivery_date','client_id')
        ->with('totalVat')
        ->with('totalDiscount')
        ->with('requestDetails')
        ->with('items');
    }
    public function totalDiscount()
    {
        return $this->hasMany(RequestDetailDiscount::class, 'request_id', 'id');
    }
    public function totalVat()
    {
        return $this->hasMany(RequestDetailVat::class, 'request_id', 'id');
    }
	public function requestDetails() {
		return $this->hasMany(RequestDetails::class ,'request_id','id');
	}
    public function items()
    {
        return $this->hasManyThrough(
            Item::class,
            RequestDetails::class,
            'request_id', // Foreign key on the environments table...
            'id', // Foreign key on the deployments table...
            'id', // Local key on the projects table...
            'item_id' // Local key on the environments table...
        );
    }
	public function client() {
		return $this->belongsTo( Client::class );
	}

	public function shop() {
		return $this->belongsTo( BadrShop::class, 'shop_id', 'serial_id' );
	}

	public function getQuantityAttribute() {
		return $this->requestDetails()->sum('quantity');
	}

    public function uppType()
    {
        if($this->app_type == 0)
        {
            return "android";
        }
        elseif($this->app_type == 1)
        {
            return "iphone";
        }
        else{
            return "web";
        }
    }
	public function getStatusDisplayAttribute() {
		$status = array(
			"0" => trans('badrrequests.cancel') ,
			"1" => trans('badrrequests.processing'),
			"2" => trans('badrrequests.done'),
			"3" => trans('badrrequests.delivering'),
// 			"3" => trans('badrrequests.delivering'),
            '4' => ' معلق لحين اكمال الدفع',
            '5' => 'معلق للتجهيز من قبل الادارة',
            '8' => ''
		);

		return $status[ $this->status ];
	}

    public function BillItemsCount()
    {
        if($this->no_bill){
            return SaleDetails::where('shop_id', $this->shop_id)->where('sale_id', $this->sale_process_id)->sum('quantity');
        }else{
            return $this->getQuantityAttribute();
        }
	}

    public function getPaymentOption()
    {
        $arr = [
            '0' => ' عند الاستلام',
            '1' => 'دفع الكتروني',
            '2' => 'تحويل بنكي',
            ];
        if(array_key_exists($this->payment_type, $arr)){
            return $arr[$this->payment_type];
        }
        return 'غير معرف';
	}


	public function getCity(){

		$city = \DB::table('cities')->find($this->city_id);
		if($city){
			$par = \DB::table('cities')->find($city->city_up);
			if($par){
				return  $par->city_name . " - " . $city->city_name;
			}
			else{
				return  $city->city_name;
			}
		}

		return '';
	}

}
