<?php

namespace Modules\BadrRequest\Entities;

use App\Traits\RoundingAttributesTrait;
use Illuminate\Database\Eloquent\Model;

class Item extends Model
{
    use RoundingAttributesTrait;
	protected $table = 'items';
	public $timestamps = false;

    public function scopeSelection($query)
    {
        return $query->select('id', 'item_name', 'sale_price', 'withDiscount', 'discount_percent', 'details', 'img', 'shop_id');
    }
	public function unit() {
		return $this->belongsTo( Unit::class );
	}

    public function stores()
    {
        return $this->belongsTo(StoreItem::class,'item_id','id');
    }


    public function slider()
    {
        return $this->hasMany(Slider::class,'item_id','id');
    }
    public function saleItemVats()
    {
        return $this->hasMany(ItemVat::class, 'item_id', 'id')->whereIn('item_vat.vat_use', [0, 1]);
    }
    public function getCards($bill_id)
    {
        $cards_query = Card::where([
            'shop_id' => $this->shop_id,
            'item_id' => $this->id,
            'sale_id' => $bill_id,
            'card_state' => 1
        ])->pluck('card_number');
        $data = [];
        $cards_enc = json_decode(json_encode($cards_query));
        if (sizeof($cards_enc) > 0) {
            foreach ($cards_query as $item) {
                $data[] = $this->decrypt($item);
            }
        }
        return $data;
    }

    private function decrypt($encrypted)
    {
        $password = '3sc3RLrpd1LBVFTsdFVaadjsdXFS';
        $method = 'aes-256-cbc';
        $key = substr(hash('sha256', $password, true), 0, 512);

        $iv = chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0) . chr(0x0);

        $decrypted = openssl_decrypt(base64_decode($encrypted), $method, $key, OPENSSL_RAW_DATA, $iv);

        $data = explode('--', $decrypted);
        $decrypt = $data[1];
        return $decrypt;
    }
    public function requestQuantity()
    {
        return $this->hasMany(RequestDetails::class, 'item_id', 'id')->join('requests', function ($k) {
            $k->on('requests.id', 'request_details.request_id');
            $k->where('requests.bill_id', '=', 0);
        })
            ->sum('quantity');
    }
    public function maxOrder()
    {
        $ignore = RequestSettings::where('shop_id', $this->shop_id)->firstOrCreate([
            'shop_id' => $this->shop_id,
            'max_items' => 5,
            'from_shop' => 1,
            'ignore_quantity' => 0,
            'bill_with_order' => 0
        ]);
        if ($ignore->ignore_quantity == '1') {
            $availableQuantity = $this->storeItems()->sum('store_quant');
        } else {
            $availableQuantity = $this->storeItems()->sum('store_quant') - $this->requestQuantity();
        }
        return $availableQuantity;
    }
    public function storeItems()
    {
        return $this->belongsTo(StoreItem::class, 'id', 'item_id');
    }
}
