<?php

namespace Modules\BadrRequest\Http\Controllers;

use App\Notifications\OrderNotify;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Modules\BadrRequest\Entities\RequestSettings;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Api\Entities\BillAdd;
use Modules\Api\Entities\RequestDetailDiscount;
use Modules\BadrRequest\Entities\BadrRequest;
use Modules\BadrRequest\Entities\BadrShop;
use Modules\BadrRequest\Entities\Card;
use Modules\BadrRequest\Entities\Client;
use Modules\BadrRequest\Entities\ClientNotification;
use Modules\BadrRequest\Entities\Item;
use Modules\BadrRequest\Entities\ItemUnit;
use Modules\BadrRequest\Entities\ItemVat;
use Modules\BadrRequest\Entities\RequestAddHistory;
use Modules\BadrRequest\Entities\RequestDetailDiscount as EntitiesRequestDetailDiscount;
use Modules\BadrRequest\Entities\RequestDetails;
use Modules\BadrRequest\Entities\RequestDetailVat;
use Modules\BadrRequest\Entities\SalePoint;
use Modules\BadrRequest\Entities\SaleProcess;
use Modules\BadrRequest\Entities\Store;
use Modules\BadrRequest\Entities\StoreItem;
use Modules\BadrRequest\Http\Requests\RequestSettingRequest;

class BadrRequestController extends Controller
{

    public function index(Request $request)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $type = get_session_val('date_type', 'integer');
        $rows = BadrRequest::where(['shop_id' => $shop_id, 'cart' => '1'])
            ->when($request->filled('client_name'), function ($query) use ($request) {
                $query->where('client_name', 'like', '%' . $request->client_name . '%');
            })->when($request->filled('date'), function ($query) use ($request) {
                $query->whereDate('created_at', $request->date);
            })->when($request->filled('number'), function ($query) use ($request) {
                $query->where('order_no', $request->number);
            })->when($request->filled('request_status'), function ($query) use ($request) {
                $query->where('status', $request->request_status);
            })->orderBy('order_no', 'desc')->paginate(10);
        return view('badrrequest::requests', compact('rows', 'type'));
    }
    public function requestLocation(Request $request, $id)
    {
        $model = BadrRequest::findOrFail($id);
        return view('badrrequest::location', compact('model'));
    }
    public function requestDetails(Request $request, $id)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $shop = BadrShop::where('serial_id', $shop_id)->first();
        $req = BadrRequest::findOrFail($id);
        $req_id = $req->id;
        $rows = RequestDetails::where(['shop_id' => $shop_id, 'request_id' => $id])->selection()->with('item')->withSum('discounts', 'discount_value')->withSum('vats', 'vat_weight')->with('vats', 'unit')->with('discounts')->get();
        $totalvats = RequestDetailVat::where(['shop_id' => $shop_id, 'request_id' => $id])
            ->sum(DB::raw('requests_details_vats.vat_weight'));
        $totalDiscounts = EntitiesRequestDetailDiscount::where(['shop_id' => $shop_id, 'request_id' => $id])
            ->sum(DB::raw('requests_details_discounts.discount_weight'));
        $sale_point_id = auth()->user()->sale_point;
        $sale_points = SalePoint::where('shop_id', $shop_id)->get();
        if (Auth::user()->change_store == 0) {
            $stores = Store::where('shop_id', $shop_id)
                ->where('id', auth()->user()->store_id)
                ->get();
        } else {
            $stores = Store::where('shop_id', $shop_id)->get();
        }
        if ($request->has('store_id')) {
            $store_id = $request->store_id;
        } else {
            $store_id = auth()->user()->store_id;
            if (is_null($store_id)) {
                $temp = $stores->first();
                $store_id = $temp->id ?? 0;
            }
        }

        $req->pay_type = $req->getPaymentOption();
        if ($req->payment_type == '2') {
            $req->image = asset('../upload/trans_files/' . $req->trans_im);
        }


        return view('badrrequest::request', compact('totalDiscounts', 'totalvats', 'shop_id',  'req_id',   'req', 'rows', 'sale_points', 'stores', 'sale_point_id', 'store_id', 'shop'));
    }
    public function updateRequest(Request $request)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $store_qty = StoreItem::where(['shop_id' => $shop_id, 'item_id' => $request->item_id])->first();
        if ($store_qty != null) {
            if ($store_qty->store_quant < $request->outgoing) {
                return response()->json([
                    'status' => false,
                    'message' => '..... الكميه في المخزن لا تكفي برجاء تعديل الكميه المطلوبه',
                ]);
            } else {
                $details = RequestDetails::where(['shop_id' => $shop_id, 'request_id' => $request->req_id, 'item_id' => $request->item_id])->first();
                $details->quantity = ($request->outgoing != 0) ? $request->outgoing : $details->quantity;
                $details->save();
                $discounts = RequestDetailDiscount::where(['shop_id' => $shop_id, 'request_id' => $request->req_id, 'item_id' => $request->item_id])->get();
                if ($discounts) {
                    foreach ($discounts as $discount) {
                        $discount->update([
                            'discount_weight' => $discount->discount_value * $request->outgoing,
                        ]);
                    }
                }
                $vats = RequestDetailVat::where(['shop_id' => $shop_id, 'request_id' => $request->req_id, 'item_id' => $request->item_id])->get();
                if ($vats) {
                    foreach ($vats as $vat) {
                        $vat->update([
                            'vat_weight' => $vat->vat_value * $request->outgoing,
                        ]);
                    }
                }
                $total = DB::table('request_details')
                    ->where('shop_id', $shop_id)
                    ->where('request_id', $request->req_id)
                    ->sum(DB::raw('(request_details.price ) * request_details.quantity'));
                $vats = DB::table('requests_details_vats')
                    ->where('shop_id', $shop_id)
                    ->where('request_id', $request->req_id)
                    ->sum(DB::raw('requests_details_vats.vat_weight '));
                $discounts = DB::table('requests_details_discounts')
                    ->where('shop_id', $shop_id)
                    ->where('request_id', $request->req_id)
                    ->sum(DB::raw('requests_details_discounts.discount_weight'));
                $requests = BadrRequest::where(['shop_id' => $shop_id, 'id' => $request->req_id])->first();
                $requests->update([
                    'net' => $total + $vats + $requests->fee,
                    'total' => $total,
                    'discount' => $discounts,
                ]);
                return response()->json([
                    'status' => true,
                    'message' => 'تم تعديل الكميه بنجاج',
                ]);
            }
        } else {
            return response()->json([
                'status' => false,
                'message' => 'المنتج غير متوفر في المخزن حاليا',
            ]);
        }
    }

    public function saveRequestStatus(Request $request, $id)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $update = BadrRequest::findOrFail($id);
        
        if (!$this->checkOrderItemsQuantity($update) ) {
            return redirect()->back()->with(['error' => 'يرجي مراجعة كميات المخازن']);
        }

        $st_msg = 'تم الحفظ بنجاح';
        $status = $update->status;
        
        if ($request->status == 2 && blank($request->delivery_date)) {
            $request->merge(['delivery_date' => now()]);
        }

        $update->update($request->all());
        DB::beginTransaction();
            if ($status != 0) {
                
                
                $cards = Card::where([
                    'shop_id' => $shop_id,
                    'request_id' => $update->id,
                ])->get();

                $cards->each(function ($card) {
                    $card->request_id = 0;
                    $card->sale_id = 0;
                    $card->card_state = 0;
                    $card->save();
                });

                $st_msg = '';
                switch ($update->status) {
                    case "0":
                        $st_msg = "تم الغاء طلبك رقم" . $update->order_no;
                        break;
                    case "1":
                        $st_msg = "جارى تنفيذ طلبك رقم" . $update->order_no;
                        break;
                    case "2":
                        $st_msg = " تم استلام طلبك رقم" . $update->order_no;
                        break;
                    case "3":
                        $st_msg = " تم شحن طلبك رقم" . $update->order_no;
                        break;
                    case "4":
                        $st_msg = "طلبك رقم" . $update->order_no . 'معلق لحين اكمال الدفع';
                        break;
                    case "5":
                        $st_msg = "طلبك رقم" . $update->order_no . 'معلق لحين التجهيز';
                        break;
                    default:
                        $st_msg =  'تم تغيير حالة الطلب رقم ' . $update->order_no;
                }
            }
        DB::commit();
        return redirect()->back()->with(['success' => $st_msg]);
    }
    
    protected function checkOrderItemsQuantity($order)
    {
        if ($order->status > 2 || request()->status == 0) return true;
        $shop_id = auth()->user()->shop_id;
        $details = RequestDetails::select('id','item_id','quantity', 'unit_id')->where('shop_id', $shop_id)->where('request_id', $order->id)->get();
        $item_quantitys = [];

        foreach($details as $detail) {
            $unit_value = ItemUnit::select('unit_value')->where(['shop_id' => $shop_id, 'item_id' => $detail->item_id, 'unit_id' => $detail->unit_id])->value('unit_value') ?? 1;

            if ( ! isset($item_quantitys[$detail->item_id]) ) {
                $item_quantitys[$detail->item_id] = 0;
            }

            $item_quantitys[$detail->item_id] += ( $detail->quantity * $unit_value );
        }
        
        
        foreach($item_quantitys as $item_id => $quantity) {
            if (StoreItem::where('item_id', $item_id)->where('store_quant', '>=', $quantity)->where('shop_id', $shop_id)->count() == 0) {
                return false;
            }
        }
        
        return true;
    }
}

