<?php

namespace Modules\BadrRequest\Http\Controllers;

use App\Notifications\OrderNotify;
use Carbon\Carbon;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Api\Entities\Client;
use Modules\BadrRequest\Entities\BadrRequest;
use Modules\BadrRequest\Entities\Card;
use Modules\BadrRequest\Entities\InvoiceDetails;
use Modules\BadrRequest\Entities\InvoiceDetailsDiscounts;
use Modules\BadrRequest\Entities\InvoiceDetailsVats;
use Modules\BadrRequest\Entities\Invoices;
use Modules\BadrRequest\Entities\Item as EntitiesItem;
use Modules\BadrRequest\Entities\RequestDetailDiscount;
use Modules\BadrRequest\Entities\RequestDetails;
use Modules\BadrRequest\Entities\RequestDetailVat;
use Modules\BadrRequest\Entities\SalePoint;

use Modules\BadrRequest\Entities\StoreItem;

class InvoiceController extends Controller
{

    public function storeDetail($requests, $invoice_id, $request_id, $shop_id, $request)
    {
        $details = RequestDetails::where(['shop_id' => $shop_id, 'request_id' => $request_id])->selection()->get();
        foreach ($details as $detail) {
            $vat = RequestDetailVat::where(['shop_id' => $shop_id, 'request_id' => $request_id, 'item_id' => $detail->item_id, 'unit_id' => $detail->unit_id])->first();
            $item_price_with_discount = $detail->price;
            if ($detail->discount_percent > 0) {
                $item_price_with_discount = $item_price_with_discount / (1 + ($item_price_with_discount / 100));
            }
            
            InvoiceDetails::Create([
                'invoice_id' => $invoice_id,
                'item_id' => $detail->item_id,
                'quantity' => $detail->quantity,
                'unit_id' => $detail->unit_id,
                'quantity_after' => $detail->quantity,
                'sale_price' => $detail->sale_price,
                'basic_sale_price' => $detail->sale_price,
                'basic_pay_price' => $detail->price,
                'total_price' => $detail->price * $detail->quantity,
                'price_without_vat' => $detail->price,
                'item_price_with_discount' => $item_price_with_discount * $detail->quantity,
                'pay_price' => $detail->item->pay_price,
                'safe_point_id' => $request->sale_point,
                'store_id' => $request->store_id,
                'add_user' => auth()->user()->id,
                'add_date' => Carbon::now(),
                'shop_id' => $shop_id,
            ]);
            if ($vat) {
                $this->storeVats($detail, $requests, $invoice_id, $vat, $request_id, $shop_id, $request);
            }
            $this->storeQuantity($requests, $detail, $request_id, $request, $invoice_id, $shop_id);
        }
    }
    public function storeVats($detail, $requests, $invoice_id, $vat, $request_id, $shop_id, $request)
    {
        $vatdetails = InvoiceDetailsVats::create([
            'invoice_id' => $invoice_id,
            'item_id' => $vat->item_id,
            'vat_id' => $vat->vat_id,
            'vat_operation' => $vat->vat_operation,
            'vat_type' => $vat->vat_type,
            'vat_value' => $vat->vat_value,
            'vat_weight' => $vat->vat_weight,
            'shop_id' => $shop_id,
            'unit_id' => $vat->unit_id,
        ]);
        $vatdetails->save();
    }

    public function storeQuantity($requests, $detail, $request_id, $request, $invoice_id, $shop_id)
    {
        // update store_item_quantity
        $storeQty = StoreItem::where(['shop_id' => $shop_id, 'store_id' => $request->store_id, 'item_id' => $detail->item_id])->first();
        $storeQty->store_quant = $storeQty->store_quant - $detail->quantity;
        $storeQty->save();
    }

    public function saveTransaction($request_id, $requests, $shop_id, $n_balance, $rest, $payment, $balance, $sale_point, $invoice_id)
    {
        $details = RequestDetails::where(['shop_id' => $shop_id, 'request_id' => $request_id])->selection()->get();
        foreach ($details as $detail) {
            DB::table('items_transaction')->insert([
                'tansaction_date' => Carbon::now(),
                'action_date' => Carbon::now(),
                'item_id' => $detail->item_id,
                'price' => $detail->price,
                'type' => 1,
                'quantity' => $detail->quantity,
                'new_quantity' => $detail->quantity,
                'user_id' => auth()->user()->id,
                'shop_id' => $shop_id,
                'item_name' => $detail->item->item_name,
                'incom_id' => 0,
                'incom_return_id' => 0,
                'sale_id' => 0,
                'back_id' => 0,
                'problem_id' => 0,
                'manuf_order' => 0,
                'order_no' => 0,
            ]);

        }
        DB::table('client_transaction')->insert([
            'shop_id' => $shop_id,
            'date_time' => Carbon::now(),
            'client_id' => $requests->client_id,
            'amount' => $requests->net, //  المدفوع فى الفاتورة
            'type' => 1,
            'effect' => 1,
            'pay_day' => Carbon::now(),
            'balance' => $n_balance, // رصيد العميل
            'bill_id' => $invoice_id,
            'bill_net_total' => $requests->net,
            'safe_balance' => $sale_point->money_point,
            'safe_point_id' => $sale_point->id,
            'user_id' => 0,
            'sale_back_id' => 0,
            'problem_id' => 0,
            'supplier_id' => 0,
            'salary_month' => '',
            'OutTransactionID' => 0,
            'spend_id' => 0,
            'request_id' => $request_id
        ]);

    }

    public function createBill(Request $request)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $request_id = $request->req_id;

        if ($request->store_id == 0) {
            return redirect()->back()->with(['error' => 'قم بتحديد المخزن اولا']);
        }
        if ($request->sale_point == 0) {
            return redirect()->back()->with(['error' => 'قم بتحديد خزينه الدفع اولا ']);
        }
        
        $details = RequestDetails::where(['shop_id' => $shop_id, 'request_id' => $request_id])->selection()->get();
        foreach ($details as $detail) {
            $storeQty = StoreItem::where(['shop_id' => $shop_id, 'store_id' => $request->store_id, 'item_id' => $detail->item_id])->first();
            if ($storeQty == null) {
                return redirect()->back()->with(['error' => 'المنتج غير متوفر في المخزن حاليا ']);
            }
            if ($storeQty->store_quant < $detail->quantity) {
                return redirect()->back()->with(['error' => ' الكميه النطلوبه غير متاحه حاليا ']);
            }
        }
        
        $sale_point = SalePoint::where('shop_id', $shop_id)->find($request->sale_point);
        $requests = BadrRequest::where('shop_id', $shop_id)->where('id', $request_id)->With('requestDetails')->with('items')->first();
        $last_bill = Invoices::where('shop_id', $shop_id)->select('bill_no')->orderBy('id', 'Desc')->first();
        $last_bill = is_null($last_bill) ? 1 : $last_bill->bill_no + 1;

        $client = Client::find($requests->client_id);
        if ($client) {
            $balance = $client->balance;
        } else {
            $balance = 0;
        }
        if ($request->cash == 1) {
            $payment = $requests->net;
            $rest = 0;
        } else {
            //debit
            $payment = 0;
            $rest = $requests->net;
        }
        $n_balance = $balance - $rest;
        $client->update([
            'balance' => $n_balance
        ]);
        DB::transaction(function () use ($request_id, $shop_id, $request, $requests, $sale_point, $last_bill, $payment, $n_balance, $balance, $rest) {
            $invoice_id = DB::table('invoices_')->insertGetId([
                'bill_no' => $last_bill,
                'store_id' => $request->store_id,
                'invoice_date' => Carbon::now(),
                'client_id' => $requests->client_id,
                'shop_id' => $shop_id,
                'status' => $requests->status,
                'coupon_id' => $requests->coupon_id,
                'bill_total' => $requests->total,
                'net_price' => $requests->net,
                'payment' =>  $payment,
                'bill_payment' =>  $payment,
                'rest' => $rest,  //باقى
                'pay_date' => Carbon::today(),
                'add_user' => auth()->user()->id,
                'balance' => $n_balance,
                'invoice_type' => '1',
                'fee' => $requests->fee,
                'delivery_date' => $requests->delivery_date,
                'request_id' => $request_id,
                'order_no' => $requests->order_no,
            ]);
            $sale_point->increment('money_point', $requests->net);
            $this->storeDetail($requests, $invoice_id, $request_id, $shop_id, $request);
            $order = DB::table('requests')->where('id', $request_id)->first();
            DB::table('requests')->where('id', $request_id)->update([
                'no_bill' => $last_bill,
                'bill_id' => $invoice_id,
                'status' => $order->status == 2 ? 2 : 3
            ]);
            $this->saveTransaction($request_id, $requests, $shop_id, $n_balance, $rest, $payment, $balance, $sale_point, $invoice_id);
        });
        $requests = BadrRequest::where('shop_id', $shop_id)->where('id', $request_id)->With('requestDetails')->with('items')->first();
        $client = Client::find($requests->client_id);
        $data = [
            'type' => 'welcome',
            'msg' => ' order status change sucessfuly .....' . 'Your order number is' . $request_id
        ];
        $client->notify(new OrderNotify($data));
        return redirect()->back()->with(['success' => 'تم انشاء الفاتوره بنجاح']);
    }
}
