<?php
namespace Modules\BadrRequest\Http\Controllers;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Modules\BadrRequest\Entities\Bank;
use Modules\BadrRequest\Http\Requests\BankRequest;

class ShopBankController extends Controller
{

    public function index(Request $request)
    {
        $banks = Bank::where('shop_id', auth()->user()->shop_id)
        ->when($request->filled('bank_key'), function ($query) use ($request) {
            $query->where('account_number', 'LIKE', '%'.$request->bank_key.'%');
            $query->orWhere('account_iban', 'LIKE', '%'.$request->bank_key.'%');
            $query->orWhere('bank_name', 'LIKE', '%'.$request->bank_key.'%');
            $query->orWhere('account_name', 'LIKE', '%'.$request->bank_key.'%');
            $query->orWhere('branch', 'LIKE', '%'.$request->bank_key.'%');
        })->orderBy('id', 'desc')->paginate(15);
        return view('badrrequest::banks', compact('banks'));
    }

    public function store(BankRequest $request)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $bank = new Bank([
            'account_number' => $request->account_number,
            'account_iban' => $request->account_iban,
            'bank_name' => $request->bank_name,
            'account_name' =>$request->account_name,
            'branch' => $request->branch,
            'shop_id' =>$shop_id
        ]);
        $bank->save();
        $html = '<tr id="bank-'.$bank->id.'">
                    <th>#</th>
                    <th>'.$bank->bank_name.' <br> <span class="text-danger h3">' . $bank->account_name . '</span> </th>
                    <th>'. $bank->account_number .' </th>
                    <th>'. $bank->account_iban .' </th>
                    <th>
                        <a href="javascript:void(0)" data-href="'.route('requests.shopbanks.update').'"
                            data-id="'.$bank->id.'"
                            data-account_iban="'.$bank->account_iban.'"
                            data-account_number="'.$bank->account_number.'"
                            data-bank_name="'.$bank->bank_name.'"
                            data-account_name="'.$bank->account_name.'"
                            data-branch="'.$bank->branch.'"
                            class="btn btn-primary edit"><i class="fa fa-edit"></i></a>
                    </th>
                    <th>
                        <a href="javascript:void(0)" data-href="'.route('requests.shopbanks.delete').'"
                            data-id="'.$bank->id.'" class="btn btn-danger delete"><i
                                class="fa fa-trash"></i></a>
                    </th>
                </tr>';
        return response()->json([
            'status' => '1',
            'id' => $bank->id,
            'html' => $html
        ]);
    }


    public function update(BankRequest $request)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $bank = Bank::where([
            'shop_id' => $shop_id,
            'id' => $request->id
        ])->first();
        $bank->update([
            'account_number' => $request->account_number,
            'account_iban' => $request->account_iban,
            'bank_name' => $request->bank_name,
            'account_name' =>$request->account_name,
            'branch' => $request->branch,
        ]);

        $html = '<tr class="edited" id="bank-'.$bank->id.'">
                    <th>#</th>
                    <th>'.$bank->bank_name.' <br> <span class="text-danger h3">' . $bank->account_name . '</span> </th>
                    <th>'. $bank->account_number .' </th>
                    <th>'. $bank->account_iban .' </th>
                    <th>
                        <a href="javascript:void(0)" data-href="'.route('requests.shopbanks.update').'"
                            data-id="'.$bank->id.'"
                            data-account_iban="'.$bank->account_iban.'"
                            data-account_number="'.$bank->account_number.'"
                            data-bank_name="'.$bank->bank_name.'"
                            data-account_name="'.$bank->account_name.'"
                            data-branch="'.$bank->branch.'"
                            class="btn btn-primary edit"><i class="fa fa-edit"></i></a>
                    </th>
                    <th>
                        <a href="javascript:void(0)" data-href="'.route('requests.shopbanks.delete').'"
                            data-id="'.$bank->id.'" class="btn btn-danger delete"><i
                                class="fa fa-trash"></i></a>
                    </th>
                </tr>';

        return response()->json([
            'status' => '1',
            'id' => $bank->id,
            'html' => $html
        ]);
    }

    public function destroy(Request $request)
    {
        $bank = Bank::findOrFail($request->id);
        $bank->delete();
        return response()->json([
            'status' => '1'
        ]);
    }
}
