<?php

namespace Modules\BillReports\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Modules\BillReports\Entities\BillHistory;
use Modules\BillReports\Entities\BillsAdd;

class BillAddController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */

    public function index()
    {
        $settings    = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'Integer'))->first();
        $export_bill = $settings->export ?? 0;
        $adds        = BillsAdd::where('bills_add.shop_id', get_session_val('serial_id', 'Integer'))->join('users', 'users.id', 'bills_add.add_user');
        $adds        = ($settings->einvoice == 1) 
                        ? $adds->whereNotNull('bills_add.einvoice_code')->whereNotNull('sub_einvoice_code')
                        : $adds->whereNull('bills_add.einvoice_code')->whereNull('sub_einvoice_code');

        // if ($export_bill == 0) {
        //     $adds = $adds->where('check_bill_type', '!=', 2);
        // }

        $adds  = $adds->selectRaw('bills_add.*, users.user_name')->paginate(20);
        $taxsubcodes = json_encode(config('einvoice.taxsubcodes'));

        return view('billreports::bill-adds.index', compact('export_bill', 'settings', 'adds', 'taxsubcodes'));
    }

    /**
     * Show the form for creating a new resource.
     * @return Response
     */
    public function create()
    {
        return view('billreports::create', compact('taxes'));
    }

    /**
     * Store a newly created resource in storage.
     * @param  Request $request
     * @return Response
     */
    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'Addition_name'         => ['required', 'string', 'max:255', Rule::unique('bills_add','Addition_name')->where('shop_id', get_session_val('serial_id', 'Integer'))],
            'addition_value'        => ['required', 'numeric'],
            'tax_einvoice_code'     => ['nullable', 'string', 'max:255'],
            'tax_sub_einvoice_code' => ['nullable', 'string', 'max:255'],
            'vat_operation'         => ['nullable', 'boolean'],
        ], [
            'Addition_name.required'         => __('billreports::bills-add.add_name_required'),
            'Addition_name.string'           => __('billreports::bills-add.Addition_name.string'),
            'Addition_name.max'              => __('billreports::bills-add.Addition_name.max'),
            'Addition_name.unique'           => __('billreports::bills-add.Addition_name.unique'),
            'addition_value.required'        => __('billreports::bills-add.add_value_required'),
            'addition_value.numeric'         => __('billreports::bills-add.add_value_number'),
            'tax_einvoice_code.string'       => __('billreports::bills-add.tax_einvoice_code.string'),
            'tax_einvoice_code.max'          => __('billreports::bills-add.tax_einvoice_code.max'),
            'tax_einvoice_code.unique'       => __('billreports::bills-add.tax_einvoice_code.unique'),
            'tax_sub_einvoice_code.string'   => __('billreports::bills-add.tax_sub_einvoice_code.string'),
            'tax_sub_einvoice_code.max'      => __('billreports::bills-add.tax_sub_einvoice_code.max'),
            'tax_sub_einvoice_code.unique'   => __('billreports::bills-add.tax_sub_einvoice_code.unique'),
        ]);

        if ($validator->fails()) {

            $errors = $validator->errors();
            //            $error_data = [];
            $data = '<ul class="list-unstyled text-center alert alert-danger">';
            foreach ($errors->all() as $error) {
                $data .= '<li>' . $error . '</li>';
                //                array_push($error_data, $error);
            }
            $data .= '</ul>';

            $response = [
                'msg' => $data,
                // 'msg' => 'من فضلك أدخل جميع الحقول وتأكد من صحة رقم الهاتف',
                'status' => false
            ];
            return response()->json($response);
        }

        $bill_add = new BillsAdd();
        // $bill_add->add               = $request->add;
        $bill_add->shop_id           = get_session_val('serial_id', 'Integer');
        $bill_add->Addition_name     = $request->Addition_name;
        $bill_add->addition_value    = $request->addition_value;
        $bill_add->bill_add_method   = $request->bill_add_method ?? 0;
        $bill_add->check_addition    = $request->check_addition;
        $bill_add->add_resturant     = $request->add_resturant ?? 0;
        $bill_add->add_role          = $request->add_role;
        $bill_add->check_bill_type   = $request->checkBillType ?? 0;
        $bill_add->add_user          = auth()->id();
        $bill_add->tax_type          = $request->tax_type;
        $bill_add->einvoice_code     = $request->tax_einvoice_code;
        $bill_add->sub_einvoice_code = $request->tax_sub_einvoice_code;
        $bill_add->vat_operation     = $request->vat_operation;
        $bill_add->save();

        return response()->json(['status' => true]);
    }

    public function edit(Request $request)
    {

        $add = BillsAdd::where('bills_add.shop_id', get_session_val('serial_id', 'Integer'))
            ->join('users', 'users.id', 'bills_add.add_user')
            ->selectRaw('bills_add.*, users.user_name')->find($request->add_id);

        $taxcodes    = $add->tax_type == 0 ? config('einvoice.taxcodes') : config('einvoice.nontaxcodes');
        $settings    = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'Integer'))->first();
        $export_bill =  $settings->export ?? 0;
        $view        = view('billreports::bill-adds.edit', compact('add', 'export_bill', 'taxcodes', 'settings'))->render();

        return response()->json($view);
    }

    public function update(Request $request)
    {
        $settings    = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'Integer'))->first();
        $rules       = [
            'Addition_name'         => ['required', 'string', 'max:255', 'unique:bills_add,Addition_name,' . $request->add_id . ',id,shop_id,' . get_session_val('serial_id', 'Integer') ],
            'addition_value'        => ['required', 'numeric'],
            'vat_operation'         => ['nullable', 'boolean'],
        ];

        $messages   = [
            'Addition_name.required'  => __('billreports::bills-add.add_name_required'),
            'Addition_name.string'    => __('billreports::bills-add.Addition_name.string'),
            'Addition_name.max'       => __('billreports::bills-add.Addition_name.max'),
            'Addition_name.unique'    => __('billreports::bills-add.Addition_name.unique'),
            'addition_value.required' => __('billreports::bills-add.add_value_required'),
            'addition_value.numeric'  => __('billreports::bills-add.add_value_number'),
        ];

        if ($settings->einvoice == 1) {
            $rules['tax_einvoice_code']               = ['required', 'string', 'max:255', 'unique:bills_add,einvoice_code,' . $request->add_id . ',id'];
            $rules['tax_sub_einvoice_code']           = ['required', 'string', 'max:255', 'unique:bills_add,sub_einvoice_code,' . $request->add_id . ',id'];
            $messages['tax_einvoice_code.required']   = __('billreports::bills-add.tax_einvoice_code.required');
            $messages['tax_einvoice_code.string']     = __('billreports::bills-add.tax_einvoice_code.string');
            $messages['tax_einvoice_code.max']        = __('billreports::bills-add.tax_einvoice_code.max');
            $messages['tax_einvoice_code.unique']     = __('billreports::bills-add.tax_einvoice_code.unique');
            $messages['tax_sub_einvoice_code.string'] = __('billreports::bills-add.tax_sub_einvoice_code.string');
            $messages['tax_sub_einvoice_code.max']    = __('billreports::bills-add.tax_sub_einvoice_code.max');
            $messages['tax_sub_einvoice_code.unique'] = __('billreports::bills-add.tax_sub_einvoice_code.unique');
        }

        $validator = Validator::make($request->all(), $rules, $messages);

        if ($validator->fails()) {

            $errors = $validator->errors();
            //            $error_data = [];
            $data = '<ul class="list-unstyled text-center alert alert-danger">';
            foreach ($errors->all() as $error) {
                $data .= '<li>' . $error . '</li>';
                //                array_push($error_data, $error);
            }
            $data .= '</ul>';

            $response = [
                'msg' => $data,
                // 'msg' => 'من فضلك أدخل جميع الحقول وتأكد من صحة رقم الهاتف',
                'status' => false
            ];
            return response()->json($response);
        }

        $bill_add = BillsAdd::where('shop_id', get_session_val('serial_id', 'Integer'))->find($request->add_id);

        $bill_add->shop_id           = get_session_val('serial_id', 'Integer');
        $bill_add->Addition_name     = $request->Addition_name;
        $bill_add->addition_value    = $request->addition_value;
        $bill_add->bill_add_method   = $request->bill_add_method ?? 0;
        $bill_add->check_addition    = $request->check_addition;
        $bill_add->add_resturant     = $request->add_resturant ?? 0;
        $bill_add->add_role          = $request->add_role;
        $bill_add->check_bill_type   = $request->checkBillType ?? 0;
        $bill_add->tax_type          = $request->tax_type;
        $bill_add->einvoice_code     = $request->tax_einvoice_code;
        $bill_add->sub_einvoice_code = $request->tax_sub_einvoice_code;
        $bill_add->vat_operation     = $request->vat_operation;
        $bill_add->save();

        return response()->json(['status' => true]);
    }

    /**
     * Remove the specified resource from storage.
     * @return Response
     */
    public function destroy(Request $request)
    {
        $check = BillsAdd::join('item_vat', 'item_vat.vat_id', 'bills_add.id')
            ->where('bills_add.shop_id', get_session_val('serial_id', 'Integer'))
            ->where('bills_add.id', $request->add_id)
            ->count();

        if ($check > 0) {
            return response()->json(['status' => false, 'msg' => __('billreports::bills-add.delete_error')]);
        } else {
            BillsAdd::where('shop_id', get_session_val('serial_id', 'Integer'))->where('id', $request->add_id)->delete();
            return response()->json(['status' => true, 'msg' => __('billreports::bills-add.delete_success')]);
        }
    }
}
