<?php

namespace Modules\Car\Http\Controllers;

use Exception;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Car\Entities\Car;
use Modules\Car\Http\Requests\CarRequest;
use Modules\Car\Http\Services\CarService;

class CarController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index()
    {
        if (request()->ajax()) {
            $rows = Car::select('id', 'car_type', 'car_color', 'car_model', 'plate_number', 'avatar')->get();
            return view('car::includes.table.body', compact('rows'));
        }
        return view('car::index');
    }

    /**
     * Show the form for creating a new resource.
     * @return Renderable
     */
    public function create()
    {
        return view('car::create');
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Renderable
     */
    public function store(CarRequest $request, CarService $service)
    {
        $row = $service->handle( $request->validated() );

        if ($row instanceof Exception) throw new Exception( $row );
        return response()->json(['message' => trans('car::trans.title.created')], 200);
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function edit(Car $car)
    {
        return view('car::update', ['row' => $car]);
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Renderable
     */
    public function update(CarRequest $request, $car, CarService $service)
    {
        $row = $service->handle( $request->validated(), $car );

        if ($row instanceof Exception) throw new Exception( $row );
        return response()->json(['message' => trans('car::trans.title.updated')], 200);
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy(Car $car)
    {
        $car->delete();
        return response()->json(['message' => trans('car::trans.title.deleted')], 200);
    }
}
