<?php

namespace Modules\Category\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Controllers\Controller;
use App\Services\Uploaders\ItemImgUploader;
use Illuminate\Support\Facades\DB;
use Intervention\Image\Facades\Image;
use Modules\Category\Entities\CardCompany;
use Modules\Category\Entities\Category;
use Modules\Category\Entities\Item;
use Modules\Category\Entities\ItemColor;
use Modules\Category\Entities\ItemPrice;
use Modules\Category\Entities\ItemSize;
use Modules\Category\Entities\ItemUnit;
use Modules\Category\Entities\PriceList;
use Modules\Category\Entities\Supplier;
use Modules\Category\Entities\Company;
use Modules\Category\Entities\Color;
use Modules\Category\Entities\Size;
use Modules\Category\Entities\SupplierItem;
use Modules\Category\Entities\Unit;
use Modules\Category\Entities\Shelf;

class ItemController extends Controller
{

    public function index()
    {
        $items = Item::whereShopId(get_session_val('serial_id', 'integer'))
            ->select('id', 'item_name', 'quantity', 'sale_price', 'sale_unit', 'supplier_id')
            ->paginate(15);
        foreach ($items as $item) {
            $cat = Category::where('id', $item->sale_unit)->first();
            if ($cat) {
                if ($cat->category_id == 0) { # main_cat
                    $item->main_cat = $cat->name;
                    $item->sub_cat = '---';
                } else { # sub_cat
                    $item->sub_cat = $cat->name;
                    $main = Category::find($cat->category_id);
                    if ($main) {

                        $item->main_cat = $main->name;
                    } else {
                        $item->main_cat = '---';
                    }
                }

            } else {
                $item->main_cat = '---';
                $item->sub_cat = '---';
            }

            $supplier = Supplier::where('id', $item->supplier_id)->first();
            if ($supplier) {
                $item->supplier = $supplier->supplier_name;
            } else {
                $item->supplier = '---';
            }
        }

        return view('category::items.index', compact('items'));
    }

    public function create()
    {
        $suppliers = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        $price_decimal = get_session_val('decimal_num_price', 'integer');
        $type = get_session_val('date_type', 'integer');

        $main_cats = Category::where('category_id', 0)->where('shop_id', get_session_val('serial_id', 'integer'))->get();

        $units = Unit::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        $shelves = Shelf::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        $companies = Company::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        $colors = Color::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        $sizes = Size::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        $pricesList = PriceList::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        $cardCompany = CardCompany::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        return view('category::items.create', compact('cardCompany', 'pricesList', 'sizes', 'colors', 'companies', 'shelves', 'units', 'main_cats', 'suppliers', 'type', 'price_decimal'));
    }

    public function store(Request $request)
    {

        $prices = $request->except('item_name', 'supplier_id', 'quantity', 'sale_unit',
            'unit_id', 'pay_price', 'sale_price', 'image', 'sub_cat', 'barcode', 'expire_date',
            'lowest_price', 'min_quantity', 'pay_date', 'available', 'game', 'about', 'details',
            'images', 'is_service', 'online', 'company_id', 'shelf', 'background_color', 'cooking_add',
            'order_show', 'sizes', 'colors', 'bolla', 'meter_ceramic', 'require_serial', 'card_company_id',
            'sub_units', '_token', 'intern_cat', "small_qty", "medium_qty", "large_qty");

        $this->validate($request, [
            'item_name' => 'required',
            'supplier_id' => 'required',
            'quantity' => 'required',
            'sale_unit' => 'required',
            'unit_id' => 'required',
            'pay_price' => 'required',
            'sale_price' => 'required',
            'image' => 'mimes:jpeg,bmp,png,jpg'
        ]);

//        try
//        {
        DB::transaction(function () use ($request, $prices) {

            # SAVE ITEM:
                #get category_id
/*
            if (is_null($request->sub_cat)) {
                $cat_id = $request->sale_unit;
            } else {
                if (is_null($request->intern_cat)) {
                    $cat_id = $request->sub_cat;
                } else {
                    $cat_id = $request->intern_cat;
                }
            }
*/
            $item = new Item();
            $item->item_name = $request->item_name;
            $item->barcode = ($request->barcode) ? $request->barcode[0] : '';
            $item->sale_unit = $request->sale_unit;
            $item->sale_price = $request->sale_price;
            $item->lowest_price = $request->lowest_price;
            $item->pay_price = $request->pay_price;
            $item->quantity = $request->quantity;
            $item->unit_id = $request->unit_id;
            $item->expire_date = ($request->expire_date) ? $request->expire_date : date('Y-m-d', strtotime('+20 years'));;
            $item->min_quantity = $request->min_quantity;
            $item->incom_date = $request->pay_date;
            $item->available = ($request->available) ? 1 : 0;
            $item->game = ($request->game) ? 1 : 0;
            $item->about = $request->about;
            $item->supplier_id = $request->supplier_id[0];
            $item->details = ($request->details) ? $request->details : '';
            $item->edit_user = auth()->id();

            $item->used_for_small = $request->small_qty ?? 1;
            $item->used_for_medium = $request->medium_qty ?? 1;
            $item->used_for_large = $request->large_qty ?? 1;

            $item->img = '';

            if ($request->has('images')) {

                $uploader     =  new ItemImgUploader($request, 'images');
                $result  =  $uploader->checkAndUpload();
                if ($result['status'])
                {
                    $item->img = $result['path'];
                }

            }

            $item->add_user = auth()->id();
            $item->shop_id = get_session_val('serial_id', 'integer');
            $item->is_service = $request->is_service;
            $item->online = ($request->online) ? 1 : 0;
            $item->company_id = $request->company_id;
            $item->shelf = $request->shelf;
            $item->edit_date = date('Y-m-d H:i:s');

            if ($request->has('background_color')) {
                $item->background_color = $request->background_color;
            }
            if ($request->has('cooking_add')) {
                $item->cooking_add = $request->cooking_add;
            }
            if ($request->has('order_show')) {
                $item->order_show = $request->order_show;
            }
            if ($request->has('meter_ceramic')) {
                $item->meter_ceramic = $request->meter_ceramic;
            }
            if ($request->has('require_serial')) {
                $item->require_serial = $request->require_serial;
            }
            if ($request->has('card_company_id')) {
                $item->card_company_id = $request->card_company_id;
            }
            if ($request->has('bolla')) {
                $item->bolla = $request->bolla;
            }

            $item->disc_value = 0;
            $item->device_id = 0;
            $item->pur_unit = 0;
            $item->pur_unit_equal = 0;
            $item->item_group = 0;

            $item->save();


            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            if ($request->has('sizes')) {
                foreach ($request->sizes as $key => $val) {
                    $itemSize = new ItemSize();
                    $itemSize->item_id = $item->id;
                    $itemSize->size_id = $val;
                    $itemSize->shop_id = get_session_val('serial_id', 'integer');
                    $itemSize->add_user = auth()->id();
                    $itemSize->edit_user = auth()->id();
                    $itemSize->add_date = date('Y-m-d H:i:s');
                    $itemSize->save();
                }
            }

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            if ($request->has('colors')) {
                foreach ($request->colors as $k => $v) {
                    $itemColor = new ItemColor();
                    $itemColor->item_id = $item->id;
                    $itemColor->color_id = $val;
                    $itemColor->shop_id = get_session_val('serial_id', 'integer');
                    $itemColor->edit_user = auth()->id();
                    $itemColor->add_user = auth()->id();
                    $itemColor->add_date = date('Y-m-d H:i:s');
                    $itemColor->edit_date = date('Y-m-d H:i:s');
                    $itemColor->save();
                }
            }

            foreach ($request->supplier_id as $value) {

                $s_i = new SupplierItem();
                $s_i->item_id = $item->id;
                $s_i->supplier_id = $value;
                $s_i->shop_id = get_session_val('serial_id', 'integer');
                $s_i->save();

            }

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            if (sizeof($prices) > 0) {

                foreach ($prices as $key => $value) {

                    $x = explode('-', $key);
                    $price_id = $x[1];

                    $item_price = new ItemPrice();
                    $item_price->item_id = $item->id;
                    $item_price->list_id = $price_id;
                    $item_price->price = $value;
                    $item_price->shop_id = get_session_val('serial_id', 'integer');
                    $item_price->add_user = auth()->id();
                    $item_price->add_date = date('Y-m-d H:i:s');
                    $item_price->edit_user = auth()->id();
                    $item_price->edit_date = date('Y-m-d H:i:s');

                    $item_price->save();

                }

            }

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

         /*   `Save the main unit for this item
            ## Saved data will like:
                1. unit_type = 3;
                2. unit_value = 1;
                3. unit_price = same item's sale_price;
                4. unit_barcode = first barcode;
                5. unit_id = same item's unit_id;
             # that's it for now XD`;*/

            if (sizeof($request->barcode) > 1) {
                if ($request->barcode[0] == null) {
                    $barcode_ = "";
                }


                $main_unit = new ItemUnit();
                $main_unit->item_id = $item->id;
                $main_unit->unit_id = $request->unit_id;
                $main_unit->unit_type = 3;
                $main_unit->unit_value = 1;
                $main_unit->unit_price = $item->sale_price;
                $main_unit->Screenhidden = 0;
                $main_unit->shop_id = get_session_val('serial_id', 'integer');
                $main_unit->unit_barcode = $barcode_;
                $main_unit->save();
            }


            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

          /*  `loop through other barcodes for this item
             # it will be saved in items_unit table.
             the other data will like:
                1. unit_type = 2 => equals the main unit (3);
                2. unit_value = 1;
                3. unit_price = same item's sale_price;
                4. unit_barcode = this looped iteration barcode;
                5. unit_id = same item's unit_id;
             # that's it for now XD`;*/

            if (sizeof($request->barcode) > 1) {
                for ($b = 1; $b < sizeof($request->barcode); $b++) {
                    $b_unit = new ItemUnit();
                    $b_unit->item_id = $item->id;
                    $b_unit->unit_id = $item->unit_id;
                    $b_unit->unit_type = 2;
                    $b_unit->unit_value = 1;
                    $b_unit->unit_price = $item->sale_price;
                    $b_unit->Screenhidden = 0;
                    $b_unit->shop_id = get_session_val('serial_id', 'integer');
                    $b_unit->unit_barcode = $request->barcode[$b];
                    $b_unit->save();
                }
            }

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

         /*   `loop through sub units for this item
             # it will be saved in items_unit table.
             the other data will like:
                1. unit_type = [0 => bigger or 1 => smaller];
                2. unit_value = from request;
                3. unit_price = from request;
                4. unit_barcode = from request;
                5. unit_id = from request;
             # that's it`;*/

            if ($request->has('sub_units')) {

//                `The Request is like this:
//                    ## string that is separated with (-).
//                    ## Sigmants are in order:
//                        1. unit_barcode.
//                        2. unit_id.
//                        3. unit_price.
//                        4. unit_type.
//                        5. unit_value.
//                        6. Screenhidden
//                Now, we are ready to go!`;

                foreach ($request->sub_units as $k => $val) {

                    $unit_arr = explode('-', $val);

                    $barcode = $unit_arr[0];
                    $u_id = $unit_arr[1];
                    $price = $unit_arr[2];
                    $type = $unit_arr[3];
                    $value = $unit_arr[4];
                    $onscreen = $unit_arr[5];

                    $sub = new ItemUnit();
                    $sub->item_id = $item->id;
                    $sub->unit_id = $u_id;
                    $sub->unit_type = $type;
                    $sub->unit_value = $value;
                    $sub->unit_price = $price;
                    $sub->Screenhidden = $onscreen;
                    $sub->shop_id = get_session_val('serial_id', 'integer');
                    $sub->unit_barcode = $barcode;
                    $sub->save();

                }
            }

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

        });

        return redirect()->back()->with([
            'success' => 'تم الحفظ بنجاح'
        ]);
//        }catch (\Exception $exception){
//            return redirect()->back()->with([
//                'error' => 'خطأ في عملية الحفظ'
//            ]);
//        }


    }

    public function show($id)
    {
        $item = Item::find($id);
        if ($item) {

            $cat = Category::where('id', $item->sale_unit)->first();
            if ($cat) {
                if ($cat->category_id == 0) { # main_cat
                    $item->main_cat = $cat->name;
                    $item->sub_cat = '---';
                } else { # sub_cat
                    $item->sub_cat = $cat->name;
                    $main = Category::find($cat->category_id);
                    if ($main) {

                        $item->main_cat = $main->name;
                    } else {
                        $item->main_cat = '---';
                    }
                }

            } else {
                $item->main_cat = '---';
                $item->sub_cat = '---';
            }

            $supplier = Supplier::where('id', $item->supplier_id)->first();
            if ($supplier) {
                $item->supplier = $supplier->supplier_name;
            } else {
                $item->supplier = '---';
            }

            $unit = Unit::find($item->unit_id);
            $item->unit = $unit->name;

            return view('category::items.show', compact('item'));
        } else {
            abort(404);
        }
    }

    public function edit($id)
    {
        $item = Item::find($id);
        if ($item) {
            $suppliers = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))->get();

            $price_decimal = get_session_val('decimal_num_price', 'integer');
            $type = get_session_val('date_type', 'integer');

            $main_cats = Category::where('category_id', 0)->where('shop_id', get_session_val('serial_id', 'integer'))->get();

            $units = Unit::where('shop_id', get_session_val('serial_id', 'integer'))->get();

            $shelves = Shelf::where('shop_id', get_session_val('serial_id', 'integer'))->get();

            $companies = Company::where('shop_id', get_session_val('serial_id', 'integer'))->get();

            $colors = Color::where('shop_id', get_session_val('serial_id', 'integer'))->get();

            $sizes = Size::where('shop_id', get_session_val('serial_id', 'integer'))->get();

            $pricesList = PriceList::where('shop_id', get_session_val('serial_id', 'integer'))->get();

            $cardCompany = CardCompany::where('shop_id', get_session_val('serial_id', 'integer'))->get();

            $cat = Category::find($item->sale_unit);
            if ($cat) {
                if ($cat->category_id == 0) {
                    $main_cat_check = 1;
                    $subs = $cat->subCategory();
                } else {
                    $main_cat_check = 0;
                    $main = Category::find($cat->category_id);
                    $m_subs = $main->subCategory();
                }
            } else {
                $main_cat_check = 2;
            }

            $item->quantity = quant_decimal($item->quantity);
            $item->sale_price = price_decimal($item->sale_price);
            $item->pay_price = price_decimal($item->pay_price);

            $sizes_ = ItemSize::where('item_id', $item->id)->pluck('size_id');
            $colors_ = ItemColor::where('item_id', $item->id)->pluck('color_id');
            $size_array = json_decode(json_encode($sizes_));
            $color_array = json_decode(json_encode($colors_));


            foreach ($pricesList as $list) {
                $item_price = ItemPrice::where('item_id', $item->id)
                    ->where('list_id', $list->id)
                    ->first();
                if ($item_price) {
                    $list->price = price_decimal($item_price->price);
                } else {
                    $list->price = 0;
                }
            }

            return view('category::items.edit', compact('item_prices', 'size_array', 'color_array', 'm_subs', 'main', 'cat', 'subs', 'main_cat_check', 'item', 'cardCompany', 'pricesList', 'sizes', 'colors', 'companies', 'shelves', 'units', 'main_cats', 'suppliers', 'type', 'price_decimal'));
        } else {
            abort(404);
        }
    }

    public function update(Request $request)
    {

    }

    public function destroy(Request $request)
    {
        $item = Item::find($request->item_id);

        if ($item) {

            ItemSize::where('item_id', $item->id)->delete();
            ItemColor::where('item_id', $item->id)->delete();
            ItemUnit::where('item_id', $item->id)->delete();
            ItemPrice::where('item_id', $item->id)->delete();

            return response()->json(['status' => true]);
        }

        return response()->json(['status' => false]);
    }

    public function storeSupplier(Request $request)
    {
        $this->validate($request, [
            'supplier_name' => 'required'
        ]);

        $supplier = new Supplier();
        $supplier->supplier_name = $request->supplier_name;
        $supplier->tel = $request->tele;
        $supplier->shop_id = get_session_val('serial_id', 'integer');
        $supplier->save();

        $option = '<option selected value="' . $supplier->id . '">' . $supplier->supplier_name . '</option>';

        $list = '<li data-original-index="' . $supplier->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $supplier->supplier_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json([
            'status' => true,
            'option' => $option,
            'list' => $list,
        ]);
    }

    public function storeShelf(Request $request)
    {
        $this->validate($request, [
            'shelf_name' => 'required'
        ]);

        $shelf = new Shelf();
        $shelf->shelf_name = $request->shelf_name;
        $shelf->shop_id = get_session_val('serial_id', 'integer');
        $shelf->add_user = auth()->id();
        $shelf->edit_user = auth()->id();
        $shelf->edit_date = date('Y-m-d H:i:s');
        $shelf->store_id = 0;

        $shelf->save();

        $option = '<option selected value="' . $shelf->id . '">' . $shelf->shelf_name . '</option>';

        $list = '<li data-original-index="' . $shelf->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $shelf->shelf_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json([
            'status' => true,
            'option' => $option,
            'list' => $list,
        ]);
    }

    public function storeCompany(Request $request)
    {
        $this->validate($request, [
            'company_name' => 'required',
            'income_ratio' => 'required',
            'sale_ratio' => 'required',
        ]);

        $company = new Company();
        $company->company_name = $request->company_name;
        $company->income_ratio = $request->income_ratio;
        $company->sale_ratio = $request->sale_ratio;
        $company->add_user = auth()->id();
        $company->add_date = date('Y-m-d H:i:s');
        $company->shop_id = get_session_val('serial_id', 'integer');
        $company->save();

        $option = '<option selected value="' . $company->id . '">' . $company->company_name . '</option>';

        $list = '<li data-original-index="' . $company->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $company->company_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json([
            'status' => true,
            'option' => $option,
            'list' => $list,
        ]);
    }

    public function storeCardCompany(Request $request)
    {
        $this->validate($request, [
            'company_name' => 'required',
            'card_count' => 'required',

        ]);

        $company = new CardCompany();
        $company->company_name = $request->company_name;
        $company->number_digital = $request->card_count;
        $company->shop_id = get_session_val('serial_id', 'integer');
        $company->save();

        $option = '<option selected value="' . $company->id . '">' . $company->company_name . '</option>';

        $list = '<li data-original-index="' . $company->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $company->company_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json([
            'status' => true,
            'option' => $option,
            'list' => $list,
        ]);
    }

    public function storeColor(Request $request)
    {
        $this->validate($request, [
            'color_name' => 'required',
            'color_code' => 'required',

        ]);

        $color = new Color();
        $color->color_name = $request->color_name;
        $color->color_code = $request->color_code;
        $color->shop_id = get_session_val('serial_id', 'integer');
        $color->save();

        $option = '<option selected value="' . $color->id . '">' . $color->color_name . '</option>';

        $list = '<li data-original-index="' . $color->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $color->color_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json([
            'status' => true,
            'option' => $option,
            'list' => $list,
        ]);
    }

    public function storeSize(Request $request)
    {
        $this->validate($request, [
            'size_name' => 'required',
            'size_code' => 'required',

        ]);

        $size = new Size();
        $size->size_name = $request->size_name;
        $size->size_code = $request->size_code;
        $size->shop_id = get_session_val('serial_id', 'integer');
        $size->save();

        $option = '<option selected value="' . $size->id . '">' . $size->size_name . '</option>';

        $list = '<li data-original-index="' . $size->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $size->size_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json([
            'status' => true,
            'option' => $option,
            'list' => $list,
        ]);
    }

    public function ajaxSubCats(Request $request)
    {
        $change = $request->change;
        $main_id = $request->main_id;
        $cat = Category::where('id', $main_id)->first();
        if ($cat) {
            $subs = Category::where('category_id', $cat->id)->where('shop_id', get_session_val('serial_id', 'integer'))->get();
            $data = ',<label class="control-label col-md-3">التصنيف الفرعي</label>';
            $data .= '<select style="margin-top: -1em;" name="sub_cat" id="sub-cat-items" class="form-control col-md-9">';
            $data .= '<option value=""></option>';
            foreach ($subs as $sub) {
                $data .= '<option value="' . $sub->id . '">' . $sub->name . '</option>';
            }

            $data .= '</select>';
            if ($change == 1) {
                return response()->json([
                    'status' => false,
                    'change' => true,
                    'data' => $data
                ]);
            } else {
                return response()->json([
                    'status' => true,
                    'change' => false,
                    'data' => $data
                ]);
            }

        } else {
            return response()->json([
                'status' => false,
            ]);
        }
    }

    public function ajaxInternSubCats(Request $request)
    {
        $change = $request->change;
        $main_id = $request->main_id;
        $cat = Category::where('id', $main_id)->first();
        if ($cat) {
            $subs = Category::where('category_id', $cat->id)->where('shop_id', get_session_val('serial_id', 'integer'))->get();
            $data = ',<label class="control-label col-md-3">التصنيف الفرعي</label>';
            $data .= '<select style="margin-top: -1em;" name="intern_cat" id="intern-sub-cat-items" class="form-control col-md-9">';
            $data .= '<option value=""></option>';
            foreach ($subs as $sub) {
                $data .= '<option value="' . $sub->id . '">' . $sub->name . '</option>';
            }

            $data .= '</select>';
            if ($change == 1) {
                return response()->json([
                    'status' => false,
                    'change' => true,
                    'data' => $data
                ]);
            } else {
                return response()->json([
                    'status' => true,
                    'change' => false,
                    'data' => $data
                ]);
            }

        } else {
            return response()->json([
                'status' => false,
            ]);
        }
    }

    public function ajaxSubUnits(Request $request)
    {

        $this->validate($request, [
            'unit_barcode' => "required",
            'unit_id' => "required",
            'unit_price' => "required",
            'unit_type' => "required",
            'unit_value' => "required",
        ]);

        if ($request->has('Screenhidden')) {
            $Screenhidden = 0;
        } else {
            $Screenhidden = 1;
        }

        $unit = Unit::find($request->unit_id);
        if ($unit) {
            $data = '<tr class="info text-center" id="' . $request->unit_barcode . '-' . $request->unit_id . '">';

            $data .= '<td>' . $unit->name . '</td>';
            $data .= '<td>' . quant_decimal($request->unit_value) . '</td>';
            $data .= '<td>' . price_decimal($request->unit_price) . '</td>';
            $data .= '<td>' . $request->unit_barcode . '</td>';
            $data .= '<td>
                        <a href="javascript:void(0)" class="btn btn-danger delete-sub-unit"
                         data-barcode="' . $request->unit_barcode . '" data-unit="' . $request->unit_id . '">حذف</a>
                      </td>';
            $data .= '</tr>';

            $data_hidden = '<input type="hidden" id="input-' . $request->unit_barcode . '-' . $request->unit_id . '" name="sub_units[]"
             value="' . $request->unit_barcode . '-' . $request->unit_id . '-' . $request->unit_price . '-' . $request->unit_type . '-' . $request->unit_value . '-' . $Screenhidden . '">';

            $n_data_hidden = '<input type="hidden" id="input-' . $request->unit_barcode . '-' . $request->unit_id . '" name="new_sub_units[]"
             value="' . $request->unit_barcode . '-' . $request->unit_id . '-' . $request->unit_price . '-' . $request->unit_type . '-' . $request->unit_value . '-' . $Screenhidden . '">';

            return response()->json([
                'data' => $data,
                'hidden' => $data_hidden,
                'n_hidden' => $n_data_hidden,
                'status' => true
            ]);
        } else {
            return response()->json([
                'status' => false,
                'data' => 'خطأ في الوحدة'
            ]);
        }


    }


}
