<?php

namespace Modules\ClientInstallments\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Modules\ClientInstallments\Entities\Client;
use Modules\ClientInstallments\Entities\ClientTransaction;
use Modules\ClientInstallments\Entities\Instalment;
use Modules\ClientInstallments\Entities\Employee;

use Modules\ClientInstallments\Entities\Helper;

class ClientInstallmentReportController extends Controller
{

    public function clientsReport()
    {
        $type = get_session_val('date_type', 'integer');
        $clients = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereIn('client_role', [0, 2])->select('id', 'client_name', 'balance')->get();
        return view('clientinstallments::clients-report', compact('clients', 'type'));
    }

    public function clientsReportResult(Request $request)
    {
        $this->validate($request, [
            'client_id' => 'required',
            'period' => 'required',
        ]);

        $today = $request->today;

        $today_date = Carbon::parse($today)->format('Y-m-d');

        $p = $request->period;
        $period = '--';
        if ($p == '1') {
            $period = 'شهر';
        } elseif ($p == '3') {
            $period = '3 شهور';
        } elseif ($p == '6') {
            $period = '6 شهور';
        } elseif ($p == '12') {
            $period = 'سنة';
        }

        $client_id = $request->client_id;

        if ($client_id == '0') {
            $clients = Client::where('shop_id', get_session_val('serial_id', 'integer'))
                ->whereIn('client_role', [0, 2])
                ->where('balance', '!=', 0)
                ->pluck('id');
            $client_name = 'الكل';
        } else {
            $clients = Client::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $request->client_id)
                ->whereIn('client_role', [0, 2])
                ->pluck('id');
            $client_name = Client::find($request->client_id)->client_name;
        }

        $needed_clients = Client::whereIn('id', $clients)
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->select('id', 'client_name', 'balance','credit_limit')
            ->get();

        //        if ($clients->count() > 0){
        //            $clients_ids = json_decode(json_encode($clients));
        //        }else {
        //            $clients_ids = [];
        //        }
        //        dd($clients_ids); # if pluck
        //         dd($clients_ids[0]->id); # if select

        //        $history_clients = Instalment::whereIn('client_id', $clients_ids)
        //            ->where('shop_id', get_session_val('serial_id', 'integer'))->pluck('client_id');
        //        $history_clients = json_decode(json_encode($history_clients));
        //        $history_clients = array_unique($history_clients);
        //        dd($history_clients);
        //        $matched_clients_ids = array_intersect($history_clients, $clients_ids);
        //        $matched_clients_ids = value($matched_clients_ids);
        //        dd($matched_clients_ids);

        //        $needed_clients = Model::whereIn('id', $matched_clients_ids)
        //            ->where('shop_id', get_session_val('serial_id', 'integer'))
        //            ->select('id', 'client_name', 'balance', 'credit_limit')
        //            ->get();
        //        dd($needed_clients);

        foreach ($needed_clients as $needed_client) {
            $delaies = Instalment::where('client_id', $needed_client->id)
                ->where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('installment_pay', 0)
                ->where('type', 0)
                ->whereDate('installment_date', '<', $today_date)
                ->sum('installment_amount');
            $needed_client->delaies = $delaies;

            $needed_client->data = Helper::getColumnsData($p, $today, $needed_client->id);
        }

        $columns = Helper::getColumns($p, $today);
        $subColumns = Helper::getSubColumns($p);
        //        dd($columns);
        return view('clientinstallments::clients-report-result', compact('subColumns', 'client_name', 'columns', 'needed_clients', 'period', 'p'));
    }

    public function lateInstallmentsReport(Request $request)
    {
        $fromDate = $request->from ?? Carbon::today()->format('Y-m-d');
        $toDate = $request->to ?? Carbon::today()->format('Y-m-d');

        $delaies = Instalment::with('client:id,client_name as name')
            ->where('installment_pay', 0)
            ->where('type', 0)
            ->whereDate('installment_date', '>=', $fromDate)
            ->whereDate('installment_date', '<=', $toDate)
            ->where('shop_id', auth()->user()->shop_id)->get();

        return view('clientinstallments::delaies-report', compact('delaies'));
    }

    public function installmentEmployeeReport(Request $request)
    {
        $employees = Employee::where([
            'shop_id' => auth()->user()->shop_id,
            'is_employee' => 1
        ])->selectRaw('id, name, money_percent')->get();


        $instalments = collect();
        $instalments_sum = 0;
        $percent_sum = 0;

        if ($request->all() != []) {
            $instalments = ClientTransaction::where('installment_id', '>', '0')
                ->where('client_transaction.shop_id', get_session_val('serial_id', 'integer'));

            if (request('employee') && !in_array(request('employee'), ['', null])) {
                $instalments = $instalments->where('employee_id', request('employee'));
            }
            if (request('from') && !in_array(request('from'), ['', null])) {
                $instalments = $instalments->whereDate('installment_date', '>=', request('from'));
            }
            if (request('to') && !in_array(request('to'), ['', null])) {
                $instalments = $instalments->whereDate('installment_date', '<=', request('to'));
            }

            $instalments = $instalments->join('users', 'users.id', 'client_transaction.employee_id')
                ->join('clients', 'clients.id', 'client_transaction.client_id')
                ->join('installments_list', 'installments_list.id', 'client_transaction.installment_id')
                ->selectRaw('pay_date, installment_date, amount, client_transaction.client_id, installment_id,
                employee_id, users.name, users.collecting_ratio,clients.client_name,
                client_transaction.money_percent, (amount *  users.collecting_ratio/ 100) as percent_amount')
                ->get();
            $instalments_sum = $instalments->sum('amount');
            $percent_sum = $instalments->sum('percent_amount');
        }
        // dd($percent_sum);
        return view('clientinstallments::install_employee_report', compact('employees', 'instalments', 'instalments_sum', 'percent_sum'));
    }
}
