<?php

namespace Modules\ClientInstallments\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Models\systemLog;
use Auth;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Modules\ClientInstallments\Entities\Client;
use Modules\ClientInstallments\Entities\ClientTransaction;
use Modules\ClientInstallments\Entities\Employee;
use Modules\ClientInstallments\Entities\Instalment;
use Modules\ClientInstallments\Entities\SalePoint;
use Modules\Invoices\Entities\Invoices;
use Redirect;


class ClientInstallmentsController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index()
    {

        // if( !request()->has('ctype')){
        //     return redirect('/')->with('error','عنوان خاطئ');
        // }

        $clientType = request('ctype') ?? '';
        if($clientType){
            $clients = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereIn('client_role',[2,0])->get(['id', 'client_name', 'balance']);
        }else{
            $clients = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereIn('client_role',[2,0])
            ->get(['id', 'client_name', 'balance']);
        }

        $employees = Employee::select('id', 'name')->where('collecting', 1)->get();

        // $client_id = request('client');
        $invoice_id = request('bill');
        // $dates['from'] = request('from') ? request('from') : Carbon::today()->format('Y-m-d');
        // $dates['to'] = request('to') ? request('to') : Carbon::today()->format('Y-m-d');

        // $instalmentType = request('itype');
        $title = $clientType == 0 ? 'العملاء' : 'الموردين';

        $instalments = Instalment::where('installments_list.shop_id', get_session_val('serial_id', 'integer'))
            ->select('installments_list.*', 'users.user_name',)
            ->orderBy('bill_id', 'desc')
            ->orderBy('installment_date', 'asc')
            ->join('clients', 'installments_list.client_id', 'clients.id')
            ->leftJoin('client_transaction', 'installments_list.id', 'client_transaction.installment_id')
            ->leftJoin('users', 'users.id', 'client_transaction.employee_id');

        if (request('from') && !in_array(request('from'), ['', null])) {
            $instalments = $instalments->whereDate('installment_date', '>=', request('from'));
        }

        if (request('to') && !in_array(request('to'), ['', null])) {
            $instalments = $instalments->whereDate('installment_date', '<=', request('to'));
        }

        if (request('ctype') && !in_array(request('ctype'), ['', null])) {
            $instalments = $instalments->where('installments_list.type',$clientType);
        }

        if (request('supplier') && !in_array(request('supplier'),['', null]) ){
            $instalments = $instalments->where('installments_list.client_id', request('supplier') );
        }

        $format = request('format');
        if ($format == 2) {
            $format = 0;
        }
        if (in_array($format, ['1', '0']) && request('format') != '0') {
            $instalments = $instalments->where('installment_pay', $format);
        }


        // $showCashButton = false;
        $totalUnPaid = 0;
        $AllInsatllments = 0;
        $instalments_sum = 0;
        $page_sum = 0;

        if($invoice_id){
            $invoice = Invoices::where(['shop_id' => get_session_val('serial_id', 'integer'), 'id'=>$invoice_id])->first();

            $instalments = $instalments->where('installments_list.bill_id', $invoice->id);

            $billUrl = route('clients.show-sale-bill', request('bill'));
            $billUrl = config('main.badr') . 'bill/p1-inst.php?id=' . request('bill');

            $all_installments = $invoice->net_price -  $invoice->payment;
            $instalmentsAdd = $invoice->insatllments_add;
            $net_rest = $all_installments - $instalmentsAdd;
            $instalmentPercentage = $instalmentsAdd > 0 ? ($instalmentsAdd / $net_rest) : 0;
            $instalments_sum = $instalments->sum('installment_amount');
            $instalments = $instalments->paginate(20);
            $page_sum = $instalments->sum('installment_amount');

            foreach ($instalments as $instalment) {
                if ($instalment->installment_pay == 0) {
                    // $showCashButton = true;
                    $totalUnPaid += $instalment->installment_amount;
                    $AllInsatllments += $instalment->installmentAddition($instalmentPercentage);
                }
            }
        }else{
            $instalments_sum = $instalments->sum('installment_amount');
            $instalments = $instalments->paginate(20);
            $page_sum = $instalments->sum('installment_amount');
        }

        $salePoints = $this->allowedStores();
        $billUrl = '';

        return view('clientinstallments::index', compact('instalments', 'employees', 'billUrl', 'clients', 'salePoints', 'title', 'instalments_sum', 'page_sum', 'AllInsatllments', 'totalUnPaid'));
    }

    public function printInstallment($id) {
        $instalments = Instalment::where('installments_list.shop_id', get_session_val('serial_id', 'integer'))
            ->where('installments_list.type', 0)
            ->where('installments_list.id', $id)
            ->select('installments_list.*', 'users.user_name', 'clients.client_name')
            ->orderBy('bill_id', 'desc')
            ->join('clients', 'installments_list.client_id', 'clients.id')
            ->leftJoin('client_transaction', 'installments_list.id', 'client_transaction.installment_id')
            ->leftJoin('users', 'users.id', 'client_transaction.employee_id')->first();

        return view('clientinstallments::print.paid-instalment', compact('instalments'));
    }

    /**
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function delete($id)
    {
        $instalment = Instalment::where('id', $id)->where('shop_id', get_session_val('serial_id', 'integer'))
            ->first();
        if ($instalment) {
            $instalment->delete();
            session()->flash('success', 'تمت عملية حذف القسط بنجاح');
            return back();
        }
        return back();
    }

    /**
     * This method return the instalment that requested by the user
     * @param Request $request
     * @return null|string|void
     */
    public function getByTypeAjax(Request $request)
    {
        $date = $request->today;
        $date = Carbon::parse($date)->format('Y-m-d');
        $type = $request->type;
        $supplier = $request->supplier;
        $instalments = null;
        $instalments = $this->getInstalmentsBasedOnType($type, $supplier, $date);
        $instalments = $this->formatInstalments($instalments);
        return $instalments;
    }

    /**
     * This method return the outstanding instalments for both all suppliers
     * or a certain one based on the $supplier parameter value
     * if the $supplier value equal zero then get all records otherwise get the target one
     * @param $supplier
     * @return mixed
     */
    private function getLateInstalment($supplier, $date)
    {
        return Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where(function ($q) use ($supplier) {
                if ($supplier != 0) {
                    $q->where('installments_list.client_id', $supplier);
                }
            })
            ->where('installment_pay', 0)
            ->where('type', 0)
            ->whereDate('installment_date', '<', $date)
            ->orderBy(\DB::raw('installment_date'))
            ->get();
    }

    /**
     * This method return the paid instalment for both all clients
     * or a certain one based on the $client parameter value
     * if the $client value equal zero then get all records otherwise get the target one
     * @param $client
     * @return mixed
     */
    private function getPaidInstalment($client)
    {
        return Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where(function ($q) use ($client) {
                if ($client != 0) {
                    $q->where('installments_list.client_id', $client);
                }
            })
            ->where('installment_pay', 1)
            ->where('type', 0)
            ->orderBy(\DB::raw('installment_date'))
            ->get();
    }

    /**
     * This method return the all instalments for both all clients
     * or a certain one based on the $client parameter value
     * if the $client value equal zero then get all records otherwise get the target one
     * @param $client
     * @return mixed
     */
    private function getAllInstallment($client)
    {
        return Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where(function ($q) use ($client) {
                if ($client != 0) {
                    $q->where('installments_list.client_id', $client);
                }
            })
            ->where('type', 0)
            ->orderBy(\DB::raw('installment_date'))
            ->get();
    }

    /**
     * This method will decide the type of the instalments whether it (paid, outstanding, all)
     * in addition determine does the request for all clients or a specific one
     * then return the result
     * @param $client
     * @param $type
     * @return mixed
     */
    private function getInstalmentsBasedOnType($type, $client, $date)
    {
        $instalments = null;
        switch ($type) {
            case "late":
                $instalments = $this->getLateInstalment($client, $date);
                break;
            case "paid":
                $instalments = $this->getPaidInstalment($client);
                break;
            case "all":
                $instalments = $this->getAllInstallment($client);
                break;
            default:
                return;
        }
        return $instalments;
    }

    /**
     * This method receive a collection of instalments then looping through each of them-
     * and style it in html table before returning the result
     * @param $instalments
     * @return string
     */
    private function formatInstalments($instalments)
    {
        $output = '';
        if (sizeof($instalments)) {
            $loop = 0;
            foreach ($instalments as $instalment) {
                $loop++;
                $class = $instalment->installment_pay == 1 ? 'class=bg-success' : '';

                $output .= '  <tr ' . $class . '>
                            <td>' . $loop . '</td>
                            <td>' . $instalment->optionalClient()->client_name . '</td>
                            <td>' . $instalment->installment_date . '</td>
                            <td>' . $instalment->pay_date . '</td>
                            <td>' . $instalment->installment_amount . '</td>
                            <td>' . $instalment->notes . '</td>';
                if ($instalment->installment_pay == 1) {

                    $output .= '<td></td><td></td>';
                } else {
                    $output .= ' <td class="noprint"><a href="' . route('clientinstalments.payingTheDebt', $instalment->id) . '" class="btn btn-circle btn-success"><i class="fa fa-money"></i></a></td>
                            <td class="noprint"><a href="' . route('clientinstalments.delete', $instalment->id) . '" class="btn btn-circle btn-danger"><i class="fa fa-minus-circle"></i></a></td>
                       ';
                }

                $output .= '</tr>';
            }
        } else {

            $output = 'لا يوجد بيانات';
        }
        return $output;
    }

    /**
     * This method will fetch the client by its id then return it
     * @param  $request (client id)
     * @return mixed;
     */
    public function getClientBalanceAjax(Request $request)
    {
        $clientObject = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('id', $request->client)
            ->first();

        if ($clientObject) {
            $scheduled = Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('type', 0)
                ->where('installment_pay', 0)
                ->where('installments_list.client_id', $clientObject->id)
                ->orderBy(\DB::raw('installment_date'))
                ->sum('installment_amount');
            return $this->formatSupplierBalance($clientObject, $scheduled);
        }
        return 'لا يوجد بيانات';
    }

    /**
     * Formatting the client balance;
     * @param $clientObject
     * @return string
     */
    private function formatSupplierBalance($clientObject, $scheduled)
    {
        $unScheduled = $clientObject->balance - $scheduled;
        $output = "<tr>";
        $output .= "<td>" . $clientObject->client_name . "</td>";
        $output .= "<td>" .  $clientObject->mobile1 . "</td>";
        $output .= "<td>" . price_decimal($clientObject->balance) . "</td>";
        $output .= "<td>" . price_decimal($scheduled) . "</td>";
        $output .= "<td>" . price_decimal($unScheduled) . "</td>";
        $output .= "<td class='noprint'><a class='btn btn-success' href='" . route('clientinstalments.rescheduleDebt', $clientObject->id) . "'>جدولة</a></td>";
        return $output;
    }

    /**
     * Get client by id and return the rescheduling page.
     * @param $id
     * @return View
     */
    public function rescheduleDebt($id)
    {
        $price_decimal = get_session_val('decimal_num_price', 'integer');
        $type = get_session_val('date_type', 'integer');

        $client = Client::where('id', $id)
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->first();
        $scheduled = Instalment::where('installments_list.client_id', $id)
            ->where('installment_pay', 0)
            ->where('type', 0)
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->orderBy(\DB::raw('installment_date'))
            ->sum('installment_amount');
        if ($scheduled >= $client->balance) {
            session()->flash('error', ' المديونيات الخاصة بالعميل ' . $client->client_name . ' تم جدولتها مسبقاً ');
            return back();
        }
        $balance = $client->balance - $scheduled;
        return view('clientinstallments::reschedule', compact('type', 'client', 'price_decimal', 'balance', 'scheduled'));
    }

    /**
     * Insert the instalments dates and amount
     */
    public function storeRescheduleDebt(Request $request)
    {
        $this->validate($request, [
            'client_id' => 'required',
            'installment_amount' => 'required',
            'installment_date.*' => 'required',
            'installment_amount.*' => 'required',
        ]);
        if (is_array($request->installment_date) && sizeof($request->installment_date)) {
            foreach ($request->installment_date as $key => $instalment_date) {
                $instalmentAmountCheck = $this->makeSureInstalmentsLessThanBalance($request->client_id);
                if ($instalmentAmountCheck === false) {
                    session()->flash('error', 'عفوا يجب أن لا يتخطى إجمالي المبلغ المجدول مبلغ الدين');
                    return redirect()->route('clientinstalments.index');
                }
                if (isset($request->installment_amount[$key])) {
                    Instalment::create([
                        'client_id' => $request->client_id,
                        'installment_amount' => $request->installment_amount[$key],
                        'installment_date' => Carbon::parse($instalment_date)->format('Y-m-d'),
                        'bill_id' => 0,
                        'installment_pay' => 0,
                        'notes' => $request->notes ? $request->notes : '',
                        'shop_id' => get_session_val('serial_id', 'integer'),
                        'type' => 0,
                        'supplier_id' => 0,
                    ]);
                }
            }
            session()->flash('success', 'تم عملية الجدولة بنجاح');
            return redirect()->route('clientinstalments.index');
        }
        return back();
    }

    /**
     * @param $id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function payingTheDebtView($id)
    {
                $type = get_session_val('date_type', 'integer');

        $instalment = Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('type', 0)->with('bill:id,bill_no')
            ->orderBy(\DB::raw('installment_date'))
            ->where('id', $id)->first();
        $salePoints = $this->allowedStores();
        $employees = Employee::where([
            'shop_id' => get_session_val('serial_id', 'integer'),
            'is_employee' => 1,
        ])->selectRaw('id, name, money_percent')->get();
        return view('clientinstallments::paying-debt', compact('type', 'instalment', 'salePoints', 'employees'));
    }


    public function payAllInstallments(Request $request)
    {

        $billId = $request->bill_id;
        $discountPercentage = $request->discount;
        $pay_date = $request->today;
        $parse_date = Carbon::parse($pay_date);
        $today = $parse_date->format('Y-m-d');
        $sale_point = $request->sale_point;
        $employee_id = $request->employee_id;
        $client_id = $request->client_id;

        $instalments = Instalment::select('installment_amount', 'id', 'installment_pay')
            ->where('installments_list.shop_id', get_session_val('serial_id', 'integer'))
            ->where('installments_list.type', 0)
            ->where('installments_list.bill_id', $billId)
            ->get();

        $billData = Invoices::select('insatllments_add', 'total_price', 'net_price', 'payment', 'bill_no')
            ->where([
                ['shop_id', get_session_val('serial_id', 'integer')],
                ['id', $billId]
            ])->first();
        $all_installments = $billData->net_price -  $billData->payment;
        $instalmentsAdd = $billData->insatllments_add;
        $net_rest = $all_installments - $instalmentsAdd;
        $instalmentPercentage = $instalmentsAdd > 0 ? ($instalmentsAdd / $net_rest) : 0;


        $applyDiscount = false;
        $insatlmentsIds = [];
        $instalmentAdd = 0;

        foreach ($instalments as $instalment) {
            if ($instalment->installment_pay == 0) {
                $applyDiscount = true;
                $insatlmentsIds[] = $instalment->id;
                $instalmentAdd += $instalment->installmentAddition($instalmentPercentage);

                $instalmentAmount = $instalment->installment_amount;
                $instalmentId = $instalment->id;

                $request = new Request();
                $request->sale_point = $sale_point;
                $request->today = $today;
                $request->installment_amount = $instalmentAmount;
                $request->employee_id = $employee_id;

                $this->payingTheDebt($request, $instalmentId);
            }
        }



        // apply discount
        if ($applyDiscount == true) {
            $client = Client::find($client_id);

            $bill_no = $billData->bill_no;
            $discountAmount = ($discountPercentage / 100) * $instalmentAdd;

            $client->balance -= $discountAmount;
            $client->save();

            $clientBalance = $client->balance;
            $this->doDiscountTransaction($client_id, $clientBalance, $discountAmount, $today, $employee_id, $bill_no);
        }


        session()->flash('print', $insatlmentsIds);

        return Redirect::back();

    }

    /**
     * Paying the debt of instalment to the client
     * @param $id
     * @return mixed
     */
    public function payingTheDebt(Request $request, $id)
    {
        $instalment = Instalment::where('shop_id', get_session_val('serial_id', 'integer'))->where('type', 0)->findOrFail($id);
        $percent = 0;
        $emp = Employee::where('id', $request->employee_id)->first();
        $percent = $emp->money_percent ?? 0;
        $client = Client::find($instalment->client_id);
        $s_point = SalePoint::find($request->sale_point);
        $today = $this->formatDate($request->today);
        $paidMoney = $request->installment_amount;

        $instalment->pay_date = now();

        DB::beginTransaction();
            if ($paidMoney < $instalment->installment_amount) {
                $instalment->installment_amount = $instalment->installment_amount - $paidMoney;
                $instalment->notes .= ' <br> ' . ' ' . "تم دفع $paidMoney جزء من القسط بتاريخ $request->today";
                $instalment->save();

                $client->balance -= $paidMoney;
                $client->save();

                $n_s_balance = $s_point->money_point + $paidMoney;
                $s_point->money_point = $n_s_balance;
                $s_point->save();

                $this->doTransaction(
                    $request->sale_point, $instalment->client_id,
                    $paidMoney, $client->balance, $today, $n_s_balance,
                    $instalment->id, $request->employee_id, $percent
                );

                $this->updateInvoicePayment($instalment->bill_id, $paidMoney);
                session()->flash('success', 'تمت عملية سداد جزء من القسط بنجاح');
                return redirect()->route('clientinstalments.index');

            } else {
                $instalment->installment_pay = 1;

                $instalment->notes .= ' <br> ' . 'تم سداد القسط كاملا' . '  ' . $request->today;
                $instalment->save();

                $client->balance = (float) $client->balance - (float) $instalment->installment_amount;
                $client->save();

                $n_s_balance = (float) $s_point->money_point + (float) $instalment->installment_amount;
                $s_point->money_point = $n_s_balance;
                $s_point->save();

                $this->doTransaction(
                    $request->sale_point, $instalment->client_id,
                    $instalment->installment_amount, $client->balance, $today, $n_s_balance,
                    $instalment->id, $request->employee_id, $percent
                );

                $this->updateInvoicePayment($instalment->bill_id, $paidMoney);
                $paidMoney -= $instalment->installment_amount;

                if ($paidMoney >= 0) {
                    $allOtherInstalments = $this->getUnpaidInstalmentsForClient($instalment->client_id, $instalment->id);

                    foreach ($allOtherInstalments as $index => $oneInstalment) {
                        if ($paidMoney <= 0) {
                            break;
                        } else {
                            if ($paidMoney > $oneInstalment->installment_amount) {

                                $toSubAmount = $oneInstalment->installment_amount;
                                $oneInstalment->installment_pay = 1;
                                $oneInstalment->notes .= ' <br> ' . 'تم دفع مبلغ ' . $paidMoney . ' بدلا من مبلغ ' . $oneInstalment->installment_amount . '  ' . $request->today;
                                $oneInstalment->save();

                                $client->balance -= $toSubAmount;
                                $client->save();

                                $n_s_balance = $s_point->money_point + $toSubAmount;
                                $s_point->money_point = $n_s_balance;
                                $s_point->save();
                            }

                            if ($paidMoney == $oneInstalment->installment_amount) {
                                $toSubAmount = $oneInstalment->installment_amount;
                                $oneInstalment->installment_pay = 1;
                                $oneInstalment->notes .= ' <br> ' . 'تم سداد القسط كاملا' . '  ' . $request->today;
                                $oneInstalment->save();

                                $client->balance -= $toSubAmount;
                                $client->save();

                                $n_s_balance = $s_point->money_point + $toSubAmount;
                                $s_point->money_point = $n_s_balance;
                                $s_point->save();
                            }

                            if ($paidMoney < $oneInstalment->installment_amount) {
                                $toSubAmount = $paidMoney;
                                $oneInstalment->installment_amount = $oneInstalment->installment_amount - $paidMoney;
                                $oneInstalment->notes .= ' <br> ' . ' تم ترحيل مبلغ ' . $paidMoney . ' زائد من القسط السابق وخصمه من هذا القسط' . '  ' . $request->today;
                                $oneInstalment->save();

                                $client->balance -= $toSubAmount;
                                $client->save();

                                $n_s_balance = $s_point->money_point + $toSubAmount;
                                $s_point->money_point = $n_s_balance;
                                $s_point->save();

                            }


                            $paidMoney = $paidMoney - $toSubAmount;

                            $this->doTransaction(
                                $request->sale_point, $instalment->client_id,
                                $toSubAmount, $client->balance, $today, $n_s_balance, $oneInstalment->id, $request->employee_id, $percent
                            );
                        }
                    }

                }

            }
        DB::commit();

        $insatlmentsIds = array($instalment->id);
        session()->flash('print', $insatlmentsIds);
        $this->createLog(Carbon::now(),"تم تسجيل القسط علي فاتوره رقم ".$instalment->bill_id);
        session()->flash('success', 'تمت عملية السداد بنجاح');
        return redirect()->route('clientinstalments.index');

    }

    /**
     * Receive the client id and the amount of instalment
     * Get the client by it's id
     * Subtract the instalment amount from the client balance
     * Save the record
     * @param $clientId
     * @param $instalmentAmount
     * @return bool|\Illuminate\Http\RedirectResponse
     */
    private function subtractInstalmentAmountFromClientBalance($clientId, $instalmentAmount)
    {
        $client = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereIn('client_role', [0, 2])
            ->where('id', $clientId)->first();
        if ($client) {
            $client->balance = $client->balance - $instalmentAmount;
            $client->save();
            return $client;
        }
        return false;
    }

    /**
     * @param $salePointId
     * @param $instalmentAmount
     * @return bool
     */
    private function subtractInstalmentAmountFromSalePoint($salePointId, $instalmentAmount)
    {
        $salePoint = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('id', $salePointId)
            ->first();
        $salePoint->money_point = $salePoint->money_point + $instalmentAmount;
        if ($salePoint->save()) {
            return $salePoint;
        }
        return false;
    }

    /**
     * Check if the amount of the instalments is less than client balance or not
     * @param $clientId
     * @return bool
     */
    private function makeSureInstalmentsLessThanBalance($clientId)
    {
        $client = $this->getClient($clientId);
        $totalAmountOfInstalment = $this->totalAmountOfInstalments($clientId);
        if ($totalAmountOfInstalment < $client->balance) {
            return true;
        }
        return false;
    }

    /**
     * Get the total amount of the instalments for one client
     * @param $clientId
     * @return float
     */
    private function totalAmountOfInstalments($clientId)
    {
        $totalAmountOfInstalment = Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('installments_list.client_id', $clientId)
            ->where('type', 0)
            ->orderBy(\DB::raw('installment_date'))
            ->where('installment_pay', 0)->sum('installment_amount');
        return $totalAmountOfInstalment;
    }

    /**
     * Get client by its id
     * @param $clientId
     * @return object $clientId
     */
    private function getClient($clientId)
    {
        $client = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereIn('client_role', [0, 2])
            ->where('id', $clientId)->first();
        return $client;
    }

    /**
     * Get unpaid instalments for a specific client
     * @param $clientId
     * @return mixed
     */
    private function getUnpaidInstalmentsForClient($clientId, $except)
    {
        return Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('installment_pay', 0)
            ->where('type', 0)
            ->where('installments_list.id', '<>', $except)
            ->orderBy(\DB::raw('installment_date'))
            ->where('installments_list.client_id', $clientId)
            ->get();
    }

    private function doTransaction($salePoint, $supplierId, $paidMoney, $supplierBalance, $date, $moneyPoint, $in_id, $emp_id, $percent)
    {

        $clientTransaction = new ClientTransaction();
        $clientTransaction->shop_id = get_session_val('serial_id', 'integer');
        $clientTransaction->date_time = $date;
        $clientTransaction->client_id = $supplierId;
        $clientTransaction->amount = $paidMoney;
        $clientTransaction->type = 2;
        $clientTransaction->effect = 1; #     1 للايداع وصفر للسحب
        $clientTransaction->pay_day = $date;
        $clientTransaction->balance = $supplierBalance; # new not old?
        $clientTransaction->bill_id = 0;
        $clientTransaction->bill_net_total = 0;
        $clientTransaction->safe_type = 1;

        $clientTransaction->safe_balance = $moneyPoint;
        $clientTransaction->safe_point_id = $salePoint;
        $clientTransaction->user_id = \Auth::id();

        $clientTransaction->sale_back_id = 0;
        $clientTransaction->problem_id = 0;
        $clientTransaction->supplier_id = 0;
        $clientTransaction->salary_month = 0;
        $clientTransaction->OutTransactionID = 0;
        $clientTransaction->spend_id = 0;

        $clientTransaction->installment_id = $in_id;
        $clientTransaction->employee_id = $emp_id ?? 0;
        $clientTransaction->money_percent = $percent;
        $clientTransaction->notes = request()->get('notes');

        $clientTransaction->save();

    }

    private function doDiscountTransaction($supplierId, $supplierBalance, $paidMoney, $date, $emp_id, $bill_no)
    {

        $clientTransaction = new ClientTransaction();
        $clientTransaction->shop_id = get_session_val('serial_id', 'integer');
        $clientTransaction->date_time = $date;
        $clientTransaction->client_id = $supplierId;
        $clientTransaction->amount = $paidMoney;
        $clientTransaction->type = 60;
        $clientTransaction->effect = 1; #     1 للايداع وصفر للسحب
        $clientTransaction->pay_day = $date;
        $clientTransaction->balance = $supplierBalance; # new not old?
        $clientTransaction->bill_id = 0;
        $clientTransaction->employee_id = $emp_id ?? 0;

        $clientTransaction->user_id = \Auth::id();
        $clientTransaction->sale_back_id = 0;
        $clientTransaction->problem_id = 0;
        $clientTransaction->supplier_id = 0;
        $clientTransaction->OutTransactionID = 0;
        $clientTransaction->spend_id = 0;
        $clientTransaction->notes = "تم تسجيل رصيد معدوم بمقيمه $paidMoney من فاتوره رقم $bill_no";

        $clientTransaction->save();

    }

    private function formatDate($date)
    {
        $time = date('H:i:s');
        $date_time = $date . ' ' . $time;

        $date = Carbon::parse($date_time)->format('Y-m-d H:i:s');
        return $date;
    }

    private function allowedStores()
    {
        $point = Auth::user()->sale_point;
        if (Auth::user()->show_points == 1 ||Auth::user()->show_points == 2)  {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id' ,'integer'))
                ->where('id', $point)
                ->get();
        } elseif (Auth::user()->show_points == 3) {
            $store_id = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->first()->store_id;
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_id', $store_id)
            ->get();

        }
        else {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id' ,'integer'))->get();
        }
        return $sale_points;
    }

    public function clientBills(Request $request)
    {
        $userBills = Instalment::select('bill_id')->where('installments_list.shop_id', get_session_val('serial_id', 'integer'))
            ->where('client_id', $request->client)->groupBy('bill_id')->get();

        $bill_ids = [];
        foreach ($userBills as $userBill) {
            $bill_id = $userBill->bill_id;

            $bill_no = Invoices::select('bill_no')->where([
                ['shop_id', get_session_val('serial_id', 'integer')],
                ['id', $bill_id]
            ])->first();

            if (!$bill_no) {
                continue;
            }

            $bill_ids[$bill_no->bill_no] = $bill_id;
        }

        return $bill_ids;
    }

    protected function updateInvoicePayment($invoice_id, $paid)
    {
        $invoice = Invoices::find($invoice_id);

        if (! is_null($invoice) && $paid > 0)
        {
            if ($invoice->payment < $invoice->net_price)
            {
                $amount = $paid;
                if ($invoice->net_price - $invoice->payment >= $paid)
                {
                    $amount = $paid;
                }
                else {
                    $amount = $invoice->net_price - $invoice->payment;
                }

                $invoice->payment += $amount;
                $invoice->rest    -= $amount;
                $invoice->save();
            }
        }
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }

}
