<?php

namespace Modules\Clients\Entities;

use Illuminate\Database\Eloquent\Model;

class ClientTransaction extends Model
{
    protected $guarded = [];
    protected $table = 'client_transaction';
    public $timestamps = false;

    public function client()
    {
        return $this->belongsTo(Client::class);
    }
    
    public function saleProcess()
    {
        return $this->belongsTo(SaleProcess::class, 'bill_id', 'id');
    }

    public function saleBack()
    {
        return $this->belongsTo(SaleBack::class, 'sale_back_id', 'id');
    }

    public function incomingBill()
    {
        return $this->belongsTo(IncomeBill::class, 'bill_id', 'id');
    }
    public function incomingBillReturn()
    {
        return $this->belongsTo(IncomeBillReturn::class, 'bill_id', 'id');
    }

    public function user()
    {
        return $this->belongsTo(User::class);
    }

    public function salePoint()
    {
        return $this->belongsTo(SalePoint::class, 'safe_point_id', 'id');
    }

    public function saleDetails()
    {
        $details = SaleDetails::join('items', 'sale_details.items_id', '=', 'items.id')
            ->join('units', 'sale_details.unit', '=', 'units.id')
            ->select('items.item_name', 'units.name', 'sale_details.quantity', 'sale_details.price', 'sale_details.total_price')
            ->where('sale_id', $this->bill_id)->get();
        return $details;
    }

    public function incomeDetails()
    {
        $details = incomeDetails::join('items', 'incoming_details.items_id', '=', 'items.id')
            ->join('units', ' incoming_details.unit', '=', 'units.id')
            ->select('items.item_name', 'units.name', ' incoming_details.quantity', ' incoming_details.price', ' incoming_details.total_price')
            ->where('incom_id', $this->bill_id)->get();
        return $details;
    }

    public function backDetails()
    {
        $details = BackDetails::join('items', 'sale_back.items_id', '=', 'items.id')
            ->join('units', 'sale_back.unit', '=', 'units.id')
            ->select('items.item_name', 'units.name', 'sale_back.quantity', 'sale_back.price', 'sale_back.bill_id')
            // ->where('back_id', $this->sale_back_id)->get();
            ->where('back_id', 27)->get();
          
        foreach($details as $d){
            if ($d->bill_id <= 0){
                $d->price = $d->price / $d->quantity;
            }
        }
        return $details;
    }

    public function checkBill($bill_type, $sale_id, $back_id, $sheet_type)
    {
        $check = '';
        if ($sheet_type == 'client') {
            if ($bill_type == 1) {
                $check = SaleProcess::where([
                    'shop_id' => get_session_val('serial_id', 'integer'),
                    'id' => $sale_id
                ])->first();
            }
            if ($bill_type == 0) {
                $check = SaleBack::where([
                    'shop_id' => get_session_val('serial_id', 'integer'),
                    'id' => $back_id
                ])->first();
            }
        } else {
            if ($bill_type == 3) {
                $check = IncomeBill::where([
                    'shop_id' => get_session_val('serial_id', 'integer'),
                    'id' => $sale_id
                ])->first();
            }
            if ($bill_type == 5) {
                $check = IncomeBillReturn::where([
                    'shop_id' => get_session_val('serial_id', 'integer'),
                    'id' => $sale_id
                ])->first();
            }
        }
        if ($check) {
            return $bill_type;
        } else {
            if ($bill_type == 2 || $bill_type == 4 || $bill_type == 6 || $bill_type == 7 || $bill_type == 9) {
                return $bill_type;
            } else {
                return '-1';
            }
        }
    }
    
    public function billNo($bill_type, $bill_id)
    {
        if ($bill_type == 1) {
            $bill_no = SaleProcess::where([
                'shop_id' => get_session_val('serial_id', 'integer'),
                'id' => $bill_id
            ])->first()->bill_no;
        } else {
            $bill_no = SaleBack::where([
                'shop_id' => get_session_val('serial_id', 'integer'),
                'id' => $bill_id
            ])->first()->bill_no;
        }
        return $bill_no;
    }
}
