<?php

namespace Modules\Clients\Http\Controllers;

use App\Http\Controllers\Controller;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Modules\Clients\Entities\Client;
use Modules\Clients\Entities\ClientTransaction;
use Modules\Clients\Http\Requests\BadBalanceRequest;

class BadBalanceController extends Controller
{
    public function index()
    {
        $rows = Client::select('id', 'client_name', 'balance', 'client_role')->where('shop_id', auth()->user()->shop_id)->get();
        
        return view('clients::bad-balance.index', compact('rows'));
    }

    public function save(BadBalanceRequest $request)
    {
        try {
            $client = Client::select('id', 'balance', 'client_role', 'shop_id')->where('shop_id', auth()->user()->shop_id)->findOrFail($request->get('client_id'));

            DB::beginTransaction();
                $client->decrement('balance', $request->get('amount'));
                $this->createTransaction($client, $request);
            DB::commit();
            
            return back()->with('success', 'تم التعديل علي الرصيد بنجاح');
        } catch (Exception $e) {
            return back()->with('error', 'هنا خطأ ما يرجي التواصل مع الدعم الفني')->withInput($request->all());
        }
    }

    protected function createTransaction(Client $client, Request $request)
    {
        $transaction =  ClientTransaction::create([
            'shop_id'   => $client->shop_id,
            'date_time' => now(),
            'client_id' => $client->id,
            'amount'    => abs( $request->get('amount') ),
            'type'      => $request->get('amount') >= 0 ? 60 : 61,
            'pay_day'   => $request->get('date'),
            'balance'   => $client->balance,
            'user_id'   => auth()->id(),
            'effect'    => $client->client_role == 1 ? true : false,
            'notes'     => 'رصيد معدوم : ' . $request->get('notes'),
        ]);
    }
}
