<?php

namespace Modules\Clients\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Controllers\Controller;
use App\Models\systemLog;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Modules\ClientInstallments\Entities\ClientTransaction;
use Modules\Clients\Entities\Badrshop;
use Modules\Clients\Entities\Client;
use Modules\Clients\Entities\ClientsGroups;
use Modules\Clients\Http\Requests\ClientGroupRequest;
use Modules\Clients\Http\Requests\UpdateClientGroupRequest;

class ClientGroupController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index()
    {
        $groups = ClientsGroups::where('shop_id', get_session_val('serial_id', 'integer'))->selection()->get();
        return view('clients::groups.clients-groups', compact('groups'));
    }
    public function store(ClientGroupRequest $request)
    {
        $check  =   ClientsGroups::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('name', $request->groupName)
            ->first();
        if ($check) {
            return back()->with(['error' => trans('client.clientGroup.groupName_unique')]);
        }
        ClientsGroups::create([
            'name' => $request->groupName,
            'group_discount' => $request->discountPercent ?? 0,
            'shop_id' => get_session_val('serial_id', 'integer'),
            'add_user' => auth()->user()->id
        ]);
        $this->createLog(Carbon::now(), trans('client.clientGroup.client_create-group') . $request->groupName);
        return back()->with(['success' => trans('client.clientGroup.save_done')]);
    }
    public function update(UpdateClientGroupRequest $request)
    {
        $check  =   ClientsGroups::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('name', $request->name)
            ->where('id', '!=', $request->id)
            ->first();
        if ($check) {
            return back()->with(['error' => trans('client.clientGroup.groupName_unique')]);
        }
        $groups = ClientsGroups::where('shop_id', get_session_val('serial_id', 'integer'))->selection()->find($request->id);
        $groups->update([
            'name' => $request->name,
            'group_discount' => $request->discount ?? 0,
            'valid' => $request->is_hidden_installments_menu ?? 1,
        ]);
        $this->createLog(Carbon::now(), trans('client.clientGroup.client_edit') . ':' . $request->groupName);
        return redirect()->back()->with(['success' => trans('client.clientGroup.edit_done')]);
    }
    /**
     * Remove the specified resource from storage.
     * @return Response
     */
    public function destroy(Request $request)
    {
        $groups = ClientsGroups::where('shop_id', get_session_val('serial_id', 'integer'))->selection()->find($request->id);
        if ($groups->clients->count() > 0) {
            if ($request->newGroup == -1) {
                return back()->with(['error' => trans('client.clientGroup.choose_group')]);
            } else {
                foreach ($groups->clients as $client) {
                    $client->group_id = $request->newGroup;
                    $client->save();
                }
                $groups->delete();
                return back()->with(['success' => trans('client.clientGroup.delete_group')]);
            }
        } else {
            $groups->delete();
            return back()->with(['success' => trans('client.clientGroup.delete_group')]);
        }
        $this->createLog(Carbon::now(),  trans('client.clientGroup.delete_group') . ':'  . $request->id);
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}
