<?php

namespace Modules\Clients\Http\Controllers;

use Carbon\Carbon;
use App\Client as AppClient;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Modules\Clients\Entities\User;
use Modules\Clients\Helpers\UCal2;
use App\Http\Controllers\Controller;
use App\Models\systemLog;
use Illuminate\Pagination\Paginator;
use Modules\Clients\Entities\Client;
use Modules\Clients\Entities\Helper;
use Intervention\Image\Facades\Image;
use Modules\Clients\Entities\Badrshop;
use Modules\Clients\Entities\SaleBack;
use Modules\Clients\Entities\SalePoint;
use Modules\Clients\Entities\BackDetails;
use Modules\Clients\Entities\ClientGlass;
use Modules\Clients\Entities\SaleDetails;
use Modules\Clients\Entities\SaleProcess;
use Modules\Clients\Entities\ClientsGroups;
use Illuminate\Pagination\LengthAwarePaginator;
use Modules\Clients\Entities\ClientTransaction;
use Modules\Clients\Entities\TransactionFactory;
use Modules\Clients\Http\Requests\ClientRequest;
use Modules\Clients\Entities\BalanceSheetRepository;
use Modules\Clients\Entities\Country;
use Modules\Clients\Entities\Invoices_;
use Modules\Invoices\Entities\Invoices;
use Modules\SpendsAndIncoms\Entities\Income;
use Modules\Invoices\Entities\InvoiceSettings;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Modules\Clients\Entities\ListPrice;
use Modules\Clients\Entities\UserGroup;
use Modules\Employee\Entities\Employee;

class ClientsController extends Controller
{
    private $transactionFactory;
    private $balanceSheetRepo;

    public function __construct(TransactionFactory $transactionFactory, BalanceSheetRepository $balanceSheetRepository)
    {
        $this->transactionFactory = $transactionFactory;
        $this->balanceSheetRepo = $balanceSheetRepository;
    }

    public function index(Request $request)
    {
        if (!authorized('can_display_clients') && !authorized('can_display_suppliers')) {
            return unAuthorized(trans('users.show_client_details') . ' , ' . trans('users.read_supplier_details'));
        }
        // تعديلات البحث
        $clients = clientGroups();
        $clients = $clients->when((authorized('can_display_suppliers') && authorized('can_display_clients')
            && (!request()->has('client_role') || request('client_role') == 2)), function ($q) {

            $q->whereIn('client_role', [0, 1, 2]);

        })->when((!authorized('can_display_suppliers') && authorized('can_display_clients')), function ($q) {

            $q->whereIn('client_role', [0,2]);

        })->when((!authorized('can_display_clients')) && authorized('can_display_suppliers'), function ($q) {

            $q->whereIn('client_role', [1,2]);
        })->when(request()->has('client_role'), function ($q) {

            if (request('client_role') == 1) {
                $q->whereIn('client_role', [2, 1]);
            } else if (request('client_role') == 0) {
                $q->whereIn('client_role', [2, 0]);
            }else{
                $q->whereIn('client_role', [2, 1, 0]);
            }
        });
        // $clients =  $clients->join('clients_groups', function ($join) {
        //     $join->on('clients_groups.id', 'clients.group_id');

        // });
        // if (!authorized('can_add_supplier')) {
        //     $clients = $clients->when((!request()->has('clients.client_role') || request('clients.client_role') == 2
        //     || request('clients.client_role') == 0), function ($q) {
        //         $q->whereIn('clients.client_role', [0, 2]);
        //     });
        // }

        // if (!authorized('can_display_clients')) {
        //     $clients = $clients->when((!request()->has('clients.client_role') || request('clients.client_role') == 2
        //     || request('clients.client_role') == 1), function ($q) {
        //         $q->whereIn('clients.client_role', [2, 1]);
        //     });
        // }

        // if (authorized('can_display_clients') && authorized('can_add_supplier')) {
        //     $clients = $clients->when((!request()->has('clients.client_role') || request('clients.client_role') == 2), function ($q) {
        //         $q->whereIn('clients.client_role', [2, 0, 1]);
        //     })->when(request()->has('clients.client_role') && (request('clients.client_role') == 0), function ($q) {
        //         $q->whereIn('clients.client_role', [2, 0]);
        //     })->when((request('clients.client_role') == 1), function ($q) {
        //         $q->whereIn('clients.client_role', [2, 1]);
        //     });
        // }


        if (request()->has('group') && request('group') >= 0) {
            $clients = $clients->where('clients.group_id', request('group'));
        }
        if (request()->has('client') && rtrim(trim(request('client'))) != '') {
            $clients = $clients->where(function ($c) {
                $c->where('clients.client_name', 'like', '%' . rtrim(trim(request('client'))) . '%')
                    ->orWhere('clients.tele', 'like', '%' . rtrim(trim(request('client'))) . '%')
                    ->orWhere('clients.code', request('client'))
                    ->orWhere('clients.mobile1', 'like', '%' . rtrim(trim(request('client'))) . '%');
            });
        }

        $clients   = $clients->orderBy('clients.id', 'desc')->paginate(request()->get('limit',20));
        $groups    = groupList();
        $settings  = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $decimal_num_price =  get_session_val('decimal_num_price', 'integer');
        return view('clients::index', compact('clients', 'groups', 'settings', 'decimal_num_price'));
    }

    public function getClientSheet($id, $from, $to, $status)
    {
        $clientTransactions = $this->balanceSheetRepo->balanceSheet($id, $from, $to, $status);

        $deleted_bills = [];
        $collection = collect();
        $clientTransactions->chunk(20)->each(function ($chucked_transactions) use (&$collection, &$deleted_bills) {
            $chucked_transactions->each(function ($ct) use (&$collection, &$deleted_bills) {

                $billCheck = null;
                $point = null;

                if ($ct->type == 1) {
                    $billCheck = Invoices_::find($ct->bill_id);
                    $saleDetails = $billCheck ? Invoices_::where('bill_no', $ct->bill_id)->get() : null;
                    $point = Invoices_::find($billCheck ? $billCheck->sale_point : 0);
                } else if ($ct->type == 0) {
                    $billCheck = Invoices_::find($ct->sale_back_id);
                    $backDetails = $billCheck ? Invoices_::where('bill_no', $ct->sale_back_id)->get() : null;
                    $point = Invoices_::find($backDetails ? $backDetails[0]->sale_point : 0);
                }
                $t = $this->collectionToObject($ct);

                $has_bill = true;
                if (in_array($ct->type, [0, 1, 2, 30, 31, 34]) && is_null($billCheck)) {
                    $has_bill = false;
                    $deleted_bills[] = $ct->bill_id;
                }

                $t->has_bill = $has_bill;
                $t->saleProcess = null;
                $t->saleBack = null;
                $t->saleDetails = null;
                $t->backDetails = null;

                $x = [];
                if ($ct->type == '1') {

                    $t->saleProcess = $billCheck;
                    $t->saleDetails = $saleDetails;
                    $t->saleBack = null;
                    $t->backDetails = null;
                } else if ($ct->type == '0') {
                    $t->saleProcess = null;
                    $t->saleBack = $billCheck;
                    $t->saleDetails = null;

                    // foreach ($backDetails as $bd) {
                    //     if ($bd->bill_id <= 0) {
                    //         $bd->price = $bd->price / $bd->quantity;
                    //     }
                    // }
                    $t->backDetails = $backDetails;
                }

                $t->salePoint = $point;
                $user = User::find($ct->user_id);
                $t->user = $user;
                $t->installment_id = $ct->installment_id;

                $collection->push($t);
            });
        });
        $deleted_bills = array_filter($deleted_bills);
        $deleted_bills = array_unique($deleted_bills);

        return [
            'data' => $collection,
            'skip' => $deleted_bills
        ];
    }

    public function paginate($items, $perPage)
    {
        $pageStart = request('page', 1);
        $offSet = ($pageStart * $perPage) - $perPage;
        $itemsForCurrentPage = $items->slice($offSet, $perPage);

        return new LengthAwarePaginator(
            $itemsForCurrentPage,
            $items->count(),
            $perPage,
            Paginator::resolveCurrentPage(),
            ['path' => Paginator::resolveCurrentPath()]
        );
    }

    public function showNew($id, Request $request)
    {

        ini_set('max_execution_time', 0);
        ini_set('max_input_time', 0);
        // ini_set('memory_limit', '256M');
        $this->updateUnknownClientTransactions($id);
        $pay_day = ClientTransaction::where('shop_id', get_session_val('serial_id', 'integer'))->orderBy('pay_day', 'ASC')->first();
        if (is_null($pay_day) || ($pay_day && $pay_day->pay_day == '0000-00-00')) {
            $activ =  get_session_val('active_date', 'integer');
        } else {
            $activ = $pay_day->pay_day;
        }
        $from = Carbon::parse($activ)->format('Y-m-d');
        if ($request->filled('from')) {
            $from = $request->from;
        }
        $to = '';
        if ($request->filled('to')) {
            $to = $request->to;
        }

        #### get content sheet transactions and deleted bills ####
        $status = 'content';
        $collectionData = $this->getClientSheet($id, $from, $to, $status);
        $collection = $collectionData['data'];
        $deleted_bills = $collectionData['skip'];

        #### get previous sheet balance ####
        $status = 'prev';
        $pre_sheetData = $this->getClientSheet($id, $from, $to, $status);
        $pre_sheet = $pre_sheetData['data'];

        $pre_balance = 0;
        $balance = 0;
        if (count($pre_sheet) > 0) {
            foreach ($pre_sheet as $sheet) {
                if (in_array($sheet->bill_id, $deleted_bills) || in_array($sheet->type, [34, 30, 31])) {
                } else {
                    if (!in_array($sheet->type, [34, 30, 31])) {
                        $balance = $sheet->balance;
                        if (!in_array($sheet->type, [4, 8])) {
                            $balance = balance($sheet, $pre_balance);
                        }
                        if (in_array($sheet->type, [0, 1, 34, 30, 31]) && $sheet->has_bill == false) {
                            $balance = $pre_balance;
                        }
                    }
                }
                $pre_balance = $balance;
            }
        }

        if ($request->has('pre_balance')) {
            $pre_balance = $request->pre_balance;
        }

        // $shop = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id' ,'integer'))->first();

        $client = Client::where('shop_id', get_session_val('serial_id', 'integer'))->where('id', $id)->first();

        if (in_array($client->client_role, [0, 2])  &  !authorized('can_display_clients')) {
            return unAuthorized(trans('users.show_clients_details'));
        } else if (in_array($client->client_role, [1, 2])  & !authorized('can_add_supplier')) {
            return unAuthorized(trans('users.read_supplier_details'));
        }

        $group = \DB::table('clients_groups')->find($client->group_id)->name ?? '';

        $client->group_name = $group;
        $newEntry = ClientTransaction::where('client_transaction.shop_id', get_session_val('serial_id', 'integer'))
            ->where('client_transaction.client_id', $id)->where('type', 4)->first();

        if ($request->updateBalance == '1') {
            //            return $collection;
            $data = [
                'collection' => $collection,
                'deleted_bills' => $deleted_bills,
                'pre_balance' => $pre_balance,
                'newEntry' => $newEntry
            ];
            return $data;
        }
        //        $collection = $this->paginate($collection, 50  );

        return view('clients::show-new', compact('from', 'to', 'client', 'shop', 'collection', 'pre_balance', 'deleted_bills', 'newEntry'));
    }

    public function showClientBalanceSheet($id, Request $request)
    {
        $first_transaction = ClientTransaction::where('shop_id', get_session_val('serial_id', 'integer'))->where('client_id', $id)->orderBy('pay_day', 'ASC')->first();
        if (is_null($first_transaction) || ($first_transaction && $first_transaction->pay_day == '0000-00-00')) {
            $activ =  get_session_val('active_date', 'integer');
        } else {
            $activ = $first_transaction->pay_day;
        }
        $from = Carbon::parse($activ)->format('Y-m-d');
        if ($request->filled('from')) {
            $from = $request->from;
        }

        $to = '';
        if ($request->filled('to')) {
            $to = $request->to;
        }

        $client = Client::where('shop_id', get_session_val('serial_id', 'integer'))->where('id', $id)->first();

        if (in_array($client->client_role, [0, 2])  &  !authorized('can_display_clients')) {
            return unAuthorized(trans('users.show_clients_details'));
        } else if (in_array($client->client_role, [1, 2])  & !authorized('can_add_supplier')) {
            return unAuthorized(trans('users.read_supplier_details'));
        }

        $transactions = ClientTransaction::leftJoin('clients', 'clients.id', 'client_transaction.client_id')
            ->leftJoin('invoices_', 'invoices_.id', 'client_transaction.bill_id')
            ->where([
                ['client_transaction.shop_id', get_session_val('serial_id', 'Integer')],
                ['client_transaction.client_id', $id]
            ])->whereDate('pay_day', '>=', $from)->when($to, function ($q) use ($to) {
                $q->whereDate('client_transaction.pay_day', '<=', $to);
            })->select(['client_transaction.*', 'invoices_.bill_no', 'clients.client_name', 'invoices_.supplier_invoice_no'])->get();

        $transactions = $this->prepareTransactions($transactions);

        $pre_balance = 0;

        if ($transactions->count() > 0) {
            if (in_array($first_transaction->type, [4, 92]) && $first_transaction->pay_day == $from) {
                $pre_balance = $first_transaction->balance;
            } else {
                $pre_balance = ClientTransaction::where([
                    ['shop_id', get_session_val('serial_id', 'Integer')],
                    ['client_id', $id],
                    ['id', '<', $transactions[0]['id']]
                ])->orderBy('id', 'desc')->first()->balance ?? 0;
            }
        }

        return view('clients::balance-sheet', compact('from', 'to', 'transactions', 'client', 'pre_balance'));
    }

    private function prepareTransactions($transactions)
    {
        foreach ($transactions as $transaction) {
            if (in_array($transaction->type, [1, 5, 2, 60])) {
                $transaction->git = $transaction->bill_net_total;
                $transaction->pay = $transaction->amount;
            } else if ($transaction->type == 90) {
                $transaction->pay = $transaction->amount;
                $transaction->git = 0;
            } else if ($transaction->type == 91) {
                $transaction->pay = 0;
                $transaction->git = $transaction->amount;
            } else {
                $transaction->pay = $transaction->bill_net_total;
                $transaction->git = $transaction->amount;
            }
            $data = [
                'invoice_no' => $transaction->bill_no,
                'invoice_id' => $transaction->bill_id,
            ];
            $transaction->invoice_no = $transaction->bill_no;
            $transaction->details_link = itemReport($transaction, getBillNo(collect($data)));
        };
        return $transactions;
    }


    public function ClientBalance(Request $request)
    {
        return Client::find($request->id)->balance ?? 0;
    }

    public function updateAllBalances(Request $request)
    {
        $pagination = 1;
        $clients = Client::where('shop_id', get_session_val('serial_id', 'integer'))->select('id', 'balance', 'client_name')->get()->toArray();
        $clients = json_encode($clients);
        // $clients = Client::where('shop_id', get_session_val('serial_id' ,'integer'))->count();
        return view('clients::update-all-balances', compact('pagination', 'clients'));

        // ini_set('max_execution_time', 0);
        // ini_set('max_input_time', 0);
        // if ($request->ajax()) {
        //     $paginate = 1;
        //     $from = null;
        //     $to = null;
        //     $transaction = collect();
        //     if ($request->has('transaction_id')) {
        //         $bill = Invoices_::where([
        //             'shop_id' => auth()->user()->shop_id,
        //             'id' => $request->transaction_id
        //         ])->first();
        //         $transaction = ClientTransaction::where([
        //             'shop_id' => auth()->user()->shop_id,
        //             'client_id' => $bill->client_id,
        //             'bill_id' => $bill->id
        //         ])->first();
        //         $to = $transaction->pay_day;
        //     }
        //     $updateBalance = '1';
        //     $request['from'] = $from;
        //     $request['to'] = $to;
        //     $request['updateBalance'] = $updateBalance;
        //     $clients = Client::whereShopId(auth()->user()->shop_id)
        //         ->when($request->has('transaction_id'), function ($query) use ($transaction) {
        //             $query->where('id', $transaction->client_id);
        //         })
        //         ->paginate($paginate);
        //     $pagination = $clients->total() / $paginate;
        //     $pagination = ceil($pagination);
        //     foreach ($clients as $client) {
        //         $data = $this->showNew($client->id, $request);
        //         $balance = 0;
        //         $pre_balance = 0;
        //         foreach ($data['collection'] as $col) {
        //             if ($request->has('transaction_id') && $col->id == $transaction->id) {
        //                 return round_money($pre_balance);
        //             }
        //             if (in_array($col->bill_id, $data['deleted_bills']) && in_array($col->type, [34, 30, 31])) {
        //             } else {
        //                 if (!in_array($col->type, [34, 30, 31])) {
        //                     $balance = balance($col, $pre_balance);
        //                     if (in_array($col->type, [4, 8])) {
        //                         $balance = $col->balance;
        //                     }
        //                     $pre_balance = $balance;
        //                 }
        //             }
        //         }
        //         if (ceil($client->balance) != ceil($balance)) {
        //             $client->balance = $balance;
        //             if (!$request->has('transaction_id')) {
        //                 $client->update();
        //             }
        //         }
        //     }

        //     $html = '<p>' . random_int(pow(2, 0), pow(2, 32)) . '</p>';
        //     return response()->json(['html' => $html]);
        // }
        // return view('clients::update-all-balances', compact('pagination'));
        // //        return 'done';
    }

    public function oldBalance(Request $request)
    {
        ini_set('max_execution_time', 0);
        ini_set('max_input_time', 0);
        $shop_id = auth()->user()->shop_id;
        $bill = SaleProcess::where([
            'shop_id' => $shop_id,
            'id' => $request->bill_id
        ])->first();
        if ($bill) {
            $transaction = ClientTransaction::where([
                'shop_id' => $shop_id,
                'client_id' => $bill->client_id,
                'bill_id' => $bill->id
            ])->first();
            if ($transaction) {
                $request['from'] = '';
                $request['to'] = $transaction->pay_day;
                $request['updateBalance'] = '1';
                $client = Client::where([
                    'shop_id' => $shop_id,
                    'id' => $transaction->client_id
                ])->first();
                if ($client) {
                    $data = $this->showNew($client->id, $request);
                    $balance = 0;
                    $pre_balance = 0;
                    foreach ($data['collection'] as $col) {
                        if ($request->has('bill_id') && $col->id == $transaction->id) {
                            $pre_balance = round_money($pre_balance);
                            break;
                        }
                        if (in_array($col->bill_id, $data['deleted_bills']) && in_array($col->type, [34, 30, 31])) {
                        } else {
                            if (!in_array($col->type, [34, 30, 31])) {
                                $balance = balance($col, $pre_balance);
                                if (in_array($col->type, [4, 8])) {
                                    $balance = $col->balance;
                                }
                                $pre_balance = $balance;
                            }
                        }
                    }
                    return \response()->json([
                        'status' => true,
                        'old_balance' => $pre_balance
                    ]);
                }
                return \response()->json(['status' => false]);
            }
            return \response()->json(['status' => false]);
        }
        return \response()->json(['status' => false]);
    }

    public function updateUnknownClientTransactions($id)
    {
        $c = ClientTransaction::where('client_id', $id)
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->get();

        foreach ($c as $s) {
            if ($s->pay_day == '0000-00-00') {
                $n_c = ClientTransaction::where('client_id', $s->client_id)
                    ->where('shop_id', get_session_val('serial_id', 'integer'))
                    ->where('id', '>', $s->id)
                    ->whereNotIn('type', [30, 31, 34])
                    ->first();
                if (is_null($n_c)) {
                    $n_c = ClientTransaction::where('client_id', $id)
                        ->where('shop_id', get_session_val('serial_id', 'integer'))
                        ->where('id', '<', $s->id)
                        ->whereNotIn('type', [30, 31, 34])
                        ->orderBy('pay_day', 'desc')->orderBy('id', 'desc')
                        ->first();
                }
                if (is_null($n_c)) {
                    $n_c = $s;
                }
                $s->pay_day = $n_c->pay_day;
                $s->save();
            }
        }
        if (count($c->sortBy('pay_day')) > 0) {
            $ss = ClientTransaction::where('client_id', $id)
                ->where('shop_id', get_session_val('serial_id', 'integer'))
                ->orderBy('pay_day')->first();

            $first = ClientTransaction::where('client_id', $id)
                ->where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('type', 4)
                ->first();
            if ($first) {
                $first->pay_day = $ss->pay_day;
                $first->save();
            }
        }
    }

    public function showNewModel($id)
    {
        $clientTransactions = $this->balanceSheetRepo->balanceSheet($id);
        $collection = collect();
        foreach ($clientTransactions as $ct) {
            $t = $this->collectionToObject($ct);
            $collection->push($t);
        }

        $shop = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();

        $client = Client::where('shop_id', get_session_val('serial_id', 'integer'))->where('id', $id)->first();

        return view('clients::show2', compact('client', 'shop', 'collection'));
    }

    public function search(Request $request)
    {
        $clients = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('client_name', "LIKE", "%" . $request->client_name . "%")
            ->get();
        return view('clients::index', compact('clients'));
    }

    private function collectionToObject($object)
    {
        $t = $this->transactionFactory->factory($object);
        $t->first_cell = $this->balanceSheetRepo->issuedToHim($object);
        $t->second_cell = $this->balanceSheetRepo->comeFromHim($object);
        $t->transaction_type = $this->transactionType($object->type, $object->bill_id, $object->sale_back_id);
        return $t;
    }

    private function transactionType($type, $billId, $backId)
    {

        $bill_size =  get_session_val('bill_size', 'integer');
        $url = "";
        switch ($type) {
            case '0': //مردود  مبيعات

                $no = Helper::getSaleBackBillNo($billId);
                $url = route('invoices.show', $billId);
                // $url = printBill($bill_size, $backId, $type = "return");
                $type = '<a href="' . $url . '"> فاتورة مرتجع بيع رقم ' . $no . '</a>';
                break;
            case '1': // فاتورة بيع
                $no = Helper::getSaleBillNo($billId);
                $url = route('invoices.show', $billId);
                //  $url = printBill($bill_size, $billId, $type = "sale");
                $type = '<a href="' . $url . '"> فاتورة بيع رقم ' . $no . '</a>';
                break;
            case '2': //عملية سداد من العميل
                $type = 'عملية سداد من العميل';

                break;
            case '3': //عملية شراء من المورد
                $type = 'عملية شراء من المورد';
                break;
            case '5': //  مرتجع مشتريات
                $type = 'مرتجع مشتريات';
                break;
            case '4': //رصيد جديد
                $type = 'رصيد جديد';
                break;
            case '8': //تعديل مباشر للرصيد
                $type = 'تعديل مباشر للرصيد';
                break;
            case '9': //مردود نقدى للعميل
                $type = 'مردود نقدي للعميل';
                break;

            case '30': //حذف صنف من فاتورة المبيعات
                $no = Helper::getSaleBillNo($billId);
                //  $url = route('invoices.show', $billId);
                $type = '<a href="' . $url . '"> حذف صنف من فاتورة بيع رقم ' . $no . '</a>';
                break;
            case '31': //إضافة صنف على الفاتورة   // تعديل فاتورة مبيعات
                $no = Helper::getSaleBillNo($billId);
                //  $url = route('invoices.show', $billId);
                $type = '<a href="' . $url . '"> اضافة صنف الي فاتورة بيع رقم ' . $no . '</a>';
                break;

            case '34': //عديل فاتورة مبيعات
                $no = Helper::getSaleBillNo($billId);
                //  $url = route('invoices.show', $billId);
                $type = '<a href="' . $url . '"> تعديل فاتورة بيع رقم ' . $no . '</a>';
                break;

            case '60': //خصم رصيد عميل
                $type = " رصيد معدوم ";
                break;

            case '63': //اضافة صنف لفاتورة مرتجع البيع
                $no = Helper::getSaleBackBillNo($backId);
                //  $url = route('invoices.show', $backId);
                $type = '<a href="' . $url . '"> اضافة صنف الي فاتورة مرتجع بيع رقم ' . $no . '</a>';
                break;
            case '64': //حذف صنف من فاتورة مرتجع البيع
                $no = Helper::getSaleBackBillNo($backId);
                //  $url = route('invoices.show', $backId);
                $type = '<a href="' . $url . '"> حذف صنف من فاتورة مرتجع بيع رقم ' . $no . '</a>';
                break;

            default:
                $type = "....";
                break;
        }
        return $type;
    }

    public function create(Request $request)
    {
        if (!authorized('can_add_supplier') && !authorized('can_add_client')) {
            return unAuthorized(trans('users.add_client') . ',' . trans('users.add_supplier'));
        }

        $list_prices = ListPrice::select('id', 'list_name as name')->pluck('name', 'id');
        $list_prices[0] = 'سعر البيع القطاعى';
        $collectors = Employee::select('id', 'name', 'collecting_ratio as ratio')->where(['shop_id' => auth()->user()->shop_id, 'is_employee' => true, 'work' => true, 'collecting' => true])->get();
        $groupLists = groupList();
        $cityLists  = $this->cityList();
        $row        = $this->getModule();
        $priceLists = $this->priceList();
        $countryLists = $this->countryList();
        $settings   = Badrshop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $embedded = 0;
        $invoice_type = null;
        if (isset($request->embedded)) {
            $embedded = $request->embedded;
        }
        $view = 'clients::create';
        if ($request->has('invoice_type')) {
            $invoice_type = $request->invoice_type;
            $view = 'clients::client-form';
        }

        $client_code = Client::maxCode();

        return view($view, compact('row', 'cityLists', 'priceLists', 'groupLists', 'settings', 'countryLists', 'embedded', 'invoice_type', 'collectors', 'list_prices', 'client_code'));
    }

    public function store(ClientRequest $request)
    {
        if (in_array($request->client_role, [0, 2])  &  !authorized('can_add_client')) {
            return unAuthorized(trans('users.add_client'));
        } else if (in_array($request->client_role, [1, 2])  & !authorized('can_add_supplier')) {
            return unAuthorized(trans('users.add_supplier'));
        }

        $row      = $this->getModule();
        $add_date = $this->day_now($row->date_type);
        $settings = Badrshop::where('serial_id', get_session_val('serial_id', 'integer'))->first();

        // New Client
        $client = new Client();

        // If E-Invoice is turn on
        if ($settings->einvoice == 1) {
            $client->regionCity     = $request->regionCity;
            $client->street         = $request->street;
            $client->buildingNumber = $request->buildingNumber;
            $client->postalCode     = $request->postalCode;
            $client->floor          = $request->floor;
            $client->room           = $request->room;
            $client->landmark       = $request->landmark;
        }
        $client->code              = $request->code;
        $client->governate         = $request->governate;
        $client->email             = $request->email;
        $client->country_id        = $request->country_id;
        $client->client_role       = $request->client_role;
        $client->client_name       = $request->client_name;
        $client->address           = $request->address ?? '';
        $client->mobile1           = $request->mobile1 ?? '';
        $client->tele              = $request->tele ?? '';
        $client->balance           = $request->balance ?? '';
        $client->group_id          = $request->group_id;
        $client->info1             = $request->national_id ?? '';
        $client->credit_limit      = $request->credit_limit ?? 0.00;
        $client->date_zero         = Carbon::now();
        $client->display_order     = $request->display_order ?? ' ';
        $client->lat               = $request->lat ?? '';
        $client->lon               = $request->lon ?? '';
        $client->addition_data     = $request->addition_data ?? '';
        $client->info2             = $request->info2 ?? '';
        $client->info3             = $request->info3 ?? '';
        $client->info4             = $request->info4 ?? '';
        $client->info5             = $request->info5 ?? '';
        $client->edit_user         = auth()->id();
        $client->shop_id           = get_session_val('serial_id', 'integer');
        $client->price_list_id     = $request->price_list_id ?? '';
        $client->city_id           = $request->city_id ?? '';
        $client->add_user          = auth()->id();
        $client->gift_points       = $request->gift_points ?? '';
        $client->notify_length     = $request->notify_length ?? '';
        $client->notify_next       = $request->notify_next ?? Carbon::now();
        $client->client_tax_number = $request->client_tax_number ?? '';
        $client->StorePass         = $request->StorePass ?? '';
        $client->StoreTransport    = $request->StoreTransport ?? '';
        $client->StoreServices     = $request->StoreServices ?? '';
        $client->client_type       = $request->client_role == 1 ? false : ($request->client_type ?? false);
        $client->collector_id      = $request->client_role != 1 && $request->client_type != 0 ? $request->collector_id : null;
        $client->add_date          = $add_date;
        $client->send_email        = $request->send_email ?? false;
        $client->send_whatsapp     = $request->send_whatsapp ?? false;
        $client->credit_limit      = $request->credit_limit ?? 0.00;
        $client->price             = $request->price ?? 0.00;

        if ($request->hasFile('image')) {
            if ($request->file('image')->isValid()) {
                $image           = $request->file('image');
                $name            = time() . '' . random_int(1111, 99999) . '.' . $image->getClientOriginalExtension();
                $img             = Image::make($request->file('image'));
                $destinationPath = '../upload/clients/';

                if (!file_exists($destinationPath)) {
                    mkdir($destinationPath, 666, true);
                }

                $img->save($destinationPath . $name);

                //$edit->update($request->except( 'submitbtn' ));
                $client->image = $name;
            }
        } else {
            $client->image = ' ';
        }
        $client->save();
        //save glasses data
        $clientGlasses            = new ClientGlass();
        $clientGlasses->f_sph1    = $request->f_sph1 ?? '';
        $clientGlasses->n_sph1    = $request->n_sph1 ?? '';
        $clientGlasses->f_cyl1    = $request->f_cyl1 ?? '';
        $clientGlasses->n_cyl1    = $request->n_cyl1 ?? '';
        $clientGlasses->f_axis1   = $request->f_axis1 ?? '';
        $clientGlasses->n_axis1   = $request->n_axis1 ?? '';
        $clientGlasses->f_sph2    = $request->f_sph2 ?? '';
        $clientGlasses->n_sph2    = $request->n_sph2 ?? '';
        $clientGlasses->f_cyl2    = $request->f_cyl2 ?? '';
        $clientGlasses->n_cyl2    = $request->n_cyl2 ?? '';
        $clientGlasses->f_axis2   = $request->f_axis2 ?? '';
        $clientGlasses->n_axis2   = $request->n_axis2 ?? '';
        $clientGlasses->client_id = $client->id;
        $clientGlasses->shop_id   = get_session_val('serial_id', 'integer');
        $clientGlasses->save();

        ClientTransaction::create([
            'shop_id' => get_session_val('serial_id', 'integer'),
            'date_time' => date('Y-m-d H:i:s'),
            'client_id' => $client->id,
            'amount' => 0,
            'type' => 4,
            'pay_day' => date('Y-m-d H:i:s'),
            'balance' => $client->balance,
            'user_id' => auth()->id()
        ]);
        $this->createLog(Carbon::now(), "تم انشاء العميل  ". $request->client_name);
        if ($request->embedded)
            Cache::set('embedded_client_' . Auth::id(), $client->id);

        session()->flash('success', 'تم الاضافه بنجاح');

        if ($request->ajax())
            return response()->json();

        if ($request->embedded) {
            return redirect()->route('clients.create', ['embedded' => $request->embedded]);
        }

        return redirect()->route('clients.index')->withInput(['client_id' => $client->id]);
    }

    private function getModule()
    {
        $user = auth()->user();
        $shop = \DB::table('badr_shop')->where('serial_id', $user->shop_id)->first();
        return $shop;
    }

    private function cityList()
    {
        //       $query="select id ,city_name,city_up from cities where not_active=1 and shop_id=".$_SESSION['shop_id']." order by id;";

        $cities = \DB::table('cities')->where('not_active', 1)->where('shop_id', get_session_val('serial_id', 'integer'))->orderBy('id')->get();
        return $cities;
    }

    private function priceList()
    {
        $priceLIsts = \DB::table('prices_list')->where('shop_id', get_session_val('serial_id', 'integer'))->get();
        return $priceLIsts;
    }



    private function countryList()
    {
        $countryList = Country::orderBy("name_ar")->get();
        return $countryList;
    }

    private function day_now($type)
    {
        if ($type == 1) {
            $time = time();
            $obj = new UCal2();
            $obj->setLang('ar');
            $islamic_date = $obj->date('Y-m-d H:i:s', 0, 1);
            return $islamic_date;
        } else {
            return date("Y-m-d H:i:s");
        }
    }

    protected function savePreviousPageNumber()
    {
        $parsedUrl = parse_url(url()->previous());
        $query = $parsedUrl['query'] ?? '';
        parse_str($query, $output);
        if (count($output)) session()->put('previous-page-query-string', $output);
    }

    protected function getPreviousPageNumber()
    {
        return session()->has('previous-page-query-string') ? session()->get('previous-page-query-string') : [];
    }

    public function edit($id)
    {
        if (!authorized('can_display_clients') & !authorized('can_add_supplier')) {
            return unAuthorized(trans('users.show_clients_details') . ' , ' . trans('users.read_supplier_details'));
        }

        $this->savePreviousPageNumber();
        $list_prices = ListPrice::select('id', 'list_name as name')->pluck('name', 'id');
        $list_prices[0] = 'سعر البيع القطاعى';
        $client = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('id', $id)->firstOrFail();

        $collectors = Employee::select('id', 'name', 'collecting_ratio as ratio')->where(['shop_id' => auth()->user()->shop_id, 'is_employee' => true, 'collecting' => true])->get();

        $invoice_type = null;
        if (in_array($client->client_role, [0, 2])  &  !authorized('can_display_clients')) {
            return unAuthorized(trans('users.show_clients_details'));
        } else if (in_array($client->client_role, [1, 2])  & !authorized('can_add_supplier')) {
            return unAuthorized(trans('users.read_supplier_details'));
        }
        $clientGlasses = ClientGlass::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('client_id', $id)->first();

        if (is_null($clientGlasses)) {
            $clientGlasses = new ClientGlass();
        }
        $groupLists = groupList();
        $cityLists  = $this->cityList();
        $row        = $this->getModule();
        $priceLists = $this->priceList();
        $countryLists = $this->countryList();
        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();

        $client_code = Client::maxCode();
        return view('clients::edit', compact('client', 'client_code', 'list_prices', 'row', 'cityLists', 'priceLists', 'groupLists', 'clientGlasses', 'settings', 'countryLists', 'collectors', 'invoice_type'));
    }

    public function update(ClientRequest $request)
    {
        $client = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('id', $request->client_id)
            ->firstOrFail();

        if (in_array($client->client_role, [0, 2])  &  !authorized('can_display_clients')) {
            return unAuthorized(trans('users.show_clients_details'));
        } else if (in_array($client->client_role, [1, 2])  & !authorized('can_add_supplier')) {
            return unAuthorized(trans('users.read_supplier_details'));
        }

        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $row      = $this->getModule();
        $add_date = $this->day_now($row->date_type);

        // If E-Invoice is turn on
        if ($settings->einvoice == 1) {
            $client->regionCity     = $request->regionCity;
            $client->street         = $request->street;
            $client->buildingNumber = $request->buildingNumber;
            $client->postalCode     = $request->postalCode;
            $client->floor          = $request->floor;
            $client->room           = $request->room;
            $client->landmark       = $request->landmark;
        }


        $client->code              = $request->code;
        $client->country_id        = $request->country_id;
        $client->governate         = $request->governate;
        $client->email             = $request->email;
        $client->client_role       = $request->client_role;
        $client->client_name       = $request->client_name;
        $client->address           = $request->address ?? '';
        $client->mobile1           = $request->mobile1 ?? '';
        $client->tele              = $request->tele ?? '';
        // $client->balance           = $request->balance ?? '';
        $client->group_id          = $request->group_id;
        $client->info1             = $request->national_id ?? '';
        $client->credit_limit      = $request->credit_limit ?? 0.00;
        $client->display_order     = $request->display_order ?? '';
        $client->lat               = $request->lat ?? '';
        $client->lon               = $request->lon ?? '';
        $client->addition_data     = $request->addition_data ?? '';
        $client->info2             = $request->info2 ?? '';
        $client->info3             = $request->info3 ?? '';
        $client->info4             = $request->info4 ?? '';
        $client->info5             = $request->info5 ?? '';
        $client->edit_user         = auth()->id();
        $client->shop_id           = get_session_val('serial_id', 'integer');
        $client->price_list_id     = $request->price_list_id ?? '';
        $client->city_id           = $request->city_id ?? '';
        $client->add_user          = auth()->id();
        $client->gift_points       = $request->gift_points ?? '';
        $client->notify_length     = $request->notify_length ?? '';
        $client->notify_next       = $request->notify_next ?? Carbon::now();
        $client->client_tax_number = $request->client_tax_number;
        $client->StorePass         = $request->StorePass ?? '';
        $client->StoreTransport    = $request->StoreTransport ?? '';
        $client->StoreServices     = $request->StoreServices ?? '';
        $client->add_date          = $add_date;
        $client->client_type       = $request->client_role == 1 ? false : ($request->client_type ?? $client->client_type);
        $client->collector_id      = $request->client_role != 1 && $request->client_type != 0 ? $request->collector_id : $client->collector_id;
        $client->send_email        = $request->send_email ?? 0;
        $client->send_whatsapp     = $request->send_whatsapp ?? 0;
        $client->price             = $request->price ?? 0;
        $client->image = $client->image;

        if ($request->hasFile('image')) {
            if ($request->file('image')->isValid()) {
                $image           = $request->file('image');
                $name            = time() . '' . random_int(1111, 99999) . '.' . $image->getClientOriginalExtension();
                $img             = \Image::make($request->file('image'));
                $destinationPath = '../upload/clients/';

                if (!file_exists($destinationPath)) {
                    mkdir($destinationPath, 666, true);
                }

                $img->save($destinationPath . $name);

                //$edit->update($request->except( 'submitbtn' ));
                $client->image = $name;
            }
        }

        $client->save();

        // update glasses data
        $clientGlasses = ClientGlass::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('client_id', $request->client_id)->first();

        if (is_null($clientGlasses)) {
            $clientGlasses = new ClientGlass();
        }

        $clientGlasses->f_sph1 = $request->f_sph1 ?? ' ';
        $clientGlasses->n_sph1 = $request->n_sph1 ?? ' ';
        $clientGlasses->f_cyl1 = $request->f_cyl1 ?? ' ';
        $clientGlasses->n_cyl1 = $request->n_cyl1 ?? ' ';
        $clientGlasses->f_axis1 = $request->f_axis1 ?? ' ';
        $clientGlasses->n_axis1 = $request->n_axis1 ?? ' ';
        $clientGlasses->f_sph2 = $request->f_sph2 ?? ' ';
        $clientGlasses->n_sph2 = $request->n_sph2 ?? ' ';
        $clientGlasses->f_cyl2 = $request->f_cyl2 ?? ' ';
        $clientGlasses->n_cyl2 = $request->n_cyl2 ?? ' ';
        $clientGlasses->f_axis2 = $request->f_axis2 ?? ' ';
        $clientGlasses->n_axis2 = $request->n_axis2 ?? ' ';
        $clientGlasses->client_id = $client->id;
        $clientGlasses->shop_id = get_session_val('serial_id', 'integer');
        $clientGlasses->save();
        $this->createLog(Carbon::now(), "تم تعديل علي العميل  ". $client->client_name);
        session()->flash('success', 'تم تعديل العميل بنجاح');
        return redirect()->route('clients.index', $this->getPreviousPageNumber());
    }

    public function delete(Request $request)
    {
        $client = Client::where('id', $request->id)->where('shop_id', get_session_val('serial_id', 'integer'))->first();

        if (in_array($client->client_role, [0, 2])  &  !authorized('can_display_clients')) {
            return unAuthorized(trans('users.show_clients_details'));
        } else if (in_array($client->client_role, [1, 2])  & !authorized('can_add_supplier')) {
            return unAuthorized(trans('users.read_supplier_details'));
        }

        $sale_process = Invoices_::where('shop_id', get_session_val('serial_id', 'integer'))->where('client_id', $client->id)->first();
        $client_transaction = ClientTransaction::where('shop_id', get_session_val('serial_id', 'integer'))->where('client_id', $client->id)->whereIn('type', [9, 2])->first();
        if ($sale_process) {
            $message = '<p class="alert alert-danger text-center">لا يمكن الحذف لوجود عمليات بيع وشراء مرتبطة بالعميل</p>';
            $type = 'error';
        } else if ($client_transaction) {
            $message = '<p class="alert alert-danger text-center">لا يمكن الحذف لوجود عمليات استلام وصرف مرتبطة بالعميل</p>';
            $type = 'error';
        } else {
            $this->createLog(Carbon::now(), "تم حذف العميل  ". $client->client_name);
            $client->delete();
            $message = '<p class="alert alert-success text-center">تم الحذف بنجاح</p>';
            $type = 'success';
        }

        return response()->json([
            'type' => $type,
            'message' => $message,
        ]);
    }

    public function ClientsPoints()
    {
        $clients = Client::where('shop_id', get_session_val('serial_id', 'integer'))->select('client_name', 'gift_points')->get();
        $point_price = Badrshop::where('serial_id', get_session_val('serial_id', 'integer'))->select('bolla_gift', 'currency')->first();
        return view('clients::clients-points', compact('clients', 'point_price'));
    }

    public function ClientOperations($id, Request $request)
    {
        $client = Client::find($id);
        return redirect()->route('clients.show', ['id' => $id, 'details' => 'true']);

        /*
        $transactions = ClientTransaction::where(function ($q) use ($request, $client) {
            $q->where('shop_id', get_session_val('serial_id' ,'integer'));
            $q->where('client_id', $client->id);
            $q->where('is_deleted', 0);
            if ($request->filled('from')) {
                $q->where('pay_day', '>=', $request->from);
            }
            if ($request->filled('to')) {
                $q->where('pay_day', '<=', $request->to);
            }
        })->whereIn('type', ['0', '1', '2', '8', '9', '60'])->orderBy('pay_day', 'asc')->get();
        $shop = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id' ,'integer'))->first();
        $currency = $shop->currency;
        $type = $shop->date_type;
        return view('clients::client_operations', compact('client', 'transactions', 'currency', 'type'));
        */
    }

    public function show($id, Request $request)
    {
        $shop = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $type = $shop->date_type;
        $client = Client::where('shop_id', get_session_val('serial_id', 'integer'))->where('id', $id)->first();
        $group = \DB::table('clients_groups')->find($client->group_id)->name ?? '';
        $client->group_name = $group;
        $transactions = ClientTransaction::where(function ($q) use ($request, $client) {
            $q->where('shop_id', get_session_val('serial_id', 'integer'));
            $q->where('client_id', $client->id);
            $q->where('is_deleted', 0);
            if ($request->filled('from')) {
                $q->where('pay_day', '>=', $request->from);
            }
            if ($request->filled('to')) {
                $q->where('pay_day', '<=', $request->to);
            }
        })->whereIn('type', ['0', '1', '2', '4', '8', '9'])
            ->orderBy('pay_day', 'asc')
            ->orderBy('id', 'asc')
            ->get();
        return view('clients::show', compact('client', 'shop', 'type', 'transactions'));
    }

    public function deleteTransaction($id)
    {
        $transaction = ClientTransaction::findOrFail($id);
        $operation = DB::transaction(function () use ($transaction) {
            $type = $transaction->type;
            $sale_point = SalePoint::findOrFail($transaction->safe_point_id);
            $client = Client::findOrFail($transaction->client_id);
            if ($type == 2) {
                $reverse_type = 9; // to client
                $effect = 0;
                $sale_point->money_point -= $transaction->amount;
                $client->balance += $transaction->amount;
            } else {
                $reverse_type = 2; // from client
                $effect = 1;
                $sale_point->money_point += $transaction->amount;
                $client->balance -= $transaction->amount;
            }
            $sale_point->save();
            $client->save();
            $reverse_transaction = new ClientTransaction();
            $reverse_transaction->client_id = $client->id;
            $reverse_transaction->amount = $transaction->amount;
            $reverse_transaction->pay_day = Carbon::today();
            $reverse_transaction->date_time = Carbon::today();
            $reverse_transaction->balance = $client->balance;
            $reverse_transaction->type = $reverse_type;
            $reverse_transaction->bill_id = 0;
            $reverse_transaction->sale_back_id = 0;
            $reverse_transaction->effect = $effect;
            $reverse_transaction->safe_balance = $sale_point->money_point;
            $reverse_transaction->safe_point_id = $sale_point->id;
            $reverse_transaction->shop_id = get_session_val('serial_id', 'integer');
            $reverse_transaction->user_id = auth()->id();
            $reverse_transaction->is_deleted = 1;
            $reverse_transaction->save();
            $transaction->is_deleted = 1;
            $transaction->save();
        });
        if ($operation) {
            session()->flash('error', 'فشل الحذف');
        } else {
            session()->flash('success', 'تم الحذف بنجاح');
        }
        return back();
    }

    public function print($id)
    {
        $client = Client::find($id);
        $bill_settings = InvoiceSettings::where('shop_id', get_session_val('serial_id', 'integer'))->first();

        return view('clients::print', compact('client', 'bill_settings'));
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}
