<?php

namespace Modules\Clients\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Controllers\Controller;
use Modules\Clients\Entities\BackDetails;
use Modules\Clients\Entities\BillsAdd;
use Modules\Clients\Entities\BillsAddHistory;
use Modules\Clients\Entities\Client;
use Modules\Clients\Entities\ClientTransaction;
use Modules\Clients\Entities\IncomeBill;
use Modules\Clients\Entities\IncomeBillReturn;
use Modules\Clients\Entities\IncomeBillReturnDetails;
use Modules\Clients\Entities\IncomeDetails;
use Modules\Clients\Entities\SaleBack;
use Modules\Clients\Entities\SaleDetails;
use Modules\Clients\Entities\SaleProcess;
use Modules\Clients\Entities\Supplier;

class SupplierReportController extends Controller
{

    public function getSuppliersBills(Request $request)
    {

        $clients = Client::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        $bills = [];

        if ($request->has(['client', 'from_date', 'to_date'])) {

            $client_id = $request->client;
            $from = $request->from_date;
            $to = $request->to_date;

            $bills = SaleProcess::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('client_id', $client_id)
                ->whereDate('sale_date', '>=', $from)
                ->whereDate('sale_date', '<=', $to)
                ->select('id', 'sale_date', 'total_price', 'discount', 'net_price', 'bill_no')
                ->get();

            $discount = $bills->sum('discount');
            $total = $bills->sum('total_price');
            $net = $bills->sum('net_price');

            $name = Client::find($client_id)->client_name;
            return view('clients::client_report_1', compact('clients', 'bills', 'name', 'discount', 'total', 'net'));

        }

        return view('clients::client_report_1', compact('clients', 'bills'));


    }

    public function showBackPurchaseBill($id)
    {
        $qrUrl = config('main.badr') . "tools/api/generate-QR-code?bill_id=" . $id . '&code=' . get_session_val('serial_id', 'integer') . '&bill_type=purchase-return';
        $qr = @file_get_contents(urldecode($qrUrl));
        $bill_ = IncomeBillReturn::where('id', $id)->first();
        if ($bill_) {
            $bill = $bill_;

            $current_balance = ClientTransaction::where([
                'shop_id' => get_session_val('serial_id', 'integer'),
                'type' => 5,
                'bill_id' => $bill->id
            ])->first()->balance;

            $old_balance = $bill->supplier_balance + ($bill->net_price - $bill->payment);

            $dis = $bill->discount;
            if ($bill->discount_type == '0') {
                $dis = $bill->discount / 100 * $bill->total_bill;
            }
            $bill->discountM = $dis;

            $details = IncomeBillReturnDetails::join('items', function ($q) {
                $q->on('items.id', 'incoming_details_return.items_id');
            })
                ->where('incoming_details_return.shop_id', get_session_val('serial_id', 'integer'))
                ->where('bill_id', $id)
                ->selectRaw(
                    'incoming_details_return.id, incoming_details_return.items_id, incoming_details_return.quantity, 
                incoming_details_return.price, incoming_details_return.total, items.item_name'
                )
                ->get();

//            dd($bill, $details);



            $img = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->logo_path;
            $currency = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->currency;
            $adds = BillsAddHistory::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('type', 5)->where('bill_id', $bill->id)->get();

            foreach ($adds as $add) {
                $ad = BillsAdd::where('shop_id', get_session_val('serial_id', 'integer'))->whereId($add->addition_id)->first();
                $add->name = $ad->Addition_name;
            }

            $supplier = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))->whereId($bill->supplier_id)->first();
            return view('clients::supplier.show-purchase-back-bill',
                compact('bill', 'details', 'supplier', 'img', 'currency', 'adds', 'current_balance', 'old_balance', 'qr'));

        } else {

            $bill = SaleProcess::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $id)
                ->where('bill_type', '4')
                ->firstOrFail();

            $details = SaleDetails::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('sale_id', $id)
                ->get();

            $img = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->logo_path;
            $currency = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->currency;
            $adds = BillsAddHistory::where('shop_id', get_session_val('serial_id', 'integer'))->where('type', 5)
                ->where('bill_id', $bill->id)->get();
            foreach ($adds as $add) {
                $ad = BillsAdd::where('shop_id', get_session_val('serial_id', 'integer'))->whereId($add->addition_id)->first();
                $add->name = $ad->Addition_name;
            }
            $supplier = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))->whereId($bill->supplier_id)->first();

            return view('clients::show-back-purchase-bill', compact('supplier', 'img', 'bill', 'details', 'currency', 'adds', 'qr'));
        }

    }

    public function showPurchaseBill($id)
    {

        $bill = IncomeBill::where('shop_id', get_session_val('serial_id', 'integer'))->where('id', $id)->firstOrFail();

        $current_balance = ClientTransaction::where([
            'shop_id' => get_session_val('serial_id', 'integer'),
            'type' => 3,
            'bill_id' => $bill->id
        ])->first()->balance;

        $old_balance = $bill->supplier_balance - ($bill->net_price - $bill->payment);

        $details = IncomeDetails::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('bill_id', $id)
            ->get();

        $img = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->logo_path;
        $bill_setting = \DB::table('bill_setting')->where('shop_id', get_session_val('serial_id', 'integer'))->first();

        $currency = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->currency;
        $adds = BillsAddHistory::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('type', 0)->where('bill_id', $bill->id)->get();
        foreach ($adds as $add) {
            $ad = BillsAdd::find($add->addition_id);
            $add->name = $ad->Addition_name ?? '';
        }
        $supplier = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))->whereId($bill->supplier_id)->first();

        $qrUrl = config('main.badr') . "tools/api/generate-QR-code?bill_id=" . $id . '&code=' . get_session_val('serial_id', 'integer') . '&bill_type=purchase';
        $qr = @file_get_contents(urldecode($qrUrl));

        return view('clients::show-purchase-bill',
            compact('bill', 'details', 'supplier', 'img', 'currency', 'adds', 'current_balance', 'old_balance', 'bill_setting', 'qr'));
    }

}
