<?php

namespace Modules\Clients\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Controllers\Controller;
use App\User;
use Illuminate\Support\Facades\DB;
use Modules\Clients\Entities\Badrshop;
use Modules\Clients\Entities\BalanceSheetRepository;
use Modules\Clients\Entities\Client;
use Modules\Clients\Entities\ClientTransaction;
use Modules\Clients\Entities\Helper;
use Modules\Clients\Entities\IncomeBill;
use Modules\Clients\Entities\IncomeDetails;
use Modules\Clients\Entities\SaleBack;
use Modules\Clients\Entities\SalePoint;
use Modules\Clients\Entities\SaleProcess;
use Modules\Clients\Entities\Supplier;
use Modules\Clients\Entities\TransactionFactory;
use Modules\Clients\Helpers\UCal2;

class SuppliersController extends Controller
{
    private $transactionFactory;
    private $balanceSheetRepo;

    public function __construct(TransactionFactory $transactionFactory, BalanceSheetRepository $balanceSheetRepository)
    {
        $this->transactionFactory = $transactionFactory;
        $this->balanceSheetRepo = $balanceSheetRepository;
    }

    public function index(Request $request)
    {
        $suppliers = Supplier::where('shop_id', get_session_val('serial_id', 'integer'));

        if ($request->has('name') && !in_array(trim(rtrim($request->name)), ['', null])) {
            $suppliers = $suppliers->where('supplier_name', 'LIKE', '%' . trim(rtrim($request->name)) . '%');
        }
        $suppliers = $suppliers->get();
        return view('clients::supplier.index', compact('suppliers'));
    }

    public function allBalances(Request $request)
    {
        $suppliers = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))
            ->select('supplier_name', 'balance')
            ->get();
        $today = Carbon::now();
        return view('clients::supplier.all_balances', compact('suppliers', 'today'));
    }

    public function show2($id, Request $request)
    {
        $pay_day = ClientTransaction::where('shop_id', get_session_val('serial_id', 'integer'))->orderBy('pay_day', 'ASC')->first();
        if (is_null($pay_day) || ($pay_day && $pay_day->pay_day == '0000-00-00')) {
            $activ =   get_session_val('active_date' ,'string');
        }else {
            $activ = $pay_day->pay_day;
        }
        $from = Carbon::parse($activ)->format('Y-m-d');
        if ($request->has('from')) {
            $from = $request->from;
        }
        $to = '';
        if ($request->has('to')) {
            $to = $request->to;
        }
        $status = 'content';
        $collection = $this->getSupplierSheet($id, $from, $to, $status);

        #### get previous sheet balance ####
        $status = 'prev';

        $pre_balance = 0;
        $balance = 0;

        if ($from != '' ){
            $pre_sheet = $this->getSupplierSheet($id, $from, $to, $status);
            if (count($pre_sheet) > 0) {
                foreach ($pre_sheet as $sheet) {
                    $balance = $sheet->balance;
                    if ($sheet->type != 8){
                        $balance = balance($sheet, $pre_balance);
                    }
                    if (in_array($sheet->type, [3,32,33]) && $sheet->has_bill == false){
                        $balance = $sheet->balance;
                    }
                    $pre_balance = $balance;
                }
            }
        }else {
            $pre_sheet = ClientTransaction::where('shop_id', get_session_val('serial_id', 'integer'))->where('supplier_id', $id)->where('type', 4)->first();
            
            if (is_null($pre_sheet) ){
                $pre_sheet = ClientTransaction::where('shop_id', get_session_val('serial_id', 'integer'))->where('supplier_id', $id)->first();
                if ($pre_sheet){
                    $pre_balance = $pre_sheet->balance;
                    $balance = $pre_balance;
                }
            }else {
                $pre_balance = $pre_sheet->balance;
                $balance = $pre_balance;
            }
        }


        $shop = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();

        $multi_currency = $shop->multi_currency;
        $module = $shop->module;

        $supplier = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))->where('id', $id)->first();

        $newEntry = ClientTransaction::where('client_transaction.shop_id', get_session_val('serial_id', 'integer'))
            ->where('client_transaction.supplier_id', $id)->where('type', 4)->first();

        $request->merge(['from' => $from, 'to' => $to]);

//        dd($collection);

        return view('clients::supplier.show-new', compact('from', 'to', 'supplier', 'shop', 'collection', 'pre_balance',  'newEntry', 'multi_currency', 'module'));
    }
    public function getsupplierSheet($id, $from, $to, $status)
    {
        $clientTransactions = $this->balanceSheetRepo->balanceSheetSupplier($id, $from, $to, $status);

        $collection = collect();
        $clientTransactions->chunk(20)->each(function ($chucked_transactions) use (&$collection) {
            $chucked_transactions->each(function ($ct) use (&$collection) {
                $billCheck = IncomeBill::find($ct->bill_id);
                $point = SalePoint::find($billCheck ? $billCheck->mony_safe : 0);
                 $t = $this->collectionToObjectSupplier($ct);
                $has_bill = true;
                if (in_array($ct->type, [3, 33, 32, 70]) && is_null($billCheck)) {
                    $has_bill = false;
                }
                $t->has_bill = $has_bill;
                $t->incomingBill = $billCheck;
                $t->salePoint  = $point;
                $user = User::find($ct->user_id);
                $t->user  = $user;
                $incomeDetails = $billCheck ? IncomeDetails::where('bill_id', $ct->bill_id)->get() : null;
                $t->incomeDetails  = $incomeDetails;

                $collection->push($t);
            });
        });
        return $collection;
    }

    public function showNewModel($id)
    {
        $clientTransactions = $this->balanceSheetRepo->balanceSheet($id);
        $collection = collect();
        foreach ($clientTransactions as $ct) {
            $t = $this->collectionToObject($ct);
            $collection->push($t);
        }

        $shop = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();

        $client = Client::where('shop_id', get_session_val('serial_id', 'integer'))->where('id', $id)->first();

        return view('clients::show2', compact('client', 'shop', 'collection'));
    }

    public function search(Request $request)
    {
        $suppliers = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('supplier_name', "LIKE", "%" . $request->client_name . "%")
            ->get();
        return view('clients::supplier.index', compact('suppliers'));
    }

    private function collectionToObjectSupplier($object)
    {
        $shop = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $module = $shop->module;

        $t = $this->transactionFactory->factorySupplier($object);
        $t->first_cell = $this->balanceSheetRepo->issuedToHimSupplier($object);
        $t->second_cell = $this->balanceSheetRepo->comeFromHimSupplier($object);
        $t->transaction_type = $this->transactionTypeSupplier($object->type, $object->bill_id, $object->sale_back_id, $module);
        return $t;
    }

    private function itemsCount($billId)
    {
        $details = IncomeDetails::where([
            'shop_id' => get_session_val('serial_id', 'integer'),
            'bill_id' => $billId
        ])
            ->select('quantity')
            ->get();
        $count = 0;
        if ($details) {
            if ($details) {
                $count = $details->sum('quantity');
            }
        }
        return 'عدد السيارات : '.$count;
    }

    private function transactionTypeSupplier($type, $billId, $backId, $module)
    {
        $shop = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $junk_bill = $shop->junk_bill;
        switch ($type) {

            case '3': //فاتورة مشتريات رقم
                //   $no = Helper::getPurchaseBillNo($billId);
                $url = route('suppliers.show-buy-bill', $billId);
                if ($junk_bill == 1) {
                    $url = route('supplierorders.print-junk', $billId);
                }
                $count = $this->itemsCount($billId);
                if ($module == 21) {
                    $type = '<a href="'.$url.'"> فاتورة مشتريات رقم '. $billId .' - '. $count .' </a>';
                } else {
                    $type = '<a href="' . $url . '"> فاتورة مشتريات رقم ' . $billId . '</a>';
                }
                $bb= \DB::table('incoming_bill')->find($billId);
                $ty = ($bb && strlen($bb->supplier_bill_no) ) ? "<br>رقم فاتورة المورد: " . $bb->supplier_bill_no : '';
                $type .= $ty;
                //                $type = ' فاتورة مشتريات رقم' . $billId;
                break;
            case '4': //رصيد جديد
                $type = 'رصيد جديد';
                break;
            case '5': //فاتورة مرتجع مشتريات
                $no = Helper::getBackPurchaseBillNo($billId);
                $url = route('suppliers.show-buy-back-bill', $billId);
                $type = '<a href="' . $url . '"> فاتورة مرتجع مشتريات ' . $no . '</a>';
                //   $type = ' فاتورة مرتجع مشتريات' . $backId;
                break;
            case '6': //دفع للشركة
                $type = 'دفع للمورد';
                break;
            case '7': //مردود  نقدى من  الشركة
                $type = 'مردود نقدى من المورد';
                break;

            case '32': //إضافة صنف جديد لفاتورة الشراء
                $url = route('suppliers.show-buy-bill', $billId);
                $type = '<a href="' . $url . '"> تعديل فى بيانات صنف فى فاتورة شراء رقم ' . $billId . '</a>';
                //                $type = 'تعديل فى بيانات صنف فى فاتورة شراء' . $billId;
                break;
            case '33': //حذف صنف من فاتورة الشراء
                $url = route('suppliers.show-buy-bill', $billId);
                $type = '<a href="' . $url . '"> حذف صنف من فاتورة الشراء رقم ' . $billId . '</a>';
                //                $type = ' حذف صنف من فاتورة الشراء' . $billId;
                break;

            case '61': //خصم رصيد مورد
                $type = "رصيد معدوم ";
                break;

            case '65': //اضافة صنف لفاتورة مرتجع شراء
                $no = Helper::getBackPurchaseBillNo($billId);
                $url = route('suppliers.show-buy-back-bill', $billId);
                $type = '<a href="' . $url . '"> اضافة صنف لفاتورة مرتجع شراء رقم ' . $no . '</a>';
                //                $type = " اضافة صنف لفاتورة مرتجع شراء رقم  ".$backId;
                break;
            case '66': //حذف صنف من فاتورة مرتجع شراء
                $no = Helper::getBackPurchaseBillNo($billId);
                $url = route('suppliers.show-buy-back-bill', $billId);
                $type = '<a href="' . $url . '"> حذف صنف من فاتورة مرتجع شراء رقم ' . $no . '</a>';
                //                $type = " حذف صنف من فاتورة مرتجع شراء رقم  ".$backId;
                break;

            case '70': //حذف صنف من فاتورة مرتجع شراء
                $no = Helper::getPurchaseBillNo($billId);
                $url = route('suppliers.show-buy-bill', $billId);
                $type = '<a href="' . $url . '"> اضافة صنف الي فاتورة شراء رقم ' . $no . '</a>';
                //                $type = " حذف صنف من فاتورة مرتجع شراء رقم  ".$backId;
                break;

            default:
                $type = "....";
                break;
        }
        return $type;
    }

    public function create()
    {
        $groupLists = $this->groupList();
        $cityLists = $this->cityList();
        $row = $this->getModule();
        $priceLists = $this->priceList();
        return view('clients::create', compact('row', 'cityLists', 'priceLists', 'groupLists'));
    }

    public function store(Request $request)
    {

        $row = $this->getModule();
        $add_date = $this->day_now($row->date_type);
        $this->validate($request, [
            'info1' => 'unique:clients,info1,NULL,shop_id',
            'client_name' => 'required',
            'client_tax_number' => 'required',
        ]);

        $client = new Client();
        $client->client_name = $request->client_name;
        $client->address = $request->address ?? ' ';
        $client->mobile1 = $request->mobile1 ?? ' ';
        $client->tele = $request->tele ?? ' ';
        $client->balance = $request->balance ?? ' ';
        $client->group_id = $request->group_id ?? ' ';
        $client->info1 = $request->info1 ?? '';
        $client->credit_limit = $request->credit_limit ?? ' ';
        $client->date_zero = Carbon::now();

        $client->display_order = $request->display_order ?? ' ';
        $client->lat = $request->lat ?? ' ';
        $client->lon = $request->lon ?? ' ';
        $client->addition_data = $request->addition_date ?? ' ';
        $client->info2 = $request->info2 ?? ' ';
        $client->info3 = $request->info3 ?? ' ';
        $client->info4 = $request->info4 ?? ' ';
        $client->info5 = $request->info5 ?? ' ';
        $client->edit_user = auth()->id();
        $client->shop_id = get_session_val('serial_id', 'integer');
        $client->price_list_id = $request->price_list_id ?? ' ';
        $client->city_id = $request->city_id ?? ' ';
        $client->add_user = auth()->id();
        $client->gift_points = $request->gift_points ?? ' ';
        $client->notify_length = $request->notify_length ?? ' ';
        $client->notify_next = $request->notify_next ?? Carbon::now();

        $client->client_tax_number = $request->client_tax_number;
        $client->StorePass = $request->StorePass ?? ' ';
        $client->StoreTransport = $request->StoreTransport ?? ' ';
        $client->StoreServices = $request->StoreServices ?? ' ';
        $client->add_date = $add_date;
        $path = '../upload/clients/';
        if ($request->hasFile('image')) {
            if ($request->file('image')->isValid()) {
                $image = $request->file('image');
                $name = time() . '' . random_int(1111, 99999) . '.' . $image->getClientOriginalExtension();
                $img = \Image::make($request->file('image'));
                $img->save($path . $name);

                //$edit->update($request->except( 'submitbtn' ));
                $client->image = $name;

            }
        } else {
            $client->image = ' ';
        }

        $client->save();
        session()->flash('success', 'تم إضافة عميل جديد بنجاح');
        return redirect()->route('clients.index');

    }

    private function getModule()
    {
        $user = auth()->user();
        $shop = \DB::table('badr_shop')->where('serial_id', $user->shop_id)->first();
        return $shop;
    }

    private function cityList()
    {
        //       $query="select id ,city_name,city_up from cities where not_active=1 and shop_id=".$_SESSION['shop_id']." order by id;";
        $cities = \DB::table('cities')->where('not_active', 1)->where('shop_id', get_session_val('serial_id', 'integer'))->orderBy('id')->get();
        return $cities;
    }

    private function priceList()
    {
        $priceLIsts = \DB::table('prices_list')->where('shop_id', get_session_val('serial_id', 'integer'))->get();
        return $priceLIsts;
    }

    private function groupList()
    {
        $groupLists = \DB::table('clients_groups')->where('shop_id', get_session_val('serial_id', 'integer'))->get();
        return $groupLists;
    }

    private function day_now($type)
    {
        if ($type == 1) {
            $time = time();
            $obj = new UCal2();
            $obj->setLang('ar');
            $islamic_date = $obj->date('Y-m-d H:i:s', 0, 1);
            return $islamic_date;
        } else {
            return date("Y-m-d H:i:s");
        }

    }

    public function edit($id)
    {
        $client = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('id', $id)->firstOrFail();

        $groupLists = $this->groupList();
        $cityLists = $this->cityList();
        $row = $this->getModule();
        $priceLists = $this->priceList();
        return view('clients::edit', compact('client', 'row', 'cityLists', 'priceLists', 'groupLists'));
    }

    public function update(Request $request)
    {

        $client = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('id', $request->client_id)
            ->firstOrFail();

        $row = $this->getModule();
        $add_date = $this->day_now($row->date_type);
        $this->validate($request, [
            //            'info1' => 'unique:clients,info1,NULL,shop_id,'.$client->id,
            'client_name' => 'required',
            'client_tax_number' => 'required',
        ]);

        $client->client_name = $request->client_name;
        $client->address = $request->address ?? ' ';
        $client->mobile1 = $request->mobile1 ?? ' ';
        $client->tele = $request->tele ?? ' ';
        $client->balance = $request->balance ?? ' ';
        $client->group_id = $request->group_id ?? ' ';
        $client->info1 = $request->info1 ?? '';
        $client->credit_limit = $request->credit_limit ?? ' ';

        $client->display_order = $request->display_order ?? ' ';
        $client->lat = $request->lat ?? ' ';
        $client->lon = $request->lon ?? ' ';
        $client->addition_data = $request->addition_date ?? ' ';
        $client->info2 = $request->info2 ?? ' ';
        $client->info3 = $request->info3 ?? ' ';
        $client->info4 = $request->info4 ?? ' ';
        $client->info5 = $request->info5 ?? ' ';
        $client->edit_user = auth()->id();
        $client->shop_id = get_session_val('serial_id', 'integer');
        $client->price_list_id = $request->price_list_id ?? ' ';
        $client->city_id = $request->city_id ?? ' ';
        $client->add_user = auth()->id();
        $client->gift_points = $request->gift_points ?? ' ';
        $client->notify_length = $request->notify_length ?? ' ';
        $client->notify_next = $request->notify_next ?? Carbon::now();

        $client->client_tax_number = $request->client_tax_number;
        $client->StorePass = $request->StorePass ?? ' ';
        $client->StoreTransport = $request->StoreTransport ?? ' ';
        $client->StoreServices = $request->StoreServices ?? ' ';
        $client->add_date = $add_date;
        if ($request->hasFile('image')) {
            if ($request->file('image')->isValid()) {
                $image = $request->file('image');
                $name = time() . '' . random_int(1111, 99999) . '.' . $image->getClientOriginalExtension();
                $img = \Image::make($request->file('image'));
                $img->save('../upload/clients/' . $name);

                //$edit->update($request->except( 'submitbtn' ));
                $client->image = $name;

            }
        } else {
            $client->image = ' ';
        }

        $client->save();
        session()->flash('success', 'تم تعديل العميل بنجاح');
        return redirect()->route('clients.index');

    }

    public function delete(Request $request)
    {
        $client = Client::where('id', $request->id)->where('shop_id', get_session_val('serial_id', 'integer'))->first();

        $sale_process = SaleProcess::where('shop_id', get_session_val('serial_id', 'integer'))->where('client_id', $client->id)->first();
        if ($sale_process) {
            $message = 'لا يمكن الحذف لوجود عمليات بيع وشراء مرتبطة بالعميل';
            $type = 'error';
        } else {
            $sale_back = SaleBack::where('shop_id', get_session_val('serial_id', 'integer'))->where('client_id', $client->id)->first();
            if ($sale_back) {
                $message = '<p class="alert alert-danger text-center">لا يمكن الحذف لوجود عمليات بيع وشراء مرتبطة بالعميل</p>';
                $type = 'error';
            } else {
                $client->delete();
                $message = '<p class="alert alert-success text-center">تم الحذف بنجاح</p>';
                $type = 'success';
            }
        }
        return response()->json([
            'type' => $type,
            'message' => $message,
        ]);
    }

    public function show($id, Request $request)
    {
        $shop = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $type = $shop->date_type;
        $supplier = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))->where('id', $id)->first();
        $transactions = ClientTransaction::where(function ($q) use ($request, $supplier) {
            $q->where('shop_id', get_session_val('serial_id', 'integer'));
            $q->where('supplier_id', $supplier->id);
            $q->where('is_deleted', 0);
            if ($request->filled('from')) {
                $q->where('pay_day', '>=', $request->from);
            }
            if ($request->filled('to')) {
                $q->where('pay_day', '<=', $request->to);
            }
        })->whereIn('type', ['3', '4', '5', '6', '7'])
        // ->orderBy('pay_day', 'asc')
            ->orderBy('id', 'asc')->get();
        return view('clients::supplier.show', compact('supplier', 'shop', 'type', 'transactions'));
    }

    public function deleteTransaction($id)
    {
        $transaction = ClientTransaction::findOrFail($id);
        $operation = DB::transaction(function () use ($transaction) {
            $type = $transaction->type;
            $sale_point = SalePoint::findOrFail($transaction->safe_point_id);
            $supplier = Supplier::findOrFail($transaction->supplier_id);
            if ($type == 6) {
                $reverse_type = 7; // from supplier
                $effect = 1;
                $sale_point->money_point += $transaction->amount;
                $supplier->balance += $transaction->amount;
            } else {
                $reverse_type = 6; // to supplier
                $effect = 0;
                $sale_point->money_point -= $transaction->amount;
                $supplier->balance -= $transaction->amount;
            }
            $sale_point->save();
            $supplier->save();
            $reverse_transaction = new ClientTransaction();
            $reverse_transaction->supplier_id = $supplier->id;
            $reverse_transaction->amount = $transaction->amount;
            $reverse_transaction->pay_day = Carbon::today();
            $reverse_transaction->balance = $supplier->balance;
            $reverse_transaction->type = $reverse_type;
            $reverse_transaction->bill_id = 0;
            $reverse_transaction->sale_back_id = 0;
            $reverse_transaction->effect = $effect;
            $reverse_transaction->safe_balance = $sale_point->money_point;
            $reverse_transaction->safe_point_id = $sale_point->id;
            $reverse_transaction->shop_id = get_session_val('serial_id', 'integer');
            $reverse_transaction->user_id = auth()->id();
            $reverse_transaction->is_deleted = 1;
            $reverse_transaction->save();
            $transaction->is_deleted = 1;
            $transaction->save();
        });
        if ($operation) {
            session()->flash('error', 'فشل الحذف');
        } else {
            session()->flash('success', 'تم الحذف بنجاح');
        }
        return back();
    }

}
