<?php

namespace Modules\Clients\Http\Requests;

use Illuminate\Validation\Rule;
use Illuminate\Foundation\Http\FormRequest;
use Modules\Clients\Entities\Badrshop;

class ClientRequest extends FormRequest
{
    public function authorize()
    {
        return true;
    }

    public function rules()
    {
        $settings = Badrshop::with('Country:id,name_en')->where('serial_id',  get_session_val('serial_id', 'integer'))->first();

        // if ($settings->Country->name_en == 'Saudi Arabia') {
        //     $national_id = 'digits_between:10,10';
        //     $mobile1     = 'regex:/(01)[0-9]{5}/';
        // } else {
        //     $national_id = 'digits_between:14,14';
        //     $mobile1     = 'regex:/(01)[0-9]{9}/';
        // }

        // if (empty($this->mobile1) && !$settings->einvoice) {
        //     $mobile1 = $national_id = 'nullable';
        // }

        $rules = [
            'client_role'       => ['required', Rule::in([0, 1, 2])],
            "client_tax_number" => ['nullable', 'numeric', 'unique:clients,client_tax_number,' . $this->client_id . ',id,shop_id,' . get_session_val('serial_id', 'integer')],
            "code"              => ['required', 'numeric', 'unique:clients,code,' . $this->client_id . ',id,shop_id,' . get_session_val('serial_id', 'integer')],
            "governate"         => ['required_if:' . $settings->einvoice == 1, 'max:255'],
            "regionCity"        => ['required_if:' . $settings->einvoice == 1, 'max:255'],
            "street"            => ['required_if:' . $settings->einvoice == 1, 'max:255'],
            "buildingNumber"    => ['required_if:' . $settings->einvoice == 1, 'numeric'],
            "postalCode"        => ['required_if:' . $settings->einvoice == 1, 'numeric'],
            "floor"             => ['required_if:' . $settings->einvoice == 1, 'numeric'],
            "room"              => ['required_if:' . $settings->einvoice == 1, 'numeric'],
            "landmark"          => ['required_if:' . $settings->einvoice == 1, 'max:255'],
            "national_id"       => [($settings->einvoice == 1 ? 'required' : 'nullable'), 'unique:clients,info1,' . $this->client_id . ',id,shop_id,' . get_session_val('serial_id', 'integer')],
            "mobile1"           => [($settings->einvoice == 1 || $settings->telephone_required == 1 ? 'required' : 'nullable'), 'unique:clients,mobile1,' . $this->client_id . ',id,shop_id,' . get_session_val('serial_id', 'integer')],
            "tele"              => ['nullable', 'unique:clients,mobile1,' . $this->client_id . ',id,shop_id,' . get_session_val('serial_id', 'integer')],
            "image"             => ['sometimes', 'image', 'max:2048'],
            'group_id'          => ['required', 'min:0'],
            'client_type'       => ['nullable'],
            'collector_id'      => ['nullable', 'exists:users,id'],
            'send_email'        => ['nullable', 'in:0,1'],
            'send_whatsapp'     => ['nullable', 'in:0,1'],
            'credit_limit'      => ['nullable','numeric','min:0'],
            'price'             => ['nullable','numeric','min:0'],
            'address'           => [(get_session_val('client_address_required', 'integer') == 1 ? 'required' : 'nullable'), 'string'],
        ];

        $rules["client_name"] = $settings->repeat_client_name == 0
            ? ['required', 'string', 'max:255', 'unique:clients,client_name,' . $this->client_id . ',id,shop_id,' . get_session_val('serial_id', 'integer')]
            : ['required', 'string', 'max:255'];

        return $rules;
    }

    public function messages()
    {
        return [

            'client_role.required'       => trans('client.client_role.required'),
            'client_role.in'             => trans('client.client_role.in'),
            'client_name.required'       => trans('client.client_name.required'),
            'client_name.string'         => trans('client.client_name.string'),
            'client_name.max'            => trans('client.client_name.max'),
            'client_name.unique'         => trans('client.client_name.unique'),
            'governate.required'         => trans('client.governate.required'),
            'governate.string'           => trans('client.governate.string'),
            'governate.max'              => trans('client.governate.max'),
            'regionCity.required'        => trans('client.regionCity.required'),
            'regionCity.string'          => trans('client.regionCity.string'),
            'regionCity.max'             => trans('client.regionCity.max'),
            'street.required'            => trans('client.street.required'),
            'street.string'              => trans('client.street.string'),
            'street.max'                 => trans('client.street.max'),
            'buildingNumber.required'    => trans('client.buildingNumber.required'),
            'buildingNumber.numeric'     => trans('client.buildingNumber.numeric'),
            'postalCode.required'        => trans('client.postalCode.required'),
            'postalCode.numeric'         => trans('client.postalCode.numeric'),
            'floor.required'             => trans('client.floor.required'),
            'floor.numeric'              => trans('client.floor.numeric'),
            'room.required'              => trans('client.room.required'),
            'room.numeric'               => trans('client.room.numeric'),
            'landmark.required'          => trans('client.landmark.required'),
            'landmark.string'            => trans('client.landmark.string'),
            'landmark.max'               => trans('client.landmark.max'),
            'client_tax_number.numeric'  => trans('client.client_tax_number.numeric'),
            'client_tax_number.unique'   => trans('client.client_tax_number.unique'),
            'image.image'                => trans('client.image.image'),
            'image.max'                  => trans('client.image.max'),
            'group_id.required'          => trans('client.clientGroup.groupName_required'),
            'credit_limit.numeric'       => trans('client.credit_limit.numeric'),
            'credit_limit.min'       => trans('client.credit_limit.min'),
        ];
    }

    public function attributes()
    {
        return [
            'national_id' => trans('client.national_id.name'),
            'mobile1'     => trans('client.mobile1.name'),
            'code'        => trans('client.code.name'),
        ];
    }
}
