<?php

namespace Modules\Commissions\Http\Controllers;

use App\Models\systemLog;
use Carbon\Carbon;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Commissions\Entities\CommissionPercent;
use Modules\Commissions\Http\Requests\StoreCommission;

class CommissionsController extends Controller
{
    public function index()
    {
        $commissions = CommissionPercent::where('shop_id',get_session_val('serial_id', 'Integer'))->get();
        return view('commissions::index', compact('commissions'));
    }

    public function storeCommissionPercent(StoreCommission $request){

        $validated = $request->validated();
        // dd($validated['from']);
        // return;

        // validate from & and to percents
        $validatePercentsResult = $this->validatePercents($validated);
        // dd($validatePercentsResult);

        if($validatePercentsResult == 'from bigger error'){
            return redirect()->back()->with('error','قيمة (الربح من) اكبر من قيمة (الربح إلي) ');
        }else if($validatePercentsResult == 'from error'){
            return redirect()->back()->with('error','قيمة (الربح من) غير صحيحة');
        }else if($validatePercentsResult == 'to error'){
            return redirect()->back()->with('error','قيمة (الربح إلي) غير صحيحة');
        }else if($validatePercentsResult == 'range error'){
            return redirect()->back()->with('error','قيم الربح تتعارض مع قيمه اخري مسجلة مسبقا');
        }

        $validated['shop_id'] = get_session_val('serial_id', 'Integer');

        if(CommissionPercent::create($validated)){
            $this->createLog(Carbon::now(), "تم اضاف عموله بنسبه " . $request->percent . "علي الربح من " .$request->from . "الي  " .$request->to);
            return redirect()->back()->with('success','تمت الاضافة بنجاح');
        }else{
            return redirect()->back()->with('error','لم تتم الاضافة');
        }

    }

    public function updateCommissionPercent(StoreCommission $request){

        $validated = $request->validated();

        $validatePercentsResult = $this->validatePercents($validated,$request->commission_id);
        // dd($validatePercentsResult);

        if($validatePercentsResult == 'from bigger error'){
            session()->flash('error','قيمة (الربح من) اكبر من قيمة (الربح إلي) ');
            return true;
        }else if($validatePercentsResult == 'from error'){
            session()->flash('error','قيمة (الربح من) غير صحيحة');
            return true;
        }else if($validatePercentsResult == 'to error'){
            session()->flash('error','قيمة (الربح إلي) غير صحيحة');
            return true;
        }else if($validatePercentsResult == 'range error'){
            session()->flash('error','قيم الربح تتعارض مع قيمه اخري مسجلة مسبقا ');
            return true;
        }

        if(CommissionPercent::where('id',$request->commission_id)->where('shop_id',get_session_val('serial_id', 'Integer'))->update($validated))
        {
            $this->createLog(Carbon::now(), "تم تعديل  عموله بنسبه " . $request->percent . "علي الربح من " .$request->from . "الي  " .$request->to);
            session()->flash('success','تم التعديل بنجاح');
        }else{
            session()->flash('error','حدث خطأ ما في التعديل');
        }
        return true;

    }

    private function validatePercents($validated,$id=null){
        if($validated['from'] >= $validated['to']){
            return 'from bigger error';
        }

        $commissions = CommissionPercent::where('shop_id',get_session_val('serial_id', 'Integer'))->get();
        foreach($commissions as $commission){
            if($id == $commission->id){
                continue;
            }


            // (y1 >= x1 && y1 <= x2) ||
            // (y2 >= x1 && y2 <= x2);
            // max(a2, b2) - min(a1, b1) < (a2 - a1) + (b2 - b1)
            // if(max( (float)$validated['to'], (float)$commission->to) < (float)$validated['to'] - ){

            // }
            if($validated['from'] >= $commission->from && $validated['from'] <= $commission->to ){
                return 'from error';
            }
            if($validated['to'] >= $commission->from && $validated['to'] <= $commission->to ){
                return 'to error';
            }
            if($validated['from'] <= $commission->from && $validated['to'] >= $commission->to){
                return 'range error';
            }
        }
    }

    public function getCommissionDetails(CommissionPercent $commission){
        return view('commissions::partials.editModal.edit-commission-percent-modal-body', compact('commission'))->render();
    }

    public function getAllCommissions(){
        $commissions = CommissionPercent::all();
        return view('commissions::partials.commissions-table-body', compact('commissions'))->render();
    }

    public function destroy(Request $request)
    {
        $id = $request->commission_id ?? null;
        $comm = CommissionPercent::where('shop_id',get_session_val('serial_id', 'Integer'))->where('id',$id)->first();

        if ($comm->delete())
        {
            $this->createLog(Carbon::now(), "تم حذف  عموله بنسبه " . $comm->percent . "علي الربح من " .$comm->from . "الي  " .$comm->to);
            session()->flash('success','تم الحذف بنجاح');
        }else{
            session()->flash('error','حدث خطأ ما في الحذف');
        }
        return true;
        // return view('commissions::partials.alert', compact('alert'))->render();
    }

    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}
