<?php

namespace Modules\Currency\Entities;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Concerns\HasEvents;

class CurrencyLog extends Model
{
    use HasEvents;

    protected $table = 'currency_logs';
    protected $fillable = ['log', 'name', 'type', 'details', 'currency_id', 'shop_id'];
    protected $casts = ['details' => 'array'];
    protected $appends = ['message'];
    public const DELETED = 0;
    public const CREATED = 1;
    public const UPDATED = 2;

    protected function getCreatedAtAttribute($value)
    {
        return Carbon::parse($value)->format('Y-m-d h:i:s A');
    }

    protected function getNameAttribute($value)
    {
        return empty($value) ? $this->details['name'] : $value;
    }

    public function getMessageAttribute()
    {
        $message = '';
        if ($this->type == 0) {
            $message = trans("currency::trans.logs.deleted", ['name' => $this->details['name']]);
        } else if($this->type == 1) {
            $message = trans("currency::trans.logs.created", ['name' => $this->details['name'], 'value' => $this->details['value']]);
        } else if($this->type == 2) {
            foreach ($this->details as $column => $data) {
                $message .= trans("currency::trans.logs.updated", ['column' => trans("currency::trans.columns.$column"), 'old' => $data['old'], 'new' => $data['new'], 'main_currency' => Currency::main()]) . "<br>";
            }
        }

        return $message;
    }


    public function currency()
    {
        return $this->belongsTo(Currency::class, 'currency_id', 'id')->select('id', 'name')->withDefault(['name' => '']);
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('shop', function (Builder $builder) {
            $builder->where('shop_id', auth()->user()->shop_id)->orderBy('created_at', 'DESC');
        });

        self::creating(function($model) {
            $model->shop_id = auth()->user()->shop_id;
            $model->name    = $model->currency->name;
        });
    }
}
