<?php

namespace Modules\Currency\Http\Controllers;

use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Maatwebsite\Excel\Facades\Excel;
use Modules\Currency\Entities\Currency;
use Modules\Currency\Entities\CurrencyLog;
use Modules\Currency\Exports\CurrenciesExport;
use Modules\Currency\Http\Requests\CurrencyRequest;
use Modules\Currency\Imports\CurrenciesImport;
use Modules\Currency\Services\ExchangeRate;

class CurrencyController extends Controller
{
    public function index()
    {
        if (request()->ajax())
            return view('currency::includes.table.body', ['rows' => Currency::withCount('salePoints')->get(), 'main_currency_name' => Currency::main()]);
        return view('currency::index');
    }

    public function logs()
    {
        return view('currency::includes.logs.body', ['rows' => CurrencyLog::select('log', 'name', 'type', 'details', 'currency_id', 'created_at')->with('currency')->paginate(5)]);
    }

    public function create()
    {
        return view('currency::create', ['main_currency' => Currency::main('*'), 'currencies' => Currency::currencies()]);
    }

    public function store(CurrencyRequest $request)
    {
        try {
            Currency::create($request->validated());
            return response()->json(['message' => trans('currency::trans.title.created')], 200);
        } catch (\Exception $e) {
            return response()->json($e, 500);
        }
    }

    public function edit($id)
    {
        $row = Currency::find($id);
        if ($row->is_main) return response()->json(trans('currency::trans.title.cannot-update-main'), 500);
        return view('currency::edit', ['row' => $row, 'main_currency' => Currency::main('*'), 'currencies' => Currency::currencies()]);
    }

    public function update(CurrencyRequest $request, $id)
    {
        try {
            $row = Currency::find($id);
            if (!$row) throw new ModelNotFoundException();
            if ($row->is_main) return response()->json(trans('currency::trans.title.cannot-update-main'), 500);
            $row->update($request->validated());
            return response()->json(['message' => trans('currency::trans.title.updated')], 200);
        } catch (\Exception $e) {
            return response()->json($e, 500);
        }
    }

    public function destroy($id)
    {
        try {
            $row = Currency::withCount('salePoints')->find($id);
            if (!$row) throw new ModelNotFoundException();
            if ($row->is_main) return response()->json(trans('currency::trans.title.cannot-delete-main'), 500);
            if ($row->sale_points_count) return response()->json(trans('currency::trans.title.cannot-delete-has-relation'), 500);
            $row->delete();
            return response()->json(['message' => trans('currency::trans.title.deleted')], 200);
        } catch (\Exception $e) {
            return response()->json($e, 500);
        }
    }

    public function export()
    {
        ob_clean();
        return Excel::download(new CurrenciesExport, 'Currencies.xlsx');
    }

    public function import()
    {
        return view('currency::import');
    }

    public function importSave(Request $request)
    {
        $request->validate(['excel_file' => 'required|file|mimes:xlx,xls,xlsx'], [], ['excel_file' => trans('currency::trans.columns.file')]);
        try {
            $message = '';
            Excel::import(new CurrenciesImport($message), $request->excel_file);
            return response()->json(['message' => $message], 200);
        } catch (\Exception $e) {
            return response()->json($e, 500);
        }
    }

    public function sync($id)
    {
        try {
            $row = Currency::find($id);
            if ($row->is_main) return response()->json(trans('currency::trans.title.cannot-update-main'), 500);
            $row->update(['value' => ExchangeRate::convert( $row->code )]);
            return response()->json(['value' => $row->value, 'message' => trans('currency::trans.title.synced', ['currency' => $row->code])], 200);
        } catch (\Exception $e) {
            return response()->json($e, 500);
        }
    }

    public function syncAll()
    {
        try {
            ini_set('max_execution_time', 9999999999999);
            foreach (Currency::select('code', 'id', 'value')->where('is_main', false)->get() as $row) {
                $row->update(['value' => ExchangeRate::convert( $row->code )]);
            }
            return response()->json(['load' => true, 'message' => trans('currency::trans.title.all-synced')], 200);
        } catch (\Exception $e) {
            return response()->json($e, 500);
        }
    }

    public function getExchangeRate()
    {
        try{
            return response()->json(['value' => ExchangeRate::convert( request()->get('code') )], 200);
        } catch (Exception $e) {
            return response()->json($e, 500);
        }
    }
}
