<?php

namespace Modules\Currency\Imports;

use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\SkipsFailures;
use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use Modules\Currency\Entities\Currency;

class CurrenciesImport implements ToCollection, WithHeadingRow
{
    use SkipsFailures;
    protected int $success  = 0;
    protected array $failes = [];

    public function __construct(public string &$message = '')
    {
        //
    }

    /**
    * @param Collection $collection
    */
    public function collection(Collection $collection)
    {
        foreach ($collection as $index => $row) {
            $row = $row->toArray();
            $this->trimData($row);
            if (! $this->validateData($row, ($index + 1))) continue;
            $this->updateCurrencyData($row);
        }
        $this->message();
    }

    protected function trimData(array &$row): void
    {
        foreach ($row as $key => $value)
            $row[$key] = trim( $value );
    }

    protected function validateData(array $row, int $index): bool
    {
        $errors = [];
        if ( blank($row['name']) ) $errors []  = trans('validation.required', ['attribute' => trans('currency::trans.columns.name')]);
        if ( blank($row['code']) ) $errors []  = trans('validation.required', ['attribute' => trans('currency::trans.columns.code')]);
        if ( blank($row['value']) ) $errors [] = trans('validation.required', ['attribute' => trans('currency::trans.columns.value')]);

        if ( count($errors) ) {
            $this->failes[$index] = implode('<br>', $errors);
            return false;
        }
        return true;
    }

    protected function updateCurrencyData(array $row): void
    {
        try {
            if ($this->store($row))
                $this->success ++;
        } catch(\Exception $e) {
            $this->failes[] = trans('currency::trans.title.cannot-save', ['name' => $row['name']]);
        }
    }

    protected function store(array $row): bool
    {
        $currency = Currency::where('name', $row['name'])->first();

        if ( is_null( $currency ) ) {
            Currency::create($row);
        } else if (! $currency->is_main) {
            $currency->update($row);
        } else {
            return false;
        }
        return true;
    }

    protected function message(): void
    {
        $messages = [];
        $messages[] = trans("currency::trans.title.pluck-updated", ['count' => $this->success]);

        if ( count($this->failes) ) {
            $messages[] = trans("currency::trans.title.pluck-failed", ['count' => count($this->failes)]);
            foreach ($this->failes as $key => $value) {
                $messages[]= trans("currency::trans.title.error-line", ['line' => $key, 'message' => $value]);
            }
        }

        $this->message = implode('<br>', $messages);
    }
}
