@section('js-scripts')
    <script src="{{ asset('tickets/js/bootstrap-select.min.js') }}"></script>

    <script>
        $(function() {
            $('.selectpicker').selectpicker({size: 8, style: 'btn-default'});

            function rows(ele = null, url = null, append = false, closure = null) {
                ele = ele ?? $('#load-currencies');
                url = url ?? window.location.href;

                ele.addClass('load');
                $.ajax({
                    url: url,
                    type: "GET",
                    success: function(response, textStatus, jqXHR) {append ? ele.append(response) : ele.empty().append(response);},
                    error: function(jqXHR) { handleErrors(jqXHR); },
                    complete: function () { ele.removeClass('load'); if (closure) closure(); }
                });
            } // AJAX CODE TO LOAD THE DATA TABLE

            rows();
            rows($('#load-logs'), "{{ route('currencies.load.logs') }}");

            $('body').on('click', '.reload-table', function(e) {
                e.preventDefault();
                rows($( $(this).data('target') ), $(this).attr('href'));
            });

            $('body').on('click', '.show-modal-form', function(e) {
                e.preventDefault();
                let btn   = $(this),
                    modal = $('#load-form');

                modal.modal('show').find('.modal-content').empty();
                $.ajax({
                    url: $(this).attr('href'),
                    type: 'GET',
                    beforeSend: function (jqHXR) { startAjax(); },
                    success: function (response, textStatus, jqXHR) {
                        modal.find('.modal-content').append(response);
                        modal.find('.selectpicker').selectpicker('refresh');
                    },
                    error: function(jqXHR) { handleErrors(jqXHR); },
                });
            });

            $('body').on('click', '#load-more-logs', function(e) {
                e.preventDefault();
                let url = $(this).attr('href');
                $(this).closest('tr').remove();
                rows($('#load-logs'), url, true, function() {
                    $('#load-logs').find('tr').each(function(key, val) {
                        if ($(this).find('td').length > 1) $(this).find('td:first').text(key + 1);
                    });
                });
            });

            $('body').on('submit', '#submit-form', function(e) {
                e.preventDefault();
                let modal = $('#load-form'),
                    form  = $(this);

                modal.find('.modal-content').addClass('load');
                $.ajax({
                    url: form.attr('action'),
                    type: form.attr('method'),
                    data: new FormData(form[0]),
                    dataType: 'JSON',
                    processData: false,
                    contentType: false,
                    beforeSend: function (jqHXR) { startAjax(); },
                    success: function (response, textStatus, jqXHR) {
                        modal.modal('hide').find('.modal-content').empty();
                        successMessage(response.message);
                        rows();
                        rows($('#load-logs'), "{{ route('currencies.load.logs') }}");
                    },
                    error: function(jqXHR) { handleErrors(jqXHR, form); },
                    complete: function (jqXHR, textStatus) { modal.find('.modal-content').removeClass('load'); }
                });
            });

            $('body').on('submit', '.delete-row', function(e) {
                e.preventDefault();
                if (!confirm("@lang('currency::trans.title.confirm-delete')")) return ;
                let form  = $(this);

                form.closest('td').addClass('load');
                $.ajax({
                    url: form.attr('action'),
                    type: form.attr('method'),
                    data: form.serialize(),
                    beforeSend: function (jqHXR) { startAjax(); },
                    success: function (response, textStatus, jqXHR) {
                        form.closest('tr').remove();
                        successMessage(response.message);
                        rows($('#load-logs'), "{{ route('currencies.load.logs') }}");
                    },
                    error: function(jqXHR) { handleErrors(jqXHR); },
                    complete: function (jqXHR, textStatus) { form.closest('td').removeClass('load');  }
                });
            });

            function handleErrors(jqXHR, form = null) {
                if (jqXHR.readyState == 0) return true;

                if ([401,401,402,403,404].includes(jqXHR.status))
                    errorMessage(jqXHR.responseJSON.message);

                else if (jqXHR.status == 422) { // List Validation Error
                    $.each(jqXHR.responseJSON.errors, function (key, val) {
                        val = Array.isArray(val) ? val[0] : val;
                        form.find(`#${key.replaceAll('.', '-')}_error`).text(val).fadeIn(300);
                    });
                } else if (typeof jqXHR.responseJSON !== 'undefined' && typeof jqXHR.responseJSON.line !== 'undefined') {
                    errorMessage('File: ' + jqXHR.responseJSON.file + ' (Line: ' + jqXHR.responseJSON.line + ')');
                } else {
                    errorMessage(jqXHR.responseJSON || jqXHR.statusText);
                }
            }

            function startAjax() {
                $('#show-alerts').empty();
                $('.validation-errors').empty();
            }

            function errorMessage(message) {
                $('#show-alerts').empty().append(`<div class="alert alert-danger" role="alert">${message}</div>`);
            }

            function successMessage(message) {
                $('#show-alerts').empty().append(`<div class="alert alert-success" role="alert">${message}</div>`);
            }

            $('body').on('click', 'a.sync-exchange-rate', function(e) {
                e.preventDefault();
                let btn = $(this);
                $.ajax({
                    url: btn.attr('href'),
                    type: "GET",
                    beforeSend: function (jqHXR) { btn.parent().addClass('load'); },
                    success: function(response) {
                        successMessage(response.message);
                        response.value
                            ? btn.closest('tr').find('span.exchange-rate-value').text(response.value)
                            : rows();
                        rows($('#load-logs'), "{{ route('currencies.load.logs') }}");
                    },
                    error: function(jqXHR) { handleErrors(jqXHR); },
                    complete: function (jqXHR, textStatus) { btn.parent().removeClass('load');  }
                });
            });

            let exchange_rate_ajax = {abort: function() {}};
            $('body').on('change', 'select#name', function() {
                exchange_rate_ajax.abort();
                let code = $(this).find('option:selected').data('code');
                $('#submit-form').find('input#code').val(code);
                exchange_rate_ajax = $.ajax({
                    url: "{{ route('currencies.exchange.rate') }}",
                    type: "GET",
                    data: {code: code},
                    success: function(response) {
                        $('#submit-form').find('input[name="value"]#value').val(response.value);
                    },
                    error: function(jqXHR) { handleErrors(jqXHR); $('#submit-form').find('input[name="value"]#value').val(0); },
                });
            });
        });
    </script>
@endsection
