<?php

namespace Modules\Currency\Services;

use Exception;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;
use Modules\Currency\Entities\Currency;
use Modules\Currency\Exceptions\ExchangeRateException;

class ExchangeRate
{
    private const API_URL = 'https://api.exchangerate.host/convert?from=%s&to=%s';

    /**
     * @inheritdoc
     * @throws ExchangeRateException
     */
    public static function convert(string $from, ?string $to = null): float
    {
        $result = 0;
        try {
            $response = Http::get( sprintf(self::API_URL, $from, self::getTo($to)) );
            $array = $response->json();

            if(!$array['success'] ?? false){
                self::logError($response);
                $result = 0;
                // throw new ExchangeRateException("Could not convert from $from to $to");
            }

            $result = (float) $array['result'];
        } catch (Exception $e) { $result = 0; }

        return $result;
    }

    private static function getTo(?string $to = null): string
    {
        return $to ?? Currency::select('code')->where('is_main', true)->first()->code ?? Currency::DEFAULT_ROW['code'];
    }

    private static function logError($response)
    {
        $data = $response->json();
        Log::error("Error in API Response of Exchange Rate Conversion", [
            'class' => __CLASS__,
            'from' => $data->query['from'],
            'to' => $data->query['to'],
            'response' => $response->body(),
            'status' => $response->status(),
        ]);
    }
}
