<?php

namespace Modules\Dairy\Entities;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;

class ClientTransaction extends Model
{
    protected $table = 'client_transaction';

    protected $fillable = ['bill_id', 'amount', 'pay_day', 'bill_net_total', 'notes', 'client_id', 'balance', 'safe_point_id', 'safe_balance', 'shop_id', 'user_id', 'date_time'];

    public $timestamps = false;

    public function getDateTimeAttribute($value)
    {
        return Carbon::parse($value)->format('Y-m-d');
    }

    public function client()
    {
        return $this->belongsTo(Client::class)->select('id', 'client_name')->withDefault(['client_name' => '']);
    }

    public function user()
    {
        return $this->belongsTo(User::class, 'user_id')->select('id', 'name')->withDefault(['name' => '']);
    }

    public function invoice()
    {
        return $this->belongsTo(Invoice::class, 'bill_id', 'id')->select('id', 'bill_total', 'payment', 'rest', 'net_price', 'store_id', 'invoice_date')->with('details');
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('shopID', function (Builder $builder) {
            $builder->where(app(__CLASS__)->getTable().'.shop_id', auth()->user()->shop_id)->orderBy('id', 'DESC');
        });

        static::creating(function($model) {
            $model->shop_id   = auth()->user()->shop_id;
            $model->user_id   = auth()->id();
            $model->date_time = day_now(get_session_val('date_type', 'Integer'));
        });
    }
}
