<?php

namespace Modules\Dairy\Entities;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Factories\HasFactory;

class Invoice extends Model
{
    use HasFactory;

    protected $table = 'invoices_';

    protected $guarded = ['id'];

    public $timestamps = false;   
    
    public function getDateTimeAttribute($value)
    {
        return Carbon::parse($this->invoices->invoice_date)->format('Y-m-d');
    }

    public function client()
    {
        return $this->belongsTo(Client::class, 'client_id', 'id')->select('id', 'client_name', 'price', 'balance');
    }

    public function store()
    {
        return $this->belongsTo(Store::class, 'store_id', 'id')->select('id', 'store_name');
    }

    public function details()
    {
        return $this->hasOne(InvoiceDetail::class, 'invoice_id', 'id')->select('id', 'quantity', 'invoice_id', 'sale_price', 'water', 'water_discount', 'protein', 'protein_bonus', 'fats', 'fats_bonus')->latest('id');
    }

    public function scopeSelectQuery(Builder $builder)
    {
        return $builder->select('id', 'invoice_date', 'client_id', 'store_id', 'shop_id', 'bill_total as total', 'order_no as receipt_no', 'fee', 'net_price');
    }

    public function scopeFilter(Builder $builder)
    {
        return $builder->when(request()->get('store'), fn ($query) => $query->where('store_id', request()->get('store')))
            ->when(request()->get('order_no'), fn($query) => $query->where('invoices_.order_no', request()->get('order_no')))
            ->when(request()->get('client'), fn ($query) => $query->where('invoices_.client_id', request()->get('client')))
            ->when(request()->get('user'), fn ($query) => $query->where('invoices_.add_user', request()->get('user')))
            ->when(request()->get('from'), fn ($query) => $query->whereDate('invoices_.invoice_date', '>=', request()->get('from')))
            ->when(request()->get('to'), fn ($query) => $query->whereDate('invoices_.invoice_date', '<=', request()->get('to')));
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('shopID', function (Builder $builder) {
            $builder->where(app(__CLASS__)->getTable().'.shop_id', auth()->user()->shop_id)->orderBy('id', 'DESC');
        });

        static::creating(function ($model) {
            $model->transaction_date =  now();
            $model->shop_id  = auth()->user()->shop_id;
            $model->add_user = auth()->id();
            $model->bill_no  = self::query()->max('bill_no') + 1;
            $model->invoice_total_disc      = 0;
            $model->invoice_total_disc_type = 1;
            $model->invoice_type = $model->invoice_type ?? 0;
            $model->invoice_total_disc_status = get_session_val('company_ratio', 'integer') == 2 ? 0 : 1;
            $model->insatllments_add = 0;
            $model->supplier_invoice_no = null;
            $model->reference_id = 0;
            $model->fee_status = true;
            $model->bill_service_id = 0;
            $model->bill_service_value  = 1;
            $model->delivery_option_id  = 0;
            $model->delivery_period_id  = 0;
            $model->delivery_man_id = 0;
            $model->currency_id = null;
            $model->currency_value = null;
        });
    }
}
