<?php

namespace Modules\Dairy\Entities;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Factories\HasFactory;

class Request extends Model
{
    use HasFactory;

    protected $table = 'requests';
    protected $fillable = ['fee', 'is_confirmed', 'total', 'net', 'bill_id', 'payment', 'rest', 'payment_type', 'notes'];

    public function client()
    {
        return $this->belongsTo(Client::class, 'client_id', 'id')->select('id', 'client_name', 'price', 'balance');
    }

    public function store()
    {
        return $this->belongsTo(Store::class, 'store_id', 'id')->select('id', 'store_name');
    }

    public function details()
    {
        return $this->hasOne(RequestDetail::class, 'request_id', 'id')->select('id', 'item_id', 'request_id', 'quantity', 'sale_price', 'protein', 'protein_bonus', 'water', 'water_discount', 'fats', 'fats_bonus')->latest('id');
    }

    public function scopeSelectQuery(Builder $builder)
    {
        return $builder->select('id', 'invoice_date', 'client_id', 'store_id', 'shop_id', 'total', 'net as net_price', 'bill_id', 'is_confirmed', 'no_bill as receipt_no', 'fee');
    }

    public function scopeFilter(Builder $builder)
    {
        return $builder->when(request()->get('store'), fn ($query) => $query->where('requests.store_id', request()->get('store')))
                        ->when(request()->get('client'), fn ($query) => $query->where('requests.client_id', request()->get('client')))
                        ->when(request()->get('order_no'), fn($query) => $query->where('requests.no_bill', request()->get('order_no')))
                        ->when(request()->get('from'), fn ($query) => $query->where('requests.invoice_date', '>=', request()->get('from')))
                        ->when(request()->get('to'), fn ($query) => $query->where('requests.invoice_date', '<=', request()->get('to')));
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('shopID', function (Builder $builder) {
            $table = app(__CLASS__)->getTable();
            $builder->where("$table.shop_id", auth()->user()->shop_id)->where("$table.bill_id", '=', 0)->orderBy('id', 'DESC');
        });

        static::updating(function($model) {
            $model->payment_type = $model->rest > 0;
        });
    }
}
