<?php

namespace Modules\Dairy\Http\Controllers;

use Illuminate\Routing\Controller;
use Modules\Dairy\Entities\Client;
use Modules\Dairy\Entities\Request as RequestModel;
use Modules\Dairy\Entities\Store;
use Modules\Dairy\Http\Requests\RequestEditRequest;
use Modules\Dairy\Http\Services\RequestService;

class RequestController extends Controller
{
    public function index()
    {
        if (request()->ajax()) {
            $rows = RequestModel::selectQuery()->filter()->with('client', 'store', 'details')->paginate(5);
            return view('dairy::includes.rows', compact('rows'));
        }

        return view('dairy::requests.index', [
            'clients' => Client::select('id', 'client_name')->whereIn('client_role', [0, 2])->pluck('client_name', 'id'),
            'stores'  => Store::select('id', 'store_name')->pluck('store_name', 'id'),
            'rows'    => collect([]),
            'title'   => "طلبات الشراء",
        ]);
    }

    public function edit(RequestModel $request)
    {
        if ($request->is_confirmed) return response()->json(['message' => 'هذا الطلب تم تاكيده بالفعل'], 500);
        $request->load('details', 'client');
        return view('dairy::requests.form', ['row' => $request]);
    }
    public function update(RequestEditRequest $request, $id)
    {
        $requestModel = RequestModel::find($id);
        if ($requestModel->is_confirmed) return response()->json(['message' => 'هذا الطلب تم تاكيده بالفعل'], 500);
        return RequestService::update($requestModel, $request->validated());
    }

    public function confirm($id)
    {
        $request = RequestModel::find($id);
        abort_if($request->is_confirmed, 500, 'هذا الطلب تم تاكيده بالفعل');
        return RequestService::confirm($request);
    }
}
