<?php

namespace Modules\Dairy\Http\Controllers;

use Illuminate\Routing\Controller;
use Modules\Dairy\Entities\{Client, Invoice, InvoiceDetail, Item, Store};
use Modules\Dairy\Http\Requests\SalesInvoiceRequest;
use Modules\Dairy\Http\Services\InvoiceService;

class SalesInvoiceController extends Controller
{
    public function index()
    {
        if (request()->ajax()) {
            $rows = Invoice::selectQuery()->filter()->whereHas('client')->with('client', 'store', 'details')->where('invoice_type', 1)->paginate(5);
            return view('dairy::includes.rows', compact('rows'));
        }

        return view('dairy::invoices.index', [
            'clients' => Client::select('id', 'client_name')->whereIn('client_role', [0, 2])->pluck('client_name', 'id'),
            'stores'  => Store::select('id', 'store_name')->pluck('store_name', 'id'),
            'rows'    => collect([]),
            'title'   => "فواتير المبيعات",
        ]);
    }

    public function create()
    {
        $title   = "إنشاء فاتورة مبيعات";
        $items   = Item::select('id', 'item_name')->pluck('item_name', 'id');
        $clients = Client::select('id', 'client_name', 'price')->whereIn('client_role', [0, 2])->get();
        $stores  = Store::select('id', 'store_name')->pluck('store_name', 'id');
        $row     = new Invoice();
        $row->details = new InvoiceDetail();
        return view('dairy::invoices.create', compact('title', 'clients', 'stores', 'items', 'row'));
    }

    public function store(SalesInvoiceRequest $request, InvoiceService $service)
    {
        $response = $service->handler($request->validated(), 1);
        if ($response['status'] != 200) {
            return back()->with('error', $response['message'])->withInput($request->validated());
        }
        return redirect()->route('dairy.sales.invoices.index')->with('success', $response['message']);
    }
}
