<?php

namespace Modules\Dairy\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;

class RequestEditRequest extends FormRequest
{
    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
        return [
            'quantity' => 'required|numeric',
            'water'    => 'required|numeric',
            'protein'  => 'required|numeric|max:3.5|min:0',
            'fats'     => 'required|numeric|max:4.1|min:0',
            'fee'      => 'required|numeric|min:0',
            'payment'  => 'required|numeric|min:0',
            'notes'    => 'nullable|string|max:300',
            'confirm'  => 'nullable',
        ];
    }

    public function attributes()
    {
        return [
            'quantity' => trans('dairy.qty'),
            'water'    => trans('dairy.per_water'),
            'protein'  => trans('dairy.per_fats'),
            'fats'     => trans('dairy.per_protein'),
            'fee'      => trans('dairy.trans_cost'),
            'payment'  => trans('dairy.payment'),
            'notes'    => trans('dairy.notes'),
        ];
    }

    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }
}
