<?php

namespace Modules\Dairy\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;

class SalesInvoiceRequest extends FormRequest
{
    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
        return [
            'client_id'  => 'required|exists:clients,id,shop_id,'.auth()->user()->shop_id,
            'store_id'   => 'required|exists:stores,id,shop_id,'.auth()->user()->shop_id,
            'item_id'    => 'required|exists:items,id,shop_id,'.auth()->user()->shop_id,
            'quantity'   => 'required|numeric|min:0',
            'water'      => 'required|numeric|min:0|max:100',
            'protein'    => 'required|numeric|min:0|max:3.5',
            'fats'       => 'required|numeric|min:0|max:4.1',
            'fee'        => 'required|numeric|min:0',
            'date'       => 'required|date',
            'order_no'   => 'required|numeric',
            'payment'    => 'required|numeric|min:0',
            'notes'      => 'nullable|string',
        ];
    }

    public function attributes()
    {
        return [
            'client_id'  => trans('dairy.supplier-name'),
            'store_id'   => trans('dairy.store_id'),
            'item_id'    => trans('items.name'),
            'quantity'   => trans('dairy.quantity'),
            'water'      => trans('dairy.per_water'),
            'protein'    => trans('dairy.per_protein'),
            'fats'       => trans('dairy.per_fats'),
            'fee'        => trans('dairy.trans_cost'),
            'order_no'   => trans('dairy.receipt_no'),
            'date'       => trans('dairy.date'),
            'payment'    => trans('dairy.payment'),
            'rest'       => trans('dairy.rest'),
            'notes'      => trans('dairy.notes'),
        ];
    }

    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }
}
