<?php

namespace Modules\Dairy\Http\Services;

class Calculator
{
    public float $quantity = 0;
    public float $price = 0;
    public float $total = 0;
    public float $net_price = 0;
    public float $payment = 0;
    public float $rest = 0;
    public float $static_net_price = 0;
    public float $water = 0;
    public float $protein = 0;
    public float $fats = 0;
    public float $water_discount = 0;
    public float $net_quantity = 0;
    public float $protein_bonus = 0;
    public float $fats_bonus = 0;
    public float $fee = 0;

    public function calculate()
    {
        $this->calculateWaterDiscount();
        $this->calculateNetQuantity();
        $this->calculateNetPrice();
        $this->calculateProteinBonus();
        $this->calculateFatsBonus();
        $this->calculateTotalPrice();
        $this->calculateEndPrice();
        $this->calculateRest();
    }

    public function setQuantity(float $quantity): Calculator
    {
        $this->quantity = $quantity;
        return $this;
    }

    public function setWater(float $water): Calculator
    {
        $this->water = $water;
        return $this;
    }

    public function setProtein(float $protein): Calculator
    {
        $this->protein = $protein;
        return $this;
    }

    public function setFats(float $fats): Calculator
    {
        $this->fats = $fats;
        return $this;
    }

    public function setPrice($price): Calculator
    {
        $this->price = $price;
        return $this;
    }

    public function setFee(?float $fee = 0): Calculator
    {
        $this->fee = $fee;
        return $this;
    }

    public function setPayment(?float $payment = 0): Calculator
    {
        $this->payment = $payment;
        return $this;
    }

    public function calculateWaterDiscount(): Calculator
    {
        $this->water_discount = $this->quantity * ($this->water / 100);
        return $this;
    }

    public function calculateNetQuantity(): Calculator
    {
        $this->net_quantity = $this->quantity - $this->water_discount;
        return $this;
    }

    public function calculateNetPrice(): Calculator
    {
        $this->static_net_price = $this->net_quantity * $this->price;
        return $this;
    }

    public function calculateProteinBonus(): Calculator
    {
        $this->protein_bonus = $this->calculateBonus($this->protein, 3.20, ['min' => 3.00, 'max' => 3.10], ['main' => 0.03, 'sub' => 0.015]);
        return $this;
    }

    public function calculateFatsBonus(): Calculator
    {
        $this->fats_bonus = $this->calculateBonus($this->fats, 3.80, ['min' => 3.50, 'max' => 3.70], ['main' => 0.05, 'sub' => 0.025]);
        return $this;
    }

    public function calculateTotalPrice(): Calculator
    {
        $this->total = $this->fats_bonus + $this->protein_bonus + $this->static_net_price;
        return $this;
    }

    public function calculateEndPrice(): Calculator
    {
        $this->net_price = $this->total + $this->fee;
        return $this;
    }

    public function calculateRest(): Calculator
    {
        $this->rest = $this->net_price - $this->payment;
        return $this;
    }

    protected function calculateBonus(float $percentage, float $start, array $zeroRange, array $bonus_values): float
    {
        $percentage = number_format($percentage, 2);
        $bonus = $result = 0;
        $last_number = substr((string) $percentage, -1);
        $percentage -= $last_number * .01;
        $percentage = number_format($percentage, 1);
        $operator   = null;

        if ( $percentage >= $zeroRange['min'] && $percentage < $start ) {
            $bonus = 0;
        } else if ( $percentage >= $start ) {
            $result = ($percentage - $zeroRange['max']) * 10;
            $operator   = '+';
        } else if ($percentage < 3.5 && $percentage >= 2.5) {
            $result = ($percentage - $zeroRange['min']) * 10;
            $operator   = '-';
        }

        $bonus = ($result * $bonus_values['main']);
        $this->calculateSubBonus($bonus, $last_number, $bonus_values, $operator);
        return number_format($bonus, 3) * $this->static_net_price;
    }

    protected function calculateSubBonus(float &$bonus, int $last_number, array $bonus_values, $operator): void
    {
        if (is_null($operator)) return;
        $add = 0;
        if ($last_number > 0 && $last_number <= 5) {
            $add = $bonus_values['sub'];
        } elseif ($last_number > 5 && $last_number <= 9) {
            $add = $bonus_values['sub'] * 2;
        }

        $bonus = eval("return $bonus $operator $add;");
    }
}
