<?php

namespace Modules\Dairy\Http\Services;

use Exception;
use Illuminate\Support\Facades\DB;
use Modules\Dairy\Entities\Client;
use Modules\Dairy\Entities\Request;

class RequestService
{
    public static function update(Request $request, array $data)
    {
        try {
            DB::beginTransaction();
                $data['price'] = Client::select('price')->where('id', $request->client_id)->first()->price;
                $calculator = self::setCalculator($data);

                $request->update([
                    'fee'     => $calculator->fee,
                    'total'   => $calculator->total,
                    'net'     => $calculator->net_price,
                    'payment' => $calculator->payment,
                    'rest'    => $calculator->rest,
                    'notes'   => $data['notes'],
                ]);

                $request->details()->update([
                    'quantity'       => $calculator->quantity,
                    'price'          => $calculator->static_net_price,
                    'sale_price'     => $calculator->price,
                    'water'          => $calculator->water,
                    'water_discount' => $calculator->water_discount,
                    'protein'        => $calculator->protein,
                    'protein_bonus'  => $calculator->protein_bonus,
                    'fats'           => $calculator->fats,
                    'fats_bonus'     => $calculator->fats_bonus
                ]);
            DB::commit();

            if ( isset($data['confirm']) && $data['confirm'] == 1 ) {
                self::confirm($request);
            }
            session()->flash('success', 'تم تعديل الطلب بنجاح');
            return response()->json([], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage()], 500);
        }
    }

    protected static function setCalculator(array $data)
    {
        $calculator = new Calculator();
        $calculator->setQuantity($data['quantity'])->setWater($data['water'])
                            ->setFats($data['fats'])
                            ->setProtein($data['protein'])
                            ->setPrice($data['price'])
                            ->setFee($data['fee'] ?? 0)
                            ->setPayment($data['payment'] ?? 0)
                            ->calculate();
        return $calculator;
    }

    public static function confirm(Request $request)
    {
        try {
            DB::beginTransaction();
                $request->update(['is_confirmed' => true]);
                $request->load('details', 'client');
                $response = ( new InvoiceService() )->handler( self::setInvoiceData($request), 0);

                if ($response['status'] != 200) {
                    throw new Exception($response['message']);
                }

                $request->update(['bill_id' => $response['invoice_id']]);
            DB::commit();
            session()->flash('success', $response['message']);
            return response()->json(['reload' => true], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage()], 500);
        }
    }

    protected static function setInvoiceData(Request $request)
    {
        return [
            'client_id' => $request->client_id,
            'store_id'  => $request->store_id,
            'order_no'  => $request->no_bill,
            'fee'       => $request->fee ?? 0,
            'notes'     => $request->notes ?? '',
            'price'     => $request->details->sale_price,
            'payment'   => $request->payment ?? 0,
            'item_id'   => $request->details->item_id,
            'quantity'  => $request->details->quantity,
            'water'     => $request->details->water,
            'protein'   => $request->details->protein,
            'fats'      => $request->details->fats,
        ];
    }
}
