<?php

namespace Modules\Debentures\Entities;

use App\Traits\RoundingAttributesTrait;
use Illuminate\Database\Eloquent\Model;
use Modules\BadrRequest\Entities\BadrShop;
use Modules\Invoices\Entities\Invoices;

class ClientTransaction extends Model
{
    use RoundingAttributesTrait;
    protected $fillable = [
        'shop_id',
        'bill_id',
        'client_id',
        'date_time',
        'amount_collected',
        'amount',
        'type',
        'effect',
        'pay_day',
        'balance',
        'safe_balance',
        'safe_point_id',
        'notes',
        'safe_type',
        'spend_id',
        'user_id',
        'supplier_id',
        'collector_id',
        'unique_columns',
    ];
    protected $table = 'client_transaction';
    public $timestamps = false;

    public function collector()
    {
        return $this->belongsTo(User::class, 'collector_id', 'id')->select('id', 'name', 'collecting_ratio')->where(['collecting' => true, 'is_employee' => true]);
    }

    public function client()
    {
        return $this->belongsTo(Client::class, 'client_id')->withDefault(['client_tax_number' => null, 'client_name' => 'mm', 'mobile1' => null, 'email' => null, 'address' => null]);
    }

    public function shop()
    {
        return $this->belongsTo(BadrShop::class, 'shop_id', 'serial_id');
    }

    public function User()
    {
        return $this->belongsTo(User::class, 'user_id');
    }

    public function invoice()
    {
        return $this->belongsTo(Invoices::class, 'bill_id', 'id');
    }
    public function supplier()
    {
        return $this->belongsTo(Supplier::class,'supplier_id','id');
    }

    public static function boot()
    {
        parent::boot();

        self::creating(function ($model) {
            $model->shop_id = auth()->user()->shop_id;
            $model->user_id = auth()->user()->id;
            $model->unique_columns = "{$model->client_id}-{$model->amount}-{$model->date_time}-{$model->shop_id}";
        });
    }
}
