<?php

namespace Modules\Debentures\Http\Controllers;

use Illuminate\Database\Eloquent\Collection;
use Illuminate\Routing\Controller;
use Modules\Debentures\Entities\Client;
use Modules\Debentures\Entities\ClientTransaction;
use Modules\Debentures\Entities\SalePoint;
use Modules\Invoices\Http\Controllers\ShareInvoiceController;

class DebentureReportController extends Controller
{
    public function addetions()
    {
        if (request()->ajax()) {
            $rows = ClientTransaction::with('invoice:id,bill_no', 'invoice.vats:vat_value,invoice_id', 'client')
                            ->whereHas('client', fn($q) => $q->ClientGroup())
                            ->whereHas('invoice')->where('shop_id', auth()->user()->shop_id)->where('type', 2)
                            ->when(request()->has('client_id') && request()->get('client_id') > 0, function($query) {
                                $query->where('client_id', request()->get('client_id'));
                            })
                            ->when(request()->get('from'), function($query) {
                                $query->whereDate('date_time', '>=', request()->get('from'));
                            })
                            ->when(request()->get('to'), function($query) {
                                $query->whereDate('date_time', '<=', request()->get('to'));
                            })->orderBy('id', 'DESC')->get();

            return view('debentures::reports.additions.rows', compact('rows'));
        }
        $rows = new Collection();
        $clients = Client::select('id', 'client_name')->where('shop_id', get_session_val('serial_id', 'integer'))->whereIn('client_role', [0, 2])
                        ->ClientGroup()
                        ->pluck('client_name', 'id');
        return view('debentures::reports.additions.index', compact('rows', 'clients'));
    }

    public function print($id) {
        $data = $this->getData($id);
        return view($this->getView($data['row']), $data);
    }

    public function show($hash_id)
    {
        return $this->print( decrypt($hash_id) );
    }

    protected function getData($id)
    {
        $row = ClientTransaction::select('id', 'pay_day', 'bill_id', 'type', 'user_id', 'client_id', 'safe_type', 'shop_id', 'collector_id', 'amount', 'notes', 'pay_day', 'date_time as date','balance')
                                    ->when(auth()->check(), fn($query) => $query->where('shop_id', auth()->user()->shop_id))->find($id);

        $row->store_id = SalePoint::select('store_id')->find($row->safe_type)->store_id ?? null;

        $data = ShareInvoiceController::getInvoice(encrypt($row->bill_id), false, $row);

        abort_if(! $data, 500);
        $data['invoice']->load('vats');
        return array_merge($data, ['row' => $row]);
    }

    protected function getView($row)
    {
        $type = $row->type == 9 ? 'payment' : 'cash';
        $type = request()->get('type', $type);
        $version = 2;

        if (auth()->check()) {
            $version = auth()->user()->debentures_print_template == 1 ? 2 : 1;
        }

        $version = request()->get('v', $version);
        $view = "debentures::print.$type.v{$version}.print";

		return $view;
    }
}
