<?php

namespace Modules\Debentures\Http\Controllers;

use App\Http\Controllers\Controller;
use Carbon\Carbon;
use DB;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Modules\Debentures\Entities\ClientTransaction;
use Modules\Debentures\Entities\IncomeBill;
use Modules\Debentures\Entities\Log;
use Modules\Debentures\Entities\SalePoint;
use Modules\Debentures\Entities\Supplier;

class supplierReport extends Controller
{
    public function index(Request $request)
    {
        // dd($request);
        $suppliers = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        $bills = DB::table('incoming_bill')
                    ->leftjoin('supplier', 'incoming_bill.supplier_id', 'supplier.id')
                    ->when($request->supplier_id != null && $request->supplier_id != -1, function($q) use($request){
                        $q->where('incoming_bill.supplier_id', $request->supplier_id);
                    })
                    ->when(($request->bill_id != null), function($q) use($request){
                        $q->where('incoming_bill.id', $request->bill_id);
                    })
                    ->when(($request->dateFrom != null) , function($q) use($request){
                        $q->where('incoming_bill.incoming_date', '>=', $request->dateFrom);
                    })
                    ->when(($request->dateTo != null), function($q) use($request){
                        $q->where('incoming_bill.incoming_date', '<=', $request->dateTo);
                    })
                    ->where([ 
                        'incoming_bill.shop_id' => get_session_val('serial_id', 'integer'),
                        'incoming_bill.pay_stat'=> 0
                    ])
                    ->select(['incoming_bill.id', 'incoming_bill.incoming_date', 'incoming_bill.net_price', 'incoming_bill.payment', 'incoming_bill.supplier_id', 'supplier.supplier_name'])
                    ->orderBy('incoming_bill.id', 'DESC')
                    ->paginate(15);

        $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        return view("debentures::later_suppliers_bills_report", compact('suppliers', 'bills', 'sale_points'));
    }

    public function suplierReceiptPayment(Request $request)
    {

        // dd($request);

        $x = \DB::transaction(function () use ($request) {
            $supplier = Supplier::findOrFail($request->supplier);
            $old_balance = $supplier->balance;
            $supplier->balance = $supplier->balance - $request->payment;
            $supplier->save();
            // dd($supplier);
            $bill = IncomeBill::where([
                'shop_id' => \auth()->user()->shop_id,
                'id' => $request->bill_id,
                'pay_stat' => 0,
                'supplier_id' => $supplier->id,
                'incoming_date' => $request->bill_date
            ])->first();

            $bill_no = $request->bill_id;
            Log::create(Carbon::now(), 'تعديل  الرصيد علي المورد    ' . $supplier->supplier_name  .  ' من خلال تسجيل ايصال دفع   ' .
                ' من فرق تعديل الصافي والمدفوع [ ( ' . price_decimal($bill->net_price) . ' - ' . price_decimal($bill->payment) . ' ) => (' . price_decimal($bill->net_price) . ' - ' . price_decimal($bill->payment + $request->payment)  . ') ]' .
                ' وتعديل  رصيد الخزنة من فرق المدفوع '  . ' [ '. price_decimal($bill->payment) . ' => ' . price_decimal($bill->payment + $request->payment) .
                ' ] من فاتورة مشتريات '  . $bill->id .  ' بتاريخ  ' . Carbon::now() . ' ' . date('H:i:s'), $bill->id, 'incomes');
            $bill->payment += $request->payment;
            // $bill->bil_payment += $request->payment;
            $bill->save();
            if ($bill->payment >= $bill->net_price) {
                $bill->pay_stat = 1;
                $bill->save();
            }
            // dd($bill);
            $sale_point = SalePoint::findOrFail($request->sale_point);
            $sale_point->money_point = $sale_point->money_point - $request->payment;
            $sale_point->save();

            // dd($sale_point);
            $transaction = new ClientTransaction();
            $transaction->date_time = Carbon::now();
            $transaction->supplier_id = $request->supplier;
            $transaction->amount = $request->payment;
            $transaction->type = 6; //دفع
            $transaction->effect = 0;   // مصروف
            $transaction->bill_id = $bill->id;
            $transaction->pay_day = $request->date_day;
            $transaction->balance = $old_balance - $request->payment;
            $transaction->notes = 'ايصال دفع خاص بفاتوره رقم '.$bill_no;
            $transaction->safe_type = $request->sale_point;
            $transaction->safe_balance = $sale_point->money_point;
            $transaction->safe_point_id = $request->sale_point;
            $transaction->user_id = Auth::user()->id;
            $transaction->shop_id = get_session_val('serial_id', 'integer');
            $transaction->save();
            // dd($transaction);
            return true;
        });
        if ($x) {
            return back()->with(['success' =>
            trans('native.correct_save')]);
        } else {
            return back()->with(['error' => 'حدث خطأ اثناء الحفظ']);
        }
    }
}