@extends('master::layouts.master')

@section('content')

    <div class="container" style="margin: 10px auto">
        <div class="row">
            <div class="col-md-12 well">
                <h1 class="text-center title-page">تقرير الفواتير الاجلة</h1>
                <form>
                    <input type="hidden" name="report">
                    <div class="form-group col-md-3">
                        <input readonly type="text" class="form-control date" name="date" placeholder="من"
                               value="{{ request('date') }}">
                    </div>
                    <div class="form-group col-md-3">
                        <input readonly type="text" class="form-control date" name="to" placeholder="الى"
                               value="{{ request('to') }}">
                    </div>
                    <div class="form-group col-md-3">
                        <input type="text" class="form-control" name="bill_no" placeholder="رقم الفاتوره"
                               value="{{ request('bill_no') }}">
                    </div>
                    <div class="form-group col-md-3">
                        <select class="form-control selectpicker" data-live-search="true" name="client_id">
                            <option value="0">كل العملاء</option>
                            @foreach($clients as $client)
                                <option @if (request('client_id') == $client->id) selected @endif value="{{ $client->id }}">{{ $client->client_name }}</option>
                            @endforeach
                        </select>
                    </div>
                    <div class="form-group col-md-3">
                        <button type="submit" class="btn btn-primary"><i class="fa fa-search"></i>بحث</button>
                    </div>
                </form>
                <table class="table table-hover table-stripped" role="presentation">
                    <thead class="bg-info">
                    <tr>
                        <th scope="col">رقم الفاتوره</th>
                        <th scope="col">التاريخ</th>
                        <th scope="col">العميل</th>
                        <th scope="col">صافى الفاتوره</th>
                        <th scope="col">المدفوع</th>
                        <th scope="col">الباقى</th>
                        <th scope="col"></th>
                    </tr>
                    </thead>
                    <tbody>
                        @forelse($bills as $bill)
                            <tr>
                                <td>{{ $bill->bill_no }}</td>
                                <td>{{ $bill->sale_date }}</td>
                                <td>{{ $bill->getClient() }}</td>
                                <td>{{ price_decimal($bill->net_price) }}</td>
                                <td>{{ price_decimal($bill->payment) }}</td>
                                <td>{{ price_decimal($bill->net_price - $bill->payment) }}</td>
                                <td>
                                    <a href="#" data-bill_no="{{ $bill->bill_no }}" data-sale_date="{{ $bill->sale_date }}"
                                       data-payment="{{ price_decimal($bill->net_price - $bill->payment) }}" data-client="{{ $bill->client_id }}"
                                       data-toggle="modal" data-target="#myModal" class="btn btn-primary pay"><i class="fa fa-money"></i>سداد</a>
                                </td>
                            </tr>
                        @empty
                            <tr>
                                <td class="danger text-center" colspan="7">لا يوجد فواتير</td>
                            </tr>
                        @endforelse
                    </tbody>
                    <tfoot>
                        <tr>
                            <td colspan="3" class="text-center">{{ trans('native.total') }}</td>
                            <td>{{ price_decimal($bills->sum('net_price')) }}</td>
                            <td>{{ price_decimal($bills->sum('payment')) }}</td>
                            <td>{{ price_decimal($bills->sum('net_price') - $bills->sum('payment')) }}</td>
                            <td></td>
                        </tr>
                    </tfoot>
                </table>
            </div>
        </div>
    </div>

    <div class="text-center">
        {!! $bills->appends(Request::capture()->except('page'))->render() !!}
    </div>



    <div id="myModal" class="modal fade" role="dialog">
        <div class="modal-dialog">
            <form action="{{ route('debentures.receipt-payment') }}" method="post">
                {{ csrf_field() }}
                <input type="hidden" name="client" id="model_client" value="">
                <input type="hidden" name="payment" id="model_payment" value="">
                <input type="hidden" name="bill_no" id="model_bill_no" value="">
                <input type="hidden" name="bill_date" id="model_bill_date" value="">
                <!-- Modal content-->
                <div class="modal-content">
                    <div class="modal-header">
                        <button type="button" class="close" data-dismiss="modal">&times;</button>
                        <h4 class="modal-title">تسجيل سند قبض</h4>
                    </div>
                    <div class="modal-body">
                        <label>التاريخ : </label>
                        <input readonly type="text" class="form-control date" name="date_day" value=" {{ date('Y-m-d') }}">
                        <label>الخزينة : </label>
                        <select name="sale_point" class="form-control">
                            @foreach($sale_points as $point)
                                <option value="{{ $point->id }}">{{ $point->point_name }}</option>
                            @endforeach
                        </select>
                    </div>
                    <div class="modal-footer">
                        <button type="button" class="btn btn-danger" data-dismiss="modal">الغاء</button>
                        <button type="submit" class="btn btn-primary">حفظ</button>
                    </div>
                </div>
            </form>
        </div>
    </div>
@endsection

@section('css-scripts')
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/bootstrap-datetimepicker-standalone.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/datepicker3.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/jquery.ptTimeSelect.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/jquery.timepicker.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/css/bootstrap-select.min.css') }}" rel="stylesheet">
@stop

@section('js-scripts')
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>
    <script src="{{ asset('tickets/jquery.timepicker.js') }}"></script>
    <script src="{{ asset('tickets/datepair.js') }}"></script>
    <script src="{{ asset('tickets/js/bootstrap-select.min.js') }}"></script>
    <script src="{{ asset('js/redirect.js') }}"></script>
    <script>
        $(function () {
            $('.selectpicker').selectpicker({
                style: 'btn-default',
                size: 8
            });

            if ('{{ get_session_val('date_type', 'Integer') }}' ==  '2') {
                $(".date").calendarsPicker({
                    calendar: $.calendars.instance('gregorian'),
                    dateFormat: 'Y-mm-dd',
                    // defaultDate: '0',
                    // selectDefaultDate: true,
                });
            } else {
                $(".date").calendarsPicker({
                    calendar: $.calendars.instance('ummalqura'),
                    dateFormat: 'Y-mm-dd',
                    // defaultDate: '0',
                    // selectDefaultDate: true,
                });
            }
        });

        $('.pay').click(function () {
            let el = $(this).closest('tr'),
                bill_no = el.find('.pay').data('bill_no'),
                sale_date = el.find('.pay').data('sale_date'),
                payment = el.find('.pay').data('payment'),
                client = el.find('.pay').data('client');
            $(document).find('#model_client').val(client);
            $(document).find('#model_payment').val(payment);
            $(document).find('#model_bill_no').val(bill_no);
            $(document).find('#model_bill_date').val(sale_date);
        });
    </script>
@stop
