<?php

namespace Modules\Delivery\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Delivery\Entities\{
    DeliveryMan,
    Invoices
};
use Illuminate\Validation\ValidationException;

class DeliveryManController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index(): Renderable
    {
        $delivery_men = DeliveryMan::where('shop_id', get_session_val('serial_id', 'Integer'))->get();
        return view('delivery::delivery_man.index', compact('delivery_men'));
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string',
            'phone' => 'required|numeric|unique:delivery_man,phone',
        ]);

        DeliveryMan::create([
            'name' => $request->name,
            'phone' => $request->phone,
            'shop_id' => get_session_val('serial_id', 'Integer'),
        ]);
    }

    /**
     * update the specified resource in storage.
     * @param Request $request
     */
    public function update(Request $request)
    {
        $request->validate([
            'name' => 'required|string',
            'phone' => 'required|numeric|unique:delivery_man,phone,' . $request->id,
        ]);

        DeliveryMan::where([
            'id' => $request->id,
            'shop_id' => get_session_val('serial_id', 'Integer')
        ])->update([
            'name' => $request->name,
            'phone' => $request->phone,
        ]);
    }

    /**
     * Remove the specified resource from storage.
     * @param Request $request
     * @return void
     * @throws ValidationException
     */
    public function destroy(Request $request): void
    {
        $invoices_count = Invoices::where([
            'shop_id' => get_session_val('serial_id', 'Integer'),
            'delivery_man_id' => $request->id
        ])->count();

        if ($invoices_count > 0) {
            throw ValidationException::withMessages(['error' => __('delivery.it_is_not_possible_to_clear_because_of_previous_operations')]);
        }

        DeliveryMan::where([
            'id' => $request->id,
            'shop_id' => get_session_val('serial_id', 'Integer')
        ])->delete();
    }

    /**
     * render index table
     * @return string
     */
    public function getIndexTable(): string
    {
        $delivery_men = DeliveryMan::where('shop_id', get_session_val('serial_id', 'Integer'))->get();
        return view('delivery::delivery_man.partials.table', compact('delivery_men'))->render();
    }

    /**
     * render edit modal body
     * @param Request $request
     * @return string
     */
    public function getEditModalBody(Request $request): string
    {
        $delivery_man = DeliveryMan::where([
            'shop_id' => get_session_val('serial_id', 'Integer'),
            'id' => $request->id
        ])->select('id', 'name', 'phone')->first();
        return view('delivery::delivery_man.partials.edit_modal_body', compact('delivery_man'))->render();
    }
}
