<?php

namespace Modules\Delivery\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Validator;
use Modules\Delivery\Entities\Client;
use Modules\Delivery\Entities\ClientTransaction;
use Modules\Delivery\Entities\DeliveryMan;
use Modules\Delivery\Entities\Invoices;
use Modules\Delivery\Entities\Log;
use Modules\Delivery\Entities\SalePoint;

class DeliveryReportController extends Controller
{
    public function index(Request $request)
    {
        $clients = clientGroups()->whereIn('client_role', [0, 2])->where('is_table', 0)->orderBy('client_name')->get(['id', 'client_name']);
        $delivery_men = DeliveryMan::where('shop_id', get_session_val('serial_id', 'Integer'))
            ->select('id', 'name')
            ->get();

        $date = 'delivery_date';
        $pay_state_operator = '>';
        if ($request->has('done')) {
            $date = 'done_date';
            $pay_state_operator = '<=';
        }
        $bills = Invoices::query()
            ->where([
                'shop_id' => get_session_val('serial_id', 'Integer'),
                'invoice_type' => 1
            ])
            ->where('fee', '>', 0)
            ->when($request->has('done'), function ($q) use ($request) {
                $q->where('fee_status', 1);
            })
            ->when(!$request->has('done'), function ($q) use ($request) {
                $q->where('fee_status', 0);
            })
            ->when($request->has('done'), function ($query) {
                $query->where('delivery_man_id', '!=', '0');
            })
            ->when($request->filled('client_mobile'), function ($query) use ($request) {
                $query->whereHas('client', function ($q) use ($request) {
                    $q->where('mobile1', $request->client_mobile);
                });
            })
            ->when($request->filled('delivery_man_mobile'), function ($query) use ($request) {
                $query->whereHas('delivery', function ($q) use ($request) {
                    $q->where('phone', $request->delivery_man_mobile);
                });
            })
            ->when($request->filled('bill_no'), function ($bill_no) use ($request) {
                $bill_no->where('bill_no', $request->bill_no);
            })
            ->when($request->filled('from'), function ($from) use ($request, $date) {
                $from->whereDate($date, '>=', $request->from);
            })
            ->when($request->filled('to'), function ($to) use ($request, $date) {
                $to->whereDate($date, '<=', $request->to);
            })
            ->when(!in_array($request->client_id, ['-1', '', null]), function ($client_id) use ($request) {
                $client_id->where('client_id', $request->client_id);
            })
            ->when(!in_array($request->delivery_id, ['-1', '0', '', null]), function ($delivery_id) use ($request) {
                $delivery_id->where('delivery_man_id', $request->delivery_id);
            })
            ->select('id', 'net_price', 'delivery_date', 'done_date', 'bill_no', 'client_id', 'delivery_man_id')
            ->with('client', 'delivery');

        $total_bills_net_price = $bills->sum('net_price');
        $bills = $bills->orderBy('id', 'desc')->paginate();

        return view('delivery::delivery_report.index', compact('bills', 'delivery_men', 'clients', 'total_bills_net_price'));
    }


    public function confirmDelivery(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'bill_id' => 'required|numeric|exists:invoices_,id',
            'delivery_man_id' => 'required|numeric|exists:delivery_man,id'
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors();
            $data = '<ul class="list-unstyled text-center alert alert-danger">';
            foreach ($errors->all() as $error) {
                $data .= '<li>' . $error . '</li>';
            }
            $data .= '</ul>';
            $response = [
                'status' => false,
                'data' => $data,
            ];
            return response()->json($response);
        }
        $bill = Invoices::where(['shop_id' => get_session_val('serial_id', 'Integer'), 'id' => $request->bill_id])
                        ->select('id', 'net_price', 'payment', 'delivery_date', 'bill_no', 'client_id', 'delivery_man_id')
                        ->first();

        if ($bill->payment == $bill->net_price) {
            $bill->update(['fee_status' => true, 'done_date' => now(), 'delivery_date' => now(), 'delivery_man_id' => $request->delivery_man_id]);
            return response()->json(['status' => true, 'remove' => true]);
        }
        
        $bill->update(['delivery_date' => now(), 'delivery_man_id' => $request->delivery_man_id]);
        $bill->load('client', 'delivery');
        $html = '
            <tr class="alert-success" id="bill_id-' . $bill->id . '">
                <td>' . $bill->bill_no . '</td>
                <td>' . $bill->client->client_name . '</td>
                <td>' . $bill->net_price . '</td>
                <td>' . $bill->client->mobile1 . '</td>
                <td>' . $bill->client->address . '</td>
                <td>
                    : ' . __('delivery.delivery_man') . $bill->delivery->name . '<br>
                    : ' . __('delivery.deliver_date') . Carbon::parse($bill->delivery_date)->format('Y-m-d h:m:i a') . '
                </td>
                <td class="noprint">
                    <a href="#" class="btn btn-primary confirm_payment" data-bill_id="' . $bill->id . '">
                    ' . __('delivery.confirm_payment') . '
                    <i class="fa fa-money"></i></a>
                </td>
            </tr>
        ';
        return response()->json([
            'status' => true,
            'data' => $html,
        ]);
    }

    public function confirmPayment(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'bill_id' => 'required|numeric|exists:invoices_,id'
        ]);
        if ($validator->fails()) {
            $errors = $validator->errors();
            $data = '<ul class="list-unstyled text-center alert alert-danger">';
            foreach ($errors->all() as $error) {
                $data .= '<li>' . $error . '</li>';
            }
            $data .= '</ul>';
            $response = [
                'status' => false,
                'data' => $data,
            ];
            return response()->json($response);
        }

        $x = \DB::transaction(function () use ($request) {
            $bill = Invoices::where([
                'shop_id' => get_session_val('serial_id', 'Integer'),
                'id' => $request->bill_id
            ])->first();

            if ($bill->payment == $bill->net_price) {
                $bill->update(['fee_status' => true, 'done_date' => now()]);
                return true;
            }

            $bill->payment = $bill->net_price;
            $bill->bill_payment = $bill->net_price;
            $bill->rest = 0;
            $bill->fee_status = 1;
            $bill->done_date = Carbon::now();
            $bill->save();

            $client = Client::findOrFail($bill->client_id);
            $old_balance = $client->balance;
            $client->balance = $client->balance - $bill->payment;
            $client->save();
            $bill_no = $bill->bill_no;

            Log::create(Carbon::now(), 'تعديل  الرصيد علي العميل    ' . $client->client_name . ' من خلال تسجيل ايصال استلام   ' .
                ' من فرق تعديل الصافي والمدفوع [ ( ' . price_decimal($bill->net_price) . ' - ' . price_decimal($bill->payment) . ' ) => (' . price_decimal($bill->net_price) . ' - ' . price_decimal($bill->net_price) . ') ]' .
                ' وتعديل  رصيد الخزنة من فرق المدفوع ' . ' [ ' . price_decimal($bill->payment) . ' => ' . price_decimal($bill->payment) .
                ' ] من فاتورة مبيعات ' . $bill->id . '[ ' . $bill->bill_no . ' ] ' . ' بتاريخ  ' . Carbon::now() . ' ' . date('H:i:s'), $bill->id, 'sales');

            $sale_point_id = ClientTransaction::where([
                'bill_id' => $bill->id,
                'shop_id' => get_session_val('serial_id', 'Integer'),
            ])->value('safe_point_id');

            $sale_point = SalePoint::findOrFail($sale_point_id);
            $sale_point->money_point = $sale_point->money_point + $bill->net_price;
            $money = $sale_point->money_point;
            $sale_point->save();

            $transaction = new ClientTransaction();
            $transaction->date_time = Carbon::now();
            $transaction->client_id = $bill->client_id;
            $transaction->amount = $bill->net_price;
            $transaction->type = 2;
            $transaction->effect = 1;
            $transaction->bill_id = $bill->id;
            $transaction->pay_day = Carbon::now();
            $transaction->balance = $old_balance - $bill->net_price;
            $transaction->notes = 'ايصال استلام خاص بفاتوره رقم ' . $bill_no;
            $transaction->safe_type = $sale_point_id;
            $transaction->safe_balance = $money;
            $transaction->safe_point_id = $sale_point_id;
            $transaction->user_id = auth()->id();
            $transaction->shop_id = get_session_val('serial_id', 'Integer');
            $transaction->save();

            return true;
        });
        if ($x) {
            return response()->json([
                'status' => true,
                'data' => 'تم الحفظ بنجاح',
            ]);
        } else {
            return response()->json([
                'status' => true,
                'data' => 'حدث خطأ اثناء الحفظ',
            ]);
        }
    }
}
