<?php

namespace Modules\Delivery\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Validation\ValidationException;
use Modules\Delivery\Entities\Invoices;
use Modules\Delivery\Entities\Period;

class PeriodsController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index(): Renderable
    {
        $periods = Period::where('shop_id', get_session_val('serial_id', 'Integer'))->get();
        return view('delivery::period.index', compact('periods'));
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|unique:delivery_options,name',
            'fee' => 'required|numeric',
            'with_periods' => 'required'
        ]);

        Period::create([
            'name' => $request->name,
            'fee' => $request->fee,
            'with_periods' => $request->with_periods,
            'shop_id' => get_session_val('serial_id', 'Integer'),
        ]);
    }

    /**
     * update the specified resource in storage.
     * @param Request $request
     */
    public function update(Request $request)
    {
        $request->validate([
            'name' => 'required|string|unique:delivery_options,name,' . $request->id,
            'fee' => 'required|numeric',
            'with_periods' => 'required'
        ]);

        Period::where([
            'id' => $request->id,
            'shop_id' => get_session_val('serial_id', 'Integer')
        ])->update([
            'name' => $request->name,
            'fee' => $request->fee,
            'with_periods' => $request->with_periods,
        ]);
    }

    /**
     * Remove the specified resource from storage.
     * @param Request $request
     * @return void
     * @throws ValidationException
     */
    public function destroy(Request $request)
    {

        $invoices_count = Invoices::where([
            'shop_id' => get_session_val('serial_id', 'Integer'),
            'delivery_option_id' => $request->id
        ])->count();

        if ($invoices_count > 0) {
            throw ValidationException::withMessages(['error' => __('delivery.it_is_not_possible_to_clear_because_of_previous_operations')]);
        }

        Period::where([
            'id' => $request->id,
            'shop_id' => get_session_val('serial_id', 'Integer')
        ])->delete();
    }


    /**
     * render index table
     * @return string
     */
    public function getIndexTable(): string
    {
        $periods = Period::where('shop_id', get_session_val('serial_id', 'Integer'))->get();
        return view('delivery::period.partials.table', compact('periods'))->render();
    }

    /**
     * render edit modal body
     * @param Request $request
     * @return string
     */
    public function getEditModalBody(Request $request): string
    {
        $period = Period::where([
            'shop_id' => get_session_val('serial_id', 'Integer'),
            'id' => $request->id
        ])->first();
        return view('delivery::period.partials.edit_modal_body', compact('period'))->render();
    }
}
