<?php

namespace Modules\Delivery\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Validation\ValidationException;
use Modules\Delivery\Entities\Invoices;
use Modules\Delivery\Entities\Region;

class RegionController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index(): Renderable
    {
        $regions = Region::where('shop_id', get_session_val('serial_id', 'Integer'))->get();
        return view('delivery::region.index', compact('regions'));
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|unique:cities,city_name',
            'fee' => 'required|numeric',
        ]);

        Region::create([
            'city_name' => $request->name,
            'fee' => $request->fee,
            'shop_id' => get_session_val('serial_id', 'Integer'),
        ]);
    }

    /**
     * update the specified resource in storage.
     * @param Request $request
     */
    public function update(Request $request)
    {
        $request->validate([
            'name' => 'required|string|unique:cities,city_name,' . $request->id,
            'fee' => 'required|numeric'
        ]);

        Region::where([
            'id' => $request->id,
            'shop_id' => get_session_val('serial_id', 'Integer')
        ])->update([
            'city_name' => $request->name,
            'fee' => $request->fee,
        ]);
    }

    /**
     * Remove the specified resource from storage.
     * @param Request $request
     * @return void
     * @throws ValidationException
     */
    public function destroy(Request $request)
    {

        $invoices_count = Invoices::where([
            'shop_id' => get_session_val('serial_id', 'Integer'),
            'city_id' => $request->id
        ])->count();

        if ($invoices_count > 0) {
            throw ValidationException::withMessages(['error' => __('delivery.it_is_not_possible_to_clear_because_of_previous_operations')]);
        }

        Region::where([
            'id' => $request->id,
            'shop_id' => get_session_val('serial_id', 'Integer')
        ])->delete();
    }


    /**
     * render index table
     * @return string
     */
    public function getIndexTable(): string
    {
        $regions = Region::where('shop_id', get_session_val('serial_id', 'Integer'))->get();
        return view('delivery::region.partials.table', compact('regions'))->render();
    }

    /**
     * render edit modal body
     * @param Request $request
     * @return string
     */
    public function getEditModalBody(Request $request): string
    {
        $region = Region::where([
            'shop_id' => get_session_val('serial_id', 'Integer'),
            'id' => $request->id
        ])->select('id', 'city_name', 'fee')->first();
        return view('delivery::region.partials.edit_modal_body', compact('region'))->render();
    }
}
