<?php

namespace Modules\Delivery\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Validation\ValidationException;
use Modules\Delivery\Entities\BillService;
use Modules\Delivery\Entities\Invoices;

class ServicesController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index(): Renderable
    {
        $services = BillService::where('shop_id', get_session_val('serial_id', 'Integer'))->get();
        return view('delivery::service.index', compact('services'));
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|unique:bill_services,name',
            'fee' => 'required|numeric',
            'use_periods' => 'required'
        ]);

        BillService::create([
            'name' => $request->name,
            'price' => $request->fee,
            'enabled' => $request->use_periods,
            'shop_id' => get_session_val('serial_id', 'Integer'),
        ]);
    }

    /**
     * update the specified resource in storage.
     * @param Request $request
     */
    public function update(Request $request)
    {
        $request->validate([
            'name' => 'required|string|unique:bill_services,name,' . $request->id,
            'fee' => 'required|numeric',
            'use_periods' => 'required'
        ]);

        BillService::where([
            'id' => $request->id,
            'shop_id' => get_session_val('serial_id', 'Integer')
        ])->update([
            'name' => $request->name,
            'price' => $request->fee,
            'enabled' => $request->use_periods,
        ]);
    }

    /**
     * Remove the specified resource from storage.
     * @param Request $request
     * @return void
     * @throws ValidationException
     */
    public function destroy(Request $request)
    {

        $invoices_count = Invoices::where([
            'shop_id' => get_session_val('serial_id', 'Integer'),
            'bill_service_id' => $request->id
        ])->count();

        if ($invoices_count > 0) {
            throw ValidationException::withMessages(['error' => __('delivery.it_is_not_possible_to_clear_because_of_previous_operations')]);
        }

        BillService::where([
            'id' => $request->id,
            'shop_id' => get_session_val('serial_id', 'Integer')
        ])->delete();
    }


    /**
     * render index table
     * @return string
     */
    public function getIndexTable(): string
    {
        $services = BillService::where('shop_id', get_session_val('serial_id', 'Integer'))->get();
        return view('delivery::service.partials.table', compact('services'))->render();
    }

    /**
     * render edit modal body
     * @param Request $request
     * @return string
     */
    public function getEditModalBody(Request $request): string
    {
        $service = BillService::where([
            'shop_id' => get_session_val('serial_id', 'Integer'),
            'id' => $request->id
        ])->select('id', 'name', 'price', 'enabled')->first();
        return view('delivery::service.partials.edit_modal_body', compact('service'))->render();
    }
}
