@extends('master::layouts.master')

@section('css-scripts')
    <style>
        .load {
            cursor: wait;
            position: relative;
        }

        .load::before {
            font-family: "Font Awesome 5 Free";
            content: "\f021";
            position: absolute;
            left: 50%;
            top: 50%;
            transform: translate(-50%, -50%);
            font-size: 19px;
            font-weight: bolder;
            color: #000;
            z-index: 1029;
            animation: load .9s linear infinite;
        }

        .load::after {
            content: "";
            display: block;
            position: absolute;
            top: 0;
            left: 0;
            bottom: 0;
            right: 0;
            background-color: rgb(255, 255, 255);
            opacity: 0.8;
            z-index: 1028;
            height: 100%;
            transition: all .5s ease-in-out;
        }

        @keyframes load {
            from {
                transform: rotate(0deg);
            }
            to {
                transform: rotate(360deg);
            }
        }

    </style>
@endsection

@section('content')
<div class=" card card-light mb-1_5 p-3">
        <div class="container-fulid">
    <!-- start alert -->
    <div id="alert_container"></div>
    <!-- end alert -->

    <h2 class="text-info  d-inline-block ">{{ __('delivery.delivery_times') }}</h2>

    <!-- start add modal -->
    @include('delivery::period.partials.add_modal')
    <!-- end add modal -->

    <!-- start edit modal -->
    @include('delivery::period.partials.edit_modal')
    <!-- end edit modal -->

    <!-- start table -->
    <div id="period_table">
        @include('delivery::period.partials.table', $periods)
    </div>
    <!-- end table -->

    </div>
    </div>
@endsection

@section('js-scripts')
    <script>
        function add() {
            $('.modal-content').addClass('load');

            let data = {
                name: $('#name').val(),
                fee: $('#fee').val(),
                with_periods: $('#with_periods').val()
            }

            $.ajax({
                url: "{{ route('periods.store') }}",
                type: "POST",
                headers: {
                    'X-CSRF-TOKEN': '{{ csrf_token() }}'
                },
                data: data,
                success: function () {
                    $('.modal-content').removeClass('load');
                    $('#add').modal('hide');
                    Reloadtable();
                    restForm();
                    resetErrors();
                },
                error: function (data) {
                    resetErrors();
                    $('.modal-content').removeClass('load');
                    $.each(data.responseJSON.errors, function (key, value) {
                        $('#' + key).addClass('is-invalid');
                        $('#' + key + '-error').html(value);
                    });
                }
            });
        }

        function edit() {
            $('.modal-content').addClass('load');

            let data = {
                id: $('#region_id').val(),
                name: $('#edit-name').val(),
                fee: $('#edit-fee').val(),
                with_periods: $('#edit_with_periods').val()
            }

            $.ajax({
                url: "{{ route('periods.update') }}",
                type: "POST",
                headers: {
                    'X-CSRF-TOKEN': '{{ csrf_token() }}'
                },
                data: data,
                success: function () {
                    $('.modal-content').removeClass('load');
                    $('#edit').modal('hide');
                    Reloadtable();
                    restForm();
                    resetErrors();
                },
                error: function (data) {
                    resetErrors();
                    $('.modal-content').removeClass('load');
                    $.each(data.responseJSON.errors, function (key, value) {
                        $('#edit-' + key).addClass('is-invalid');
                        $('#edit-' + key + '-error').html(value);
                    });
                }
            });
        }


        $('#edit').on('show.bs.modal', function (event) {
            $('.modal-content').addClass('load');
            let id = $(event.relatedTarget).data('id');

            $.ajax({
                type: "POST",
                url: "{{ route('periods.edit-modal-body') }}",
                headers: {
                    'X-CSRF-TOKEN': '{{ csrf_token() }}'
                },
                data: {
                    id
                },
                success: function (response) {
                    $('.modal-content').removeClass('load');
                    $('#edit_modal_body').html(response);
                }
            });
        })


        function destroy(id) {

            Swal.fire({
                title: '{{ __('delivery.are_you_sure') }}',
                text: '{{ __('delivery.you_will_not_be_able_to_undo_the_deletion') }}',
                icon: 'warning',
                showCancelButton: true,
                confirmButtonColor: '#d33',
                cancelButtonColor: '#3085d6',
                confirmButtonText: 'نعم, حذف!',
                cancelButtonText: 'تراجع'
            }).then((result) => {
                if (result.isConfirmed) {
                    $.ajax({
                        type: "POST",
                        url: "{{ route('periods.destroy') }}",
                        data: {
                            id
                        },
                        headers: {
                            'X-CSRF-TOKEN': '{{ csrf_token() }}'
                        },
                        success: function () {
                            Reloadtable();
                        },
                        error: function (data) {
                            $.each(data.responseJSON.errors, function (key, value) {
                                $('#alert_container').html(`
                                    <div class="alert alert-danger alert-dismissible fade show" role="alert">
                                        <strong>${value}</strong>
                                        <button type="button" class="close" data-dismiss="alert" aria-label="Close">
                                            <span aria-hidden="true">&times;</span>
                                        </button>
                                    </div>
                                `);
                            });
                        }
                    });
                }
            })

        }


        function restForm() {
            $('#name, #edit-name, #fee, #edit-fee, #edit_use_periods').val('');
        }

        function resetErrors() {
          $('#name, #edit-name, #fee, #edit-fee, #edit_use_periods').removeClass('is-invalid');
        }


        function Reloadtable() {
            $('.table-responsive').addClass('load');

            $.ajax({
                type: "GET",
                url: "{{ route('periods.table') }}",
                headers: {
                    'X-CSRF-TOKEN': '{{ csrf_token() }}'
                },
                success: function (data) {
                    $('#period_table').html(data);
                    $('.table-responsive').removeClass('load');
                }
            });
        }


        $('#add, #edit').on('hidden.bs.modal', function () {
            restForm();
        });


        $(document).on('change', '#edit_with_periods', function() {
          let value = $(this).val();
          $(document).find('#edit_with_periods').val(value);
        })

    </script>
@endsection
