<?php

namespace Modules\ECommerce\Http\Controllers;

use App\Events\EmailBillEvent;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Event;
use Modules\ECommerce\Entities\BadrRequest;
use Modules\ECommerce\Entities\BadrShop;
use Modules\ECommerce\Entities\BillAdd;
use Modules\ECommerce\Entities\Card;
use Modules\ECommerce\Entities\Client;
use Modules\ECommerce\Entities\ClientNotification;
use Modules\ECommerce\Entities\Color;
use Modules\ECommerce\Entities\Company;
use Modules\ECommerce\Entities\Item;
use Modules\ECommerce\Entities\RequestAddHistory;
use Modules\ECommerce\Entities\RequestDetails;
use Modules\ECommerce\Entities\SaleDetails;
use Modules\ECommerce\Entities\SalePoint;
use Modules\ECommerce\Entities\SaleProcess;
use Modules\ECommerce\Entities\Size;
use Modules\ECommerce\Entities\Store;
use Modules\ECommerce\Entities\StoreItem;
use Modules\ECommerce\Entities\RequestSettings;
use Illuminate\Support\Facades\DB;
use Modules\Category\Entities\ItemUnit;
use Modules\ECommerce\Entities\BillAdd as BillsAdd;
use Modules\Invoices\Entities\Invoices;

class BadrRequestController extends Controller
{



    public function index(Request $request)
    {
        $type = get_session_val('date_type', 'Integer');

        $rows = BadrRequest::where(function ($q) use ($request) {
            if ($request->filled('client_name')) {
                $q->whereHas('client', function ($query) use ($request) {
                    $query->where('client_name', 'like', '%' . $request->client_name . '%');
                });
            }

            if ($request->filled('date')) {
                $q->whereDate('created_at', $request->date);
            }

            if ($request->filled('number')) {
                $q->where('order_no', $request->number);
            }

            if ($request->filled('request_status')) {
                $q->where('status', $request->request_status);
            }

        })->where('shop_id', get_session_val('serial_id', 'integer'))->orderBy('order_no', 'desc')->paginate(10);

        $shop = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
        foreach ($rows->chunk(50) as $rows_ch) {
            foreach ($rows_ch as $row) {
                // $b = SaleProcess::find($row->no_bill);
                $b = Invoices::where([
                    ['shop_id', get_session_val('serial_id', 'Integer')],
                    ['invoice_type', '1'],
                    ['id', $row->no_bill]
                ])->first();
                $row->no_bill = $b->bill_no ?? '';
                $row->sale_process_id = $b->id ?? '';
                $row->payment_option = $row->getPaymentOption();
            }
        }

        return view('ecommerce::requests', compact('rows', 'shop', 'type'));
    }

    public function checkChargingItems($items, $shop_id)
    {

        $ret = false;
        $arr = [];
        foreach ($items as $item) {

            $it = Item::where([
                'shop_id' => $shop_id,
                'id' => $item->item_id
            ])->select('card_company_id')->first();

            if ($it) {

                $ch = $it->card_company_id;
                if (in_array($ch, ['0', null])) {
                    $arr[] = '1';
                }
            }
        }
        if (sizeof($arr) > 0) {
            $ret = true;
        }
        return $ret;
    }

    public function requestDetails(Request $request, $id)
    {

        $shop = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();

        $setting = RequestSettings::where('shop_id', get_session_val('serial_id', 'integer'))->first();
        if (is_null($setting)) {
            $setting = new RequestSettings();
            $setting->fee = 0;
            $setting->min_purchase = 0;
            $setting->max_charge = 0;
            $setting->shop_id = get_session_val('serial_id', 'integer');
            $setting->save();
        }

        $req_ = BadrRequest::findOrFail($id);

        $coupon = DB::table('request_coupons')->where([
            'shop_id' => get_session_val('serial_id', 'integer'),
            'id' => $req_->coupon_id
        ])->first();

        $req_id = $req_->id;

        $mandobeen = DB::table('users')->where('shop_id', get_session_val('serial_id', 'integer'))->where('type', 1)->get();

        $req = Invoices::where([
            ['shop_id', get_session_val('serial_id', 'Integer')],
            ['invoice_type', '1'],
            ['id', $req_->no_bill]
        ])->first();
 
//        if (is_null($req)) {
            $adds = RequestAddHistory::where('request_id', $req_->id)->sum('addition_value');

            $rows = RequestDetails::where('request_details.shop_id', get_session_val('serial_id', 'integer'))
                ->where('request_details.request_id', $id)
                ->join('items', 'items.id', 'request_details.item_id')
                ->leftJoin('bills_add', 'items.vat_id', 'bills_add.id')
                ->select('request_details.*', 'items.item_name', 'items.vat_id', 'items.vat_state', 'bills_add.addition_value')

                ->get();

            $req_->adds = $adds;

            foreach ($rows as $row) {

                $row->cards = [];

                $item = Item::find($row->items_id ?? $row->item_id);
                $isCard = $this->cardCheck($item);
                $row->isCard = $isCard;
                $price = $row->price;
                $price_original = $price;

                $t = 'تطبق الضريبة';
                if ($item->vat_state == 2) {
                    $v = DB::table('bills_add')
                        ->where('shop_id', get_session_val('serial_id', 'integer'))
                        ->where('id', $item->vat_id)->first();
                    $vat = '';
                    if ($v) {
                        $price_original = $price - $v->addition_value;
                        if ($v->check_addition == 0) {
                            $price_original = $price / (1 + $v->addition_value / 100);
                        }
                        $price_original = price_decimal($price_original, get_session_val('serial_id', 'integer'));

                        $currency = get_session_val('currency', 'string');
                        $vat .= $v->Addition_name . ' ';
                        $vat .= price_decimal($v->addition_value, get_session_val('serial_id', 'integer')) . ' ';
                        $vat .= ($v->check_addition == '0') ? '%' : $currency;
//                . ' ' . $v->addition_value . ' ' . ($v->check_addition == '0') ? '%' : $currency;
                    }
//            return $vat;
                    // $t = ' المنتج شامل الضريبة - ' . $vat;
                    $t = $vat;
                }
                if ($item->vat_state == 0) {
                    $t = 'بدون ضريبة';
                }
                if ($item->vat_state == '1') {
                    $ads = BillsAdd::where('shop_id', get_session_val('serial_id', 'integer'))->select('Addition_value', 'check_addition')->get();

                    $all_vat = 0;
                    foreach ($ads as $ad) {
                        $p = $row->price;
                        $type = $ad->check_addition;
                        if ($type == '0') {
                            $all_vat += ($ad->Addition_value / 100) * $p;
                        } else {
                            $all_vat += $ad->Addition_value;
                        }
                    }
                    $row->additions = $all_vat;
                } else {
                    $row->additions = 0;
                }

                $row->vat_txt = $t;

                $row->o_price = $price_original;

                $vat = 0.00;

                $vs_query_ = DB::table('bills_add')
                    ->where('shop_id', get_session_val('serial_id', 'integer'))
                    ->where('check_addition', 0);
                $vs = $vs_query_->get();
                if ($item->vat_state == 2) {
                    $v_ = $vs_query_->where('id', $item->vat_id)->first();
                    if ($v_) {
                        $basic_price = $item->sale_price / (1 + ($v_->addition_value / 100));
                    }
                }
                if ($item->vat_state != '0') {
                    foreach ($vs as $vc) {
                        $type = $vc->check_addition;
                        $val = $vc->addition_value;
                        if ($type) {
                            $vat += $val;
                        } else {
                            $vat += $row->price * $val / 100;
                        }
                    }
                }

                $row->vats = $vat;

                $discount = $row->o_price - $row->sale_price;

                $row->discount = $discount;
            }

            $req = $req_;

            $req->bill = $req_->no_bill;
            $req->order_no = $req_->order_no;
            $req->status_display = $req_->status_display;
            $req->status = $req_->status;
            $req->req_id = $req_->id;

            $sale_point_id = auth()->user()->sale_point;

            if (Auth::user()->point_change == 0) {
                $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                    ->where('id', auth()->user()->sale_point)
                    ->get();
            } else {
                $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))->get();
            }

            if (Auth::user()->change_store == 0) {
                $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))
                    ->where('id', auth()->user()->store_id)
                    ->get();
            } else {
                $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get();
            }
            if ($request->has('store_id')) {
                $store_id = $request->store_id;
            } else {
                $store_id = auth()->user()->store_id;

                if (is_null($store_id)) {
                    $temp = $stores->first();
                    $store_id = $temp->id ?? 0;
                }
            }

            $chargeAdd = $this->checkChargingItems($rows, get_session_val('serial_id', 'integer'));

            foreach ($rows as $item) {
                $unit_id = $item->unit_id;
                $item_id = $item->item_id;
            

            $unit__ = DB::table('units')->where('id', $unit_id)->first();

            $item__ = DB::table('items')->where('id', $item_id)->first();


            if ($item__) {
                if ($item__->unit_id == $item->unit_id) {
                    $item->unit_eq = 1;
                    $item->unit_div = 0;
                    $item->the_unit = $item->unit_id;
                } else {

                    $tem_unit = ItemUnit::where('shop_id', get_session_val('serial_id', 'integer'))->where('item_id', $item__->id)->first();
                    if ($tem_unit) {
                        $item->unit_eq = $tem_unit->unit_value;
                        $item->unit_div = $tem_unit->unit_type;
                        $item->the_unit = $tem_unit->unit_id;
                    } else {
                        $item->unit_eq = 1;
                        $item->unit_div = 0;
                        $item->the_unit = $item->unit_id;
                    }
                }
            } else {
                $item->unit_eq = 1;
                $item->unit_div = 0;
                $item->the_unit = 0;
            }


            if ($unit__) {
                $name_ = $unit__->name;
            } else {
                $name_ = '';
            }
            $item->unit_name = $name_;

            if ($item__) {
                $i_name_ = $item__->item_name;
                $item->min_qty = $item__->min_quantity;

                if ($request->has('store_id')) {
                    $s_id = $request->store_id;

                    $cardCheck = $this->cardCheck($item__);

                    if ($cardCheck != '0') {
                        $cards_query = Card::where([
                            'shop_id' => get_session_val('serial_id', 'integer'),
                            'item_id' => $item__->id,
                            'request_id' => 0,
                            'sale_id' => 0,
                            'card_state' => 0
                        ]);
                        $cards_count = $cards_query->count();
                        $item->store_qty = $cards_count;
                    } else {
                        if (in_array($s_id, [null, 0, 'x'])) {
                            $item->store_qty = $item__->quantity;
                        } else {
                            $s_item = StoreItem::where('item_id', $item__->id)
                                ->where('shop_id', get_session_val('serial_id', 'integer'))
                                ->where('store_id', $s_id)
                                ->first();

                            if (is_null($s_item)) {
                                $item->store_qty = 0;
                            } else {
                                $item->store_qty = $s_item->store_quant;
                            }
                        }
                    }

                } else {
                    $item->store_qty = 0;
                    $item->min_qty = 0;

                }

            } else {
                $i_name_ = '';
                $item->store_qty = 0;
            }

            $item->item_name = $i_name_;

            $color = Color::find($item->color_id);
            $size = Size::find($item->size_id);

            if ($color) {
                $item->color = $color->color_name;
            } else {
                $item->color = '';
            }

            if ($size) {
                $item->size = $size->size_name;
            } else {
                $item->size = '';
            }
			}

            $req->pay_type = $req_->getPaymentOption();
            if ($req_->payment_type == '2') {
                $req->image = config('main.badr') . 'upload/trans_files/' . $req_->trans_img;
                // $req->image = 'http://pos.cablshop.com/' . 'upload/trans_files/' . $req_->trans_img;
            }


            $money_vats = DB::table('bills_add')
                ->where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('check_addition', 1)->sum('addition_value');

            return view('ecommerce::request', compact('coupon', 'money_vats', 'req_id', 'chargeAdd', 'setting', 'adds', 'mandobeen', 'req', 'rows', 'sale_points', 'stores', 'sale_point_id', 'store_id', 'shop'));
    }

    private function updateItemCards($item, $qty)
    {
        $cards = $this->getNewCards($item, $qty);

        if ($cards && sizeof($cards) == $qty) {
            return $cards;
        }
        return false;
    }

    private function checkAvailability($order_id)
    {
        // return true;

        $details = RequestDetails::where([
            'shop_id' => get_session_val('serial_id', 'integer'),
            'request_id' => $order_id
        ]);


        $available_items = [];
        $available_cards = [];

        $items_r = $details->select('quantity', 'item_id')->get();

        $items_r->chunk(10)->each(function ($itenms_ch) use (&$available_items, $order_id, &$available_cards) {
            $itenms_ch->each(function ($it) use (&$available_items, $order_id, &$available_cards) {
                $item = Item::find($it->item_id);

                if ($item) {
                    $available_items[] = '1';
                    $is_com = $this->cardCheck($item);

                    if ($is_com != '0') {
                        $required_qty = $it->quantity;

                        $cards_query = Card::where([
                            'shop_id' => $item->shop_id,
                            'item_id' => $it->item_id,
                            'request_id' => $order_id
                        ]);

                        $saved_qty = $cards_query->count();

                        if ($required_qty > $saved_qty) {

                            $neededQty = $required_qty - $saved_qty;

                            if ($neededQty > 0) {
                                $updateItemCardsIds = $this->updateItemCards($item, $neededQty);
                                if ($updateItemCardsIds) {
                                    $this->updateCards($updateItemCardsIds, '0', $order_id);
                                } else {
                                    $available_cards[] = '1';
                                }
                            }
                        } 
                    }
                }
            });
        });

        if (sizeof($available_items) > 0 && sizeof($available_cards) == 0) {

            return true;
        }
        return false;
    }

    public function confirmRequest(Request $request, $id)
    {


/*
        `
        "cash" => "1"
          "store_id" => "x"
          "sale_point" => "1"
          "outgoing" => array:3 [▼
            999 => "1.00"
            1000 => "1.00"
            1001 => "1.00"
          ]
          "totalall" => "247.19"
          "discount" => "0.00"
          "net" => "255.95"
          "fee" => "7.68"
          "final" => "263.63"
        `;
*/

        if ($request->totalall <= 0) {
            return back()->with([
                'error' => 'لا يمكن عمل فاتورة بدون مبلغ'
            ]);
        }
        $check_qty = $this->checkAvailability($id);
        if ($check_qty) {



            $sale_point = SalePoint::find($request->sale_point);
            if (!$sale_point){
             return  back()->with(['error'=>'برجاء التأكد من وجود خزينة دفع']);

            }
//		return $request;
            $tr = DB::transaction(function () use ($id, $request, $sale_point) {

//		return Carbon::today()->format("Y-m-d");

                $order = BadrRequest::findOrFail($id);
                $net = $request->net;
                $final = $order->net;
                $total = $request->totalall;
                $discount = $request->discount;
                $fee = $order->fee;
                $rows = RequestDetails::where('shop_id', get_session_val('serial_id', 'integer'))->where('request_id', $id)->get();

//		return $row
//s;

                $bill_no_ = Invoices::where([
                    ['shop_id', get_session_val('serial_id', 'Integer')],
                    ['invoice_type', '1'],
                ])->orderBy('id', 'DESC')->first();

                if ($bill_no_) {
                    $bill_no_m = $bill_no_->max('bill_no');
                    $bill_no = $bill_no_m + 1;
                } else {
                    $bill_no = 1;
                }

                $store_id = $request->store_id;

                $client = Client::find($order->client_id);
                if ($client) {
                    $balance = $client->balance;
                } else {
                    $balance = 0;
                }
                if ($request->cash == 1) {
                    //cash
                    $payment = $final;
                    $rest = 0;
                } else {
                    //debit
                    $payment = 0;
                    $rest = $final;
                }
                $n_balance = $balance - $rest;

                if (in_array($request->store_id, [0, 'x'])) {
                    $store_id = 0;
                }


                $sale_process = DB::table('invoices_')->insertGetId([
                    'date_process' => Carbon::now(),
                    'sale_date' => Carbon::now(),
                    'store' => $store_id,
                    'total_price' => $total,
                    'discount' => $discount,
                    'discount_type' => 1, // 0 ---> % or  precent  1---> value
                    'net_price' => $final,
                    'payment' => $payment,
                    'bil_payment' => $payment,
                    'rest' => $rest,  //باقى
                    'client_id' => $order->client_id,
                    'clientOldBalance' => $balance,  // رصيد العميل القديم
                    'pay_stat' => $request->cash, // حاله الدفع
                    'balance' => $n_balance, // رصيد العميل الجديد
                    'pay_date' => Carbon::today(),
                    'add_user' => auth()->user()->id,
                    'shop_id' => get_session_val('serial_id', 'integer'),
                    'bill_no' => $bill_no,
                    'notes' => '',
                    'StoreTransport' => 0,
                    'StoreServices' => 0,
                    'fee' => $fee,
                    'invoice_type' => 1
                ]);

                $sale_process_data = Invoices::where([
                    ['shop_id', get_session_val('serial_id', 'Integer')],
                    ['invoice_type', '1']
                ])->findOrFail($sale_process);

                $order->no_bill = $sale_process_data->id;
                $order->save();

                foreach ($rows as $row) {

                    $item = Item::findOrFail($row->item_id);

                    $store_item = StoreItem::where('item_id', $row->item_id)
                        ->where('shop_id', get_session_val('serial_id', 'integer'))
                        ->where('store_id', $request->store_id)->first();

                    if (in_array($store_id, [0, 'x'])) {
                        $item_quantity = $item->quantity;
                    } else {

                        if ($store_item) {
                            $item_quantity = $store_item->store_quant;
                        } else {
                            $item_quantity = 0;
                        }

                    }

                    // `check for the exact qty to reduce`;

                    $outgoing_ = $request->outgoing[$row->id];
                    $u_id = $row->unit_id;



                    if ($item->unit_id == $u_id) {
                        $outgoing = $outgoing_;
                    } else {
                        $xx = ItemUnit::where('item_id', $item->id)
                            ->where('unit_id', $u_id)
                            ->where('shop_id', get_session_val('serial_id', 'integer'))
                            ->first();

                        if ($xx) {
                            $type = $xx->unit_type;
                            if ($type == 0) {
                                $outgoing = $outgoing_ * $xx->unit_value;
                            } else {
                                $outgoing = $outgoing_ / $xx->unit_value;
                            }
                        } else {
                            $outgoing = $outgoing_;
                        }
                    }
                    $cards_ids = $this->getCards($item, $outgoing_, $id);

                    if ($cards_ids != '0') {
                        //    $cards = serialize($cards_ids);
                        $this->updateCards($cards_ids, $sale_process, $id);
                    }/* else {
                        $cards = null;
                    }*/

                    DB::table('invoices_details')->insert([
                        'invoice_id' => $sale_process,
                        'item_id' => $row->items_id,
                        'shop_id' => $order->shop_id,
                        'quantity' => $outgoing_,
                        'unit_id' => $row->unit_id,
                        'quantity_after' => $item_quantity - $outgoing,
                        'pay_price' => $item->pay_price,
                        'sale_price' => $row->price,
                        'total_price' => $outgoing * $row->price,
                        'basic_sale_price' => $item->sale_price,
                        'basic_pay_price' => $item->pay_price,
                        'item_about' => '',
                        'not_sold' => 0,
                        'discount' => $order->discount / $outgoing_,
                        'discount_type' => 1,
                        'pay_vats_weight' => 0,
                        'sale_ratio' => 1,
                        'bolla' => 0,
                        'safe_point_id' => $request->sale_point,
                        'store_id' => $store_id,
                        'add_user' => auth()->user()->id,
                        'color_id' => $row->color_id,
                        'size_id' => $row->size_id,
                    ]);

                    if ($store_id == 0) {
                        $item->quantity = $item->quantity - $outgoing;
                        $item->save();
                    } elseif ($store_item) {
//				return $store_item;
                        $store_item->store_quant = $store_item->store_quant - $outgoing;
                        $store_item->save();
                    }

                }

                if ($request->cash == 1) {

                    $sale_point->money_point = $sale_point->money_point + $final;
                    $sale_point->save();
                    $balance = $client->balance;
                } else {
                    $balance = $client->balance + $final;
                    $client->balance = $balance;
                    $client->save();

                }
                //remember to fix it and make it after discount and addition
                DB::table('client_transaction')->insert([
                    'shop_id' => get_session_val('serial_id', 'integer'),
                    'date_time' => Carbon::now(),
                    'client_id' => $order->client_id,
                    'amount' => $final,//  المدفوع فى الفاتورة
                    'type' => 1,
                    'effect' => 1,
                    'pay_day' => Carbon::now(),
                    'balance' => $balance,// رصيد العميل
                    'bill_id' => $sale_process,
                    'bill_net_total' => $final,
                    'safe_type' => 1,
                    'safe_balance' => $sale_point->money_point,
                    'safe_point_id' => $sale_point->id,
                    'user_id' => auth()->user()->id,
                    'sale_back_id' => 0,
                    'problem_id' => 0,
                    'supplier_id' => 0,
                    'salary_month' => '',
                    'OutTransactionID' => 0,
                    'spend_id' => 0,

                ]);

                $adds = RequestAddHistory::where('request_id', $order->id)->where('shop_id', get_session_val('serial_id', 'integer'))->get();

                $settings = RequestSettings::where('shop_id', get_session_val('serial_id', 'integer'))->first();
                $net_all = $final - $fee;
                $order->status = 2;
                $order->save();

                $msg= " تم استلام طلبك رقم" .$order->order_no;
                $client_notification = new ClientNotification();
                $client_notification->client_id = $client->id;
                $client_notification->order_id = $order->id;
                $client_notification->message = $msg;
                $client_notification->type = 1;
                $client_notification->shop_id = get_session_val('serial_id', 'integer');
                $client_notification->save();


                foreach ($adds as $add_) {
                    $add = DB::table('bills_add')
                        ->where('shop_id', get_session_val('serial_id', 'integer'))
                        ->where('id', $add_->addition_id)
                        ->first();

                    if ($add) {
                        $add_type = $add->check_addition;
                        if ($add_type == 0) {
                            $added_val = $net * $add->addition_value / 100;
                        } else {
                            $added_val = $add->addition_value;
                        }
                    }

                    $billAdd = new RequestAddHistory();
                    $billAdd->shop_id = get_session_val('serial_id', 'integer');
                    $billAdd->bill_id = $sale_process_data->id;
                    $billAdd->addition_value = $added_val;
                    $billAdd->addition_id = $add->id;

                    $billAdd->save();
                }

                $data = [
                    'name' => $order->client_name,
                    'email' => $order->email,
                    'bill_id' => $sale_process,
                    'shop_id' => get_session_val('serial_id', 'integer')
                ];

                return $data;
                
                //Event::fire(new EmailBillEvent($data));

            });
            
            $m = $this->handleMain($tr);
                        
            flash()->success('تم انشاء الفاتوره بنجاح');

            return back()->with(['saved' => true]);

        } else {


            return back()->with([
                'error' => 'لا يمكن عمل فاتورة لان الكمية المطلوبة لا تكافئ الكمية المنصرفة'
            ]);
        }
    }

    protected function getCards($item, $qty, $order_id)
    {
        $cardCheck = $this->cardCheck($item);

        if ($cardCheck != '0') {
            $cards_query = Card::where([
                'shop_id' => get_session_val('serial_id', 'integer'),
                'item_id' => $item->id,
                'request_id' => $order_id,
                //  'sale_id' => 0,
                //   'card_state' => 0
            ]);
            $cards = $cards_query->pluck('id')->take($qty);

            $cards = json_decode(json_encode($cards));
            return $cards;
        } else {
            return false;
        }

    }

    protected function getNewCards($item, $qty)
    {

        $cards_query = Card::where([
            'shop_id' => get_session_val('serial_id', 'integer'),
            'item_id' => $item->id,
            'request_id' => 0,
            'sale_id' => 0,
            'card_state' => 0
        ]);
        $cards = $cards_query->pluck('id')->take($qty);
        if (sizeof($cards) > 0) {
            $cards = json_decode(json_encode($cards));
            return $cards;
        }
        return false;
    }

    protected function updateCards($ids, $sale_id, $request_id)
    {
        foreach ($ids as $id) {
            $card = Card::find($id);
            $card->sale_id = $sale_id;
            $card->card_state = 1;
            $card->request_id = $request_id;
            $card->save();
        }
    }

    private function cardCheck($item)
    {
        $cardCompany_id = $item->card_company_id;
        if (is_null($cardCompany_id)) {
            return '0';
        }
        return $cardCompany_id;
    }

    public function saveRequestStatus(Request $request, $id)
    {
        $update = BadrRequest::findOrFail($id);
        $update->update($request->all());
        if ($update->status == 0) {
            $cards = $cards_query = Card::where([
                'shop_id' => get_session_val('serial_id', 'integer'),
                'request_id' => $update->id,
            ])->get();
            $cards->each(function ($card) {
                $card->request_id = 0;
                $card->sale_id = 0;
                $card->card_state = 0;
                $card->save();
            });
        }
        $client = Client::find($update->client_id);

        $st_msg = '';
        switch ($update->status) {
            case "0":
                $st_msg= "تم الغاء طلبك رقم" . $update->order_no ;
                break;
            case "1":
                $st_msg= "جارى تنفيذ طلبك رقم" .$update->order_no;
                break;
            case "2":
                $st_msg= " تم استلام طلبك رقم" .$update->order_no;
                break;
            case "3":
                $st_msg = " تم شحن طلبك رقم".$update->order_no;
                break;
                case "4":
                    $st_msg = "طلبك رقم" . $update->order_no.  'معلق لحين اكمال الدفع' ;
                break;
                case "5":
                    $st_msg = "طلبك رقم" . $update->order_no.  'معلق لحين التجهيز' ;
                break;
            default:
                echo 'تم تغيير حالة الطلب رقم ' . $update->order_no;
        }

        $msg = $st_msg;
        $client_notification = new ClientNotification();
        $client_notification->client_id = $client->id;
        $client_notification->order_id = $update->id;
        $client_notification->message = $msg;
        $client_notification->type = 1;
        $client_notification->shop_id = get_session_val('serial_id', 'integer');
        $client_notification->save();

        $msg = 'تم تغيير حالة الطلب رقم ' . $update->order_no;

        flash()->success('تم الحفظ بنجاح');

        return back();


    }

    public function requestLocation(Request $request, $id)
    {

        $model = BadrRequest::findOrFail($id);

        return view('ecommerce::location', compact('model'));
    }

    public function requestSettings()
    {

        $settings = RequestSettings::where('shop_id', get_session_val('serial_id', 'integer'))->first();
        if (is_null($settings)) {
            $settings = new RequestSettings();
            $settings->fee = 0;
            $settings->min_purchase = 0;
            $settings->max_charge = 0;
            $settings->max_cards = 5;
            $settings->max_items = 20;
            $settings->epay = 0;
            $settings->bank = 1;
            $settings->delivery = 1;
            $settings->shop_id = get_session_val('serial_id', 'integer');
            $settings->save();
        }

        return view('ecommerce::settings', compact('settings'));
    }

    public function updateRequestSettings(Request $request)
    {

        $s = RequestSettings::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('id', $request->settings_id)
            ->first();

        $this->validate($request, [
            'fee' => 'required',
            'min_purchase' => 'required',
            'max_charge' => 'required',
        ], [
            'fee.required' => 'رسوم الشحن مطلوبة',
            'min_purchase.required' => 'الحد الادني للطلب مطلوبة',
            'max_charge.required' => 'الحد الاقصي لفرض رسوم شحن مطلوبة',
        ]);

        $s->fee = $request->fee;
        $s->fee_type = $request->fee_type;
        $s->min_purchase = $request->min_purchase;
        $s->lat = $request->lat;
        $s->lon = $request->lon;
        $s->max_cards = $request->max_cards;
        $s->max_items = $request->max_items;
        $s->epay = $request->epay ? 1 : 0;
        $s->bank = $request->bank ? 1 : 0;
        $s->delivery = $request->delivery ? 1 : 0;
        $s->max_charge = $request->max_charge ;
        $s->save();


        return redirect()->back()->with([
            'success' => 'تم الحفظ'
        ]);

    }
    public function checkOrderCards($order_id)
    {
        $details = RequestDetails::where([
            'request_id' => $order_id,
            'shop_id' => get_session_val('serial_id', 'integer')
        ])
            ->select('id', 'item_id', 'quantity')
            ->get();
    }

    public function saveBillNative(Request $request)
    {

        session()->put('shop_id', $request->shop_id);

        $order_id = $request->order_id;

        $check_cards = $this->checkAvailability($order_id);
        if ($check_cards) {

            $items_ = RequestDetails::where('shop_id', get_session_val('serial_id', 'integer'))->where('request_id', $order_id)->get();

            $ch_cards = $this->getRequestCardQty($order_id, get_session_val('serial_id', 'integer'), $items_);

            if (sizeof($ch_cards) > 0) {

                return ['bill_id' => 'x'];
            } else {
                try {
                    $sale_point = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                        ->where('point_name', 'الدفع الالكتروني')
                        ->first();
                    if (is_null($sale_point)) {
                        $sale_point = new SalePoint();
                        $sale_point->shop_id = get_session_val('serial_id', 'integer');
                        $sale_point->point_name = 'الدفع الالكتروني';
                        $sale_point->save();
                    }

                    $tr = DB::transaction(function () use ($order_id, $sale_point) {

                        $order = BadrRequest::find($order_id);

                        $rows = RequestDetails::where('shop_id', $order->shop_id)->where('request_id', $order_id)->get();

                        $bill_no_ = Invoices::where([
                            ['shop_id', get_session_val('serial_id', 'Integer')],
                            ['invoice_type', '1'],
                        ])->orderBy('id', 'DESC')->get();

                        if (count($bill_no_) > 0) {
                            $bill_no_m = $bill_no_->max('bill_no');
                            $bill_no = $bill_no_m + 1;
                        } else {
                            $bill_no = 1;
                        }

                        $store_id = $sale_point->store_id;

                        $client = Client::find($order->client_id);
                        if ($client) {
                            $balance = $client->balance;
                        } else {
                            $balance = 0;
                        }

                        $rest = 0;

                        $n_balance = $balance - $rest;

                        if (in_array($sale_point->store_id, [0, 'x'])) {
                            $store_id = 0;
                        }

                        $sale_process = DB::table('invoices_')->insertGetId([
                            'transaction_date' =>Carbon::now(),
                            'invoice_date' => Carbon::now(),
                            'bill_total' => $order->total,
                            'add_user' => 0,
                            'client_id' => $order->client_id,
                            'net_price' => $order->net,
                            'payment' => $order->net,
                            'bill_payment' => $order->net,
                            'rest' => $rest,
                            'store_id' => $store_id,
                            'bill_no' => $bill_no,
                            'notes' => 'فاتورة دفع الكتروني',
                            'shop_id' => $order->shop_id,
                            'balance' => $n_balance,
                            'pay_date' => Carbon::today(),
                            'StoreTransport' => 0,
                            'StoreServices' => 0,
                            'fee' => $order->fee,     
                            'invoice_type' => 1     
                        ]);

                        $sale_process_data = Invoices::where([
                            ['shop_id', get_session_val('serial_id', 'Integer')],
                            ['invoice_type', '1']
                        ])->findOrFail($sale_process)();
                        $order->no_bill = $sale_process;
                        $order->status = 2;
                        $order->save();

                        foreach ($rows as $row) {

                            $item = Item::findOrFail($row->item_id);

                            $store_item = StoreItem::where('item_id', $row->item_id)
                                ->where('shop_id', $order->shop_id)
                                ->where('store_id', $sale_point->store_id)
                                ->first();

                            if (in_array($store_id, [0, 'x'])) {
                                $item_quantity = $item->quantity;
                            } else {
                                if ($store_item) {
                                    $item_quantity = $store_item->store_quant;
                                } else {
                                    $item_quantity = 0;
                                }

                            }

                            $outgoing_ = $row->quantity;
                            $u_id = $row->unit_id;

                            if ($item->unit_id == $u_id) {
                                $outgoing = $outgoing_;
                            } else {
                                $xx = ItemUnit::where('item_id', $item->id)
                                    ->where('unit_id', $u_id)
                                    ->where('shop_id', $order->shop_id)
                                    ->first();

                                if ($xx) {
                                    $type = $xx->unit_type;
                                    if ($type == 0) {
                                        $outgoing = $outgoing_ * $xx->unit_value;
                                    } else {
                                        $outgoing = $outgoing_ / $xx->unit_value;
                                    }
                                } else {
                                    $outgoing = $outgoing_;
                                }
                            }

                            DB::table('invoices_details')->insert([
                                'invoice_id' => $sale_process,
                                'item_id' => $row->items_id,
                                'shop_id' => $order->shop_id,
                                'quantity' => $outgoing_,
                                'unit_id' => $row->unit_id,
                                'quantity_after' => $item_quantity - $outgoing,
                                'pay_price' => $item->pay_price,
                                'sale_price' => $row->price,
                                'total_price' => $outgoing * $row->price,
                                'basic_sale_price' => $item->sale_price,
                                'basic_pay_price' => $item->pay_price,
                                'item_about' => '',
                                'not_sold' => 0,
                                'discount' => $order->discount / $outgoing_,
                                'discount_type' => 1,
                                'pay_vats_weight' => 0,
                                'sale_ratio' => 1,
                                'bolla' => 0,
                                'safe_point_id' => $sale_point->id,
                                'store_id' => $store_id,
                                'add_user' => 0,
                                'color_id' => $row->color_id,
                                'size_id' => $row->size_id,
                            ]);


                            if ($store_id == 0) {
                                $item->quantity = $item->quantity - $outgoing;
                                $item->save();
                            } elseif ($store_item) {
                                $store_item->store_quant = $store_item->store_quant - $outgoing;
                                $store_item->save();
                            }

                            $cards_query = Card::where([
                                'shop_id' => $order->shop_id,
                                'item_id' => $row->item_id,
                                'request_id' => $order_id
                            ])->get();

                            $cards_query->each(function ($card) use ($sale_process) {
                                $card->sale_id = $sale_process;
                                $card->card_state = 1;
                                $card->save();
                            });
                        }

                        //cash
                        //remember to fix it and make it after discount and addition
                        $sale_point->money_point = $sale_point->money_point + $order->net;
                        $sale_point->save();
                        $balance = $client->balance;


                        //remember to fix it and make it after discount and addition
                        DB::table('client_transaction')->insert([
                            'shop_id' => get_session_val('serial_id', 'integer'),
                            'date_time' => Carbon::now(),
                            'client_id' => $order->client_id,
                            'amount' => $order->net,//  المدفوع فى الفاتورة
                            'type' => 1,
                            'effect' => 1,
                            'pay_day' => Carbon::now(),
                            'balance' => $balance,// رصيد العميل
                            'bill_id' => $sale_process,
                            'bill_net_total' => $order->net,
                            'safe_type' => 1,
                            'safe_balance' => $sale_point->money_point,
                            'safe_point_id' => $sale_point->id,
                            'user_id' => 0,
                            'sale_back_id' => 0,
                            'problem_id' => 0,
                            'supplier_id' => 0,
                            'salary_month' => '',
                            'OutTransactionID' => 0,
                            'spend_id' => 0,

                        ]);

                        $ads = RequestAddHistory::where('request_id', $order_id)
                            ->where('shop_id', $order->shop_id)->get();

                        foreach ($ads as $ad) {
                            $ad->bill_id = $sale_process;
                            $ad->save();
                        }

                        $client = Client::find($order->client_id);
                        $data = [
                            'name' => $client->client_name,
                            'email' => $client->email,
                            'bill_id' => $sale_process,
                            'shop_id' => $order->shop_id
                        ];
                        
                        $this->handleMain($data);

                       // Event::fire(new EmailBillEvent($data));

                        return ['bill_id' => $sale_process];
                    });

                    return ['bill_id' => $tr['bill_id']];

                } catch (\Exception $e) {
                    $request = BadrRequest::find($order_id);
                    $request->status = 5;
                    $request->save();

                    return ['bill_id' => '0'];

                }
            }
        } else {


            $request = BadrRequest::find($order_id);
            $request->status = 5;
            $request->save();

            return ['bill_id' => '0'];
        }
    }

    public function getRequestCardQty($order_id, $shop_id, $details)
    {
        $obj = [];
        foreach ($details as $detail) {
            $item = Item::find($detail->item_id);
            $cardCheck = $this->cardCheck($item);

            if ($cardCheck != '0') {

                $c_data = Card::where([
                    'shop_id' => $shop_id,
                    'request_id' => $order_id,
                    'item_id' => $detail->item_id
                ])->count();

                if ($detail->quantity != $c_data) {
                    $obj[] = '0';
                }
            }
        }
        return $obj;
    }

    public function createBillData($id)
    {

        $bill = Invoices::where([
            ['shop_id', get_session_val('serial_id', 'Integer')],
            ['invoice_type', '1'],
            ['id', $id]
        ])->first();

        $adds = RequestAddHistory::where('bill_id', $id)
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->sum('addition_value');

        $details = SaleDetails::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('sale_id', $id)
            ->select('items_id', 'sale_id', 'id', 'total_price', 'quantity', 'date_sale', 'item_name', 'unit', 'price')
            ->get();
        
        $url = config('main.badr')."CardData.php";
        $shop_id = get_session_val('serial_id', 'integer');
        $b_id = $id;
        $curl = curl_init();
        
        curl_setopt_array($curl, array(
        CURLOPT_URL => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "POST",
        CURLOPT_POSTFIELDS => "------WebKitFormBoundary7MA4YWxkTrZu0gW\r\nContent-Disposition: form-data; name=\"shop_id\"\r\n\r\n".$shop_id."\r\n------WebKitFormBoundary7MA4YWxkTrZu0gW\r\nContent-Disposition: form-data; name=\"bill_id\"\r\n\r\n".$b_id."\r\n------WebKitFormBoundary7MA4YWxkTrZu0gW--",
        CURLOPT_HTTPHEADER => array(
        "cache-control: no-cache",
        "content-type: multipart/form-data; boundary=----WebKitFormBoundary7MA4YWxkTrZu0gW",
        "postman-token: 625c1be0-64ab-a8a4-7a9a-3fe9b800adfb"
        ),
        ));
        
        $response = curl_exec($curl);
        $err = curl_error($curl);
        
        curl_close($curl);
        
        $cards = json_decode($response, true);
        
        foreach ($details as $detail) {
            $item = Item::find($detail->items_id);
            if (is_array($cards)){        

                $detail->cards = array_key_exists($item->id, $cards) ? $cards[$item->id] : [];
            }else {
                $detail->cards = [];
            }
        }
        return compact('bill', 'details', 'adds');
    }

    public function mailTest()
    {


        $client = Client::find(131);
        $data = [
            'name' => 'mahmoud',
            'email' => 'mahmoudarafat33@gmail.com',
            'bill_id' => 195,
            'shop_id' => 8521
        ];

        session()->put('shop_id', 8521);

        $this->handleMain($data);
          // Event::fire(new EmailBillEvent($data));

        /*

         $data = [
                    'name' => 'mahmoud',
                    'email' => 'mahmoudarafat33@gmail.com',
                    'bill_id' => 185,
                    'shop_id' => 8521
                ];

        $email = $data['email'];
        $name = $data['name'];
        $b_id = $data['bill_id'];
        $shop_id = $data['shop_id'];
        session()->put('shop_id', $shop_id);
        $data2 = $this->createMailData($b_id, $shop_id);
        $shop = DB::table('badr_shop')->where('serial_id', $shop_id)
            ->select('logo_path', 'serial_id', 'shop_name')->first();
        return view('bill-mail2')->with(['data' => $data2, 'sub' => $data, 'shop' => $shop]);
    */

    }

    public function createMailData($id)
    {

        $bill = Invoices::where([
            ['shop_id', get_session_val('serial_id', 'Integer')],
            ['invoice_type', '1'],
            ['id', $id]
        ])->first();
        /*
        $adds = RequestAddHistory::where('bill_id', $id)
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->select('addition_id', 'addition_value')
            ->get();
        */

        $adds = RequestAddHistory::where('bill_id', $bill->id)
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->sum('addition_value');

        /*

         foreach ($adds as $add) {
             $aa = DB::table('bills_add')->where('id', $add->addition_id)->first();
             if ($aa) {
                 $aa_name = $aa->Addition_name;
             } else {
                 $aa_name = '--';
             }
             $add->name = $aa_name;
         }*/

        $details = SaleDetails::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('sale_id', $id)
            ->select('items_id', 'sale_id', 'id', 'total_price', 'quantity', 'date_sale', 'item_name', 'unit', 'price')
            ->get();

        foreach ($details as $detail) {
            $item = Item::find($detail->items_id);
            $detail->cards = $item->getCards($id) ?? [];
        }
        return compact('bill', 'details', 'adds');
    }


    public function saveBillNativeMobile(Request $request)
    {


        $fort_id = $request->fort_id;
        $msg = '';
        session()->put('shop_id', $request->shop_id);

        $responseCheck = null;

        $order_id = $request->order_id;
        $check_cards = $this->checkAvailability($order_id);
      
        if ($check_cards) {

            $items_ = RequestDetails::where('shop_id', get_session_val('serial_id', 'integer'))->where('request_id', $order_id)->get();
            $ch_cards = $this->getRequestCardQty($order_id, get_session_val('serial_id', 'integer'), $items_);

            if (sizeof($ch_cards) > 0) {
                $responseCheck = ['bill_id' => 'x'];
            } else {
//                try {
                $sale_point = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                    ->where('point_name', 'الدفع الالكتروني')
                    ->first();
                if (is_null($sale_point)) {
                    $sale_point = new SalePoint();
                    $sale_point->shop_id = get_session_val('serial_id', 'integer');
                    $sale_point->point_name = 'الدفع الالكتروني';
                    $sale_point->save();
                }

                $tr = DB::transaction(function () use ($order_id, $sale_point, $request) {


                    $feedback = $request->feedback;
                    $feedback_response = $request->feedback_response;

                    $order = BadrRequest::find($order_id);


                    $rows = RequestDetails::where('shop_id', $order->shop_id)->where('request_id', $order_id)->get();

                    $bill_no_ = Invoices::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoice_type', '1']
                    ])->orderBy('id', 'DESC')->first();

                    if ($bill_no_) {
                        $bill_no_m = $bill_no_->bill_no;
                        $bill_no = $bill_no_m + 1;
                    } else {
                        $bill_no = 1;
                    }

                    $store_id = $sale_point->store_id;

                    $client = Client::find($order->client_id);
                    if ($client) {
                        $balance = $client->balance;
                    } else {
                        $balance = 0;
                    }


                    //cash
//                        $payment = $request->net;
                    $rest = 0;

                    $n_balance = $balance - $rest;

                    if (in_array($sale_point->store_id, [0, 'x'])) {
                        $store_id = 0;
                    }

                    $sale_process = DB::table('invoices_')->insertGetId([
                        'transaction_date' =>Carbon::now(),
                        'invoice_date' => Carbon::now(),
                        'bill_total' => $order->total,
                        'add_user' => 0,
                        'client_id' => $order->client_id,
                        'net_price' => $order->net,
                        'payment' => $order->net,
                        'bill_payment' => $order->net,
                        'rest' => $rest,
                        'store_id' => $store_id,
                        'bill_no' => $bill_no,
                        'notes' => 'فاتورة دفع الكتروني',
                        'shop_id' => $order->shop_id,
                        'balance' => $n_balance,
                        'pay_date' => Carbon::today(),
                        'StoreTransport' => 0,
                        'StoreServices' => 0,
                        'fee' => $order->fee,     
                        'invoice_type' => 1     
                    ]);

                    $sale_process_data = Invoices::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoice_type', '1']
                    ])->findOrFail($sale_process);

                    $order->no_bill = $sale_process;
                    $order->status = 2;
                    $order->feedback_response = $feedback_response;
                    $order->feedback = $feedback;

                    $order->save();

                    foreach ($rows as $row) {

                        $item = Item::findOrFail($row->item_id);

                        $store_item = StoreItem::where('item_id', $row->item_id)
                            ->where('shop_id', $order->shop_id)
                            ->where('store_id', $sale_point->store_id)
                            ->first();

                        if (in_array($store_id, [0, 'x'])) {
                            $item_quantity = $item->quantity;
                        } else {
                            if ($store_item) {
                                $item_quantity = $store_item->store_quant;
                            } else {
                                $item_quantity = 0;
                            }
                        }

                        // `check for the exact qty to reduce`;
                        $outgoing_ = $row->quantity;
                        $u_id = $row->unit_id;

                        if ($item->unit_id == $u_id) {
                            $outgoing = $outgoing_;
                        } else {
                            $xx = ItemUnit::where('item_id', $item->id)
                                ->where('unit_id', $u_id)
                                ->where('shop_id', $order->shop_id)
                                ->first();

                            if ($xx) {
                                $type = $xx->unit_type;
                                if ($type == 0) {
                                    $outgoing = $outgoing_ * $xx->unit_value;
                                } else {
                                    $outgoing = $outgoing_ / $xx->unit_value;
                                }
                            } else {
                                $outgoing = $outgoing_;
                            }
                        }

                        DB::table('invoices_details')->insert([
                            'invoice_id' => $sale_process,
                            'item_id' => $row->items_id,
                            'shop_id' => $order->shop_id,
                            'quantity' => $outgoing_,
                            'unit_id' => $row->unit_id,
                            'quantity_after' => $item_quantity - $outgoing,
                            'pay_price' => $item->pay_price,
                            'sale_price' => $row->price,
                            'total_price' => $outgoing * $row->price,
                            'basic_sale_price' => $item->sale_price,
                            'basic_pay_price' => $item->pay_price,
                            'item_about' => '',
                            'not_sold' => 0,
                            'discount' => $order->discount / $outgoing_,
                            'discount_type' => 1,
                            'pay_vats_weight' => 0,
                            'sale_ratio' => 1,
                            'bolla' => 0,
                            'safe_point_id' => $sale_point->id,
                            'store_id' => $store_id,
                            'add_user' => 0,
                            'color_id' => $row->color_id,
                            'size_id' => $row->size_id,
                        ]);


                        if ($store_id == 0) {
                            $item->quantity = $item->quantity - $outgoing;
                            $item->save();
                        } elseif ($store_item) {
                            $store_item->store_quant = $store_item->store_quant - $outgoing;
                            $store_item->save();
                        }

                        $cards_query = Card::where([
                            'shop_id' => $order->shop_id,
                            'item_id' => $row->item_id,
                            'request_id' => $order_id
                        ])->get();

                        $cards_query->each(function ($card) use ($sale_process) {
                            $card->sale_id = $sale_process;
                            $card->card_state = 1;
                            $card->save();
                        });
                    }

                    //cash
                    //remember to fix it and make it after discount and addition
                    $sale_point->money_point = $sale_point->money_point + $order->net;
                    $sale_point->save();
                    $balance = $client->balance;


                    //remember to fix it and make it after discount and addition
                    DB::table('client_transaction')->insert([
                        'shop_id' => get_session_val('serial_id', 'integer'),
                        'date_time' => Carbon::now(),
                        'client_id' => $order->client_id,
                        'amount' => $order->net,//  المدفوع فى الفاتورة
                        'type' => 1,
                        'effect' => 1,
                        'pay_day' => Carbon::now(),
                        'balance' => $balance,// رصيد العميل
                        'bill_id' => $sale_process,
                        'bill_net_total' => $order->net,
                        'safe_type' => 1,
                        'safe_balance' => $sale_point->money_point,
                        'safe_point_id' => $sale_point->id,
                        'user_id' => 0,
                        'sale_back_id' => 0,
                        'problem_id' => 0,
                        'supplier_id' => 0,
                        'salary_month' => '',
                        'OutTransactionID' => 0,
                        'spend_id' => 0,

                    ]);

                    $ads = RequestAddHistory::where('request_id', $order_id)
                        ->where('shop_id', $order->shop_id)->get();

                    foreach ($ads as $ad) {
                        $ad->bill_id = $sale_process;
                        $ad->save();
                    }

                    $client = Client::find($order->client_id);
                    $data = [
                        'name' => $order->client_name,
                        'email' => $order->email,
                        'bill_id' => $sale_process,
                        'shop_id' => $order->shop_id
                    ];

                    

                    //  Event::fire(new EmailBillEvent($data));

                    return ['bill_id' => $sale_process];
                });

                $responseCheck = ['bill_id' => $tr['bill_id'], 'shop_id' => $request->shop_id];
                
                $this->handleMain($responseCheck);
            }
        } else {

            $order = BadrRequest::find($order_id);
            $order->status = 5;
            $order->save();

            $msg = 'فشلت عملية الدفع لعدم توافر الكمية المطلوبة.';
            return response()->json([
                'status' => false,
                'msg' => $msg
            ]);
        }
        $order = BadrRequest::find($order_id);
        if (!in_array($responseCheck, ['0', 'x', null])) {

            $b = Invoices::where([
                ['shop_id', get_session_val('serial_id', 'Integer')],
                ['invoice_type', '1'],
                ['id', $responseCheck]
            ])->first();

            $b->notes = $b->notes . ' - ' . $fort_id;
            $b->save();

            $order->status = 3;
            $msg .= 'تمت عملية الدفع وحفظ الطلب بنجاح';

        } else {
            if ($responseCheck == 'x') {
                $order->status = 4;
                $msg .= 'فشلت عملية الدفع لعدم توافر الكمية المطلوبة.
                             وسيتم اعلامك فور توافر الكمية.
                              يمكنك محاولة الدفع مرة أخري من صفحة الطلب.';

            } else {
                $order->status = 5;
                $msg .= 'تم حفظ الطلب ولكن فشلت عملية الدفع. يمكنك محاولة الدفع مرة أخري من صفحة الطلب.';
            }
        }
        $order->save();

        return response()->json([
            'status' => true,
            'msg' => $msg
        ]);

    }


public function sendMailTest(){
    
     $data = [
                    'name' => "mahmoud",
                    'email' => "mahmoudarafat33@gmail.com",
                    'bill_id' => 117951,
                    'shop_id' => 3970
                ];

                $this->handleMain($data);
    return 'done';
}


 private function make_post_body($post_fields)
{
	$post_body = '';
	foreach ($post_fields as $key => $value) {
	  
		$post_body .= urlencode($key) . '=' . urlencode($value) . '&';
	  
	}
	
	$post_body = rtrim($post_body, '&');
	
	return $post_body;
}

public function getCardNumbersNative($event){
    $b_id = $event['bill_id'];
    $shop_id = $event['shop_id'];
    $url = config('main.badr')."CardData.php";
    
    $curl = curl_init();
    
    curl_setopt_array($curl, array(
    CURLOPT_URL => $url,
    CURLOPT_RETURNTRANSFER => true,
    CURLOPT_ENCODING => "",
    CURLOPT_MAXREDIRS => 10,
    CURLOPT_TIMEOUT => 30,
    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
    CURLOPT_CUSTOMREQUEST => "POST",
    CURLOPT_POSTFIELDS => "------WebKitFormBoundary7MA4YWxkTrZu0gW\r\nContent-Disposition: form-data; name=\"shop_id\"\r\n\r\n".$shop_id."\r\n------WebKitFormBoundary7MA4YWxkTrZu0gW\r\nContent-Disposition: form-data; name=\"bill_id\"\r\n\r\n".$b_id."\r\n------WebKitFormBoundary7MA4YWxkTrZu0gW--",
    CURLOPT_HTTPHEADER => array(
    "cache-control: no-cache",
    "content-type: multipart/form-data; boundary=----WebKitFormBoundary7MA4YWxkTrZu0gW",
    "postman-token: 625c1be0-64ab-a8a4-7a9a-3fe9b800adfb"
    ),
    ));
    
    $response = curl_exec($curl);
    $err = curl_error($curl);
    
    curl_close($curl);
    
    return $response;
}



public function handleMain($event)
    {
     /*   $email = $event['email'];
        $name = $event['name'];*/
        
        $b_id = $event['bill_id'];
        $shop_id = $event['shop_id'];
        $url = config('main.badr')."mail.php";
    
    
      
        $curl = curl_init();
        
        curl_setopt_array($curl, array(
          CURLOPT_URL => $url,
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "POST",
          CURLOPT_POSTFIELDS => "------WebKitFormBoundary7MA4YWxkTrZu0gW\r\nContent-Disposition: form-data; name=\"shop_id\"\r\n\r\n".$shop_id."\r\n------WebKitFormBoundary7MA4YWxkTrZu0gW\r\nContent-Disposition: form-data; name=\"bill_id\"\r\n\r\n".$b_id."\r\n------WebKitFormBoundary7MA4YWxkTrZu0gW--",
          CURLOPT_HTTPHEADER => array(
            "cache-control: no-cache",
            "content-type: multipart/form-data; boundary=----WebKitFormBoundary7MA4YWxkTrZu0gW",
            "postman-token: 625c1be0-64ab-a8a4-7a9a-3fe9b800adfb"
          ),
        ));
        
        $response = curl_exec($curl);
        $err = curl_error($curl);
        
        curl_close($curl);
        
        
        return $response;
        
      /*  if ($err) {
          echo "cURL Error #:" . $err;
        } else {
          echo $response;
        }
*/
/*
        $data = $this->createMailData2($b_id, $shop_id);

        \Mail::send('bill-mail', ['data' => $data, 'sub' => $event], function ($m) use ($email, $name) {
            $m->from('info@cablshop.com', 'NestStation');
            $m->to($email, $name)->subject('Order Success');
        });*/
    }

    public function createMailData2($id, $shop_id){
/*
        $bill = SaleProcess::find($id);

        $adds = RequestAddHistory::where('bill_id', $id)
            ->where('shop_id', $shop_id)
            ->select('addition_id', 'addition_value')
            ->get();

        foreach ($adds as $add) {
            $aa = DB::table('bills_add')->where('id', $add->addition_id)->first();
            if ($aa) {
                $aa_name = $aa->Addition_name;
            } else {
                $aa_name = '--';
            }
            $add->name = $aa_name;
        }

        $details = SaleDetails::where('shop_id', $shop_id)
            ->where('sale_id', $id)
            ->select('items_id', 'sale_id', 'id', 'total_price', 'quantity', 'date_sale', 'item_name', 'unit', 'price')
            ->get();

        foreach ($details as $detail){
            $item = Item::find($detail->items_id);
            
            $cardCheck = $this->cardCheck($item);

            $sent = [];
           
           if ($cardCheck != '0') {
                $cards_query = Card::where([
                'shop_id' => $item->shop_id,
                'item_id' => $item->id,
                'sale_id' => $id,
                'card_state' => 1
                    ])->selectRaw('id, item_id, card_number, type')->get();
    
                if (count($cards_query) > 0) {
                    foreach ($cards_query as $card) {
                        if ($card->type == 1){
                            $sent[] = decryptMe($card);
                        }else {
                            $sent[] = decryptMeNew($card);
                        }
                    }
                }
            }
            
            
            $detail->cards = $sent;
        }
        return compact('bill', 'details', 'adds');*/
    }








    public function WebStyle()
    {
        $project_path_url = str_replace("\\", '/', base_path());
        return view('ecommerce::web-style', compact('project_path_url'));
    }

    public function ChangeStyle(Request $request)
    {
        $shop = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
        if ($shop) {
            $shop->style = $request->style;
            $shop->save();

//            DB::table('badr_shop')->update(['style'=>$request->style])->where('badr_shop.serial_id', get_session_val('serial_id', 'integer'));

            return back()->with(['success' => trans('native.correct_save')]);

        } else {
            return back()->with(['error' => trans('native.Error_save')]);


        }
    }












}

