<?php

namespace Modules\EInvoice\Http\Controllers;

use App\Services\Einvoice\Einvoice;
use App\Services\Einvoice\Invoice\InvoiceDocument;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;
use Illuminate\Support\Arr;
use Modules\EInvoice\Entities\EinvoiceItemCode;

class ItemCodeController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return Response
     */
    public function index(Request $request)
    {
        $itemCodes = EinvoiceItemCode::orderBy('id', 'desc');

        // Search in einvoices.
        if ($request->has('search') && !in_array($request->search, ['', ' ', null])) {
            $itemCodes = $itemCodes->where(function ($query) use ($request) {
                $query->where('codeTypeName', 'like', '%' . $request->search . '%')
                    ->orWhere('itemCode', 'like', '%' . $request->search . '%')
                    ->orWhere('codeNameSecondaryLang', 'like', '%' . $request->search . '%')
                    ->orWhere('codeNamePrimaryLang', 'like', '%' . $request->search . '%')
                    ->orWhere('parentItemCode', 'like', '%' . $request->search . '%')
                    ->orWhere('parentCodeNameSecondaryLang', 'like', '%' . $request->search . '%')
                    ->orWhere('parentCodeNamePrimaryLang', 'like', '%' . $request->search . '%')
                    ->orWhere('ownerTaxpayer_nameAr', 'like', '%' . $request->search . '%')
                    ->orWhere('ownerTaxpayer_name', 'like', '%' . $request->search . '%')
                    ->orWhere('requesterTaxpayer_nameAr', 'like', '%' . $request->search . '%')
                    ->orWhere('requesterTaxpayer_name', 'like', '%' . $request->search . '%');
            });
        }

        if ($request->has('status') && !in_array($request->status, ['', ' ', null])) {
            $itemCodes = $itemCodes->where('status', $request->status);
        }

        if ($request->has('active') && !in_array($request->active, ['', ' ', null])) {
            $itemCodes = $itemCodes->where('active', $request->active);
        }

        if ($request->has('ActiveFrom') && !in_array($request->ActiveFrom, ['', ' ', null])) {
            $itemCodes = $itemCodes->whereDate('activeFrom', '>=', $request->ActiveFrom);
        }

        if ($request->has('ActiveTo') && !in_array($request->ActiveTo, ['', ' ', null])) {
            $itemCodes = $itemCodes->whereDate('activeTo', '<=', $request->ActiveTo);
        }

        $itemCodes = !is_null($itemCodes)
            ? $itemCodes->paginate(20)
            : collect();

        $itemCodes->each(function ($itemCode) {
            $type                   = $this->getStatusType($itemCode->status);
            $itemCode->status_color = $type['color'];
            $itemCode->status_text  = $type['text'];
            $itemCode->status_icon  = $type['icon'];

            // Get active status.
            $itemCodeActive         = $itemCode->active == 1 ? 'active' : 'inactive';
            $activeType             = $this->getStatusType($itemCodeActive);
            $itemCode->active_color = $activeType['color'];
            $itemCode->active_text  = $activeType['text'];
        });

        return view('einvoice::itemcodes.index', compact('itemCodes'));
    }

    /**
     * Get the status type of the document.
     *
     * @param $status
     * @return array
     */
    protected function getStatusType($statusType)
    {
        $data = [
            'Approved' => [
                'color' => 'text-success',
                'text'  => __('einvoice.Approved'),
                'icon'  => 'fas fa-check-circle'
            ],
            'Rejected' => [
                'color' => 'text-danger',
                'text'  => __('einvoice.Rejected'),
                'icon'  => 'fas fa-ban'
            ],
            'Submitted' => [
                'color' => 'text-purpel',
                'text'  => __('einvoice.wait_for_approval'),
                'icon'  => 'fas fa-sync'
            ],
            'active' => [
                'color' => 'text-success',
                'text'  => __('extends.yes'),
            ],
            'inactive' => [
                'color' => 'text-danger',
                'text'  => __('extends.no'),
            ],
        ];

        return $data[$statusType] ?? [
            'color' => '',
            'text'  => '',
            'icon'  => ''
        ];
    }

    /**
     * Edit the specified resource.
     *
     * @return Response
     */
    public function edit($item_Code, Request $request)
    {
        // $einvoice = new Einvoice();
        $itemCode = EinvoiceItemCode::where('itemCode', $item_Code)->first();

        if ($request->isMethod('PUT')) {
            if ($request->status == 'Submitted') {
                $data = [
                    "itemCode"      => $request->itemCode,
                    "codeName"      => $request->codeName,
                    "codeNameAr"    => $request->codeNameAr,
                    "description"   => $request->codeDescriptionPrimaryLang,
                    "descriptionAr" => $request->codeDescriptionSecondaryLang,
                    "parentCode"    => $request->parentItemCode,
                    "activeFrom"    => date('Y-m-d\TH:i:s\Z', strtotime($request->activeFrom)),
                    "activeTo"      => date('Y-m-d\TH:i:s\Z', strtotime($request->activeTo))
                ];

                $result = Einvoice::updateEgsCode($request->codeUsageRequestID, $data);
            } else {
                $data   = $request->except(['_token', '_method', 'status', 'codeUsageRequestID']);
                $result = Einvoice::updateCode($itemCode['itemCode'], $data, $itemCode['codeTypeName']);
            }

            // Update Item Code in Database
            $data = Arr::except($data, ["codeName", "codeNameAr", "description", "descriptionAr", "parentCode", "codeDescriptionSecondaryLang", "codeDescriptionPrimaryLang"]);

            if ($request->parentItemCode != null) {
                $data['parentItemCode'] = $request->parentItemCode;
            }

            if ($request->codeName != null) {
                $data['codeNamePrimaryLang'] = $request->codeName;
            }

            if ($request->codeNameAr != null) {
                $data['codeNameSecondaryLang'] = $request->codeNameAr;
            }

            $data['descriptionPrimaryLang']   = $request->codeDescriptionPrimaryLang;
            $data['descriptionSecondaryLang'] = $request->codeDescriptionSecondaryLang;

            $message = isset($result->original['error']['message'])
                ? $result->original['error']['details']
                : __('einvoice.updated_failed');

            if ($result->status() == 200) {
                $updated = EinvoiceItemCode::where('itemCode', $item_Code)->update($data);
                return back()->with('success', __('einvoice.updated_successfully'));
            } else {
                return back()->with('einvoiceErrors', $message);
            }
        }

        return view('einvoice::itemcodes.edit', compact('itemCode'));
    }

    /**
     * Request Code Reuse.
     *
     * @param Request $request
     * @return Response
     */
    public function reuse($itemCode, Request $request)
    {
        $data     = $request->merge(['codetype' => 'EGS', 'itemCode' => $itemCode, 'OrdDir' => 'Descending']);
        $itemCode = Einvoice::requestCodeReuse(collect()->push($data)->toArray())->original;

        if (isset($itemCode['passedItems']) && !empty($itemCode['passedItems'])) {
            // $itemCode = EinvoiceItemCode::create([]);
            return $itemCode
                ? response()->json(['success' => true,  'message' => __('einvoice.send_successfully')])
                : response()->json(['error' => true, 'message' => __('einvoice.error_in_update_invoice')]);
        }
        return response()->json(['success' => false, 'message' => __('einvoice.send_failed')]);
    }

    /**
     * Get published itemCodes.
     *
     * @param Request $request
     * @return Response
     */
    public function published(Request $request)
    {
        $pageSize   = 20;
        $newQueries = [
            'OrdDir' => 'Descending',
            'Ps'     => $pageSize,
            'Pn'     => $request->page ?? 1,
        ];
        $query      = $this->getUrlQuery($request, $newQueries, $pageSize);

        $einvoice   = new Einvoice();
        $response   = Einvoice::searchPublishedCodes($query, $request->codeType ?? 'EGS')->original;

        // Check if there is any publishedCodes.
        if (!is_null($response) && isset($response['result'])) {
            $publishedCodes  = $response['result'] ?? collect();
            $totalCount      = isset($response['metadata']) ? $response['metadata']['totalCount'] : 0;
            $publishedCodes  = paginate($publishedCodes, $pageSize, $totalCount);

            return view('einvoice::itemcodes.published', compact('publishedCodes'));
        } else {
            $publishedCodes = collect();
            return view('einvoice::itemcodes.published', compact('publishedCodes'))->with('einvoiceErrors', $response);
        }
    }

    /**
     * Get query parameters.
     *
     * @param request
     * @param pageSize
     * @return Response
     */
    public function getUrlQuery($request, $newQueries)
    {
        $urlQuery = [];

        foreach ($request->query() as $key => $value) {
            if (!empty($value)) {
                $urlQuery[$key] = $value;
            } else {
                unset($urlQuery[$key]);
            }
        }

        if ($request->has(['search', 'searchcat']) && !in_array($request->search, ['', ' ', null]) && !in_array($request->searchcat, ['', ' ', null])) {
            $urlQuery[$request->searchcat] = $request->search;
        }

        $currentQueries = Arr::except($urlQuery, ['_token', '_method', 'search', 'searchcat', 'codeType', 'page']);
        $allQueries     = array_merge($currentQueries, $newQueries);
        $query          = http_build_query($allQueries);

        return $query;
    }

    // /**
    //  * Update ItemCode.
    //  *
    //  * @return Response
    //  */
    // public function updateCode(Request $request)
    // {
    //     $itemCode = EinvoiceItemCode::where('status', 'Submitted')->where('itemCode', $request->itemCode)->first();
    //     if ($itemCode) {
    //         $einvoiceObject = new Einvoice();
    //         $documentData   = $einvoiceObject->getEgsCodeDetails($itemCode->itemCode);

    //         if ($documentData && isset($documentData->original) && is_array($documentData->original) && isset($documentData->original['status'])) {
    //             $state                                 = $documentData->original['status'];
    //             $itemCode->status                      = $state;
    //             $itemCode->codeID                      = $documentData->original['codeID'];
    //             $itemCode->parentCodeNamePrimaryLang   = $documentData->original['parentCodeName'];
    //             $itemCode->parentCodeNameSecondaryLang = $documentData->original['parentCodeNameAr'];
    //             $itemCode->ownerTaxpayer_rin           = $documentData->original['ownerTaxpayer']['rin'];
    //             $itemCode->ownerTaxpayer_name          = $documentData->original['ownerTaxpayer']['name'];
    //             $itemCode->ownerTaxpayer_nameAr        = $documentData->original['ownerTaxpayer']['nameAr'];

    //             if (isset($documentData->original['requesterTaxpayer'])) {
    //                 $itemCode->requesterTaxpayer_rin           = $documentData->original['requesterTaxpayer']['rin'];
    //                 $itemCode->requesterTaxpayer_name          = $documentData->original['requesterTaxpayer']['name'];
    //                 $itemCode->requesterTaxpayer_nameAr        = $documentData->original['requesterTaxpayer']['nameAr'];
    //             } else {
    //                 $itemCode->requesterTaxpayer_rin           = $documentData->original['ownerTaxpayer']['rin'];
    //                 $itemCode->requesterTaxpayer_name          = $documentData->original['ownerTaxpayer']['name'];
    //                 $itemCode->requesterTaxpayer_nameAr        = $documentData->original['ownerTaxpayer']['nameAr'];
    //             }

    //             $update_code      = $itemCode->save();

    //             if ($update_code) {
    //                 $type         = $this->getStatusType($state);
    //                 $status_color = $type['color'];
    //                 $status_text  = $type['text'];
    //                 $status_icon  = $type['icon'];
    //                 $txt          = '<td class="' . $status_color . ' ' . $state . '" data-uuid="' . $request->uuid . '"><i class="' . $status_icon . '"></i>' . $status_text . '</td>';

    //                 return response()->json(['status' => true, 'msg' => $txt]);
    //             }

    //             return response()->json(['status' => false, 'msg' => __('einvoice.load_status_failed')]);
    //         }

    //         return response()->json(['status' => false, 'msg' => __('extends.load_error')]);
    //     }
    // }
    /**
     * Update ItemCode.
     *
     * @return Response
     */
    public function updateCode(Request $request)
    {
        $itemCode = EinvoiceItemCode::where('status', 'Submitted')->where('itemCode', $request->itemCode)->first();

        if ($itemCode) {
            $einvoiceObject = new InvoiceDocument();
            $documentData   = $einvoiceObject->getEgsCodeDetails($itemCode->itemCode);

            if ($documentData && isset($documentData->original) && is_array($documentData->original) && isset($documentData->original['hasUsageRegistered'])) {
                $state                                 = $documentData->original['hasUsageRegistered'] ? 'Approved' : 'Rejected';
                $itemCode->status                      = $state;
                $itemCode->codeID                      = $documentData->original['codeID'];
                $itemCode->parentCodeNamePrimaryLang   = $documentData->original['parentCodeName'];
                $itemCode->parentCodeNameSecondaryLang = $documentData->original['parentCodeNameAr'];
                $itemCode->ownerTaxpayer_rin           = $documentData->original['taxpayer']['rin'];
                $itemCode->ownerTaxpayer_name          = $documentData->original['taxpayer']['name'];
                $itemCode->ownerTaxpayer_nameAr        = $documentData->original['taxpayer']['nameAr'];
                $update_code                           = $itemCode->save();

                if ($update_code) {
                    $type         = $this->getStatusType($state);
                    $status_color = $type['color'];
                    $status_text  = $type['text'];
                    $status_icon  = $type['icon'];
                    $txt          = '<td class="' . $status_color . ' ' . $state . '" data-uuid="' . $request->uuid . '"><i class="' . $status_icon . '"></i>' . $status_text . '</td>';
                    $taxpayer_txt = '<td>' . app()->getLocale() == 'ar' ? $itemCode->ownerTaxpayer_nameAr : $itemCode->ownerTaxpayer_name . '</td>';

                    return response()->json(['status' => true, 'msg' => $txt, 'taxpayer' => $taxpayer_txt]);
                }

                return response()->json(['status' => false, 'msg' => __('einvoice.load_status_failed')]);
            }

            return response()->json(['status' => false, 'msg' => __('extends.load_error')]);
        }
    }
}
