<?php

namespace Modules\EInvoice\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\DB;
use Modules\EInvoice\Entities\Country;
use Modules\EInvoice\Entities\EinvoiceItemCode;
use Modules\EInvoice\Entities\Einvoices;
use Modules\EInvoice\Entities\Invoice;
use Modules\EInvoice\Entities\Item;
use Modules\EInvoice\Entities\Settings;

class SettingsController extends Controller
{
    public function index(Request $request)
    {
        if ($request->isMethod('post')) {
            $response = $this->checkCount();
            if ($response->getStatusCode() !== 200)
                return back()->with('error', $response->original['messages']);

            $this->saveData($request);
            return back()->with(['success' => __('extends.save_sucessfully')]);
        }

        $settings = Settings::join('badr_shop', 'badr_shop.serial_id', '=', 'einvoice_settings.shop_id')
            ->shop()
            ->first() ?? new Settings();

        $general_einvoice_settings = collect(DB::table('general_einvoice_settings')->get());
        $countries  = Country::all();
        $currencies = $countries->map(function ($currency) {
            return [
                'code'    => $currency['currency_code'],
                'name_en' => $currency['currency_name_en'],
                'name_ar' => $currency['currency_name_ar'],
                'symbol'  => $currency['currrency_symbol'],
            ];
        });
        return view('einvoice::settings.index', compact('settings', 'countries', 'currencies', 'general_einvoice_settings'));
    }

    public function checkCount()
    {
        $is_live = Settings::select('is_live')->first()->is_live;
        if (is_null($is_live) || $is_live == request()->is_live)
            return response()->json(['status' => true], 200);

        $pass = true;
        $messages = "لا يمكن تحويل بسبب وجود بيانات <br>";
        $check['einvoices']     = Einvoices::whereHas('invoice', function ($query) { $query->where('shop_id', auth()->user()->shop_id); })->count();
        $check['invoices']      = Invoice::where('shop_id', auth()->user()->shop_id)->count();
        $check['items']         = Item::where('shop_id', auth()->user()->shop_id)->count();
        $check['items_code']    = EinvoiceItemCode::where('shop_id', auth()->user()->shop_id)->count();

        if ($check['invoices'] > 0) {
            $pass = false;
            $messages .= "لديك {$check['invoices']} فاتورة <br> ";
        }
        if ($check['einvoices'] > 0) {
            $pass = false;
            $messages .= "لديك {$check['einvoices']} فاتورة إلكترونية <br> ";
        }
        if ($check['items'] > 0) {
            $pass = false;
            $messages .= "لديك {$check['items']} منتج <br> ";
        }
        if ($check['items_code'] > 0) {
            $pass = false;
            $messages .= "لديك {$check['items_code']} منتج له كود ضريبي <br> ";
        }
        $messages .= 'يرجي حذف كل البيانات الخاصة بهذا المود للتحويل';

        if ($pass) {
            return response()->json(['status' => true], 200);
        }
        return response()->json(['messages' => $messages], 500);
    }

    protected function saveData($request)
    {
        $rules = [
            'activity_id'      => ['required'],
            'branch_id'        => ['required'],
            'country_id'       => ['required'],
            'registration_id'  => ['required'],
            'company_name'     => ['required', 'string', 'max:255'],
            'client_id'        => ['required_if:is_live,0'],
            'client_secret'    => ['required_if:is_live,0'],
            'client_id_live'   => ['required_if:is_live,1'],
            'client_secret_live'=> ['required_if:is_live,1'],
            'is_live'          => ['required', 'in:0,1'],
            'governate'        => ['required'],
            'regionCity'       => ['required'],
            'street'           => ['required'],
            'buildingNumber'   => ['required'],
            'postalCode'       => ['required'],
            'floor'            => ['required'],
            'room'             => ['required'],
            'landmark'         => ['required'],
        ];

        $messages = [
            'activity_id.required'     => __('messages.activity_id.required'),
            'branch_id.required'       => __('messages.branch_id.required'),
            'country_id.required'      => __('messages.country_id.required'),
            'registration_id.required' => __('messages.registration_id.required'),
            'company_name.required'    => __('messages.company_name.required'),
            'company_name.string'      => __('messages.company_name.string'),
            'company_name.max'         => __('messages.company_name.max'),
            'client_id'                => __('messages.client_id.required_if'),
            'client_secret'            => __('messages.client_secret.required_if'),
            'client_id_live'           => __('messages.client_id_live.required_if'),
            'client_secret_live'       => __('messages.client_secret_live.required_if'),
            'governate'                => __('messages.required'),
            'regionCity'               => __('messages.required'),
            'street'                   => __('messages.required'),
            'buildingNumber'           => __('messages.required'),
            'postalCode'               => __('messages.required'),
            'floor'                    => __('messages.required'),
            'room'                     => __('messages.required'),
            'landmark'                 => __('messages.required'),
        ];

        $validated = $request->validate($rules, $messages);
        $country = Country::find($request->country_id);
        $validated['shop_id']     = get_session_val('serial_id', 'integer');
        $validated['currency_id'] = $country->currency_code;
        $validated['iso_alpha2']  = $country->iso_alpha2;

        Cache::set('is_live', $request->is_live);
        Settings::updateOrCreate(['shop_id' => $validated['shop_id']], $validated);
    }
}
