@extends('master::layouts.master')

@section('content')
    <div class="container-fluid">
        <div class="row">
            <h1 class="text-primary">{{ __('einvoice.all_einvoices') }}</h1>

            <form class="col-md-12 mt-2">
                <div class="form-row row">
                    <div class="form-group col-md-4">
                        <input type="text" name="search" id="search" class="form-control col-md-12"
                            placeholder="{{ __('einvoice.invoice_search') }}" value="{{ request('search') }}">
                    </div>
                    <div class="form-group col-md-3">
                        <select name="status" id="status" class="form-control selectpicker col-md-12"
                            title="{{ __('einvoice.all_status') }}">
                            @foreach ($statuses as $status => $statusName)
                                <option value="{{ $status }}" {{ request('status') == $status ? 'selected' : '' }}>
                                    {{ $statusName }}
                                </option>
                            @endforeach
                        </select>
                    </div>
                    <div class="form-group col-md-2">
                        <input type="text" name="date_from" id="date_from" class="form-control col-md-12 date"
                            placeholder="{{ __('einvoice.date_from') }}" value="{{ request('date_from') }}">
                    </div>
                    <div class="form-group col-md-2">
                        <input type="text" name="date_to" id="date_to" class="form-control col-md-12 date"
                            placeholder="{{ __('einvoice.date_to') }}" value="{{ request('date_to') }}">
                    </div>
                    <div class="form-group col-md-1 d-flex flex-column justify-content-end">
                        <button class="btn btn-primary" type="submit">{{ __('einvoice.search') }}</button>
                    </div>
                </div>
            </form>
        </div>

        <table class="table table-hover table-bordered table-striped bg-white" role="presentation">
            <thead>
                <tr class="info">
                    <th scope="col">#</th>
                    <th scope="col">{{ __('einvoice.invoice_id') }}</th>
                    <th scope="col">{{ __('einvoice.einvoice_id') }}</th>
                    <th scope="col">{{ __('einvoice.client_name') }}</th>
                    <th scope="col">{{ __('einvoice.typeName') }}</th>
                    <th scope="col">{{ __('einvoice.status') }}</th>
                    <th scope="col">{{ __('einvoice.invoice_date') }}</th>
                    <th scope="col">{{ __('einvoice.sync_date') }}</th>
                    <th scope="col">{{ __('extends.action') }}</th>
                </tr>
            </thead>
            <tbody>
                @notnull($invoices)
                    @foreach ($invoices as $invoice)
                        <tr>
                            <td>{{ request('page') == 1 || !request('page') ? $loop->iteration : $loop->iteration + $invoices->perPage() }}
                            </td>
                            <td>{{ $invoice['invoice']['bill_no'] ?? '---' }}</td>
                            <td>{{ $invoice['uuid'] }}</td>
                            <td>{{ $invoice['receiverName'] }}</td>
                            <td>{{ app()->isLocale('ar') ? $invoice['documentTypeNameSecondaryLang'] : $invoice['documentTypeNamePrimaryLang'] }}
                            </td>
                            <td class="{{ $invoice['status_color'] }} {{ $invoice['status'] }}"
                                data-uuid="{{ $invoice['uuid'] }}"><i class="{{ $invoice['status_icon'] }}"></i>
                                {{ $invoice['status_text'] }}</td>
                            <td>{{ $invoice['invoice']['invoice_date'] ?? now() }}</td>
                            <td>{{ date('Y-m-d H:m:s', strtotime($invoice['dateTimeIssued'])) }}</td>

                            @if (!in_array($invoice['uuid'], ['', null]))
                                <td>
                                    <a href="{{ $invoice['publicUrl'] }}" class="btn btn-info" target="_blank" rel="noopener noreferrer"
                                        title="{{ __('extends.show') }}">{{ __('عرض الفاتورة عند الضرائب') }}</a>

                                    <a href="{{ route('einvoice.documents.show', $invoice['uuid']) }}" class="btn btn-primary"
                                        title="{{ __('extends.show') }}">{{ __('extends.show') }}</a>

                                    <a href="{{ route('einvoice.documents.download', $invoice['uuid']) }}"
                                        class="btn btn-success" target="_blank" rel="noopener noreferrer"
                                        title="{{ __('extends.print') }}">{{ __('einvoice.download_pdfFile') }}</a>

                                    {{-- Reject Invoice --}}
                                    {{-- @if (now()->format('Y-m-d H:m:s') <= date('Y-m-d H:m:s', strtotime($invoice['canbeRejectedUntil'])) &&
                                        is_null($invoice['rejectRequestDate']))
                                        <a href="{{ route('einvoice.documents.reject', $invoice['uuid']) }}"
                                            class="btn btn-warning cancel_reject"
                                            title="{{ __('einvoice.reject') }}">{{ __('einvoice.reject') }}</a>
                                    @elseif (is_null($invoice['declineRejectRequestDate']))
                                        <a href="{{ route('einvoice.documents.declineReject', $invoice['uuid']) }}"
                                            class="btn btn-warning cancel_reject"
                                            title="{{ __('einvoice.decline_reject') }}">{{ __('einvoice.decline_reject') }}</a>
                                    @endif --}}

                                    {{-- Cancel Invoice --}}
                                    {{-- @if (now()->format('Y-m-d H:m:s') <= date('Y-m-d H:m:s', strtotime($invoice['canbeCancelledUntil'])) &&
                                        is_null($invoice['cancelRequestDate']))
                                        <a href="{{ route('einvoice.documents.cancel', $invoice['uuid']) }}"
                                            class="btn btn-danger cancel_reject"
                                            title="{{ __('einvoice.cancel') }}">{{ __('einvoice.cancel') }}</a>
                                    @elseif (is_null($invoice['declineCancelRequestDate']))
                                        <a href="{{ route('einvoice.documents.declineCancel', $invoice['uuid']) }}"
                                            class="btn btn-danger cancel_reject"
                                            title="{{ __('einvoice.decline_cancel') }}">{{ __('einvoice.decline_cancel') }}</a>
                                    @endif --}}
                                </td>
                            @endif
                        </tr>
                    @endforeach
                    @elsenull
                    <tr>
                        <td colspan="9" class="text-center">{{ __('einvoice.null_einvoice') }}</td>
                    </tr>
                @endnotnull
            </tbody>
        </table>

        {{-- Pagination --}}
        @notnull($invoices)
            <div class="row d-flex justify-content-center">{{ $invoices->appends(request()->query())->links() }}</div>
        @endnotnull
    </div>
@endsection

@section('js-scripts')
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>

    <script>
        $(document).ready(function() {
            $.ajaxSetup({
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                }
            });

            $('.selectpicker').selectpicker({
                style: 'btn-default',
                size: 8
            });

            $(".date").calendarsPicker({
                calendar: $.calendars.instance('gregorian'),
                dateFormat: 'Y-mm-dd',
                defaultDate: "0",
            });
        });
    </script>

    <script>
        $(function() {
            $(document).find('.Submitted').each(function() {
                var This = $(this);
                var text = This.text();
                var uuid = This.data('uuid');

                This.html("<i class='fa fa-spinner fa-spin'></i> {{ __('einvoice.load_status') }}");

                setTimeout(() => {
                    $.ajax({
                        type: "POST",
                        async: false,
                        url: "{{ route('einvoice.documents.updateStatus') }}",
                        data: {
                            '_token': '{{ csrf_token() }}',
                            'uuid': uuid
                        },
                        dataType: "json",
                        success: function(response) {
                            if (response.status == true) {
                                This.replaceWith(response.msg);
                            } else {
                                This.html(`<i class="fa fa-sync"></i> ${text}`);
                            }
                        }
                    });
                }, 2000);
            });
        });
    </script>
@endsection

@section('css-scripts')
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/bootstrap-datetimepicker-standalone.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/datepicker3.css') }}" rel="stylesheet">
@stop
