@extends('master::layouts.master')

@section('content')
    <div class="container-fluid">
        <h2 class="text-primary d-inline-block mb-3">
            {{ __('einvoice.einvoice_number', ['number' => $document->bill_no ?? '---']) }}</h2>

        {{-- If the document is invalid, an error message is displayed --}}
        @if (!is_null($document->validationErrors) && !empty($document->validationErrors))
            <div class="col-md-12">
                <ul class="alert alert-danger text-center list-unstyled">
                    @foreach ($document->validationErrors as $validationError)
                        <li>{{ $validationError }}</li>
                    @endforeach
                </ul>
            </div>
        @endif

        {{-- Issuer & Reciver --}}
        <div class="col-md-12 mb-2">
            <div class="row">
                <div class="form-group col-md-6">
                    <div class="bg-info text-white p-1 pr-2">{{ __('einvoice.issuer') }}</div>
                    <div class="bg-white p-2">
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('einvoice.name') }} :</div>
                            <div class="col-md-9">{{ $document->issuerName }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('einvoice.type') }} :</div>
                            <div class="col-md-9">{{ $document->issuer_role_txt }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('einvoice.branch_address') }} :</div>
                            <div class="col-md-9">
                                {{ $document->issuer_buildingNumber .
                                    ' ' .
                                    $document->issuer_street .
                                    ' ' .
                                    __('extends.floor_number') .
                                    ' ' .
                                    $document->issuer_floor .
                                    ' ' .
                                    __('extends.room_number') .
                                    ' ' .
                                    $document->issuer_room .
                                    ' ' .
                                    $document->issuer_regionCity .
                                    ' ' .
                                    $document->issuer_governate }}
                            </div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('einvoice.tax_code') }} :</div>
                            <div class="col-md-9">{{ $document->issuerId }}</div>
                        </div>
                    </div>
                </div>

                <div class="form-group col-md-6">
                    <div class="bg-info text-white p-1 pr-2">{{ __('einvoice.receiver') }}</div>
                    <div class="bg-white p-2">
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('einvoice.name') }} :</div>
                            <div class="col-md-9">{{ $document->receiverName }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('einvoice.type') }} :</div>
                            <div class="col-md-9">{{ $document->receiver_role_txt }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('einvoice.branch_address') }} :</div>
                            <div class="col-md-9">
                                {{ $document->receiver_buildingNumber .
                                    ' ' .
                                    $document->receiver_street .
                                    ' ' .
                                    __('extends.floor_number') .
                                    ' ' .
                                    $document->receiver_floor .
                                    ' ' .
                                    __('extends.room_number') .
                                    ' ' .
                                    $document->receiver_room .
                                    ' ' .
                                    $document->receiver_regionCity .
                                    ' ' .
                                    $document->receiver_governate }}
                            </div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">
                                {{ $document->receiverType == 0 ? __('einvoice.tax_code') : __('einvoice.national_id') }}
                                :</div>
                            <div class="col-md-9">{{ $document->receiverId }}</div>
                        </div>
                    </div>
                </div>
            </div>
        </div>

        {{-- Invoice Items --}}
        <div class="col-md-12">
            <table class="table table-hover table-bordered table-striped bg-white" role="presentation">
                <thead>
                    <tr class="bg-info text-white">
                        <th scope="col">{{ __('einvoice.item_name') }}</th>
                        <th scope="col">{{ __('einvoice.item_code') }}</th>
                        <th scope="col">{{ __('einvoice.item_description') }}</th>
                        <th scope="col">{{ __('einvoice.quantity') }}</th>
                        <th scope="col">{{ __('einvoice.unit_price') }}</th>
                    </tr>
                </thead>
                <tbody>
                    @foreach ($document->details as $item)
                        <tr>
                            <td>{{ app()->isLocale('ar') ? $item->itemSecondaryName : $item->itemPrimaryName }}
                            </td>
                            <td>{{ $item->itemCode }}</td>
                            <td>{{ $item->description }}</td>
                            <td>{{ $item->quantity }}</td>
                            <td>{{ $item->unitValue_amountEGP }}</td>
                        </tr>
                    @endforeach
                </tbody>
            </table>
        </div>

        {{-- Invoice Totals --}}
        <div class="col-md-12 mb-2">
            <div class="row">
                <div class="form-group col-md-8">
                    <div class="bg-info text-white p-1 pr-2">{{ __('einvoice.invoice_details') }}</div>
                    <div class="bg-white p-2">
                        <div class="form-row row p-1">
                            <div class="col-md-2 text-primary">{{ __('einvoice.einvoice_id') }} :</div>
                            <div class="col-md-10">{{ $document->uuid }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-2 text-primary">{{ __('einvoice.sync_date') }} :</div>
                            <div class="col-md-10">{{ date('Y-m-d H:m:s', strtotime($document->dateTimeIssued)) }}
                            </div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-2 text-primary">{{ __('einvoice.invoice_date') }} :</div>
                            <div class="col-md-10">{{ $document->invoice->invoice_date }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-2 text-primary">{{ __('einvoice.status') }} :</div>
                            <div class="col-md-10 {{ $document['status_color'] }}"><i
                                    class="{{ $document['status_icon'] }}"></i>
                                {{ $document['status_text'] }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-2 text-primary">{{ __('einvoice.invoice_link') }} :</div>
                            <a href="{{ $document['publicUrl'] }}" class="btn btn-primary"
                                target="_blanck">{{ __('einvoice.go_to_einvoice') }}</a>
                        </div>
                    </div>
                </div>
                <div class="col-md-4">
                    <div class="total">
                        <div class="bg-info text-white p-1 pr-2">{{ __('einvoice.invoice_totals') }}</div>
                        <div class="bg-white p-2">
                            <div class="form-row row p-1">
                                <div class="col-md-7 text-primary">{{ __('einvoice.total_sales') }} :</div>
                                <div class="col-md-5">{{ $document->totalSales }}</div>
                            </div>
                            <div class="form-row row p-1">
                                <div class="col-md-7 text-primary">{{ __('einvoice.total_discount') }} :</div>
                                <div class="col-md-5">{{ $document->totalDiscount }}</div>
                            </div>
                            <div class="form-row row p-1">
                                <div class="col-md-7 text-primary">{{ __('einvoice.non_taxable_items_discount') }} :
                                </div>
                                <div class="col-md-5">{{ $document->totalItemsDiscountAmount }}</div>
                            </div>
                            <div class="form-row row p-1">
                                <div class="col-md-7 text-primary">{{ __('einvoice.extra_invoice_discount') }} :</div>
                                <div class="col-md-5">{{ $document->extraDiscountAmount }}</div>
                            </div>
                            <div class="form-row row p-1">
                                <div class="col-md-7 text-primary">{{ __('einvoice.total_tax') }} :</div>
                                <div class="col-md-5">{{ $document->taxTotals }}</div>
                            </div>
                            <div class="form-row row p-1">
                                <div class="col-md-7 text-primary">{{ __('einvoice.total_amount') }} :</div>
                                <div class="col-md-5">{{ $document->total }}</div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
@endsection

@section('js-scripts')
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>
    <script src="{{ asset('tickets/js/bootstrap-select.min.js') }}"></script>

    <script>
        $(document).ready(function() {
            $.ajaxSetup({
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                }
            });
            $('.selectpicker').selectpicker({
                style: 'btn-default',
                size: 8
            });
        });
    </script>

    <script>
        $('.cancel_reject').on('click', function(e) {
            e.preventDefault();

            var url = $(this).attr('href');
            var reason = prompt("{{ __('einvoice.enter_reason') }} :");

            if (reason != null) {
                $.ajax({
                    type: "POST",
                    url: url,
                    data: {
                        reason: reason,
                        _token: '{{ csrf_token() }}'
                    },
                    dataType: "json",
                    success: function(response) {

                        // If response is cancel or decline cancel the invoice.
                        if (['cancel', 'declineCancel'].includes(response.type)) {
                            if (['array', 'object'].includes(typeof response.message)) {
                                $.each(response.message, function(index, value) {
                                    $('#messages').removeClass('hidden').empty().addClass(
                                        'alert alert-danger').append(value['message']);
                                });
                            } else if (response.code != null) {
                                $('#messages').removeClass('hidden').empty().addClass(
                                    'alert alert-danger').append(response.message);
                            } else {
                                $('#messages').removeClass('hidden').addClass('alert alert-success')
                                    .append(response.message);
                            }
                        }

                        // If response is reject or decline reject the invoice.
                        if (['reject', 'declineReject'].includes(response.type)) {
                            if (['array', 'object'].includes(typeof response.message)) {
                                $.each(response.message, function(index, value) {
                                    $('#messages').removeClass('hidden').empty().addClass(
                                        'alert alert-danger').append(value['message']);
                                });
                            } else if (response.code != null) {
                                $('#messages').removeClass('hidden').empty().addClass(
                                    'alert alert-danger').append(response.message);
                            } else {
                                $('#messages').removeClass('hidden').addClass('alert alert-success')
                                    .append(response.message);
                            }
                        }
                    }
                });
            }
        });
    </script>
@endsection

@section('css-scripts')
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/bootstrap-datetimepicker-standalone.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/datepicker3.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/css/bootstrap-select.min.css') }}" rel="stylesheet">
@stop
