@extends('master::layouts.master')

@section('content')
    <div class="container-fluid">
        <div class="row">
            <h1 class="text-primary">{{ __('einvoice.itemcodes.title') }}</h1>

            <form class="col-md-12 mt-2">
                <div class="form-row row">

                    {{-- Search Input --}}
                    <div class="form-group col-md-3">
                        <input type="text" name="search" id="search" class="form-control col-md-12"
                            placeholder="{{ __('einvoice.itemcodes.search') }}" value="{{ request('search') }}">
                    </div>

                    {{-- Status of item codes --}}
                    <div class="form-group col-md-2">
                        <select name="status" id="select_status" class="form-control selectpicker col-md-12 status"
                            title="{{ __('einvoice.itemcodes.select_status') }}">
                            <option value="Submitted" @if (request('status') == 'Submitted') selected @endif>
                                {{ __('einvoice.wait_for_approval') }}</option>
                            <option value="Approved" @if (request('status') == 'Approved') selected @endif>
                                {{ __('einvoice.Approved') }}</option>
                            <option value="Rejected" @if (request('status') == 'Rejected') selected @endif>
                                {{ __('einvoice.Rejected') }}</option>
                        </select>
                    </div>

                    {{-- Active of item codes --}}
                    <div class="form-group col-md-2">
                        <select name="active" id="select_active" class="form-control selectpicker col-md-12 active"
                            title="{{ __('einvoice.itemcodes.select_active') }}">
                            <option value="1" @if (request('active') == "1") selected @endif>
                                {{ __('einvoice.itemcodes.active') }}</option>
                            <option value="0" @if (request('active') == "0") selected @endif>
                                {{ __('einvoice.itemcodes.NotActive') }}</option>
                        </select>
                    </div>

                    {{-- Active From --}}
                    <div class="form-group col-md-2">
                        <input type="text" name="ActiveFrom" id="ActiveFrom" class="form-control col-md-12 date"
                            placeholder="{{ __('einvoice.date_from') }}" value="{{ request('ActiveFrom') }}">
                    </div>

                    {{-- Active To --}}
                    <div class="form-group col-md-2">
                        <input type="text" name="ActiveTo" id="ActiveTo" class="form-control col-md-12 date"
                            placeholder="{{ __('einvoice.date_to') }}" value="{{ request('ActiveTo') }}">
                    </div>

                    {{-- Submit --}}
                    <div class="form-group col-md-1">
                        <button class="btn btn-primary" type="submit">{{ __('einvoice.search') }}</button>
                    </div>
                </div>
            </form>
        </div>
    </div>

    <table class="table table-hover table-bordered table-striped bg-white" role="presentation">
        <thead>
            <tr class="info">
                <th scope="col">#</th>
                <th scope="col">{{ __('einvoice.itemcodes.codeTypeName') }}</th>
                <th scope="col">{{ __('einvoice.itemcodes.itemCode') }}</th>
                <th scope="col">{{ __('einvoice.itemcodes.codeName') }}</th>
                <th scope="col">{{ __('einvoice.itemcodes.description') }}</th>
                <th scope="col">{{ __('einvoice.itemcodes.parentCodeID') }}</th>
                <th scope="col">{{ __('einvoice.itemcodes.parentCodeName') }}</th>
                <th scope="col">{{ __('einvoice.itemcodes.activeFrom') }}</th>
                <th scope="col">{{ __('einvoice.itemcodes.activeTo') }}</th>
                <th scope="col">{{ __('einvoice.itemcodes.active') }}</th>
                <th scope="col">{{ __('einvoice.itemcodes.status') }}</th>
                <th scope="col">{{ __('einvoice.itemcodes.ownerTaxpayer.name') }}</th>
                <th scope="col">{{ __('extends.action') }}</th>
            </tr>
        </thead>
        <tbody>
            @notnull($itemCodes)
                @foreach ($itemCodes as $itemcode)
                    <tr>
                        <td>{{ request('page') == 1 || !request('page') ? $loop->iteration : $loop->iteration + $itemCodes->perPage() }}
                        </td>
                        <td>{{ $itemcode['codeTypeName'] ?? '---' }}</td>
                        <td>{{ $itemcode['itemCode'] ?? '---' }}</td>
                        <td>{{ app()->isLocale('ar') ? $itemcode['codeNameSecondaryLang'] ?? '---' : $itemcode['codeNamePrimaryLang'] ?? '---' }}
                        </td>
                        <td>{{ app()->isLocale('ar') ? $itemcode['descriptionSecondaryLang'] ?? '---' : $itemcode['descriptionPrimaryLang'] ?? '---' }}
                        </td>
                        <td>{{ $itemcode['parentItemCode'] ?? '---' }}</td>
                        <td>{{ app()->isLocale('ar') ? $itemcode['parentCodeNameSecondaryLang'] ?? '---' : $itemcode['parentCodeNamePrimaryLang'] ?? '---' }}
                        </td>
                        <td>{{ in_array($itemcode['activeFrom'], ['', ' ', null]) ? '---' : date('Y-m-d H:m:s', strtotime($itemcode['activeFrom'])) }}
                        </td>
                        <td>{{ in_array($itemcode['activeTo'], ['', ' ', null]) ? '---' : date('Y-m-d H:m:s', strtotime($itemcode['activeTo'])) }}
                        </td>
                        <td class="{{ $itemcode['active_color'] }}">{{ $itemcode['active_text'] }}</td>
                        <td class="{{ $itemcode['status_color'] }} {{ $itemcode['status'] }}"
                            data-itemcode="{{ $itemcode['itemCode'] }}"><i class="{{ $itemcode['status_icon'] }}"></i>
                            {{ $itemcode['status_text'] }}</td>
                        <td class="taxpayer">{{ app()->isLocale('ar') ? $itemcode['ownerTaxpayer_nameAr'] ?? '---' : $itemcode['ownerTaxpayer_name'] ?? '---' }}
                        </td>
                        <td>
                            <a href="{{ route('einvoice.itemcodes.edit', $itemcode['itemCode']) }}"
                                class="btn btn-primary edit">{{ __('einvoice.itemcodes.edit') }}</a>
                        </td>
                    </tr>
                @endforeach
                @elsenull
                <tr>
                    <td colspan="14" class="text-center">{{ __('einvoice.itemcodes.null') }}</td>
                </tr>
            @endnotnull
        </tbody>
    </table>

    {{-- Pagination --}}
    @notnull($itemCodes)
        <div class="text-center">{{ $itemCodes->links() }}</div>
    @endnotnull
    </div>
@endsection

@section('js-scripts')
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>
    <script src="{{ asset('tickets/js/bootstrap-select.min.js') }}"></script>

    <script>
        $(document).ready(function() {
            $.ajaxSetup({
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                }
            });

            $('.selectpicker').selectpicker({
                style: 'btn-default',
                size: 8
            });

            $(".date").calendarsPicker({
                calendar: $.calendars.instance('gregorian'),
                dateFormat: 'Y-mm-dd',
                defaultDate: "0",
            });
        });
    </script>

    <script>
        $(function() {
            $(document).find('.Submitted').each(function() {
                var This = $(this);
                var text = This.text();
                var itemCode = This.data('itemcode');

                This.html("<i class='fa fa-spinner fa-spin'></i> {{ __('einvoice.load_status') }}");

                setTimeout(() => {
                    $.ajax({
                        type: "POST",
                        async: false,
                        url: "{{ route('einvoice.itemcodes.updateCode') }}",
                        data: {
                            '_token': '{{ csrf_token() }}',
                            'itemCode': itemCode,
                        },
                        dataType: "json",
                        success: function(response) {
                            if (response.status == true) {
                                This.replaceWith(response.msg);
                                $('.taxpayer').replaceWith(response.taxpayer);
                            } else {
                                This.html(`<i class="fa fa-sync"></i> ${text}`);
                            }
                        }
                    });
                }, 2000);
            });
        });
    </script>
@endsection

@section('css-scripts')
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/bootstrap-datetimepicker-standalone.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/datepicker3.css') }}" rel="stylesheet">

    <style>
        input:disabled {
            cursor: context-menu !important;
        }
    </style>
@stop
