@extends('master::layouts.master')

@section('content')
    <div class="container-fluid">
        <div class="row">
            <h1 class="text-primary">{{ __('einvoice.notifications.title') }}</h1>
            <form class="col-md-12 mt-2 mb-3">
                <div class="form-row row">
                    <div class="col-md-3">
                        <input type="text" name="search" id="search" class="form-control col-md-12"
                            placeholder="{{ __('einvoice.notifications.search') }}" value="{{ request('search') }}">
                    </div>
                    <div class="col-md-2">
                        <select name="type" id="type" class="form-control selectpicker col-md-12" data-none-selected-text="{{ trans('native.no-selected') }}"
                            title="{{ __('einvoice.notifications.type.choose') }}">
                            <option value="1" @if (request('type') == 1) selected @endif>
                                {{ __('einvoice.notifications.type.delegationInvite') }}</option>
                            <option value="2" @if (request('type') == 2) selected @endif>
                                {{ __('einvoice.notifications.type.OTP') }}</option>
                            <option value="3" @if (request('type') == 3) selected @endif>
                                {{ __('einvoice.notifications.type.ProfileDataValidation') }}
                            </option>
                            <option value="4" @if (request('type') == 4) selected @endif>
                                {{ __('einvoice.notifications.type.GenericNotification') }}
                            </option>
                            <option value="5" @if (request('type') == 5) selected @endif>
                                {{ __('einvoice.notifications.type.ReceiveDownloadReady') }}
                            </option>
                            <option value="6" @if (request('type') == 6) selected @endif>
                                {{ __('einvoice.notifications.type.DocumentReceived') }}</option>
                            <option value="7" @if (request('type') == 7) selected @endif>
                                {{ __('einvoice.notifications.type.DocumentValidated') }}</option>
                            <option value="8" @if (request('type') == 8) selected @endif>
                                {{ __('einvoice.notifications.type.DocumentCancelled') }}</option>
                            <option value="9" @if (request('type') == 9) selected @endif>
                                {{ __('einvoice.notifications.type.DocumentRejected') }}</option>
                        </select>
                    </div>
                    <div class="col-md-2">
                        <select name="status" id="status" class="form-control selectpicker col-md-12" data-none-selected-text="{{ trans('native.no-selected') }}"
                            title="{{ __('einvoice.notifications.status.choose') }}">
                            <option value="pending" @if (request('status') == 'pending') selected @endif>
                                {{ __('einvoice.notifications.status.pending') }}</option>
                            <option value="batched" @if (request('status') == 'batched') selected @endif>
                                {{ __('einvoice.notifications.status.batched') }}</option>
                            <option value="delivered" @if (request('status') == 'delivered') selected @endif>
                                {{ __('einvoice.notifications.status.delivered') }}</option>
                            <option value="error" @if (request('status') == 'error') selected @endif>
                                {{ __('einvoice.notifications.status.error') }}</option>
                        </select>
                    </div>
                    <div class="col-md-2">
                        <select name="channel" id="channel" class="form-control selectpicker col-md-12" data-none-selected-text="{{ trans('native.no-selected') }}"
                            title="{{ __('einvoice.notifications.channel.choose') }}">
                            <option value="sms" @if (request('channel') == 'sms') selected @endif>
                                {{ __('einvoice.notifications.channel.sms') }}</option>
                            <option value="email" @if (request('channel') == 'email') selected @endif>
                                {{ __('einvoice.notifications.channel.email') }}</option>
                            <option value="push" @if (request('channel') == 'push') selected @endif>
                                {{ __('einvoice.notifications.channel.push') }}</option>
                            <option value="system" @if (request('channel') == 'system') selected @endif>
                                {{ __('einvoice.notifications.channel.system') }}</option>
                        </select>
                    </div>
                    <div class="col-md-1">
                        <input type="text" name="dateFrom" id="dateFrom" class="form-control col-md-12 date"
                            placeholder="{{ __('einvoice.date_from') }}" value="{{ request('dateFrom') }}">
                    </div>
                    <div class="col-md-1">
                        <input type="text" name="dateTo" id="dateTo" class="form-control col-md-12 date"
                            placeholder="{{ __('einvoice.date_to') }}" value="{{ request('dateTo') }}">
                    </div>
                    <div class="col-md-1">
                        <button type="submit" class="btn btn-primary">{{ __('einvoice.search') }}</button>
                    </div>
                </div>
            </form>
        </div>
    </div>

    <table class="table table-hover table-bordered table-striped bg-white" role="presentation">
        <thead>
            <tr class="info">
                <th scope="col">#</th>
                <th scope="col">{{ __('einvoice.notifications.notificationId') }}</th>
                <th scope="col">{{ __('einvoice.notifications.typeId') }}</th>
                <th scope="col">{{ __('einvoice.notifications.finalMessage') }}</th>
                <th scope="col">{{ __('einvoice.notifications.typeName') }}</th>
                <th scope="col">{{ __('einvoice.notifications.eamil_address') }}</th>
                <th scope="col">{{ __('einvoice.notifications.attemptDateTime') }}</th>
                <th scope="col">{{ __('einvoice.notifications.receivedDateTime') }}</th>
                <th scope="col">{{ __('einvoice.notifications.deliveredDateTime') }}</th>
            </tr>
        </thead>
        <tbody>
            @notnull($notifications)
                @foreach ($notifications as $notification)
                    <tr>
                        <td>{{ request('page') == 1 || !request('page') ? $loop->iteration : $loop->iteration + $notifications->perPage() }}
                        </td>
                        <td>{{ $notification['notificationId'] }}</td>
                        <td>{{ $notification['typeId'] }}</td>
                        <td>{{ $notification['finalMessage'] }}</td>
                        <td>{{ $notification['typeName'] }}</td>
                        <td>{{ $notification['address'] }}</td>
                        <td>{{ collect($notifications[0]['deliveryAttempts'])->pluck('attemptDateTime')->implode(',') }}</td>
                        <td>{{ $notification['receivedDateTime'] }}</td>
                        <td>{{ $notification['deliveredDateTime'] }}</td>
                    </tr>
                @endforeach
                @elsenull
                <tr>
                    <td colspan="10" class="text-center">{{ __('einvoice.notifications.null') }}</td>
                </tr>
            @endnotnull
        </tbody>
    </table>

    {{-- Pagination --}}
    @notnull($notifications)
        <div class="row d-flex justify-content-center">{{ $notifications->links() }}</div>
    @endnotnull
    </div>
@endsection

@section('js-scripts')
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>
    <script src="{{ asset('tickets/js/bootstrap-select.min.js') }}"></script>

    <script>
        $(document).ready(function() {
            $.ajaxSetup({
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                }
            });

            $('.selectpicker').selectpicker({
                style: 'btn-default',
                size: 8
            });
        });
    </script>
@endsection

@section('css-scripts')
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/bootstrap-datetimepicker-standalone.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/datepicker3.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/css/bootstrap-select.min.css') }}" rel="stylesheet">

    <style>
        .filter-option,
        .filter-option-inner {
            padding: 0 !important;
            margin: 0 !important;
        }
    </style>
@stop
