@extends('master::layouts.master')

@section('content')
    <div class="container-fluid">
        <div class="row">
            <h1 class="text-primary">
                {{ __('einvoice.packageRequests.add') }}
            </h1>

            @include('master::layouts.messages')

            <form class="col-md-12" method="POST">
                @csrf

                {{-- Package Type & Format --}}
                <div class="form-row row">
                    <div class="form-group col-md-6">
                        <label for="type" class="control-label">{{ __('einvoice.packageRequests.type') }}
                            :</label>
                        <select name="type" id="type" class="form-control selectpicker col-md-12"
                            title="{{ __('einvoice.packageRequests.select_type') }}" required>
                            <option value="Full">{{ __('einvoice.packageRequests.full') }}</option>
                            <option value="Summary">{{ __('einvoice.packageRequests.summary') }}</option>
                        </select>
                    </div>
                    <div class="form-group col-md-6">
                        <label class="control-label" for="format">{{ __('einvoice.packageRequests.format') }}
                            :</label>
                        <select name="format" id="format" class="form-control selectpicker col-md-12"
                            title="{{ __('einvoice.packageRequests.select_format') }}" required>
                            <option value="CSV">CSV</option>
                            <option value="JSON">JSON</option>
                            <option value="XML">XML</option>
                        </select>
                    </div>
                </div>

                {{-- Document Type Names && Document Statuses --}}
                <div class="form-row row">
                    <div class="form-group col-md-6">
                        <label class="control-label"
                            for="documentTypeNames">{{ __('einvoice.packageRequests.documentTypeNames') }}
                            :</label>
                        <select name="documentTypeNames" id="documentTypeNames" class="form-control selectpicker col-md-12"
                            title="{{ __('einvoice.packageRequests.select_documentTypeNames') }}" multiple required>
                            @foreach ($documentTypes as $documentType)
                                <option value="{{ $documentType['name'] }}">{{ $documentType['description'] }}</option>
                            @endforeach
                        </select>
                    </div>
                    <div class="form-group col-md-6">
                        <label class="control-label" for="statuses">{{ __('einvoice.packageRequests.statuses') }}
                            :</label>
                        <select name="statuses" id="statuses" class="form-control selectpicker col-md-12"
                            title="{{ __('einvoice.packageRequests.select_statuses') }}" multiple required>
                            <option value="Valid">{{ __('einvoice.Valid') }}</option>
                            <option value="Invalid">{{ __('einvoice.Invalid') }}</option>
                            <option value="Rejected">{{ __('einvoice.Rejected') }}</option>
                            <option value="Cancelled">{{ __('einvoice.Cancelled') }}</option>
                        </select>
                    </div>
                </div>

                {{-- Date from & Date to --}}
                <div class="form-row row">
                    <div class="form-group col-md-6">
                        <label for="dateFrom" class="control-label">{{ __('einvoice.date_from') }} :</label>
                        <input type="text" name="dateFrom" id="dateFrom" class="form-control col-md-12 date" required>
                    </div>
                    <div class="form-group col-md-6">
                        <label for="dateTo" class="control-label">{{ __('einvoice.date_to') }} :</label>
                        <input type="text" name="dateTo" id="dateTo" class="form-control col-md-12 date" required>
                    </div>
                </div>

                {{-- Receiver Sender Id && Branch Number --}}
                <div class="form-row row">
                    <div class="form-group col-md-6">
                        <label for="receiverSenderId" class="control-label d-block">{{ __('einvoice.packageRequests.receiverSenderId') }} :</label>
                        <input type="number" name="receiverSenderId" id="receiverSenderId" class="form-control col-md-6 d-inline-block">
                        <select name="receiverSenderType" id="receiverSenderType" class=" form-control selectpicker col-md-5"
                            title="{{ __('einvoice.packageRequests.select_receiverSenderType') }}">
                            <option value="0">{{ __('einvoice.packageRequests.supplier') }}</option>
                            <option value="1">{{ __('einvoice.packageRequests.customer') }}</option>
                            <option value="2">{{ __('einvoice.packageRequests.foreign') }}</option>
                        </select>
                    </div>
                    <div class="form-group col-md-6">
                        <label for="branchNumber" class="control-label">{{ __('einvoice.packageRequests.branchNumber') }}
                            :</label>
                        <input type="number" name="branchNumber" id="branchNumber" class="form-control col-md-12">
                    </div>
                </div>

                {{-- Code Value && Code Type --}}
                <div class="form-row row">
                    <div class="form-group col-md-6" id="codeinput">
                        <label for="codeValue" class="control-label">{{ __('einvoice.packageRequests.codeValue') }} :</label>
                        <input type="text" name="codeValue" id="codeValue" class="form-control col-md-12">
                    </div>
                    <div class="form-group col-md-6">
                        <label for="codeType" class="control-label d-block">{{ __('einvoice.packageRequests.codeType') }} :</label>
                        <select name="codeType" id="codeType" class="col-md-5 form-control selectpicker"
                            title="{{ __('einvoice.packageRequests.select_codeType') }}">
                            <option value="EGS">EGS</option>
                            <option value="GS1">GS1</option>
                        </select>
                        <button class="btn btn-info pull-left" id="add_code"><i class="fa fa-plus"></i>
                            {{ __('einvoice.add_code') }}</button>
                    </div>
                </div>

                {{-- Submit --}}
                <div class="form-row row">
                    <button type="submit" class="btn btn-primary pull-left"
                        id="save">{{ __('extends.submit') }}</button>
                </div>

                {{-- Table to append codes in it --}}
                <table class="table table-hover table-bordered table-striped bg-white mt-2 hidden" id="codestable" role="presentation">
                    <thead>
                        <tr class="info">
                            <th class="text-center">{{ __('einvoice.packageRequests.codeValue') }}</th>
                            <th class="text-center">{{ __('einvoice.packageRequests.codeType') }}</th>
                            <th class="text-center">{{ __('extends.action') }}</th>
                        </tr>
                    </thead>
                    <tbody></tbody>
                </table>
            </form>
        </div>
    </div>
@endsection

@section('js-scripts')
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>
    <script src="{{ asset('tickets/js/bootstrap-select.min.js') }}"></script>

    <script>
        $(document).ready(function() {
            $.ajaxSetup({
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                }
            });

            $('.selectpicker').selectpicker({
                style: 'btn-default',
                size: 8
            });

            $(".date").calendarsPicker({
                calendar: $.calendars.instance('gregorian'),
                dateFormat: 'Y-mm-dd',
                defaultDate: "0",
            });
        });
    </script>

    <script>
        // Disable Csv Option if type is full.
        $('#type').on('change', function() {
            if ($(this).val() == 'Full'){
                $("#format option[value='CSV']").attr('disabled', 'disabled');
                $('#format').selectpicker('refresh');
            }else {
                $("#format option[value='CSV']").removeAttr('disabled');
                $('#format').selectpicker('refresh');
            }
        });

        // Add code to table.
        $(document).on('click', '#add_code', function(e) {
            e.preventDefault();

            var itemCodes = [];
            var codeType = $("#codeType").val();
            var codeValue = $("#codeValue").val();

            if (!['', ' ', null].includes(codeValue) && !['', ' ', null].includes(codeType)) {
                if ((codeType == 'EGS' && codeValue.indexOf('EG') != -1) || codeType == 'GS1') {
                    $("#codestable tbody").append(`<tr id="tr-${codeValue}">
                    <td class="text-center">${codeValue}</td>
                    <td class="text-center">${codeType}</td>
                    <td class="text-center"><button class="btn btn-danger btn-sm remove_code" data-codeValue="${codeValue}">
                        <i class="fa fa-trash"></i></button></td>
                    </tr>`);

                    itemCodes.push({
                        codeValue: codeValue,
                        codeType: codeType
                    });

                    $("#codestable tbody").append(
                        `<input type="hidden" name="itemCodes[]" value='${JSON.stringify(itemCodes)}'>`);

                    $("#codeValue").val('');
                    $("#egs_error").remove();
                    $("#codeType").selectpicker('deselectAll');
                    $("#codeValue").removeClass('invalid');
                    $("#codestable").removeClass('hidden');
                } else {
                    $("#codeValue").addClass('invalid');

                    if (!$("#egs_error").length) {
                        $("#codeValue").after(
                            '<small class="col-md-offset-3 col-md-12 text-danger" id="egs_error">يجب ان يبدأ الكود ب EG</small>'
                        );
                    }
                }
            }
        });

        // Remove code from table.
        $(document).on('click', '.remove_code', function(e) {
            e.preventDefault();

            var codeValue = $(this).data('codevalue');
            $(`#tr-${codeValue}`).remove();

            if ($("#codestable tbody tr").length == 0){
                $("#codestable").addClass('hidden');
            }
        });
    </script>
@endsection

@section('css-scripts')
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/bootstrap-datetimepicker-standalone.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/datepicker3.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/css/bootstrap-select.min.css') }}" rel="stylesheet">
@stop
