@extends('master::layouts.master')

@section('content')
    <div class="container-fluid">
        <div class="row mb-3 align-items-center">
            <h1 class="text-primary d-inline-block">{{ __('einvoice.packageRequests.title') }}</h1>
            <a href="{{ route('einvoice.packageRequests.addRequestPackage') }}"
                class="btn btn-primary h-100 mr-auto">
                <i class="fa fa-plus"></i> {{ __('einvoice.packageRequests.add') }}</a>
        </div>
    </div>

    <table class="table table-hover table-bordered table-striped bg-white" role="presentation">
        <thead>
            <tr class="info">
                <th scope="col">#</th>
                <th scope="col">{{ __('einvoice.packageRequests.packageId') }}</th>
                <th scope="col">{{ __('einvoice.packageRequests.status') }}</th>
                <th scope="col">{{ __('einvoice.packageRequests.type') }}</th>
                <th scope="col">{{ __('einvoice.packageRequests.format') }}</th>
                <th scope="col">{{ __('einvoice.packageRequests.requestorTypeId') }}</th>
                <th scope="col">{{ __('einvoice.packageRequests.requestorTaxpayerRIN') }}</th>
                <th scope="col">{{ __('einvoice.packageRequests.requestorTaxpayerName') }}</th>
                <th scope="col">{{ __('einvoice.packageRequests.submissionDate') }}</th>
                <th scope="col">{{ __('einvoice.packageRequests.deletionDate') }}</th>
                <th scope="col">{{ __('einvoice.packageRequests.isExpired') }}</th>
                <th scope="col">{{ __('extends.action') }}</th>
            </tr>
        </thead>
        <tbody>
            @notnull($packages)
                @foreach ($packages as $package)
                    <tr>
                        <td>{{ request('page') == 1 || !request('page') ? $loop->iteration : $loop->iteration + $packages->perPage() }}
                        </td>
                        <td>{{ $package['packageId'] }}</td>
                        <td>{{ $package['status'] == 1
                            ? __('einvoice.packageRequests.inProgress')
                            : ($package['status'] == 2
                                ? __('einvoice.packageRequests.complete')
                                : ($package['status'] == 3
                                    ? __('einvoice.packageRequests.error')
                                    : __('einvoice.packageRequests.deleted'))) }}
                        </td>
                        <td>{{ $package['type'] == 1 ? __('einvoice.packageRequests.full') : __('einvoice.packageRequests.summary') }}
                        </td>
                        <td>{{ $package['format'] == 1 ? 'CSV' : ($package['format'] == 2 ? 'XML' : 'JSON') }}</td>
                        <td>{{ $package['requestorTypeId'] == 1
                            ? __('einvoice.packageRequests.intermediary')
                            : ($package['requestorTypeId'] == 2
                                ? __('einvoice.packageRequests.taxpayer')
                                : ($package['requestorTypeId'] == 3
                                    ? __('einvoice.packageRequests.internalUser')
                                    : __('einvoice.packageRequests.ERP'))) }}
                        </td>
                        <td>{{ $package['requestorTaxpayerRIN'] }}</td>
                        <td>{{ $package['requestorTaxpayerName'] }}</td>
                        <td>{{ date('Y-m-d H:m:s', strtotime($package['submissionDate'])) }}</td>
                        <td>{{ in_array($package['deletionDate'], ['', ' ', null]) ? '---' : date('Y-m-d H:m:s', strtotime($package['deletionDate'])) }}
                        </td>
                        <td>{{ $package['isExpired'] == 1 ? __('einvoice.yes') : __('einvoice.no') }}</td>
                        <td>
                            <a href="{{ route('einvoice.packageRequests.download', $package['packageId']) }}"
                                class="btn btn-success" title="{{ __('extends.print') }}">{{ __('extends.download') }}</a>
                        </td>
                    </tr>
                @endforeach
                @elsenull
                <tr>
                    <td class="text-center" colspan="12">{{ __('einvoice.packageRequests.null') }}</td>
                </tr>
            @endnotnull
        </tbody>
    </table>

    {{-- Pagination --}}
    @notnull($packages)
        <div class="row justify-content-center">{{ $packages->links() }}</div>
    @endnotnull
    </div>
@endsection

@section('js-scripts')
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>
    <script src="{{ asset('tickets/js/bootstrap-select.min.js') }}"></script>

    <script>
        $(document).ready(function() {
            $.ajaxSetup({
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                }
            });

            $('.selectpicker').selectpicker({
                style: 'btn-default',
                size: 8
            });
        });
    </script>
@endsection

@section('css-scripts')
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/bootstrap-datetimepicker-standalone.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/datepicker3.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/css/bootstrap-select.min.css') }}" rel="stylesheet">
@stop
