@extends('master::layouts.master')

@section('content')
    <div class="container-fluid">
        <h1 class="text-primary">{{ __('einvoice.einvoice_settings') }}</h1>

        <form class="col-md-12 mt-3" method="POST" id="einvoice-form" action="{{ route('einvoice.settings.index') }}">
            {{ csrf_field() }}

            {{-- Company Name && Activity --}}
            <div class="form-row row">
                <div class="form-group col-md-6">
                    <label for="company_name" class="control-label">{{ __('einvoice.company_name') }}
                        :</label>
                    <input type="text" name="company_name" id="company_name" class="form-control col-md-12"
                           value="{{ $settings ? ( $settings->company_name != '' ? $settings->company_name : $settings->ower_name ) : '' }}">
                </div>
                <div class="form-group col-md-6">
                    <label for="activity_id" class="control-label">{{ __('einvoice.activity') }}:</label>
                    <select class="form-control selectpicker col-md-12" name="activity_id"
                            title="{{ __('einvoice.choose_activity') }}" data-live-search="true">
                        @foreach (config('einvoice.activitycodes') as $activity)
                            <option value="{{ $activity['code'] }}"
                                @if ($settings && $settings->activity_id == $activity['code']) selected @endif>
                                {{ App::isLocale('ar') ? $activity['Desc_ar'] : $activity['Desc_en'] }}</option>
                        @endforeach
                    </select>
                </div>
            </div>

            {{-- Country & Currency --}}
            <div class="form-row row">
                <div class="form-group col-md-6">
                    <label for="country_id" class="control-label">{{ __('einvoice.country') }} :</label>
                    <select class="form-control selectpicker col-md-12" name="country_id" id="countries"
                            title="{{ __('einvoice.choose_country') }}" data-live-search="true">
                        @foreach ($countries as $country)
                            <option value="{{ $country->id }}" data-currency_code="{{ $country->currency_code }}"
                                    data-currency_name="{{ App::isLocale('ar') ? $country->currency_name_ar : $country->currency_name_en }}"
                                    @if ($settings && $settings->country_id == $country->id) selected @endif>
                                {{ App::isLocale('ar') ? $country->name_ar : $country->name_en }}</option>
                        @endforeach
                    </select>
                </div>
                <div class="form-group col-md-6">
                    <label for="currency_id" class="control-label">{{ __('einvoice.currency') }} :</label>
                    <input type="hidden" name="currency_id" id="currency_id" class="form-control col-md-12"
                           value="{{ $settings->currency_id ?? '' }}">
                    <input type="text" name="currency_text" id="currency_text" class="form-control col-md-12"
                           value="{{ $settings->currency_text ?? '' }}" disabled>
                </div>
            </div>

            {{-- Tax Number & Branch --}}
            <div class="form-row row">
                <div class="form-group col-md-6">
                    <label for="registration_id" class="control-label">{{ __('einvoice.registration_id') }}
                        :</label>
                    <div class="col-md-12" style="padding: 0; margin:0">
                        <input type="number" name="registration_id" id="registration_id" class="form-control"
                               value="{{ $settings->registration_id ?? '' }}">
                        <small>{{ __('einvoice.invoice_note') }}</small>
                    </div>
                </div>
                <div class="form-group col-md-6">
                    <label for="branch_id" class="control-label">{{ __('einvoice.branch_code') }} :</label>
                    <div class="col-md-12" style="padding: 0; margin:0">
                        <input type="number" name="branch_id" id="branch_id" class="form-control"
                               value="{{ $settings->branch_id ?? '' }}">
                        <small>{{ __('einvoice.invoice_note') }}</small>
                    </div>
                </div>
            </div>

            @include('einvoice::settings.inputs')


            {{-- Client ID && Client Secret --}}
            @include('einvoice::settings.mode-section')

            <button type="submit" class="btn btn-primary float-left">{{ __('extends.save') }}</button>

            <button class="btn btn-warning text-white" type="button" id="sync-token">
                <i class="fa fa-sync"></i> تحديث الختم
            </button>
        </form>
    </div>
@endsection

@section('js-scripts')
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>
    <script src="{{ asset('tickets/js/bootstrap-select.min.js') }}"></script>

    <script>
        $(document).ready(function() {
            $('[data-mode="test"]').slideUp(0);
            $.ajaxSetup({
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                }
            });
            $('.selectpicker').selectpicker({
                style: 'btn-default',
                size: 8
            });

            $('body').on('change', 'input[name="is_live"]', function () {
                let [current, target] = $(this).val() == 1 ? ['test', 'live'] : ['live', 'test'];
                $(`[data-mode="${current}"]`).fadeOut(function () { $(`[data-mode="${target}"]`).fadeIn(200); });
            });

            $('input[name="is_live"]:checked').change();

            $('button[type="submit"]').on('click', function (e) {
                e.preventDefault();
                checkHasEinvoiceWhenChangeSettingMode();
            });

            function checkHasEinvoiceWhenChangeSettingMode() {
                $('#show-mode-error-message').html('');
                $.ajax({
                    url: '{{ route("einvoice.settings.check.count") }}',
                    type: 'get',
                    data: {is_live: $('input[name="is_live"]:checked').val()},
                    success: () => $('#einvoice-form').submit(),
                    error: (response) => $('#show-mode-error-message').html(`<div class='alert alert-danger '>${response.responseJSON.messages}</div>`).removeClass('hidden'),
                });
            }
        });
    </script>

    <script>
        $("#countries").change(function() {
            var currency_code = $('#countries option:selected').data('currency_code');
            var currency_text = $('#countries option:selected').data('currency_name');

            $('#currency_id').val(currency_code);
            $('#currency_text').val(currency_text);
        });

        checkExtension();

        $('body').on('click', '#sync-token', function() {
            if (! checkExtension()) return true;

            new Promise(function(resolve, reject){
                SignerDigital.getSelectedCertificate().then(function (signDataResp) {
                        console.log(signDataResp);
                        let certificateDetails = JSON.parse(signDataResp);
                        $.ajax({
                            url: '{{ route("invoices.set.thumbprint") }}',
                            type: 'get',
                            data: {thumbprint: certificateDetails.CertThumbPrint},
                            success: function (response) {
                                alert(response.message);
                            },
                            error: function (error) {
                                alert(error.responseJSON.message);
                            }
                        });
                    },
                    function () {
                        alert('ألرجاء توصيل الختم الرقمي');
                    }
                );
            });
        });

        function checkExtension()
        {
            if (typeof SignerDigital !== 'undefined') return true;
            if (!confirm("Please Install This Extension 'SignerDigital'")) return false;

            window.open("{{ config('app.signer_extension_url') }}", '_blank');
            return false;
        }

        $(function() {
            $('.alert-success').delay(3000).fadeOut('slow');
            $('.alert-danger').delay(3000).fadeOut('slow');
        });
    </script>
@endsection

@section('css-scripts')
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/bootstrap-datetimepicker-standalone.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/datepicker3.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/css/bootstrap-select.min.css') }}" rel="stylesheet">
@stop
