<?php

Route::group([
    'middleware' => 'web',
    'prefix'     => 'einvoice/invoice',
    'as'         => 'einvoice.',
], function () {
    // Einvoice Settings Routes.
    Route::match(['get', 'post'], '/settings', 'SettingsController@index')->name('settings.index');
    Route::get('settings/check/count', 'SettingsController@checkCount')->name('settings.check.count');
    Route::get('/', 'SubmitController@index')->name('submit.index');

    // Einvoice Document Routes.
    Route::group(['prefix' => 'documents', 'as' => 'documents.'], function () {
        Route::get('/', 'DocumentController@index')->name('index');
        Route::get('/{id}', 'DocumentController@show')->name('show');
        Route::post('/{documentUUID}/submit', 'DocumentController@submit')->name('submit');
        Route::post('/{documentUUID}/cancel', 'DocumentController@cancel')->name('cancel');
        Route::post('/{documentUUID}/reject', 'DocumentController@reject')->name('reject');
        Route::post('/{documentUUID}/declineCancel', 'DocumentController@declineCancel')->name('declineCancel');
        Route::post('/{documentUUID}/declineReject', 'DocumentController@declineReject')->name('declineReject');
        Route::get('/{documentUUID}/download', 'DocumentController@download')->name('download');
        Route::post('/update-status', 'DocumentController@updateStatus')->name('updateStatus');
    });

    // Einvoice Notifications.
    Route::group(['prefix' => 'notifications', 'as' => 'notifications.'], function () {
        Route::get('/', 'NotificationController@index')->name('index');
    });

    // Einvoice Item Codes.
    Route::group(['prefix' => 'itemcodes', 'as' => 'itemcodes.'], function () {
        Route::get('/', 'ItemCodeController@index')->name('index');
        Route::match(['get', 'put'], '/{itemCode}/edit', 'ItemCodeController@edit')->name('edit');
        Route::put('/reuse/{codeID}', 'ItemCodeController@reuse')->name('reuse');
        Route::get('/published', 'ItemCodeController@published')->name('published');
        Route::post('/update-code', 'ItemCodeController@updateCode')->name('updateCode');
    });

    // Einvoice Document Package Requests.
    Route::group(['prefix' => 'packageRequests', 'as' => 'packageRequests.'], function () {
        Route::match(['get', 'post'], '/add', 'PackageRequestController@add')->name('addRequestPackage');
        Route::get('/', 'PackageRequestController@index')->name('index');
        Route::get('/{id}', 'PackageRequestController@show')->name('show');
        Route::get('download/{id}', 'PackageRequestController@download')->name('download');
    });
});

Route::group([
    'middleware' => 'api',
    'prefix'     => 'einvoice',
    'as'         => 'einvoice.',
], function () {
    Route::group(['prefix' => 'notifications', 'as' => 'notifications.'], function () {
        Route::match(['get', 'post', 'put'], '/receive', 'NotificationController@receive')->name('receive');
    });
});
