<?php

namespace Modules\Employee\Http\Controllers;
use App\Models\systemLog;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Controllers\Controller;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use Modules\Employee\Entities\Employee;
use Modules\Employee\Entities\EmployeeTransaction;
use Modules\Employee\Entities\SalePoint;
use Modules\Employee\Entities\Store;

class EmployeeController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index(Request $request)
    {
        $user_sale_point = SalePoint::whereId(auth()->user()->sale_point)->first();
        $user_store = $user_sale_point->store_id;
        $stores = Store::select('id', 'store_name')
            ->where('shop_id', auth()->user()->shop_id)
            ->when(auth()->user()->change_store == 0, function ($q) use ($user_store) {
                $q->where('id', $user_store);
            })->pluck('store_name', 'id');


        $employees = Employee::where([
            'shop_id' => auth()->user()->shop_id,
            'is_employee' => 1
        ]);

        if (auth()->user()->change_store == 0 && $request->all() == []) {
            $employees = $employees->where('store_id', $user_store);
        } else {
            if ($request->has('store_id') && !in_array($request->store_id, ['-1', '', null])) {
                $employees = $employees->where('store_id', $request->store_id);
            }
        }

        if ($request->has('filter')) {
            if (in_array($request->filter, [0, 1])) {
                $employees = $employees->where('work', $request->filter);
            }
        }

        $employees = $employees->get();
        return view('employee::index', compact('employees', 'stores'));
    }

    public function show($id)
    {
        $session = null;

        if (session()->has('salary')) {
            $session = session('salary');
            session()->forget('salary');
        }

        $employee = Employee::withCount('clients')->findOrFail($id);
        $img = get_session_val('logo_path', 'string');

        if ($employee->month_deduct > 0) {
            $employee->money_out_ = $employee->month_deduct;
        } else {
            $employee->money_out_ = $employee->money_out;
        }

        $point = Auth::user()->sale_point;
        if (Auth::user()->show_points == 1 ||Auth::user()->show_points == 2) {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->get();
        }elseif (Auth::user()->show_points == 3) {
            $store_id = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->first()->store_id;
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_id', $store_id)
          ->get();

        } else {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        }
        $employee_transaction = $employee->transaction()->paginate(20);

        return view('employee::show', compact('session', 'img', 'employee', 'employee_transaction', 'sale_points'));
    }

    public function generateUsername($name)
    {
        if (is_null($name)) {
            $u_name = $name . get_session_val('serial_id', 'integer');
        } else {
            $u_name = $name . random_int(00, 99);
        }
        $user = Employee::where('user_name', $u_name)->first();

        if (is_null($user)) {
            return $u_name;
        } else {
            $this->generateUsername($u_name);
        }
    }

    public function store(Request $request)
    {

        $this->validate($request, [
            'name' => 'required',
            'start_work' => 'required',
            'collecting' => ['nullable', 'bool'],
            'collecting_ratio' => ['required_if:collecting,1'],
            'tel' => ['nullable', 'unique:users,tel', "string", "max:" . (get_session_val('country', 'integer') == 187 ? 10 : 11)],
            'salary' => ['required', 'numeric']
        ]);


        $u_name = $this->generateUsername($request->name);

        $che = Employee::where([
            'shop_id' => get_session_val('serial_id', 'integer'),
            'name' => $request->name
        ])->first();
        if ($che) {
            return back()->with(['error' => 'اسم الموظف مسجل من قبل']);
        }

        $emp = new Employee();
        $emp->is_employee = 1;
        $emp->name = $request->name;
        $emp->user_name = $u_name;
        $emp->work = $request->work ? 1 : 0;
        $emp->salary = $request->salary ?? 0;
        $emp->money_out = $request->money_out ?? 0;
        $emp->job = $request->job ?? '';
        $emp->tel = $request->tel ?? '';
        $emp->address = $request->address ?? '';
        $emp->start_work = $request->start_work;
        $emp->collecting = $request->collecting ?? 0;
        $emp->collecting_ratio = $request->collecting_ratio ?? 0;

        $emp->password = $request->password ?? '';
        $emp->email = $request->email ?? '';
        $emp->birth_date = $request->birth_date ?? '0000-00-00';
        $emp->employe_date = $request->employe_date ?? '0000-00-00';
        $emp->city_id = $request->city_id ?? 0;
        $emp->store_id = $request->store_id ?? 0;
        $emp->credit = $request->credit ?? 0;
        $emp->recipient = $request->recipient ?? 0;
        $emp->privilege = $request->privilege ?? 0;
        $emp->add_user = auth()->id();
        $emp->sale_price = $request->sale_price ?? 0;
        $emp->send_alerts = $request->send_alerts ?? 0;
        $emp->show_problems = $request->show_problems ?? 0;
        $emp->dbpassword = $request->dbpassword ?? '';
        $emp->shop_id = get_session_val('serial_id', 'integer');
        $emp->email = $request->email ?? '';
        $emp->covenant_value = $request->covenant_value ?? 0;
        $emp->covenant_receive_date = $request->covenant_receive_date ?? null;
        $emp->covenant_destruction_date = $request->covenant_destruction_date ?? null;
        $emp->covenant_description = $request->covenant_description ?? null;
        $emp->money_percent = $request->money_percent ?? 0;
        $emp->save();

        $this->createLog(Carbon::now(), " تم تسجيل الموظف".$request->user_name);
        return redirect()->back()->with([
            'success' => 'تم الحفظ بنجاح'
        ]);
    }


    public function edit(Request $request)
    {
        $item = Employee::find($request->id);

        $user_store = SalePoint::select('store_id')->whereId(auth()->user()->sale_point)->first()?->store_id;
        $stores = Store::select('id', 'store_name')
                        ->where('shop_id', auth()->user()->shop_id)
                        ->when(auth()->user()->change_store == 0, function ($q) use ($user_store) {
                            $q->where('id', $user_store);
                        })->pluck('store_name', 'id');

        if (auth()->user()->change_store == '1' || $user_store == 0) {
            $stores[0] = 'سوق الجوالات';
        }

        return view('employee::form', compact('item', 'stores'));
    }

    public function update(Request $request)
    {
        $emp = Employee::findOrFail($request->item_id);
        $this->validate($request, [
            'name' => 'required',
            'start_work' => 'required',
            'collecting' => ['nullable', 'bool'],
            'collecting_ratio' => ['required_if:collecting,1'],
            'tel' => ['nullable', 'unique:users,tel,'. $emp->id]
        ]);

        $che = Employee::where([
            'shop_id' => get_session_val('serial_id', 'integer'),
            'user_name' => $request->name
        ])->where('id', '!=', $emp->id)->first();
        if ($che) {
            return back()->with(['error' => 'اسم المستخدم مسجل من قبل']);
        }

        $emp->name = $request->name;

        $emp->collecting = $request->collecting ?? 0;
        $emp->collecting_ratio = $request->collecting_ratio ?? $emp->collecting_ratio;
        $emp->work = $request->work ? 1 : 0;
        $emp->salary = $request->salary ?? 0;
        $emp->money_out = $request->money_out ?? 0;
        $emp->job = $request->job ?? '';
        $emp->tel = $request->tel ?? '';
        $emp->address = $request->address ?? '';
        $emp->start_work = $request->start_work;
        $emp->covenant_value = $request->covenant_value ?? 0;
        $emp->covenant_receive_date = $request->covenant_receive_date ?? null;
        $emp->covenant_destruction_date = $request->covenant_destruction_date ?? null;
        $emp->covenant_description = $request->covenant_description ?? null;
        $emp->store_id = $request->store_id ?? 0;
        $emp->money_percent = $request->money_percent ?? 0;

        $emp->save();
        systemLog::create(date('Y-m-d H:i:s'), 'تعديل بيانات الموظف ' . $emp->user_name);

        return redirect()->back()->with(['success' => 'تم التعديل بنجاح']);
    }


    public function destroy(Request $request)
    {
        $id = $request->employee_id;
        $employee = employee::where('id', $id)->withCount('clients')
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->first();

        if ($employee->clients_count) {
            return redirect()->back()->with([
                'error' => "لا يمكن حذف الموظف لان لديه عملاء جملة"
            ]);
        }

        if ($employee->transaction()->count() > 0) {
            return redirect()->back()->with([
                'error' => "لا يمكن حذف موظف له عمليات"
            ]);
        }
        $this->createLog(Carbon::now(), " تم حذف بيانات الموظف".$employee->user_name);
        $employee->delete();
        return redirect()->back()->with([
            'success' => "تم الحذف بنجاح"
        ]);
    }

    public function paysallery()
    {
        return view('employee::pay-sallery', compact('employee', 'employee_transaction'));
    }


    public function reportSalary(Request $request)
    {

                $type = get_session_val('date_type', 'integer');

        $employees = collect();

        $all_raise = 0;
        $all_cut = 0;
        $all_debt = 0;
        $all_sallery = 0;
        $all_net = 0;

        if ($request->month && $request->year) {

            $m = $request->month;
            if (strlen($m) == 1) {
                $m = '0' . $m;
            }
            $y = $request->year;

            $month = $y . '-' . $m;
            $monthe = $m . '/' . $y;
            $employees = EmployeeTransaction::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('monthe', $monthe)
                ->where('type', 43)
                ->get();
            foreach ($employees as $employee) {
                $emp = Employee::find($employee->em_id);
                $n = '';
                if ($emp) {
                    $n = $emp->name;
                }
                $employee->name = $n;
            }

        }

        return view('employee::report_salary', compact('employees', 'type', 'all_cut', 'all_debt', 'all_net', 'all_raise', 'all_sallery'));

    }

    private function typeFormat($type)
    {
        $types = [
            '42' => 'سلف',
            '41' => 'خصم',
            '43' => 'رواتب',
            '40' => 'حوافز',
        ];
        return $types[$type];
    }

    private function getUserName($id)
    {
        $user = Employee::find($id);
        if ($user) {
            return $user->user_name;
        }
        return '';
    }

    public function showDetail(Request $request)
    {

                $type = get_session_val('date_type', 'integer');

        $employees = collect();

        $all_raise = 0;
        $all_cut = 0;
        $all_debt = 0;
        $all_sallery = 0;
        $all_net = 0;

        $m = $request->month;
        if (strlen($m) == 1) {
            $m = '0' . $m;
        }
        $y = $request->year;

        $month = $y . '-' . $m;
        $monthe = $m . '/' . $y;
        $em_id = $request->em_id;

        $employees = EmployeeTransaction::where('shop_id', get_session_val('serial_id', 'integer'));

        if ($request->em_id) {
            $employees = $employees->where('em_id', $em_id);
        }
        if ($monthe) {
            $employees = $employees->where('monthe', $monthe);
        }
        $employees = $employees->where('type', 43)
            ->get();

        foreach ($employees as $employee) {
            $emp = Employee::find($employee->em_id);
            $n = '';
            $job = '';
            $money_out = '';
            if ($emp) {
                $n = $emp->name;
                $job = $emp->job;
                $money_out = $emp->money_out;
            }
            $employee->name = $n;
            $employee->job = $job;
            $employee->money_out = $money_out;
        }

        return view('employee::print', compact('employees', 'type', 'all_cut', 'all_debt', 'all_net', 'all_raise', 'all_sallery'));
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}
