<?php

namespace Modules\Excel\Entities;

use App\Traits\RoundingAttributesTrait;
use Illuminate\Database\Eloquent\Model;
use Modules\Excel\Entities\Scopes\ShopIdScope;
use Modules\Item\Entities\Category;
use Modules\Item\Entities\StoreItem;

class Item extends Model
{
    use RoundingAttributesTrait;
    protected $table = 'items';
    public $timestamps = false;
    protected $guarded = [];

    public function invoice_details()
    {
        return $this->hasMany(InvoiceDetail::class, 'item_id');
    }

    public function category()
    {
        return $this->belongsTo(Category::class, 'sale_unit')->select('id', 'name')->withDefault(['name' => '']);
    }

    public function storeQty()
    {
        return $this->hasMany(StoreItem::class, 'item_id');
    }

    public function supplier()
    {
        return $this->belongsTo(Client::class, 'supplier_id')->select('id', 'client_name as name')->withDefault(['name' => '']);
    }

    public function unit()
    {
        return $this->belongsTo(Unit::class, 'unit_id')->select('id', 'name')->withDefault(['name' => '']);
    }

    protected static function booted()
    {
        static::addGlobalScope(new ShopIdScope);
    }
}
