<?php

namespace Modules\Excel\Http\Controllers;

use App\Models\systemLog;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Maatwebsite\Excel\Facades\Excel;
use Maatwebsite\Excel\Validators\ValidationException;
use Modules\Excel\Entities\Store;
use Modules\Excel\Exports\ClientsExport;
use Modules\Excel\Exports\ItemsExport;
use Modules\Excel\Imports\AddClientsImport;
use Modules\Excel\Imports\AddItemsImport;
use Modules\Excel\Imports\DeleteItemsImport;
use Modules\Excel\Http\Requests\ExcelRequest;

class ExcelController extends Controller
{
    public function editItems(Request $request)
    {
        $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        return view('excel::edit_items', ['stores' => $stores]);
    }

    public function editItemsStore(ExcelRequest $request)
    {
        try {

            switch ($request->operation) {
                case '1': // add
                    Excel::import(new AddItemsImport($request->store), $request->sheet);
                    break;
                case '2': // delete
                    Excel::import(new deleteItemsImport($request->store), $request->sheet);
                    break;
            }

        } catch (ValidationException $e) {
            $failures = $e->failures();
            $errors = $this->groupRowValidationErrors($failures);
            return redirect()->back()->with('import_validation_errors', $errors);
        }
        return redirect()->back();
    }

    public function clients(Request $request)
    {
        return view('excel::edit_clients');
    }

    public function storeClients(ExcelRequest $request)
    {
        try {
            Excel::import(new AddClientsImport($request->store), $request->sheet);
            $this->createLog(Carbon::now(), " تم رفع عملاء بالاكسل");
        } catch (ValidationException $e) {
            $failures = $e->failures();
            $errors = $this->groupRowValidationErrors($failures);
            return redirect()->back()->with('import_validation_errors', $errors);
        }
        return redirect()->back();
    }

    /**
     * group rows validation errors by row number
     * @param array $failures
     * @return array
     */
    protected function groupRowValidationErrors(array $failures): array
    {
        $errors = [];
        foreach ($failures as $failure) {
            if (!array_key_exists($failure->row(), $errors)) {
                $errors[$failure->row()] = [];
            }
            $errors[$failure->row()][] = $failure->errors()[0];
        }
        return $errors;
    }

    public function exportItems(Request $request)
    {
        ob_clean();
        return Excel::download(new ItemsExport($request->has('empty')), 'items.xlsx');
    }

    public function exportClients(Request $request)
    {
        ob_clean();
        return Excel::download(new ClientsExport($request->has('empty')), 'clients.xlsx');
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}
