<?php

namespace Modules\Excel\Imports;

use Modules\Clients\Entities\Client;
use Maatwebsite\Excel\Concerns\{SkipsFailures,
    SkipsOnError,
    ToCollection,
    WithChunkReading,
    WithHeadingRow,
    WithValidation,
    SkipsEmptyRows
};

use Illuminate\Support\{
    Collection,
    Facades\DB
};

use Illuminate\Validation\Rule;
use Modules\Clients\Entities\ClientTransaction;
use Modules\Clients\Entities\Country;
use Throwable;

class AddClientsImport implements ToCollection, WithChunkReading, WithHeadingRow, WithValidation, SkipsOnError, SkipsEmptyRows
{
    use SkipsFailures;
    protected $clients;
    protected int $country_id;

    public function __construct()
    {
        ini_set('max_execution_time', 999999999999999999);
        $this->country_id = get_session_val('country', 'integer');
    }

    /**
     * @param array $row
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function collection(Collection $rows)
    {
        if ($rows->isEmpty()) return redirect()->back()->with('error', 'الملف فارغ');
        foreach ($rows as $row) {
            $this->handleImport($row);
        }
        return redirect()->back()->with('imported_clients', $this->clients)->with('import_success', ' :تم اضافه العملاء و الموردين التاليين');
    }

    public function chunkSize(): int
    {
        return 1000;
    }

    protected function trimData(&$row)
    {
        foreach ($row as $key => $value) {
            $row[$key] = trim($value);
        }
    }

    /**
     * store items
     * @param $row
     * @return mixed
     */
    protected function storeClient($row)
    {
        $this->trimData($row);

        $client = Client::updateOrCreate(['client_name' => $row['client_name'], 'shop_id' => get_session_val('serial_id' ,'integer')], [
            'governate'         => $row['governmante'] ?? null,
            'regionCity'        => $row['city'] ?? null,
            'street'            => $row['street'] ?? null,
            'buildingNumber'    => $row['block_no'] ?? null,
            'postalCode'        => $row['postal_code'] ?? null,
            'floor'             => $row['floor_no'] ?? null,
            'room'              => $row['room_no'] ?? null,
            'landmark'          => $row['landmark'] ?? null,
            'country_id'        => Country::select('id')->where('name_ar', 'LIKE', "%{$row['country']}%")->first()->id ?? $this->country_id,
            'client_role'       => $row['client_role_no'] ?? 0,
            'address'           => $row['address'] ?? '',
            'mobile1'           => $row['mobile'] ?? '',
            'balance'           => $row['balance'] ?? 0,
            'info1'             => $row['national_id'] ?? '',
            'addition_data'     => $row['notes'] ?? '',
            'city_id'           => $row['city_id'] ?? '',
            'client_tax_number' => $row['tax_number'] ?? '',
            'email'             => $row['email'] ?? null,
            'add_user'          => auth()->id(),
            'add_date'          => date("Y-m-d H:i:s"),
        ]);

        return $client;
    }

    protected function storeClientTransaction(Client $client)
    {
        ClientTransaction::create([
            'shop_id' => get_session_val('serial_id' ,'integer'),
            'date_time' => date('Y-m-d H:i:s'),
            'client_id' => $client->id,
            'amount' => 0,
            'type' => 4,
            'pay_day' => date('Y-m-d H:i:s'),
            'balance' => $client->balance,
            'user_id' => auth()->id()
        ]);
    }

    /**
     * @return array
     */
    public function rules(): array
    {
        $e_invoice_attr_status = get_session_val('einvoice', 'integer') ? 'required' : 'nullable';

        return [
            'client_role'       =>    ['nullable', Rule::in(['client', 'supplier', 'both', 'عميل', 'مورد', 'كلاهما' , ''])],
            "tax_number"        =>    ['nullable', 'numeric'],
            "governmante"       =>    [$e_invoice_attr_status, 'string', 'max:255'],
            "city"              =>    [$e_invoice_attr_status, 'string', 'max:255'],
            "street"            =>    [$e_invoice_attr_status, 'string', 'max:255'],
            "block_no"          =>    [$e_invoice_attr_status, 'numeric'],
            "postal_code"       =>    [$e_invoice_attr_status, 'numeric'],
            "floor_no"          =>    [$e_invoice_attr_status, 'numeric'],
            "room_no"           =>    [$e_invoice_attr_status, 'numeric'],
            "landmark"          =>    [$e_invoice_attr_status, 'string', 'max:255'],
            "national_id"       =>    ['nullable', 'digits_between:14,14'],
            "mobile"            =>    ['nullable'],
            "client_name"       =>    ['required', 'string', 'max:255'],
            "country"           =>    ['nullable']
        ];
    }

    /**
     * @param Throwable $e
     */
    public function onError(Throwable $e)
    {
        return redirect()->back()->with('error', $e);
    }

    /**
     * handle add items
     * @param $row
     * @return void
     */
    protected function handleImport($row): void
    {
        DB::transaction(function () use ($row) {
            $row['client_role_no'] = $this->detectClientRole($row);

            $client = $this->storeClient($row);
            $this->storeClientTransaction($client);
            $this->clients[] = $row['client_name'];

            // if (! $this->isClientExists($row)) {
            //     $client = $this->storeClient($row);
            //     $this->storeClientTransaction($client);
            //     $this->clients[] = $row['client_name'];
            // }
        });
    }

    protected function isClientExists($row)
    {
        return Client::when(get_session_val('repeat_client_name', 'integer') == 0, function($q) use($row){
                    $q->where('client_name', $row['client_name']);
                })
                ->when($row['client_role_no'] != 0 && $row['tax_number'], function($q) use($row){
                    $q->orWhere('client_tax_number', $row['tax_number']);
                })
                ->when($row['client_role_no'] != 1, function($q) use($row){
                    $q->orWhere('info1', $row['national_id']);
                })
                ->orWhere('mobile1', $row['mobile'])
                ->first();
    }

    public function headingRow(): int
    {
        return 2;
    }

    protected function detectClientRole($row)
    {
        return [
            'client' => 0,
            'عميل' => 0,
            '' => 0,
            'supplier' => 1,
            'مورد' => 1,
            'both' => 2,
            'كلاهما' => 2,
        ][$row['client_role']] ?? 0;
    }
}
