<?php

namespace Modules\Excel\Imports;

use App\Models\systemLog;
use Carbon\Carbon;
use Modules\Excel\Services\{SupplierService, UnitsService, ItemsTypeService};
use Maatwebsite\Excel\Concerns\{SkipsFailures,
    SkipsOnError,
    ToCollection,
    WithChunkReading,
    WithHeadingRow,
    WithValidation,
    SkipsEmptyRows
};
use Modules\Excel\Entities\{Item, StoreItem};
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Collection;
use Modules\Invoices\Entities\InvoiceDetails;
use Modules\Invoices\Entities\Invoices;
use Throwable;


class AddItemsImport implements ToCollection, WithChunkReading, WithHeadingRow, WithValidation, SkipsOnError, SkipsEmptyRows
{
    use SkipsFailures;

    protected $store_id;
    protected $items;

    private $supplierService;
    private $unitsService;
    private $itemsTypeService;

    public function __construct($store_id)
    {
        ini_set('max_execution_time', 999999999999999999);
        $this->store_id = $store_id;
        $this->itemsTypeService = new ItemsTypeService();
        $this->supplierService = new SupplierService();
        $this->unitsService = new UnitsService();
    }

    /**
     * @param array $row
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function collection(Collection $rows)
    {
        ini_set('max_execution_time', 18000);
        if ($rows->isEmpty()) return redirect()->back()->with('error', 'الملف فارغ');
        foreach ($rows as $row) {
            $this->handleImport($row);
        }
        $this->createLog(Carbon::now(), "تم اضافه اصناف بالاكسل ");
        return redirect()->back()->with('imported_items', $this->items)->with('import_success', ' :تم اضافه الاصناف التاليه');
    }

    public function chunkSize(): int
    {
        return 1000;
    }

    protected function trimData(&$row)
    {
        foreach ($row as $key => $value) {
            $row[$key] = trim($value);
        }
        
        // $row['item_name'] = $row['asm_alsnf_item_name'];
        // $row['barcode'] = $row['albarkod_barcode'];
        // $row['about'] = $row['alosf_about'];
        // $row['sale_price'] = $row['saar_albyaa_sale_price'];
        // $row['pay_price'] = $row['saar_alshraaa_pay_price'];
        // $row['quantity'] = $row['alkmy_quantity'];
        // $row['classification'] = $row['altsnyf_classification'];
        // $row['unit'] = $row['alohd_unit'];
        // $row['supplier'] = $row['almord_supplier'];
        // $row['lowest_sale_price'] = $row['akl_saar_byaa_lowest_sale_price'];
        // $row['minimum_quantity'] = $row['alhd_aladny_llkmy_minimum_quantity'];
    }

    /**
     * store items
     * @param $row
     * @return mixed
     */
    protected function storeItem($row)
    {
        $this->trimData($row);

        return Item::updateOrCreate(['item_name' => $row['item_name'], 'barcode' => $row['barcode']], [
            'about' => $row['about'] ?? ' ',
            'sale_price' => $row['sale_price'] ?? 0,
            'lowest_price' => $row['lowest_sale_price'] ?? 0,
            'pay_price' => $row['pay_price'] ?? 0,
            'quantity' => $row['quantity'] ?? 0,
            'min_quantity' => $row['minimum_quantity'] ?? 0,
            'supplier_id' => $this->supplierService->setSupplierName($row['supplier'])->getSupplierId(),
            'unit_id' => $this->unitsService->setUnitName($row['unit'])->getUnitId(),
            'sale_unit' => $this->itemsTypeService->setItemsTypeName($row['classification'])->getItemsTypeId(),
            'shop_id' => get_session_val('serial_id', 'integer'),
            'add_user' => auth()->id(),
            'vat_state' => 0,
            'purchase_vat_state' => 0,
            'is_service' => 0
        ]);
    }

    /**
     * create item transaction
     * @param $row
     * @param $item
     * @return void
     */
    protected function storeItemTransaction($row, $item): void
    {
        if ($row['quantity'] > 0)
        {
            $invoice = Invoices::create([
                'invoice_date' => date('Y-m-d h:i:s'),
                'transaction_date' => date('Y-m-d h:i:s'),
                'add_user' => auth()->user()->id,
                'client_id' => 0,
                'store_id' => $this->store_id,
                'notes' => '',
                'shop_id' => get_session_val('serial_id', 'Integer'),
                'invoice_type' => 7,
                'invoice_total_disc' => 0,
                'invoice_total_disc_type' => 1,
                'invoice_total_disc_status' => 1
            ]);

            InvoiceDetails::create([
                'invoice_id' => $invoice->id,
                'shop_id' => get_session_val('serial_id', 'Integer'),
                'item_id' => $item->id,
                'quantity' => $row['quantity'],
                'quantity_after' => $row['quantity'],
                'unit_id' => $item->unit_id,
                'store_id' => $this->store_id,
            ]);
        }
    }

    /**
     * store items quantity in the store
     * @param $item
     * @param $quantity
     * @return void
     */
    protected function storeItemQuantity($item, $quantity): void
    {
        StoreItem::updateOrCreate([
            'item_id' => $item->id,
            'store_id' => $this->store_id
        ],[
            'item_id' => $item->id,
            'store_id' => $this->store_id,
            'store_quant' => $quantity,
            'shop_id' => get_session_val('serial_id', 'integer'),
            'add_user' => auth()->id(),
        ]);
    }

    /**
     * @return array
     */
    public function rules(): array
    {
        // return [
        //     'asm_alsnf_item_name' => ['required'],
        //     'albarkod_barcode' => ['required'],
        //     'alosf_about' => 'nullable',
        //     'saar_albyaa_sale_price' => 'required|numeric',
        //     'saar_alshraaa_pay_price' => 'required|numeric',
        //     'alkmy_quantity' => 'nullable|numeric',
        //     'altsnyf_classification' => [ 'nullable', 'string'],
        //     'alohd_unit' => [ 'nullable', 'string'],
        //     'almord_supplier' => [ 'nullable', 'string'],
        //     'akl_saar_byaa_lowest_sale_price' => 'nullable|numeric',
        //     'alhd_aladny_llkmy_minimum_quantity' => 'nullable|numeric',
        // ];
        
        return [
            'item_name' => ['required'],
            'barcode' => ['required'],
            'about' => 'nullable',
            'sale_price' => 'required|numeric',
            'pay_price' => 'required|numeric',
            'quantity' => 'nullable|numeric',
            'classification' => [ 'nullable', 'string'],
            'unit' => [ 'nullable', 'string'],
            'supplier' => [ 'nullable', 'string'],
            'lowest_sale_price' => 'nullable|numeric',
            'minimum_quantity' => 'nullable|numeric',
        ];
    }

    /**
     * @param Throwable $e
     */
    public function onError(Throwable $e)
    {
        return redirect()->back()->with('error', $e);
    }

    /**
     * handle add items
     * @param $row
     * @return void
     */
    protected function handleImport($row): void
    {
        DB::transaction(function () use ($row) {

            $item = $this->storeItem($row);
            $this->storeItemTransaction($row, $item);
            $this->storeItemQuantity($item, $row['quantity']);
            $this->items[] = $row['item_name'];
            // if (! $this->isItemExists($row))
            // {
            //     $item = $this->storeItem($row);
            //     $this->storeItemTransaction($row, $item);
            //     $this->storeItemQuantity($item, $row['quantity']);
            //     $this->items[] = $row['item_name'];
            // }
        });
    }

    protected function isItemExists($row)
    {
        return Item::where('item_name', $row['item_name'])->orWhere('barcode', $row['barcode'])->first();
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}


