<?php

namespace Modules\Excel\Imports;

use App\Models\systemLog;
use Maatwebsite\Excel\Concerns\{SkipsFailures,
    SkipsOnError,
    ToCollection,
    WithChunkReading,
    WithHeadingRow,
    WithValidation,
    SkipsEmptyRows};
use Modules\Excel\Entities\{Item, ItemColor, ItemPrice, ItemSize, ItemUnit, StoreItem};
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Collection;
use Illuminate\Validation\Rule;
use Throwable;


class DeleteItemsImport implements ToCollection, WithChunkReading, WithHeadingRow, WithValidation, SkipsOnError, SkipsEmptyRows
{
    use SkipsFailures;

    protected $store_id;
    protected $items;


    public function __construct($store_id)
    {
        ini_set('max_execution_time', 999999999999999999);
        $this->store_id = $store_id;
    }

    /**
     * @param array $row
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function collection(Collection $rows)
    {
        if ($rows->isEmpty()) return redirect()->back()->with('error', 'الملف فارغ');
        foreach ($rows as $row) {
            $this->handleImport($row);
        }
        $this->createLog(Carbon::now(), "تم حذف اصناف بالاكسل ");
        return redirect()->back()->with('imported_items', $this->items)->with('import_success', ' :تم حذف الاصناف التاليه');
    }

    public function chunkSize(): int
    {
        return 1000;
    }

    /**
     * @return array
     */
    public function rules(): array
    {
        return [
            'item_name' => ['required', 'string', Rule::exists('items', 'item_name')->where('shop_id', get_session_val('serial_id', 'integer'))],
        ];
    }

    /**
     * @param Throwable $e
     */
    public function onError(Throwable $e)
    {
        return redirect()->back()->with('error', $e);
    }

    /**
     * handling delete items
     * @param $row
     * @return void
     */
    protected function handleImport($row): void
    {
        DB::transaction(function () use ($row) {
            $item = Item::where([
                ['item_name', trim($row['item_name'])],
                ['shop_id', get_session_val('serial_id', 'integer')]
                ])->first();

            if ($item && $item->invoice_details->count() == 0) {
                ItemSize::where('item_id', $item->id)->delete();
                ItemColor::where('item_id', $item->id)->delete();
                ItemUnit::where('item_id', $item->id)->delete();
                ItemPrice::where('item_id', $item->id)->delete();
                StoreItem::where('item_id', $item->id)->delete();
                $item->delete();
                $this->items[] = $row['item_name'];
            }
        });
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}


