<?php

namespace Modules\Excel\Imports;

use Exception;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\SkipsEmptyRows;
use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\WithChunkReading;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use Modules\Excel\Entities\Item;
use Modules\Excel\Entities\ItemPrice;
use Modules\Excel\Entities\PriceList;

class UpdateItemPricesImport implements ToCollection, WithChunkReading, WithHeadingRow, SkipsEmptyRows
{
    protected $shop_id;
    protected $failed_rows = [];
    protected $success_rows = 0;
    protected $item_id;
    protected $list_id;

    public function __construct()
    {
        ini_set('max_execution_time', 999999999999999999);
        $this->shop_id = get_session_val('serial_id', 'integer');
    }

    /**
    * @param Collection $collection
    */
    public function collection(Collection $collection)
    {
        if ($collection->isEmpty()) return redirect()->back()->with('error', 'الملف فارغ');

        try {
            foreach ($collection as $index => $row) {
                $this->initRowData($row);
                if ($this->itemHandler($row)) {
                    $this->success_rows++;
                } else {
                    $this->failed_rows [$index + 2] = "{$row['item_name']}  |  {$row['barcode']}  |  {$row['list_name']}  |  {$row['list_price']}  |  {$row['list_quantity']}";
                }
            }

            if ($this->success_rows > 0)
                session()->put('success_rows', "تم تعديل $this->success_rows من القوائم بنجاح");

            if (count($this->failed_rows) > 0)
                session()->put('failed_rows', $this->failed_rows);
        } catch(Exception $e) {
            dd($e);
            return redirect()->back()->with('error', $e);
        }
    }

    protected function itemHandler($row)
    {
        try {
            if (! $this->checkItemExists($row) ) return false;
            $this->checkListExists( $row['list_name'] );
            $this->createItemList( $row );
            return true;
        } catch (Exception $e) {
            dd($e, '2');
            return false;
        }
    }

    protected function initRowData(&$row)
    {
        $row['item_name']  = isset( $row['item_name'] )  ? trim( $row['item_name'] )  : '';
        $row['barcode']    = isset( $row['barcode'] )    ? trim( $row['barcode'] )    : '';
        $row['list_name']  = isset( $row['list_name'] )  ? trim( $row['list_name'] )  : '';
        $row['list_price'] = isset( $row['list_price'] ) ? trim( $row['list_price'] ) : 0;
        $row['list_quantity'] = isset( $row['list_quantity'] )   ? trim( $row['list_quantity'] )   : 0;
    }

    protected function checkItemExists($row)
    {
        $this->item_id = Item::select('id')->where('shop_id', $this->shop_id)->where('item_name', $row['item_name'])
                            ->when($row['barcode'], function($query) use($row) {
                                $query->where('barcode', $row['barcode']);
                            })->first()->id ?? false;
        return $this->item_id;
    }

    protected function checkListExists(?string $list_name)
    {
        if (empty($list_name)) {
            $PriceList = PriceList::select('id')->first();
        } else {
            $PriceList = PriceList::updateOrCreate(['list_name' => $list_name]);
        }
        $this->list_id = $PriceList->id;
    }

    protected function createItemList($row)
    {
        ItemPrice::updateOrCreate([
                    'item_id' => $this->item_id,
                    'list_id' => $this->list_id
                ], [
                    'item_id'    => $this->item_id,
                    'list_id'    => $this->list_id,
                    'price'      => $row['list_price'],
                    'list_quant' => $row['list_quantity'],
                ]);
    }

    public function chunkSize(): int
    {
        return 1000;
    }

    public function headingRow(): int
    {
        return 1;
    }
}
