<?php

namespace Modules\Installer\Http\Controllers;

use Exception;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Schema;
use Modules\Currency\Entities\Currency;
use Modules\Installer\Entities\{
    Badrshop,
    BarcodeSetting,
    BillSetting,
    Client,
    ClientTransaction,
    Company,
    Country,
    InvoiceSettings,
    ItemType,
    SalePoint,
    SalesMan,
    Setting,
    Store,
    Unit,
    User
};
use Modules\Installer\Http\Requests\InstallerRequest;
use PDO;

class InstallerController extends Controller
{
    public $database;

    public function index()
    {
        $mac_address = $this->getMAC();
        return view('installer::index', compact('mac_address'));
    }

    public function install(InstallerRequest $request)
    {
        ini_set('max_execution_time', 1200000000);
        try {
            $this->createDatabase();
            $this->createStructure();
            $this->createDumpingData();
            $this->createShop($request);
            return redirect()->route('login')->with('success', trans('installer::trans.installed-successfully'))->withInput(['user_name' => $request->user_name]);
        } catch (Exception $e) {
            auth()->logout();
            Schema::dropDatabaseIfExists($this->database);
            return redirect()->back()->with('error', "DB Error : {$e->getMessage()}")->withInput($request->all());
        }
    }

    protected function getMAC(): string
    {
        try {
            return substr( exec('getmac') , 0, 17);

            ob_start();
            system('getmac');
            $content = ob_get_contents();
            ob_clean();
            return substr($content, strpos($content,'\\')-20, 17);
        } catch (Exception $e) { return ''; }
    }

    protected function createDatabase(): void
    {
        $config = Schema::getConnection()->getConfig();
        $this->database = $config['database'];
        $dbh = new PDO("mysql:host={$config['host']}", $config['username'], $config['password']);
        $dbh->exec("CREATE SCHEMA `{$config['database']}` DEFAULT CHARACTER SET {$config['charset']} COLLATE {$config['collation']};");
    }

    protected function createStructure(): void
    {
        $query = file_get_contents( __DIR__.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'stubs'.DIRECTORY_SEPARATOR.'structure.stub' );
        DB::statement($query);
    }

    protected function createDumpingData(): void
    {
        $query = file_get_contents( __DIR__.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'stubs'.DIRECTORY_SEPARATOR.'data.stub' );
        DB::statement($query);
    }

    protected function createShop($request): void
    {
        [$active_id, $serial_id] = explode(':', $request->active_id);

        // $name = \App\Services\Location\GeoLocationController::index()->country ?? 'Egypt';
        // $name = $name == 'invalid_ip' ? 'Egypt' : $name;
        $name = 'Egypt';
        $country = Country::select('id', 'currency_name_ar', 'iso_alpha2')->with('zone')->where('name_en', $name)->first() ?? new Country();
        Badrshop::create([
            'plan' => 1,
            'shop_name' => $request->shop_name,
            'u_no' => 20,
            'telephone' => $request->telephone,
            'ower_name' => $request->owner_name,
            'serial_id' => $serial_id,
            'active_id' => $active_id,
            'module' => 1,
            'zone' => $country->zone?->zone_name ?? 'Africa/Cairo',
            'style' => 1,
            'date_type' => 2,
            'online' => 0,
            'Regist_purchases' => 3,
            'active_date' => date('Y-m-d H:i:s'),
            'bill_no_record' => 12,
            'currency' => $country->currency_name_ar,
            'notify' => 0,
            'run_date' => now()->addYear(),
        ]);

        $store = Store::create([
            'store_name' => trans('default.default-model', ['model' => trans('default.store')]),
            'shop_id' =>  $serial_id,
        ]);

        $safe_point = SalePoint::create([
            'point_name' => trans('default.default-model', ['model' => trans('default.safe_point')]),
            'shop_id' =>  $serial_id,
            'store_id' => $store->id
        ]);

        $user = User::create([
            'name' => $request->user_name,
            'email' => $request->user_name,
            'password' => Hash::make($request->password),
            'shop_id' => $serial_id,
            'user_name' => $request->user_name,
            'users' => 1,
            'setting' => 1,
            'sale_point' => $safe_point->id
        ]);

        auth()->setUser($user);

        $currency = Currency::create(Currency::DEFAULT_ROW);
        DB::statement("UPDATE `currencies` SET `is_main` = 1 WHERE `id` = $currency->id");

        BillSetting::create(['shop_id' => $serial_id, 'bill_header' => '', 'message_bill' => '']);

        Setting::create([
            'shop_id' => $serial_id,
            'company_name' => $request->shop_name,
            'branch_id' => 0,
            'country_id' => $country->id,
            'currency_id' => $country->currency_name_ar,
            'registration_id' => ' ',
            'client_id' => ' ',
            'client_secret' => ' ',
        ]);

        $client = Client::create([
            'client_name' => trans('default.default-model', ['model' => trans('default.client-supplier')]),
            'shop_id' => $serial_id,
            'client_role' => 2
        ]);

        SalesMan::create([
            'sales_name' => trans('default.default-model', ['model' => trans('default.sales_man')]),
            'shop_id' => $serial_id
        ]);

        ItemType::create([
            'name' => trans('default.default-model', ['model' => trans('default.category')]),
            'shop_id' => $serial_id
        ]);

        InvoiceSettings::create([
            'shop_id' => $serial_id
        ]);

        BarcodeSetting::create([
            'shop_id' => $serial_id
        ]);

        Unit::create([
            'name' => trans('default.default-model', ['model' => trans('default.unit')]),
            'shop_id' => $serial_id
        ]);

        Company::create([
            'company_name' => trans('default.default-model', ['model' => trans('default.company')]),
            'shop_id' => $serial_id
        ]);

        ClientTransaction::create(
            [
                'date_time' => date("Y-m-d H:i:s"),
                'client_id' => $client->id,
                'amount' => 0,
                'type' => 4,
                'effect' => 2,
                'pay_day' => date("Y-m-d H:i:s"),
                'balance' => 0,
                'user_id' => $user->id,
                'shop_id' => $serial_id
            ],
            [
                'date_time' => date("Y-m-d H:i:s"),
                'amount' => 0,
                'type' => 4,
                'effect' => 2,
                'pay_day' => date("Y-m-d H:i:s"),
                'user_id' => $user->id,
                'shop_id' => $serial_id,
                'safe_type' => 1,
                'safe_balance' => 0,
                'safe_point_id' => $safe_point->id
            ],
            [
                'date_time' => date("Y-m-d H:i:s"),
                'amount' => 0,
                'type' => 4,
                'effect' => 1,
                'pay_day' => date("Y-m-d H:i:s"),
                'user_id' => $user->id,
                'shop_id' => $serial_id,
                'safe_type' => 2,
                'safe_balance' => 0,
            ]
        );
    }
}
