<?php

namespace Modules\Invoices\Entities;

use App\Traits\RoundingAttributesTrait;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Modules\Clients\Entities\ListPrice;
use Modules\Clients\Entities\User;

class Client extends Model
{
    use RoundingAttributesTrait;
    protected $guarded = ['id'];
    protected $table = 'clients';

    public function collector()
    {
        return $this->belongsTo(User::class, 'collector_id', 'id')->select('name', 'id', 'collecting_ratio')->where(['is_employee' => true, 'collecting' => true, 'shop_id' => auth()->user()->shop_id]);
    }

    public function type()
    {
        return $this->belongsTo(ListPrice::class, 'client_type', 'id')->select('list_name as name', 'id')->where(['shop_id' => auth()->user()->shop_id])->withDefault(['name' => 'قطاعي']);
    }

    public function sales()
    {
        return $this->hasMany(Invoices::class, 'client_id', 'id')->where('invoice_type', 1);
    }

    public function scopePerGroup(Builder $builder)
    {
        return $builder->when(auth()->user()->clients_group, function($query) {
                    $query->whereIn('clients.group_id', auth()->user()->groups->pluck('id')->toArray());
                });
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('shopID', function (Builder $builder) {
            $builder->where('clients.shop_id', auth()->user()->shop_id);
        });
    }
}
