<?php

namespace Modules\Invoices\Entities;

use App\Traits\RoundingAttributesTrait;
use Illuminate\Database\Eloquent\Model;
use Modules\Invoices\Entities\Client;
use Modules\Invoices\Entities\Unit;
use Modules\Invoices\Entities\ItemUnit;
use Modules\Item\Entities\Category;
use Modules\Item\Entities\ItemPrice;
use Modules\Item\Entities\ItemVat;

class Item extends Model
{
    use RoundingAttributesTrait;
    protected $guarded = ['id'];
    protected $table = 'items';
    public $timestamps = false;

    public function purchaseVat()
    {
        return $this->belongsTo(BillAdd::class, 'purchase_vat_id', 'id')->withDefault(['addition_value' => 0]);
    }

    public function client()
    {
        return $this->belongsToMany(Client::class);
    }

    public function units()
    {
        $it = $this;

        $main = Unit::find($it->unit_id);
        $all_units = collect();

        if ($main) {
            $main->unit_id = $main->id;
            $main->price = $it->pay_price;
            $main->sale = $it->sale_price;
            $all_units->push($main);
        }

        if ( get_session_val("units_sup", "Integer"))
        {
            $units = ItemUnit::where([['item_id', $it->id], ['unit_type', '<>', 2], ['unit_type', '<>', 3]])
            ->select('id', 'unit_id', 'unit_price', 'unit_value', 'unit_type')
            ->get();

            $units->each(function ($s_unit) use ($all_units, $it) {
                $item = $it;
                $unit = Unit::where('shop_id', get_session_val('serial_id', 'Integer'))
                    ->select('name')
                    ->find($s_unit->unit_id);
                $s_unit->name = $unit->name??'';
                $val = $s_unit->unit_value;
                $ty = $s_unit->unit_type;
                $price = round_money($item->pay_price * $val);
                $s_unit->price = round_money($price);
                $s_unit->sale = round_money($s_unit->unit_price);
                $all_units->push($s_unit);
            });
        }


        return $all_units;
    }

    public function Details()
    {
        return $this->belongsTo(InvoiceDetails::class);
    }
    
    public function Unit()
    {
        return $this->belongsTo(Unit::class, 'unit_id', 'id');
    }

    public function subUnits()
    {
        return $this->hasMany(ItemUnit::class, 'item_id')->where('unit_type', 0);
    }

    public function prices()
    {
        return $this->hasMany(ItemPrice::class, 'item_id');
    }

    public function vats()
    {
        return $this->hasMany(ItemVat::class, 'item_id');
    }

    public function availableQty()
    {
        return $this->hasMany(StoreItem::class, 'item_id');
    }
        public function mainSupplier()
    {
        return $this->belongsTo(Client::class, 'supplier_id');
    }

    public function category()
    {
        return $this->belongsTo(Category::class, 'sale_unit');
    }
}
