<?php

namespace Modules\Invoices\Entities;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;

class Request extends Model
{
    protected $guarded = ['id'];
    protected $table = 'requests';
    protected $casts = [
        'id' =>'integer',
        'created_at'=>'string',
        'status'=>'integer',
        'order_no'=>'integer',
        'installment_dates' => 'array',
    ];

    public function details()
    {
        return $this->hasMany(RequestDetails::class, 'request_id', 'id');
    }

    public function client(): BelongsTo
    {
        return $this->belongsTo(Client::class, 'client_id', 'id');
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('shopID', function (Builder $builder) {
            $builder->where('requests.shop_id', auth()->user()->shop_id)->where('requests.order_type', 2);
        });

        static::creating(function($model) {
            $model->order_type = 2;
            $model->shop_id = auth()->user()->shop_id;
        });
    }
}
