<?php

namespace Modules\Invoices\Http\Controllers;

use App\helpers\ShortUrl;
use App\Services\PurchaseInvoice;
use App\Services\ReturnPurchaseInvoice;
use App\Services\ReturnSellInvoice;
use App\Services\SellInvoice;
use App\Services\Validators\InvoiceValidate;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Invoices\Entities\BillAdd;
use Modules\Invoices\Entities\BillService;
use Modules\Invoices\Entities\Clients;
use Modules\Invoices\Entities\Currency;
use Modules\Invoices\Entities\DeliveryMan;
use Modules\Invoices\Entities\Invoices;
use Modules\Invoices\Entities\Item;

// use App\Models\Item;
use Modules\Invoices\Entities\ItemType;
use Modules\Invoices\Entities\Period;
use Modules\Invoices\Entities\Region;
use Modules\Invoices\Entities\SalePoint;
use Modules\Invoices\Entities\Store;
use Modules\Invoices\Entities\StoreItem;
use Modules\Invoices\Entities\Unit;
use Carbon\Carbon;
use Illuminate\Contracts\Session\Session;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\DB;
use Modules\Invoices\Entities\ClientTransaction;
use Modules\Invoices\Entities\InvoiceDetails;
use Modules\Invoices\Entities\InvoiceDetailsVats;
use Modules\Invoices\Entities\ItemSerial;
use Modules\Invoices\Entities\ItemTransaction;
use Modules\Invoices\Entities\ItemUnit;
use Modules\Invoices\Http\Requests\PendingInvoiceRequest;
use Modules\Manufacture\Http\Services\ManufactureStoreService;
use Modules\Settings\Entities\BadrShop;
use Modules\Invoices\Entities\SaleDetails;
use Modules\Invoices\Http\Requests\NewInvoiceRequest;
use Modules\Invoices\Entities\Einvoices;
use Mockery\Generator\StringManipulation\Pass\Pass;
use Modules\Invoices\Entities\InvoiceSettings;
use Modules\Invoices\Http\Controllers\Invoices\Invoice;
use Modules\Item\Entities\Category;
use Modules\Item\Entities\ItemPrice;
use Modules\Item\Entities\PriceList;
use App\Services\Einvoice\Einvoice;
use Modules\EInvoice\Entities\Settings;
use Modules\Invoices\Entities\Client;
use Modules\Invoices\Entities\Request as EntitiesRequest;
use Modules\Invoices\Entities\User;
use PHPUnit\Exception;
use Salla\ZATCA\GenerateQrCode;
use Salla\ZATCA\Tags\InvoiceDate;
use Salla\ZATCA\Tags\InvoiceTaxAmount;
use Salla\ZATCA\Tags\InvoiceTotalAmount;
use Salla\ZATCA\Tags\Seller;
use Salla\ZATCA\Tags\TaxNumber;

class InvoicesController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    private ?EntitiesRequest $pendingInvoice = null;
    public function index(string $invoice_type, Request $request)
    {

        $user = auth()->user();
        $users = User::all();
        $groupIds = $user->groups->pluck('id');
        $thumbprint = Settings::getThumbprint();
        $invoices = Invoices::leftJoin('clients', function ($join) use ($groupIds, $invoice_type) {
            if (auth()->user()->clients_group == 0) {
                $join->on('clients.id', '=', 'invoices_.client_id');
                if ($invoice_type  ==  'sales' && auth()->user()->clients_group !== 0) {
                    $join->join('clients_groups', function ($join) {
                        $join->on('clients_groups.id', 'clients.group_id')->where('clients_groups.valid', '=', '1');
                    });
                }
            } else {
                $join->on('clients.id', '=', 'invoices_.client_id');
                $join->whereIn('clients.group_id', $groupIds);
            }
        })

            ->with('einvoice:invoice_id,publicUrl')
            ->where('invoices_.shop_id', get_session_val('serial_id', 'integer'))
            ->where('invoices_.invoice_type', ($this->detectInvoiceType($invoice_type))
                ->getInvoiceTypeNo())
            ->orderBy('invoices_.id', 'desc')->select('invoices_.*', 'clients.client_name');
        if ($request->has('user_id') && !blank($request->get('user_id'))) {
            if ($request->get('user_id') != -1) {
                $invoices = $invoices->where('invoices_.add_user', $request->user_id);
            }
        }

        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();

        if ($request->has('search') && !in_array($request->search, ['', ' ', null])) {
            $invoices = $invoices->where(function ($query) use ($request) {
                $query
                    ->where('clients.client_name', 'like', '%' . $request->search . '%')
                    ->orWhere('invoices_.bill_no', 'like', '%' . $request->search . '%');
            });
        }
        if ($request->has('dateFrom') && !in_array($request->dateFrom, ['', ' ', null])) {
            $invoices = $invoices->whereDate('invoice_date', '>=', $request->dateFrom);
        }

        if ($request->has('dateTo') && !in_array($request->dateTo, ['', ' ', null])) {
            $invoices = $invoices->whereDate('invoice_date', '<=', $request->dateTo);
        }

        if ($request->has('dateTo') && !blank($request->get('dateTo'))) {
            $invoices = $invoices->whereDate('invoice_date', '<=', $request->dateTo);
        }

        if ($request->has('sync_type') && !blank($request->get('sync_type'))) {
            if ($request->get('sync_type') == 0) {
                $invoices = $invoices->whereIn('submissionId', ['', ' '])->whereIn('invoice_type', [1, 3]);
            } else {
                $invoices = $invoices->whereNotIn('submissionId', ['', ' ']);
            }
        }

        $invoices = $invoices->withCount('backInvoice')->with('childInvoiceDetails', 'InvoiceDetails')->paginate(20);
        return view('invoices::index', compact('invoices', 'invoice_type', 'settings', 'thumbprint', 'users'));
    }

    public function setThumbprint()
    {
        try {
            Settings::first()->update(['thumbprint' => request()->get('thumbprint')]);
            return response()->json(['message' => 'تم تحديث الختم بنجاح'], 200);
        } catch (Exception $e) {
            return response()->json(['message' => 'حدث خطأ أثناء تحديث الختم'], 500);
        }
    }

    /**
     * Show the form for creating a new resource.
     * @return Response
     */
    public function create(string $invoice_type, Request $request)
    {
        $pendingInvoice = $this->pendingInvoice;

        $thumbprint = Settings::getThumbprint();
        $stores = Store::where('shop_id', get_session_val('serial_id', 'Integer'))
            ->when(!authorized('can_change_invoice_store'), function ($q) {
                $q->limit(1);
            })->get();

        $bill_add = BillAdd::where('shop_id', get_session_val('serial_id', 'Integer'))->whereIn('add_role', ['0', '2', '3'])->get();
        $units = Unit::where('shop_id', get_session_val('serial_id', 'Integer'))->get();
        $items = Item::where('shop_id', get_session_val('serial_id', 'Integer'))
            ->whereRaw("barcode regexp '^[0-9]+$'")->orderByRaw('CAST(barcode as UNSIGNED) DESC')->take(1)->get();
        $classifications = ItemType::where('shop_id', get_session_val('serial_id', 'Integer'))
                                ->when(authorized('define_categories'), function($query) {
                                    $query->whereIn('id', getUserCategoriesId());
                                })->get();

        $sale_points = SalePoint::leftJoin('currencies', 'sale_points.currency_id', '=', 'currencies.id')
            ->where('sale_points.shop_id', get_session_val('serial_id', 'Integer'))
            ->when(Auth::user()->show_points == 1 , function ($q) {
                $q->where('sale_points.id', get_session_val('sale_point', 'Integer', null, null, Auth::user()->id, 'user'));
            })
            ->when(Auth::user()->show_points == 2 , function ($q) {
                $q->where('sale_points.id', get_session_val('sale_point', 'Integer', null, null, Auth::user()->id, 'user'));
            })
            ->when(Auth::user()->show_points == 3 , function ($q)
            {
                $point = Auth::user()->sale_point;
                $store_id = SalePoint::where('sale_points.shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->first()->store_id;
                $q
                ->where('sale_points.store_id', $store_id);
            })
            ->select(
                'money_point',
                'sale_points.id as point_id',
                'point_name',
                'currency_id',
                'name',
                'value',
                'currencies.name as currency',
                'point_type'
            )
            ->get();

        // depends on type
        $invoice = $this->detectInvoiceType($invoice_type);

        $clients = Client::query()->perGroup();

        //  فلاتر الاضافه في الفواتير
        if (in_array($invoice_type, ['sales', 'returned sales', 'sales offers'])) {
            $clients = $clients->whereIn('clients.client_role', [0, 2])
                    ->leftJoin('clients_groups', function ($e) {
                        $e->on('clients_groups.id', '=', 'clients.group_id')
                            ->where('clients_groups.valid', 1)
                            ->where('clients_groups.shop_id', get_session_val('serial_id', 'integer'));
                    })
                    ->where(function ($r) {
                        $r->where('clients.group_id', '==', '0');
                        $r->orWhere(function ($s) {
                            $s->where('clients_groups.id', '>', '0')->where('clients_groups.valid', 1);
                        });
                    })->select('clients.*', 'clients_groups.group_discount');
        } else if (in_array($invoice_type, ['purchases', 'returned purchases'])) {
            $clients = $clients->whereIn('client_role',  [1, 2]);
        }

        $clients = $clients->withCount('sales')->get();

        [$bg_color] = $invoice->create();
        $invoice_vat_use = ($this->detectInvoiceType($invoice_type))->getVatUse();

        $invoices = Invoices::with('client')->where([
            ['shop_id', get_session_val('serial_id', 'integer')],
            ['invoice_type', $invoice->getInvoiceTypeNo()]
        ])->whereDate('transaction_date', '=', date('Y-m-d'))->orderBy('id', 'desc')->limit(20)->get();

        $currency = get_session_val('currency', 'string');
        $invoice = null;
        $invoiceItemIds = null;
        $invoiceNeededCategories = null;
        $invoiceItems = null;
        if ($request->id) {
            $invoice = Invoices::with([
                'InvoiceDetails.Item.subUnits.Unit',
                'InvoiceDetails.Item.Unit',
                'InvoiceDetails.Item.prices',

                'InvoiceDetails.Item.vats.invoiceAdd',
                'InvoiceDetails.Item.availableQty',
                'vats',
                'Store',
                'childInvoiceDetails',
                'InvoiceDetails.SoldSerials',
            ])
                ->where('shop_id', get_session_val('serial_id', 'integer'))
                ->with('client', function ($q){
                    $q->withCount('sales');
                })
                ->findOrFail($request->id);

            $invoiceDetails = $invoice->InvoiceDetails()->with(['item', 'item.category', 'item.category.parentCategory'])->get();
            $invoiceItemIds = $invoiceDetails->pluck('item_id')->toArray();
            $invoiceItems = $invoiceDetails->keyBy('item_id');
            $invoiceNeededCategories = [];
            $invoiceDetails->each(function ($detail) use (&$invoiceNeededCategories) {
                $invoiceNeededCategories = array_merge($invoiceNeededCategories, $this->recursiveReturnCategoryId($detail->item?->category));
            })->toArray();
        }
        $categories = $multi_prices_lists_names = collect();
        if (get_session_val('screen_type', 'integer') == 1) {
            $categories = Category::with('pressSubCategory')
                ->where([
                    ['shop_id', get_session_val('serial_id', 'integer')],
                    ['category_id', 0],
                    ['hiddentouch',0]

                ])
                ->get();
                //->dd()
            ;

            $multi_prices_lists_names = PriceList::where('shop_id', get_session_val('serial_id', 'integer'))->select(['id', 'list_name'])->get();
        }

        $add_role = optional($bill_add->first())->add_role ?? 3;
        $Obarcode = $items->first()->barcode ?? 0;
        $barcode = $Obarcode + 1;
        $currency = get_session_val('currency', 'string');

        $regions  = collect();
        if (get_session_val('delivery_fee_type', 'integer') == 1) {
            $regions = Region::where('shop_id', get_session_val('serial_id', 'integer'))
                ->select('id', 'city_name', 'fee')
                ->get();
        }

        $services = collect();
        $periods  = collect();
        if (get_session_val('delivery_fee_type', 'integer') == 2) {
            $services = BillService::where('shop_id', get_session_val('serial_id', 'integer'))
                ->select('id', 'name', 'price', 'enabled')
                ->get();

            $periods = Period::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('with_periods', 1)
                ->select('id', 'name', 'fee', 'with_periods')
                ->get();
        }

        $delivery_men = collect();
        if (get_session_val('home_delivery', 'integer') == 2) {
            $delivery_men = DeliveryMan::where('shop_id', get_session_val('serial_id', 'integer'))
                ->select('id', 'name')
                ->get();
        }

        $max_bill_number = maxInvoiceNumber($invoice_type, $stores->first()->id);
        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
        return view('invoices::create', compact(
            'max_bill_number',
            'invoices',
            'invoice',
            'items',
            'invoice_type',
            'clients',
            'categories',
            'settings',
            'stores',
            'bill_add',
            'units',
            'classifications',
            'sale_points',
            'add_role',
            'barcode',
            'bg_color',
            'currency',
            'invoice_vat_use',
            'thumbprint',
            'multi_prices_lists_names',
            'regions',
            'services',
            'periods',
            'delivery_men',
            'pendingInvoice',
            'invoiceItemIds',
            'invoiceItems',
            'invoiceNeededCategories',
        ));
    }

    public function recursiveReturnCategoryId(?Category $category){
        if (!$category) return [];
        $array = [$category->id];
        if($category->parentCategory){
            $array = array_merge($array, $this->recursiveReturnCategoryId($category->parentCategory));
        };
        return $array;
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Response
     */
    public function store(NewInvoiceRequest $request, $invoice_type)
    {
        if($request->get('pendingInvoice')){
            $pendingInvoice = User::find(auth()->user()->id)->requests()->findOrFail($request->get('pendingInvoice'));
            $pendingInvoice->details()->delete();
            $pendingInvoice->delete();
        }

        $invoice_controller = ($this->detectInvoiceType($invoice_type));
        $response = $invoice_controller->store($request, $invoice_type);

        if ($response['status']) {
            // ShareInvoiceController::mail(\Illuminate\Support\Facades\Crypt::encrypt($response['id']));
            // $whatsapp_data = ShareInvoiceController::whatsApp($request->client_id, route('show.invoice', encrypt($response['id'])), false);

            session()->flash('send-invoice-as-mail', route('invoices.send-email', $response['id']));
            session()->flash('whatsapp_redirect', route('send.whatsapp.message', ['invoice' => $response['id'], 'client' => $request->client_id, 'forceSend' => false]));

            (new ManufactureStoreService)->createFromBill($response['id'], $invoice_controller->reshaped_ingrediants);

            if (get_session_val('einvoice', 'Integer') == 1 && !blank($request->cert_token) && get_session_val('submit_einvoice', 'Integer') == 0) {
                $requestEinvoice      = new Request();
                $requestEinvoice->ids = [$response['id']];
                $requestEinvoice->cert_token = $request->cert_token;

                if ($this->submitInvoice($requestEinvoice, true)) {

                    if (isset($request->reference_id)) {
                        return redirect()->route('invoices.new', compact('invoice_type'))->with([
                            'success' => trans('invoices.success_create_bill'), 'status' => true, 'invoice_id' => $response['id'],
                            'success' => 'تم الحفظ في الضرائب'
                        ]);
                    }

                    return redirect()->back()->with([
                        'success' => trans('invoices.success_create_bill'), 'status' => true, 'invoice_id' =>  $response['id'],
                        'success' => 'تم الحفظ في الضرائب',
                        'preview' => true
                    ]);
                } else {
                    return redirect()->back()->with([
                        'success' => trans('invoices.success_create_bill'), 'status' => true, 'invoice_id' =>  $response['id'],
                        'error' => 'لم يتم الحفظ في الضرائب'
                    ]);
                }
            }

            if (isset($request->reference_id)) {
                return redirect()->route('invoices.new', compact('invoice_type'))->with(['success' =>trans('invoices.success_create_bill'), 'status' => true, 'invoice_id' => $response['id']]);
            }

            return redirect()->route('invoices.new', compact('invoice_type'))->with(['success' => trans('invoices.success_create_bill'), 'status' => true, 'invoice_id' => $response['id']]);
        } else {
            return redirect()->route('invoices.new', compact('invoice_type'))->withErrors($response['msg']);
        }
    }

    public function checkBillNumber(Request $request)
    {
        $generate_by = get_session_val('billno_show', 'integer');
        $bill_number = $request->get('bill_no');

        if ($generate_by != 1) return response()->json(['status' => true], 200);


        $check = Invoices::select('bill_no')->where('bill_no', $bill_number)->where('shop_id', get_session_val('serial_id', 'Integer'))
                        ->where('invoices_.invoice_type', convertInvoiceTypeNameToNumber($request->get('invoice_type', 'sale')))
                        ->exists();

        return $check
                ? response()->json(['status' => false], 500)
                : response()->json(['status' => true], 200);
    }

    /**
     * Show the specified resource.
     * @return Response
     */
    public function show($id, $use_email = false)
    {
        $invoice = Invoices::where('shop_id', get_session_val('serial_id', 'integer'))->with('client:id,client_name,balance,mobile1,email', 'InvoiceDetails.unit')
            ->withSum('vats', 'vat_weight')->findOrFail($id);
        $total_basic_price = 0;

        $invoice->qr = self::getQrCode($invoice);
        foreach ($invoice->InvoiceDetails as $index => $detail) {
            $item = Item::find($detail->item_id);

            /**
             * Invoice type:
             * (invoice_type = 0) => فاتورة شراء , (invoice_type = 1) => فاتورة بيع
             * (invoice_type = 2) => فاتورة مرتجع شراء , (invoice_type = 3) => فاتورة مرتجع بيع
             */
            // $item_price = in_array($invoice->invoice_type, [0, 2]) ? $detail->pay_price : $detail->basic_sale_price;

            if (in_array($invoice->invoice_type, [0, 2])) {
                if ($invoice->invoice_total_disc_status == 1) {
                    $total_basic_price += ($detail->basic_pay_price * $detail->quantity);
                    $detail['item_price'] = $item->basic_pay_price ?? 0;
                } else {
                    $total_basic_price += ($detail->pay_price * $detail->quantity);
                    $detail['item_price'] = $item->pay_price ?? 0;
                }
            } else {
                if ($invoice->invoice_total_disc_status == 1) {
                    if ($detail->discount > 0) {
                        $total_basic_price += ($detail->basic_sale_price * $detail->quantity);
                        $detail['item_price'] = $item->basic_sale_price ?? 0;
                    } else {
                        $total_basic_price += ($detail->sale_price * $detail->quantity);
                        $detail['item_price'] = $item->sale_price ?? 0;
                    }
                } else {
                    $total_basic_price += ($detail->sale_price * $detail->quantity);
                    $detail['item_price'] = $item->sale_price ?? 0;
                }
            }

            /**
             * Item Discount
             * (discount_type = 0) => نسبة , (discount_type = 1) => مبلغ
             */
            // $discount  = ($detail->discount_type == 1) ? $detail->discount : ($detail->discount * $item_price) / 100;
            // $discount *= $detail->quantity;

            // Add to invoice details.
            $detail['item_name'] = $item->item_name ?? '';
            $detail['item_about'] = $item->about ?? '';

            // Add to invoice.
            // $invoice->totalDiscount += $discount;
        }

        $invoice->totalDiscount = $invoice->invoice_total_disc_type == 1 ? (float)$invoice->invoice_total_disc : (float)$total_basic_price * ((float)$invoice->invoice_total_disc / 100);
        /**
         * Invoice type:
         * (invoice_type = 0) => فاتورة شراء , (invoice_type = 1) => فاتورة بيع
         * (invoice_type = 2) => فاتورة مرتجع شراء , (invoice_type = 3) => فاتورة مرتجع بيع
         */
        $invoice->type = $invoice->invoice_type == 0
            ? trans('invoices.type.purchase')
            : ($invoice->invoice_type == 1
                ? trans('invoices.type.sales')
                : ($invoice->invoice_type == 2
                    ? trans('invoices.type.purchase_back')
                    : ($invoice->invoice_type == 3 ? trans('invoices.type.sales_back') : trans('invoices.type.sales_offers'))));

        if ($use_email) {
            return response()->json(['invoice' => $invoice]);
        }

        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();

        return view('invoices::show', compact('invoice', 'settings'));
    }

    public static function getQrCode($invoice)
    {
        try {
            $shop_id = $invoice->shop_id;

            $vat_number = DB::table('bill_setting')->where('shop_id', $shop_id)->first()->vat_number ?? '';
            $shop = DB::table('badr_shop')->where('serial_id', $shop_id)->first();
            $shop_name = $shop->shop_name ?? '';
            $p_scale = $shop->decimal_num_price ?? 0;

            if ($invoice->type == 1) {
                if ($invoice->store_id > 0) {
                    $store_details = DB::table('stores')->select('tax_number', 'store_name')->where([['shop_id', $shop_id], ['id', $invoice->store_id]])->first();
                    $vat_number = $store_details->tax_number;
                    $shop_name = $store_details->store_name;
                }
            }

            // data:image/png;base64, .........
            $qrcode = GenerateQrCode::fromArray([
                new Seller($shop_name), // seller name
                new TaxNumber($vat_number), // seller tax number
                new InvoiceDate($invoice->transaction_date), // invoice date as Zulu ISO8601 @see https://en.wikipedia.org/wiki/ISO_8601
                new InvoiceTotalAmount( round_money($invoice->net_price) ), // invoice total amount
                new InvoiceTaxAmount( round_money($invoice->vats_sum_vat_weight) ) // invoice tax amount
            ])->render();

            return view('QR-code', compact('qrcode'))->render();
        } catch(\Exception $e) {
            return '';
        }
    }

    /**
     * Show the form for editing the specified resource.
     * @return Response
     */
    public function edit()
    {
        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
        return view('invoices::edit', compact('settings'));
    }

    /**
     * Update the specified resource in storage.
     * @param  Request $request
     * @return Response
     */
    public function update(Request $request)
    {
    }

    /**
     * Remove the specified resource from storage.
     * @return Response
     */
    public function destroy()
    {
    }

    private function detectInvoiceType($invoice_type)
    {
        switch (strtolower($invoice_type)) {
            case 'purchases':
                return new PurchaseController();
            case 'returned purchases':
                return new ReturnedPurchasesController();
            case 'returned sales':
                return new ReturnedSalesController();
            case 'sales offers':
                return new SellOfferController();
            default:
                return new SalesController();
        }
    }

      public function search(Request $request, $invoice_type)
    {
        $store_id = $request->store_id;

        $select_vat_status = 'purchase_vat_state';
        if ($invoice_type == 'sales' or $invoice_type == 'returned sales' or $invoice_type == 'sales offers') {
            $select_vat_status = 'vat_state';
        }

        $select = 'SELECT TRUNCATE( store_items.store_quant, ' . get_session_val('decimal_num_quant', 'Integre') . ')
            FROM store_items WHERE store_items.item_id = items.id AND store_items.store_id = ' . $store_id . '
            ';

        $manufacturing_val = '1';
        if ((int)$request->ingrediants > 0) {
            $manufacturing_val = '2';
        }
        // $select = '   TRUNCATE( store_items.store_quant, '. get_session_val('decimal_num_quant', 'Integre') .')';
        // dd((int)$request->ingrediants, $manufacturing_val);

        if (!is_null($request->item_name)) {
            $item_name = $request->item_name;
            $items = Item::where(function ($q) use ($item_name, $invoice_type) {
                $q->where('item_name', 'like', '%' . $item_name . '%')
                    ->orWhere('about', 'like', '%' . $item_name . '%')
                    ->orWhere('purchase_code', 'like', '%' . $item_name . '%');
            })->where('available', 1)
            ->when(authorized('define_categories'), function($query) {
                $query->whereIn('items.sale_unit', getUserCategoriesId());
            })

                ->when((($invoice_type == 'sales' || $invoice_type == 'returned purchases') & get_session_val('sale_details', 'Integer') == 0), function ($q) use ($store_id, $manufacturing_val, $invoice_type) {
                    $q->leftJoin('store_items', 'store_items.item_id', 'items.id');

                    if ($manufacturing_val == '2') {
                        $q->Where('items.manufacturing', $manufacturing_val);
                    } else {

                        if (get_session_val('manufacturing', 'integer') == 2 && in_array($invoice_type, ['sales', 'returned sales']) && get_session_val('definite_while_selling', 'integer') == 1) {
                            $q->Where(function ($q) use ($store_id, $manufacturing_val) {
                                $q->where(function ($qq) use ($store_id, $manufacturing_val) {
                                    $qq->where('store_items.store_id', $store_id);
                                    $qq->where('items.manufacturing', '0');
                                });
                                $q->orWhere('items.is_service', 1);
                                $q->orWhere('items.manufacturing', $manufacturing_val);
                            });
                        } else {
                            $q->Where(function ($q) use ($store_id, $manufacturing_val) {
                                $q->where('store_items.store_id', $store_id);
                                $q->orWhere('items.is_service', 1);
                            });
                        }
                    }
                })
                ->when(($invoice_type == 'sales' || $invoice_type == 'returned sales' || $invoice_type == 'sales offers'),
                    function ($q) {
                        $q->join('items_type', function ($join) {
                            $join->on('items_type.id', 'items.sale_unit');
                            $join->where('items_type.hiddentouch', '!=', '1');
                        });
                    }
                )

                ->leftJoin('item_vat', 'item_vat.item_id', 'items.id')
                ->leftJoin('bills_add', 'bills_add.id', 'item_vat.vat_id')
                ->leftjoin('companies', 'companies.id', '=', 'items.company_id')
                ->where('items.shop_id', get_session_val('serial_id', 'Integer'))
                ->select(DB::raw('
                    items.id , items.pay_price, items.sale_price, items.unit_id, items.about,
                items.require_serial, items.item_name, items.about, items.lowest_price,
                items.' . $select_vat_status . ' AS vat_state, purchase_code, income_ratio,
                (' . $select . ')  AS store_quant , expire_date,
                GROUP_CONCAT(item_vat.vat_id) AS vats,
                GROUP_CONCAT(bills_add.addition_value) AS vats_values,
                GROUP_CONCAT(bills_add.add_role) AS vats_usage_status,
                GROUP_CONCAT(bills_add.check_addition) AS vats_types,
                GROUP_CONCAT(item_vat.vat_type) AS vats_opers ,
                GROUP_CONCAT(item_vat.vat_use) AS vats_usage ,
                items.purchase_vat_state,
                items.is_service,
                items.manufacturing
            '))
                ->groupBy('items.id')
                // ->orderByRaw('LENGTH(items.item_name) ASC')
                // ->toSql();
                // dd($items, $manufacturing_val, $request->item_name);
                ->limit(12)->get();

            if ($manufacturing_val == '2') {
                $html = view('invoices::layouts.search.search_ingrediants', compact('items', 'store_id'))->render();
            } else {
                $html = view('invoices::layouts.search.search_result', compact('items', 'store_id'))->render();
            }

            if (count($items)) {
                return response()->json([
                    'status' => '1',
                    'data' => $html,
                ]);
            } else {
                return response()->json([
                    'status' => '0',
                    'data' => $html,
                ]);
            }
        } else {
            return response()->json([
                'status' => '0',
                'data' => '',
            ]);
        }
    }

    public function itemUnits(Request $request)
    {
        $id = $request->item_id;
        $item = Item::find($id);

        $all_units = $item->units();
        $data = '';

        foreach ($all_units as $unit_) {
            $selected = '';
            if ($unit_->unit_id == $item->unit_id) {
                $selected = 'selected';
            }
            $data .= '<option ' . $selected . ' value="' . $unit_->unit_id . '" data-price="' . $unit_->price . '" data-unit-val="' . ($unit_->unit_value ?? 1) . '"
            data-sale="' . $unit_->sale . '" data-item="' . $id . '" data-low="' . $item->lowest_price . '" data-expire="' . $item->expire_date . '">' . $unit_->name . '</option>';
        }

        return response()->json([
            'status' => '1',
            'data' => $data
        ]);
    }

    public function barcodeSearch(Request $request, $invoice_type)
    {
        $barcode = $request->barcode;
        $store_id = $request->store_id;

        if ($request->embedded) {
            $barcode = Cache::get('embedded_item_' . Auth::id()) ?? $request->barcode;
            Cache::forget('embedded_item_' . Auth::id());
        }


        $item = Item::query();
        if ($request->has('item_id') && $request->item_id > 0) {
            $item = $item->where('items.id', $request->item_id);
        } else {
            $item = $item->where('barcode', $barcode);
        }

        $select_vat_status = 'purchase_vat_state';
        if ($invoice_type == 'sales' or $invoice_type == 'returned sales' or $invoice_type == 'sales offers') {
            $select_vat_status = 'vat_state';
        }

        $select = "SELECT TRUNCATE( store_items.store_quant, ".get_session_val('decimal_num_quant', 'Integre').")
            FROM store_items WHERE store_items.item_id = items.id AND store_items.store_id = $store_id limit 1";

        // $select = ' TRUNCATE( store_items.store_quant, '. get_session_val('decimal_num_quant', 'Integre') .')';

        $item = $item->where('items.shop_id', get_session_val('serial_id', 'Integer'))
            ->where('available', 1)
            ->when((($invoice_type == 'sales' || $invoice_type == 'returned purchases') && get_session_val('sale_details', 'Integer') == 0), function ($q) use ($store_id) {
                $q->leftJoin('store_items', 'store_items.item_id', 'items.id');
                $q->Where(function ($q) use ($store_id) {
                    $q->where('store_items.store_id', $store_id);
                    $q->orWhere('items.is_service', 1);
                    $q->orWhere('items.manufacturing', '1');
                });
            })
            ->leftJoin('item_vat', 'item_vat.item_id', 'items.id')
            ->leftJoin('bills_add', 'bills_add.id', 'item_vat.vat_id')
            ->leftjoin('companies', 'companies.id', '=', 'items.company_id')
            ->where('items.shop_id', get_session_val('serial_id', 'Integer'))
            ->selectRaw(
                '
                items.id, items.pay_price, items.sale_price, items.unit_id, items.about, items.expire_date,
                items.require_serial, items.item_name, items.about, items.lowest_price, items.is_service, items.manufacturing,
                items.' . $select_vat_status . ' AS vat_state, purchase_code, income_ratio,
                GROUP_CONCAT(item_vat.vat_id) AS vats,
                GROUP_CONCAT(bills_add.addition_value) AS vats_values,
                GROUP_CONCAT(bills_add.add_role) AS vats_usage_status,
                GROUP_CONCAT(bills_add.check_addition) AS vats_types,
                GROUP_CONCAT(item_vat.vat_type) AS vats_opers,
                GROUP_CONCAT(item_vat.vat_use) AS vats_usage ,
                items.purchase_vat_state ,(' . $select . ')  AS store_quant '
            )
            ->groupBy('items.id')
            ->first();

        if ($item) {

            $qty = $item->store_quant ?? 0;

            $unit = $item->unit_id;
            $units = $item->units();
            $unitsOptions = '';
            foreach ($units as $unit_) {
                $selected = '';
                $pay = $item->pay_price * $unit_->unit_value;
                if ($item->unit_id == $unit_->unit_id) {
                    $selected = 'selected';
                    $pay = $item->pay_price;
                }
                $unitsOptions .= '<option ' . $selected . ' value="' . $unit_->unit_id . '" data-price="' . $pay . '" data-unit-val="' . ($unit_->unit_value ?? 1) . '"
                    data-sale="' . $unit_->sale . '" data-item="' . $item->id . '" data-detail="" data-low="' . $item->lowest_price . '" data-expire="' . $item->expire_date . '">' . $unit_->name . '</option>';
            }

            return response()->json([
                'status' => true,
                'item_id' => $item->id,
                'purchase_code' => $item->purchase_code,
                'name' => $item->item_name,
                'sale' => round_money($item->sale_price),
                'pay' => round_money($item->pay_price),
                'low' => round_money($item->lowest_price),
                'expire' => $item->expire_date,
                'vat_state' => $item->vat_state,
                'vats_ids' => $item->vats,
                'vats_values' => $item->vats_values,
                'vats_types' => $item->vats_types,
                'vats_opers' => $item->vats_opers,
                'vats_usage' => $item->vats_usage,
                'vats_usage_status' => $item->vats_usage_status,
                'unit' => $unit,
                'units' => $unitsOptions,
                'qty' => round_quantity($qty),
                'company_income' => $item->income_ratio ?? 0,
                'serial' => $item->require_serial,
                'is_service' => $item->is_service
            ]);
        } else {
            # no main item at all

            $unit_sub = ItemUnit::where('items_unit.unit_barcode', $barcode)
                ->where('items_unit.shop_id', get_session_val('serial_id', 'Integer'))
                ->join('items', 'items.id', 'items_unit.item_id')
                ->leftJoin('item_vat', 'item_vat.item_id', 'items.id')
                ->leftJoin('bills_add', 'bills_add.id', 'item_vat.vat_id')
                ->selectRaw(
                    '
                    items_unit.item_id, items.pay_price * items_unit.unit_value, items_unit.unit_price as sale_price,
                    items_unit.unit_value, items_unit.unit_id, items.about, items.expire_date,
                    items.require_serial, items.item_name, items.about, items.lowest_price,
                    items.' . $select_vat_status . ' AS vat_state, items.purchase_code,
                    GROUP_CONCAT(item_vat.vat_id) AS vats,
                    GROUP_CONCAT(bills_add.addition_value) AS vats_values,
                    GROUP_CONCAT(bills_add.check_addition) AS vats_types,
                    GROUP_CONCAT(item_vat.vat_type) AS vats_opers,
                    GROUP_CONCAT(bills_add.add_role) AS vats_usage_status,
                    GROUP_CONCAT(item_vat.vat_use) AS vats_usage ,
                    items.purchase_vat_state ,(' . $select . ')  AS store_quant '
                )->first();

            if ($unit_sub) {
                $unit = $unit_sub->unit_id;
                $item = Item::find($unit_sub->item_id);

                if ($item) {

                    $qty = 0;
                    $store_items = StoreItem::where('item_id', $item->id)
                        ->where('shop_id', get_session_val('serial_id', 'Integer'))
                        ->where('store_id', $store_id)
                        ->first();
                    $qty += $store_items->store_quant ?? 0;

                    $units = $item->units();
                    $unitsOptions = '';
                    $pay = 0;
                    $sale = 0;
                    foreach ($units as $unit_) {
                        $selected = '';
                        if ($unit == $unit_->unit_id) {
                            $selected = 'selected';
                            $pay = $unit_->price;
                            $sale = $unit_->sale;
                        }
                        $unitsOptions .= '<option ' . $selected . ' value="' . $unit_->unit_id . '" data-price="' . $pay . '" data-unit-val="' . ($unit_->unit_value ?? 1) . '"
                            data-sale="' . $unit_->unit_price . '" data-item="' . $item->id . '" data-detail="" data-low="' . $item->lowest_price . '" data-expire="' . $item->expire_date . '">' . $unit_->name . '</option>';
                    }
                    return response()->json([
                        'status' => true,
                        'item_id' => $unit_sub->item_id,
                        'name' => $unit_sub->item_name,
                        'sale' => round_money($sale),
                        'pay' => round_money($pay),
                        'low' => round_money($unit_sub->lowest_price),
                        'expire' => $unit_sub->expire_date,
                        'purchase_code' => $unit_sub->purchase_code,
                        'vat_state' => $unit_sub->vat_state,
                        'vats_ids' => $unit_sub->vats,
                        'vats_values' => $unit_sub->vats_values,
                        'vats_types' => $unit_sub->vats_types,
                        'vats_opers' => $unit_sub->vats_opers,
                        'vats_usage' => $unit_sub->vats_usage,
                        'vats_usage_status' => $item->vats_usage_status,
                        'unit' => $unit,
                        'units' => $unitsOptions,
                        'qty' => round_quantity($qty),
                        'company_income' => $item->income_ratio ?? 0,
                        'serial' => $item->require_serial,
                        'is_service' => $item->is_service
                    ]);
                } else {
                    return response()->json([
                        'status' => false,
                        'msg' => $request->embedded == true ? '' : trans('native.item_recognition_error')
                    ]);
                }
            }
            return response()->json([
                'status' => false,
                'msg' => $request->embedded == true ? '' : trans('native.item_recognition_error')
            ]);
        }
    }

    public function ItemsData(Request $request)
    {
        $items = Item::join('multi_supplier', function ($q) use ($request) {
            $q->on('items.id', '=', 'multi_supplier.item_id');
            $q->where('multi_supplier.supplier_id', $request->supplier_id);
        })
            ->select(
                'items.id',
                'items.item_name',
                'items.pay_price',
                'items.sale_price',
                'items.purchase_vat_state as vat_state',
                'items.purchase_vat_id as vat_id',
                'items.unit_id',
                'items.require_serial',
                'items.purchase_code'
            )
            ->where('items.shop_id', get_session_val('serial_id', 'integer'))
            ->where('items.is_service', 0)
            ->get();
        $html = view('invoices::layouts.search.items-data', compact('items'))->render();
        if (count($items)) {
            return response()->json([
                'status' => '1',
                'data' => $html,
            ]);
        } else {
            return response()->json([
                'status' => '0',
                'data' => '<option selected>لا يوجد اصناف لهذا المورد</option>',
            ]);
        }
    }

    public function checkSerial(Request $request, $invoice_type)
    {
        $check = ItemSerial::where([
            'shop_id' => get_session_val('serial_id', 'Integer'),
            'serial' => $request->serial
        ])->when($invoice_type == 'sales' || $invoice_type == 'returned purchases', function ($q) {
            $q->where('sold', 0);
        })
            ->when($invoice_type == 'returned sales', function ($q) {
                $q->where('sold', 1);
            })
            ->first();

        if ($check) {
            return \response()->json([
                'status' => '1',
                'data' => $check
            ]);
        } else {
            return \response()->json([
                'status' => '0',
            ]);
        }
    }

    public function payAverage(Request $request)
    {
        $total = avg_pay_price($request->item_id, $request->new_qty, $request->new_pay, $request->new_unit_value);

        if ($total) {
            return \response()->json([
                'status' => '1',
                'data' => $total
            ]);
        } else {
            return \response()->json([
                'status' => '0',
            ]);
        }
    }

    public function get_last_sale_price(Request $request)
    {
        $last_sale_price = 0.0;
        $invoice_type = $request->invoice_type;
        $item_id = $request->item_id;
        $unit_id = $request->unit_id;
        $client_id = $request->client_id;

        if ($client_id) {
            if ($item_id) {
                if ($unit_id) {
                    if ($invoice_type == 'sales' || $invoice_type == 'returned sales') {
                        $last_sale_price = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                            ->where([
                                ['invoices_.shop_id', get_session_val('serial_id', 'Integer')],
                                ['invoices_details.item_id', $item_id],
                                ['invoices_details.unit_id', $unit_id],
                                ['invoices_.client_id', $client_id]
                            ])
                            // ->when($invoice_type == 'purchases', fn($query) => $query->where('invoice_type', 0))
                            ->when($invoice_type == 'sales', fn($query) => $query->where('invoice_type', 1))
                            // ->when($invoice_type == 'returned purchases', fn($query) => $query->where('invoice_type', 2))
                            // ->when($invoice_type == 'returned sales', fn($query) => $query->where('invoice_type', 3))
                            ->orderBy('invoices_.id', 'DESC')
                            ->limit(1)
                            ->value('invoices_details.sale_price');
                    }
                }
            }
        }

        if ($last_sale_price) {
            return \response()->json([
                'status' => '1',
                'data' => doubleval($last_sale_price)
            ]);
        } else {
            return \response()->json([
                'status' => '0',
            ]);
        }
    }

    public function print($id, Request $request)
    {
        $data = ShareInvoiceController::getInvoice( encrypt( $id ) );
        $data['preview'] = isset($request->preview);
        
        $printing_template = $request->has('target') ? $request->get('target') : Auth::user()->invoice_print_template;
        $folder = $request->get('view', env('INVOICE_PRINT')) == 'bills' ? "bills" : "print";
        $view = $printing_template ? "invoices::$folder.tax_bill" : "invoices::$folder.view_price";
        return view($view, $data);

        $invoice = Invoices::with('collector')->when($id > 0, function ($q) use ($id) {
            $q->where('id', $id);
            $q->where('shop_id', get_session_val('serial_id', 'Integer'));
        })->when(isset($request->preview), function ($q) {
            $q->where('shop_id', 0);
        })->with('client:id,client_name,balance,mobile1,tele,email,address,client_tax_number', 'collector', 'InvoiceDetails.Unit', 'InvoiceDetails.Item', 'vats')->first() ?? new Invoices();;

        $invoice->qr = $this->getQrCode($invoice);

        $invoice_settings = InvoiceSettings::where('shop_id', get_session_val('serial_id', 'Integer'))->with('shop')->first() ?? new InvoiceSettings();
        $store_settings   = Store::where([['shop_id', get_session_val('serial_id', 'Integer')], ['id', $invoice->store_id]])->first() ?? new Store();

        if (get_session_val('printing_shop_info_src', 'integer')) {
            $header = $store_settings->bill_header ?? '';
            $footer = $store_settings->message_bill ?? '';
        } else {
            $header = $invoice_settings->bill_header ?? '';
            $footer = $invoice_settings->message_bill ?? '';
        }

        $title = __('invoices.invoice_title', ['invoice' => trans('invoices.' . $this->getInvoiceTypeFromNo($invoice->invoice_type) . '.title')]);


        $settings = BadrShop::with('getCountry')->where('serial_id', get_session_val('serial_id', 'integer'))->first();

        $preview = isset($request->preview);
        $printing_template = $request->get('target', Auth::user()->invoice_print_template);
        $view = $printing_template ? 'invoices::bills.tax_bill' : 'invoices::bills.view_price';

        return view($view, compact(
            'invoice',
            'preview',
            'invoice_settings',
            'title',
            'store_settings',
            'header',
            'footer',
            'settings'
        ));
    }

    private function getInvoiceTypeFromNo($no)
    {
        $types = [
            '0' => 'purchases',
            '1' => 'sales',
            '2' => 'returned purchases',
            '3' => 'returned sales',
            '23' => 'sales offers',
        ];

        return $types[$no] ?? '';
    }

    /**
     * Submit invoice to tax server (E-Invoice)
     *
     * @param Request $request
     * @param bool $bool_response
     * @return JsonResponse | bool
     */
    public function submitInvoice(Request $request, bool $bool_response = false): JsonResponse|bool
    {
        if (!empty($request->ids) && !blank($request->cert_token)) {
            $result = Einvoice::submitDocuments($request->ids, $request->cert_token);

            if ($bool_response) {
                return ($result->status() >= 200 && $result->status() < 300 && !empty($result->original['acceptedDocuments'])) ? true : false;
            }

            if ($result->status() >= 200 && $result->status() < 300) {
                if (!empty($result->original['acceptedDocuments']) && empty($result->original['rejectedDocuments'])) {
                    return response()->json(['status' => 'success', 'message' => __('einvoice.send_successfully')]);
                }

                $reasons     = [];
                $rejectedIds = [];

                if (!empty($result->original['rejectedDocuments'])) {
                    foreach ($result->original['rejectedDocuments'] as $rejectedDocument) {
                        $reasons['rejectedDocuments'][] = $rejectedDocument['error']['message'];
                    }
                }

                if (isset($result->original['rejectedBills']) && !empty($result->original['rejectedBills'])) {
                    $rejectedBills = $result->original['rejectedBills'];

                    if (isset($rejectedBills['failedBills']) && !empty($rejectedBills['failedBills'])) {
                        foreach ($rejectedBills['failedBills'] as $failedBill) {
                            $reasons['failedBills'][] = implode(', ', $failedBill);
                        }

                        // Push $rejectedBills['failedBills'] keys to $rejectedIds
                        $rejectedIds = array_merge($rejectedIds, array_keys($rejectedBills['failedBills']));
                    }

                    if (isset($rejectedBills['notFoundBills']) && !empty($rejectedBills['notFoundBills'])) {
                        foreach ($rejectedBills['notFoundBills'] as $notFoundBill) {
                            $reasons['notFoundBills'][] = implode(', ', $notFoundBill);
                        }

                        // Push $rejectedBills['notFoundBills'] keys to $rejectedIds
                        $rejectedIds = array_merge($rejectedIds, array_keys($rejectedBills['notFoundBills']));
                    }
                }

                // Get Difference between $rejectedIds and $request->ids if $rejectedIds not empty
                $rejectedIds = !empty($rejectedIds) ? array_diff($request->ids, $rejectedIds) : $rejectedIds;

                return response()->json([
                    'error'       => 'error',
                    'message'     => __('einvoice.some_documents_submitted_and_some_rejected'),
                    'reasons'     => $reasons,
                    'rejectedIds' => $rejectedIds
                ]);
            }

            return $this->getErrorResponse($result);
        }

        return response()->json(['status' => 'error', 'message' => __('einvoice.no_documents')]);
    }

    /**
     * This Method Return Error Response.
     * @param $result
     * @return JsonResponse
     */
    private function getErrorResponse($result): JsonResponse
    {
        if ($result->status() === 500) {
            $response = $result->original['error'] ? $result->original['error']->original : $result->original;
            return response()->json(['status' => 'failed', 'message' => $response ?? __('einvoice.send_failed')]);
        }

        if ($result->status() === 400 && $result->original['status'] === 'payload_error' && isset($result->original['payload_msg'])) {
            return response()->json(['status' => 'payload_error', 'message' => $result->original['payload_msg']]);
        }

        if ($result->status() === 422 && $result->original['status'] === 'rejectedBills' && $result->original['rejectedBills']) {

            if (!empty($result->original['rejectedBills']['failedBills'])) {
                foreach ($result->original['rejectedBills']['failedBills'] as $billNo => $failedBill) {
                    $message[] = __('einvoice.failedBills', ['billNo' => $billNo, 'reason' => implode(' , ', $failedBill)]);
                }
            }

            if (!empty($result->original['rejectedBills']['notFoundBills'])) {
                $message = __('einvoice.notFoundBills', [
                    'notFoundBills' => implode(' , ', $result->original['rejectedBills']['notFoundBills'])
                ]);
            }

            return response()->json(['status' => 'rejectedBills', 'message' => $message]);
        }

        $response = $result->original['rejectedDocuments'] ?? $result->original['error'] ?? $result->original;
        $response = is_array($response) && empty($response) ? __('einvoice.send_failed') : $response;
        return response()->json(['status' => 'error', 'message' => $response]);
    }

    public function getClients($invoice_type)
    {
        $invoice = $this->detectInvoiceType($invoice_type);
        $clients = clientGroups();

        if (in_array($invoice_type, ['sales', 'returned sales', 'sales offers'])) {
            $clients =  $clients->whereIn('clients.client_role', [0, 2]);
            if ($invoice_type  ==  'sales') {
                $clients =  $clients->leftjoin('clients_groups', 'clients_groups.id', 'clients.group_id')
                    ->select('clients.*',  'clients_groups.group_discount');
            }
        } else if (in_array($invoice_type, ['purchases', 'returned purchases'])) {
            $clients = $clients->whereIn('client_role',  [1, 2]);
        }

        [$bg_color] = $invoice->create();
        return response()->json([
            'status' => true,
            'clients' => $clients->get()
        ]);
    }

    public function getItemPrices(Request $request)
    {
        $details =  PriceList::join('items_prices', 'items_prices.list_id', 'prices_list.id')->where([
            ['items_prices.shop_id', get_session_val('serial_id', 'Integer')],
            ['items_prices.item_id', $request->id]
        ])->select(DB::raw("
            GROUP_CONCAT(prices_list.id) AS lists_ids,
            GROUP_CONCAT(prices_list.list_name) AS lists_names,
            GROUP_CONCAT(items_prices.price) AS lists_prices,
            GROUP_CONCAT(items_prices.list_quant) AS lists_qtys
        "))->first();

        return response()->json([
            'status' => true,
            'data' => $details
        ]);
    }

    public function sendWhatsAppMessage($invoice, $client)
    {
        try {
            $response = ShareInvoiceController::whatsApp($client, route('show.invoice', encrypt($invoice)), request()->get('forceSend', true));
            return response()->json($response, 200);
        } catch (Exception $e) {
            return response()->json(['status' => false], 500);
        }
    }

    public function collector(Request $request)
    {
        $client = Client::select('id', 'collector_id', 'client_type')->with('collector', 'type')->where('client_type', '!=', false)->where('shop_id', auth()->user()->shop_id)->find($request->get('id'));
        return $client
            ? response()->json(['collector' => $client->collector, 'type_name' => $client->type->name], 200)
            : response()->json(['status' => false], 500);
    }

    public function sendEmail($id)
    {
        $test = ShareInvoiceController::mail( encrypt($id) );
        dd($test);
    }

    public function updatePendingInvoiceData(string $invoice_type, PendingInvoiceRequest $request, $pendingInvoice)
    {
        $pendingInvoiceData = $request->validated();

        $pendingInvoice = Cache::remember("pending_invoice_{$invoice_type}_{$request->randomPageNumber}", 60 ,function () use ($pendingInvoice, $invoice_type) {
            $user = User::find(auth()->id());
            $pendingInvoice = $user->requests()->firstOrCreate(['id' => $pendingInvoice], ['type' => $this->InvoiceTypeToInteger($invoice_type)]);
            return $pendingInvoice;
        });

        $pendingInvoice->update($pendingInvoiceData);
        $items = $request->get('items', []);
        $allItemsIds = array_column($items, 'item_id');
        $pendingInvoice->details()->whereNotIn('item_id', $allItemsIds)->delete();
        foreach ($items as $item){
            $pendingInvoice->details()->updateOrCreate([
                'item_id' => $item['item_id'],
            ], $item);
        }

        return response()->json([
            'status' => true,
            'data' => [
                'id' => $pendingInvoice->id,
            ]
        ]);
    }

    public function pendings(String $invoice_type, Request $request)
    {
        $clients = clientGroups()->get();
        $users = User::select('id', 'name')->pluck('name', 'id');
        $pendingInvoices = EntitiesRequest::where('type', $this->InvoiceTypeToInteger($invoice_type))
                            ->with('client')
                            ->when($request->filled('user_id'), function ($query) use ($request) {
                                $query->where('user_id', $request->get('user_id'));
                            })->when($request->filled('search'), function ($query) use ($request) {
                                    $query->where(function ($query) use ($request){
                                        $query->whereHas('client', function ($query) use ($request) {
                                            $query->where('client_name', 'like', '%' . $request->search . '%');
                                        })->orWhere('requests.no_bill', 'like', '%' . $request->search . '%');
                                    });
                            })->when($request->filled('dateFrom'), function ($query) use ($request) {
                                    $query->whereDate('invoice_date', '>=', $request->dateFrom);
                            })->when($request->filled('dateTo'), function ($query) use ($request) {
                                    $query->whereDate('invoice_date', '<=', $request->dateTo);
                            })->paginate(20);

        return view('invoices::pendings', compact('users', 'pendingInvoices', 'invoice_type', 'clients'));
    }

    public function pendingsCreate(string $invoice_type, Request $request, $pendingInvoice)
    {
        $pendingInvoice = EntitiesRequest::where('type', $this->InvoiceTypeToInteger($invoice_type))->findOrFail($pendingInvoice);
        $this->pendingInvoice = $pendingInvoice;

        return $this->create($invoice_type, $request);
    }

    public function pendingsDestroy(string $invoice_type, Request $request, $pendingInvoice)
    {
        $pendingInvoice = EntitiesRequest::where('type', $this->InvoiceTypeToInteger($invoice_type))->findOrFail($pendingInvoice);
        $pendingInvoice->details()->delete();
        $pendingInvoice->delete();

        return redirect()->route('invoices.pendings', $invoice_type)->with('success', trans('invoices.outstanding_invoice_deleted'));
    }

    public static function InvoiceTypeToInteger($invoice_type)
    {
        /**
         * Invoice type:
         * (invoice_type = 0) => فاتورة شراء , (invoice_type = 1) => فاتورة بيع
         * (invoice_type = 2) => فاتورة مرتجع شراء , (invoice_type = 3) => فاتورة مرتجع بيع
         */
        return [
            'purchases' => 0,
            'sales' => 1,
            'returned purchases' => 2,
            'returned sales' => 3,
        ][strtolower($invoice_type)];
    }

    public function getItemCounts(Request $request)
    {
        $sales = InvoiceDetails::where('shop_id', auth()->user()->shop_id)
                                ->whereHas('Invoice', function($query) use($request) {
                                    $query->where('client_id', $request->client_id)->where('invoice_type', 1);
                                });

        $returned = InvoiceDetails::where('shop_id', auth()->user()->shop_id)
                                ->whereHas('Invoice', function($query) use($request) {
                                    $query->where('client_id', $request->client_id)->where('invoice_type', 3);
                                });
        if(is_array($request->item_id)){
            $sales = $sales->select(['item_id', \DB::raw('sum(`quantity`) as quantity') ])->whereIn('item_id', $request->item_id)->groupBy('item_id')->get()->keyBy('item_id');
            $returned = $returned->select(['item_id', \DB::raw('sum(`quantity`) as quantity')])->whereIn('item_id', $request->item_id)->groupBy('item_id')->get()->keyBy('item_id');
            return response()->json(['sales' => $sales, 'returned' => $returned], 200);
        } else {
            $sales = $sales->where('item_id', $request->item_id)->sum('quantity');
            $returned = $returned->where('item_id', $request->item_id)->sum('quantity');
            return response()->json(['sales' => round_money($sales), 'returned' => round_money($returned)], 200);
        }
    }

    public function backAll(Request $request, $type, $id)
    {
        $invoice = Invoices::where('shop_id', auth()->user()->shop_id)->where('invoice_type', 1)->find($id);
        $new_request = new NewInvoiceRequest($this->setData($invoice));
        $response = $this->store($new_request, "returned sales");
        return back();
    }

    protected function setData($invoice)
    {
        $sale_point_id = User::select('sale_point')->find($invoice->add_user)->sale_point ?? 0;

        [$item_details, $items_vats, $vats_total] = $this->setDetails($invoice->id);
        return [
            "_token"                => csrf_token(),
            "pendingInvoice"        => false,
            "reference_id"          => $invoice->id,
            "client_id"             => $invoice->client_id,
            "invoice_date"          => now()->format('Y-m-d'),
            "store_id"              => $invoice->store_id,
            "bill_no"               => Invoices::where('shop_id', auth()->user()->shop_id)->where('invoice_type', 3)->max('bill_no') + 1,
            "item_result"           => "0",
            "amount"                => "",
            "item"                  => '',
            "sale_price"            => "",
            "expire_date"           => "",
            "item_details"          => $item_details,
            "actual_prices"         => [],
            "invoice_total"         => $invoice->bill_total,
            "calculated_total"      => $invoice->bill_total,
            "invoice_discount_type" => 1,
            "sent_discount_amount"  => 0,
            "sent_discount_type"    => 0,
            "vats_total"            => $vats_total,
            "net"                   => $invoice->net_price,
            "pay_method"            => $invoice->payment == $invoice->bill_total,
            "cash_safe_point_id"    => $sale_point_id,
            "cash_paid"             => $invoice->bill_payment,
            "paid"                  => $invoice->payment,
            "rest"                  => $invoice->rest,
            "notes"                 => "",
            "items_vats"            => implode(',', $items_vats)
        ];
    }

    protected function setDetails($invoice_id)
    {
        $details = InvoiceDetails::where('invoice_id', $invoice_id)->get();
        $item_details = $items_vats = [];
        $vats_total = 0;
        foreach ($details as $detail) {
            $vat = InvoiceDetailsVats::select('vat_id', 'vat_type', 'vat_value', 'vat_weight', 'vat_operation')->where('shop_id', auth()->user()->shop_id)
                                        ->where('item_id', $detail->item_id)->where('unit_id', $detail->unit_id)->where('invoice_id', $invoice_id)->first();
                                    // item_id + '/' + vat_id + '/' + vat_type + '/' + vat_value + '/' + vat_weight + '/' + vat_operation + '/' + unit_id;
            $items_vats[] = "{$detail->item_id}/{$vat->vat_id}/{$vat->vat_type}/{$vat->vat_value}/{$vat->vat_weight}/{$vat->vat_operation}/{$detail->unit_id}";

            $vats_total += $vat->vat_weight;

            $index = rand(100000000000, 999999999999);
            $item_details [$index] = "$detail->item_id/$detail->quantity/$detail->unit_id/$detail->item_pay/
            $detail->total/$detail->sale_price/$detail->sale_price/$detail->expire_date/$detail->discount_operation/
            $detail->discount/$detail->discount_type/$detail->basic_sale_price/$detail->basic_pay_price/0/$index";
        }
        return [$item_details, $items_vats, $vats_total];
    }
}
