<?php
    namespace Modules\Invoices\Http\Controllers;

    use App\Models\Instalment;
use App\Services\Invoice;
use Illuminate\Http\Request;
use Modules\Invoices\Entities\Client;
use Modules\Invoices\Entities\Clients;
use Modules\Invoices\Entities\ClientTransaction;
use Modules\Invoices\Entities\Invoices;
    use Modules\Invoices\Http\Requests\NewInvoiceRequest;

    abstract class MainController
    {
    
        protected $vat_use, $invoice_type, $client_roles;
        protected $invoice_id = null;
        public $reshaped_ingrediants = [] ;

        public function create()
        {
            $clients = $this->getClients();
            $bg_color = $this->getFormBgColor();
            return [ $clients, $bg_color];
        }

        abstract public function store(NewInvoiceRequest $request);

        abstract protected function getFormBgColor();
    
        protected function getClients()
        {
            return Clients::where('clients.shop_id', get_session_val('serial_id', 'Integer'))
                    ->leftJoin('currencies', 'clients.currency_id', '=', 'currencies.id')
                    ->whereIn('clients.client_role', $this->client_roles)
                    ->selectRaw(
                        'clients.id, clients.client_name, clients.balance, clients.currency_id, currencies.name as currency, has_vat')
                    ->get();
        }
    
        public function getVatUse()
        {
            return $this->vat_use;
        }
    
        public function getInvoiceTypeNo()
        {
            return Invoices::getInvoiceNo($this->invoice_type);
        }
        protected function generateInstallments(Request $request, Invoice $invoice, $is_client = true)
        {
            if ($request->rest > 0 && (int) (get_session_val('insatllments', 'integer') && authorized('can_pay_by_installments')) == 1)
            {
                foreach ($request->installment_dates as $key => $date) {
                    Instalment::create([
                        'client_id'  =>  $is_client == true ? $request->client_id : 0,
                        'supplier_id'  =>  $is_client == false ? $request->client_id : 0,
                        'installment_amount'  =>  $request->installment_value ,
                        'installment_date'  =>  $date,
                        'bill_id'  =>  $this->invoice_id ,
                        'shop_id'  => get_session_val('serial_id', 'integer') ,
                        'notes' => $request->installment_notes ?? ' ',
                        'type' => $is_client == true ? 0 : 1
                    ]);
                }

                $this->generateClientInstallmentTrans($request, $invoice);
            }

        }

        protected function generateClientInstallmentTrans(Request $request, Invoice $invoice)
        {
            if ($request->installment_add_value > 0)
            {
                ClientTransaction::create([
                    'shop_id' => get_session_val('serial_id', 'Integer'),
                    'user_id' => auth()->user()->id,
                    'amount' => $request->installment_add_value,
                    'pay_day' => $invoice->getInvoice()->transaction_date,
                    'date_time' => day_now(get_session_val('date_type', 'Integer')),
                    'balance' => $invoice->getClient()->getNewBalance() +  (($invoice->getInvoice()->invoice_type == 0 ? -1 : 1) *  $request->installment_add_value),
                    'bill_net_total' => $invoice->getInvoice()->net_price,
                    'bill_id' => $invoice->getInvoice()->id,
                    'safe_point_id' => $invoice->getSafe()->getSafePoint()->id,
                    'client_id' => $request->client_id,
                    'safe_balance' => $invoice->getSafe()->getSafePoint()->money_point,
                    'notes' => $invoice->getInvoice()->notes,
                    'type' => $invoice->getInvoice()->invoice_type == 0 ? 90 : 91,
                    'effect' => 0
                ]);

                $this->updateClientBalance($request->client_id, $invoice->getClient()->getNewBalance() + (($invoice->getInvoice()->invoice_type == 0 ? -1 : 1) *  $request->installment_add_value) );
            }
        }

        protected function updateClientBalance($client_id, $new_balance)
        {
            Client::find($client_id)->update([
                'balance' => $new_balance
            ]);
        }

        protected function addNewIngrediant($current_saved_detail_id, $sent_detail_unique_id, $request)
        {
            if(get_session_val('manufacturing', 'integer') == 2 && get_session_val('definite_while_selling', 'integer') == 1)
            {
                if($request->ingrediants)
                {
                    $this->reshaped_ingrediants[$current_saved_detail_id] = $request->ingrediants[$sent_detail_unique_id];
                }
            }
        }

        abstract public function manageInvoicesSerials(Request $request, $invoice_id, $store_id, $invoice_date);
        
        abstract protected function newInvoiceLog();
    }
    
?>