<?php

namespace Modules\Invoices\Http\Controllers;

use App\Exceptions\CreatingInvoiceException;
use App\Services\InvoiceDetailVat;
use App\Services\PurchaseDetail;
use App\Services\PurchaseInvoice;
use App\Services\PurchaseInvoiceClientTrans;
use App\Services\Updaters\InvoiceItemUnit;
use App\Services\Updaters\InvoiceStoreItemIncUpdater;
use App\Services\Updaters\PurchaseInvoiceItemUpdater;
use App\Services\Validators\InvoiceValidate;
use Illuminate\Http\Request;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\DB;
use Modules\Invoices\Entities\ItemSerial;
use Modules\Invoices\Entities\ItemUnit;
use Modules\Invoices\Http\Requests\NewInvoiceRequest;
use Modules\User\Entities\Log;

class PurchaseController extends MainController
{
    protected $vat_use = '0,2';
    protected $invoice_type = 'purchases';
    protected $client_roles = [1, 2];

    public function store(NewInvoiceRequest $request)
    {
        $invoice_validator = new InvoiceValidate($request);

        if ($invoice_validator->validate())
        {
            try {
                DB::transaction(function() use($request){

                    $new_invoice = new PurchaseInvoice($request);
                    $new_invoice->create();
                                        
                    $this->invoice_id = $new_invoice->getInvoice()->id;
                    $this->newInvoiceLog();

                    foreach ($request->item_details as $detail_id => $detail) {

                        $new_invoice_detail = new PurchaseDetail($this->invoice_id, 0, $this->invoice_type, $new_invoice->getClient()->getClient()?->id, explode('/', $detail));
                        $new_invoice_detail->create();

                        $item = $new_invoice_detail->getItem();
                        $item_unit_id = $item->unit_id;

                        $invoice_detail_store_item = new InvoiceStoreItemIncUpdater($item->id, $new_invoice->getInvoice()->store_id, $item_unit_id, $new_invoice_detail);
                        $invoice_detail_store_item->update();

                        $x = explode('/', $detail);
                        $item_unit = ItemUnit::where([
                            'shop_id' => get_session_val('serial_id', 'Integer'),
                            'item_id' => $item->id,
                            'unit_id' => $x[2]
                        ])->first();

                        $n_price = 0;
                        if ($item_unit_id == $x[2]) {
                            $n_price = $x[3];
                        } else {
                            $n_price = $x[3] / ($item_unit->unit_value ?? 1);
                        }

                        $invoice_detail_item = new PurchaseInvoiceItemUpdater($item, $request, $new_invoice_detail);
                        $invoice_detail_item->update($n_price);

                        $new_invoice_detail->getDetail()->quantity_after = $invoice_detail_store_item->getNewQty();
                        $new_invoice_detail->getDetail()->save();

                        if ($item_unit_id == $x[2])
                        {
                            InvoiceItemUnit::updateManyPrice($x[0], $x[2], $x[5]);
                        }

                        $new_invoice->setTotalDisc($new_invoice_detail->getInvoiceTotalDisc());

                    }

                    if ($request->items_vats)
                    {
                        $invoice_detail_vats = new InvoiceDetailVat($this->invoice_id, $request->items_vats);
                        $invoice_detail_vats->createMany();
                    }
    
                    $new_invoice->updateTotalDisc();

                    $new_invoice->getClient()->updateBalance();
                    $new_invoice->getSafe()->update($new_invoice->getInvoice()->payment);

                    $client_transaction = new PurchaseInvoiceClientTrans($new_invoice, $new_invoice->getSafe(), $new_invoice->getClient());
                    $client_transaction->create();

                    $this->generateInstallments($request, $new_invoice, false);

                    $this->manageInvoicesSerials($request, $this->invoice_id, $new_invoice->getInvoice()->store_id, $new_invoice->getInvoice()->invoice_date);
                });
            } catch (\Exception $exception) {
                throw new CreatingInvoiceException($this->invoice_type, get_class($exception), $exception->getMessage());
            }
            return ['status' => true, 'id' => $this->invoice_id];
        }
        return ['status' => false, 'msg' => $invoice_validator->msg];
    }

    protected function getFormBgColor()
    {
        return '#f7e6e6';
    }

    public function manageInvoicesSerials(Request $request, $invoice_id, $store_id, $invoice_date)
    {
        if ( get_session_val('show_item_serial', 'integer') == '1' && is_array($request->item_serials) && sizeof($request->item_serials) > 0 ) {

            foreach ($request->item_serials as $item_id => $serial) {
                $s = explode(',', $serial);
                foreach ($s as $index => $serial_number) {
                    $data = new ItemSerial();
                    $data->item_id = $item_id;
                    $data->serial = $serial_number;
                    $data->income_id = $invoice_id;
                    $data->sale_id = 0;
                    $data->sold = 0;
                    $data->shop_id = get_session_val('serial_id', 'Integer');
                    $data->add_user = auth()->user()->id;
                    $data->add_date = Carbon::now();
                    $data->store_id = $store_id;
                    
                    if (is_array($request->serials_notes) && sizeof($request->serials_notes) > 0) {
                        if (array_key_exists($item_id, $request->serials_notes)) {
                            $notes = $request->serials_notes[$item_id];
                            $explode = explode(',', $notes);
                            if (isset($explode[$index]) && $explode[$index] != '-*-'){
                                $data->notes = $explode[$index];
                            }else {
                                $data->notes = '';
                            }
                        }
                    }
                    $data->save();
                }
            }
        }
    }

    protected function newInvoiceLog()
    {
        Log::create([
            'user_id' => auth()->user()->id,
            'date_event' => date('Y-m-d H:i:s'),
            'event' => trans('log.new_purchase_invoice') . trans('log.invoice_no') . $this->invoice_id,
            'shop_id' => get_session_val('serial_id', 'integer'),
            'type' => 0
        ]);
    }

}