<?php

namespace Modules\Invoices\Http\Controllers;

use App\Exceptions\CreatingInvoiceException;
use App\Services\InvoiceDetailVat;
use App\Services\ReturnPurchaseDetail;
use App\Services\ReturnPurchaseInvoice;
use App\Services\ReturnPurchaseInvoiceClientTrans;
use App\Services\Updaters\InvoiceStoreItemDescUpdater;
use App\Services\Validators\InvoiceValidate;
use App\Services\Validators\InvoiceValidateDetails;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Modules\Invoices\Entities\ItemSerial;
use Modules\User\Entities\Log;

class ReturnedPurchasesController extends MainController
{

    protected $vat_use = '0,2';
    protected $invoice_type = 'returned purchases';
    protected $client_roles = [1, 2];

    public function store(Request $request)
    {
        $invoice_validator = new InvoiceValidateDetails($request);

        if ($invoice_validator->validate())
        {
            try {
                DB::transaction(function() use($request){

                    $new_invoice = new ReturnPurchaseInvoice($request);
                    $new_invoice->create();

                    $this->invoice_id = $new_invoice->getInvoice()->id;
                    $this->newInvoiceLog();

                    foreach ($request->item_details as $detail_id => $detail) {

                        $new_invoice_detail = new ReturnPurchaseDetail($this->invoice_id, 0, $this->invoice_type, $new_invoice->getClient()->getClient()?->id, explode('/', $detail));
                        $new_invoice_detail->create();

                        $item = $new_invoice_detail->getItem();
                        $item_unit_id = $item->unit_id;

                        $invoice_detail_store_item = new InvoiceStoreItemDescUpdater($item->id, $new_invoice->getInvoice()->store_id, $item_unit_id, $new_invoice_detail);
                        $invoice_detail_store_item->update();

                        $new_invoice_detail->getDetail()->quantity_after = $invoice_detail_store_item->getNewQty();
                        $new_invoice_detail->getDetail()->save();

                        $new_invoice->setTotalDisc($new_invoice_detail->getInvoiceTotalDisc());

                    }

                    if ($request->items_vats)
                    {
                        $invoice_detail_vats = new InvoiceDetailVat($this->invoice_id, $request->items_vats);
                        $invoice_detail_vats->createMany();
                    }

                    $new_invoice->updateTotalDisc();

                    $new_invoice->getClient()->updateBalance();
                    $new_invoice->getSafe()->update($new_invoice->getInvoice()->payment);

                    $client_transaction = new ReturnPurchaseInvoiceClientTrans($new_invoice, $new_invoice->getSafe(), $new_invoice->getClient());
                    $client_transaction->create();

                    $this->manageInvoicesSerials($request, $this->invoice_id, $new_invoice->getInvoice()->store_id, $new_invoice->getInvoice()->invoice_date);

                });
            } catch (\Exception $exception) {
                throw new CreatingInvoiceException($this->invoice_type, get_class($exception), $exception->getMessage());
            }
            return ['status' => true, 'id' => $this->invoice_id];
        }
        return ['status' => false, 'msg' => $invoice_validator->msg];
    }

    protected function getFormBgColor()
    {
        return '#f7efe6';
    }

    public function manageInvoicesSerials(Request $request, $invoice_id, $store_id, $invoice_date)
    {
        if ( get_session_val('show_item_serial', 'integer') == '1' && is_array($request->item_serials) && sizeof($request->item_serials) > 0 ) {

            foreach ($request->item_serials as $item_id => $serial) {
                $s = explode(',', $serial);
                foreach ($s as $index => $serial_number) {

                    $data = ItemSerial::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['serial', $serial_number],
                        ['item_id', $item_id]
                    ])->first();

                    if ($data)
                    {
                        $data->income_id = $invoice_id;
                        $data->sale_id = $invoice_id;
                        $data->sold = 1;
                        $data->sold_date = $invoice_date;
                        $data->save();
                    }
                }
            }
        }
    }

    protected function newInvoiceLog()
    {
        Log::create([
            'user_id' => auth()->user()->id,
            'date_event' => date('Y-m-d H:i:s'),
            'event' => trans('log.new_return_purchase_invoice') . trans('log.invoice_no') . $this->invoice_id,
            'shop_id' => get_session_val('serial_id', 'integer'),
            'type' => 2
        ]);
    }
}
