<?php

namespace Modules\Invoices\Http\Controllers;

use App\Exceptions\CreatingInvoiceException;
use App\Services\InvoiceDetailVat;
use App\Services\SellDetail;
use App\Services\SellInvoice;
use App\Services\SellInvoiceClientTrans;
use App\Services\Updaters\InvoiceNegClient;
use App\Services\Updaters\InvoicePosSafe;
use App\Services\Updaters\InvoiceStoreItemDescUpdater;
use App\Services\Validators\InvoiceValidate;
use App\Services\Validators\InvoiceValidateDetails;
use App\Services\Validators\InvoiceValidateNetworkSafe;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Modules\Invoices\Entities\Client;
use Modules\Invoices\Entities\ItemSerial;
use Modules\User\Entities\Log;

class SalesController extends MainController
{
    protected $vat_use = '0,1';
    protected $invoice_type = 'sales';
    protected $client_roles = [0, 2];

    public function store(Request $request)
    {

        $invoice_validator = new InvoiceValidateDetails($request);

        if ($invoice_validator->validate()) {
            try {

                DB::transaction(function () use ($request) {
                    $new_invoice = new SellInvoice($request);
                    $new_invoice->create();
                    $this->invoice_id = $new_invoice->getInvoice()->id;
                    $this->newInvoiceLog();
                    foreach ($request->item_details as $detail_id => $detail) {

                        $new_invoice_detail = new SellDetail($this->invoice_id, 0, $this->invoice_type, $new_invoice->getClient()->getClient()?->id, explode('/', $detail));
                        $new_invoice_detail->create();
                        $item = $new_invoice_detail->getItem();
                        $item_unit_id = $item->unit_id;

                        $invoice_detail_store_item = new InvoiceStoreItemDescUpdater($item->id, $new_invoice->getInvoice()->store_id, $item_unit_id, $new_invoice_detail);
                        $invoice_detail_store_item->update();

                        $new_invoice_detail->getDetail()->quantity_after = $invoice_detail_store_item->getNewQty();
                        $new_invoice_detail->getDetail()->save();

                        $new_invoice->setTotalDisc($new_invoice_detail->getInvoiceTotalDisc());

                        $this->addNewIngrediant($new_invoice_detail->getDetail()->id, $new_invoice_detail->getSentUniqueId(), $request);
                    }
                    if ($request->items_vats) {
                        $invoice_detail_vats = new InvoiceDetailVat($this->invoice_id, $request->items_vats);
                        $invoice_detail_vats->createMany();
                    }

                    $new_invoice->updateTotalDisc();

                    // $new_invoice->getClient()->updateBalance();

                    $cash_client = new InvoiceNegClient($request, 'cash_paid');

                    $network_client = new InvoiceNegClient($request, 'network_paid');

                    if ($request->cash_paid > 0) {
                        $new_invoice->getSafe()->update($request->cash_paid);
                        $client_transaction = new SellInvoiceClientTrans($new_invoice, $new_invoice->getSafe(), $cash_client);
                        $client_transaction->create();

                        $network_client->decreaseAddBalance($request->cash_paid);
                        $network_client->setOldBalance($cash_client->getNewBalance() - $cash_client->getAddBalance());
                    }

                    if (get_session_val('netowrk_payment', 'Integer') == 1 && $request->network_safe_point_id && $request->network_paid > 0) {
                        $network_safe = new InvoicePosSafe((float) $request->network_safe_point_id);

                        $network_safe->update($request->network_paid);

                        // $client_network_transaction = new SellInvoiceClientTrans($new_invoice, $network_safe, $new_invoice->getClient());
                        $client_network_transaction = new SellInvoiceClientTrans($new_invoice, $network_safe, $network_client);
                        $client_network_transaction->create();

                        $cash_client->decreaseAddBalance($request->network_paid);
                        $cash_client->setOldBalance($network_client->getNewBalance() - $network_client->getAddBalance());
                    } else if (!($request->cash_paid > 0)) {
                        $new_invoice->getSafe()->update((float) $request->cash_paid ?? 0);
                        $client_transaction = new SellInvoiceClientTrans($new_invoice, $new_invoice->getSafe(), $cash_client);
                        $client_transaction->create();
                    }

                    $new_invoice->getClient()->updateBalance();

                    $this->generateInstallments($request, $new_invoice);

                    $this->manageInvoicesSerials($request, $this->invoice_id, $new_invoice->getInvoice()->store_id, $new_invoice->getInvoice()->invoice_date);
                });
            } catch (\Exception $exception) {
                dd($exception);
                throw new CreatingInvoiceException($this->invoice_type, get_class($exception), $exception->getMessage());
            }

            return ['status' => true, 'id' => $this->invoice_id];
        }

        return ['status' => false, 'msg' => $invoice_validator->msg];
    }

    protected function getFormBgColor()
    {
        return '#e6f3f7';
    }

    public function manageInvoicesSerials(Request $request, $invoice_id, $store_id, $invoice_date)
    {
        if (get_session_val('show_item_serial', 'integer') == '1' && is_array($request->item_serials) && sizeof($request->item_serials) > 0) {
            foreach ($request->item_serials as $item_id => $serial) {
                $s = explode(',', $serial);
                foreach ($s as $index => $serial_number) {

                    $data = ItemSerial::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['serial', $serial_number],
                        ['item_id', $item_id]
                    ])->first();

                    if ($data) {
                        // $data->income_id = $invoice_id;
                        $data->sale_id = $invoice_id;
                        $data->sold = 1;
                        $data->sold_date = $invoice_date;
                        $data->save();
                    }
                }
            }
        }
    }

    protected function newInvoiceLog()
    {
        Log::create([
            'user_id' => auth()->user()->id,
            'date_event' => date('Y-m-d H:i:s'),
            'event' => trans('log.new_sale_invoice') . trans('log.invoice_no') . $this->invoice_id,
            'shop_id' => get_session_val('serial_id', 'integer'),
            'type' => 1
        ]);
    }
}
