<?php

namespace Modules\Invoices\Http\Controllers;

use App\Exceptions\CreatingInvoiceException;
use App\Services\InvoiceDetailVat;
use App\Services\SellDetail;
use App\Services\SellOfferInvoice;
use App\Services\Updaters\InvoiceStoreItemDescUpdater;
use App\Services\Validators\InvoiceValidateDetails;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Modules\User\Entities\Log;

class SellOfferController extends MainController
{
    protected $vat_use = '0,1';
    protected $invoice_type = 'sales offers';
    protected $client_roles = [0, 2];

    public function store(Request $request)
    {
        $invoice_validator = new InvoiceValidateDetails($request);

        if ($invoice_validator->validate())
        {
            try {
                DB::transaction(function() use($request){

                    $new_invoice = new SellOfferInvoice($request);
                    $new_invoice->create();
                                        
                    $this->invoice_id = $new_invoice->getInvoice()->id;
                    $this->newInvoiceLog();

                    foreach ($request->item_details as $detail_id => $detail) {

                        $new_invoice_detail = new SellDetail($this->invoice_id, 0, $this->invoice_type, $new_invoice->getClient()->getClient()->id, explode('/', $detail));
                        $new_invoice_detail->create();

                        $item = $new_invoice_detail->getItem();
                        $item_unit_id = $item->unit_id;

                        $invoice_detail_store_item = new InvoiceStoreItemDescUpdater($item->id, $new_invoice->getInvoice()->store_id, $item_unit_id, $new_invoice_detail);

                        $new_invoice_detail->getDetail()->quantity_after = $invoice_detail_store_item->fakeNewQty();
                        $new_invoice_detail->getDetail()->save();

                        $new_invoice->setTotalDisc($new_invoice_detail->getInvoiceTotalDisc());

                    }

                    if ($request->items_vats)
                    {
                        $invoice_detail_vats = new InvoiceDetailVat($this->invoice_id, $request->items_vats);
                        $invoice_detail_vats->createMany();
                    }

                    $new_invoice->updateTotalDisc();
                });
            } catch (\Exception $exception) {
                throw new CreatingInvoiceException($this->invoice_type, get_class($exception), $exception->getMessage());
            }
            return ['status' => true, 'id' => $this->invoice_id];
        }
        return ['status' => false, 'msg' => $invoice_validator->msg];
    }

    protected function getFormBgColor()
    {
        return '#e6ebf7';
    }

    public function manageInvoicesSerials(Request $request, $invoice_id, $store_id, $invoice_date)
    {
    }

    protected function newInvoiceLog()
    {
        Log::create([
            'user_id' => auth()->user()->id,
            'date_event' => date('Y-m-d H:i:s'),
            'event' => trans('log.new_sale_offer_invoice') . trans('log.invoice_no') . $this->invoice_id,
            'shop_id' => get_session_val('serial_id', 'integer'),
            'type' => 2
        ]);
    }
}