@extends('master::layouts.master')
@section('css-scripts')
    <style>
        .bg-grey-dark {
            background-color: #f1f1f1;
        }

        #section-to-print{
            font-size: {{$invoice_settings->font_size}}px !important;
            white-space: initial !important;
            max-width: 100%;
        }

        @page {
                margin: 0mm;
            }

        @media print {
            body * {
                visibility: hidden;
                padding-top: 0px !important;
                /* display: none; */
            }
            .table td, .table th, .table{
                border: 1px solid black;
                border-collapse: collapse;
            }
            #main_side_bar, .global-header, #eJOY__extension_root{
                height: 1px;
                display: none !important;
                padding: 0px !important;
                margin: 0px !important;
            }
            .container{
                width: 100% !important;
            }
            #section-to-print * {
                    visibility: visible;
                    margin-top: 0px;
                    font-size: {{$invoice_settings->font_size}}px !important;
                    direction: rtl !important;
                    font-family: Arial, Helvetica, sans-serif !important;
                    font-weight: bolder !important;
                    color: black !important;
                    background-color: white !important;
                    white-space: initial !important;
            }
            @media print {
                table{
                    width: 100%;
                }
                p {
                    border-bottom: unset;
                }
            }
        }

        #section-to-print table {
                border: 3px solid rgb(24, 24, 24);
                border-collapse: collapse !important;
                width: 100% !important;
                font-weight: bold;
            }

            #section-to-print table th, #section-to-print table td {
                border: 2px solid #000 !important;
                padding: 0 10px !important;
            }

    </style>
@endsection

@section('content')
    <div class="container" id="section-to-print">
        @php $is_cash_bill = $invoice->payment == $invoice->net_price || $settings->getCountry?->name_en != "Saudi Arabia"; @endphp

        <div class="col-12 mt-4">

            <div class="row mb-4">
                <div class="col-8">
                    <img src="{{ asset($shop->logo_path) }}" alt="logo" style="max-width: 250px;">
                </div>
                <div class="col-4 bg-grey-dark p-2">
                    <p>
                        <span class="fw-bold">
                            رقم الفاتورة:
                        </span>
                        {{$invoice?->bill_no}}
                    </p>
                    <p>
                        <span class="fw-bold">
                            التاريخ :
                        </span>
                        {{\Carbon\Carbon::parse($invoice->invoice_date)->format('Y-m-d')}}
                    </p>
                    <p>
                        <span class="fw-bold">
                            اسم العميل:
                        </span>
                        {{$invoice?->client?->client_name}}
                    </p>
                </div>
            </div>

            <div class="mt-2 text-center" style=" color: #000">
                {!! $settings->bill_header !!}
            </div>

            @php
                $bill_total_without_vat = 0;
            @endphp

            <table>
                <thead>
                    <th style="width: 2% !important">م</th>
                    <th style="width: 30% !important">الصنف</th>
                    <th style="width: 10% !important">الكمية</th>
                    <th style="width: 10% !important">السعر الاساسى</th>
                    <th style="width: 10% !important">الاضافه</th>
                    <th style="width: 10% !important">الخصم</th>
                    <th style="width: 10% !important">الاجمالي</th>
                    <th style="width: 10% !important">الملاحظات</th>
                </thead>
                <tbody>

                @php $bill_total_without_vat = $bill_total_discounts = $total_adds = 0; @endphp
                @foreach ($invoice->InvoiceDetails as $i => $row)

                    <tr>
                        @php
                            $item_vat   = $invoice->vats?->where('item_id', $row->item_id)->where('unit_id', $row->unit_id) ?? collect();
                            $vat_value  = $item_vat->sum('vat_value');
                            $vat_weight = $item_vat->sum('vat_weight');
                            $item_price = in_array($invoice->invoice_type, [0, 2]) ? $row->basic_pay_price : $row->basic_sale_price;

                            if ($row->Item?->vat_state == 2 && $shop->apply_vats) {
                                $item_price = $item_price / (1 + $vat_value / 100);
                            }

                            $total_item_price = $item_price * $row->quantity;
                            $bill_total_without_vat += $total_item_price;

                            $item_discount = $item_adds = $show_item_discount = $show_item_adds = 0;

                            if ($row->discount > 0) {
                                if ($row->discount_type == 1) {
                                    $item_discount = $row->discount * $row->quantity;
                                } else {
                                    $item_discount = $total_item_price * ($row->discount / 100);
                                }
                            }

                            $show_item_discount = ( $total_item_price - ( $total_item_price - $item_discount ) ) * 100 / $total_item_price;
                            if ($row->discount_operation) {
                                $item_adds = $item_discount;
                                $show_item_adds = $show_item_discount;
                                $item_discount = $show_item_discount = 0;
                            }

                            $bill_total_discounts += $item_discount;
                            $net_item_price = $total_item_price + $item_adds - $item_discount + $vat_weight;
                        @endphp

                        <td style="border-left: 1px solid #000 !important; border-bottom: 2px dashed #cccccc; text-align: center"> {{ $loop->iteration }} </td>
                        <td style="border-left: 1px solid #000 !important; border-right: 1px solid #000 !important; border-bottom: 2px dashed #cccccc; text-align: right; padding-right: 5px;"> {{ $row->Item?->item_name }} </td>
                        <td style="border-left: 1px solid #000 !important; border-right: 1px solid #000 !important; border-bottom: 2px dashed #cccccc; text-align: center"> {{ round_quantity((float) $row->quantity) }} </td>
                        <td style="border-left: 1px solid #000 !important; border-right: 1px solid #000 !important; border-bottom: 2px dashed #cccccc; text-align: center"> {{ round_money((float) $item_price) }} </td>
                        <td style="border-left: 1px solid #000 !important; border-right: 1px solid #000 !important; border-bottom: 2px dashed #cccccc; text-align: center"> {{ round_money((float) $show_item_adds) }} %</td>
                        <td style="border-left: 1px solid #000 !important; border-right: 1px solid #000 !important; border-bottom: 2px dashed #cccccc; text-align: center"> {{ round_money((float) $show_item_discount) }} %</td>
                        <td style="border-right: 1px solid #000 !important; border-bottom: 2px dashed #cccccc; padding: 10px !important; text-align: center"> {{ round_money((float) $net_item_price) }} </td>
                        <td style="border-right: 1px solid #000 !important; border-bottom: 2px dashed #cccccc; padding: 10px !important; text-align: center"> {{ $row->item_about }} </td>
                    </tr>
                @endforeach
                </tbody>
            </table>

            <div style="float: left; padding: 15px; width: 40%; background: #e7e7e7; margin-top: 15px;">
                <p style="color: #000; font-weight: bold; margin-bottom: 10px">
                    <span style="display: inline-block; width: 40%">الاجمالى</span>
                    <span style="display: inline-block; width: 30%">{{ $invoice->net_price }}</span>
                </p>
                <p style="color: #000; font-weight: bold; margin-bottom: 10px">
                    <span style="display: inline-block; width: 40%">رصيد العميل السابق</span>
                    @if ($invoice->balance > 0)
                        @php $old_balance = eval("return $invoice->balance - $invoice->rest;"); @endphp
                    @else
                        @php $old_balance = eval("return $invoice->balance + $invoice->rest;"); @endphp
                    @endif
                    <span style="display: inline-block; width: 30%">{{ $old_balance }}</span>
                </p>
                <p style="color: #000; font-weight: bold; margin-bottom: 10px">
                    <span style="display: inline-block; width: 40%">المدفوع</span>
                    <span style="display: inline-block; width: 30%">{{ $invoice->payment }}</span>
                </p>
                <p style="color: #000; font-weight: bold; margin-bottom: 10px">
                    <span style="display: inline-block; width: 40%">الباقي علي العميل</span>
                    <span style="display: inline-block; width: 30%">{{ $invoice->balance }}</span>
                </p>
                <p style="color: #000; font-weight: bold">
                    <span style="display: inline-block; width: 30%">ملاحظات</span>
                    <span style="display: inline-block; width: 69%">{{ $invoice->notes }}</span>
                </p>
            </div>
        </div>

        <div class="mt-2 text-center" style=" color: #000">
            {!! $settings->message_bill !!}
        </div>

    </div>
@endsection
