@extends('master::layouts.master')

@section('content')
    <div class="container-fluid">
        <h1 class="text-primary">
            {{ __('invoices.invoice.title', ['no' => (!in_array($invoice->bill_no, ['', ' ', null]) ? $invoice->bill_no : '---')]) }}
        </h1>

        {{-- Issuer & Reciver --}}
        <div class="col-md-12 mt-4 mb-3">
            <div class="row">
                <div class="form-group col-md-6">
                    <div class="bg-info text-white p-1 pr-2">{{ __('invoices.details.title') }}</div>
                    <div class="bg-white p-2">
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('invoices.details.invoiceNo') }} :</div>
                            <div class="col-md-9">{{ $invoice->bill_no }}</div>
                        </div>

                        @if ($invoice->supplier_invoice_no)
                            <div class="form-row row p-1">
                                <div class="col-md-3 text-primary"> @lang('invoices.invoices.supplier_invoice_no') </div>
                                <div class="col-md-9">{{ $invoice->supplier_invoice_no }}</div>
                            </div>
                        @endif

                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('invoices.details.invoiceType') }} :</div>
                            <div class="col-md-9">{{ $invoice->type }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('invoices.details.invoiceDate') }} :</div>
                            <div class="col-md-9">{{ $invoice->invoice_date }}</div>
                        </div>
                    </div>
                </div>

                <div class="form-group col-md-6">
                    <div class="bg-info text-white p-1 pr-2">{{ __('invoices.client.title') }}</div>
                    <div class="bg-white p-2">
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary"> {{ $invoice->invoice_type ? trans('client.client-name') : trans('client.supplier-name') }}:</div>
                            <div class="col-md-9">{{ $invoice->client->client_name ?? '---' }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('invoices.client.mobile') }} :</div>
                            <div class="col-md-9">{{ $invoice->client->mobile1 ?? '---' }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-3 text-primary">{{ __('invoices.client.balance') }} :</div>
                            <div class="col-md-9">{{ $invoice->client->balance ?? '---' }}</div>
                        </div>
                    </div>
                </div>
            </div>
        </div>

        {{-- Invoice Items --}}
        <div class="col-md-12 table-responsive">
            <table class="table table-hover table-bordered table-striped bg-white">
                <thead>
                    <tr class="bg-info text-white">
                        <th scope="col">{{ __('invoices.details.item_name') }}</th>
                        <th scope="col">{{ __('invoices.details.item_description') }}</th>
                        <th scope="col">{{ __('category.unit') }}</th>
                        <th scope="col">{{ __('invoices.details.quantity') }}</th>
                        <th scope="col">{{ __('invoices.details.unit-price') }}</th>
                        <th scope="col">{{ __('invoices.details.total-price') }}</th>
                    </tr>
                </thead>
                <tbody>
                    @php
                        $total_without_vat = 0;
                    @endphp
                    @foreach ($invoice->InvoiceDetails as $detail)
                    @php
                        $item_total = $detail->price_without_vat * $detail->quantity;
                        $total_without_vat += $item_total;
                    @endphp
                        <tr>
                            <td>{{ $detail->item_name }}</td>
                            <td>{{ $detail->item_about }}</td>
                            <td>{{ $detail->unit?->name }}</td>
                            <td>{{ round_quantity($detail->quantity) }}</td>
                            <td>{{ round_money($detail->price_without_vat) }}</td>
                            <td>{{ round_money($item_total) }}</td>
                        </tr>
                    @endforeach
                </tbody>
            </table>
        </div>

        {{-- Invoice Totals --}}
        <div class="col-md-12 mt-3">
            <div class="row justify-content-end">
                <div class="col-md-2"> {!! $invoice->qr !!} </div>
                <div class="col-md-4"></div>
                @php
                    $invoice_discount = $invoice->invoice_total_disc_type == 1 ? (float)$invoice->invoice_total_disc : (float)$total_basic_price * ( (float)$invoice->invoice_total_disc / 100);
                    $bill_total = $invoice->InvoiceDetails->sum('item_price_with_discount') + $invoice_discount;
                    $bill_total = $total_without_vat + $invoice_discount;
                @endphp

                <div class="form-group col-md-4">
                    <div class="bg-info text-white p-1 pr-2">{{ __('invoices.totals.title') }}</div>
                    <div class="bg-white p-2">
                        <div class="form-row row p-1">
                            <div class="col-md-5 text-primary">{{ __('invoices.totals.total_amount') }} :</div>
                            <div class="col-md-7">{{ round_money(  $bill_total ) }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-5 text-primary">{{ __('invoices.totals.total_discount') }} :</div>
                            <div class="col-md-7">{{ round_money((float) $invoice_discount) }}</div>
                        </div>
                        @if ($settings->apply_vats)

                            <div class="form-row row p-1">
                                <div class="col-md-5 text-primary">{{ __('invoices.totals.total_vats') }} :</div>
                                <div class="col-md-7">{{ round_money($invoice->vats_sum_vat_weight ?? 0) }}</div>
                            </div>
                        @endif

                        @if($invoice->invoice_type == 1 &&  get_session_val('show_delivery_charges_in_the_invoice', 'Integer') == 1)
                        <div class="form-row row p-1">
                            <div class="col-md-5 text-primary">{{ __('invoices.delivery_fee') }} :</div>
                            <div class="col-md-7">{{ round_money($invoice->fee) }}</div>
                        </div>
                        @endif

                        <div class="form-row row p-1">
                            <div class="col-md-5 text-primary">{{ __('invoices.totals.total_sales') }} :</div>
                            <div class="col-md-7">{{ round_money($invoice->net_price) }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-5 text-primary">{{ __('invoices.totals.payment') }} :</div>
                            <div class="col-md-7">{{ round_money($invoice->payment) }}</div>
                        </div>
                        <div class="form-row row p-1">
                            <div class="col-md-5 text-primary">{{ __('invoices.totals.rest') }} :</div>
                            <div class="col-md-7">{{ round_money($invoice->rest) }}</div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
@endsection

@section('js-scripts')
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>
    <script src="{{ asset('tickets/js/bootstrap-select.min.js') }}"></script>

    <script>
        $(document).ready(function() {
            $.ajaxSetup({
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                }
            });
            $('.selectpicker').selectpicker({
                style: 'btn-default',
                size: 8
            });

            if ('{{ get_session_val('date_type', 'Integer') }}' == '2') {
                $(".date").calendarsPicker({
                    calendar: $.calendars.instance('gregorian'),
                    dateFormat: 'Y-mm-dd',
                    defaultDate: "0",
                    // selectDefaultDate: true
                });
            } else {
                $(".date").calendarsPicker({
                    calendar: $.calendars.instance('ummalqura'),
                    dateFormat: 'Y-mm-dd',
                    defaultDate: "0",
                    // selectDefaultDate: true
                });
            }
        });
    </script>
@endsection

@section('css-scripts')
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/bootstrap-datetimepicker-standalone.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/datepicker3.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/css/bootstrap-select.min.css') }}" rel="stylesheet">

    <style>
        h3 {
            display: inline-block;
        }

        .btn-danger {
            padding: 6px 12px !important;
        }

        .mg-top {
            margin-top: 10px;
        }

        .invoice-header {
            margin-bottom: 15px;
        }

        .header {
            padding: 5px;
        }

        .info,
        .total-info,
        .details-info {
            background-color: #f5f5f5;
            padding-top: 8px;
            width: 100%;
        }
    </style>
@stop
