<?php

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

Route::group(['middleware'=> ['web','auth']], function()
{
    Route::get('invoices/set/thumbprint', 'InvoicesController@setThumbprint')->name('invoices.set.thumbprint');
    Route::get('invoices/send-email/{id}', 'InvoicesController@sendEmail')->name('invoices.send-email');
    Route::match(['get', 'post'], 'invoices/send/whatsapp/invoice/{invoice}/client/{client}', 'InvoicesController@sendWhatsAppMessage')->name('send.whatsapp.message');

    Route::group(['prefix' => 'invoices/{invoice_type}', 'as' => 'invoices'], function()
    {
        Route::middleware(['type:invoice_type'])->group(function () {
            Route::get('/new', 'InvoicesController@create')->name('.new');
            Route::post('back/all/{id}', 'InvoicesController@backAll')->name('.back.all');
            Route::post('/store', 'InvoicesController@store')->name('.store');
            Route::get('/search-items', 'InvoicesController@search')->name('.search-items');
            Route::post('/item-units', 'InvoicesController@itemUnits')->name('.item-units');
            Route::post('/barcode-search', 'InvoicesController@barcodeSearch')->name('.barcode-search');
            Route::get('/check-serial', 'InvoicesController@checkSerial')->name('.check-serial');
            Route::get('/', 'InvoicesController@index')->name('.index');
            Route::get('/getClients', 'InvoicesController@getClients')->name('.getClients');
            Route::put('/updatePendingInvoiceData/{pendingInvoice}', 'InvoicesController@updatePendingInvoiceData')->name('.updatePendingInvoiceData');
            Route::get('/pendings', 'InvoicesController@pendings')->name('.pendings');
            Route::get('/pendings/{pendingInvoice}', 'InvoicesController@pendingsCreate')->name('.pendings.create');
            Route::delete('/pendings/{pendingInvoice}', 'InvoicesController@pendingsDestroy')->name('.pendings.destroy');
        });

        Route::get('/getItemPrices', 'InvoicesController@getItemPrices')->name('.getItemPrices');

        Route::get('/unsupported type', function(){
            return view('invoices::unsupported_type');
        })->name('.unsupported_type');
        Route::get('/error while saving', function(){
            return view('invoices::exception');
        })->name('.save_exception');

    });

    Route::get('/items-data', 'InvoicesController@ItemsData')->name('items-data');
    Route::get('/avg-pay-price', 'InvoicesController@payAverage')->name('avg-pay-price');
    Route::get('/last-sale-price', 'InvoicesController@get_last_sale_price')->name('last-sale-price');
    Route::get('invoice/check/bill/number', 'InvoicesController@checkBillNumber')->name('invoice.check.bill.number');

});

// Show the invoice.
Route::group([
    'middleware' => ['web', 'auth'],
    'prefix'     => 'invoices',
    'as'         => 'invoices.'
], function () {
    Route::get('/show/{invoice}', 'InvoicesController@show')->name('show');
    Route::get('/print/{invoice}', 'InvoicesController@print')->name('print');
    Route::get('/client/collector', 'InvoicesController@collector')->name('client.collector');

    // E-Invoice Submit.
    Route::post('einvoice/submit', 'InvoicesController@submitInvoice')->name('einvoice.submit');


    Route::get('item/count', 'InvoicesController@getItemCounts')->name('item.count');
});
