<?php

namespace Modules\Item\Entities;

use App\Traits\RoundingAttributesTrait;
use Illuminate\Database\Eloquent\Model;

class Item extends Model
{
    use RoundingAttributesTrait;
    protected $table = 'items';

    public $timestamps = false;

    protected $guarded = [];

    public function categoryName()
    {
        $cat = Category::where('id', $this->sale_unit);
        return $cat->name;
    }

    public function getImgAttribute($value)
    {
        return $value;
    }

    public function getDetailsAttribute($value)
    {
        return in_array($value, ['', ' ', null]) ? '---' : $value;
    }

    public function getImages()
    {
        $images = ItemImage::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('item_id', $this->id)
            ->get();

        return $images;
    }

    public function vats()
    {
        return $this->belongsToMany(Vat::class, 'item_vat', 'item_id', 'vat_id')->withPivot('vat_use');
    }

    public function invoice_details()
    {
        return $this->hasMany(InvoiceDetail::class, 'item_id');
    }

    public function transactions()
    {
        return $this->hasMany(ItemTransaction::class, 'item_id');
    }
    /**
     * Get the item E-Invoice code.
     *
     * @param  string  $value
     * @return string
     */
    public function getItemEinvoiceCodeAttribute($value)
    {
        return in_array($value, ['', ' ', null]) ? '' : explode('-', $value)[2];
    }

    public function stores()
    {
        return $this->belongsToMany(Stores::class, 'store_items', 'item_id', 'store_id')
            ->withPivot(
            'store_quant',
            'damaged',
            'default_quant',
            'shelf',
            'shop_id',
            'add_user',
            'add_date',
            'edit_user',
            'edit_date'
        );
    }

    // Category
    public function category()
    {
        return $this->belongsTo(Category::class, 'sale_unit');
    }

        public function storeQty()
    {
        return $this->hasMany(StoreItem::class, 'item_id');
    }

    public function subUnits()
    {
        return $this->hasMany(ItemUnit::class, 'item_id')->where('unit_type', 0);
    }

    public function supplier()
    {
        return $this->belongsTo(Client::class, 'supplier_id');
    }

    public function unit()
    {
        return $this->belongsTo(Unit::class, 'unit_id');
    }

    public function itemVats()
    {
        return $this->hasMany(ItemVat::class, 'item_id');
    }

    public function multiPrices()
    {
        return $this->hasMany(ItemPrice::class, 'item_id');
    }
}
