<?php

namespace Modules\Item\Http\Controllers;

use Faker\Provider\Base;
use Illuminate\Http\Request;
use Modules\Item\Entities\Item;
use Illuminate\Routing\Controller;
use Modules\Item\Entities\BadrShop;
use Modules\Item\Entities\BarcodeSetting;
use Illuminate\Contracts\Support\Renderable;
use Modules\Item\Http\Requests\BarcodeRequest;

class BarcodeController extends Controller
{
    /**
     * Show Settings Page for Barcode
     *
     * @return Renderable
     */
    public function settings(BarcodeRequest $request)
    {
        $settings = BarcodeSetting::where('shop_id', get_session_val('serial_id', 'integer'))->first();

        if ($request->isMethod('post')) {
            $validated       = $request->validated();
            $barcodeSettings = BarcodeSetting::updateOrCreate(
                ['shop_id' => get_session_val('serial_id', 'integer')],
                [
                    "support"        => $validated["barcode_reader"],
                    "showitem"       => $validated["show_item_name"],
                    "show_about"     => $validated["show_item_description"],
                    "price"          => $validated["show_price"],
                    "show_shop_name" => $validated["show_shop_name"],
                    "print_barcode"  => $validated["print_barcode_number"],
                    "support_scale"  => $validated["barcode_scale_support"],
                    "show_unit_name" => $validated["show_unit_name"],
                    "barcodecount"   => $validated["barcode_number"],
                    "barcodestyle"   => $validated["barcode_template"],
                    "scale_start"    => $validated["scale_start_code"] ?? null,
                    "fontsize"       => $validated["text_size"],
                    "scale_item"     => $validated["item_code_digits"],
                    "barcodeheight"  => $validated["barcode_height"],
                    "barcodewidth"   => $validated["barcode_width"],
                    "logoheight"     => $validated["logo_height"],
                    "logowidth"      => $validated["logo_width"],
                    "codimgheight"   => $validated["barcode_reader_code_height"],
                    "codimgwidth"    => $validated["barcode_reader_code_width"],
                    "itemup"         => $validated["ticket_top_margin"],
                    "itemdown"       => $validated["ticket_bottom_margin"],
                    "itemleft"       => $validated["ticket_left_margin"],
                    "itemright"      => $validated["ticket_right_margin"],
                    "pageup"         => $validated["page_top_margin"],
                    "pagedown"       => $validated["page_bottom_margin"],
                    "pageleft"       => $validated["page_left_margin"],
                    "pageright"      => $validated["page_right_margin"],
                ]
            );

            return $barcodeSettings == true
                ? back()->with('success', __('extends.save_sucessfully'))
                : back()->with('error', __('extends.save_failed'));
        }

        return view('item::barcode.settings', compact('settings'));
    }

    /**
     * Show Barcode Page
     *
     * @return Renderable
     */
    public function printBarcode(Request $request)
    {
        // dd($request->all());
        $items    = Item::with('subUnits.unitName', 'unit', 'category')->where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $ticket   = BarcodeSetting::where('shop_id', get_session_val('serial_id', 'integer'))->first();

        if ($request->isMethod('post')) {
            $page_count     = 1;
            $requestedItems = json_decode($request->items, true);

            $requestedItems = collect($requestedItems)->map(function ($item) {
                if ($item['id'])
                {
                    $item_ = Item::find($item['id']);
                    $item['item'] = $item_;
                    if (! isset($item['barcode']))
                    {
                        $item['barcode'] = $item_->barcode;
                        $item['price'] = $item_->sale_price;
                    }

                    return $item;
                }
            });

            return view('item::barcode.printTemplate', compact('requestedItems', 'ticket', 'page_count', 'settings'))->render();
        }


        return view('item::barcode.printBarcode', compact('items'));
    }
}
