<?php

namespace Modules\Category\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Controllers\Controller;
use App\Services\Uploaders\CategoryImgUploader;
use Illuminate\Support\Facades\File;
use Modules\Category\Entities\Category;
use Modules\Category\Entities\Item;

class CategoryController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index()
    {
        //        dd(Category::find('243195a659f23ee08c'));
        $categories = Category::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('category_id', 0)
            ->orderBy('display_order')->get();
        return view('category::main.index', compact('categories'));
    }

    /**
     * Show the form for creating a new resource.
     * @return Response
     */
    public function create()
    {
        return view('category::create');
    }

    /**
     * Store a newly created resource in storage.
     * @param  Request $request
     * @return Response
     */
    public function store(Request $request)
    {
       $this->validate($request,[
            'name' => 'unique:items_type',
        ]);

        $categories = Category::where('shop_id' , get_session_val('serial_id' ,'integer'))->get();
        foreach ($categories as $category) {
            if ($request->name == $category->name ){
              return  redirect()->back()->with('error' , 'التصنيف موجود بالفعل');
            }
        }

        $cat = new Category();
        $cat->name = $request->name;
        $cat->display_order = $request->display_order ?? 0;
        $cat->published = $request->published ? 1 : 0;
        $cat->upload_img = "";

        $uploader     =  new CategoryImgUploader($request, 'image');
        $result  =  $uploader->checkAndUpload();

        if ($result['status'])
        {
            $cat->upload_img = $result['path'];
        }

        $cat->category_id = $request->type;
        $cat->shop_id = get_session_val('serial_id' ,'integer');
        $cat->arrange_type = $request->arrange_type? 2 : 1;
        $cat->hiddentouch = $request->hiddentouch? 1 : 0;
        $cat->kitchen_use = $request->kitchen_use? 1 : 0;
        $cat->show_min_qty = $request->show_min_qty? 1 : 0;

        $cat->save();

        return redirect()->back()->with(['success' => 
        trans('native.correct_save')]);
    }

    public function show()
    {
        return view('category::show');
    }

    /*
        public function edit(Request $request)
        {
            $item = Category::find($request->id);

            $data = "";
            if ($item) {
                $data .= '

                   <input type="hidden" name="_token" value="'.csrf_token().'">
                   <input type="hidden" name="item_id" value="'.$item->id.'">
                    <div class="form-group">
                        <label for="name">Name</label>
                        <input class="form-control" name="name" id="name" value="' . $item->name . '">
                    </div>';
                if ($item->upload_img == '' OR is_null($item->upload_img)) {
                    $data .= '<div class="form-group">
                                <label for="image">Image</label>
                                <input type="file" class="form-control" name="image" id="image">
                            </div>';
                } else {
                    $data .= '<div class="row">
                        <div class="col-md-4">
                            <div class="form-group">
                                <label for="image">Image</label>
                                <input type="file" class="form-control" name="image" id="image">
                            </div>
                            </div>
                        <div class="col-md-8">
                            <img height="60" src="' . $item->upload_img . '">
                        </div>
                    </div>';

                }


                $data .= '<div class="form-group">
                        <label for="type">Type</label>
                        <select class="selectpicker form-control" name="type" id="type"
                                data-live-search="true">';
                if ($item->category_id == '0') {
                    $data .= '<option selected value="0" style="background: #5cb85c; color: #fff;">رئيسـي</option>';
                } else {
                    $data .= '<option value="0" style="background: #5cb85c; color: #fff;">رئيسـي</option>';
                }
                $categories = Category::where('shop_id', get_session_val('serial_id', 'integer'))->get();
                foreach ($categories->where('category_id', '0') as $__cat) {
                    if ($item->category_id == $__cat->id) {
                        $data .= '<option selected style="padding-left: 1em; background: #87d6c6; color: #fff;"
                                        value="' . $__cat->id . '">' . $__cat->name . '</option>';
                    } else {
                        $data .= '<option style="padding-left: 1em; background: #87d6c6; color: #fff;"
                                        value="' . $__cat->id . '">' . $__cat->name . '</option>';
                    }
                }

                $data .= '
                        </select>
                    </div>';
            } else {
                $data .= '<p class="alert alert-danger">حدث خطأ ما !!!!</p>';
            }


            return response()->json([
                'data' => $data
            ]);
        }
      */
    public function edit(Request $request)
    {
        $item = Category::find($request->id);
        $data = "";
        if ($item) {
            $data .= '

               <input type="hidden" name="_token" value="' . csrf_token() . '">
               <input type="hidden" name="item_id" value="' . $item->id . '">
                <div class="form-group">
                    <label for="name">اسم التصنيف</label>
                    <input class="form-control" name="name" id="name" value="' . $item->name . '">
                </div>
                <div class="form-group">
                    <label for="name"> ترتيب الظهور</label>
                    <input  type="number" min="0" class="form-control" name="display_order" id="display_order" value="' . $item->display_order . '">
                </div>';
            if ($item->upload_img == '' or is_null($item->upload_img)) {
                $data .= '<div class="form-group">
                            <label for="image">الصورة</label>
                            <input type="file" class="form-control" name="image" id="image">
                        </div>';
            } else {
                $data .= '<div class="row">
                    <div class="col-md-4">
                        <div class="form-group">
                            <label for="image">الصورة</label>
                            <input type="file" class="form-control" name="image" id="image">
                        </div>
                        </div>
                    <div class="col-md-8">
                        <img height="60" src="' . config('main.badr') . 'upload/' . $item->upload_img . '">
                    </div>
                </div>';
            }


            $data .= '<div class="form-group" id="type_div">
                    <label for="type">التصنيف</label>';

            $main_cats = Category::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('category_id', 0)->get();
            $data .= '<script>$("#sale_unit2").treeselect();$("#type_div").find("button").css("width", "100%");</script>';
            $data .= '<select class="form-control" name="type" id="sale_unit2">';
            if ($item->category_id == '0') {
                $data .= '<option selected value="0">تصنيف رئيسي</option>';
            } else {
                $data .= '<option value="0">تصنيف رئيسي</option>';
            }
            foreach ($main_cats as $cat) {
                if ($cat->id == $item->category_id) {
                    $data .= '<option selected value="' . $cat->id . '">' . $cat->name . '</option>';
                } else {
                    $data .= '<option value="' . $cat->id . '">' . $cat->name . '</option>';
                }
                $data .= getSubCatOptionsWithSelected($cat->id, $item->category_id);
            }

            $data .= '</select></div>';

            $checked = '';
            if ($item->published) {
                $checked = 'checked';
            }
            $data .= ' <div class="form-group">
                            <label for="published col-sm-3" style="margin-top: 10px;"> اظهار التصنيف اونلاين</label>
                            <input type="checkbox" class="form-control col-sm-1" name="published" id="published" ' . $checked . '>
                        </div>';

            $checked3 = '';
            $checked4 = '';
            if ($item->arrange_type == 2) {
                $checked3 = 'checked';
            }
            $data .= ' <div class="form-group">
                            <label for="arrange_type col-sm-3" style="margin-top: 10px;">  عرض الوحدات الفرعية   </label>
                            <input type="checkbox" class="form-control col-sm-1" name="arrange_type" id="arrange_type" ' . $checked3 . '>
                        </div>';

            $checked2 = '';
            if ($item->hiddentouch) {
                $checked2 = 'checked';
            }
            $checked7 = '';
            if ($item->kitchen_use) {
                $checked7 = 'checked';
            }
            $checked8 = '';
            if ($item->manufacturing_use) {
                $checked8 = 'checked';
            }
            $data .= ' <div class="form-group">
                            <label for="hiddentouch col-sm-3" style="margin-top: 10px;">  اخفاء بالمبيعات </label>
                            <input type="checkbox" class="form-control col-sm-1" name="hiddentouch" id="hiddentouch" ' . $checked2 . '>
                        </div>
                        <div class="form-group">
                            <label for="kitchen_use col-sm-3" style="margin-top: 10px;">  طباعة في المطبخ  </label>
                            <input type="checkbox" class="form-control col-sm-1" name="kitchen_use" id="kitchen_use" ' . $checked7 . '>
                        </div>
                        <div class="form-group">
                            <label for="show_min_qty col-sm-3" style="margin-top: 10px;">  اظهار في تقرير النواقص </label>
                            <input type="checkbox" class="form-control col-sm-1" name="show_min_qty" id="show_min_qty" ' . $checked4 . '>
                        </div>
                         <div class="form-group">
                            <label for="show_min_qty col-sm-3" style="margin-top: 10px;">  اظهار في تصنيع الموقع </label>
                            <input type="checkbox" class="form-control col-sm-1" name="manufacturing_use" id="manufacturing_use" ' . $checked8 . '>
                        </div>
                        ';
        } else {
            $data .= '<p class="alert alert-danger">حدث خطأ ما !!!!</p>';
        }


        return response()->json([
            'data' => $data
        ]);
    }

    public function update(Request $request)
    {

        $this->validate($request, [
            'name' => 'required',
        ]);

        $cat = Category::find($request->item_id);

        if ($cat) {
            $cat->name = $request->name;
            $cat->display_order = $request->display_order;
            $cat->published = $request->published ? 1 : 0;

            $uploader     =  new CategoryImgUploader($request, 'image');
            $result  =  $uploader->checkAndUpload();

            if ($result['status'])
            {
                $uploader->clean($cat->upload_img);
                $cat->upload_img = $result['path'];
            }

            $cat->category_id = $request->type;

            $cat->arrange_type = $request->arrange_type ? 2 : 1;
            $cat->hiddentouch = $request->hiddentouch ? 1 : 0;
            $cat->kitchen_use = $request->kitchen_use ? 1 : 0;
            $cat->show_min_qty = $request->show_min_qty ? 1 : 0;
            $cat->manufacturing_use = $request->manufacturing_use ? 1 : 0;
            $cat->save();


            $sub_ids = $this->getsubs($cat->id);
            if ($cat->published == 1) {
                $s = 1;
            } else {
                $s = 0;
            }
            Category::whereIn('id', $sub_ids)->update([
                'published' => $s
            ]);


            return redirect()->back()->with(['success' => 'تم التعديل بنجاح']);
        }
    }

    public function getsubs($id)
    {
        $a = [];
        $units = Category::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('category_id', $id)->pluck('id');
        $units = json_decode(json_encode($units));
        for ($i = 0; $i < sizeof($units); $i++) {
            $data = $this->getSubs($units[$i]);
            $a = array_merge($a, $units);
        }
        return $a;
    }

    public function destroy(Request $request)
    {

        $id = $request->category_id;
        $category = Category::where('id', $id)
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->first();

        $sub = $category->subCategory();
        if (count($sub) > 0) {

            return redirect()->back()->with([
                'error' => trans('native.cannot_deleted_contain')  
            ]);
        } else {

            $check = Item::where('sale_unit', $id)->where('shop_id', get_session_val('serial_id', 'integer'))->get();
            if (count($check)) {

                return redirect()->back()->with([
                    'error' => trans('native.cannot_deleted_contain')  
                ]);
            } else {

                $category->delete();
                return redirect()->back()->with([
                    'success' => "تم الحذف بنجاح"
                ]);
            }
        }
    }
}
