<?php

namespace Modules\Item\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Imports\ItemImport;
use App\Models\systemLog;
use App\Models\User;
use App\Services\Uploaders\ItemImgUploader;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Intervention\Image\Facades\Image;
use Maatwebsite\Excel\Facades\Excel;
use Modules\EInvoice\Entities\Settings;
use Illuminate\Validation\Rule;
use Modules\Invoices\Entities\Invoices;
use Modules\Invoices\Entities\InvoiceDetails;
use Modules\Item\Entities\{
    BadrShop,
    BillAdd,
    CardCompany,
    Category,
    Client,
    Color,
    Company,
    Item,
    ItemColor,
    ItemImage,
    ItemPrice,
    ItemSize,
    ItemType,
    ItemUnit,
    ItemVat,
    Manufacturingcats,
    PriceList,
    Shelf,
    Size,
    Store,
    Stores,
    Unit,
    Vat,
    StoreItem
};
use Modules\Item\Entities\SupplierItem;
use Modules\SalePoints\Entities\clientTransaction;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Modules\BadrRequest\Entities\SalePoint;

class ItemController extends Controller
{
    public function syncItemVats()
    {
        ini_set('max_execution_time', 999999999999999999);
        ItemVat::truncate();
        $add_id = BillAdd::select('id')->first()->id;
        $arr = [];
        foreach (Item::select('id')->where('shop_id', get_session_val('serial_id', 'integer'))->pluck('id') as $id) {
            $arr[] = [
                'item_id'  => $id,
                'vat_id'   => $add_id,
                'vat_use'  => 0,
                'shop_id'  => get_session_val('serial_id', 'integer')
            ];
        }

        ItemVat::insert($arr);
        return back();
    }

    public function getCategoriesId()
    {
        $cats = Category::select('id', 'category_id')->where('category_id', 0)->where('shop_id', get_session_val('serial_id', 'integer'))->with('subs')->get()->toArray();
        $ids = [];
        foreach ($cats as $cat) {
            $this->getCatId($cat, $ids);
        }
        return $ids;
    }

    protected function getCatId(array $cat, &$ids)
    {
        if (in_array(request()->cat_id, [$cat['id'], $cat['category_id']]) || in_array($cat['category_id'], $ids))
            $ids[$cat['id']] = $cat['id'];

        if ($cat['subs']) {
            foreach ($cat['subs'] as $sub) {
                $this->getCatId($sub, $ids);
            }
        }
    }

    public function index(Request $request)
    {
        if (!authorized('can_display_items')) {
            return unAuthorized(trans('users.show_items'));
        }

        $decimal_num_quant = get_session_val('decimal_num_quant', 'integer');
        $decimal_num_price = get_session_val('decimal_num_price', 'integer');

        $items = Item::select('id', 'item_name as name', 'manufacturing', 'item_einvoice_code', 'supplier_id', 'sale_unit', 'barcode',
                                DB::raw("TRUNCATE(sale_price ,$decimal_num_price) as price"),
                                DB::raw("TRUNCATE(pay_price ,$decimal_num_price) as pay_price")
                            )
            ->with('supplier:id,client_name as name', 'category:id,name')
            ->withSum([
                'storeQty as quantity' => function ($query) use ($request) {
                    $query->when($request->store_id, function ($query) use ($request) {
                        $query->where('store_id', $request->store_id);
                    });
                }
            ], 'store_quant')
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->when(authorized('define_categories'), function($query) {
                $query->whereIn('items.sale_unit', getUserCategoriesId());
            })
            ->when($request->sup_id, fn ($query) => $query->where('supplier_id', $request->sup_id))
            ->when($request->search && strlen($request->search), function ($query) use ($request) {
                $query->where(function ($query) use ($request) {
                    $query->when(!$request->barcode, function ($query) use ($request) {
                            $query->orWhere('item_name', 'LIKE', "%{$request->search}%")->orWhere('about', 'LIKE', "%{$request->search}%");
                        })->when($request->barcode, function($query) use($request) {
                            $query->whereHas('subUnits', function($query) use($request) {
                                        $query->where('unit_barcode', 'LIKE', "%{$request->search}%");
                                    })->orWhere('barcode', 'LIKE', "%{$request->search}%");
                        });
                });
            })
            ->when($request->store_id, function ($query) use ($request) {
                $query->whereHas('storeQty', fn ($query) => $query->where('store_id', $request->store_id));
            })
            ->when(request()->has('cat_id') && request()->get('cat_id') > 0, function ($query) {
                $query->whereIn('sale_unit', $this->getCategoriesId());
            })
            ->orderBy('id', 'desc')->paginate(20);


        $cats      = Category::select('id', 'name')->whereShopId(get_session_val('serial_id', 'integer'))->orderBy('name')->get();
        $stores    = Store::select('id', 'store_name')->whereShopId(get_session_val('serial_id', 'integer'))->when(!auth()->user()->change_store, function ($query) {
            $query->where('id', auth()->user()->sale_point);
        })->get();
        // ***********************************************************************************
        //يتاح للمستخدم الموردين الموجودين في المجموعه الخاصه بيه فقط
        if (auth()->user()->clients_group == 0) {
            $suppliers = Client::where(['shop_id' => get_session_val('serial_id', 'integer')])->whereIn('client_role', [1, 2])->get();
        } else {
            $groupIds = auth()->user()->groups->pluck('id');
            $suppliers = Client::select('id', 'client_name')->where(['shop_id' => get_session_val('serial_id', 'integer')])
                ->whereIn('client_role', [1, 2])
                ->whereIn('group_id', $groupIds)->get();
        }
        // ***************************************************************************************

        $settings  = Badrshop::select('einvoice')->whereSerialId(get_session_val('serial_id', 'integer'))->first();
        return view('item::index', compact('items', 'cats', 'suppliers', 'stores', 'settings'));
    }

    public function indexOld(Request $request)
    {
        if (!authorized('can_display_items')) {
            return unAuthorized(trans('users.show_items'));
        }

        $decimal_num_quant = get_session_val('decimal_num_quant', 'integer');
        $decimal_num_price = get_session_val('decimal_num_price', 'integer');

        $query = \DB::table('items')->selectRaw(
            "items.id,
            item_name,manufacturing,
            TRUNCATE(sale_price ,$decimal_num_price) as sale_price ,
            min_quantity,
            TRUNCATE(pay_price ,$decimal_num_price) as pay_price ,
            TRUNCATE(lowest_price ,$decimal_num_price) as lowest_price ,
            about,barcode,
            bolla ,
            items.incom_date,
            client_name as supplier_name,
            incom_date,
            factories.name as factory_name,
            items.factory_id,
            items_type.name ,items_type.id as typeid,
            items.expire_date, purchase_code, sale_code,items.item_einvoice_code,items.parent_code,
            SUM(store_items.store_quant) as quantity,
            stores.store_name, stores.id AS store_id"
        );

        $typeJoin = 'leftJoin';

        if ($request->cat_id > 0) {
            $typeJoin = 'join';
        }

        $query = $query->$typeJoin('items_type', function ($q) use ($request) {
            $q->on('items.sale_unit', 'items_type.id');
            $q->where('items_type.shop_id', get_session_val('serial_id', 'integer'));

            if ($request->cat_id > 0) {
                $q->where(function ($q) use ($request) {
                    $q->where('items_type.id', $request->cat_id)->orWhere('items_type.category_id', $request->cat_id);
                });
            }
        });

        $query = $query->leftJoin('items_unit', function ($q) {
            $q->on('items.id', 'items_unit.item_id');
        });

        $clientJoin = 'leftJoin';

        if ($request->sup_id > 0) {
            $clientJoin = 'join';
        }

        $query->$clientJoin('clients', function ($q) use ($request) {
            $q->on('clients.id', 'items.supplier_id');
            $q->where('clients.shop_id', get_session_val(
                'serial_id',
                'integer'
            ));

            if ($request->sup_id) {
                $q->where('clients.id', $request->sup_id);
            }
        });

        $query->leftJoin('factories', function ($q) {
            $q->on('factories.id', 'items.factory_id');
        })
            ->where('items.shop_id', get_session_val('serial_id', 'integer'));

        if ($request->search && strlen($request->search)) {
            $query = $query->where(function ($w) use ($request) {
                $w->where('barcode', 'LIKE', "%" . $request->search . "%");
                // $w->orWhere('unit_barcode', 'LIKE', "%" . $request->search . "%");

                if (!$request->barcode) {
                    $w->orWhere('item_name', 'LIKE', "%" . $request->search . "%");
                    $w->orWhere('about', 'LIKE', "%" . $request->search . "%");
                }
            });
        }

        $StoreJoin = 'leftJoin';

        $store_id = null;
        if (!auth()->user()->change_store) {
            $store_id = SalePoint::select('store_id')->where('id', auth()->user()->sale_point)->first()->store_id ?? null;
        }

        $store_id = $request->store_id > 0 ? $request->store_id : $store_id;

        if ($store_id) {
            $StoreJoin = 'join';
        }

        $query = $query->$StoreJoin('store_items', function ($q) use ($store_id) {
            $q->on('store_items.item_id', 'items.id');
            if ($store_id) {
                $q->where('store_items.store_id', $store_id);
            }
            $q->where('store_items.shop_id', get_session_val('serial_id', 'integer'));
        })
            ->$StoreJoin('stores', function ($q) use ($store_id) {
                $q->on('stores.id', 'store_items.store_id');
                $q->when(!is_null($store_id), function ($qq) use ($store_id) {
                    $qq->where('store_items.store_id', $store_id);
                });
                $q->where('stores.shop_id', get_session_val('serial_id', 'integer'));
            });

        $items     = $query->orderBy('items.id', 'desc')->distinct()->groupBy('items.id')->simplePaginate(20);
        $cats      = Category::whereShopId(get_session_val('serial_id', 'integer'))->orderBy('name')->get();
        $stores    = Store::whereShopId(get_session_val('serial_id', 'integer'))->when(!auth()->user()->change_store, function ($query) {
            $query->where('id', auth()->user()->sale_point);
        })->get();
        // ***********************************************************************************
        //يتاح للمستخدم الموردين الموجودين في المجموعه الخاصه بيه فقط
        $user = User::Find(auth()->user()->id);
        if (auth()->user()->clients_group == 0) {
            $suppliers = Client::where(['shop_id' => get_session_val('serial_id', 'integer')])->whereIn('client_role', [1, 2])->get();
        } else {
            $groupIds = $user->groups->pluck('id');
            $suppliers = Client::where(['shop_id' => get_session_val('serial_id', 'integer')])
                ->whereIn('client_role', [1, 2])
                ->whereIn('group_id', $groupIds)->get();
        }
        // ***************************************************************************************

        $settings  = Badrshop::whereSerialId(get_session_val('serial_id', 'integer'))->first();
        return view('item::index', compact('items', 'cats', 'suppliers', 'stores', 'settings'));
    }

    public function create(Request $request)
    {

        if (!authorized('can_add_item')) {
            return unAuthorized(trans('users.add_new_item'));
        }

        $suppliers         = Client::where('shop_id', get_session_val('serial_id', 'integer'))->whereIn('client_role', [1, 2])->get();
        $price_decimal     = get_session_val('decimal_num_price', 'integer');
        $type              = get_session_val('date_type', 'integer');
        // $main_cats         = Category::where('category_id', 0)->where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $main_cats         = Category::where('shop_id', get_session_val('serial_id', 'integer'))
                                        ->when(authorized('define_categories'), function($query) {
                                            $query->whereIn('id', getUserCategoriesId());
                                        })->get();
        $stores            = Stores::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $units             = Unit::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $shelves           = Shelf::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $companies         = Company::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $colors            = Color::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $sizes             = Size::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $price_lists        = PriceList::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $cardCompany       = CardCompany::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $settings          = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $einvoice_settings = Settings::shop()->first();
        $barcode           = Item::where('shop_id', get_session_val('serial_id', 'integer'))->orderBy('id', 'desc')->first()->barcode ?? 1000000;
        $barcode           += 1;

        $vats              = $settings->einvoice == 1
            ? Vat::where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereNotNUll('einvoice_code')
            ->get()
            : Vat::where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereNull('einvoice_code')
            ->whereNull('sub_einvoice_code')
            ->get();

        $embedded = 0;
        if (isset($request->embedded)) {
            $embedded = 1;
        }

        return view('item::create', compact(
            'settings',
            'vats',
            'cardCompany',
            'price_lists',
            'sizes',
            'colors',
            'companies',
            'shelves',
            'units',
            'main_cats',
            'stores',
            'suppliers',
            'type',
            'price_decimal',
            'einvoice_settings',
            'barcode',
            'embedded'
        ));
    }

    public function store(Request $request)
    {
        //   dd($request->all());
        $prices = $request->except(
            'item_name',
            'supplier_id',
            'quantity',
            'sale_unit',
            'unit_id',
            'pay_price',
            'sale_price',
            'image',
            'sub_cat',
            'barcode',
            'n_barcode',
            'expire_date',
            'lowest_price',
            'min_quantity',
            'pay_date',
            'available',
            'game',
            'about',
            'details',
            'images',
            'is_service',
            'online',
            'company_id',
            'shelf',
            'background_color',
            'cooking_add',
            'order_show',
            'sizes',
            'colors',
            'bolla',
            'meter_ceramic',
            'require_serial',
            'card_company_id',
            'sub_units',
            '_token',
            'intern_cat',
            "small_qty",
            "medium_qty",
            "large_qty",
            "manufacturing",
            "unit_einvoice_code",
            "parent_code",
            "vats",
            "vats_ids",
            'item_einvoice_code',
            "vat_state",
            "vat_ids",
            "purchase_vat_state",
            "purchase_vat_ids",
            "category",
            'stores',
            'print_barcode',
            'einvoice_vat_state',
            'online',
            'is_special',
        );

        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $rules    = [
            'item_name'     => ['required', 'string', 'max:255', 'unique:items,item_name,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')],
            'barcode'       => ['required', 'unique:items,barcode,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')],
            'n_barcode'     => ['sometimes', 'unique:items_unit,unit_barcode,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')],
            'sub_unit_barcode' => ['sometimes',
             'unique:items_unit,unit_barcode,NULL,id,shop_id,' . get_session_val('serial_id', 'integer'),
             'unique:items,barcode,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')
            ],
            'supplier_id'   => ['required'],
            'quantity'      => ['required'],
            'category'      => ['required'],
            'unit_id'       => ['required'],
            'pay_price'     => ['required'],
            'sale_price'    => ['required'],
            'images' => ['nullable', 'mimes:png,jpg,jpeg', 'max:2000'],
            'manufacturing' => ['nullable'],
            'stores'        => ['sometimes', Rule::unique('store_items', 'store_id')->where(function ($query) {
                return $query->where('shop_id', get_session_val('serial_id', 'integer'))->where('item_id', request()->item_id);
            })],
        ];

        $messages = [
            'item_name.required'     => __('items.item_name.required'),
            'item_name.string'       => __('items.item_name.string'),
            'item_name.max'          => __('items.item_name.max'),
            'item_name.unique'       => __('items.item_name.unique'),
            'barcode.required'       => __('items.barcode.required'),
            'barcode.unique'         => __('items.barcode.unique'),
            'sub_unit_barcode.unique' => __('items.sub_unit_barcode.unique'),
            'n_barcode.unique'       => __('items.n_barcode.unique'),
            'supplier_id.required'   => __('items.supplier_id.required'),
            'quantity.required'      => __('items.quantity.required'),
            'category.required'      => __('items.category.required'),
            'unit_id.required'       => __('items.unit_id.required'),
            'pay_price.required'     => __('items.pay_price.required'),
            'sale_price.required'    => __('items.sale_price.required'),
            'image.mimes'            => __('items.image.mimes'),
            'manufacturing.nullable' => __('items.manufacturing.nullable'),
            'stores.unique'          => __('items.stores.unique'),
        ];

        if ($settings->einvoice == 1) {
            // $rules['item_einvoice_code'] = ['required', 'unique:items,item_einvoice_code,' . $request->item_id . ',id'];
            $rules['unit_einvoice_code'] = ['required'];
            $rules['parent_code']        = ['required'];
            $rules['vats']               = ['nullable', 'array'];

            // $messages['item_einvoice_code.required'] = __('items.item_einvoice_code_required');
            // $messages['item_einvoice_code.unique']   = __('items.item_einvoice_code_unique');
            $messages['unit_einvoice_code.required'] = __('items.unit_einvoice_code_required');
            $messages['parent_code.required']        = __('items.parent_code_required');
        }

        if ($request->quantity > 0) {

            $rules['stores'] = 'required';
        }

        $this->validate($request, $rules, $messages);

        $item = new Item();

        DB::transaction(function () use ($request, $prices, $settings, $item) {

            /*
            if (is_null($request->sub_cat)) {
                $cat_id = $request->sale_unit;
            } else {
                if (is_null($request->intern_cat)) {
                    $cat_id = $request->sub_cat;
                } else {
                    $cat_id = $request->intern_cat;
                }
            }
            */

            $einvoice_settings        = Settings::shop()->first();
            $item->item_name          = $request->item_name;
            $item->barcode            = ($request->barcode) ? $request->barcode[0] : '';
            $item->sale_unit          = $request->category;
            $item->sale_price         = $request->sale_price;
            $item->lowest_price       = $request->lowest_price;
            $item->pay_price          = $request->pay_price;
            // $item->quantity           = $request->quantity;
            $item->unit_id            = $request->unit_id;
            $item->expire_date        = ($request->expire_date) ? $request->expire_date : date('Y-m-d', strtotime('+20 years'));;
            $item->min_quantity       = $request->min_quantity;
            $item->incom_date         = $request->pay_date ?? now();
            $item->available          = ($request->available) ? 1 : 0;
            $item->online          = ($request->online) ? 1 : 0;
            $item->is_special          = ($request->is_special) ? 1 : 0;
            $item->game               = ($request->game) ? 1 : 0;
            $item->about              = $request->about ?? ' ';
            $item->supplier_id        = $request->supplier_id[0];
            $item->details            = ($request->details) ? $request->details : '';
            $item->edit_user          = auth()->id();
            $item->used_for_small     = $request->small_qty ?? 1;
            $item->used_for_medium    = $request->medium_qty ?? 1;
            $item->used_for_large     = $request->large_qty ?? 1;
            $item->manufacturing      = $request->manufacturing ?? 0;
            $item->parent_code        = $request->parent_code;
            $item->item_einvoice_code = ($settings->einvoice == 1 && !is_null($request->item_einvoice_code))
                ? 'EG-' . $einvoice_settings->registration_id . '-' . $request->item_einvoice_code
                : null;
            $item->vat_state          = $request->vat_state ?? 0;
            $item->vat_id             = 0;
            $item->purchase_vat_state = $request->purchase_vat_state ?? 0;
            $item->purchase_vat_id    = 0;

            $item->img                = '';

            $uploader     =  new ItemImgUploader($request, 'images');
            $result  =  $uploader->checkAndUpload();
            if ($result['status']) {
                $item->img = $result['path'];
            }

            $item->add_user   = auth()->id();
            $item->shop_id    = get_session_val('serial_id', 'integer');
            $item->is_service = (int) $request->has('is_service') ?? 0;
            $item->online     = ($request->online) ? 1 : 0;
            $item->company_id = $request->company_id;
            $item->shelf      = $request->shelf;
            $item->edit_date  = date('Y-m-d H:i:s');

            if ($request->has('background_color')) {
                $item->background_color = $request->background_color;
            }
            if ($request->has('cooking_add')) {
                $item->cooking_add = $request->cooking_add;
            }
            if ($request->has('order_show')) {
                $item->order_show = $request->order_show;
            }
            if ($request->has('meter_ceramic')) {
                $item->meter_ceramic = $request->meter_ceramic;
            }
            if ($request->has('require_serial')) {
                $item->require_serial = $request->require_serial;
            }
            if ($request->has('card_company_id')) {
                $item->card_company_id = $request->card_company_id;
            }
            if ($request->has('bolla')) {
                $item->bolla = $request->bolla;
            }

            $item->disc_value     = 0;
            $item->device_id      = 0;
            $item->pur_unit       = 0;
            $item->pur_unit_equal = 0;
            $item->item_group     = 0;
            if ($request->has('order_max_quantity')) {
                $item->order_max_quantity = $request->order_max_quantity;
            }

            $item->save();
            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            // Sales Vat
            if ($request->vat_state != 0 && $request->has('vat_ids')) {
                foreach ($request->vat_ids as $vat_id) {
                    $sales_vat = explode('-', $vat_id);
                    ItemVat::create([
                        'item_id'  => $item->id,
                        'vat_id'   => $sales_vat[0],
                        'vat_use'  => 1,
                        'shop_id'  => get_session_val('serial_id', 'integer'),
                        'vat_type' => $sales_vat[1],
                    ]);
                }
            }

            // Purchase Vat
            if ($request->purchase_vat_state != 0 && $request->has('purchase_vat_ids')) {
                foreach ($request->purchase_vat_ids as $purchase_vat_id) {
                    $purchase_vat = explode('-', $purchase_vat_id);
                    ItemVat::create([
                        'item_id'  => $item->id,
                        'vat_id'   => $purchase_vat[0],
                        'vat_use'  => 2,
                        'shop_id'  => get_session_val('serial_id', 'integer'),
                        'vat_type' => $purchase_vat[1],
                    ]);
                }
            }

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            // store
            if ($request->has('stores') && !is_null($request->stores)) {
                StoreItem::insert([
                    'store_id'    => $request->stores,
                    'item_id'     => $item->id,
                    'store_quant' => $request->quantity,
                    'shop_id'     => get_session_val('serial_id', 'integer'),
                    'add_user'    => auth()->id(),
                ]);

                $invoice = Invoices::create([
                    'invoice_date' => date('Y-m-d h:i:s'),
                    'transaction_date' => date('Y-m-d h:i:s'),
                    'add_user' => auth()->user()->id,
                    'client_id' => 0,
                    'store_id' => $request->stores,
                    'notes' => '',
                    'shop_id' => get_session_val('serial_id', 'Integer'),
                    'invoice_type' => 7,
                    'invoice_total_disc' => 0,
                    'invoice_total_disc_type' => 1,
                    'invoice_total_disc_status' => 1
                ]);

                InvoiceDetails::create([
                    'invoice_id' => $invoice->id,
                    'shop_id' => get_session_val('serial_id', 'Integer'),
                    'item_id' => $item->id,
                    'quantity' => $request->quantity,
                    'quantity_after' => $request->quantity,
                    'unit_id' => $item->unit_id,
                    'store_id' => $request->stores,
                ]);
            }

            if ($request->embedded)
                Cache::set('embedded_item_' . Auth::id(), $item->barcode);

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            if ($request->has('sizes')) {
                foreach ($request->sizes as $key => $val) {
                    $itemSize            = new ItemSize();
                    $itemSize->item_id   = $item->id;
                    $itemSize->size_id   = $val;
                    $itemSize->shop_id   = get_session_val('serial_id', 'integer');
                    $itemSize->add_user  = auth()->id();
                    $itemSize->edit_user = auth()->id();
                    $itemSize->add_date  = date('Y-m-d H:i:s');
                    $itemSize->save();
                }
            }

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            if ($request->has('colors')) {
                foreach ($request->colors as $key => $val) {
                    $itemColor            = new ItemColor();
                    $itemColor->item_id   = $item->id;
                    $itemColor->color_id  = $val;
                    $itemColor->shop_id   = get_session_val('serial_id', 'integer');
                    $itemColor->edit_user = auth()->id();
                    $itemColor->add_user  = auth()->id();
                    $itemColor->add_date  = date('Y-m-d H:i:s');
                    $itemColor->edit_date = date('Y-m-d H:i:s');
                    $itemColor->save();
                }
            }

            foreach ($request->supplier_id as $value) {

                $s_i = new SupplierItem();
                $s_i->item_id = $item->id;
                $s_i->supplier_id = $value;
                $s_i->shop_id = get_session_val('serial_id', 'integer');
                $s_i->save();
            }

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            // if (sizeof($prices) > 0) {
            //     foreach ($prices as $key => $value) {
            //         $x                     = explode('-', $key);
            //         $price_id              = $x[1];
            //         $item_price            = new ItemPrice();
            //         $item_price->item_id   = $item->id;
            //         $item_price->list_id   = $price_id;
            //         $item_price->price     = $value;
            //         $item_price->shop_id   = get_session_val('serial_id', 'integer');
            //         $item_price->add_user  = auth()->id();
            //         $item_price->add_date  = date('Y-m-d H:i:s');
            //         $item_price->edit_user = auth()->id();
            //         $item_price->edit_date = date('Y-m-d H:i:s');
            //         $item_price->save();
            //     }
            // }

            // dd($request->all());
            if ($request->item_prices) {
                foreach ($request->item_prices as $key => $value) {

                    if ($value > 0 || $request->item_prices_qtys[$key] > 0) {
                        $item_price                =  new ItemPrice();
                        $item_price->item_id       =  $item->id;
                        $item_price->list_id       =  $request->item_prices_ids[$key];
                        $item_price->price         =  $value;
                        $item_price->list_quant    =  $request->item_prices_qtys[$key];
                        $item_price->shop_id       =  get_session_val('serial_id', 'integer');
                        $item_price->add_user      =  auth()->id();
                        $item_price->add_date      =  date('Y-m-d H:i:s');
                        $item_price->edit_user     =  auth()->id();
                        $item_price->edit_date     =  date('Y-m-d H:i:s');
                        $item_price->save();
                    }
                }
            }


            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            /*   `Save the main unit for this item
            ## Saved data will like:
                1. unit_type = 3;
                2. unit_value = 1;
                3. unit_price = same item's sale_price;
                4. unit_barcode = first barcode;
                5. unit_id = same item's unit_id;
             # that's it for now XD`;*/

            if ($request->has('n_barcode') && sizeof($request->n_barcode) >= 1) {

                $request->barcode = array_merge($request->barcode, $request->n_barcode);
            }

            if (sizeof($request->barcode) > 1) {
                $main_unit                     = new ItemUnit();
                $main_unit->item_id            = $item->id;
                $main_unit->unit_id            = $request->unit_id;
                $main_unit->unit_type          = 3;
                $main_unit->unit_value         = 1;
                $main_unit->unit_price         = $item->sale_price;
                $main_unit->Screenhidden       = 0;
                $main_unit->shop_id            = get_session_val('serial_id', 'integer');
                $main_unit->unit_barcode       = ($request->barcode[0] == null) ? "" : $request->barcode[0];
                $main_unit->unit_einvoice_code = ($settings->einvoice == 1) ? (Unit::findOrFail($request->unit_id)->unit_einvoice_code ?? '') : "";
                $main_unit->save();
            }



            if ($request->has('sub_unit_barcode') && sizeof($request->sub_unit_barcode) >= 1) {

                for ($sub = 0; $sub < sizeof($request->sub_unit_barcode); $sub++) {
                    $sub_unit                   = new ItemUnit();
                    $sub_unit->item_id           = $item->id;
                    $sub_unit->unit_id            = $request->sub_unit_badr_id[$sub];
                    $sub_unit->unit_type          = 2;
                    $sub_unit->unit_value         = $request->sub_bar_unit_value[$sub];
                    $sub_unit->unit_price         = $request->sub_bar_unit_price[$sub];
                    $sub_unit->Screenhidden       = 0;
                    $sub_unit->unit_barcode       = $request->sub_unit_barcode[$sub];
                    $sub_unit->unit_einvoice_code = $settings->einvoice == 1 ? (Unit::findOrFail($item->unit_id)->unit_einvoice_code ?? '') : "";
                    $sub_unit->shop_id            = get_session_val('serial_id', 'integer');
                    $sub_unit->save();
                }

            }


            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            /*  `loop through other barcodes for this item
             # it will be saved in items_unit table.
             the other data will like:
                1. unit_type = 2 => equals the main unit (3);
                2. unit_value = 1;
                3. unit_price = same item's sale_price;
                4. unit_barcode = this looped iteration barcode;
                5. unit_id = same item's unit_id;
             # that's it for now XD`;*/

            if (sizeof($request->barcode) > 1) {
                for ($b = 1; $b < sizeof($request->barcode); $b++) {
                    $b_unit                     = new ItemUnit();
                    $b_unit->item_id            = $item->id;
                    $b_unit->unit_id            = $item->unit_id;
                    $b_unit->unit_type          = 2;
                    $b_unit->unit_value         = 1;
                    $b_unit->unit_price         = $item->sale_price;
                    $b_unit->Screenhidden       = 0;
                    $b_unit->unit_barcode       = $request->barcode[$b];
                    $b_unit->unit_einvoice_code = $settings->einvoice == 1 ? (Unit::findOrFail($item->unit_id)->unit_einvoice_code ?? '') : "";
                    $b_unit->shop_id            = get_session_val('serial_id', 'integer');
                    $b_unit->save();
                }
            }

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            /*   `loop through sub units for this item
             # it will be saved in items_unit table.
             the other data will like:
                1. unit_type = [0 => bigger or 1 => smaller];
                2. unit_value = from request;
                3. unit_price = from request;
                4. unit_barcode = from request;
                5. unit_id = from request;
             # that's it`;*/

            if ($request->has('sub_units')) {

                //                `The Request is like this:
                //                    ## string that is separated with (-).
                //                    ## Sigmants are in order:
                //                        1. unit_barcode.
                //                        2. unit_id.
                //                        3. unit_price.
                //                        4. unit_type.
                //                        5. unit_value.
                //                        6. Screenhidden
                //                        7. unit_einvoice_code.
                //                Now, we are ready to go!`;

                foreach ($request->sub_units as $k => $val) {

                    $unit_arr                = explode('-', $val);
                    $barcode                 = $unit_arr[0];
                    if ($settings->einvoice == 1) {
                        $unit_einvoice_code  = trim($unit_arr[1]);
                    }
                    $u_id                    = ($settings->einvoice == 1) ? $unit_arr[2] : $unit_arr[1];
                    $price                   = ($settings->einvoice == 1) ? $unit_arr[3] : $unit_arr[2];
                    $type                    = ($settings->einvoice == 1) ? $unit_arr[4] : $unit_arr[3];
                    $value                   = ($settings->einvoice == 1) ? $unit_arr[5] : $unit_arr[4];
                    $onscreen                = ($settings->einvoice == 1) ? $unit_arr[6] : $unit_arr[5];

                    $sub                     = new ItemUnit();
                    $sub->item_id            = $item->id;
                    $sub->unit_id            = $u_id;
                    $sub->unit_type          = $type;
                    $sub->unit_value         = $value;
                    $sub->unit_price         = $price;
                    $sub->Screenhidden       = $onscreen;
                    $sub->unit_barcode       = $barcode;
                    $sub->unit_einvoice_code = $unit_einvoice_code ?? null;
                    $sub->shop_id            = get_session_val('serial_id', 'integer');
                    $sub->save();
                }
                $this->createLog(Carbon::now(), "تم انشاء وحده فرعيه للصنف  ". $request->item_name);
            }

            /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

            // /*   `loop through vats for this item
            //      # it will be saved in item_vat table.
            //         the other data will like:
            //         1. vats = from request;
            //      # that's it`;
            // */

            // if ($request->has('vats') && $request->vats != null) {
            //     foreach ($request->vats as $vat_id => $vat_use) {
            //         $vat          = new ItemVat();
            //         $vat->item_id = $item->id;
            //         $vat->vat_id  = $vat_id;
            //         $vat->vat_use = $vat_use[0];
            //         $vat->shop_id = get_session_val('serial_id', 'integer');
            //         $vat->save();

            //         // Change vat_State in items table
            //         $item->vat_state          = ($vat_use == 1 || $vat_use == 0) ? $request->einvoice_vat_state : 0;
            //         $item->purchase_vat_state = ($vat_use == 2 || $vat_use == 0) ? $request->einvoice_vat_state : 0;
            //         $item->save();
            //     }
            // }
            $this->createLog(Carbon::now(), " تم انشاء صنف  ".$request->item_name);
        });

        return redirect()->back()->with(['success' => __('extends.save_sucessfully'), 'item_id' => $item->id, 'print_barcode' => $request->print_barcode ?? 0]);
    }

    public function show($id, Request $request)
    {
        $store_id = SalePoint::select('store_id')->where('id', auth()->user()->sale_point)->first()->store_id ?? null;
        $item = Item::with([
            'stores' => function ($query) use ($store_id) {
                $query->when(!auth()->user()->change_store, function ($query) use ($store_id) {
                    $query->where('store_id', $store_id);
                });
            }
        ])
            ->where('items.id', $id)
            ->first();

        if ($item) {
            $cat = Category::where('id', $item->sale_unit)->first();

            if ($cat) {
                if ($cat->category_id == 0) { # main_cat
                    $item->main_cat = $cat->name;
                    $item->sub_cat  = '---';
                } else { # sub_cat
                    $item->sub_cat = $cat->name;
                    $main           = Category::find($cat->category_id);
                    $item->main_cat = $main ? $main->name : '---';
                }
            } else {
                $item->main_cat = '---';
                $item->sub_cat  = '---';
            }

            $supplier       = Client::where('id', $item->supplier_id)->first();
            $item->supplier = $supplier ? $supplier->client_name : '---';
            $item->unit     = Unit::find($item->unit_id)->name ?? '---';

            return view('item::show', compact('item'));
        } else {
            abort(404);
        }
    }

    public function destroy(Request $request)
    {
        if (!authorized('can_delete_items')) {
            return response()->json(['status' => false, 'message' => ' تأكد من تفعيل صلاحية حذف صنف']);
        }
        $item = Item::find($request->item_id);
        if ($item && $item->invoice_details->count() < 2) {
            ItemSize::where('item_id', $item->id)->delete();
            ItemColor::where('item_id', $item->id)->delete();
            ItemUnit::where('item_id', $item->id)->delete();
            ItemPrice::where('item_id', $item->id)->delete();
            $this->createLog(Carbon::now(), " تم حذف الصنف  ". $item->item_name);
            $item->delete();

            return response()->json(['status' => true, 'message' => __('extends.delete_sucessfully')]);
        } else {
            return response()->json(['status' => false, 'message' => __('extends.cannot_delete')]);
        }
    }

    public function storeSupplier(Request $request)
    {
        $settings = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();

        $rules = [
            'mobile1' => ['required', 'regex:/^[0-9]{10}$/', 'unique:clients,mobile1'],
        ];

        $rules["supplier_name"] = $settings->repeat_client_name == 0
            ? ['required', 'string', 'max:255', 'unique:clients,client_name']
            : ['required', 'string', 'max:255'];

        $messages = [
            'supplier_name.required' => __('items.supplier_name.required'),
            'supplier_name.string' => __('items.supplier_name.string'),
            'supplier_name.max' => __('items.supplier_name.max'),
            'supplier_name.unique' => __('items.supplier_name.unique'),
            'mobile1.required' => __('items.mobile1.required'),
            'mobile1.regex' => __('items.mobile1.regex'),
            'mobile1.unique' => __('items.mobile1.unique')
        ];

        $this->validate($request, $rules, $messages);

        $client = new Client();
        $client->shop_id = get_session_val('serial_id', 'integer');
        $client->client_name = $request->supplier_name;
        $client->mobile1 = $request->tele;
        $client->save();

        $option = '<option selected value="' . $client->id . '">' . $client->client_name . '</option>';

        $list = '<li data-original-index="' . $client->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $client->client_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json([
            'status' => true,
            'option' => $option,
            'list' => $list,
        ]);
    }

    public function storeShelf(Request $request)
    {
        $rules = [
            'shelf_name' => ['required', 'unique:store_shelfs,shelf_name,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')]
        ];

        $messages = [
            'shelf_name.required' => __('items.shelf_name.required'),
            'shelf_name.unique' => __('items.shelf_name.unique')
        ];

        $this->validate($request, $rules, $messages);

        $shelf = new Shelf();
        $shelf->shelf_name = $request->shelf_name;
        $shelf->shop_id = get_session_val('serial_id', 'integer');
        $shelf->add_user = auth()->id();
        $shelf->edit_user = auth()->id();
        $shelf->edit_date = date('Y-m-d H:i:s');
        $shelf->store_id = 0;

        $shelf->save();

        $option = '<option selected value="' . $shelf->id . '">' . $shelf->shelf_name . '</option>';

        $list = '<li data-original-index="' . $shelf->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $shelf->shelf_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json([
            'status' => true,
            'option' => $option,
            'list' => $list,
        ]);
    }

    public function storeCompany(Request $request)
    {
        $rules = [
            'company_name' => ['required', 'unique:companies,company_name,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')],
            'income_ratio' => ['required', 'numeric'],
            'sale_ratio' => ['required', 'numeric'],
        ];

        $messages = [
            'company_name.required' => __('items.company_name.required'),
            'company_name.unique' => __('items.company_name.unique'),
            'income_ratio.required' => __('items.income_ratio.required'),
            'income_ratio.numeric' => __('items.income_ratio.numeric'),
            'sale_ratio.required' => __('items.sale_ratio.required'),
            'sale_ratio.numeric' => __('items.sale_ratio.numeric'),
        ];

        $this->validate($request, $rules, $messages);

        $company = new Company();
        $company->company_name = $request->company_name;
        $company->income_ratio = $request->income_ratio;
        $company->sale_ratio = $request->sale_ratio;
        $company->add_user = auth()->id();
        $company->add_date = date('Y-m-d H:i:s');
        $company->shop_id = get_session_val('serial_id', 'integer');
        $company->save();

        $option = '<option selected value="' . $company->id . '">' . $company->company_name . '</option>';
        $list = '<li data-original-index="' . $company->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $company->company_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json(['status' => true, 'option' => $option, 'list' => $list,]);
    }

    public function storeCardCompany(Request $request)
    {
        $rules = [
            'company_name' => ['required', 'unique:companies,company_name,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')],
            'card_count' => ['required', 'numeric'],
        ];

        $messages = [
            'company_name.required' => __('items.company_name.required'),
            'company_name.unique' => __('items.company_name.unique'),
            'card_count.required' => __('items.card_count.required'),
            'card_count.numeric' => __('items.card_count.numeric'),
        ];

        $this->validate($request, $rules, $messages);

        $company = new CardCompany();
        $company->company_name = $request->company_name;
        $company->number_digital = $request->card_count;
        $company->shop_id = get_session_val('serial_id', 'integer');
        $company->save();

        $option = '<option selected value="' . $company->id . '">' . $company->company_name . '</option>';
        $list = '<li data-original-index="' . $company->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $company->company_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json(['status' => true, 'option' => $option, 'list' => $list]);
    }

    public function storeColor(Request $request)
    {
        $rules = [
            'color_name' => ['required', 'unique:colors,color_name,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')],
            'color_code' => ['required', 'unique:colors,color_code,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')],
        ];

        $messages = [
            'color_name.required' => __('items.color_name.required'),
            'color_name.unique' => __('items.color_name.unique'),
            'color_code.required' => __('items.color_code.required'),
            'color_code.unique' => __('items.color_code.unique'),
        ];

        $this->validate($request, $rules, $messages);

        $color = new Color();
        $color->color_name = $request->color_name;
        $color->color_code = $request->color_code;
        $color->shop_id = get_session_val('serial_id', 'integer');
        $color->save();

        $option = '<option selected value="' . $color->id . '">' . $color->color_name . '</option>';
        $list = '<li data-original-index="' . $color->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $color->color_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json(['status' => true, 'option' => $option, 'list' => $list]);
    }

    public function storeSize(Request $request)
    {
        $rules = [
            'size_name' => ['required', 'unique:sizes,size_name,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')],
            'size_code' => ['required', 'unique:sizes,size_code,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')],
        ];

        $messages = [
            'size_name.required' => __('items.size_name.required'),
            'size_name.unique' => __('items.size_name.unique'),
            'size_code.required' => __('items.size_code.required'),
            'size_code.unique' => __('items.size_code.unique'),
        ];

        $this->validate($request, $rules, $messages);

        $size = new Size();
        $size->size_name = $request->size_name;
        $size->size_code = $request->size_code;
        $size->shop_id = get_session_val('serial_id', 'integer');
        $size->save();

        $option = '<option selected value="' . $size->id . '">' . $size->size_name . '</option>';
        $list = '<li data-original-index="' . $size->id . '">
                    <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                    <span class="text"> ' . $size->size_name . ' </span>
                    <span class="glyphicon glyphicon-ok check-mark"></span>
                    </a>
                    </li>';

        return response()->json(['status' => true, 'option' => $option, 'list' => $list]);
    }

    public function ajaxSubCats(Request $request)
    {
        $change = $request->change;
        $main_id = $request->main_id;
        $cat = Category::where('id', $main_id)->first();

        if ($cat) {
            $subs = Category::where('category_id', $cat->id)->where('shop_id', get_session_val('serial_id', 'integer'))->get();
            $data = '<label class="control-label col-md-3">{{ __("items.sub_category") }}</label>';
            $data .= '<select style="margin-top: -1em;" name="sub_cat" id="sub-cat-items" class="form-control col-md-9">';
            $data .= '<option value=""></option>';

            foreach ($subs as $sub) {
                $data .= '<option value="' . $sub->id . '">' . $sub->name . '</option>';
            }
            $data .= '</select>';

            return $change == 1
                ? response()->json(['status' => false, 'change' => true, 'data' => $data])
                : response()->json(['status' => true, 'change' => false, 'data' => $data]);
        } else {
            return response()->json(['status' => false]);
        }
    }

    public function ajaxInternSubCats(Request $request)
    {
        $change = $request->change;
        $main_id = $request->main_id;
        $cat = Category::where('id', $main_id)->first();

        if ($cat) {
            $subs = Category::where('category_id', $cat->id)->where('shop_id', get_session_val('serial_id', 'integer'))->get();
            $data = '<label class="control-label col-md-3">{{ __("items.sub_category") }}</label>';
            $data .= '<select style="margin-top: -1em;" name="intern_cat" id="intern-sub-cat-items" class="form-control col-md-9">';
            $data .= '<option value=""></option>';

            foreach ($subs as $sub) {
                $data .= '<option value="' . $sub->id . '">' . $sub->name . '</option>';
            }
            $data .= '</select>';

            return $change == 1
                ? response()->json(['status' => false, 'change' => true, 'data' => $data])
                : response()->json(['status' => true, 'change' => false, 'data' => $data]);
        } else {
            return response()->json([
                'status' => false,
            ]);
        }
    }

    public function ajaxSubUnits(Request $request)
    {
        $request->unit_value = eval("return $request->unit_value ;");
        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();

        $rules = [
            'unit_barcode' => ["required", Rule::unique('items_unit', 'unit_barcode')->where(function ($query) use ($request) {
                return $query->where('shop_id', get_session_val('serial_id', 'integer'));
            })->ignore($request->id, 'id')],
            'unit_id'      => ["required", "unique:items_unit,id," . $request->unit_id . ",id,shop_id," . get_session_val('serial_id', 'integer')],
            'unit_price'   => ["required", "numeric"],
            'unit_type'    => ["required"],
            'unit_value'   => ["required"],
        ];

        $messages = [
            'unit_barcode.required' => __('items.unit_barcode_required'),
            'unit_barcode.unique'   => __('items.unit_barcode_unique'),
            'unit_id.required'      => __('items.unit_id_required'),
            'unit_id.unique'        => __('items.unit_id_unique'),
            'unit_price.required'   => __('items.unit_price_required'),
            'unit_price.numeric'    => __('items.unit_price_numeric'),
            'unit_type.required'    => __('items.unit_type_required'),
            'unit_value.required'   => __('items.unit_value_required'),
            'unit_value.numeric'    => __('items.unit_value_numeric'),
        ];

        if ($settings->einvoice == 1) {
            $rules['unit_einvoice_code'] = "required";
        }

        $this->validate($request, $rules, $messages);

        $Screenhidden = $request->has('Screenhidden') ? 0 : 1;
        $unit = Unit::find($request->unit_id);

        if ($unit) {
            $data = '<tr id="' . $request->unit_barcode . '-' . $request->unit_id . '" class="tr-' . $request->unit_id . '">';
            $data .= '<td class="text-center" id="' . $request->unit_barcode . '-' . $request->unit_id . '-name">' . $unit->name . '</td>';

            if ($settings->einvoice == 1) {
                $data .= '<td class="text-center" id="' . $request->unit_barcode . '-' . $request->unit_id . '-code">' . $unit->unit_einvoice_code . '</td>';
            }

            $data .= '<td class="text-center" id="' . $request->unit_barcode . '-' . $request->unit_id . '-unit_value">' . quant_decimal($request->unit_value) . '</td>';
            $data .= '<td class="text-center" id="' . $request->unit_barcode . '-' . $request->unit_id . '-price">' . price_decimal($request->unit_price) . '</td>';
            $data .= '<td class="text-center unit_pay_price" data-qty="' . $request->unit_value . '" id="' . $request->unit_barcode . '-' . $request->unit_id . '-pay_price"></td>';
            $data .= '<td class="text-center"
            id="' . $request->unit_barcode . '-'
                . $request->unit_id . '-barcode">'
                . $request->unit_barcode .'<div class="card card-body hidden">
                <ul class="list-inline unit-barcode-list p-0 m-0" ></ul>
                 <div  id="list_bar-'.$request->unit_id.'"></div>


        </div></td>';
            $data .= '<td class="text-center">
             <button type="button" id="new-sub-barcode-btn"
             data-sub_unit_id="' . $request->unit_id . '"
             data-sub_unit_value="' . $request->unit_value . '"
             data-sub_unit_price="' . $request->unit_price . '"
             class="btn btn-info col-md-9" data-toggle="modal"
             data-target="#new-sub-barcode-modal">
             <i class="fa fa-plus-circle"></i>   اضافه الباركود
         </button>
            </td>';
            $data .= '<td class="text-center">';
            if ($request->add_unit != true || $request->edit_unit == true) {
                $data .= '<a href="javascript:void(0)" class="btn btn-success btn-xs"
                                style="padding: 5px 16px 3px 15px; margin-left: 5px;"
                                data-tr="' . $request->unit_barcode . '-' . $request->unit_id . '"
                                data-target="#updateSubUnitElementModal" data-href="' . $request->unit_id . '"
                                data-toggle="modal">
                                <i class="fa fa-edit"></i>
                            </a>';
            }

            $data .= '<a href="javascript:void(0)" class="btn btn-danger btn-xs delete-sub-unit"
                            data-barcode="' . $request->unit_barcode . '" data-unit="' . $request->unit_id . '">
                            <i class="fa fa-trash"></i>
                      </a>
                    </td>
                </tr>';

            $data_hidden_value   = $request->unit_barcode . '-';
            $n_data_hidden_value = $request->unit_barcode . '-';

            if ($settings->einvoice == 1) {
                $data_hidden_value   .= $request->unit_einvoice_code . '-';
                $n_data_hidden_value .= $request->unit_einvoice_code . '-';
            }

            $data_hidden_value   .= $request->unit_id . '-' . $request->unit_price . '-' . $request->unit_type . '-' . $request->unit_value . '-' . $Screenhidden;
            $n_data_hidden_value .= $request->unit_id . '-' . $request->unit_price . '-' . $request->unit_type . '-' . $request->unit_value . '-' . $Screenhidden;

            $data_hidden = '<input type="hidden" id="input-' . $request->unit_barcode . '-' . $request->unit_id . '" name="sub_units[]" class="tr-' . $request->unit_id . '"
             value="' . $data_hidden_value . '">';

            $n_data_hidden = '<input type="hidden" id="input-' . $request->unit_barcode . '-' . $request->unit_id . '" name="new_sub_units[]" class="tr-' . $request->unit_id . '"
             value="' . $n_data_hidden_value . '">';

            return response()->json([
                'status'   => true,
                'data'     => $data,
                'hidden'   => $data_hidden,
                'n_hidden' => $n_data_hidden,
            ]);
        } else {
            return response()->json(['status' => false, 'data' => __('items.unit.not_found')]);
        }
    }

    public function manufacturingcats(Item $item, Request $request)
    {
        if ($request->has('cats_ids')) {
            $cats = explode(',', $request->cats_ids);

            foreach ($cats as $cat) {
                $manuf_cat = Manufacturingcats::where([
                    'category_id' => $cat,
                    'item_id' => $item->id
                ])->count();

                if ($manuf_cat == 0) {
                    Manufacturingcats::create([
                        'item_id' => $item->id,
                        'category_id' => (int)$cat,
                        'shop_id' => get_session_val('serial_id', 'integer')
                    ]);
                }
            }
        }

        $cats = Manufacturingcats::where('item_id', $item->id)->orderBy('id')->get();
        $categories = ItemType::orderBy('name')->get();

        return view('item::manufacturing', compact('item', 'categories', 'cats'));
    }

    public function deleteManufacturingCats(Request $request)
    {
        $cat = Manufacturingcats::find($request->cat_id);
        if ($cat) {
            $cat->delete();
            return response()->json(['msg' => __('items.category.delete_success')]);
        } else {
            return response()->json(['msg' => __('items.category.not_found')]);
        }
    }

    public function postItemImages(Request $request)
    {
        $this->validate($request, [
            'image_link' => 'mimes:jpg,png,jpeg'
        ]);

        if ($request->has('image_link')) {
            $file = $request->file('image_link');
            $ext = strtolower($file->getClientOriginalExtension());
            $file_name = date("Y-m-d") . '_' . time() . '.' . $ext;
            $destinationPath = '../upload';
            $file->move($destinationPath, $file_name);
        }

        $state = $request->has('image_state') ? 1 : 0;
        $image = new ItemImage();
        $image->image_link = isset($file_name) ? '../upload/' . $file_name : '';
        $image->image_state = $state;
        $image->image_text = $request->image_text;
        $image->shop_id = get_session_val('serial_id', 'integer');
        $image->item_id = $request->item_id;
        $image->save();

        return back()->with(['success' => __('extends.add_sucessfully')]);
    }

    public function updateItemImage(Request $request)
    {
        $item = Item::find($request->item_id);
        if ($item) {
            $image = ItemImage::where('item_id', $item->id)
                ->where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $request->image_id)
                ->first();

            if ($image) {
                $image->image_state = $request->has('image_state') ? 1 : 0;
                $image->image_text = $request->image_text;
                $file = $request->file('image_link');
                $ext = strtolower($file->getClientOriginalExtension());
                $file_name = date("Y-m-d") . '_' . time() . '.' . $ext;
                $destinationPath = '../upload';

                if (!file_exists($destinationPath)) {
                    mkdir($destinationPath, 666, true);
                }

                $file->move($destinationPath, $file_name);

                $old_img = $image->image_link;
                $image->image_link = isset($file_name) ? $destinationPath . '/' . $file_name : '';

                unlink($old_img);
                $image->update();

                return back()->with(['success' => __('extends.update_sucessfully')]);
            } else {
                return back()->with(['error' => __('extends.image_not_found')]);
            }
        } else {
            return back()->with(['error' => __('extends.item_not_found')]);
        }
    }

    public function deleteItemImage(Request $request)
    {
        $image = ItemImage::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('id', $request->image_id)
            ->first();

        if ($image) {
            $image->delete();

            if (file_exists($image->image_link)) {
                unlink($image->image_link);
            }

            return back()->with([
                'success' => __('extends.delete_sucessfully')
            ]);
        } else {
            return back()->with([
                'error' => __('extends.image_not_found')
            ]);
        }
    }

    public function excel()
    {
        $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        return view('item::add_excel', ['stores' => $stores]);
    }

    public function uploadPrices(Request $request)
    {
        $this->validate($request, [
            'sheet' => 'required|mimes:xls,xlsx,csv',
        ]);

        ini_set('max_execution_time', 0);
        ini_set('max_input_time', 0);

        if ($request->file('sheet')->isValid()) {
            $file = $request->file('sheet');
            $shop_id = get_session_val('serial_id', 'integer');
            $name_one = 'upload/' . $shop_id . '/sheet/';
            $extension = $file->getClientOriginalExtension(); // getting file extension
            $fileName = 'prices-' . random_int(11111111, 99999999) . '.' . $extension; // renameing file
            $file->move($name_one, $fileName); // uploading file to given path
            $data = $this->importPrices($name_one . $fileName, $shop_id);
            return back()->with(['success' => 'نجحت العملية', 'data' => $data]);
        } else {
            return redirect()->route('excel.prices-index')->with(['error' => 'لقد حدث خطأ ما. يرجي المحاولة مرة أخري.']);
        }
    }

    public function importPrices($file, $shop_id)
    {
        $result = [];
        $text = '<ul>';
        Excel::selectSheetsByIndex(0)->load($file, function ($sheet) use ($shop_id, &$result, &$text) {
            $sheet->noHeading();
            // $size = sizeof($sheet->toArray());
            // dd(array_filter($sheet->toArray()));
            $prices_list = [];
            if (sizeof($sheet->toArray()) > 1) {
                foreach ($sheet->toArray() as $k => $row) {
                    // if (array_key_exists('0', $row)) {
                    // array_pull($row, '0');
                    // }
                    $array_size = sizeof($row);
                    $peices = [];

                    if ($k == 0) {

                        $row = array_filter($row);
                        // dd($row);
                        $prices_list = $row;
                        // dd($prices_list, 5454545);
                        $count_of_prices_list = sizeof($prices_list);
                        continue;
                    }
                    #merge prices with list as key and values
                    $prices_array = [];
                    for ($x = 0; $x <= $count_of_prices_list; $x++) {
                        foreach ($prices_list as $j => $list) {
                            // dd($j);
                            $prices_array['normal'] = $row[1];
                            $prices_array[$list] = $row[$j];
                        }
                    }
                    // dd($prices_array);
                    $item_name = array_values($row)[0];

                    $list_name = array_values($row)[1];

                    $item_name = trim(rtrim($item_name));
                    $list_name = trim(rtrim($list_name));

                    $price = array_values($row)[2];
                    $list_quant = array_values($row)[3];
                    if ($list_quant == '' || $list_quant == null) {
                        $list_quant = 1;
                    }

                    $item_text = '';
                    if (!is_null($item_name)) {

                        $item_check = Item::where([
                            'shop_id' => $shop_id,
                            'item_name' => $item_name,
                        ])->selectRaw('id, item_name, sale_price')->first();
                        if ($item_check) {

                            if (!is_null($list_name)) {
                                $list_check = PriceList::where([
                                    'list_name' => $list_name,
                                    'shop_id' => $shop_id,
                                ])->first();
                                // dd($list_check);
                                if ($list_check) {
                                    if ($price == '' || $price == null) {
                                        $price = 0;
                                    }
                                    // if ($price != '' && $price != null && is_numeric($price) && $price >= 0) {
                                    if (is_numeric($price) && $price > 0) {
                                        $item_price_check = ItemPrice::where([
                                            'item_id' => $item_check->id,
                                            'list_id' => $list_check->id,
                                            // 'list_quant' => $list_quant,
                                            'shop_id' => $shop_id,
                                        ])->first();
                                        // dd($item_price_check);
                                        if ($item_price_check) {
                                            $item_price_check->price = $price;
                                            $item_price_check->list_quant = $list_quant;
                                            $item_price_check->save();
                                            if ($item_price_check->save()) {
                                                $item_text .= '<p class="text-success"> تم حفظ ' . $list_name . ': ' . $price . '</p>';
                                            } else {
                                                $item_text .= '<li><p class="text-danger"> لم يتم تحديث  ' . $list_name . ': ' . $price . '</p></li> ';
                                            }
                                        } else {
                                            //add new price
                                            $new_item_price = new ItemPrice();
                                            $new_item_price->item_id = $item_check->id;
                                            $new_item_price->list_id = $list_check->id;
                                            $new_item_price->price = $price;
                                            $new_item_price->shop_id = $shop_id;
                                            $new_item_price->list_quant = $list_quant;
                                            $new_item_price->add_user = \auth()->id();
                                            $new_item_price->add_date = Carbon::now();
                                            $new_item_price->edit_user = \auth()->id();
                                            $new_item_price->edit_date = Carbon::now();

                                            if ($new_item_price->save()) {
                                                $item_text .= '<p class="text-success"> تم حفظ ' . $list_name . ': ' . $price . '</p>';
                                            } else {
                                                $item_text .= '<p class="text-danger"> لم يتم تحديث  ' . $list_name . ': ' . $price . '</p> ';
                                            }
                                        }
                                    } else {
                                        $item_text .= '<p class="text-danger"> لم يتم تحديث  ' . $list_name . ': ' . $price . '</p> ';
                                    }
                                } else {
                                    $item_text .= '<p class="text-danger"> فشل الحفظ ' . $list_name . ': ' . $price . ' يرجي تعريف قائمة الاسعار اولا</p></li> ';
                                }
                            } else {
                                $text .= '<p class="text-danger"> فشل الحفظ   ' . $item_name . ' برجاء اكمال بيانات الصنف </p> ';
                            }


                            // array_splice($prices_array,1,1);
                            // array_splice($prices_array,1,1);
                            /* foreach($prices_array as $list_name => $price){
                                if ($list_name == 'normal'){
                                    $item_check->sale_price = $price;
                                    $item_check->save();
                                    continue;
                                }
                                $list_name = trim(rtrim($list_name));
                                // dd($list_name);

                            }*/
                            $text .= '<h3>' . ($item_name ?? ' صنف غير معرف ') . '</h3>' . $item_text . '<hr>';
                        } else {
                            $text .= '<p class="text-danger"> فشل الحفظ ' . $item_name . ' يرجي تعريف الصنف اولا</p> ';
                        }
                    }
                }
            } else {
                $text .= '<h1 class="text-danger">الملف فارغ ....</h1> ';
            }
        });
        $text .= '</ul>';
        // dd($text);
        if ($text == "<ul></ul>") {
            $text = '<h1 class="text-danger">الملف فارغ ....</h1> ';
        }
        return $text;
    }


    public function importItemExcel(Request $request)
    {

        //    dd($request->all());
        Excel::import(new ItemImport, $request->sheet);


        return redirect()->back();
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}
