<?php

namespace Modules\Item\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Controller;
use App\Models\systemLog;
use App\Services\Uploaders\ItemImgUploader;
use Carbon\Carbon;
use Illuminate\Support\Facades\File;
use Intervention\Image\Facades\Image;
use Modules\EInvoice\Entities\Settings;
use Illuminate\Validation\Rule;
use Modules\Invoices\Entities\Invoices;
use Modules\Invoices\Entities\InvoiceDetails;
use Exception;
use Modules\Item\Entities\{
    Vat,
    Item,
    Size,
    Unit,
    BadrShop,
    Color,
    Shelf,
    Company,
    ItemVat,
    Category,
    ItemSize,
    ItemUnit,
    ItemColor,
    ItemImage,
    ItemPrice,
    PriceList,
    CardCompany,
    Client,
    SupplierItem,
    Stores,
    StoreItem,
};


class ItemEditController extends Controller
{
    public function edit($id, Request $request)
    {
        if (!authorized('can_edit_item')) {
            return unAuthorized(trans('users.edit_items'));
        }

        $store_id     = $request->store_id ?? 0;
        $sub          = null;
        $main         = null;
        $m_subs       = null;
        $interns      = null;
        $item_prices  = null;
        $item         = Item::where('shop_id', get_session_val('serial_id', 'integer'))->with('storeQty.store')->find($id);

        if ($item) {
            $suppliers     = Client::whereShopId(get_session_val('serial_id', 'integer'))->whereIn('client_role', [1, 2])->get();
            $price_decimal = get_session_val('decimal_num_price', 'integer');
            $type          = get_session_val('date_type', 'integer');
            $main_cats     = Category::where('shop_id', get_session_val('serial_id', 'integer'))
                                        ->when(authorized('define_categories'), function($query) {
                                            $query->whereIn('id', getUserCategoriesId());
                                        })->get();
            $units         = Unit::where('shop_id', get_session_val('serial_id', 'integer'))->get();
            $shelves       = Shelf::where('shop_id', get_session_val('serial_id', 'integer'))->get();
            $companies     = Company::where('shop_id', get_session_val('serial_id', 'integer'))->get();
            $colors        = Color::where('shop_id', get_session_val('serial_id', 'integer'))->get();
            $sizes         = Size::where('shop_id', get_session_val('serial_id', 'integer'))->get();
            $pricesList    = PriceList::where('shop_id', get_session_val('serial_id', 'integer'))->get();
            $cardCompany   = CardCompany::where('shop_id', get_session_val('serial_id', 'integer'))->get();
            $settings      = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
            $cat           = Category::find($item->sale_unit);
            $stores        = Stores::with(['items' => function ($query) use ($id) {
                $query->where('item_id', $id);
            }])->where('shop_id', get_session_val('serial_id', 'integer'))->get();

            if ($cat) {
                #if category_id = 0 => main cat
                # show its subs
                if ($cat->category_id == 0) {
                    $main_cat_check = 1;
                    $subs = $cat->subCategory();

                    # if category_id != 0 => sub or intern
                    #if no subs => intern
                    #if subs => sub and we need to get subs
                } else {

                    $check = Category::find($cat->category_id);

                    #sub_cat
                    if ($check->category_id == 0) {
                        $main_cat_check = 0;
                        $main = $check;
                        $m_subs = $check->subCategory();
                    } else { # intern
                        $main = Category::find($check->category_id);
                        $sub = $check;
                        $m_subs = $main->subCategory();
                        ## m_subs ??????
                        $interns = $check->subCategory();
                        $main_cat_check = 3;
                    }
                }
            } else {
                $main_cat_check = 2;
            }

            $item->quantity     = round_quantity($item->quantity);
            $item->sale_price   = round_money($item->sale_price);
            $item->pay_price    = round_money($item->pay_price);
            $item->lowest_price = round_money($item->lowest_price);

            if ($settings->eivoice == 1) {
                $item->unit_einvoice_code = Unit::find($item->unit_id)->unit_einvoice_code;
            }

            $su_items        = SupplierItem::where('item_id', $item->id)->pluck('supplier_id');
            $sizes_          = ItemSize::where('item_id', $item->id)->pluck('size_id');
            $colors_         = ItemColor::where('item_id', $item->id)->pluck('color_id');
            $size_array      = json_decode(json_encode($sizes_));
            $color_array     = json_decode(json_encode($colors_));
            $suppliers_array = json_decode(json_encode($su_items));

            foreach ($pricesList as $list) {
                $item_price = ItemPrice::where('item_id', $item->id)
                    ->where('list_id', $list->id)
                    ->first();
                if ($item_price) {
                    $list->price = round_money($item_price->price);
                } else {
                    $list->price = 0;
                }
            }

            $sub_units_bars = ItemUnit::where('unit_type', 2)->where('item_id', $item->id)->get();
            $all_sub_units  = ItemUnit::whereIn('unit_type', [0, 1])->where('item_id', $item->id)->get();

            foreach ($all_sub_units as $all_sub_unit) {
                $unit__             = Unit::find($all_sub_unit->unit_id);
                $all_sub_unit->name = ($unit__) ? $unit__->name : '---';
            }

            $compact = [];

            $vats = $settings->einvoice == 1
                ? Vat::where('shop_id', get_session_val('serial_id', 'integer'))
                ->whereNotNull('einvoice_code')
                ->get()
                : Vat::where('shop_id', get_session_val('serial_id', 'integer'))
                ->whereNull('einvoice_code')
                ->whereNull('sub_einvoice_code')
                ->get();

            $sales_vat = ItemVat::where('item_id', $item->id)
                ->where(function ($query) {
                    $query->where('vat_use', 0)
                        ->orWhere('vat_use', 1);
                })
                ->where('shop_id', get_session_val('serial_id', 'integer'))
                ->pluck('vat_id')
                ->toArray() ?? [];

            $purchase_vat = ItemVat::where('item_id', $item->id)
                ->where(function ($query) {
                    $query->where('vat_use', 0)
                        ->orWhere('vat_use', 2);
                })
                ->where('shop_id', get_session_val('serial_id', 'integer'))
                ->pluck('vat_id')
                ->toArray() ?? [];

            array_push($compact, 'sales_vat', 'purchase_vat');

            // if ($settings->einvoice == 1) {
            //     $vats  = Vat::where('shop_id', get_session_val('serial_id', 'integer'))
            //         ->whereNotNull('einvoice_code')
            //         ->whereNotNull('sub_einvoice_code')
            //         ->get();
            //     $vats_ = ItemVat::where('item_id', $item->id)
            //         ->where('shop_id', get_session_val('serial_id', 'integer'))
            //         ->pluck('vat_id');

            //     $vat_array = json_decode(json_encode($vats_));

            //     array_push($compact, 'vat_array');
            // } else {
            //     $vats = Vat::where('shop_id', get_session_val('serial_id', 'integer'))
            //         ->whereNull('einvoice_code')
            //         ->whereNull('sub_einvoice_code')
            //         ->get();

            //     $sales_vat = ItemVat::where('item_id', $item->id)
            //         ->where('vat_use', 1)
            //         ->where('shop_id', get_session_val('serial_id', 'integer'))
            //         ->pluck('vat_id')
            //         ->toArray() ?? [];

            //     $purchase_vat = ItemVat::where('item_id', $item->id)
            //         ->where('vat_use', 2)
            //         ->where('shop_id', get_session_val('serial_id', 'integer'))
            //         ->pluck('vat_id')
            //         ->toArray() ?? [];

            //     array_push($compact, 'sales_vat', 'purchase_vat');
            // }

            $item_prices  = ItemPrice::where([
                ['item_id', $id],
                ['shop_id', get_session_val('serial_id', 'integer')]
            ])->select(['id', 'list_id', 'price', 'list_quant'])->get();

            $price_lists = PriceList::where('shop_id', get_session_val('serial_id', 'integer'))->get();

            $einvoice_settings = Settings::shop()->first();

            array_push(
                $compact,
                'sub',
                'interns',
                'all_sub_units',
                'sub_units_bars',
                'suppliers_array',
                'item_prices',
                'size_array',
                'color_array',
                'm_subs',
                'main',
                'cat',
                'main_cat_check',
                'item',
                'cardCompany',
                'pricesList',
                'sizes',
                'colors',
                'companies',
                'shelves',
                'units',
                'main_cats',
                'suppliers',
                'type',
                'price_decimal',
                'vats',
                'settings',
                'einvoice_settings',
                'stores',
                'price_lists',
                'store_id'
            );

            return view('item::edit', compact($compact));
        } else {
            abort(404);
        }
    }

    public function update(Request $request)
    {
        if (!authorized('can_edit_item')) {
            return unAuthorized(trans('users.edit_items'));
        }
        $prices = $request->except(
            'item_name',
            'supplier_id',
            'quantity',
            'sale_unit',
            'unit_id',
            'pay_price',
            'sale_price',
            'image',
            'sub_cat',
            'barcode',
            'expire_date',
            'lowest_price',
            'min_quantity',
            'pay_date',
            'available',
            'game',
            'about',
            'details',
            'images',
            'is_service',
            'online',
            'company_id',
            'shelf',
            'background_color',
            'cooking_add',
            'order_show',
            'sizes',
            'colors',
            'bolla',
            'meter_ceramic',
            'require_serial',
            'card_company_id',
            'sub_units',
            '_token',
            'item_id',
            $request->item_id,
            'n_barcode',
            'new_sub_units',
            'intern_cat',
            "small_qty",
            "medium_qty",
            "large_qty",
            "manufacturing",
            "unit_einvoice_code",
            "parent_code",
            "vats",
            "vats_ids",
            "item_einvoice_code",
            "vat_state",
            "vat_ids",
            "purchase_vat_state",
            "purchase_vat_ids",
            "stores",
            "einvoice_vat_state",
            'online',
            'is_special'
        );

        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $rules    = [
            'item_name'     => ['required', 'string', 'max:255', 'unique:items,item_name,' . $request->item_id . ',id,shop_id,' . get_session_val('serial_id', 'integer')],
            'barcode'       => ['required', 'unique:items,barcode,' . $request->item_id . ',id,shop_id,' . get_session_val('serial_id', 'integer')],
            'n_barcode'     => ['sometimes', 'unique:items_unit,unit_barcode,' . $request->item_id . ',item_id,shop_id,' . get_session_val('serial_id', 'integer')],
            'sub_unit_barcode' => [
                'sometimes',
                'unique:items_unit,unit_barcode,NULL,id,shop_id,' . get_session_val('serial_id', 'integer'),
                'unique:items,barcode,NULL,id,shop_id,' . get_session_val('serial_id', 'integer')
            ],
            'supplier_id'   => ['required'],
            'quantity'      => ['required'],
            'sale_unit'     => ['required'],
            'unit_id'       => ['required'],
            'pay_price'     => ['required'],
            'sale_price'    => ['required'],
            'image'         => ['mimes:jpeg,bmp,png,jpg'],
            'manufacturing' => ['nullable'],
            'images' => ['nullable', 'mimes:png,jpg,jpeg', 'max:2000'],
            'stores'        => ['required_if:quantity,>0', Rule::unique('store_items', 'store_id')->where(function ($query) use ($request) {
                return $query->where('shop_id', get_session_val('serial_id', 'integer'))->where('item_id', $request->item_id);
            })->ignore($request->item_id, 'item_id')],
        ];

        $messages = [
            'item_name.required'        => __('items.item_name.required'),
            'item_name.string'          => __('items.item_name.string'),
            'item_name.max'             => __('items.item_name.max'),
            'item_name.unique'          => __('items.item_name.unique'),
            'barcode.required'          => __('items.barcode.required'),
            'barcode.unique'            => __('items.barcode.unique'),
            'n_barcode.unique'          => __('items.n_barcode.unique'),
            'sub_unit_barcode.unique' => __('items.sub_unit_barcode.unique'),
            'supplier_id.required'      => __('items.supplier_id.required'),
            'quantity.required'         => __('items.quantity.required'),
            'sale_unit.required'        => __('items.sale_unit.required'),
            'unit_id.required'          => __('items.unit_id.required'),
            'pay_price.required'        => __('items.pay_price.required'),
            'sale_price.required'       => __('items.sale_price.required'),
            'image.mimes'               => __('items.image.mimes'),
            'manufacturing.nullable'    => __('items.manufacturing.nullable'),
            'item_einvoice_code.unique' => __('items.item_einvoice_code.unique'),
            'stores.unique'             => __('items.stores.unique'),
        ];

        if ($settings->einvoice == 1) {
            // $rules['item_einvoice_code'] = ['required', 'unique:items,item_einvoice_code,' . $request->item_id . ',id'];
            $rules['unit_einvoice_code'] = ['required'];
            $rules['parent_code']        = ['required'];
            // $rules['vats']               = ['nullable', 'array'];

            // $messages['item_einvoice_code.required'] = __('items.item_einvoice_code_required');
            // $messages['item_einvoice_code.unique']   = __('items.item_einvoice_code_unique');
            $messages['unit_einvoice_code.required'] = __('items.unit_einvoice_code_required');
            $messages['parent_code.required']        = __('items.parent_code_required');
        }

        $this->validate($request, $rules, $messages);

        $item     = Item::find($request->item_id);
        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();

        if ($item) {
            DB::transaction(function () use ($request, $prices, $item, $settings) {
                $cat_id = is_null($request->sub_cat)
                    ? $request->sale_unit
                    : (is_null($request->intern_cat) ? $request->sub_cat : $request->intern_cat);

                $einvoice_settings        = Settings::shop()->first();
                $item->item_name          = $request->item_name;
                $item->barcode            = ($request->barcode) ? $request->barcode[0] : '';
                $item->sale_unit          = $cat_id;
                $item->sale_price         = $request->sale_price;
                $item->lowest_price       = $request->lowest_price;
                $item->pay_price          = $request->pay_price;
                // $item->quantity           = $request->quantity;
                $item->unit_id            = $request->unit_id;
                $item->expire_date        = ($request->expire_date) ? $request->expire_date : date('Y-m-d', strtotime('+1 years'));;
                $item->min_quantity       = $request->min_quantity;
                $item->incom_date         = $request->pay_date ?? now();
                $item->available          = ($request->available) ? 1 : 0;
                $item->online          = ($request->online) ? 1 : 0;
                $item->is_special          = ($request->is_special) ? 1 : 0;
                $item->game               = ($request->game) ? 1 : 0;
                $item->about              = $request->about ?? ' ';
                $item->supplier_id        = $request->supplier_id[0];
                $item->details            = ($request->details) ? $request->details : '';
                $item->edit_user          = auth()->id();
                $item->used_for_small     = $request->small_qty ?? 1;
                $item->used_for_medium    = $request->medium_qty ?? 1;
                $item->used_for_large     = $request->large_qty ?? 1;
                $item->manufacturing      = $request->manufacturing;
                $item->parent_code        = $request->parent_code;
                $item->item_einvoice_code = ($settings->einvoice == 1 && !is_null($request->item_einvoice_code))
                    ? 'EG-' . $einvoice_settings->registration_id . '-' . $request->item_einvoice_code
                    : null;
                $item->vat_state          = $request->vat_state ?? 0;
                $item->vat_id             = 0;
                $item->purchase_vat_state = $request->purchase_vat_state ?? 0;
                $item->purchase_vat_id    = 0;

                if ($request->file('images')) {
                    $uploader     =  new ItemImgUploader($request, 'images');
                    $result  =  $uploader->checkAndUpload();

                    if ($result['status']) {
                        $uploader->clean($item->img);
                        $item->img = $result['path'];
                    }
                }

                $item->is_service = ($request->is_service) ? 1 : 0;
                $item->online     = ($request->online) ? 1 : 0;
                $item->company_id = $request->company_id;
                $item->shelf      = $request->shelf;
                $item->edit_date  = date('Y-m-d H:i:s');

                if ($request->has('background_color')) {
                    $item->background_color = $request->background_color;
                }
                if ($request->has('cooking_add')) {
                    $item->cooking_add = $request->cooking_add;
                }
                if ($request->has('order_show')) {
                    $item->order_show = $request->order_show;
                }
                if ($request->has('meter_ceramic')) {
                    $item->meter_ceramic = $request->meter_ceramic;
                }
                if ($request->has('require_serial')) {
                    $item->require_serial = $request->require_serial;
                }
                if ($request->has('card_company_id')) {
                    $item->card_company_id = $request->card_company_id;
                }
                if ($request->has('bolla')) {
                    $item->bolla = $request->bolla;
                }
                if ($request->has('order_max_quantity')) {
                    $item->order_max_quantity = $request->order_max_quantity;
                }

                $item->update();

                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                // Sales Vat
                if ($request->vat_state != 0 && $request->has('vat_ids')) {
                    $oldVats    = ItemVat::where('item_id', $item->id)->where('vat_use', 1)->pluck('vat_id')->toArray();
                    $newVats    = $request->vat_ids;
                    $deleteVats = array_diff($oldVats, $newVats);
                    $addVats    = array_diff($newVats, $oldVats);

                    ItemVat::where('item_id', $item->id)->where('vat_use', 1)->whereIn('vat_id', $deleteVats)->delete();

                    foreach ($addVats as $vat_id) {
                        $sales_vat = explode('-', $vat_id);
                        ItemVat::updateOrCreate(
                            [
                                'item_id'  => $item->id,
                                'vat_id'   => $sales_vat[0],
                                'vat_use'  => 1,
                                'shop_id'  => get_session_val('serial_id', 'integer'),
                                'vat_type' => $sales_vat[1],
                            ],
                            [
                                'item_id' => $item->id,
                                'vat_use' => 1,
                            ]
                        );
                    }
                } else {
                    ItemVat::where('item_id', $item->id)->where('vat_use', 1)->delete();
                }

                // Purchase Vat
                if ($request->purchase_vat_state != 0 && $request->has('purchase_vat_ids')) {
                    $oldVats    = ItemVat::where('item_id', $item->id)->where('vat_use', 2)->pluck('vat_id')->toArray();
                    $newVats    = $request->purchase_vat_ids;
                    $deleteVats = array_diff($oldVats, $newVats);
                    $addVats    = array_diff($newVats, $oldVats);

                    ItemVat::where('item_id', $item->id)->where('vat_use', 2)->whereIn('vat_id', $deleteVats)->delete();

                    foreach ($addVats as $purchase_vat_id) {
                        $purchase_vat = explode('-', $purchase_vat_id);
                        ItemVat::updateOrCreate(
                            [
                                'item_id'  => $item->id,
                                'vat_id'   => $purchase_vat[0],
                                'vat_use'  => 2,
                                'shop_id'  => get_session_val('serial_id', 'integer'),
                                'vat_type' => $purchase_vat[1],
                            ],
                            [
                                'item_id' => $item->id,
                                'vat_use' => 2,
                            ]
                        );
                    }
                } else {
                    ItemVat::where('item_id', $item->id)->where('vat_use', 2)->delete();
                }

                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                // store
                if ($request->has('stores') && !is_null($request->stores)) {

                    $store_item = StoreItem::where([
                        ['item_id', $item->id],
                        ['shop_id', get_session_val('serial_id', 'integer')],
                        ['store_id', $request->stores]
                    ])->first();

                    if ($store_item) {
                        $old_qty = $store_item->store_quant;
                        $new_qty = $request->quantity;
                        $qty_diff = $new_qty - $old_qty;
                        if ($old_qty != $new_qty) {
                            // 27 transaction type
                            $invoice = Invoices::create([
                                'invoice_date' => date('Y-m-d h:i:s'),
                                'transaction_date' => date('Y-m-d h:i:s'),
                                'add_user' => auth()->user()->id,
                                'client_id' => 0,
                                'store_id' => $request->stores,
                                'notes' => '',
                                'shop_id' => get_session_val('serial_id', 'Integer'),
                                'invoice_type' => 9,
                                'invoice_total_disc' => 0,
                                'invoice_total_disc_type' => 1,
                                'invoice_total_disc_status' => 1
                            ]);

                            InvoiceDetails::create([
                                'invoice_id' => $invoice->id,
                                'shop_id' => get_session_val('serial_id', 'Integer'),
                                'item_id' => $item->id,
                                'quantity' => $qty_diff,
                                'quantity_after' => $new_qty,
                                'unit_id' => $item->unit_id,
                                'store_id' => $request->stores,
                            ]);
                        }
                    }

                    StoreItem::updateOrCreate(
                        [
                            'item_id' => $item->id,
                            'shop_id' => get_session_val('serial_id', 'integer'),
                            'store_id'    => $request->stores
                        ],
                        [
                            'store_id'    => $request->stores,
                            'item_id'     => $item->id,
                            'store_quant' => $request->quantity,
                            'shop_id'     => get_session_val('serial_id', 'integer'),
                            'add_user'    => auth()->id(),
                        ]
                    );
                }


                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                if ($request->has('sizes')) {

                    $pre_sizes     = ItemColor::where('item_id', $item->id)->pluck('size_id');
                    $pre_sizes     = json_decode(json_encode($pre_sizes)); //[1,2,3]
                    $new_sizes     = $request->sizes; //[2,5]
                    $deleted_sizes = array_diff($pre_sizes, $new_sizes); //[1,3]
                    $added_sizes   = array_diff($new_sizes, $pre_sizes); //[5]

                    ItemColor::where('item_id', $item->id)->whereIn('size_id', $deleted_sizes)->delete();

                    foreach ($added_sizes as $k => $v) {
                        $itemSize            = new ItemSize();
                        $itemSize->item_id   = $item->id;
                        $itemSize->size_id   = $v;
                        $itemSize->shop_id   = get_session_val('serial_id', 'integer');
                        $itemSize->add_user  = auth()->id();
                        $itemSize->edit_user = auth()->id();
                        $itemSize->add_date  = date('Y-m-d H:i:s');
                        $itemSize->save();
                    }
                } else {
                    ItemSize::where('item_id', $item->id)->delete();
                }

                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                if ($request->has('colors')) {
                    $pre_colors     = ItemColor::where('item_id', $item->id)->pluck('color_id');
                    $pre_colors     = json_decode(json_encode($pre_colors));
                    $new_colors     = $request->colors;
                    $deleted_colors = array_diff($pre_colors, $new_colors);
                    $added_colors   = array_diff($new_colors, $pre_colors);

                    ItemColor::where('item_id', $item->id)->whereIn('color_id', $deleted_colors)->delete();

                    foreach ($added_colors as $k => $v) {
                        $itemColor            = new ItemColor();
                        $itemColor->item_id   = $item->id;
                        $itemColor->color_id  = $v;
                        $itemColor->shop_id   = get_session_val('serial_id', 'integer');
                        $itemColor->edit_user = auth()->id();
                        $itemColor->add_user  = auth()->id();
                        $itemColor->add_date  = date('Y-m-d H:i:s');
                        $itemColor->edit_date = date('Y-m-d H:i:s');
                        $itemColor->save();
                    }
                } else {
                    ItemColor::where('item_id', $item->id)->delete();
                }

                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                $pre_suppliers     = SupplierItem::where('item_id', $item->id)->pluck('supplier_id');
                $pre_suppliers     = json_decode(json_encode($pre_suppliers));
                $new_suppliers     = $request->supplier_id;
                $deleted_suppliers = array_diff($pre_suppliers, $new_suppliers);
                $added_suppliers   = array_diff($new_suppliers, $pre_suppliers);

                SupplierItem::where('item_id', $item->id)->whereIn('supplier_id', $deleted_suppliers)->delete();

                foreach ($added_suppliers as $value) {
                    $s_i              = new SupplierItem();
                    $s_i->item_id     = $item->id;
                    $s_i->supplier_id = $value;
                    $s_i->shop_id     = get_session_val('serial_id', 'integer');
                    $s_i->save();
                }

                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                // if (sizeof($prices) > 0) {
                //     foreach ($prices as $key => $value) {
                //         if ($value > 0) {
                //             $x          = explode('-', $key);
                //             $price_id   = $x[1];
                //             $item_price = ItemPrice::where('item_id', $item->id)->where('list_id', $price_id)->first();
                //             if ($item_price) {
                //                 $item_price->price = $value;
                //                 $item_price->save();
                //             } else {
                //                 $item_price            = new ItemPrice();
                //                 $item_price->item_id   = $item->id;
                //                 $item_price->list_id   = $price_id;
                //                 $item_price->price     = $value;
                //                 $item_price->shop_id   = get_session_val('serial_id', 'integer');
                //                 $item_price->add_user  = auth()->id();
                //                 $item_price->add_date  = date('Y-m-d H:i:s');
                //                 $item_price->edit_user = auth()->id();
                //                 $item_price->edit_date = date('Y-m-d H:i:s');
                //                 $item_price->save();
                //             }
                //         }
                //     }
                // }

                if ($request->item_prices) {
                    foreach ($request->item_prices as $key => $value) {

                        $list_id = $request->item_prices_ids[$key];
                        $current_list_item_price = ItemPrice::where([
                            ['list_id', $list_id],
                            ['item_id', $item->id],
                            ['shop_id', get_session_val('serial_id', 'integer')]
                        ])->first();

                        if ($current_list_item_price) {
                            if ($value > 0 || $request->item_prices_qtys[$key] > 0) {
                                $current_list_item_price->price = $value;
                                $current_list_item_price->list_quant    = $request->item_prices_qtys[$key];
                                $current_list_item_price->edit_user     =  auth()->id();
                                $current_list_item_price->edit_date     =  date('Y-m-d H:i:s');
                                $current_list_item_price->save();
                            } else {
                                $current_list_item_price->delete();
                            }
                        } else if ($value > 0 || $request->item_prices_qtys[$key] > 0) {
                            $item_price                =  new ItemPrice();
                            $item_price->item_id       =  $item->id;
                            $item_price->list_id       =  $list_id;
                            $item_price->price         =  $value;
                            $item_price->list_quant    =  $request->item_prices_qtys[$key];
                            $item_price->shop_id       =  get_session_val('serial_id', 'integer');
                            $item_price->add_user      =  auth()->id();
                            $item_price->add_date      =  date('Y-m-d H:i:s');
                            $item_price->edit_user     =  auth()->id();
                            $item_price->edit_date     =  date('Y-m-d H:i:s');
                            $item_price->save();
                        }
                    }
                }


                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                /* `Save the main unit for this item
                    ## Saved data will like:
                        1. unit_type = 3;
                        2. unit_value = 1;
                        3. unit_price = same item's sale_price;
                        4. unit_barcode = first barcode;
                        5. unit_id = same item's unit_id;
                    # that's it for now XD`;
                */

                if (sizeof($request->barcode) > 1) {
                    $barcode_  = ($request->barcode[0] == null) ?  "" : $request->barcode[0];
                    $main_unit = ItemUnit::where('item_id', $item->id)->where('unit_type', 3)->first();

                    if ($main_unit) {
                        $main_unit->unit_barcode       = $barcode_;
                        $main_unit->unit_einvoice_code = Unit::findOrFail($request->unit_id)->unit_einvoice_code;
                        $main_unit->save();
                    } else {
                        $main_unit                     = new ItemUnit();
                        $main_unit->item_id            = $item->id;
                        $main_unit->unit_id            = $request->unit_id;
                        $main_unit->unit_type          = 3;
                        $main_unit->unit_value         = 1;
                        $main_unit->unit_price         = $item->sale_price;
                        $main_unit->Screenhidden       = 0;
                        $main_unit->shop_id            = get_session_val('serial_id', 'integer');
                        $main_unit->unit_barcode       = $barcode_;
                        $main_unit->unit_einvoice_code = Unit::findOrFail($request->unit_id)->unit_einvoice_code;
                        $main_unit->save();
                    }
                }


                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                /* `loop through other barcodes for this item
                    # it will be saved in items_unit table.
                    the other data will like:
                        1. unit_type = 2 => equals the main unit (3);
                        2. unit_value = 1;
                        3. unit_price = same item's sale_price;
                        4. unit_barcode = this looped iteration barcode;
                        5. unit_id = same item's unit_id;
                    # that's it for now XD`;
                */

                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                $current_barcodes = [];
                if (sizeof($request->barcode) > 1) {
                    for ($b         = 1; $b < sizeof($request->barcode); $b++) {
                        $bb         = explode('-', $request->barcode[$b]);
                        $barcode_id = $bb[1];
                        $b_unit     = ItemUnit::where('item_id', $item->id)
                            ->where('unit_type', 2)
                            ->where('id', $barcode_id)
                            ->first();
                        if ($b_unit) {
                            array_push($current_barcodes, $barcode_id);
                        }
                    }
                }

                $pre_barcodes = ItemUnit::where('item_id', $item->id)
                    ->where('unit_type', 2)
                    ->pluck('id');

                $pre_barcodes     = json_decode(json_encode($pre_barcodes));
                $deleted_barcodes = array_diff($pre_barcodes, $current_barcodes);

                ItemUnit::where('item_id', $item->id)->whereIn('id', $deleted_barcodes)->delete();

                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                if (sizeof($request->barcode) > 1) {

                    for ($b = 1; $b < sizeof($request->barcode); $b++) {
                        $bb         = explode('-', $request->barcode[$b]);
                        $barcode    = $bb[0];
                        $barcode_id = $bb[1];

                        $b_unit = ItemUnit::where('item_id', $item->id)
                            ->where('unit_type', 2)
                            ->where('id', $barcode_id)
                            ->first();

                        if ($b_unit) {
                            array_push($current_barcodes, $barcode_id);
                            $b_unit->unit_barcode       = $barcode;
                            $b_unit->unit_einvoice_code = Unit::findOrFail($item->unit_id)->unit_einvoice_code;
                            $b_unit->save();
                        } else {
                            $b_unit                     = new ItemUnit();
                            $b_unit->item_id            = $item->id;
                            $b_unit->unit_id            = $item->unit_id;
                            $b_unit->unit_type          = 2;
                            $b_unit->unit_value         = 1;
                            $b_unit->unit_price         = $item->sale_price;
                            $b_unit->Screenhidden       = 0;
                            $b_unit->shop_id            = get_session_val('serial_id', 'integer');
                            $b_unit->unit_barcode       = $barcode;
                            $b_unit->unit_einvoice_code = Unit::findOrFail($item->unit_id)->unit_einvoice_code;
                            $b_unit->save();
                        }
                    }
                }

                # for new barcode
                if ($request->has('n_barcode')) {

                    foreach ($request->n_barcode as $bar) {
                        $b_unit               = new ItemUnit();
                        $b_unit->item_id      = $item->id;
                        $b_unit->unit_id      = $item->unit_id;
                        $b_unit->unit_type    = 2;
                        $b_unit->unit_value   = 1;
                        $b_unit->unit_price   = $item->sale_price;
                        $b_unit->Screenhidden = 0;
                        $b_unit->shop_id      = get_session_val('serial_id', 'integer');
                        $b_unit->unit_barcode = $bar;
                        $b_unit->save();
                    }
                }


                if ($request->has('sub_unit_barcode') && sizeof($request->sub_unit_barcode) >= 1) {

                    for ($sub = 0; $sub < sizeof($request->sub_unit_barcode); $sub++) {
                        $sub_unit = ItemUnit::where('shop_id', get_session_val('serial_id', 'integer'))
                            ->where('unit_type', 2)
                            ->where('item_id', $item->id)
                            ->where('unit_id', $request->sub_unit_badr_id[$sub])
                            ->where('unit_barcode', $request->sub_unit_barcode[$sub])
                            ->first();
                        // حاله لو البار هو وهنعدل في القيمه او المقابل
                        if ($sub_unit != null) {

                            $sub_unit                   = new ItemUnit();
                            $sub_unit->unit_value         = $request->sub_bar_unit_value[$sub];
                            $sub_unit->unit_price         = $request->sub_bar_unit_price[$sub];
                            $sub_unit->Screenhidden       = 0;
                            $sub_unit->save();
                        } elseif ($sub_unit == null) {

                            //في حاله اني اضيف اصلا بار كود مش موجود
                            $sub_unit                   = new ItemUnit();
                            $sub_unit->item_id           = $item->id;
                            $sub_unit->unit_id            = $request->sub_unit_badr_id[$sub];
                            $sub_unit->unit_type          = 2;
                            $sub_unit->unit_value         = $request->sub_bar_unit_value[$sub];
                            $sub_unit->unit_price         = $request->sub_bar_unit_price[$sub];
                            $sub_unit->Screenhidden       = 0;
                            $sub_unit->unit_barcode       = $request->sub_unit_barcode[$sub];
                            $sub_unit->unit_einvoice_code = $settings->einvoice == 1 ? (Unit::findOrFail($item->unit_id)->unit_einvoice_code ?? '') : "";
                            $sub_unit->shop_id            = get_session_val('serial_id', 'integer');
                            $sub_unit->save();
                        }


                        // $sub_unit                   = new ItemUnit();
                        // $sub_unit->item_id           = $item->id;
                        // $sub_unit->unit_id            = $request->sub_unit_badr_id[$sub];
                        // $sub_unit->unit_type          = 2;
                        // $sub_unit->unit_value         = $request->sub_bar_unit_value[$sub];
                        // $sub_unit->unit_price         = $request->sub_bar_unit_price[$sub];
                        // $sub_unit->Screenhidden       = 0;
                        // $sub_unit->unit_barcode       = $request->sub_unit_barcode[$sub];
                        // $sub_unit->unit_einvoice_code = $settings->einvoice == 1 ? (Unit::findOrFail($item->unit_id)->unit_einvoice_code ?? '') : "";
                        // $sub_unit->shop_id            = get_session_val('serial_id', 'integer');
                        // $sub_unit->save();
                    }
                }



                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                $current_sub_units = [];
                if ($request->has('sub_units')) {
                    foreach ($request->sub_units as $k => $val) {
                        $unit_arr = explode('-', $val);
                        $id       = $unit_arr[0];
                        $sub_unit = ItemUnit::where('id', $id)->first();

                        if ($sub_unit) {
                            array_push($current_sub_units, $id);
                        }
                    }
                    $pre_sub_units = ItemUnit::where('item_id', $item->id)
                        ->whereIn('unit_type', [0, 1])
                        ->where('shop_id', get_session_val('serial_id', 'integer'))
                        ->pluck('id');

                    $pre_sub_units     = json_decode(json_encode($pre_sub_units));
                    $deleted_sub_units = array_diff($pre_sub_units, $current_sub_units);

                    ItemUnit::where('item_id', $item->id)->whereIn('id', $deleted_sub_units)->delete();
                    $this->createLog(Carbon::now(), "تم  حذف وحده فرعيه للصنف  ". $item->item_name);
                }
                if ($request->has('sub_units') && $request->sub_units[0] == null) {
                    $deleted_sub_units = ItemUnit::where('item_id', $item->id)
                        ->whereIn('unit_type', [2])
                        ->where('shop_id', get_session_val('serial_id', 'integer'))
                        ->pluck('id');
                    $this->createLog(Carbon::now(), "تم حذف وحده فرعيه للصنف  ". $item->item_name);
                }

                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                /*  `loop through sub units for this item
                    # it will be saved in items_unit table.
                        the other data will like:
                        1. unit_type = [0 => bigger or 1 => smaller];
                        2. unit_value = from request;
                        3. unit_price = from request;
                        4. unit_barcode = from request;
                        5. unit_id = from request;
                    # that's it`;
                */

                if ($request->has('sub_units') && $request->sub_units[0] != null) {

                    /*    `The Request is like this:
                    ## string that is separated with (-).
                    ## Sigmants are in order:
                        1. unit_barcode.
                        2. unit_id.
                        3. unit_price.
                        4. unit_type.
                        5. unit_value.
                        6. Screenhidden
                        Now, we are ready to go!`;
                    */

                    foreach ($request->sub_units as $k => $val) {
                        try{
                            $unit_arr           = explode('-', $val);
                            $id                 = $unit_arr[0];
                            $barcode            = $unit_arr[1];
                            $unit_val =  $unit_arr[2];
                        }catch (Exception $e) {
                            continue;
                        }
                        if ($settings->einvoice == 1) {
                            $unit_einvoice_code = trim($unit_arr[2]);
                        }
                        try{
                            $u_id               = ($settings->einvoice == 1) ? $unit_arr[3] : $unit_arr[2];
                            $price              = ($settings->einvoice == 1) ? $unit_arr[4] : $unit_arr[3];
                            $type               = ($settings->einvoice == 1) ? $unit_arr[5] : $unit_arr[4];
                            $value              = ($settings->einvoice == 1) ? $unit_arr[6] : $unit_arr[5];
                            $onscreen           = ($settings->einvoice == 1) ? $unit_arr[7] : $unit_arr[6];


                        }catch (Exception $e) {
                            // dd($unit_arr[6]);
                        }



                        if (!$u_id) continue;
                        $sub_unit = ItemUnit::where('id', $id)->first();
                        $new_sub_unit_bar = ItemUnit::where('unit_id', $unit_val)
                            ->where('unit_type', 2)
                            ->where('item_id', $item->id)->get();

                        if ($new_sub_unit_bar->count() > 0) {
                            foreach ($new_sub_unit_bar as $new_sub) {
                                $new_sub->unit_value = $value;
                                $new_sub->unit_price = $price;
                                $new_sub->save();
                            }
                        }
                        if ($sub_unit) {
                            array_push($current_sub_units, $id);
                            $sub_unit->unit_id            = $u_id;
                            $sub_unit->unit_type          = $type;
                            $sub_unit->unit_value         = $value;
                            $sub_unit->unit_price         = $price;
                            $sub_unit->Screenhidden       = $onscreen;
                            $sub_unit->unit_barcode       = $barcode;
                            $sub_unit->unit_einvoice_code = $unit_einvoice_code ?? null;
                            $sub_unit->save();
                        } else {
                            $sub_unit = new ItemUnit();
                            $sub_unit->item_id            = $item->id;
                            $sub_unit->unit_id            = $u_id;
                            $sub_unit->unit_type          = $type;
                            $sub_unit->unit_value         = $value;
                            $sub_unit->unit_price         = $price;
                            $sub_unit->Screenhidden       = $onscreen;
                            $sub_unit->shop_id            = get_session_val('serial_id', 'integer');
                            $sub_unit->unit_barcode       = $barcode;
                            $sub_unit->unit_einvoice_code = $unit_einvoice_code ?? null;
                            $sub_unit->save();
                        }
                    }
                }

                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                /*    `loop through new sub units for this item
             # it will be saved in items_unit table.
             the other data will like:
                1. unit_type = [0 => bigger or 1 => smaller];
                2. unit_value = from request;
                3. unit_price = from request;
                4. unit_barcode = from request;
                5. unit_id = from request;
             # that's it`;*/

                if ($request->has('new_sub_units')) {
                    /*
                    `The Request is like this:
                    ## string that is separated with (-).
                    ## Sigmants are in order:
                        1. unit_barcode.
                        2. unit_id.
                        3. unit_price.
                        4. unit_type.
                        5. unit_value.
                        6. Screenhidden
                Now, we are ready to go!`;*/

                    foreach ($request->new_sub_units as $k => $val) {

                        $unit_arr           = explode('-', $val);
                        $barcode            = $unit_arr[0];
                        if ($settings->einvoice == 1) {
                            $unit_einvoice_code = trim($unit_arr[1]);
                        }
                        $u_id               = ($settings->einvoice == 1) ? $unit_arr[2] : $unit_arr[1];
                        $price              = ($settings->einvoice == 1) ? $unit_arr[3] : $unit_arr[2];
                        $type               = ($settings->einvoice == 1) ? $unit_arr[4] : $unit_arr[3];
                        $value              = ($settings->einvoice == 1) ? $unit_arr[5] : $unit_arr[4];
                        $onscreen           = ($settings->einvoice == 1) ? $unit_arr[6] : $unit_arr[5];

                        $sub_unit = ItemUnit::where('item_id', $item->id)
                            ->where('unit_type', $type)
                            ->where('unit_barcode', $barcode)
                            ->first();

                        if ($sub_unit) {
                        } else {
                            $sub_unit = new ItemUnit();
                            $sub_unit->item_id            = $item->id;
                            $sub_unit->unit_id            = $u_id;
                            $sub_unit->unit_type          = $type;
                            $sub_unit->unit_value         = $value;
                            $sub_unit->unit_price         = $price;
                            $sub_unit->Screenhidden       = $onscreen;
                            $sub_unit->shop_id            = get_session_val('serial_id', 'integer');
                            $sub_unit->unit_barcode       = $barcode;
                            $sub_unit->unit_einvoice_code = $unit_einvoice_code ?? null;
                            $sub_unit->save();
                        }
                    }
                }

                /* =-=-=-=-=-=-=-=-=-=-=-=-=-=-==-=-=-=-=*/

                // /*   `loop through vats for this item
                //     # it will be saved in item_vat table.
                //         the other data will like:
                //         1. vats = from request;
                //     # that's it`;
                //  */
                // if ($settings->einvoice == 1) {
                //     if ($request->has('vats')) {
                //         $old_vats     = ItemVat::where('shop_id', get_session_val('serial_id', 'integer'))->where('item_id', $item->id)->pluck('vat_id')->toArray();
                //         $deleted_vats = array_diff($old_vats, $request->vats);

                //         $item->vats()->detach($deleted_vats);

                //         foreach ($request->vats as $vat_id => $vat_use) {

                //             $vat = ItemVat::where('item_id', $item->id)
                //                 ->where('shop_id', get_session_val('serial_id', 'integer'))
                //                 ->where('vat_id', $vat_id)
                //                 ->first();

                //             if (!$vat) {
                //                 $vat          = new ItemVat();
                //                 $vat->item_id = $item->id;
                //                 $vat->vat_id  = $vat_id;
                //                 $vat->vat_use = $vat_use;
                //                 $vat->shop_id = get_session_val('serial_id', 'integer');
                //                 $vat->save();

                //                 // Change vat_State in items table
                //                 $item->vat_state          = ($vat_use == 1 || $vat_use == 0) ? $request->einvoice_vat_state : 0;
                //                 $item->purchase_vat_state = ($vat_use == 2 || $vat_use == 0) ? $request->einvoice_vat_state : 0;
                //                 $item->save();
                //             }
                //         }
                //     } else {
                //         ItemVat::where('item_id', $item->id)
                //             ->where('shop_id', get_session_val('serial_id', 'integer'))
                //             // ->where('einvoice_code', '!=', null)
                //             ->delete();
                //     }
                // }

                $this->createLog(Carbon::now(), " تم تعديل علي صنف  ".  $item->item_name);

            });

            return redirect()->back()->with(['success' => __('extends.edit_sucessfully')]);
        } else {
            abort(404);
        }
    }

    public function editSubUnit(Request $request)
    {
        $sub_unit = ItemUnit::find($request->id);
        $units    = Unit::whereShopId(get_session_val('serial_id', 'integer'))->where('id', '!=', $request->unit_id)->get();
        $settings = BadrShop::whereSerialId(get_session_val('serial_id', 'integer'))->first();

        if ($sub_unit) {
            $data = view('item::item_subunit.edit_subunit_render', compact('sub_unit', 'units', 'settings'))->render();
            return response()->json(['status' => true, 'data' => $data]);
        } else {
            return response()->json(['status' => false, 'data' => __('extends.something_wrong')]);
        }
    }

    public function updateSubUnit(Request $request)
    {
        $request->unit_value = eval("return $request->unit_value ;");

        $rules = [
            "unit_id"      => ['required', 'integer', Rule::unique('items_unit', 'unit_id')->where(function ($query) use ($request) {
                return $query->where('item_id', $request->item_id)->whereShopId(get_session_val('serial_id', 'integer'));
            })->ignore($request->item_id, 'item_id')],
            "unit_value"   => ['required'],
            "unit_price"   => ['required', 'numeric'],
            "unit_barcode" => ['required', 'string', Rule::unique('items_unit', 'unit_barcode')->where(function ($query) use ($request) {
                return $query->where('item_id', $request->item_id)->where('unit_id', '=', $request->unit_id)->whereShopId(get_session_val('serial_id', 'integer'));
            })->ignore($request->item_id, 'item_id')],
            "unit_type"    => ['required', 'integer'],
            "current_id"   => ['required', 'integer'],
            "sub_id"       => ['required', 'integer'],
        ];

        $messages = [
            "unit_id.required"      => __('items.sub_unit.unit_id_required'),
            "unit_id.integer"       => __('items.sub_unit.unit_id_integer'),
            "unit_id.unique"        => __('items.sub_unit.unit_id_unique'),
            "unit_value.required"   => __('items.sub_unit.unit_value_required'),
            "unit_value.numeric"    => __('items.sub_unit.unit_value_numeric'),
            "unit_price.required"   => __('items.sub_unit.unit_price_required'),
            "unit_price.numeric"    => __('items.sub_unit.unit_price_numeric'),
            "unit_barcode.required" => __('items.sub_unit.unit_barcode_required'),
            "unit_barcode.string"   => __('items.sub_unit.unit_barcode_string'),
            "unit_barcode.unique"   => __('items.sub_unit.unit_barcode_unique'),
            "unit_type.required"    => __('items.sub_unit.unit_type_required'),
            "unit_type.integer"     => __('items.sub_unit.unit_type_integer'),
            "current_id.required"   => __('items.sub_unit.current_id_required'),
            "current_id.integer"    => __('items.sub_unit.current_id_integer'),
            "sub_id.required"       => __('items.sub_unit.sub_id_required'),
            "sub_id.integer"        => __('items.sub_unit.sub_id_integer'),
        ];

        $validator = $request->validate($rules, $messages);

        if ($validator) {
            $sub_unit         = ItemUnit::find($request->sub_id);
            $old_unit_id      = $sub_unit->unit_id;
            $old_unit_barcode = $sub_unit->unit_barcode;

            if ($sub_unit) {
                $sub_unit->unit_id      = $request->unit_id;
                $sub_unit->unit_type    = $request->unit_type;
                $sub_unit->unit_value   = $request->unit_value;
                $sub_unit->unit_price   = $request->unit_price;
                $sub_unit->Screenhidden = $request->has('Screenhidden') ? 1 : 0;
                $sub_unit->unit_barcode = $request->unit_barcode;
                $sub_unit->save();

                $unit       = Unit::find($request->unit_id);
                $pay_price  = Item::select('pay_price')->find($sub_unit->item_id)->pay_price ?? 0;
                $unit->name = is_null($unit) ? '' : $unit->name;

                $einvoice = get_session_val('einvoice', 'integer') == 1 ? '-' . $sub_unit->unit_einvoice_code . '-' : '-';
                $value = $sub_unit->id . '-' . $sub_unit->unit_barcode . $einvoice . $sub_unit->unit_id . '-' . $sub_unit->unit_price . '-' . $sub_unit->unit_type . '-' . $sub_unit->unit_value . '-' . $sub_unit->Screenhidden;

                $data = '';
                $data = '<tr class="info text-center" id="' . $sub_unit->unit_barcode . '-' . $sub_unit->unit_id . '">';
                $data .= '<td>' . $unit->name . '</td>';
                $data .= '<td>' . quant_decimal($sub_unit->unit_value) . '</td>';
                $data .= '<td>' . price_decimal($sub_unit->unit_price) . '</td>';
                $data .= '<td>' . price_decimal($pay_price * $sub_unit->unit_value) . '</td>';
                $data .= '<td>' . $sub_unit->unit_barcode . '</td>';
                $data .= '<td class="text-center">
                                <button type="button" id="new-sub-barcode-btn"
                                data-sub_unit_id="'.$sub_unit->unit_id.'"
                                data-sub_unit_value="'.$sub_unit->unit_value.'"
                                data-sub_unit_price="'.$sub_unit->unit_price.'"
                                class="btn btn-info col-md-9" data-toggle="modal"
                                data-target="#new-sub-barcode-modal">
                                <i class="fa fa-plus-circle"></i>   اضافه الباركود
                            </button>';
                $data .= '<td>
                            <a class="btn btn-sm btn-warning" data-toggle="modal"
                               data-target="#updateSubUnitElementModal" data-href="' . $sub_unit->id . '" style="margin-left:5px;">
                                <span class="glyphicon glyphicon-pencil" data-toggle="tooltip" data-placement="top"
                                    title="Remove this Element">'.trans("extends.edit").'</span>
                            </a>
                            <a href="javascript:void(0)" class="btn btn-danger delete-sub-unit"
                             data-barcode="' . $sub_unit->unit_barcode . '" data-unit="' . $sub_unit->unit_id . '">'.trans("extends.delete").'</a>
                          </td>';
                $data .= '</tr>';

                $data_hidden = '<input type="hidden" id="input-' . $sub_unit->id .'" name="sub_units[]"
                 value="' . $sub_unit->id . '-' . $sub_unit->unit_barcode . '-' . $sub_unit->unit_id . '-' . $sub_unit->unit_price . '-' . $sub_unit->unit_type . '-' . $sub_unit->unit_value . '-' . $sub_unit->Screenhidden . '">';

                return response()->json([
                    'status'   => true,
                    'id'       => $sub_unit->id,
                    'data'     => $data,
                    'hidden'   => $data_hidden,
                    'row_id'   => $old_unit_barcode . '-' . $old_unit_id,
                    'input_id' => 'input-' . $sub_unit->id,
                    'value'    => $value,
                    'message'  => __('extends.update_sucessfully'),
                ]);
            } else {
                return response()->json(['status' => false, 'message' => __('extends.update_failed')]);
            }
        } else {
            return response()->json(['status' => false, 'message' => __('extends.something_wrong')]);
        }
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}
