<?php

namespace Modules\Item\Http\Controllers;

use App\Services\Einvoice\Einvoice;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Modules\Item\Entities\Item;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Modules\EInvoice\Entities\Settings;
use Modules\Item\Entities\EinvoiceStage;
use Modules\Item\Entities\EinvoiceItemCode;

class ItemEinvoiceController extends Controller
{
    /**
     * Get And Ser the e-invoice code for the item.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Modules\Item\Entities\Item  $item
     * @return Response
     */
    public function egsCode($id, Request $request)
    {
        // Get the item.
        $item = Item::find($id);
        $item_code = EinvoiceItemCode::where('itemCode', $item->getAttributes()['item_einvoice_code'])->first();
        $einvoice_settings = Settings::shop()->first() ?? new Settings();

        // If Request is POST.
        if (request()->isMethod('post')) {

            // variables will remove from the request.
            $except = ['_token'];

            // If the item activeTo date is null or empty remove it from the request.
            $request->activeTo == null
                ? array_push($except, 'activeTo')
                : ($request['activeTo'] = date('Y-m-d\TH:i:s\Z', strtotime($request->activeTo)));

            // Get the data.
            $data               = $request->except($except);
            $data['itemCode']   = 'EG-' . $einvoice_settings->registration_id . '-' . $data['itemCode'];
            $data['activeFrom'] = date('Y-m-d\TH:i:s\Z', strtotime($request->activeFrom));
            $data               = collect()->push($data);

            // Create the e-invoice logs and save the data.
            return $this->createEinvoice($item, $data);
        }

        return view('item::einvoice.CreateEgscode', compact('item', 'einvoice_settings', 'item_code'));
    }

    /**
     * Create Einvoice logs and save data.
     *
     * @param $data
     * @return array
     */
    public function createEinvoice($item, $data)
    {
        $data             = $data->toArray();
        $itemCode         = $data[0]['itemCode'];
        $itemCodeLogs     = EinvoiceStage::where('item_code', $itemCode)->whereIn('status', [0, 1, 2])->orderBy('stage', 'asc')->get();
        $submitLogCount   = EinvoiceStage::where('item_code', $itemCode)->where('stage', 7)->whereIn('status', [3])->count();

        // Intialize the e-invoice item code stages (Intialize stages for the item code).
        $itemCodeLogs->count() == 0
            ? Einvoice::logEinvoiceStage('item_code', $itemCode)
            : Log::channel('einvoice')->info('Re-Submit the e-invoice item code , item code : ' . $itemCode . ' (Retrying from stage : ' . $itemCodeLogs->first()->stage . ')');

        // Update value of einvoice stages.
        $itemCodeLogs     = EinvoiceStage::where('item_code', $itemCode)->whereIn('status', [0, 1, 2])->orderBy('stage', 'asc')->get();
        $submitLogCount   = EinvoiceStage::where('item_code', $itemCode)->where('stage', 7)->whereIn('status', [3])->count();

        // Get stages from einvoice stage table.
        $submitLog          = $itemCodeLogs->where('stage', 7);
        $submitLog          = ($submitLog->count() == 1) ? $submitLog->first()->toArray() : [];
        $operationLogs      = $itemCodeLogs->whereNotIn('stage', [7])->values();
        $operationLogsCount = count($operationLogs);

        // Get the submitted document data from einvoice stage table
        if (empty($submitLog) && $submitLogCount > 0) {
            $itemCodeRequest  = EinvoiceStage::where('item_code', $itemCode)->where('stage', 7)->select('request')->first();
            $itemCodeResponse = EinvoiceStage::where('item_code', $itemCode)->where('stage', 7)->select('response')->first();
            $itemCodeRequest  = json_decode($itemCodeRequest->request, true);
            $itemCodeData     = json_decode($itemCodeResponse->response, true);

            $status           = (isset($itemCodeData['passedItems']) && !is_null($itemCodeData['passedItems']) && !empty($itemCodeData['passedItems']))
                                && ($itemCodeData['passedItemsCount'] ?? 0) > 0
                                ? 200 : 400;
        }

        // Submit the e-invoice item code.
        if (!empty($submitLog) || $submitLogCount == 0 || (!empty($itemCodeData['failedItems']) && ($itemCodeData['passedItemsCount'] ?? 0) == 0)) {

            // Log item code in einvoice stage table for stage 7 (Reqeust).
            Einvoice::logEinvoiceStage('item_code', $itemCode, 7, 1, $data);

            // Send the data to the Taxpayer API.
            $response          = Einvoice::createEgsCode($data);
            $itemCodesResponse = $response->original;
            $status            = $response->status();

            if ($status == 500)
                return back()->with('error', $itemCodesResponse)->withInput(request()->all());

            $itemCodesStatus   = in_array($status, ['200', '201']) ? 3 : 2;

            // Log item code in einvoice stage table for stage 7 (Response).
            Einvoice::logEinvoiceStage('item_code', $itemCode, 7, $itemCodesStatus, $data, $itemCodesResponse);

            // Log Error if failed to submit the e-invoice item code.
            if (in_array($status, [200, 201]) && isset($itemCodesResponse['passedItems']) && count($itemCodesResponse['passedItems'])) {
                Log::channel('einvoice')->info('Successfully submitted the e-invoice item code , item code : ' . $itemCode);
                $itemCodeData = $itemCodesResponse;
            } else {
                if (isset($itemCodesResponse['failedItems'])) {
                    $errors = collect($itemCodesResponse['failedItems'])->map(function ($code) {
                                    return $code['errors'];
                                })->flatten()->implode(', ');
                } else {
                    $errors = $itemCodesResponse ? $itemCodesResponse->getText() : 'حدث خطأ ما';
                }

                Log::channel('einvoice')->error('Failed to submit the e-invoice item code , item code : ' . $itemCode . ' , Error : ' . $errors);

                return back()->with(['einvoiceCodeErrors' => explode(',', $errors)])->withInput(request()->all());
            }
        }


        // Update the item with the e-invoice code if the request was successful.
        if (in_array($status, [200, 201]) || ($operationLogsCount > 0 && count($itemCodeData) )) {

            // Get the item code from einvoice stage table.
            $updateItemLog = $operationLogs->where('stage', 8);
            $updateItemLog = $updateItemLog->count() == 1 ? $updateItemLog->first()->toArray() : [];

            if (!empty($updateItemLog) && in_array($updateItemLog['status'], [0, 1, 2])) {

                // Log item code in einvoice stage table for stage 8 (Reqeust).
                Einvoice::logEinvoiceStage('item_code', $itemCode, 8, 1, $itemCode);

                // Update the item with the e-invoice code if the request was successful.
                $item->item_einvoice_code = $itemCode;
                $itemCodeSaved            = $item->save();
                $itemCodesStatus          = $itemCodeSaved ? 3 : 2;
                $itemCodesResponse        = $itemCodeSaved ? __('extends.update_sucessfully') : __('extends.update_failed');

                // Log item code in einvoice stage table for stage 8 (Response).
                Einvoice::logEinvoiceStage('item_code', $itemCode, 8, $itemCodesStatus, $itemCode, $itemCodesResponse);
            }


            // Get the item code from einvoice stage table (stage 9).
            $updateItemCodeLog = $operationLogs->where('stage', 9);
            $updateItemCodeLog = $updateItemCodeLog->count() == 1 ? $updateItemCodeLog->first()->toArray() : [];

            if (!empty($updateItemCodeLog) && in_array($updateItemCodeLog['status'], [0, 1, 2])) {
                // Log item code in einvoice stage table for stage 9 (Reqeust).
                Einvoice::logEinvoiceStage('item_code', $itemCode, 9, 1, $itemCode);

                // Save the item with the e-invoice code if the request was successful.
                DB::transaction(function () use ($data, $itemCode, $itemCodeData) {
                    $itemCodeData_ = EinvoiceItemCode::where('itemCode', $itemCode);
                    $itemCodeData_ = $itemCodeData_->exists() ? $itemCodeData_->first() : null;

                    $itemcodeSaveDB = null;
                    if ($itemCodeData_ == null) {
                        $itemcodeSaveDB = EinvoiceItemCode::create([
                            "codeUsageRequestID"          => isset($itemCodeData['passedItems']) ? $itemCodeData['passedItems'][0]['codeUsageRequestId'] : '',
                            "codeTypeName"                => $data[0]['codeType'],
                            "itemCode"                    => $data[0]['itemCode'],
                            "codeNamePrimaryLang"         => $data[0]['codeName'],
                            "codeNameSecondaryLang"       => $data[0]['codeNameAr'],
                            "descriptionPrimaryLang"      => $data[0]['description'],
                            "descriptionSecondaryLang"    => $data[0]['descriptionAr'],
                            "parentItemCode"              => $data[0]['parentCode'],
                            "requestCreationDateTimeUtc"  => now(),
                            "codeCreationDateTimeUtc"     => now(),
                            "activeFrom"                  => $data[0]['activeFrom'],
                            "activeTo"                    => $data[0]['activeTo'] ?? null,
                            "active"                      => true,
                            "status"                      => 'Submitted',
                            "statusReason"                => $data[0]['requestReason'] ?? null,
                            "shop_id"                     => get_session_val('serial_id', 'integer')
                        ]);
                    }

                    $itemcodeSaveDBStatus   = $itemcodeSaveDB ? 3 : 2;
                    $itemcodeSaveDBResponse = $itemcodeSaveDB ? __('extends.save_sucessfully') : __('extends.save_failed');

                    // Log item code in einvoice stage table for stage 9 (Response).
                    Einvoice::logEinvoiceStage('item_code', $itemCode, 9, $itemcodeSaveDBStatus, $itemCode, $itemcodeSaveDBResponse);
                });
            }

            // If the data is valid.
            return redirect()->route('items.index')->with(['success' => trans('extends.save_sucessfully')]);
        } else {
            return back()->with(['error' => trans('extends.save_failed')]);
        }
    }
}
