<?php

namespace Modules\Category\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Controllers\Controller;
use Illuminate\Validation\Rule;
use Intervention\Image\Facades\Image;
use Modules\Category\Entities\BillAdd;
use Modules\Category\Entities\Category;
use Modules\Category\Entities\Color;
use Modules\Category\Entities\Company;
use Modules\Category\Entities\Item;
use Modules\Category\Entities\ItemGroup;
use Modules\Category\Entities\ItemPrice;
use Modules\Category\Entities\ItemType;
use Modules\Category\Entities\MultiSupplier;
use Modules\Category\Entities\PriceList;
use Modules\Category\Entities\Size;
use Modules\Category\Entities\Store;
use Modules\Category\Entities\Supplier;
use Modules\Category\Entities\Unit;
use Modules\Category\Entities\ItemTransaction;

class ItemGroupController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index(Request $request)
    {
        $price_decimal = get_session_val('decimal_num_price', 'integer');
        $cats = ItemType::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $suppliers = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $groups = ItemGroup::where('items_group.shop_id', get_session_val('serial_id', 'integer'))
            ->leftJoin('items', 'items.item_group', 'items_group.id')
            ->leftJoin('supplier', 'items.supplier_id', 'supplier.id')
            ->leftJoin('items_type', 'items.sale_unit', 'items_type.id')
            ->leftJoin('store_items', 'items.id', 'store_items.item_id')
            ->select(
                'items_group.group_name',
                'store_items.id as sid',
                'store_items.store_quant',
                'items.item_name',
                'items_group.id',
                'items_group.sale_price',
                'min_quantity',
                'pay_price',
                'about',
                'bolla',
                'items.incom_date',
                'supplier_name',
                'items.sale_unit',
                'incom_date',
                'items_type.name', 'items_type.id as typeid'
            );
        if ($request->filled('name')) {
            $groups = $groups->where('items.item_name', 'like', '%' . $request->name . '%');

        }
        if ($request->filled('sale_unit')) {
            $groups = $groups->where('items.sale_unit', $request->sale_unit);

        }
        if ($request->filled('supplier_id')) {
            $groups = $groups->where('items.supplier_id', $request->supplier_id);

        }
        if ($request->filled('store_id')) {
            $groups = $groups->where('store_items.store_id', $request->store_id);

        }


        $groups = $groups->groupBy('items_group.group_name')->get();
        return view('category::item-group.index', compact('groups', 'cats', 'suppliers', 'stores'));
    }

    /**
     * Show the form for creating a new resource.
     * @return Response
     */
    public function create()
    {
        $sizes = Size::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $colors = Color::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $suppliers = Supplier::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $main_cats = Category::where('category_id', 0)->where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $units = Unit::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $companies = Company::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $shop = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $price_decimal = $shop->decimal_num_price;
        $bill_adds = $shop->bill_adds;
        $adds = collect();
        if ($bill_adds == 0) {
            $adds = BillAdd::where([
                'shop_id' => get_session_val('serial_id', 'integer'),
//                'add_role' != '3',
                'check_addition' => 0
            ])->get();
        }
        $multi_price = $shop->multi_price;
        $lists = collect();
        if ($multi_price == 1) {
            $lists = PriceList::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        }
        return view('category::item-group.create',
            compact('sizes', 'colors', 'suppliers', 'main_cats', 'units', 'companies', 'price_decimal', 'bill_adds', 'bill_adds', 'adds',
                'multi_price', 'lists'));
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Response
     */
    public function store(Request $request)
    {
//        dd($request->all());
        $this->validate($request, [
            'group_name' => 'required|' . Rule::unique('items_group')->where(function ($query) use ($request) {
                    return $query->where('group_name', $request->group_name)
                        ->where('shop_id', get_session_val('serial_id', 'integer'));
                }),
//            'about' => 'required',
            'sizes' => 'required|array',
            'colors' => 'required|array',
            'supplier_id' => 'required',
            'unit_id' => 'required',
//            'company_id' => 'required',
            'pay_price' => 'required',
            'pay_date' => 'required',
            'sale_price' => 'required',
//            'available' => 'required',
//            'is_service' => 'required',
//            'details' => 'required',
            'sale_unit' => 'required',
            'image' => 'image|mimes:jpg,jpeg,bmp,png',
//            'vat_state' => 'required|in:0,1,2'
        ]);
        $transaction = \DB::transaction(function () use ($request) {
            $multi_price = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->multi_price;
            $item_group = new ItemGroup();
            $item_group->shop_id = get_session_val('serial_id', 'integer');
            $item_group->group_name = $request->group_name;
            $item_group->sale_price = $request->sale_price;
            $item_group->details = $request->details;
            if ($request->has('image')) {
                $file = $request->file('image');
                $ext = strtolower($file->getClientOriginalExtension());
                $file_name = date("Y-m-d") . '_' . time() . '.' . $ext;
                $thumb_name = 'thumb_' . date("Y-m-d") . '_' . time() . '.' . $ext;
                $destinationPath = '../upload';
                $img = Image::make($file->getRealPath());
                $img->resize(200, 200, function ($constraint) {
                    $constraint->aspectRatio();
                })->save($destinationPath . '/' . $thumb_name);
                $file->move($destinationPath, $file_name);
                $item_group->img = $file_name;
            }
            $item_group->save();
            foreach ($request->sizes as $size) {
                $size_ = Size::where('shop_id', get_session_val('serial_id', 'integer'))->where('id', $size)->first();
                foreach ($request->colors as $color) {
                    $color_ = Color::where('shop_id', get_session_val('serial_id', 'integer'))->where('id', $color)->first();
                    $item = new Item();
                    $item->item_name = $item_group->group_name . ' ' . $color_->color_name . ' ' . $size_->size_name;
                    $item->sale_price = $item_group->sale_price;
                    $item->sale_unit = $request->sale_unit;
                    $item->quantity = 0;
                    $item->pay_price = $request->pay_price;
                    $item->about = $request->about;
                    $item->min_quantity = 0;
//                $item->supplier_id = $request->supplier_id;
                    $item->add_user = auth()->user()->id;
                    $item->img = 'upload/'.$item_group->img;
                    $item->incom_date = $request->pay_date;
                    $item->barcode = $item_group->id . '' . $color_->color_code . '' . $size_->size_code;
                    $item->shop_id = get_session_val('serial_id', 'integer');
                    $item->is_service = $request->is_service ?? 0;
                    $item->unit_id = $request->unit_id;
//                $item->company_id = $request->company_id;
                    $item->online = $request->online ?? 0;
                    $item->details = $item_group->details ?? "";
                    $item->item_group = $item_group->id ?? 0;
                    $item->size_id = $size_->id ?? 0;
                    $item->color_id = $color_->id ?? 0;
                    $item->vat_state = $request->vat_state ?? 1;
                    $item->vat_id = $request->vat_id ?? 0;
                    $item->save();
                    $item_transaction = ItemTransaction::create([
                        'tansaction_date' => Carbon::now(),
                        'action_date' => Carbon::now(),
                        'item_id' => $item->id,
                        'type' => 4,
                        'price' => $item->sale_price,
                        'quantity' => $item->quantity,
                        'new_quantity' => $item->quantity,
                        'remain_quantity' => $item->quantity,
                        'incom_id' => 0,
                        'store_id' => 0,
                        'user_id' => auth()->user()->id,
                        'shop_id' => auth()->user()->shop_id,
                    ]);
                    $item_supplier = MultiSupplier::create([
                        'item_id' => $item->id,
                        'supplier_id' => $request->supplier_id,
                        'shop_id' => get_session_val('serial_id', 'integer')
                    ]);
                    if ($multi_price == 1 && $request->has('lists') && is_array($request->lists) && sizeof($request->lists)) {
                        foreach ($request->lists as $key => $value) {
                            $x = explode('-', $value);
                            $price = $x[0];
                            $qty = $x[1];
                            $item_price = new ItemPrice();
                            $item_price->list_id = $key;
                            $item_price->item_id = $item->id;
                            $item_price->price = $price;
                            $item_price->list_quant = $qty;
                            $item_price->shop_id = get_session_val('serial_id', 'integer');
                            $item_price->add_user = auth()->id();
                            $item_price->save();
                        }
                    }
                }
            }
        });
        if ($transaction) {
            return back()->withErrors('خطأ اثناء الحفظ');
        } else {
            return back()->with(['success' => 
            trans('native.correct_save')]);
        }
    }

    /**
     * Show the specified resource.
     * @return Response
     */
    public function show($id)
    {


        return view('category::show');
    }

    /**
     * Show the form for editing the specified resource.
     * @return Response
     */
    public function edit()
    {
        return view('category::edit');
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @return Response
     */
    public function update(Request $request)
    {
    }

    /**
     * Remove the specified resource from storage.
     * @return Response
     */
    public function destroy()
    {
    }
}
