<?php

namespace Modules\Item\Http\Requests;

use __PHP_Incomplete_Class;
use Illuminate\Foundation\Http\FormRequest;

class BarcodeRequest extends FormRequest
{
    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
        $rule = $this->isMethod('POST') ? 'required' : 'nullable';
        return [
            "barcode_reader"             => [$rule, "boolean"],
            "show_item_name"             => [$rule, "boolean"],
            "show_item_description"      => [$rule, "boolean"],
            "show_price"                 => [$rule, "boolean"],
            "show_shop_name"             => [$rule, "boolean"],
            "print_barcode_number"       => [$rule, "boolean"],
            "barcode_scale_support"      => [$rule, "boolean"],
            "show_unit_name"             => [$rule, "boolean"],
            "barcode_number"             => [$rule, "integer"],
            "barcode_template"           => [$rule, "integer"],
            "scale_start_code"           => ['nullable', 'integer'],
            "text_size"                  => [$rule, "numeric"],
            "item_code_digits"           => [$rule, "numeric"],
            "barcode_height"             => [$rule, "numeric"],
            "barcode_width"              => [$rule, "numeric"],
            "logo_height"                => [$rule, "numeric"],
            "logo_width"                 => [$rule, "numeric"],
            "barcode_reader_code_height" => [$rule, "numeric"],
            "barcode_reader_code_width"  => [$rule, "numeric"],
            "ticket_top_margin"          => [$rule, "numeric"],
            "ticket_bottom_margin"       => [$rule, "numeric"],
            "ticket_left_margin"         => [$rule, "numeric"],
            "ticket_right_margin"        => [$rule, "numeric"],
            "page_top_margin"            => [$rule, "numeric"],
            "page_bottom_margin"         => [$rule, "numeric"],
            "page_left_margin"           => [$rule, "numeric"],
            "page_right_margin"          => [$rule, "numeric"],
        ];
    }

    /**
     * Get the error messages for the defined validation rules.
     *
     * @return array
     */
    public function messages()
    {
        return [
            "barcode_reader.required"             => __("items.barcode.barcode_reader_required"),
            "barcode_reader.boolean"              => __("items.barcode.barcode_reader_boolean"),
            "show_item_name.required"             => __("items.barcode.show_item_name_required"),
            "show_item_name.boolean"              => __("items.barcode.show_item_name_boolean"),
            "show_item_description.required"      => __("items.barcode.show_item_description_required"),
            "show_item_description.boolean"       => __("items.barcode.show_item_description_boolean"),
            "show_price.required"                 => __("items.barcode.show_price_required"),
            "show_price.boolean"                  => __("items.barcode.show_price_boolean"),
            "show_shop_name.required"             => __("items.barcode.show_shop_name_required"),
            "show_shop_name.boolean"              => __("items.barcode.show_shop_name_boolean"),
            "print_barcode_number.required"       => __("items.barcode.print_barcode_number_required"),
            "print_barcode_number.boolean"        => __("items.barcode.print_barcode_number_boolean"),
            "barcode_scale_support.required"      => __("items.barcode.barcode_scale_support_required"),
            "barcode_scale_support.boolean"       => __("items.barcode.barcode_scale_support_boolean"),
            "barcode_number.required"             => __("items.barcode.barcode_number_required"),
            "barcode_number.integer"              => __("items.barcode.barcode_number_integer"),
            "barcode_template.required"           => __("items.barcode.barcode_template_required"),
            "barcode_template.integer"            => __("items.barcode.barcode_template_integer"),
            "scale_start_code.integer"            => __("items.barcode.scale_start_code_integer"),
            "text_size.required"                  => __("items.barcode.text_size_required"),
            "text_size.numeric"                   => __("items.barcode.text_size_numeric"),
            "item_code_digits.required"           => __("items.barcode.item_code_digits_required"),
            "item_code_digits.numeric"            => __("items.barcode.item_code_digits_numeric"),
            "barcode_height.required"             => __("items.barcode.barcode_height_required"),
            "barcode_height.numeric"              => __("items.barcode.barcode_height_numeric"),
            "barcode_width.required"              => __("items.barcode.barcode_width_required"),
            "barcode_width.numeric"               => __("items.barcode.barcode_width_numeric"),
            "logo_height.required"                => __("items.barcode.logo_height_required"),
            "logo_height.numeric"                 => __("items.barcode.logo_height_numeric"),
            "logo_width.required"                 => __("items.barcode.logo_width_required"),
            "logo_width.numeric"                  => __("items.barcode.logo_width_numeric"),
            "barcode_reader_code_height.required" => __("items.barcode.barcode_reader_code_height_required"),
            "barcode_reader_code_height.numeric"  => __("items.barcode.barcode_reader_code_height_numeric"),
            "barcode_reader_code_width.required"  => __("items.barcode.barcode_reader_code_width_required"),
            "barcode_reader_code_width.numeric"   => __("items.barcode.barcode_reader_code_width_numeric"),
            "ticket_top_margin.required"          => __("items.barcode.ticket_top_margin_required"),
            "ticket_top_margin.numeric"           => __("items.barcode.ticket_top_margin_numeric"),
            "ticket_bottom_margin.required"       => __("items.barcode.ticket_bottom_margin_required"),
            "ticket_bottom_margin.numeric"        => __("items.barcode.ticket_bottom_margin_numeric"),
            "ticket_left_margin.required"         => __("items.barcode.ticket_left_margin_required"),
            "ticket_left_margin.numeric"          => __("items.barcode.ticket_left_margin_numeric"),
            "ticket_right_margin.required"        => __("items.barcode.ticket_right_margin_required"),
            "ticket_right_margin.numeric"         => __("items.barcode.ticket_right_margin_numeric"),
            "page_top_margin.required"            => __("items.barcode.page_top_margin_required"),
            "page_top_margin.numeric"             => __("items.barcode.page_top_margin_numeric"),
            "page_bottom_margin.required"         => __("items.barcode.page_bottom_margin_required"),
            "page_bottom_margin.numeric"          => __("items.barcode.page_bottom_margin_numeric"),
            "page_left_margin.required"           => __("items.barcode.page_left_margin_required"),
            "page_left_margin.numeric"            => __("items.barcode.page_left_margin_numeric"),
            "page_right_margin.required"          => __("items.barcode.page_right_margin_required"),
            "page_right_margin.numeric"           => __("items.barcode.page_right_margin_numeric"),
        ];
    }

    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }
}
