@extends('master::layouts.master')

@section('content')

<div class="card card-light mb-1_5 p-3">

    <div class="container-fluid mb-3">
        <div class="row mb-3">
            <h1 class="text-info">{{ __('items.add_item') }}</h1>
        </div>

        <div class="row">
            <form id="item_form" class="form-horizontal w-100 custome-form" action="{{ route('items.store') }}" method="post"
                enctype="multipart/form-data">
                @csrf
                <input class="hidden" name="embedded" value={{ $embedded ?? 0 }}>

                {{-- Item Name & About --}}
                <div class="form-row row">
                    <div class="form-group col-md-6">
                        <label class="control-label" for="item_name">{{ __('items.name') }} :</label>
                        <input type="text" class="form-control col-md-12 input-normal" name="item_name" id="item_name"
                            value="{{ old('item_name') ? old('item_name') : '' }}" autofocus>
                    </div>
                    <div class="form-group col-md-6">
                        <label class="control-label" for="about">{{ __('items.about') }} :</label>
                        <input type="text" class="form-control col-md-12 input-normal" name="about" id="about"
                            value="{{ old('about') ? old('about') : '' }}" />
                    </div>
                </div>

                {{-- Item Quantity & Min Quantity --}}
                <div class="form-row row">
                    <div class="form-group col-md-6">
                        <label class="control-label" for="quantity">{{ __('items.quantity.name') }} :</label>
                        <input type="number" class="form-control col-md-12 input-normal" name="quantity" id="quantity" step="0.01"
                            value="{{ old('quantity') ? old('quantity') : quant_decimal(0) }}">
                        <small class="text-danger hidden" id="store_note"></small>
                    </div>
                    <div class="form-group col-md-6">
                        <label class="control-label" for="min_quantity">{{ __('items.min_quantity') }} :</label>
                        <input type="number" class="form-control col-md-12 input-normal" name="min_quantity" id="min_quantity"
                            value="{{ old('min_quantity') ? old('min_quantity') : quant_decimal(0) }}">
                    </div>
                </div>

                {{-- Item Store --}}
                <div class="form-row row">
                    <div class="form-group col-md-6 hidden" id="store_div">
                        <label class="control-label" for="stores">{{ __('native.store') }} :</label>
                        <select class="form-control selectpicker col-md-12 input-normal"
                            data-none-selected-text="{{ trans('native.no-selected') }}" name="stores" id="stores"
                            title="{{ __('items.choose_item_store') }}">
                            @foreach ($stores as $store)
                                <option {{ old('stores') == $store->id || $loop->first ? 'selected' : '' }}
                                    value="{{ $store->id }}" data-store_name="{{ $store->store_name }}">
                                    {{ $store->store_name }}</option>
                            @endforeach
                        </select>
                    </div>
                </div>

                {{-- Item Pay Price --}}
                <div class="form-row row">
                    @if(authorized('can_add_purchases_invoices'))
                    <div class="form-group col-md-6">
                        <label class="control-label" for="pay_price">{{ __('items.pay_price.name') }} :</label>
                        <input type="number" class="form-control col-md-12 input-normal" name="pay_price" id="pay_price" step="0.01"
                            value="{{ old('pay_price') ? old('pay_price') : quant_decimal(0) }}">
                    </div>
                    @endif

                    <div class="form-group col-md-6">
                        <label class="control-label" for="pay_date">{{ __('items.purchase_date') }} :</label>
                        <input type="text" class="form-control col-md-12 date input-normal" name="pay_date" id="pay_date"
                            value="{{ old('pay_date') ? old('pay_date') : '' }}">
                    </div>
                </div>

                {{-- Item Sale Price --}}
                <div class="form-row row">
                    <div class="form-group col-md-6">
                        <label class="control-label" for="sale_price">{{ __('items.sale_price.name') }} :</label>
                        <input type="number" class="form-control col-md-12 input-normal" name="sale_price" id="sale_price"
                            step="0.01" value="{{ old('sale_price') ? old('sale_price') : quant_decimal(0) }}">
                    </div>

                    <div class="form-group col-md-6">
                        <label class="control-label" for="lowest_price">{{ __('items.min_sale_price') }} :</label>
                        <input type="number" class="form-control col-md-12 input-normal" name="lowest_price" step="0.01"
                            value="{{ old('lowest_price') ? old('lowest_price') : quant_decimal(0) }}" id="lowest_price">
                    </div>
                </div>

                @if (get_session_val('multi_price', 'Integer') == 1)

                    <div class="form-row row">
                        <h6 class="col-md-6 text-info">
                            {{ __('multiPrices.price_lists') }}
                        </h6>
                    </div>

                    <div class="form-row row">
                        @foreach ($price_lists as $price_list)
                            <div class="form-group col-md-3 form-inline">
                                <label class="control-label col-md-4 ">{{ $price_list->list_name }} :</label>
                                <input type="number" class="hidden input-normal " name="item_prices_ids[]"
                                    value="{{ $price_list->id }}">

                                <input type="text" class="form-control input-normal use-only-numbers col-md-8 list_price" name="item_prices[]"
                                    min="0" data-list="{{ $price_list->id }}"
                                    value="{{ old('item_prices')[$loop->index] ?? 0 }}">
                            </div>
                            <div class="form-group col-md-3 form-inline">
                                <label class="control-label col-md-4">{{ __('multiPrices.required_qty') }} :</label>
                                <input type="text" class="form-control input-normal use-only-numbers col-md-8 list_price_qty"
                                    name="item_prices_qtys[]" min="0" data-list="{{ $price_list->id }}"
                                    value="{{ old('item_prices_qtys')[$loop->index] ?? 0 }}">
                            </div>
                        @endforeach
                    </div>
                @endif


                {{-- Item Barcode --}}
                <div class="form-row row">
                    <div class="form-group col-md-6">
                        <label class="control-label" for="barcode">{{ __('items.barcode.name') }} :</label>
                        <input type="number" value="{{ $barcode }}" class="form-control input-normal col-md-12"
                            name="barcode[]" id="barcode">
                    </div>

                    <div class="form-group col-md-6 align-self-end">
                        <button type="button" id="new-barcode-btn" class="btn btn-info col-md-6" data-toggle="modal"
                            data-target="#new-barcode-modal">
                            <i class="fa fa-plus-circle"></i> {{ __('items.add_barcode') }}
                        </button>

                        <button class="btn btn-primary col-md-5 mr-1 hidden" type="button" data-toggle="collapse"
                            id="newBarcode-btn" data-target="#newBarcode" aria-expanded="false"
                            aria-controls="newBarcode"><i class="fa fa-barcode"></i>
                            {{ __('items.show_barcodes') }}</button>

                        <div class="collapse" id="newBarcode">
                            <div class="card card-body">
                                <ul class="list-inline barcode-list p-0 m-0"></ul>
                                <div id="barcodes"></div>
                            </div>
                        </div>
                    </div>
                </div>

                {{-- Item E-Invoice --}}
                @if ($settings->einvoice == 1)
                    <div class="form-row row">
                        <div class="form-group col-md-6">
                            <label class="control-label" for="unit_einvoice_code">{{ __('items.unit_einvoice_code') }}
                                :</label>
                            <input type="text" class="form-control input-normal col-md-12" name="unit_einvoice_code"
                                id="unit_einvoice_code" value="{{ old('unit_einvoice_code') ?? '' }}" readonly>
                        </div>
                        <div class="form-group col-md-6">
                            <label class="control-label" for="item_einvoice_code">{{ __('items.item_einvoice_code') }}
                                :</label>
                            <div class="input-group">
                                <input type="text" class=" input-normal form-control col-md-9" name="item_einvoice_code"
                                    id="item_einvoice_code"
                                    value="{{ old('item_einvoice_code') ? old('item_einvoice_code') : '' }}"
                                    style="border-left: 0; border-radius: 0 4px 4px 0 !important;">
                                <input type="text" class="input-normal form-control col-md-3"
                                    value="{{ '-EG-' . ($einvoice_settings->registration_id ?? '') }}" disabled
                                    style="border-right: 0; border-radius: 4px 0 0 4px; cursor: context-menu">
                            </div>
                        </div>
                    </div>
                @endif

                {{-- GPC Code && Item Expiry Date --}}
                <div class="form-row row">
                    @if ($settings->einvoice == 1)
                        <div class="form-group col-md-6">
                            <label class="control-label" for="gpc_code">{{ __('items.gpc') }}
                                :</label>
                            <input type="text" class="input-normal form-control col-md-12" name="parent_code" id="gpc_code"
                                value="{{ old('parent_code') ? old('parent_code') : '' }}">
                        </div>
                    @endif

                    <div class="form-group col-md-6">
                        <label class="control-label" for="expire_date">{{ __('items.expire_date') }} :</label>
                        <input type="text" class="input-normal form-control col-md-12 date_" name="expire_date" id="expire_date"
                            value="{{ old('expire_date') ? old('expire_date') : '' }}">
                    </div>
                </div>

                {{-- Main Unit && Item Category --}}
                <div class="form-row row">
                    <div class="form-group col-md-6">
                        <label class="control-label" for="unit_id">{{ __('items.main_unit') }} :</label>
                        <select data-live-search="true" id="unit_id" class="form-control input-normal selectpicker col-md-12"
                            data-none-selected-text="{{ trans('native.no-selected') }}" name="unit_id">
                            @foreach ($units as $unit_)
                                {{-- <option value="{{ $unit_->id }}" data-name="{{ $unit_->name }}"
                                    data-unit_einvoice_code="{{ $unit_->unit_einvoice_code }}">
                                    {{ $unit_->name }}</option> --}}
                                <option {{ old('unit_id') == $unit_->id ? 'selected' : '' }} value="{{ $unit_->id }}"
                                    data-name="{{ $unit_->name }}"
                                    data-unit_einvoice_code="{{ $unit_->unit_einvoice_code }}">{{ $unit_->name }}
                                </option>
                            @endforeach
                        </select>
                    </div>

                    <div class="form-group col-md-6" id="sale_unit_div">
                        <label class="control-label" for="category">{{ __('items.sale_unit.name') }} :</label>
                        <select class="form-control input-normal selectpicker col-md-12" name="category" id="sale_unit"
                            title="{{ __('items.choose_item_cat') }}">
                            @foreach ($main_cats as $cat)
                                <option {{ old('category') == $cat->id ? 'selected' : '' }} value="{{ $cat->id }}"
                                    {{ $loop->first ? 'selected' : '' }}>{{ $cat->name }}</option>
                                {{ getSubCatOptions($cat->id) }}
                            @endforeach
                        </select>
                    </div>
                </div>

                {{-- Supplier && Shelf --}}
                <div class="form-row row">
                    <div class="form-group col-md-6">
                        <label class="control-label" for="supplier_id">{{ __('items.supplier') }} :</label>
                        <select data-live-search="true" multiple class=" input-normal form-control col-md-12 selectpicker"
                            name="supplier_id[]" id="supplier_id" title="{{ __('items.chooseSupplier') }}">
                            @foreach ($suppliers as $supplier)
                                <option value="{{ $supplier->id }}" {{ $loop->first ? 'selected' : '' }}>
                                    {{ $supplier->client_name }}
                                </option>
                            @endforeach
                        </select>
                    </div>

                    {{-- <div class="form-group col-md-6">
                        <label class="control-label">{{ __('items.store_shelf') }} :</label>
                        <select id="shelf" class="form-control input-normal col-md-12 selectpicker" data-live-search="true"
                            name="shelf" title="{{ __('items.chooseShelf') }}">
                            @foreach ($shelves as $shelf)
                                <option {{ old('shelf') == $shelf->id ? 'selected' : '' }} value="{{ $shelf->id }}">
                                    {{ $shelf->shelf_name }}</option>
                            @endforeach
                        </select>
                    </div> --}}
                </div>

                {{-- Item Vats & Stores --}}
                {{-- @if ($settings->einvoice == 1) --}}
                {{-- <div class="form-row row">
                        <div class="form-group col-md-6">
                            <label class="control-label" for="einvoice_vat_state">{{ __('items.einvoice_vat_state') }}
                                :</label>
                            <select class="col-md-12 form-control selectpicker input-normal" name="einvoice_vat_state" data-none-selected-text="{{ trans('native.no-selected') }}"
                                id="einvoice_vat_state">
                                <option value="0" {{ old('einvoice_vat_state') == 0 ? 'selected' : '' }}
                                    id="einvoice_vat_state_exempt">
                                    {{ __('items.exempt') }}</option>
                                <option value="1" {{ old('einvoice_vat_state') == 1 ? 'selected' : '' }}
                                    id="einvoice_vat_state_not_include">{{ __('items.not_include') }}</option>
                                <option value="2" {{ old('einvoice_vat_state') == 2 ? 'selected' : '' }}
                                    id="einvoice_vat_state_include">
                                    {{ __('items.include') }}</option>
                            </select>
                        </div>
                        <div class="form-group col-md-6 hidden" id="vat_id_div">
                            <label class="control-label" for="vat_id">{{ __('items.item_taxs') }}
                                :</label>
                            <select data-live-search="true" id="vat_id" class="form-control input-normal selectpicker col-md-12" data-none-selected-text="{{ trans('native.no-selected') }}"
                                name="vats_ids[]" title="{{ __('items.choose_tax') }}" multiple>
                                @foreach ($vats as $vat)
                                    <option value="{{ $vat->id . '-' . $vat->vat_operation }}"
                                        data-name="{{ $vat->Addition_name }}" data-value="{{ $vat->addition_value }}">
                                        {{ $vat->Addition_name }}
                                    </option>
                                @endforeach
                            </select>
                        </div>
                    </div> --}}
                {{-- @else --}}
                @if (get_session_val('apply_vats', 'integer'))
                    {{-- Sales Item Vat --}}
                    <div class="form-row row">
                        <div class="form-group col-md-6">
                            <label class="control-label" for="sales_vat_div">{{ __('items.sales_item_vats_type') }}
                                :</label>
                            <select class="col-md-12 form-control input-normal selectpicker" name="vat_state"
                                data-none-selected-text="{{ trans('native.no-selected') }}" id="sales_vat_select">
                                <option value="0" {{ old('vat_state') == 0 ? 'selected' : '' }} id="sales_exempt">
                                    {{ __('items.exempt') }}</option>
                                <option value="1" {{ old('vat_state') == 1 ? 'selected' : '' }}
                                    id="sales_not_include">{{ __('items.not_include') }}</option>
                                <option value="2" {{ old('vat_state') == 2 ? 'selected' : '' }} id="sales_include">
                                    {{ __('items.include') }}</option>
                            </select>
                        </div>

                        <div class="form-group col-md-6 @if ((!old('vat_ids') && empty(old('vat_ids'))) || in_array(old('vat_state'), [0, '0'])) hidden @endif"
                            id="sales_item_vats">
                            <label class="control-label" for="sales_vat_id">{{ __('items.sales_item_vats') }}
                                :</label>
                            <select data-live-search="true" id="sales_vat_id" multiple
                                class="form-control selectpicker input-normal col-md-12" name="vat_ids[]"
                                data-none-selected-text="{{ trans('native.no-selected') }}">
                                @foreach ($vats as $index => $vat)
                                    <option {{ old('vat_ids') && in_array($vat->id, old('vat_ids')) ? 'selected' : '' }}
                                        value="{{ $vat->id . '-' . $vat->vat_operation }}"
                                        data-name="{{ $vat->Addition_name }}" data-value="{{ $vat->addition_value }}">
                                        {{ $vat->Addition_name }}
                                    </option>
                                @endforeach
                            </select>
                        </div>
                    </div>

                    {{-- Purchase Item Vat --}}
                    <div class="form-row row">
                        <div class="form-group col-md-6">
                            <label class="control-label" for="purchase_vat_div">{{ __('items.purchase_item_vats') }}
                                :</label>
                            <select class="col-md-12 form-control input-normal selectpicker" name="purchase_vat_state"
                                data-none-selected-text="{{ trans('native.no-selected') }}" id="purchase_vat_select">
                                <option value="0" {{ old('purchase_vat_state') == 0 ? 'selected' : '' }}
                                    id="purchase_exempt">{{ __('items.exempt') }}</option>
                                <option value="1" {{ old('purchase_vat_state') == 1 ? 'selected' : '' }}
                                    id="purchase_not_include">{{ __('items.not_include') }}</option>
                                <option value="2" {{ old('purchase_vat_state') == 2 ? 'selected' : '' }}
                                    id="purchase_include">{{ __('items.include') }}</option>
                            </select>
                        </div>

                        <div class="form-group col-md-6 @if ((!old('purchase_vat_ids') && empty(old('purchase_vat_ids'))) || in_array(old('purchase_vat_state'), [0, '0'])) hidden @endif"
                            id="purcahse_item_vats">
                            <label class="control-label" for="purcahse_vat_id">{{ __('items.purchase_item_vats') }}
                                :</label>
                            <select data-live-search="true" id="purcahse_vat_id"
                                class="form-control selectpicker input-normal col-md-12"
                                data-none-selected-text="{{ trans('native.no-selected') }}" name="purchase_vat_ids[]"
                                multiple>
                                @foreach ($vats as $index => $vat)
                                    <option value="{{ $vat->id . '-' . $vat->vat_operation }}"
                                        {{ old('purchase_vat_ids') && in_array($vat->id, old('purchase_vat_ids')) ? 'selected' : '' }}
                                        data-name="{{ $vat->Addition_name }}" data-value="{{ $vat->addition_value }}">
                                        {{ $vat->Addition_name }}
                                    </option>
                                @endforeach
                            </select>
                        </div>
                    </div>
                @endif
                {{-- @endif --}}

                {{-- Item Company --}}
                {{-- @if ($settings->items_company != 0)
                    <div class="form-row row">
                        <div class="form-group col-md-6">
                            <label class="control-label" for="company_id">{{ __('items.variable_price_list') }} :</label>
                            <select data-live-search="true" class="form-control col-md-12 selectpicker input-normal" name="company_id"
                                id="company_id" title="{{ __('items.choose_variable_list') }}">
                                @foreach ($companies as $company)
                                    <option  value="{{ $company->id }}" {{(old("company_id") == $company->id? "selected":"") }}>{{ $company->company_name }}
                                    </option>
                                @endforeach
                            </select>
                        </div>

                        <div class="form-group col-md-6 d-flex flex-column justify-content-end">
                            <button type="button" id="new-company-btn" class="btn btn-info col-md-6"
                                data-toggle="modal" data-target="#new-company-modal">
                                <i class="fa fa-plus-circle"></i> {{ __('items.add_variable_price_list') }}
                            </button>
                        </div>
                    </div>
                @endif --}}

                {{-- Item Vats --}}
                <div class="row">
                    <div class="col-md-12">
                        <table class="table zui-gray-table zui-table-rounded table-bordered table-hover bg-white hidden" id="vats">
                            <thead>
                                <th class="text-center">{{ __('items.tax_name') }}</th>
                                <th class="text-center">{{ __('items.tax_value') }}</th>
                                <th class="text-center">{{ __('items.tax_use') }}</th>
                                <th class="text-center">{{ __('extends.action') }}</th>
                            </thead>
                            <tbody></tbody>
                        </table>
                    </div>
                </div>
                
                <div class="row">
                    @if ( get_session_val('ecomme', 'integer') == 1 )
                        <div class="form-group col-md-6">
                            <label class="control-label" for="order_max_quantity">اقصي عدد للطلب من المتجر :</label>
                            <input type="number" min="0" class="form-control col-md-12 input-normal" name="order_max_quantity" id="order_max_quantity" value="{{ old('order_max_quantity') ?? 0 }}">
                        </div>
                    @endif
                </div>

                {{-- Item Sub Unit --}}
                <div class="row">
                    <div id="sub_units_inputs"></div>
                    <div class="col-md-12">
                        <table class="table  zui-gray-table zui-table-rounded table-bordered table-hover bg-white hidden" id="sub_units">
                            <thead>
                                <th class="text-center">{{ __('items.unit.title') }}</th>
                                @if ($settings->einvoice == 1)
                                    <th class="text-center">{{ __('items.unit.einvoice_code') }}</th>
                                @endif
                                <th class="text-center">{{ __('items.unit.value') }}</th>
                                <th class="text-center">{{ __('items.unit.price') }}</th>
                                <th class="text-center">{{ __('items.unit.pay-price') }}</th>
                                <th class="text-center">{{ __('items.unit.barcode') }}</th>
                                <th class="text-center">{{ __('items.add_barcode') }}</th>
                                <th class="text-center">{{ __('extends.action') }}</th>
                            </thead>
                            <tbody></tbody>
                        </table>
                    </div>
                </div>

                {{-- Item Size && Color --}}
                @if ($settings->color_size != 0)
                    <hr>
                    <div class="form-row row">
                        <h4 class="text-primary d-block">{{ __('items.select_item_color_Size') }}</h4>
                    </div>
                    <div class="form-row row">
                        <div class="form-group col-md-6">
                            <label class="control-label" for="size_id">{{ __('items.sizes') }} :</label>
                            <select data-live-search="true" id="size_id" title="{{ __('items.choose_size') }}"
                                class="form-control input-normal selectpicker col-md-12"
                                data-none-selected-text="{{ trans('native.no-selected') }}" name="sizes[]" multiple>
                                @foreach ($sizes as $size)
                                    <option value="{{ $size->id }}"
                                        {{ old('sizes[]') == $size->id ? 'selected' : '' }}
                                        data-name="{{ $size->size_name }}">
                                        {{ $size->size_name }}
                                    </option>
                                @endforeach
                            </select>
                        </div>
                        <div class="form-group col-md-6 d-flex flex-column justify-content-end">
                            <button type="button" class="btn btn-info col-md-6" id="new-size-btn" data-toggle="modal"
                                data-target="#new-size-modal">
                                <i class="fa fa-plus-circle"></i> {{ __('items.add_size') }}
                            </button>
                        </div>
                    </div>

                    <div class="form-row row">
                        <div class="form-group col-md-6">
                            <label class="control-label" for="color_id">{{ __('items.colors') }} :</label>
                            <select data-live-search="true" id="color_id" class="form-control input-normal selectpicker col-md-12"
                                data-none-selected-text="{{ trans('native.no-selected') }}" name="colors[]"
                                title="{{ __('items.choose_color') }}" multiple>

                                @foreach ($colors as $color)
                                    <option value="{{ $color->id }}"
                                        {{ old('colors[]') == $color->id ? 'selected' : '' }}
                                        data-name="{{ $color->color_name }}">
                                        {{ $color->color_name }}
                                    </option>
                                @endforeach
                            </select>
                        </div>
                        <div class="form-group col-md-6 d-flex flex-column justify-content-end">
                            <button type="button" class="btn btn-info col-md-6" id="new-color-btn" data-toggle="modal"
                                data-target="#new-color-modal">
                                <i class="fa fa-plus-circle"></i> {{ __('items.add_color') }}
                            </button>
                        </div>
                    </div>
                    <hr>
                @endif

                {{-- Price List --}}
                {{-- @if ($settings->multi_price != 0 && $pricesList->count())
                    <!--<hr>-->
                    <!--<div class="form-row row">-->
                    <!--    <h4 class="text-primary">{{ __('items.price_list') }}</h4>-->
                    <!--</div>-->
                    <!--<div class="form-row row">-->
                    <!--    @foreach ($pricesList as $price)-->
                    <!--        <div class="form-group col-md-4">-->
                    <!--            <label class="control-label" for="prices"> {{ $price->list_name }} :</label>-->
                    <!--            <input type="text" class="form-control col-md-12" value="{{ $price->price }} "-->
                    <!--                name="price-{{ $price->id }}" >-->
                    <!--        </div>-->
                    <!--    @endforeach-->
                    <!--</div>-->
                    <!--<hr>-->
                @endif --}}

                {{-- Seller Commission --}}
                @if ($settings->bolla != 0)
                    <hr>
                    <div class="form-row row">
                        <div class="form-group col-md-6">
                            <label class="control-label" for="bolla">{{ __('items.seller_commission') }} :</label>
                            <input type="text" class="form-control input-normal col-md-12" name="bolla"
                                value="{{ old('bolla') ? old('bolla') : quant_decimal(0) }}" id="bolla">
                        </div>
                    </div>
                    <hr>
                @endif

                {{-- Meters Per Carton --}}
                @if ($settings->module == 7)
                    <hr>
                    <div class="form-row row">
                        <div class="form-group col-md-6">
                            <label class="control-label" for="meter_ceramic">{{ __('items.meters_per_carton') }}
                                :</label>
                            <input type="text" class="form-control input-normal col-md-12" name="meter_ceramic" id="meter_ceramic"
                                value="{{ old('meter_ceramic') ? old('meter_ceramic') : '' }}">
                        </div>
                    </div>
                    <hr>
                @endif

                {{-- Background Color && Order Show --}}
                @if ($settings->module == 6)
                    <hr>
                    <div class="form-row row">
                        <div class="form-group col-md-6">
                            <label class="control-label" for="background_color">{{ __('items.background_color') }}
                                :</label>
                            <input type="color" class="form-control input-normal col-md-12 jscolor" name="background_color"
                                id="background_color"
                                value="{{ old('background_color') ? old('background_color') : '' }}">
                        </div>
                        <div class="form-group col-md-6">
                            <label class="control-label" for="order_show">{{ __('items.order_show') }} :</label>
                            <input type="text" class="form-control input-normal col-md-12" name="order_show"
                                value="{{ old('order_show') ? old('order_show') : quant_decimal(0) }}" id="order_show">
                        </div>
                    </div>
                    <hr>
                @endif

                {{-- Amount used for cars --}}
                @if ($settings->module == 13)
                    <hr>
                    <div class="form-row row">
                        <div class="form-group col-md-4">
                            <label class="control-label" for="small_qty">{{ __('items.amount_used_for_small_cars') }}
                                :</label>
                            <input type="text" class="form-control input-normal col-md-12 numbercheck" name="small_qty"
                                id="small_qty" value="{{ old('small_qty') ? old('small_qty') : quant_decimal(0) }}" />
                        </div>

                        <div class="form-group col-md-4">
                            <label class="control-label" for="small_price">{{ __('items.amount_used_for_medium_cars') }}
                                :</label>
                            <input type="text" class="form-control input-normal col-md-12 numbercheck" name="medium_qty"
                                id="medium_qty" value="{{ old('small_price') ? old('small_price') : '1' }}" />
                        </div>

                        <div class="form-group col-md-4">
                            <label class="control-label" for="medium_price">{{ __('items.amount_used_for_large_cars') }}
                                :</label>
                            <input type="text" class="form-control input-normal col-md-12 numbercheck" name="large_qty"
                                id="large_qty" value="{{ old('large_qty') ? old('large_qty') : '2' }}" />
                        </div>
                    </div>
                    <hr>
                @endif

                {{-- Shipping Companies --}}
                @if ($settings->module == 8)
                    <hr>
                    <div class="form-row row">
                        <div class="form-group col-md-6">
                            <label class="control-label" for="card_company_id">{{ __('items.shipping_companies') }}
                                :</label>
                            <select data-live-search="true" name="card_company_id" id="card_company_id"
                                class="form-control input-normal selectpicker col-md-12"
                                data-none-selected-text="{{ trans('native.no-selected') }}"
                                title="{{ __('items.select_shipping_company') }}">
                                @foreach ($cardCompany as $card)
                                    <option value="{{ $card->id }}" data-name="{{ $card->company_name }}">
                                        {{ $card->company_name }}</option>
                                @endforeach
                            </select>
                        </div>
                        <div class="form-group col-md-6 d-flex flex-column justify-content-end">
                            <a href="javascript:void(0)" class="btn btn-info col-md-6" id="new-color-btn"><i
                                    class="fa fa-plus-circle"></i> {{ __('items.add_shipping_company') }}
                            </a>
                        </div>
                    </div>
                    <hr>
                @endif

                {{-- Item Options --}}
                <div class="form-group col-md-12 p-3 bg-white">
                    <h6 for="item_options">{{ __('items.item_options') }} :</h6>

                    {{-- Item Buttons --}}
                    <div class="form-row row text-center">
                        @if (get_session_val('units_sup', 'Integer') == 1)
                            <div class="form-group col-md-4">
                                <label class="control-label col-md-8"
                                    for="add-sub-units">{{ __('items.add_sub_units') }}
                                    :</label>
                                <a href="javascript:void(0)" class="btn btn-info" id="add-sub-units">
                                    <i class="fa fa-plus-circle"></i> {{ __('extends.add') }}
                                </a>
                            </div>
                        @endif

                        <div class="form-inline col-md-4">
                            <label class="control-label col-md-8" for="available">{{ __('items.show_inventory') }}
                                :</label>
                            <div class="col-md-4">
                                <input type="checkbox" class="form-control input-normal" name="available" id="available"
                                    {{ old('available') ?? 'checked' }}>
                            </div>
                        </div>
                        <div class="form-inline col-md-4">
                            <label class="control-label col-md-8" for="online">{{ __('items.show_online_store') }}
                                :</label>
                            <div class="col-md-4">
                                <input type="checkbox" class="form-control input-normal" name="online" id="online"
                                    {{ old('online') ?? 'checked' }}>
                            </div>
                        </div>
                        <div class="form-inline col-md-4" style="margin-right: 15px;">
                            <label class="control-label col-md-8" for="is_special">{{ __('items.is_special') }}
                                :</label>
                            <div class="col-md-4">
                                <input type="checkbox" class="form-control input-normal" name="is_special" id="is_special"
                                    {{ old('is_special') ?? 'checked' }}>
                            </div>
                        </div>
                        <div class="form-inline col-md-4">
                            <label class="control-label col-md-8" for="is_service" style="display: block">
                                {{ __('items.service_item') }} :
                                <small class="text-info" style="float: left">@lang('items.hidden-in-inventory')</small>
                            </label>
                            <div class="col-md-4">
                                <input type="checkbox" class="form-control input-normal" name="is_service" id="is_service"
                                    {{ old('is_service') ? 'checked' : '' }}>
                            </div>
                        </div>

                        {{-- <div class="form-group col-md-4">
                            <label class="control-label col-md-8" for="new-shelf-btn">{{ __('items.add_shelf') }}
                                :</label>
                            <a href="javascript:void(0)" id="new-shelf-btn" class="btn btn-info">
                                <i class="fa fa-plus-circle"></i> {{ __('extends.add') }}
                            </a>
                        </div>

                        <div class="form-group col-md-4">
                            <label class="control-label col-md-8" for="new-supplier-btn">{{ __('items.add_supplier') }}
                                :</label>
                            <a href="javascript:void(0)" id="new-supplier-btn" class="btn btn-info">
                                <i class="fa fa-plus-circle"></i> {{ __('extends.add') }}
                            </a>
                        </div> --}}
                    </div>

                    {{-- <div class="form-row row text-center"> --}}
                    {{-- <div class="form-inline col-md-4">
                            <label class="control-label col-md-8" for="available">{{ __('items.show_inventory') }}
                                :</label>
                            <div class="col-md-4">
                                <input type="checkbox" class="form-control input-normal" name="available" id="available"
                                    {{ old('available') ? 'checked' : '' }}>
                            </div>
                        </div> --}}

                    {{-- <div class="form-inline col-md-4">
                            <label class="control-label col-md-8" for="online">{{ __('items.show_online_store') }}
                                :</label>
                            <div class="col-md-4">
                                <input type="checkbox" class="form-control input-normal" name="online" id="online" {{(old('online')?'checked':'')}} >
                            </div>
                        </div> --}}

                    {{-- <div class="form-inline col-md-4">
                            <label class="control-label col-md-8" for="is_service">{{ __('items.service_item') }}
                                :</label>
                            <div class="col-md-4">
                                <input type="checkbox" class="form-control input-normal" name="is_service" id="is_service"
                                    {{ old('is_service') ? 'checked' : '' }}>
                            </div>
                        </div> --}}

                    {{-- @if ($settings->bill_time_alert)
                            <div class="form-inline col-md-4">
                                <label class="control-label col-md-8" for="game">{{ __('items.game_with_time') }}
                                    :</label>
                                <div class="col-md-4">
                                    <input type="checkbox" class="form-control input-normal" name="game" id="game" {{(old('game')?'checked':'')}}>
                                </div>
                            </div>
                        @endif --}}

                    {{-- @if ($settings->module == 6)
                            <div class="form-inline col-md-4">
                                <label class="control-label col-md-8" for="cooking_add">{{ __('items.add_on_item') }}
                                    :</label>
                                <div class="col-md-4">
                                    <input type="checkbox" class="form-control input-normal" name="cooking_add" id="cooking_add"
                                        {{ old('cooking_add') ? 'checked' : '' }}>
                                </div>
                            </div>
                        @endif --}}

                    {{-- @if ($settings->module == 8 && $settings->show_item_serial != 0)
                            <div class="form-inline col-md-4">
                                <label class="control-label col-md-8"
                                    for="require_serial">{{ __('items.require_serial') }} :</label>
                                <div class="col-md-4">
                                    <input type="checkbox" class="form-control input-normal" name="require_serial"
                                        id="require_serial" {{ old('require_serial') ? old('require_serial') : '' }}>
                                </div>
                            </div>
                        @endif
                    </div> --}}

                    {{-- Manufacturing --}}
                    @if ($settings->manufacturing)
                        <div class="form-row row text-center mt-3 mb-3">
                            <div class="form-inline col-md-4">
                                <label for="main_item" class="control-label col-md-8">{{ __('items.main_item') }}
                                    :</label>
                                <input id="main_item" class="col-md-4 input-normal" name="manufacturing" type="radio"
                                    value="0" checked>
                            </div>

                            <div class="form-inline col-md-4">
                                <label for="main_manuf"
                                    class="control-label col-md-8">{{ __('items.main_manufacturer') }} :</label>
                                <input id="main_manuf" class="col-md-4 input-normal" name="manufacturing" type="radio"
                                    value="1">
                            </div>

                            <div class="form-inline col-md-4">
                                <label for="item_manuf"
                                    class="control-label col-md-8">{{ __('items.item_manufacturer') }} :</label>
                                <input id="item_manuf" class="col-md-4 input-normal" name="manufacturing" type="radio"
                                    value="2">
                            </div>
                        </div>
                    @endif
                </div>

                {{-- Item Details && Images --}}
                <div class="form-row row mb-4">
                    <div class="form-group col-md-6">
                        <label class="control-label" for="details">{{ __('items.details') }} :</label>
                        <textarea class="form-control  input-normal h-100" name="details" style="resize: none" id="details" rows="7"></textarea>
                    </div>

                    <div class="form-group col-md-6">
                        <label class="control-label d-block" for="game">{{ __('items.image.name') }} :</label>
                        <div class="text-center">
                            <input type="file" class="mb-3" name="images" accept="image/*"
                                onchange="loadFile(event)">
                            <img id="output" width="200" height="200" src="">
                        </div>
                    </div>
                </div>
                <div class="form-row row mb-4 justify-content-center mt-4">
                <div class="form-group col-md-2 col-6  mt-4 ">

                <button class="btn btn-primary  w-100 ">{{ __('extends.save') }}</button>
                    </div>
                <div class="form-group col-md-2 col-6   mt-4">

                <button class="btn btn-success ml-1 w-100"
                    id="save_print">{{ __('extends.save_and_print') }}</button>
                    </div>
                    </div>
            </form>

            <div class="col-md-12">
                <input type="hidden" name="date_now" id="date_now">
                <div id="dataContent"></div>
            </div>
        </div>
    </div>

                    </div>
    <div class="modal fade" id="new-sub-barcode-modal" tabindex="-1" role="dialog" aria-labelledby="myModalLabel"
    aria-hidden="true">
    <div class="modal-dialog">
        <div class="modal-content">
            <div class="modal-header">
                <button type="button" class="close" data-dismiss="modal">&times;</button>
                <h4 class="modal-title" style="margin-left:200px">@lang('items.add-barcode-for-unit')</h4>
            </div>
            <div class="modal-body">
                {{-- <form id="new-barcode-form-modal" action="" method="post"> --}}
                {{-- {{ csrf_field() }} --}}
                <div class="modal-body" id="modal-loader-u">

                    <div class="form-group">
                        <input type="hidden" name="unit_id" id="sub_bar_unit_id" value=""/>
                        <input type="hidden" name="unit_value" id="sub_bar_unit_value" value=""/>
                        <input type="hidden" name="unit_price" id="sub_bar_unit_price" value=""/>
                        <label class="control-label" for="barcode"> @lang('items.barcode.name') :</label>
                        <input type="number" class="form-control" name="new-sub-barcode[]" id="new-sub-barcode"
                            autofocus />
                    </div>

                </div>


                {{-- </form> --}}
            </div>
            <div class="modal-footer">
                <a href="javascript:void(0)" class="btn btn-primary"
                    id="add-sub-barcode-btn">{{ trans('extends.save-btn') }} </a>
                <button type="button" class="btn btn-default" data-dismiss="modal">{{ trans('extends.close') }}
                </button>
            </div>
        </div>

    </div>
</div>
    </div>
    @include('item::popups')
@stop

@section('css-scripts')
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
@stop

@section('js-scripts')
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>

    <script>
        $(function() {
            if ('{{ get_session_val('date_type', 'Integer') }}' == '2') {
                $(".date").calendarsPicker({
                    calendar: $.calendars.instance('gregorian'),
                    dateFormat: 'Y-mm-dd',
                    defaultDate: "0",
                    selectDefaultDate: true
                });

                $(".date_").calendarsPicker({
                    calendar: $.calendars.instance('gregorian'),
                    dateFormat: 'Y-mm-dd',

                });
            } else {
                $(".date").calendarsPicker({
                    calendar: $.calendars.instance('ummalqura'),
                    dateFormat: 'Y-mm-dd',
                    defaultDate: "0",
                    selectDefaultDate: true
                });

                $(".date_").calendarsPicker({
                    calendar: $.calendars.instance('ummalqura'),
                    dateFormat: 'Y-mm-dd',
                });
            }
        });
    </script>

    {{-- Print Barcode --}}
    <script>
        $("#save_print").click(function(e) {
            e.preventDefault();
            $("#item_form").append('<input type="hidden" name="print_barcode" value="1">');
            $("#item_form").submit();
        });

        $('#item_form').on('submit', function() {
            $('button').attr('disabled', true);
        });

        var success = '{{ session()->has('success') }}';
        var print_barcode = '{{ session()->get('print_barcode') }}';
        if (success && print_barcode == 1) {
            $('#printBarcodeModal').modal('show');
        }

        $('#printBarcodeForm').on('submit', function(e) {
            var items = [{
                id: parseInt('{{ session()->get('item_id') }}'),
                quantity: parseInt($('#printBarcodeForm input[name="quantity"]').val()),
            }];
            $('#printBarcodeForm').append(`<input type="hidden" name="items" value='${JSON.stringify(items)}'>`);
        });
    </script>

    {{-- Item Vats --}}
    <script>
        $("#vat_id").change(function(e) {
            e.preventDefault();

            $("#vats").removeClass('hidden');
            $("#vats >tbody").empty();

            $('#vat_id option:selected').each(function() {
                var vat_id = $(this).val();

                $("#vats >tbody").append(
                    `<tr class="vat-row" id="tr-${vat_id}">
                        <td class="text-center">${$(this).data('name')}</td>
                        <td class="text-center">${$(this).data('value')}</td>
                        <td class="text-center">
                            <label for="both-${vat_id}">كلاهما:</label>
                            <input id="both-${vat_id}" name="vats[${vat_id}]" type="radio" value="0" style="margin-left: 10px;" checked>

                            <label for="pay-${vat_id}">مبيعات:</label>
                            <input id="pay-${vat_id}" name="vats[${vat_id}]" type="radio" value="1" style="margin-left: 10px;">

                            <label for="sale-${vat_id}">مشتريات:</label>
                            <input id="sale-${vat_id}" name="vats[${vat_id}]" type="radio" value="2">
                        </td>
                        <td class="text-center">
                            <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-vat"
                                data-vat_id=${vat_id}>
                                <i class="fa fa-trash"></i>
                            </a>
                        </td>
                    </tr>`);
            });

            var table_length = $(document).find(".vat-row").length;

            table_length == 0 && $("#vats").addClass('hidden');
        });

        $(document).on('change', '#quantity', function() {
            var value = $(this).val();
            // $('#min_quantity').val(value);

            if (value > 0) {
                var store_name = $('#stores option:selected').data('store_name');
                $("#store_div").removeClass('hidden');
                $('#store_note').removeClass('hidden').empty().text(
                    `{{ __('items.add_store_note') }} ${store_name}`);

            } else {
                $(this).val(0);
                $("#store_div").addClass('hidden');
                $('#store_note').addClass('hidden');
            }
        });

        $(document).on('change', '#stores', function() {
            var store_name = $('#stores option:selected').data('store_name');
            $('#store_note').removeClass('hidden').empty().text(`{{ __('items.add_store_note') }} ${store_name}`);
        });

        $(document).on('click', '.delete-vat', function() {
            var id = $(this).data('vat_id');
            var vat_use = $(this).data('vat_use');
            var currentVats = $('#vat_id').val();
            var currentSalesVats = $("#sales_vat_id").val();
            var currentPurchasesVats = $("#purcahse_vat_id").val();
            var table_length = $(document).find(".vat-row").length;
            var einvoice = {{ $settings->einvoice }};

            if (confirm("@lang('items.confirm-to-delete-vat')?")) {
                $(`#tr-${id}-${vat_use}`).remove();

                if (einvoice == 1) {
                    // remove from vats array
                    currentVats = jQuery.grep(currentVats, function(value) {
                        return value != id;
                    });

                    $('#vat_id').val(currentVats);
                    $('#vat_id').selectpicker('refresh', {
                        noneSelectedText: "{{ trans('native.no-selected') }}"
                    });

                    table_length == 1 && $("#vats").addClass('hidden');
                } else {
                    // If E-Invoice is disabled
                    if (vat_use == 1 && $(document).find(".sales").length > 0) {
                        currentSalesVats = jQuery.grep(currentSalesVats, function(value) {
                            return value != id;
                        });

                        $('#sales_vat_id').val(currentSalesVats);
                        $('#sales_vat_id').selectpicker('refresh', {
                            noneSelectedText: "{{ trans('native.no-selected') }}"
                        });

                        if (['', ' ', null].includes(currentSalesVats)) {
                            alert('يجب تحديد ضريبة للمبيعات');
                            $("#sales_exempt").prop('checked', true);
                            $('#sales_item_vats').addClass('hidden');
                            $('#sales_vat_id option').first().prop('selected', true);
                            $('#sales_vat_id').selectpicker('refresh', {
                                noneSelectedText: "{{ trans('native.no-selected') }}"
                            });
                        }
                    } else if ($(document).find(".sales").length == 0) {
                        $("#sales_exempt").prop('checked', true);
                        $('#sales_item_vats').addClass('hidden');
                    }

                    if (vat_use == 2 && $(document).find(".purchase").length > 0) {
                        currentPurchasesVats = jQuery.grep(currentPurchasesVats, function(value) {
                            return value != id;
                        });

                        $('#purcahse_vat_id').val(currentPurchasesVats);
                        $('#purcahse_vat_id').selectpicker('refresh', {
                            noneSelectedText: "{{ trans('native.no-selected') }}"
                        });

                        if (['', ' ', null].includes(currentPurchasesVats)) {
                            $("#purchase_exempt").prop('checked', true);
                            $('#purcahse_item_vats').addClass('hidden');
                            $('#purcahse_vat_id option').first().prop('selected', true);
                            $('#purcahse_vat_id').selectpicker('refresh', {
                                noneSelectedText: "{{ trans('native.no-selected') }}"
                            });
                        }
                    } else if ($(document).find(".purchase").length == 0) {
                        $("#purchase_exempt").prop('checked', true);
                        $('#purcahse_item_vats').addClass('hidden');
                    }

                    table_length == 1 && $("#vats").addClass('hidden');
                }
            }
        });
    </script>

    {{-- Item Vats If E-Invoice Disabled --}}
    <script>
        // Sales Vats State
        $(document).on('change', '#sales_vat_select', function() {
            var item_vat_state = $('#sales_vat_select').val();

            if (item_vat_state == 1) {
                $("#sales_vat_id").attr('multiple', 'multiple');
                $("#sales_vat_id").selectpicker('destroy');
                $("#sales_vat_id").selectpicker();
                $(document).find(".sales").length == 0 && $('#sales_vat_id option').first().prop('selected', true);
                $('#sales_vat_id').selectpicker('refresh', {
                    noneSelectedText: "{{ trans('native.no-selected') }}"
                });
                $('#sales_item_vats').removeClass('hidden');
            } else {
                $('#sales_vat_id').removeAttr('multiple');
                $("#sales_vat_id").selectpicker('destroy');
                $("#sales_vat_id").selectpicker();
                $('#sales_item_vats').removeClass('hidden');
            }

            if (item_vat_state > 0) {
                $("#vats").removeClass('hidden');
                $(document).find(".sales").remove();
                $('#sales_vat_id option:selected').each(function() {
                    var vat_id = $(this).val();

                    $("#vats >tbody").append(
                        `<tr class="vat-row sales" id="tr-${vat_id}-1">
                            <td class="text-center">${$(this).data('name')}</td>
                            <td class="text-center">${$(this).data('value')}</td>
                            <td class="text-center">مبيعات</td>
                            <td class="text-center">
                                <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-vat"
                                    data-vat_id=${vat_id} data-vat_use="1">
                                    <i class="fa fa-trash"></i>
                                </a>
                            </td>
                        </tr>`);
                });
            } else {
                $(document).find(".sales").remove();
                $('#sales_item_vats').addClass('hidden');
            }
        });

        // Sales Vats selectpicker
        $(document).on('change', '#sales_vat_id', function(e) {
            var item_vat_state = $(this).val();

            if (['', ' ', null].includes($(this).val())) {
                alert('يجب تحديد ضريبة للمبيعات');
                $(document).find(".sales").remove();
                $("#sales_exempt").prop('checked', true);
                $('#sales_item_vats').addClass('hidden');
                $('#sales_vat_id option').first().prop('selected', true);
                return false;
            }

            $("#vats").removeClass('hidden');
            $(document).find(".sales").remove();

            $('#sales_vat_id option:selected').each(function() {
                var vat_id = $(this).val();

                $("#vats >tbody").append(
                    `<tr class="vat-row sales" id="tr-${vat_id}-1">
                        <td class="text-center">${$(this).data('name')}</td>
                        <td class="text-center">${$(this).data('value')}</td>
                        <td class="text-center">{{ __('extends.sales') }}</td>
                        <td class="text-center">
                            <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-vat"
                                data-vat_id=${vat_id} data-vat_use="1">
                                <i class="fa fa-trash"></i>
                            </a>
                        </td>
                    </tr>`);
            });
        });

        // Purchases Vats State
        $(document).on('change', '#purchase_vat_select', function() {
            var item_vat_state = $('#purchase_vat_select').val();

            if (item_vat_state == 1) {
                $("#purcahse_vat_id").attr('multiple', 'multiple');
                $("#purcahse_vat_id").selectpicker('destroy');
                $("#purcahse_vat_id").selectpicker();
                $(document).find(".purchase").length == 0 && $('#purcahse_vat_id option').first().prop('selected',
                    true);
                $('#purcahse_vat_id').selectpicker('refresh', {
                    noneSelectedText: "{{ trans('native.no-selected') }}"
                });
                $('#purcahse_item_vats').removeClass('hidden');
            } else {
                $('#purcahse_vat_id').removeAttr('multiple');
                $("#purcahse_vat_id").selectpicker('destroy');
                $("#purcahse_vat_id").selectpicker();
                $('#purcahse_item_vats').removeClass('hidden');
            }

            if (item_vat_state > 0) {
                $("#vats").removeClass('hidden');
                $(document).find(".purchase").remove();
                $('#purcahse_vat_id option:selected').each(function() {
                    var vat_id = $(this).val();

                    $("#vats >tbody").append(
                        `<tr class="vat-row purchase" id="tr-${vat_id}-2">
                            <td class="text-center">${$(this).data('name')}</td>
                            <td class="text-center">${$(this).data('value')}</td>
                            <td class="text-center">مشتريات</td>
                            <td class="text-center">
                                <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-vat"
                                    data-vat_id=${vat_id} data-vat_use="2">
                                    <i class="fa fa-trash"></i>
                                </a>
                            </td>
                        </tr>`);
                });
            } else {
                $(document).find(".purchase").remove();
                $('#purcahse_item_vats').addClass('hidden');
            }
        });

        // Purchases Vats selectpicker
        $(document).on('change', '#purcahse_vat_id', function(e) {
            var item_vat_state = $('#purchase_vat_select').val();

            if (['', ' ', null].includes($(this).val())) {
                alert('يجب تحديد ضريبة للمشتريات');
                $(document).find(".purchase").remove();
                $("#purchase_exempt").prop('checked', true);
                $('#purcahse_item_vats').addClass('hidden');
                $('#purcahse_vat_id option').first().prop('selected', true);
                return false;
            }

            $("#vats").removeClass('hidden');
            $(document).find(".purchase").remove();

            $('#purcahse_vat_id option:selected').each(function() {
                var vat_id = $(this).val();

                $("#vats >tbody").append(
                    `<tr class="vat-row purchase" id="tr-${vat_id}-2">
                        <td class="text-center">${$(this).data('name')}</td>
                        <td class="text-center">${$(this).data('value')}</td>
                        <td class="text-center">مشتريات</td>
                        <td class="text-center">
                            <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-vat"
                                data-vat_id=${vat_id} data-vat_use="2">
                                <i class="fa fa-trash"></i>
                            </a>
                        </td>
                    </tr>`);
            });
        });
    </script>

    {{-- Item Vats If E-Invoice Enabled --}}
    <script>
        $("#einvoice_vat_state").on('change', function() {
            var einvoice_vat_state = $(this).val();
            if (einvoice_vat_state != 0) {
                $('#vat_id_div').removeClass('hidden');
            }
        });
    </script>

    <script>
        var loader = "<div align='center' id='loader'>" +
            "<i class='fa fa-spinner fa-pulse fa-5x fa-fw'></i>" +
            "</div><br>";

        var decimal_value = '{{ $price_decimal }}';

        $.ajaxSetup({
            headers: {
                'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content'),
                type: "GET",
                data: null,
                contentType: false,
                cache: false,
                processData: false,
                timeout: 3000,
            }
        });

        // $('#item_name').focusout(function() {
        //     $('#about').val($('#item_name').val());
        // });

        function syncUnitQty() {
            let price = $("#pay_price").val();
            $('#sub_units .unit_pay_price').each(function() {
                let target = $(this).closest('tr').attr('id')+"-unit_value",
                    quantity = Number($(`#${target}`).text());
                $(this).text( quantity * price );
            });
        }

        $('#lowest_price').val($('#sale_price').val());

        $('#pay_price').change(function(e) {
            e.preventDefault();
            if ($(this).val() == '' || $(this).val() == ' ' || $(this).val() == null || $(this).val() < 0) {
                $('#pay_price').val(0);
            }

            syncUnitQty();
        });

        $('#lowest_price').change(function(e) {
            e.preventDefault();
            if ($(this).val() == '' || $(this).val() == ' ' || $(this).val() == null || $(this).val() < 0) {
                $('#lowest_price').val(0);
            }
        });

        $('#lowest_price').val($('#sale_price').val());

        $('#sale_price').focusout(function() {
            // $('#lowest_price').val($(this).val());
            if ($(this).val() == '' || $(this).val() == ' ' || $(this).val() == null || $(this).val() < 0) {
                $(this).val(0);
                $('#lowest_price').val(0);
            }
        });

        var unitId = $("#unit_id option:selected");
        $("#unit_einvoice_code").val(unitId.data("unit_einvoice_code"));
        $.map($('#unit option'), function(val, index) {
            if ($(val).val() == unitId.val()) {
                $(val).addClass('hidden');
                $('#unit').val($('#unit option').not('.hidden').first().val());
                $('#unit').selectpicker('refresh', {
                    noneSelectedText: "{{ trans('native.no-selected') }}"
                });
            } else {
                $(val).removeClass('hidden');
                $('#unit').selectpicker('refresh', {
                    noneSelectedText: "{{ trans('native.no-selected') }}"
                });
            }
        });

        $("#unit_id").change(function(e) {
            e.preventDefault();
            var unitId = $("#unit_id option:selected");
            $("#unit_einvoice_code").val(unitId.data("unit_einvoice_code"));
            $.map($('#unit option'), function(val, index) {
                if ($(val).val() == unitId.val()) {
                    $(val).addClass('hidden');
                    $('#unit').val($('#unit option').not('.hidden').first().val());
                    $('#unit').selectpicker('refresh', {
                        noneSelectedText: "{{ trans('native.no-selected') }}"
                    });
                } else {
                    $(val).removeClass('hidden');
                    $('#unit').selectpicker('refresh', {
                        noneSelectedText: "{{ trans('native.no-selected') }}"
                    });
                }
            });
        });

        $(document).on('click', '#new-supplier-btn', function(e) {
            e.preventDefault();
            $('#new-supplier-modal').modal('show');
        });

        // Unit E-Invoice Code
        var code = $("#unit option:selected").data("unit_einvoice_code");
        $("#modal_unit_einvoice_code").val(code);

        // Add Unit E-Invoice Code When Select Unit
        $("#unit").change(function(e) {
            e.preventDefault();
            code = $("#unit option:selected").data("unit_einvoice_code");
            $("#modal_unit_einvoice_code").val(code);
        });
        /********************************************************************************************************************************************/
        // New Supplier Modal
        $('#new-supplier-form-modal').on('submit', function(e) {
            e.preventDefault();
            data = new FormData($('#new-supplier-form-modal')[0]);
            $.ajax({
                url: '{{ route('items.supplier.store') }}',
                type: "post",
                data: data,
                contentType: false,
                cache: false,
                processData: false, // <==
                timeout: 3000,
                beforeSend: function() {
                    $('#modal_client').empty().append(loader);
                },
                success: function(data) {
                    $(document).find('#supplier_id').prev('div').find('ul').append(data.list);
                    $(document).find('#supplier_id').append(data.option);
                    $('.selectpicker').selectpicker('refresh', {
                        noneSelectedText: "{{ trans('native.no-selected') }}"
                    });
                    $('#messages').removeClass('hidden').addClass('alert alert-success').html(
                        '{{ __('extends.save_sucessfully') }}');
                    $('#new-supplier-modal').modal('toggle');
                },
                error: function(xhr) {
                    var error = '<ul class="list-unstyled mb-0 pr-0">';
                    $.each(xhr.responseJSON.errors, function(index, value) {
                        error += '<li>' + value + '</li>';
                    });
                    error += "</ul>";

                    var supplier_name = $("#supplier_name").val();
                    var tele = $("#tele").val();

                    if (['', ' ', null].includes(supplier_name)) {
                        $('#supplier_name').focus();
                        $('label[for="supplier_name"]').addClass('text-danger');
                        $("#supplier_name").addClass('is-invalid');
                    } else {
                        $('label[for="supplier_name"]').removeClass('text-danger');
                        $("#supplier_name").removeClass('is-invalid');
                    }

                    if (['', ' ', null].includes(tele)) {
                        $('#tele').focus();
                        $('label[for="tele"]').addClass('text-danger');
                        $("#tele").addClass('is-invalid');
                    } else {
                        $('label[for="tele"]').removeClass('text-danger');
                        $("#tele").removeClass('is-invalid');
                    }

                    $('#new-supplier-modal').modal('toggle');
                    $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(
                        error);
                }
            });
        });

        $('#new-supplier-modal').on('hidden.bs.modal', function() {
            $("#supplier_name").val('');
            $("#supplier_name").removeClass('is-invalid');
            $('label[for="supplier_name"]').removeClass('text-danger');

            $("#tele").val('');
            $("#tele").removeClass('is-invalid');
            $('label[for="tele"]').removeClass('text-danger');
        });
        /********************************************************************************************************************************************/

        /*----------------------------------------------------------------------------------------------------------*/
        // Barcode Modal
        $(document).on('click', '#new-barcode-btn', function(e) {
            e.preventDefault();
            $('#barcode').val() == '' ? $('#barcode').focus() : $('#new-barcode-modal').modal('show');
        });
        $(document).on('click', '#new-sub-barcode-btn', function() {

             // let unit_id = $(this).data('sub_unit_id');
             let unit_value =$(this).data('sub_unit_value');
            let unit_price =$(this).data('sub_unit_price');
            var unit_id = $(this).data('sub_unit_id');
           $("#new-sub-barcode-modal #sub_bar_unit_id").val( unit_id );
           $("#new-sub-barcode-modal #sub_bar_unit_price").val( unit_price );
           $("#new-sub-barcode-modal #sub_bar_unit_value").val( unit_value );
           $('#new-sub-barcode-modal').modal('show');

        });

        $('#add-barcode-btn').on('click', function(e) {
            var barcode = $('#new-barcode').val();

            if (!['', ' ', null].includes(barcode)) {

                $("#newBarcode-btn").removeClass('hidden');
                $(document).find('#newBarcode').removeClass('hidden').collapse('show');

                $('#barcodes').append(
                    `<input type="hidden" id="h-${barcode}" name="n_barcode[]" value="${barcode}">`);
                $('.barcode-list').append(`<li class="external_barcode list-inline-item ml-1" id="l-${barcode}">
                    <a href="javascript:void(0)" class="text-danger delete" data-ope="${barcode}"><i class="fa fa-times"></i></a>
                    <i id="${barcode}">${barcode}</i>
                    </li>`);

                $('#new-barcode').val('');
                $('#new-barcode-label').removeClass('text-danger');
                $('#new-barcode').removeClass('is-invalid');
                $("#newBarcode-btn").removeClass('hidden');
                $('#new-barcode-modal').modal('toggle');
            } else {
                $('#new-barcode').focus();
                $('#new-barcode-label').addClass('text-danger');
                $('#new-barcode').addClass('is-invalid');
            }
        });
        $('#add-sub-barcode-btn').on('click', function(e) {
            var unit_id = $('#sub_bar_unit_id').val();
            var barcode = $('#new-sub-barcode').val();

            var unit_value = $('#sub_bar_unit_value').val();
            var unit_price = $('#sub_bar_unit_price').val();

            if (!['', ' ', null].includes(barcode)) {
                $(`#list_bar-${unit_id}`).append(
                    `<input type="hidden" id="sub-${barcode}" name="sub_unit_barcode[]" value="${barcode}">`,
                    `<input type="hidden" id="sub-id-${unit_id}" name="sub_unit_badr_id[]" value="${unit_id}">`,
                    `<input type="hidden" id="sub-value-${unit_value}" name="sub_bar_unit_value[]" value="${unit_value}">`,
                    `<input type="hidden" id="sub-price-${unit_price}" name="sub_bar_unit_price[]" value="${unit_price}">`,
                    );

                $('.card').removeClass('hidden');
                $(`#list_bar-${unit_id}`).append(`<li class="external_barcode list-inline-item ml-1" id="subbarcode-${barcode}">
                    <a href="javascript:void(0)" class="text-danger sub_delete" data-ope="${barcode}"><i class="fa fa-times"></i></a>
                    <i id="${barcode}">${barcode}</i>
                    </li>`);

                $('#new-sub-barcode').val('');
                $('#sub_bar_unit_id').val('');
                $('#sub_bar_unit_value').val('');
                $('#sub_bar_unit_price').val('');
                $('#new-barcode-label').removeClass('text-danger');
                $('#new-sub-barcode').removeClass('is-invalid');
                $('#new-sub-barcode-modal').modal('toggle');
            } else {
                $('#new-sub-barcode').focus();
                $('#new-barcode-label').addClass('text-danger');
                $('#new-sub-barcode').addClass('is-invalid');
            }
        });
        $('#new-barcode-modal').on('hidden.bs.modal', function() {
            $('#new-barcode').val('');
            $('#new-barcode-label').removeClass('text-danger');
            $('#new-barcode').removeClass('is-invalid');
        });
        $(document).on('click', '.delete', function() {
            rel = $(this).data('ope');
            console.log(rel);
            return false;
            $('#h-' + rel).remove();
            $('#l-' + rel).remove();

            if ($(document).find('.external_barcode').length <= 0) {
                $(document).find('#newBarcode-btn').addClass('hidden');
                $(document).find('#newBarcode').addClass('hidden');
            }
        });
        $(document).on('click', '.sub_delete', function() {
            rel = $(this).data('ope');
            $('#sub-' + rel).remove();
            $('#subbarcode-' + rel).remove();
            if ($(document).find('.external_barcode').length <= 0) {
                $(document).find('.external_barcode').addClass('hidden');
            }
        });
        /*----------------------------------------------------------------------------------------------------------*/

        /*----------------------------------------------------------------------------------------------------------*/
        // Delete SubUnit
        $(document).on('click', '.delete-sub-unit', function() {
            barcode = $(this).data('barcode');
            unit = $(this).data('unit');

            $(`#${barcode}-${unit}`).remove();
            $(`#input-${barcode}-${unit}`).remove();

            var table_length = $('#sub_units >tbody >tr').length;
            table_length == 0 && $('#sub_units').addClass('hidden'); // If table is empty hide it
        });
        /*----------------------------------------------------------------------------------------------------------*/

        $(document).on('click', '#subcat-show', function() {
            if ($('#sale_unit').val() == '') {
                alert('@lang("items.select-category")');
            } else {
                var main_id = $('#sale_unit').val();
                getSubCats(main_id);
            }
        });

        $(document).on('click', '#intern-subcat-show', function() {
            if ($('#sale_unit').val() == '') {
                alert('@lang("items.select-category")');
            } else if ($('#sub-cat-items').val() == '' || $('#sub-cat-items').val() == null) {
                alert('@lang("items.select-sub-category")');
            } else {
                var sub_id = $('#sub-cat-items').val();
                getInternCats(sub_id);
            }
        });


        $(document).on('change', '#sale_unit', function() {
            var main_id = $('#sale_unit').val();
            if (main_id != '') {
                getSubCats(main_id);
            }
        });

        $(document).on('change', '#sub-cat-items', function() {
            var sub_id = $('#sub-cat-items').val();
            if (sub_id != '' || sub_id != null) {
                getInternCats(sub_id);
            }
        });

        function getSubCats(id) {
            data = new FormData();
            data.append('main_id', id);
            data.append('_token', '{{ csrf_token() }}');
            $.ajax({
                url: '{{ route('items.ajax.get-sub-cats') }}',
                type: "post",
                data: data,
                contentType: false,
                cache: false,
                processData: false, // <==
                timeout: 3000,
                beforeSend: function() {
                    $('#modal_client').empty().append(loader);
                },
                success: function(data) {
                    $('#subcat-div').empty().append(data.data);
                    $('#intern-subcat-div').removeClass('hidden');
                },
                error: function(xhr) {
                    var error = '<ul class="alert alert-danger">';
                    $.each(xhr.responseJSON.errors, function(index, value) {
                        error += '<li>' + value + '</li>';
                    });
                    error += "</ul>";
                    $('#currentErrorMessage').append(error);
                },
            });
        }

        function getInternCats(id) {
            data = new FormData();
            data.append('main_id', id);
            data.append('_token', '{{ csrf_token() }}');
            $.ajax({
                url: '{{ route('items.ajax.get-intern-sub-cats') }}',
                type: "post",
                data: data,
                contentType: false,
                cache: false,
                processData: false, // <==
                timeout: 3000,
                beforeSend: function() {
                    $('#modal_client').empty().append(loader);
                },
                success: function(data) {
                    $('#intern-subcat-div').empty().append(data.data);
                },
                error: function(xhr) {
                    var error = '<ul class="alert alert-danger">';
                    $.each(xhr.responseJSON.errors, function(index, value) {
                        error += '<li>' + value + '</li>';
                    });
                    error += "</ul>";
                    $('#currentErrorMessage').append(error);
                },
            });
        }

        $(document).on('click', '#add-sub-units', function() {
            $('#modal_unit_barcode').val('');
            $('#modal_unit_price').val('');
            $('#modal_unit_value').val('');
            $('#new-sub-unit-modal').modal('show');
            setTimeout(() => {
                // $('#new-sub-unit-modal').find('select').selectpicker('refresh');
            }, 200);
        });

        /********************************************************************************************************************************************/
        // New Sub Unit Modal
        $(document).on('click', '#add-new-sub-unit-btn', function(e) {
            e.preventDefault();
            var data = new FormData($('#new-sub-unit-form-modal')[0]);
            $('#show-modal-errors').addClass('hidden').empty();
            data.append('add_unit', true);
            data.append('_token', '{{ csrf_token() }}');

            $.ajax({
                url: "{{ route('items.ajax.get-sub-unit-row') }}",
                type: "post",
                data: data,
                contentType: false,
                cache: false,
                processData: false, // <==
                beforeSend: function() {
                    $('#modal_client').empty().append(loader);
                },
                success: function(data) {
                    $('#sub_units >tbody >tr').length == 0 && $('#sub_units').removeClass('hidden');
                    $('#sub_units').append(data.data);
                    $('#sub_units_inputs').append(data.hidden);
                    $('#new-sub-unit-modal').modal('toggle');
                    syncUnitQty();
                },
                error: function(xhr) {
                    var error = '<ul class="list-unstyled mb-0 pr-0">';
                    $.each(xhr.responseJSON.errors, function(index, value) {
                        error += '<li>' + value + '</li>';
                    });
                    error += "</ul>";

                    var unit = $("#unit").val();
                    var unit_value = $("#modal_unit_value").val();
                    var unit_price = $("#modal_unit_price").val();
                    var unit_barcode = $("#modal_unit_barcode").val();

                    if (['', ' ', null].includes(unit)) {
                        $("#unit").focus();
                        $("#unit").addClass('is-invalid');
                        $("label[for='unit']").addClass('text-danger');
                    } else {
                        $("#unit").removeClass('is-invalid');
                        $("label[for='unit']").removeClass('text-danger');
                    }

                    if (['', ' ', null].includes(unit_value)) {
                        $("#modal_unit_value").focus();
                        $("#modal_unit_value").addClass('is-invalid');
                        $("label[for='modal_unit_value']").addClass('text-danger');
                    } else {
                        $("#modal_unit_value").removeClass('is-invalid');
                        $("label[for='modal_unit_value']").removeClass('text-danger');
                    }

                    if (['', ' ', null].includes(unit_price)) {
                        $("#modal_unit_price").focus();
                        $("#modal_unit_price").addClass('is-invalid');
                        $("label[for='modal_unit_price']").addClass('text-danger');
                    } else {
                        $("#modal_unit_price").removeClass('is-invalid');
                        $("label[for='modal_unit_price']").removeClass('text-danger');
                    }

                    if (['', ' ', null].includes(unit_barcode)) {
                        $("#modal_unit_barcode").focus();
                        $("#modal_unit_barcode").addClass('is-invalid');
                        $("label[for='modal_unit_barcode']").addClass('text-danger');
                    } else {
                        $("#modal_unit_barcode").removeClass('is-invalid');
                        $("label[for='modal_unit_barcode']").removeClass('text-danger');
                    }

                    $('#show-modal-errors').removeClass('hidden').empty().addClass('alert alert-danger').append(error);
                }
            });
        });

        $('#new-sub-unit-modal').on('hidden.bs.modal', function() {
            $("#unit").val('');
            $("#unit").removeClass('is-invalid');
            $("label[for='unit']").removeClass('text-danger');

            $("#modal_unit_value").val('');
            $("#modal_unit_value").removeClass('is-invalid');
            $("label[for='modal_unit_value']").removeClass('text-danger');

            $("#modal_unit_price").val('');
            $("#modal_unit_price").removeClass('is-invalid');
            $("label[for='modal_unit_price']").removeClass('text-danger');

            $("#modal_unit_barcode").val('');
            $("#modal_unit_barcode").removeClass('is-invalid');
            $("label[for='modal_unit_barcode']").removeClass('text-danger');
        });
        /********************************************************************************************************************************************/

        /********************************************************************************************************************************************/
        // New Company Modal
        $('#new-company-form-modal').on('submit', function(e) {
            e.preventDefault();
            data = new FormData($('#new-company-form-modal')[0]);
            $.ajax({
                url: '{{ route('items.company.store') }}',
                type: "post",
                data: data,
                contentType: false,
                cache: false,
                processData: false, // <==
                beforeSend: function() {
                    $('#modal_client').empty().append(loader);
                },
                success: function(data) {
                    $(document).find('#company_id').prev('div').find('ul').append(data.list);
                    $(document).find('#company_id').append(data.option);
                    $('.selectpicker').selectpicker('refresh', {
                        noneSelectedText: "{{ trans('native.no-selected') }}"
                    });
                    $('#messages').removeClass('hidden').empty().addClass('alert alert-success').append(
                        "{{ __('extends.save_sucessfully') }}");
                    $('#new-company-modal').modal('toggle');
                },
                error: function(xhr) {
                    var error = '<ul class="list-unstyled mb-0">';
                    $.each(xhr.responseJSON.errors, function(index, value) {
                        error += '<li>' + value + '</li>';
                    });
                    error += "</ul>";

                    var company_name = $("#company_name").val();
                    var income_ration = $("#income_ration").val();
                    var sale_ratio = $("#sale_ratio").val();

                    if (['', ' ', null].includes(company_name)) {
                        $("#company_name").focus();
                        $("#company_name").addClass('is-invalid');
                        $("label[for='company_name']").addClass('text-danger');
                    } else {
                        $("#company_name").removeClass('is-invalid');
                        $("label[for='company_name']").removeClass('text-danger');
                    }

                    if (['', ' ', null].includes(income_ration)) {
                        $("#income_ration").focus();
                        $("#income_ration").addClass('is-invalid');
                        $("label[for='income_ration']").addClass('text-danger');
                    } else {
                        $("#income_ration").removeClass('is-invalid');
                        $("label[for='income_ration']").removeClass('text-danger');
                    }

                    if (['', ' ', null].includes(sale_ratio)) {
                        $("#sale_ratio").focus();
                        $("#sale_ratio").addClass('is-invalid');
                        $("label[for='sale_ratio']").addClass('text-danger');
                    } else {
                        $("#sale_ratio").removeClass('is-invalid');
                        $("label[for='sale_ratio']").removeClass('text-danger');
                    }

                    $('#new-company-modal').modal('toggle');
                    $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(
                        error);
                }
            });
        });

        $('#new-company-modal').on('hidden.bs.modal', function() {
            $("#company_name").val('');
            $("#company_name").removeClass('is-invalid');
            $("label[for='company_name']").removeClass('text-danger');

            $("#income_ration").val('');
            $("#income_ration").removeClass('is-invalid');
            $("label[for='income_ration']").removeClass('text-danger');

            $("#sale_ratio").val('');
            $("#sale_ratio").removeClass('is-invalid');
            $("label[for='sale_ratio']").removeClass('text-danger');
        });
        /********************************************************************************************************************************************/

        /********************************************************************************************************************************************/
        // New Item Color Modal
        $('#new-color-form-modal').on('submit', function(e) {
            e.preventDefault();
            data = new FormData($('#new-color-form-modal')[0]);
            $.ajax({
                url: '{{ route('items.color.store') }}',
                type: "post",
                data: data,
                contentType: false,
                cache: false,
                processData: false,
                timeout: 3000,
                beforeSend: function() {
                    $('#modal_client').empty().append(loader);
                },
                success: function(data) {
                    $(document).find('#color_id').prev('div').find('ul').append(data.list);
                    $(document).find('#color_id').append(data.option);
                    $('.selectpicker').selectpicker('refresh', {
                        noneSelectedText: "{{ trans('native.no-selected') }}"
                    });
                    $('#messages').removeClass('hidden').empty().addClass('alert alert-success').append(
                        "{{ __('extends.save_sucessfully') }}");
                    $('#new-color-modal').modal('toggle');
                },
                error: function(xhr) {
                    var error = '<ul class="list-unstyled mb-0 pr-0">';
                    $.each(xhr.responseJSON.errors, function(index, value) {
                        error += '<li>' + value + '</li>';
                    });
                    error += "</ul>";

                    var color_name = $("#color_name").val();
                    var color_code = $("#color_code").val();

                    if (['', ' ', null].includes(color_name)) {
                        $("#color_name").focus();
                        $("#color_name").addClass('is-invalid');
                        $("label[for='color_name']").addClass('text-danger');
                    } else {
                        $("#color_name").removeClass('is-invalid');
                        $("label[for='color_name']").removeClass('text-danger');
                    }

                    if (['', ' ', null].includes(color_code)) {
                        $("#color_code").focus();
                        $("#color_code").addClass('is-invalid');
                        $("label[for='color_code']").addClass('text-danger');
                    } else {
                        $("#color_code").removeClass('is-invalid');
                        $("label[for='color_code']").removeClass('text-danger');
                    }

                    $('#new-color-modal').modal('toggle');
                    $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(
                        error);
                }
            });
        });

        $('#new-color-modal').on('hidden.bs.modal', function() {
            $("#color_name").val('');
            $("#color_name").removeClass('is-invalid');
            $("label[for='color_name']").removeClass('text-danger');

            $("#color_code").val('');
            $("#color_code").removeClass('is-invalid');
            $("label[for='color_code']").removeClass('text-danger');
        });
        /********************************************************************************************************************************************/

        /********************************************************************************************************************************************/
        // New Item Size Modal
        $('#new-size-form-modal').on('submit', function(e) {
            e.preventDefault();
            data = new FormData($('#new-size-form-modal')[0]);
            $.ajax({
                url: '{{ route('items.size.store') }}',
                type: "post",
                data: data,
                contentType: false,
                cache: false,
                processData: false, // <==
                timeout: 3000,
                beforeSend: function() {
                    $('#modal_client').empty().append(loader);
                },
                success: function(data) {
                    $(document).find('#size_id').prev('div').find('ul').append(data.list);
                    $(document).find('#size_id').append(data.option);
                    $('.selectpicker').selectpicker('refresh', {
                        noneSelectedText: "{{ trans('native.no-selected') }}"
                    });
                    $('#messages').removeClass('hidden').empty().addClass('alert alert-success').append(
                        "{{ __('extends.save_sucessfully') }}");
                    $('#new-size-modal').modal('toggle');
                },
                error: function(xhr) {
                    var error = '<ul class="alert alert-danger">';
                    $.each(xhr.responseJSON.errors, function(index, value) {
                        error += '<li>' + value + '</li>';
                    });
                    error += "</ul>";

                    var size_name = $("#size_name").val();
                    var size_code = $("#size_code").val();

                    if (['', ' ', null].includes(size_name)) {
                        $("#size_name").focus();
                        $("#size_name").addClass('is-invalid');
                        $("label[for='size_name']").addClass('text-danger');
                    } else {
                        $("#size_name").removeClass('is-invalid');
                        $("label[for='size_name']").removeClass('text-danger');
                    }

                    if (['', ' ', null].includes(size_code)) {
                        $("#size_code").focus();
                        $("#size_code").addClass('is-invalid');
                        $("label[for='size_code']").addClass('text-danger');
                    } else {
                        $("#size_code").removeClass('is-invalid');
                        $("label[for='size_code']").removeClass('text-danger');
                    }

                    $('#new-size-modal').modal('toggle');
                    $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(
                        error);
                }
            });
        });

        $('#new-size-modal').on('hidden.bs.modal', function() {
            $("#size_name").val('');
            $("#size_name").removeClass('is-invalid');
            $("label[for='size_name']").removeClass('text-danger');

            $("#size_code").val('');
            $("#size_code").removeClass('is-invalid');
            $("label[for='size_code']").removeClass('text-danger');
        });
        /********************************************************************************************************************************************/

        $(document).on('submit', '#new_item_form', function(e) {
            e.preventDefault();

            if ($('#item_name').val() == '' || $('#item_name').val() == null) {
                e.preventDefault();
                $('#item_name').focus();
                $('#item_name:focus').addClass('invalid');
            } else {

                $('#item_name').removeClass('invalid');
            }

            if ($('#category').val() == '' || $('#category').val() == null) {
                e.preventDefault();
                $('select#category').focus();
                $('select#category').closest('.bootstrap-select').find('button').addClass('invalid');
            } else {

                $('select#category').closest('.bootstrap-select').find('button').removeClass('invalid');
            }

            if ($('#min_quantity').val() == '' || $('#min_quantity').val() == null) {
                e.preventDefault();
                $('#min_quantity').focus();
                $('#min_quantity:focus').addClass('invalid');
            } else {

                $('#min_quantity').removeClass('invalid');
            }

            if ($('#pay_date').val() == '' || $('#pay_date').val() == null) {
                e.preventDefault();
                $('#pay_date').focus();
                $('#pay_date:focus').addClass('invalid');
            } else {

                $('#pay_date').removeClass('invalid');
            }

            if ($('#lowest_price').val() == '' || $('#lowest_price').val() == null) {
                e.preventDefault();
                $('#lowest_price').focus();
                $('#lowest_price:focus').addClass('invalid');
            } else {

                $('#lowest_price').removeClass('invalid');
            }

            if ($('#barcode').val() == '' || $('#barcode').val() == null) {
                e.preventDefault();
                $('#barcode').focus();
                $('#barcode:focus').addClass('invalid');
            } else {

                $('#barcode').removeClass('invalid');
            }

            if ($('#supplier_id').val() == '' || $('#supplier_id').val() == null) {
                e.preventDefault();
                $('select#supplier_id').focus();
                $('select#supplier_id').closest('.bootstrap-select').find('button').addClass('invalid');
            } else {

                $('select#supplier_id').closest('.bootstrap-select').find('button').removeClass('invalid');
            }

            if ($('#quantity').val() == '' || $('#quantity').val() == null) {
                e.preventDefault();
                $('#quantity').focus();
                $('#quantity').addClass('invalid');
            } else {

                $('#quantity').removeClass('invalid');
            }
            /*
                        if ($('select#sale_unit').val() == '' || $('select#sale_unit').val() == null) {
                            e.preventDefault();
                            // $('#sale_unit').focus().select();
                            // $('select#sale_unit').closest('.bootstrap-select').find('button').addClass('invalid');
                            $('#sale_unit_div').find("button").addClass('invalid');
                        } else {

                            // $('select#sale_unit').closest('.bootstrap-select').find('button').removeClass('invalid');
                            $('#sale_unit_div').find("button").removeClass('invalid');
                        }
            */

            if ($.trim($('#sale_unit_div').find("button").text()) == 'Choose one') {
                e.preventDefault();
                $('#sale_unit_div').find("button").addClass('invalid');
            } else {
                $('#sale_unit_div').find("button").removeClass('invalid');
            }

            if ($('#unit_id').val() == '' || $('#unit_id').val() == null) {
                e.preventDefault();
                $('#unit_id').focus().select();
                $('select#unit_id').closest('.bootstrap-select').find('button').addClass('invalid');

            } else {

                $('select#unit_id').closest('.bootstrap-select').find('button').removeClass('invalid');
            }

            if ($('#pay_price').val() == '' || $('#pay_price').val() == null) {
                e.preventDefault();
                $('#pay_price').focus();
                $('#pay_price').addClass('invalid');
            } else {
                $('#pay_price').removeClass('invalid');
            }

            if ($('#sale_price').val() == '' || $('#sale_price').val() == null) {
                e.preventDefault();
                $('#sale_price').focus();
                $('#sale_price').addClass('invalid');
            } else {
                $('#sale_price').removeClass('invalid');
            }
        });

        /********************************************************************************************************************************************/
        // New Shelf Modal
        $(document).on('click', '#new-shelf-btn', function(e) {
            e.preventDefault();
            $('#new-shelf-modal').modal('show');
        });

        $('#new-shelf-form-modal').on('submit', function(e) {
            e.preventDefault();
            data = new FormData($('#new-shelf-form-modal')[0]);
            $.ajax({
                url: '{{ route('items.shelf.store') }}',
                type: "post",
                data: data,
                contentType: false,
                cache: false,
                processData: false, // <==
                timeout: 3000,
                beforeSend: function() {
                    $('#modal_client').empty().append(loader);
                },
                success: function(data) {
                    $(document).find('#shelf').prev('div').find('ul').append(data.list);
                    $(document).find('#shelf').append(data.option);
                    $('.selectpicker').selectpicker('refresh', {
                        noneSelectedText: "{{ trans('native.no-selected') }}"
                    });
                    $('#messages').removeClass('hidden').empty().addClass('alert alert-success').append(
                        "{{ __('extends.save_sucessfully') }}");
                    $('#new-shelf-modal').modal('toggle');
                },
                error: function(xhr) {
                    var error = '<ul class="list-unstyled mb-0 pr-0">';
                    $.each(xhr.responseJSON.errors, function(index, value) {
                        error += '<li>' + value + '</li>';
                    });
                    error += "</ul>";

                    var shelf_name = $("input[name='shelf_name']").val();
                    if (['', ' ', null].includes(shelf_name)) {
                        $("input[name='shelf_name']").focus();
                        $('label[for="shelf_name"]').addClass('text-danger');
                        $("input[name='shelf_name']").addClass('is-invalid');
                    } else {
                        $("input[name='shelf_name']").removeClass('is-invalid');
                        $('label[for="shelf_name"]').removeClass('text-danger');
                    }

                    $('#new-shelf-modal').modal('toggle');
                    $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(
                        error);
                }
            });
        });

        $('#new-shelf-modal').on('hidden.bs.modal', function() {
            $("input[name='shelf_name']").val('');
            $("input[name='shelf_name']").removeClass('is-invalid');
            $('label[for="shelf_name"]').removeClass('text-danger');
        });

        /********************************************************************************************************************************************/
    </script>

    <script>
        $('.selectpicker').selectpicker({
            size: 8,
            noneSelectedText: '{{ trans('native.no-selected') }}'

        });
    </script>


    <script>
        var loadFile = function(event) {
            var output = document.getElementById('output');
            output.src = URL.createObjectURL(event.target.files[0]);
        };
    </script>



    <script src="{{ asset('tickets/js/jscolor.js') }}"></script>
    <script src="{{ asset('tickets/scanner.js') }}"></script>
<script>
     $(document).scannerDetection({

timeBeforeScanTest: 2000, // wait for the next character for upto 200ms
avgTimeByChar: 40, // it's not a barcode if a character takes longer than 100ms
// preventDefault: false,

endChar: [13],
onKeyDetect: function (barcode, qty) {
    validScan = true;
} // main callback function	,
,
onComplete: function (barcode) {
    return false;
},
onError: function (string, qty) {
   return false;
}

});

</script>

@stop
