@extends('master::layouts.master')
@section('content')
<div class="card card-light mb-1_5 p-3">
   <div class="container-fluid mb-3">
      <div class="row mb-3">
         <h1 class="text-info">{{ __('items.edit_item') . ' : ' . $item->item_name }}</h1>
      </div>
      <div class="row">
         <form id="edit_item_form" class="form-horizontal w-100 custome-form" action="{{ route('items.update', [$item->id]) }}"
            method="post" enctype="multipart/form-data">
            @csrf
            {{-- Item ID --}}
            <input name="item_id" type="hidden" value="{{ $item->id }}">
            {{-- Item Name & About --}}
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="item_name">{{ __('items.name') }} :</label>
                  <input type="text" class="form-control col-md-12 input-normal" name="item_name" id="item_name"
                     value="{{ old('item_name') ?? $item->item_name }}">
               </div>
               <div class="form-group col-md-6">
                  <label class="control-label" for="about">{{ __('items.about') }} :</label>
                  <input type="text" class="form-control col-md-12 input-normal" name="about" id="about"
                     value="{{ old('about') ?? $item->about }}">
               </div>
            </div>
            {{-- Item Quantity & Min Quantity --}}
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="quantity">{{ __('items.quantity.name') }} :</label>
                  <input type="number" class="form-control col-md-12 input-normal" step="0.01" name="quantity" id="quantity"
                     value="{{ old('quantity') ?? (count($item->storeQty) >= 1 ? $item->storeQty->first()->store_quant : quant_decimal(0)) }}">
                  <small class="text-danger @if (count($item->storeQty) == 0) hidden @endif"
                     id="store_note">{{ count($item->storeQty) >= 1 ? __('items.store_note', ['store' => $item->storeQty->first()->store->store_name]) : '' }}</small>
               </div>
               <div class="form-group col-md-6">
                  <label class="control-label" for="min_quantity">{{ __('items.min_quantity') }} :</label>
                  <input type="number" step="0.01" class="form-control col-md-12 input-normal" name="min_quantity"
                     id="min_quantity" value="{{ old('min_quantity') ?? $item->min_quantity }}">
               </div>
            </div>
            {{-- Item Store --}}
            <div class="form-row row">
               <div class="form-group col-md-6 @if (count($item->storeQty) == 0) hidden @endif" id="store_div">
                  <label class="control-label" for="stores">{{ __('native.store') }} :</label>
                  <select class="form-control selectpicker col-md-12 input-normal" name="stores" id="stores"
                     data-none-selected-text="{{ trans('native.no-selected') }}"
                     data-store_count="{{ count($item->storeQty) }}" title="{{ __('items.choose_item_store') }}">
                  @foreach ($stores as $store)
                  <option {{ old('stores') == $store->id || $loop->first ? 'selected' : '' }}
                  data-quant="{{ $store->items->first()->pivot->store_quant ?? quant_decimal(0) }}"
                  data-store_name="{{ $store->store_name }}" value="{{ $store->id }}">
                  {{ $store->store_name }}</option>
                  @endforeach
                  </select>
               </div>
            </div>
            {{-- Item Pay Price --}}
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="pay_price">{{ __('items.pay_price.name') }} :</label>
                  <input type="number" step="0.01" class="form-control col-md-12 input-normal" name="pay_price" id="pay_price"
                     value="{{ old('pay_price') ?? $item->pay_price }}">
               </div>
               <div class="form-group col-md-6">
                  <label class="control-label" for="pay_date">{{ __('items.purchase_date') }} :</label>
                  <input type="text" class="form-control col-md-12 date input-normal" name="pay_date" id="pay_date"
                     value="{{ old('pay_date') ?? $item->pay_date }}">
               </div>
            </div>
            {{-- Item Sale Price --}}
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="sale_price">{{ __('items.sale_price.name') }} :</label>
                  <input type="number" step="0.01" class="form-control col-md-12 input-normal" name="sale_price"
                     id="sale_price" value="{{ old('sale_price') ?? $item->sale_price }}">
               </div>
               <div class="form-group col-md-6">
                  <label class="control-label" for="lowest_price">{{ __('items.min_sale_price') }} :</label>
                  <input type="number" step="0.01" class="form-control col-md-12 input-normal" name="lowest_price"
                     id="lowest_price" value="{{ old('lowest_price') ?? $item->lowest_price }}">
               </div>
            </div>
            {{-- price_list --}}
            @if (get_session_val('multi_price', 'Integer') == 1)
            <div class="form-row row">
               <h6 class="col-md-6">
                  {{ __('multiPrices.price_lists') }}
               </h6>
            </div>
            <div class="form-row row">
               @foreach ($price_lists as $price_list)
               @php
               $current_list = $item_prices->where('list_id', $price_list->id)->first();
               $current_price = $current_list->price ?? 0;
               $current_qty = $current_list->list_quant ?? 0;
               @endphp
               <div class="form-group col-md-3 form-inline">
                  <label class="control-label col-md-4 ">{{ $price_list->list_name }} :</label>
                  <input type="number" class="hidden" name="item_prices_ids[]"
                     value="{{ $price_list->id }}">
                  <input type="text" class="form-control col-md-8 list_price use-only-numbers input-normal" name="item_prices[]"
                     data-list="{{ $price_list->id }}"
                     value="{{ old('item_prices')[$loop->index] ?? $current_price }}">
               </div>
               <div class="form-group col-md-3 form-inline">
                  <label class="control-label col-md-4">{{ __('multiPrices.required_qty') }} :</label>
                  <input type="text" class="form-control use-only-numbers col-md-8 list_price_qty input-normal"
                     name="item_prices_qtys[]" data-list="{{ $price_list->id }}"
                     value="{{ old('item_prices_qtys')[$loop->index] ?? $current_qty }}">
               </div>
               @endforeach
            </div>
            @endif
            {{-- Item Barcode --}}
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="barcode">{{ __('items.barcode.name') }} :</label>
                  <input type="number" class="form-control col-md-12 input-normal" name="barcode[]" id="barcode"
                     value="{{ old('barcode')[0] ?? $item->barcode }}">
               </div>
               <div class="form-group col-md-6 align-self-end">
                  <button type="button" id="new-barcode-btn" class="btn btn-info col-md-6" data-toggle="modal"
                     data-target="#new-barcode-modal">
                  <i class="fa fa-plus-circle"></i> {{ __('items.add_barcode') }}
                  </button>
                  <button class="btn btn-primary col-md-5 mr-1 @if ($sub_units_bars->count() <= 0) hidden @endif"
                     type="button" data-toggle="collapse" id="newBarcode-btn" data-target="#newBarcode"
                     aria-expanded="false" aria-controls="newBarcode"><i class="fa fa-barcode"></i>
                  {{ __('items.show_barcodes') }}</button>
                  <div class="collapse" id="newBarcode">
                     <div class="card card-body">
                        <ul class="list-inline barcode-list p-0 m-0">
                           @foreach ($sub_units_bars as $sub_unit_item)
                           <li class="external_barcode list-inline-item ml-1"
                              id="l-{{ $sub_unit_item->unit_barcode }}">
                              @if ($sub_unit_item->isFree())
                              <a href="javascript:void(0)" class="text-danger delete"
                                 data-ope="{{ $sub_unit_item->unit_barcode }}"><i
                                 class="fa fa-times"></i></a>
                              @endif
                              <i id="' + $('#new-barcode').val() + '">{{ $sub_unit_item->unit_barcode }}</i>
                           </li>
                           @endforeach
                        </ul>
                        <div id="barcodes">
                           @foreach ($sub_units_bars as $sub_unit_i)
                           <input type="hidden" id="h-{{ $sub_unit_i->unit_barcode }}" name="barcode[]"
                              value="{{ $sub_unit_i->unit_barcode }}-{{ $sub_unit_i->id }}">
                           @endforeach
                        </div>
                        <div id="new-barcodes"></div>
                     </div>
                  </div>
               </div>
            </div>
            {{-- Item E-Invoice --}}
            @if ($settings->einvoice == 1)
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="unit_einvoice_code">{{ __('items.unit_einvoice_code') }}
                  :</label>
                  <input type="text" class="form-control col-md-12 input-normal" name="unit_einvoice_code"
                     id="unit_einvoice_code"
                     value="{{ old('unit_einvoice_code') ?? $item->unit_einvoice_code }}" readonly>
               </div>
               <div class="form-group col-md-6">
                  <label class="control-label" for="item_einvoice_code">{{ __('items.item_einvoice_code') }}
                  :</label>
                  <div class="input-group">
                     <input type="text" class="form-control col-md-9 input-normal" name="item_einvoice_code"
                        id="item_einvoice_code"
                        value="{{ old('item_einvoice_code') ?? $item->item_einvoice_code }}"
                        style="border-left: 0; border-radius: 0 8px 8px 0 !important;">
                     <input type="text" class="form-control col-md-3 input-normal"
                        value="{{ '-EG-' . ($einvoice_settings->registration_id ?? '') }}" disabled
                        style="border-right: 0; border-radius: 8px 0 0 8px; cursor: context-menu">
                  </div>
               </div>
            </div>
            @endif
            {{-- GPC Code && Item Expiry Date --}}
            <div class="form-row row">
               @if ($settings->einvoice == 1)
               <div class="form-group col-md-6">
                  <label class="control-label" for="gpc_code">{{ __('items.gpc') }}
                  :</label>
                  <input type="text" class="form-control col-md-12 input-normal" name="parent_code" id="gpc_code"
                     value="{{ old('parent_code') ?? $item->parent_code }}">
               </div>
               @endif
               <div class="form-group col-md-6">
                  <label class="control-label" for="expire_date">{{ __('items.expire_date') }} :</label>
                  <input type="text" class="form-control col-md-12 input-normal date_" name="expire_date" id="expire_date"
                     value="{{ old('expire_date') ?? $item->expire_date }}">
               </div>
            </div>
            {{-- Main Unit && Item Category --}}
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="unit_id">{{ __('items.main_unit') }} :</label>
                  <select data-live-search="true" id="unit_id" class="form-control input-normal selectpicker col-md-12"
                     data-none-selected-text="{{ trans('native.no-selected') }}" name="unit_id">
                     @foreach ($units as $unit_)
                     @if ($unit_->id == $item->unit_id)
                     <option selected value="{{ $unit_->id }}" data-name="{{ $unit_->name }}"
                        data-unit_einvoice_code="{{ $unit_->unit_einvoice_code }}">
                        {{ $unit_->name }}
                     </option>
                     @else
                     <option value="{{ $unit_->id }}" data-name="{{ $unit_->name }}"
                        data-unit_einvoice_code="{{ $unit_->unit_einvoice_code }}">
                        {{ $unit_->name }}
                     </option>
                     @endif
                     @endforeach
                  </select>
               </div>
               <div class="form-group col-md-6" id="sale_unit_div">
                  <label class="control-label" for="sale_unit">{{ __('items.sale_unit.name') }} :</label>
                  <select class="form-control selectpicker input-normal col-md-12" name="sale_unit" id="sale_unit"
                     data-none-selected-text="{{ trans('native.no-selected') }}"
                     title="{{ __('items.choose_item_cat') }}">
                     @foreach ($main_cats as $cat)
                     @if ($cat->id == (old('sale_unit') ?? $item->sale_unit))
                     <option selected value="{{ $cat->id }}">{{ $cat->name }}</option>
                     @else
                     <option value="{{ $cat->id }}">{{ $cat->name }}</option>
                     @endif
                     {{ getSubCatOptionsWithSelected($cat->id, old('sale_unit') ?? $item->sale_unit) }}
                     @endforeach
                  </select>
               </div>
            </div>
            {{-- Supplier && Shelf --}}
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="supplier_id">{{ __('items.supplier') }} :</label>
                  <select data-live-search="true" multiple class="input-normal form-control col-md-12 selectpicker"
                     name="supplier_id[]" id="supplier_id" title="{{ __('items.chooseSupplier') }}">
                     @foreach ($suppliers as $supplier)
                     @if (in_array($supplier->id, $suppliers_array))
                     <option selected value="{{ $supplier->id }}">
                        {{ $supplier->client_name }}
                     </option>
                     @else
                     <option value="{{ $supplier->id }}" {{ $loop->first ? 'selected' : '' }}>
                     {{ $supplier->client_name }}
                     </option>
                     @endif
                     @endforeach
                  </select>
               </div>
               {{--
               <div class="form-group col-md-6">
                  <label class="control-label">{{ __('items.store_shelf') }} :</label>
                  <select id="shelf" class="form-control col-md-12 input-normal selectpicker" data-live-search="true"
                     name="shelf" title="{{ __('items.chooseShelf') }}">
                     @foreach ($shelves as $shelf)
                     @if ($shelf->id == (old('shelf') ?? $item->shelf))
                     <option selected value="{{ $shelf->id }}">{{ $shelf->shelf_name }}
                     </option>
                     @else
                     <option value="{{ $shelf->id }}">{{ $shelf->shelf_name }}</option>
                     @endif
                     @endforeach
                  </select>
               </div>
               --}}
            </div>
            {{-- Item Vats --}}
            {{-- @if ($settings->einvoice == 1) --}}
            {{--
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="einvoice_vat_state">{{ __('items.einvoice_vat_state') }}
                  :</label>
                  <select class="col-md-12 form-control input-normal selectpicker" name="einvoice_vat_state" data-none-selected-text="{{ trans('native.no-selected') }}"
                     id="einvoice_vat_state">
                  <option value="0" {{ old('einvoice_vat_state') == 0 || $item->vat_state == 0 || $item->purchase_vat_state == 0 ? 'selected' : '' }}
                  id="einvoice_vat_state_exempt">
                  {{ __('items.exempt') }}</option>
                  <option value="1" {{ old('einvoice_vat_state') == 1 || ($item->vat_state == 1 && $item->purchase_vat_state == 1) ? 'selected' : '' }}
                  id="einvoice_vat_state_not_include">{{ __('items.not_include') }}</option>
                  <option value="2" {{ old('einvoice_vat_state') == 2 || ($item->vat_state == 2 && $item->purchase_vat_state == 2) ? 'selected' : '' }}
                  id="einvoice_vat_state_include">
                  {{ __('items.include') }}</option>
                  </select>
               </div>
               <div class="form-group col-md-6 @if ($item->vat_state == 0 && $item->purchase_vat_state == 0) hidden @endif" id="vat_id_div">
                  <label class="control-label" for="vat_id">{{ __('items.item_taxs') }}
                  :</label>
                  <select data-live-search="true" id="vat_id" class="input-normal form-control selectpicker col-md-12" data-none-selected-text="{{ trans('native.no-selected') }}"
                     name="vats_ids[]" title="{{ __('items.choose_tax') }}" multiple>
                     @foreach ($vats as $vat)
                     @if (in_array($vat->id, $vat_array))
                     <option selected value="{{ $vat->id . '-' . $vat->vat_operation }}"
                        data-name="{{ $vat->Addition_name }}"
                        data-value="{{ $vat->addition_value }}">{{ $vat->Addition_name }}
                     </option>
                     @else
                     <option value="{{ $vat->id . '-' . $vat->vat_operation }}"
                        data-name="{{ $vat->Addition_name }}"
                        data-value="{{ $vat->addition_value }}">{{ $vat->Addition_name }}
                     </option>
                     @endif
                     @endforeach
                  </select>
               </div>
            </div>
            --}}
            {{-- @else --}}
            {{-- Sales Item Vat --}}
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="sales_vat_select">{{ __('items.sales_item_vats_type') }}
                  :</label>
                  <select class="col-md-12 form-control input-normal selectpicker" name="vat_state" id="sales_vat_select"
                     data-none-selected-text="{{ trans('native.no-selected') }}">
                  <option value="0" @if ((old('vat_state') ?? $item->vat_state) == 0) selected @endif id="sales_exempt">
                  {{ __('items.exempt') }}</option>
                  <option value="1" @if ((old('vat_state') ?? $item->vat_state) == 1) selected @endif
                  id="sales_not_include">{{ __('items.not_include') }}</option>
                  <option value="2" @if ((old('vat_state') ?? $item->vat_state) == 2) selected @endif id="sales_include">
                  {{ __('items.include') }}</option>
                  </select>
               </div>
               <div class="form-group col-md-6 @if ($item->vat_state == 0) hidden @endif"
                  id="sales_item_vats">
                  <label class="control-label" for="sales_vat_id">{{ __('items.sales_item_vats') }}
                  :</label>
                  <select data-live-search="true" id="sales_vat_id" multiple
                     class="form-control input-normal selectpicker col-md-12" name="vat_ids[]"
                     data-none-selected-text="{{ trans('native.no-selected') }}">
                  @foreach ($vats as $vat)
                  <option value="{{ $vat->id . '-' . $vat->vat_operation }}"
                  data-name="{{ $vat->Addition_name }}" data-value="{{ $vat->addition_value }}"
                  @if (in_array($vat->id, $sales_vat) || (old('vat_ids') && in_array($vat->id, old('vat_ids')))) selected @endif>{{ $vat->Addition_name }}
                  </option>
                  @endforeach
                  </select>
               </div>
            </div>
            {{-- Purchase Item Vat --}}
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="purchase_vat_select">{{ __('items.purchase_item_vats') }}
                  :</label>
                  <select class="col-md-12 form-control selectpicker input-normal" name="purchase_vat_state"
                     data-none-selected-text="{{ trans('native.no-selected') }}" id="purchase_vat_select">
                  <option value="0" @if ((old('purchase_vat_state') ?? $item->purchase_vat_state) == 0) selected @endif
                  id="purchase_exempt">{{ __('items.exempt') }}</option>
                  <option value="1" @if ((old('purchase_vat_state') ?? $item->purchase_vat_state) == 1) selected @endif
                  id="purchase_not_include">{{ __('items.not_include') }}</option>
                  <option value="2" @if ((old('purchase_vat_state') ?? $item->purchase_vat_state) == 2) selected @endif
                  id="purchase_include">{{ __('items.include') }}</option>
                  </select>
               </div>
               <div class="form-group col-md-6 @if ($item->purchase_vat_state == 0) hidden @endif"
                  id="purcahse_item_vats">
                  <label class="control-label" for="purcahse_vat_id">{{ __('items.purchase_item_vats') }}
                  :</label>
                  <select data-live-search="true" id="purcahse_vat_id" class="form-control input-normal selectpicker col-md-12"
                     name="purchase_vat_ids[]" multiple
                     data-none-selected-text="{{ trans('native.no-selected') }}">
                  @foreach ($vats as $vat)
                  <option value="{{ $vat->id . '-' . $vat->vat_operation }}"
                  data-name="{{ $vat->Addition_name }}" data-value="{{ $vat->addition_value }}"
                  @if (in_array($vat->id, $purchase_vat) || (old('purchase_vat_ids') && in_array($vat->id, old('purchase_vat_ids')))) selected @endif>{{ $vat->Addition_name }}
                  </option>
                  @endforeach
                  </select>
               </div>
            </div>
            {{-- @endif --}}
            {{-- Item Company --}}
            {{-- @if ($settings->items_company != 0)
            <div class="form-row row">
               <div class="form-group col-md-6">
                  <label class="control-label" for="company_id">{{ __('items.variable_price_list') }} :</label>
                  <select data-live-search="true" class="form-control input-normal col-md-12 selectpicker" name="company_id"
                     id="company_id" title="{{ __('items.choose_variable_list') }}">
                     @foreach ($companies as $company)
                     @if ($company->id == $item->company_id)
                     <option selected value="{{ $company->id }}">
                        {{ $company->company_name }}
                     </option>
                     @else
                     <option value="{{ $company->id }}">{{ $company->company_name }}
                     </option>
                     @endif
                     @endforeach
                  </select>
               </div>
               <div class="form-group col-md-6 d-flex flex-column justify-content-end">
                  <button type="button" id="new-company-btn" class="btn btn-info col-md-6"
                     data-toggle="modal" data-target="#new-company-modal">
                  <i class="fa fa-plus-circle"></i> {{ __('items.add_variable_price_list') }}
                  </button>
               </div>
            </div>
            @endif --}}
            {{-- Item Vats --}}
            <div class="row mt-3">
               <div class="col-md-12">
                  <div class="table-responsive">
                     <table
                        class="table  zui-gray-table zui-table-rounded  table-bordered table-hover bg-white
                        @if ($item->vats->count() == 0) hidden @endif"
                        id="vats">
                        <thead>
                           <th class="text-center">{{ __('items.tax_name') }}</th>
                           <th class="text-center">{{ __('items.tax_value') }}</th>
                           <th class="text-center">{{ __('items.tax_use') }}</th>
                           <th class="text-center">{{ __('extends.action') }}</th>
                        </thead>
                        <tbody>
                           @foreach ($item->vats as $vat)
                           <tr class="vat-row {{ $vat->pivot->vat_use == 1 ? 'sales' : 'purchase' }}"
                              id="tr-{{ $vat->id }}-{{ $vat->pivot->vat_use }}">
                              <td class="text-center">{{ $vat->Addition_name }}</td>
                              <td class="text-center">{{ $vat->addition_value }}</td>
                              {{-- @if ($settings->einvoice == 1)
                              <td class="text-center">
                                 <div class="input-group d-flex justify-content-between">
                                    <input id="both-{{ $vat->id }}"
                                    name="vats[{{ $vat->id }}]" type="radio" value="0"
                                    @if ($vat->pivot->vat_use == 0) checked @endif>
                                    <label
                                       for="both-{{ $vat->id }}">{{ __('extends.both') }}</label>
                                    <input id="pay-{{ $vat->id }}"
                                    name="vats[{{ $vat->id }}]" type="radio" value="1""
                                    @if ($vat->pivot->vat_use == 1) checked @endif>
                                    <label
                                       for="pay-{{ $vat->id }}">{{ __('extends.sales') }}</label>
                                    <input id="sale-{{ $vat->id }}"
                                    name="vats[{{ $vat->id }}]" type="radio" value="2"
                                    @if ($vat->pivot->vat_use == 2) checked @endif>
                                    <label
                                       for="sale-{{ $vat->id }}">{{ __('extends.purchase') }}</label>
                                 </div>
                              </td>
                              @else --}}
                              <td class="text-center">
                                 {{ $vat->pivot->vat_use == 1 ? __('extends.sales') : __('extends.purchase') }}
                              </td>
                              {{-- @endif --}}
                              <td class="text-center">
                                 <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-vat"
                                    data-vat_id="{{ $vat->id }}"
                                    data-vat_use="{{ $vat->pivot->vat_use }}">
                                 <i class="fa fa-trash"></i>
                                 </a>
                              </td>
                           </tr>
                           @endforeach
                        </tbody>
                     </table>
                  </div>
               </div>
            </div>
            
            <div class="row">
                @if ( get_session_val('ecomme', 'integer') == 1 )
                    <div class="form-group col-md-6">
                        <label class="control-label" for="order_max_quantity">اقصي عدد للطلب من المتجر :</label>
                        <input type="number" min="0" class="form-control col-md-12 input-normal" name="order_max_quantity" id="order_max_quantity" value="{{ old('order_max_quantity') ?? $item->order_max_quantity }}">
                    </div>
                @endif
            </div>
            
            {{-- Item Sub Unit --}}
            @if ($all_sub_units->count())
            <div class="row">
               <div id="sub_units_inputs">
                  @foreach ($all_sub_units as $sub_u)
                  @php
                  $einvoice = $settings->einvoice == 1 ? '-' . $sub_u->unit_einvoice_code . '-' : '-';
                  $value = $sub_u->id . '-' . $sub_u->unit_barcode . $einvoice . $sub_u->unit_id . '-' . $sub_u->unit_price . '-' . $sub_u->unit_type . '-' . $sub_u->unit_value . '-' . $sub_u->Screenhidden;
                  @endphp
                  <input type="hidden" name="sub_units[]" class="input-{{ $sub_u->unit_id }}"
                     id="input-{{ $sub_u->id }}"
                     value="{{ $value }}">
                  @endforeach
               </div>
               <div class="col-md-12">
                  <div class="table-responsive">
                     <table class="table  zui-gray-table zui-table-rounded  table-hover bg-white" id="sub_units">
                        <thead>
                           <th class="text-center">{{ __('items.unit.title') }}</th>
                           @if ($settings->einvoice == 1)
                           <th class="text-center">{{ __('items.unit.einvoice_code') }}</th>
                           @endif
                           <th class="text-center">{{ __('items.unit.value') }}</th>
                           <th class="text-center">{{ __('items.unit.price') }}</th>
                           <th class="text-center">{{ __('items.unit.pay-price') }}</th>
                           <th class="text-center">{{ __('items.unit.barcode') }}</th>
                           <th class="text-center">  @lang('items.add-barcode-for-unit') </th>
                           <th class="text-center">{{ __('extends.action') }}</th>
                        </thead>
                        <tbody>
                           @foreach ($all_sub_units as $sub_unit)
                           <tr id="{{ $sub_unit->unit_barcode . '-' . $sub_unit->unit_id }}"
                              class="tr-{{ $sub_unit->unit_id }}">
                              <td class="text-center"
                                 id="{{ $sub_unit->unit_barcode . '-' . $sub_unit->unit_id . '-' . 'name' }}">
                                 {{ $sub_unit->name }}
                              </td>
                              @if ($settings->einvoice == 1)
                              <td class="text-center"
                                 id="{{ $sub_unit->unit_barcode . '-' . $sub_unit->unit_id . '-' . 'code' }}">
                                 {{ $sub_unit->unit_einvoice_code }}
                              </td>
                              @endif
                              <td class="text-center"
                                 id="{{ $sub_unit->unit_barcode . '-' . $sub_unit->unit_id . '-' . 'unit_value' }}">
                                 {{ quant_decimal($sub_unit->unit_value) }}
                              </td>
                              <td class="text-center"
                                 id="{{ $sub_unit->unit_barcode . '-' . $sub_unit->unit_id . '-' . 'price' }}">
                                 {{ price_decimal($sub_unit->unit_price) }}
                              </td>
                              <td class="text-center unit_pay_price" data-qty="{{ $sub_unit->unit_value }}"
                                 id="{{ $sub_unit->unit_barcode . '-' . $sub_unit->unit_id . '-' . 'pay_price' }}">
                                 {{ price_decimal($sub_unit->unit_value * $item->pay_price) }}
                              </td>
                              <td class="text-center"
                                 id="{{ $sub_unit->unit_barcode . '-' . $sub_unit->unit_id . '-' . 'barcode' }}">
                                 {{ $sub_unit->unit_barcode }}
                                 <br>
                                 <div
                                    class="card card-body @if ($sub_units_bars->count() > 0)
                                    @else hidden @endif">
                                    <ul class="list-inline unit-barcode-list p-0 m-0"></ul>
                                    <div id="list_bar-{{$sub_unit->unit_id}}">
                                       @foreach ($sub_units_bars as $sub_units_bar)
                                       <li class="external_barcode list-inline-item ml-1" id="subbarcode-{barcode}">
                                          <i id="{{ $sub_units_bar->unit_barcode }}">
                                          {{ $sub_units_bar->unit_barcode }}</i>
                                       </li>
                                       <br>
                                       {{--
                                       <li class="external_barcode list-inline-item ml-1" id="subbarcode-{{$sub_units_bar->unit_barcode}}">
                                          <a href="javascript:void(0)" class="text-danger sub_delete" data-ope="{{$sub_units_bar->unit_barcode}}">
                                          <i class="fa fa-times"></i></a>
                                          <i id="{{$sub_units_bar->unit_barcode}}">{{$sub_units_bar->unit_barcode}}</i>
                                       </li>
                                       --}}
                                       @endforeach
                                       {{-- <i
                                          id="{{ $sub_unit->unit_barcode . '-' . $sub_unit->unit_id . '-' . 'barcode' }}">
                                       {{ $sub_units_bar->unit_barcode }}</i> --}}
                                       <br>
                                    </div>
                                    </li>
                                    </ul>
                                 </div>
                              </td>
                              <td class="text-center">
                                 <button type="button" id="new-sub-barcode-btn"
                                    data-sub_unit_id="{{ $sub_unit->unit_id }}"
                                    data-sub_unit_value="{{ $sub_unit->unit_value }}"
                                    data-sub_unit_price="{{ $sub_unit->unit_price }}"
                                    class="btn btn-info col-md-9" data-toggle="modal"
                                    data-target="#new-sub-barcode-modal">
                                 <i class="fa fa-plus-circle"></i>   @lang('items.add-barcode')
                                 </button>
                              </td>
                              <td class="text-center">
                                 <a class="btn btn-xs btn-warning" data-toggle="modal" data-target="#updateSubUnitElementModal" data-href="{{ $sub_unit->id }}" style="margin-left:5px;">
                                 <i class="fa fa-edit"></i>
                                 </a>
                                 @if ($sub_unit->isFree())
                                 <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-sub-unit"
                                 data-barcode="{{ $sub_unit->unit_barcode }}"
                                 data-id="{{ $sub_unit->id }}"
                                 @foreach ($sub_units_bars as $sub_units_bar)
                                 data-sub-barcode="{{ $sub_units_bar->unit_barcode }} .'-'." @endforeach
                                 data-unit="{{ $sub_unit->unit_id }}">
                                 <i class="fa fa-trash"></i>
                                 </a>
                                 @endif
                              </td>
                           </tr>
                           @endforeach
                        </tbody>
                     </table>
                  </div>
               </div>
            </div>
      </div>
   </div>
   @endif
   {{-- Item Sub Unit Table --}}
   <div class="row">
   <div id="new_sub_units_inputs"></div>
   <div class="col-md-12">
   <div class="table-responsive">
   <table class="table zui-gray-table zui-table-rounded table-bordered table-hover bg-white hidden" id="new_sub_units">
   <thead>
   <th class="text-center">{{ __('items.unit.title') }}</th>
   @if ($settings->einvoice == 1)
   <th class="text-center">{{ __('items.unit.einvoice_code') }}</th>
   @endif
   <th class="text-center">{{ __('items.unit.value') }}</th>
   <th class="text-center">{{ __('items.unit.price') }}</th>
   <th class="text-center">{{ __('items.unit.pay-price') }}</th>
   <th class="text-center">{{ __('items.unit.barcode') }}</th>
   <th class="text-center">@lang('items.add-barcode-for-unit')</th>
   <th class="text-center">{{ __('extends.action') }}</th>
   </thead>
   <tbody></tbody>
   </table>
   </div>
   </div>
   </div>
   {{-- Item Size && Color --}}
   @if ($settings->color_size != 0)

   <hr>

        <div class="form-row row">
     <h4 class="text-info d-block">{{ __('items.select_item_color_Size') }}</h4>
        </div>
        <div class="form-row row">
      <div class="form-group col-md-6">
       <label class="control-label" for="size_id">{{ __('items.sizes') }} :</label>
        <select data-live-search="true" id="size_id" title="{{ __('items.choose_size') }}"
         class="form-control selectpicker col-md-12 input-normal" name="sizes[]" multiple
         data-none-selected-text="{{ trans('native.no-selected') }}">
          @foreach ($sizes as $size)
           @if (in_array($size->id, $size_array))
           <option selected value="{{ $size->id }}" data-name="{{ $size->size_name }}">
           {{ $size->size_name }}
           </option>
            @else
          <option value="{{ $size->id }}" data-name="{{ $size->size_name }}">
           {{ $size->size_name }}
          </option>
          @endif
           @endforeach
         </select>
       </div>
       <div class="form-group col-md-6 d-flex flex-column justify-content-end">
        <button type="button" class="btn btn-info col-md-6" id="new-size-btn" data-toggle="modal"
         data-target="#new-size-modal">
         <i class="fa fa-plus-circle"></i> {{ __('items.add_size') }}
         </button>
        </div>
        </div>
        <div class="form-row row">
    <div class="form-group col-md-6">
      <label class="control-label" for="color_id">{{ __('items.colors') }} :</label>
      <select data-live-search="true" id="color_id" class="form-control selectpicker input-normal col-md-12"
         data-none-selected-text="{{ trans('native.no-selected') }}" name="colors[]"
         title="{{ __('items.choose_color') }}" multiple>
        @foreach ($colors as $color)
         @if (in_array($color->id, $color_array))
       <option selected value="{{ $color->id }}" data-name="{{ $color->color_name }}">
        {{ $color->color_name }}
     </option>
      @else
     <option value="{{ $color->id }}" data-name="{{ $color->color_name }}">
      {{ $color->color_name }}
      </option>
      @endif
      @endforeach
      </select>
    </div>
    <div class="form-group col-md-6 d-flex flex-column justify-content-end">
      <button type="button" class="btn btn-info col-md-6" id="new-color-btn" data-toggle="modal"
         data-target="#new-color-modal">
          <i class="fa fa-plus-circle"></i> {{ __('items.add_color') }}
       </button>
    </div>
        </div>
   <hr>
   @endif

   {{-- Price List --}}
   {{-- @if ($settings->multi_price != 0 && $pricesList->count())
   <hr>
   <div class="form-row row">
   <h4 class="text-info">{{ __('items.price_list') }}</h4>
   </div>
   <div class="form-row row">
   @foreach ($pricesList as $price)
   <div class="form-group col-md-4">
   <label class="control-label" for="prices"> {{ $price->list_name }} :</label>
   <input type="text" class="form-control col-md-12 input-normal" value="{{ $price->price }}"
      name="price-{{ $price->id }}" />
   </div>
   @endforeach
   </div>
   <hr>
   @endif --}}
   {{-- Seller Commission --}}
   @if ($settings->bolla != 0)
   <hr>
   <div class="form-row row">
   <div class="form-group col-md-6">
   <label class="control-label" for="bolla">{{ __('items.seller_commission') }} :</label>
   <input type="text" class="form-control input-normal col-md-12" name="bolla" value="{{ $item->bolla }}"
      id="bolla">
   </div>
   </div>
   <hr>
   @endif
   {{-- Meters Per Carton --}}
   @if ($settings->module == 7)
   <hr>
   <div class="form-row row">
   <div class="form-group col-md-6">
   <label class="control-label" for="meter_ceramic">{{ __('items.meters_per_carton') }}
   :</label>
   <input type="text" class="form-control col-md-12 input-normal" name="meter_ceramic"
      value="{{ $item->meter_ceramic }}" id="meter_ceramic" />
   </div>
   </div>
   <hr>
   @endif
   {{-- Background Color && Order Show --}}
   @if ($settings->module == 6)
   <hr>
   <div class="form-row row">
   <div class="form-group col-md-6">
   <label class="control-label" for="background_color">{{ __('items.background_color') }}
   :</label>
   <input type="color" class="form-control col-md-12 jscolor input-normal" name="background_color"
      value="{{ $item->background_color }}" id="background_color">
   </div>
   <div class="form-group col-md-6">
   <label class="control-label" for="order_show">{{ __('items.order_show') }} :</label>
   <input type="text" class="form-control col-md-12 input-normal" name="order_show" value="{{ $item->order_show }}"
      id="order_show">
   </div>
   </div>
   <hr>
   @endif
   {{-- Amount used for cars --}}
   @if ($settings->module == 13)
   <hr>
   <div class="form-row row">
   <div class="form-group col-md-4">
   <label class="control-label" for="small_qty">{{ __('items.amount_used_for_small_cars') }}
   :</label>
   <input type="text" class="form-control col-md-12 numbercheck" name="small_qty" id="small_qty"
      value="{{ $item->used_for_small }}" />
   </div>
   <div class="form-group col-md-4">
   <label class="control-label" for="small_price">{{ __('items.amount_used_for_medium_cars') }}
   :</label>
   <input type="text" class="form-control col-md-12 numbercheck" name="medium_qty" id="medium_qty"
      value="{{ $item->used_for_medium }}" />
   </div>
   <div class="form-group col-md-4">
   <label class="control-label" for="medium_price">{{ __('items.amount_used_for_large_cars') }}
   :</label>
   <input type="text" class="form-control col-md-12 numbercheck" name="large_qty" id="large_qty"
      value="{{ $item->used_for_large }}" />
   </div>
   </div>
   <hr>
   @endif
   {{-- Shipping Companies --}}
   @if ($settings->module == 8)
   <hr>
   <div class="form-row row">
   <div class="form-group col-md-6">
   <label class="control-label" for="card_company_id">{{ __('items.shipping_companies') }}
   :</label>
   <select data-live-search="true" name="card_company_id" id="card_company_id"
      class="form-control selectpicker col-md-12"
      data-none-selected-text="{{ trans('native.no-selected') }}"
      title="{{ __('items.select_shipping_company') }}">
   @foreach ($cardCompany as $card)
   @if ($card->id == $item->card_company_id)
   <option selected value="{{ $card->id }}" data-name="{{ $card->company_name }}">
   {{ $card->company_name }}</option>
   @else
   <option value="{{ $card->id }}" data-name="{{ $card->company_name }}">
   {{ $card->company_name }}</option>
   @endif
   @endforeach
   </select>
   </div>
   <div class="form-group col-md-6 d-flex flex-column justify-content-end">
   <a href="javascript:void(0)" class="btn btn-info col-md-6" id="new-color-btn"><i
      class="fa fa-plus-circle"></i> {{ __('items.add_shipping_company') }}
   </a>
   </div>
   </div>
   <hr>
   @endif
   {{-- Item Options --}}
<div class="form-group col-md-12 p-3 bg-white">
      <h4 for="item_options" class="text-info">{{ __('items.item_options') }} :</h4>
       {{-- Item Buttons --}}
     <div class="form-row row text-center">
      @if (get_session_val('units_sup', 'Integer') == 1)
     <div class="form-group col-md-4">
      <label class="control-label col-md-8" for="add-sub-units">{{ __('items.add_sub_units') }}
       :</label>
        <a href="javascript:void(0)" class="btn btn-info" id="add-sub-units"
          data-sub_units="{{ $all_sub_units->count() }}">
           <i class="fa fa-plus-circle"></i> {{ __('extends.add') }}
        </a>
     </div>
      @endif
    <div class="form-inline col-md-4">
     <label class="control-label col-md-8" for="available">{{ __('items.show_inventory') }}
      :</label>
       <div class="col-md-4">
       <input type="checkbox" class="form-control" name="available" id="available"
        @if ($item->available == 1) checked @endif>
        </div>
    </div>
    <div class="form-inline col-md-4">
        <label class="control-label col-md-8" for="online">{{ __('items.show_online_store') }}
         :</label>
          <div class="col-md-4">
          <input type="checkbox" class="form-control" name="online" id="online"
           @if ($item->online == 1) checked @endif>
           </div>
    </div>
    <div class="form-inline col-md-4">
        <label class="control-label col-md-8" for="is_special">{{ __('items.is_special') }}
         :</label>
          <div class="col-md-4">
          <input type="checkbox" class="form-control" name="is_special" id="is_special"
           @if ($item->is_special == 1) checked @endif>
           </div>
    </div>
   <div class="form-inline col-md-4">
      <label class="control-label col-md-8" for="is_service">{{ __('items.service_item') }}:</label>
     <div class="col-md-4">
      <input type="checkbox" class="form-control" name="is_service" id="is_service"
       @if ($item->is_service == 1) checked @endif>
      </div>
     <small class="italic">@lang('items.hidden-in-report')</small>
   </div>
   {{-- <div class="form-group col-md-4">
   <label class="control-label col-md-8" for="new-shelf-btn">{{ __('items.add_shelf') }}
   :</label>
   <a href="javascript:void(0)" id="new-shelf-btn" class="btn btn-info">
   <i class="fa fa-plus-circle"></i> {{ __('extends.add') }}
   </a>
   </div>
   <div class="form-group col-md-4">
   <label class="control-label col-md-8" for="new-supplier-btn">{{ __('items.add_supplier') }}
   :</label>
   <a href="javascript:void(0)" id="new-supplier-btn" class="btn btn-info">
   <i class="fa fa-plus-circle"></i> {{ __('extends.add') }}
   </a>
   </div> --}}
</div>
   {{-- <div class="form-row row text-center">
   <div class="form-inline col-md-4">
   <label class="control-label col-md-8" for="available">{{ __('items.show_inventory') }}
   :</label>
   <div class="col-md-4">
   <input type="checkbox" class="form-control" name="available" id="available"
   @if ($item->available == 1) checked @endif>
   </div>
   </div> --}}
   {{-- <div class="form-inline col-md-4">
   <label class="control-label col-md-8" for="online">{{ __('items.show_online_store') }}
   :</label>
   <div class="col-md-4">
   <input type="checkbox" class="form-control" name="online" id="online"
   @if ($item->online == 1) checked @endif>
   </div>
   </div> --}}
   {{-- <div class="form-inline col-md-4">
   <label class="control-label col-md-8" for="is_service">{{ __('items.service_item') }}
   :</label>
   <div class="col-md-4">
   <input type="checkbox" class="form-control" name="is_service" id="is_service"
   @if ($item->is_service == 1) checked @endif>
   </div>
   </div> --}}
   {{-- @if ($settings->bill_time_alert)
   <div class="form-inline col-md-4">
   <label class="control-label col-md-8" for="game">{{ __('items.game_with_time') }}
   :</label>
   <div class="col-md-4">
   <input type="checkbox" class="form-control" name="game" id="game"
   @if ($item->game == 1) checked @endif>
   </div>
   </div>
   @endif --}}
   {{-- @if ($settings->module == 6)
   <div class="form-inline col-md-4">
   <label class="control-label col-md-8" for="cooking_add">{{ __('items.add_on_item') }}
   :</label>
   <div class="col-md-4">
   <input type="checkbox" class="form-control" name="cooking_add" id="cooking_add"
   @if ($item->cooking_add == 1) checked @endif>
   </div>
   </div>
   @endif
   @if ($settings->module == 8 && $settings->show_item_serial != 0)
   <div class="form-inline col-md-4">
   <label class="control-label col-md-8"
      for="require_serial">{{ __('items.require_serial') }} :</label>
   <div class="col-md-4">
   <input type="checkbox" class="form-control" name="require_serial"
   id="require_serial" @if ($item->require_serial == 1) checked @endif>
   </div>
   </div>
   @endif
   </div> --}}
   {{-- Manufacturing --}}
   @if ($settings->manufacturing)
   <div class="form-row row text-center mt-3 mb-3">
   <div class="form-inline col-md-4">
   <label for="main_item" class="control-label col-md-8">{{ __('items.main_item') }}
   :</label>
   <input id="main_item" class="col-md-4" name="manufacturing" type="radio" value="0"
   @if ($item->manufacturing == 0) checked @endif>
   </div>
   <div class="form-inline col-md-4">
   <label for="main_manuf" class="control-label col-md-8">{{ __('items.main_manufacturer') }} :</label>
   <input id="main_manuf" class="col-md-4" name="manufacturing" type="radio" value="1"
   @if ($item->manufacturing == 1) checked @endif>
   </div>
   <div class="form-inline col-md-4">
   <label for="item_manuf" class="control-label col-md-8">{{ __('items.item_manufacturer') }} :</label>
   <input id="item_manuf" class="col-md-4" name="manufacturing" type="radio" value="2"
   @if ($item->manufacturing == 2) checked @endif>
   </div>
   </div>
   @endif
   </div>
   {{-- Item Details && Images --}}
   <div class="form-row row mb-4">
   <div class="form-group col-md-6">
   <label class="control-label" for="details">{{ __('items.details') }} :</label>
   <textarea class="form-control h-100" name="details" style="resize: none" id="details" rows="7">{{ $item->details }}</textarea>
   </div>
   <div class="form-group col-md-6">
   <label class="control-label d-block" for="game">{{ __('items.image.name') }} :</label>
   <div class="text-center">
   <input type="file" class="mb-3" name="images" accept="image/*" onchange="loadFile(event)">
   <img id="output" width="200" height="200" src="{{ asset($item->img) }}">
   </div>
   </div>
   </div>

   <div class="form-row row mb-4 mt-5 justify-content-center">
   <div class="form-group col-md-2">

   <button class="btn btn-success w-100">{{ __('extends.save') }}</button>
   </div>
   </div>
   </form>
   <div class="col-md-12">
      <input type="hidden" name="date_now" id="date_now">
      <div id="dataContent"></div>
   </div>
</div>
</div>
<div class="modal fade" id="new-sub-barcode-modal" tabindex="-1" role="dialog" aria-labelledby="myModalLabel"
   aria-hidden="true">
   <div class="modal-dialog">
      <div class="modal-content">
         <div class="modal-header">
            <button type="button" class="close" data-dismiss="modal">&times;</button>
            <h4 class="modal-title" style="margin-left:200px">@lang('items.add-barcode-for-unit')</h4>
         </div>
         <div class="modal-body">
            {{--
            <form id="new-barcode-form-modal" action="" method="post">
               --}}
               {{-- {{ csrf_field() }} --}}
               <div class="modal-body" id="modal-loader-u">
                  <div class="form-group">
                     <input type="hidden" name="unit_id" id="sub_bar_unit_id" value="" />
                     <input type="hidden" name="unit_value" id="sub_bar_unit_value" value="" />
                     <input type="hidden" name="unit_price" id="sub_bar_unit_price" value="" />
                     <label class="control-label" for="barcode"> @lang('items.barcode.name') :</label>
                     <input type="number" class="form-control" name="sub_barcode[]" id="new-sub-barcode"
                        autofocus />
                  </div>
               </div>
               {{--
            </form>
            --}}
         </div>
         <div class="modal-footer">
            <a href="javascript:void(0)" class="btn btn-primary"
               id="add-sub-barcode-btn">{{ trans('extends.save-btn') }} </a>
            <button type="button" class="btn btn-default" data-dismiss="modal">{{ trans('extends.close') }}
            </button>
         </div>
      </div>
   </div>
</div>
@include('item::popups')
@stop
@section('css-scripts')
<link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
@stop
@section('js-scripts')
<script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
<script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
<script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
<script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
<script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
<script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>
<script>
   $(function() {
       $('#details').val('');
       $('#details').val('{{ $item->details }}');
       $(document).scrollTop(0);

       $('#sale_unit_div').find("button").parent().addClass("col-md-9");
       $('#sale_unit_div').find("ul").css("text-align", "left");
       $('#sale_unit_div').find("ul").first().css("width", "100%");
       $('#sale_unit_div').find("button").css("width", "100%");
       $('.bts_dropdown ').css({
           "margin": "0",
           "padding": "0"
       });

   });

   $(function() {
       if ('{{ get_session_val('date_type', 'Integer') }}' == '2') {
           $(".date").calendarsPicker({
               calendar: $.calendars.instance('gregorian'),
               dateFormat: 'Y-mm-dd',
               defaultDate: "0",
               selectDefaultDate: true
           });

           $(".date_").calendarsPicker({
               calendar: $.calendars.instance('gregorian'),
               dateFormat: 'Y-mm-dd',

           });
       } else {
           $(".date").calendarsPicker({
               calendar: $.calendars.instance('ummalqura'),
               dateFormat: 'Y-mm-dd',
               defaultDate: "0",
               selectDefaultDate: true

           });

           $(".date_").calendarsPicker({
               calendar: $.calendars.instance('ummalqura'),
               dateFormat: 'Y-mm-dd',
           });
       }

       $(function() {
           if ($('#quantity').val() > 0) {
               $('#quantity').trigger('change');
           }
       });

       $(document).on('change', '#quantity', function() {
           var value = $(this).val();
           if (value >= 0) {
               var store_name = $('#stores option:selected').data('store_name');
               var store_count = $('#stores').data('store_count');
               var text = store_count >= 1 ? `{{ __('items.edit_store_note') }} ${store_name}` :
                   `{{ __('items.add_store_note') }} ${store_name}`

               $("#store_div").removeClass('hidden');
               $('#store_note').removeClass('hidden').empty().text(text);
           } else {
               $(this).val(0);
               $("#store_div").addClass('hidden');
               $('#store_note').addClass('hidden');
           }
       });

       $(document).on('change', '#stores', function() {
           var store_name = $('#stores option:selected').data('store_name');
           $('#store_note').removeClass('hidden').empty().text(
               `{{ __('items.edit_store_note') }} ${store_name}`);
           var qty = parseFloat($(this).children('option:selected').data('quant'));
           $('#quantity').val(qty);
       });
   });
</script>
{{-- Item Vats --}}
<script>
   $("#vat_id").change(function(e) {
       e.preventDefault();

       $("#vats >tbody").empty();
       $("#vats").removeClass('hidden');

       $('#vat_id option:selected').each(function() {
           var vat_id = $(this).val();

           $("#vats >tbody").append(
               `<tr class="vat-row" id="tr-${vat_id}">
                   <td class="text-center">${$(this).data('name')}</td>
                   <td class="text-center">${$(this).data('value')}</td>
                   <td class="text-center">
                       <label for="both-${vat_id}">كلاهما:</label>
                       <input id="both-${vat_id}" name="vats[${vat_id}]" type="radio" value="0" style="margin-left: 10px;" checked>

                       <label for="pay-${vat_id}">مبيعات:</label>
                       <input id="pay-${vat_id}" name="vats[${vat_id}]" type="radio" value="1" style="margin-left: 10px;">

                       <label for="sale-${vat_id}">مشتريات:</label>
                       <input id="sale-${vat_id}" name="vats[${vat_id}]" type="radio" value="2">
                   </td>
                   <td class="text-center">
                       <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-vat"
                           data-vat_id=${vat_id}>
                           <i class="fa fa-trash"></i>
                       </a>
                   </td>
               </tr>`);
       });

       var table_length = $(document).find(".vat-row").length;

       table_length == 0 && $("#vats").addClass('hidden');
   });

   $(document).on('click', '.delete-vat', function() {
       var id = $(this).data('vat_id');
       var vat_use = $(this).data('vat_use');
       var currentVats = $('#vat_id').val();
       var currentSalesVats = $("#sales_vat_id").val();
       var currentPurchasesVats = $("#purcahse_vat_id").val();
       var table_length = $(document).find(".vat-row").length;
       var einvoice = {{ $settings->einvoice }};

       if (confirm('هل تريد حذف الضريبة ؟')) {
           $(`#tr-${id}-${vat_use}`).remove();

           if (einvoice == 1) {
               // remove from vats array
               currentVats = jQuery.grep(currentVats, function(value) {
                   return value != id;
               });

               $('#vat_id').val(currentVats);
               $('#vat_id').selectpicker('refresh', {
                   noneSelectedText: "{{ trans('native.no-selected') }}"
               });

               table_length == 1 && $("#vats").addClass('hidden');
           } else {
               // If E-Invoice is disabled
               if (vat_use == 1 && $(document).find(".sales").length > 0) {
                   currentSalesVats = jQuery.grep(currentSalesVats, function(value) {
                       return value != id;
                   });

                   $('#sales_vat_id').val(currentSalesVats);
                   $('#sales_vat_id').selectpicker('refresh', {
                       noneSelectedText: "{{ trans('native.no-selected') }}"
                   });

                   if (['', ' ', null].includes(currentSalesVats)) {
                       alert('يجب تحديد ضريبة للمبيعات');
                       $("#sales_exempt").prop('checked', true);
                       $('#sales_item_vats').addClass('hidden');
                       $('#sales_vat_id option').first().prop('selected', true);
                       $('#sales_vat_id').selectpicker('refresh', {
                           noneSelectedText: "{{ trans('native.no-selected') }}"
                       });
                   }
               } else if ($(document).find(".sales").length == 0) {
                   $("#sales_exempt").prop('checked', true);
                   $('#sales_item_vats').addClass('hidden');
               }

               if (vat_use == 2 && $(document).find(".purchase").length > 0) {
                   currentPurchasesVats = jQuery.grep(currentPurchasesVats, function(value) {
                       return value != id;
                   });

                   $('#purcahse_vat_id').val(currentPurchasesVats);
                   $('#purcahse_vat_id').selectpicker('refresh', {
                       noneSelectedText: "{{ trans('native.no-selected') }}"
                   });

                   if (['', ' ', null].includes(currentPurchasesVats)) {
                       $("#purchase_exempt").prop('checked', true);
                       $('#purcahse_item_vats').addClass('hidden');
                       $('#purcahse_vat_id option').first().prop('selected', true);
                       $('#purcahse_vat_id').selectpicker('refresh', {
                           noneSelectedText: "{{ trans('native.no-selected') }}"
                       });
                   }
               } else if ($(document).find(".purchase").length == 0) {
                   $("#purchase_exempt").prop('checked', true);
                   $('#purcahse_item_vats').addClass('hidden');
               }

               table_length == 1 && $("#vats").addClass('hidden');
           }
       }
   });
</script>
{{-- Item Vats If E-Invoice Disabled --}}
<script>
   // Sales Vats State
   $(document).on('change', '#sales_vat_select', function() {
       var item_vat_state = $(this).val();

       if (item_vat_state == 1) {
           $("#sales_vat_id").attr('multiple', 'multiple');
           $("#sales_vat_id").selectpicker('destroy');
           $("#sales_vat_id").selectpicker();
           $(document).find(".sales").length == 0 && $('#sales_vat_id option').first().prop('selected', true);
           $('#sales_vat_id').selectpicker('refresh', {
               noneSelectedText: "{{ trans('native.no-selected') }}"
           });
           $('#sales_item_vats').removeClass('hidden');
       } else {
           $('#sales_vat_id').removeAttr('multiple');
           $("#sales_vat_id").selectpicker('destroy');
           $("#sales_vat_id").selectpicker({
               noneSelectedText: "{{ trans('native.no-selected') }}"
           });
           $('#sales_item_vats').removeClass('hidden');
       }

       if (item_vat_state > 0) {
           $("#vats").removeClass('hidden');
           $(document).find(".sales").remove();
           $('#sales_vat_id option:selected').each(function() {
               var vat_id = $(this).val();

               $("#vats >tbody").append(
                   `<tr class="vat-row sales" id="tr-${vat_id}-1">
                       <td class="text-center">${$(this).data('name')}</td>
                       <td class="text-center">${$(this).data('value')}</td>
                       <td class="text-center">{{ __('extends.sales') }}</td>
                       <td class="text-center">
                           <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-vat"
                               data-vat_id=${vat_id} data-vat_use="1">
                               <i class="fa fa-trash"></i>
                           </a>
                       </td>
                   </tr>`);
           });
       } else {
           $(document).find(".sales").remove();
           $('#sales_item_vats').addClass('hidden');
       }
   });

   // Sales Vats selectpicker
   $(document).on('change', '#sales_vat_id', function(e) {
       var item_vat_state = $(this).val();

       if (['', ' ', null].includes($(this).val())) {
           alert("{{ __('extends.sales_vat_alert') }}");
           $(document).find(".sales").remove();
           $("#sales_exempt").prop('checked', true);
           $('#sales_item_vats').addClass('hidden');
           $('#sales_vat_id option').first().prop('selected', true);
           return false;
       }

       $("#vats").removeClass('hidden');
       $(document).find(".sales").remove();

       $('#sales_vat_id option:selected').each(function() {
           var vat_id = $(this).val();

           $("#vats >tbody").append(
               `<tr class="vat-row sales" id="tr-${vat_id}-1">
                   <td class="text-center">${$(this).data('name')}</td>
                   <td class="text-center">${$(this).data('value')}</td>
                   <td class="text-center">{{ __('extends.sales') }}</td>
                   <td class="text-center">
                       <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-vat"
                           data-vat_id=${vat_id} data-vat_use="1">
                           <i class="fa fa-trash"></i>
                       </a>
                   </td>
               </tr>`);
       });
   });

   // Purchases Vats State
   $(document).on('change', '#purchase_vat_select', function() {
       var item_vat_state = $(this).val();

       if (item_vat_state == 1) {
           $("#purcahse_vat_id").attr('multiple', 'multiple');
           $("#purcahse_vat_id").selectpicker('destroy');
           $("#purcahse_vat_id").selectpicker();
           $(document).find(".purchase").length == 0 && $('#purcahse_vat_id option').first().prop('selected',
               true);
           $('#purcahse_vat_id').selectpicker('refresh', {
               noneSelectedText: "{{ trans('native.no-selected') }}"
           });
           $('#purcahse_item_vats').removeClass('hidden');
       } else {
           $('#purcahse_vat_id').removeAttr('multiple');
           $("#purcahse_vat_id").selectpicker('destroy');
           $("#purcahse_vat_id").selectpicker({
               noneSelectedText: "{{ trans('native.no-selected') }}"
           });
           $('#purcahse_item_vats').removeClass('hidden');
       }

       if (item_vat_state > 0) {
           $("#vats").removeClass('hidden');
           $(document).find(".purchase").remove();
           $('#purcahse_vat_id option:selected').each(function() {
               var vat_id = $(this).val();

               $("#vats >tbody").append(
                   `<tr class="vat-row purchase" id="tr-${vat_id}-2">
                       <td class="text-center">${$(this).data('name')}</td>
                       <td class="text-center">${$(this).data('value')}</td>
                       <td class="text-center">{{ __('extends.purchase') }}</td>
                       <td class="text-center">
                           <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-vat"
                               data-vat_id=${vat_id} data-vat_use="2">
                               <i class="fa fa-trash"></i>
                           </a>
                       </td>
                   </tr>`);
           });
       } else {
           $(document).find(".purchase").remove();
           $('#purcahse_item_vats').addClass('hidden');
       }
   });

   // Purchases Vats selectpicker
   $(document).on('change', '#purcahse_vat_id', function(e) {
       var item_vat_state = $('#purchase_vat_select').val();

       if (['', ' ', null].includes($(this).val())) {
           alert("{{ __('extends.purchase_vat_alert') }}");
           $(document).find(".purchase").remove();
           $("#purchase_exempt").prop('checked', true);
           $('#purcahse_item_vats').addClass('hidden');
           $('#purcahse_vat_id option').first().prop('selected', true);
           return false;
       }

       $("#vats").removeClass('hidden');
       $(document).find(".purchase").remove();
       $('#purcahse_vat_id option:selected').each(function() {
           var vat_id = $(this).val();

           $("#vats >tbody").append(
               `<tr class="vat-row purchase" id="tr-${vat_id}-2">
                   <td class="text-center">${$(this).data('name')}</td>
                   <td class="text-center">${$(this).data('value')}</td>
                   <td class="text-center">{{ __('extends.purchase') }}</td>
                   <td class="text-center">
                       <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-vat"
                           data-vat_id=${vat_id} data-vat_use="2">
                           <i class="fa fa-trash"></i>
                       </a>
                   </td>
               </tr>`);
       });
   });
</script>
{{-- Item Vats If E-Invoice Enabled --}}
<script>
   $("#einvoice_vat_state").on('change', function() {
       var einvoice_vat_state = $(this).val();
       if (einvoice_vat_state != 0) {
           $('#vat_id_div').removeClass('hidden');
       }
   });
</script>
<script>
   var loader = `<div class="text-center">
           <div class="spinner-border text-primary" role="status">
               <span class="sr-only">Loading...</span>
           </div>
       </div>`;

   var decimal_value = '{{ $price_decimal }}';

   $.ajaxSetup({
       headers: {
           'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content'),
           type: "GET",
           data: null,
           contentType: false,
           cache: false,
           processData: false,
       }
   });

   // $('#item_name').focusout(function() {
   //     $('#about').val($('#item_name').val());
   // });

   $('#sale_price').focusout(function() {
       $('.prices').val($('#sale_price').val());
   });

   var unitId = $("#unit_id option:selected");
   $("#unit_einvoice_code").val(unitId.data("unit_einvoice_code"));
   $.map($('#unit option'), function(val, index) {
       if ($(val).val() == unitId.val()) {
           $(val).addClass('hidden');
           $('#unit').val($('#unit option').not('.hidden').first().val());
           $('#unit').selectpicker('refresh', {
               noneSelectedText: "{{ trans('native.no-selected') }}"
           });
       } else {
           $(val).removeClass('hidden');
           $('#unit').selectpicker('refresh', {
               noneSelectedText: "{{ trans('native.no-selected') }}"
           });
       }
   });

   $("#unit_id").change(function(e) {
       e.preventDefault();
       var unitId = $("#unit_id option:selected");
       $("#unit_einvoice_code").val(unitId.data("unit_einvoice_code"));
       $.map($('#unit option'), function(val, index) {
           if ($(val).val() == unitId.val()) {
               $(val).addClass('hidden');
               $('#unit').val($('#unit option').not('.hidden').first().val());
               $('#unit').selectpicker('refresh', {
                   noneSelectedText: "{{ trans('native.no-selected') }}"
               });
           } else {
               $(val).removeClass('hidden');
               $('#unit').selectpicker('refresh', {
                   noneSelectedText: "{{ trans('native.no-selected') }}"
               });
           }
       });
   });

   $(document).on('click', '#new-supplier-btn', function(e) {
       e.preventDefault();
       $('#new-supplier-modal').modal('show');
   });

   /********************************************************************************************************************************************/
   // New Supplier Modal
   $('#new-supplier-form-modal').on('submit', function(e) {
       e.preventDefault();
       data = new FormData($('#new-supplier-form-modal')[0]);
       $.ajax({
           url: "{{ route('items.supplier.store') }}",
           type: "post",
           data: data,
           contentType: false,
           cache: false,
           processData: false, // <==
           beforeSend: function() {
               $('#modal_client').empty().append(loader);
           },
           success: function(data) {
               1
               $(document).find('#supplier_id').prev('div').find('ul').append(data.list);
               $(document).find('#supplier_id').append(data.option);
               //$('.selectpicker').selectpicker('refresh', {noneSelectedText : "{{ trans('native.no-selected') }}"});
               $('#messages').removeClass('hidden').addClass('alert alert-success').html(
                   '{{ __('extends.save_sucessfully') }}');
               $('#new-supplier-modal').modal('toggle');
           },
           error: function(xhr) {
               var error = '<ul class="list-unstyled mb-0 pr-0">';
               $.each(xhr.responseJSON.errors, function(index, value) {
                   error += '<li>' + value + '</li>';
               });
               error += "</ul>";

               var supplier_name = $("#supplier_name").val();
               var tele = $("#tele").val();

               if (['', ' ', null].includes(supplier_name)) {
                   $('#supplier_name').focus();
                   $('label[for="supplier_name"]').addClass('text-danger');
                   $("#supplier_name").addClass('invalid');
               } else {
                   $('label[for="supplier_name"]').removeClass('text-danger');
                   $("#supplier_name").removeClass('invalid');
               }

               if (['', ' ', null].includes(tele)) {
                   $('#tele').focus();
                   $('label[for="tele"]').addClass('text-danger');
                   $("#tele").addClass('invalid');
               } else {
                   $('label[for="tele"]').removeClass('text-danger');
                   $("#tele").removeClass('invalid');
               }

               $('#new-supplier-modal').modal('toggle');
               $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(
                   error);
           }
       });
   });

   $('#new-supplier-modal').on('hidden.bs.modal', function() {
       $("#supplier_name").val('');
       $("#supplier_name").removeClass('invalid');
       $('label[for="supplier_name"]').removeClass('text-danger');

       $("#tele").val('');
       $("#tele").removeClass('invalid');
       $('label[for="tele"]').removeClass('text-danger');
   });
   /********************************************************************************************************************************************/

   /*----------------------------------------------------------------------------------------------------------*/
   // Barcode Modal
   $(document).on('click', '#new-barcode-btn', function(e) {
       e.preventDefault();
       $('#barcode').val() == '' ? $('#barcode').focus() : $('#new-barcode-modal').modal('show');
   });
   $(document).on('click', '#new-sub-barcode-btn', function() {
       let unit_value =$(this).data('sub_unit_value');
       let unit_price =$(this).data('sub_unit_price');
       var unit_id = $(this).data('sub_unit_id');
      $("#new-sub-barcode-modal #sub_bar_unit_id").val( unit_id );
      $("#new-sub-barcode-modal #sub_bar_unit_price").val( unit_price );
      $("#new-sub-barcode-modal #sub_bar_unit_value").val( unit_value );
      $('#new-sub-barcode-modal').modal('show');

   });

   $('#add-barcode-btn').on('click', function(e) {
       var barcode = $('#new-barcode').val();

       if (!['', ' ', null].includes(barcode)) {
           $("#newBarcode-btn").removeClass('hidden');
           $(document).find('#newBarcode').removeClass('hidden').collapse('show');

           $('#barcodes').append(
               `<input type="hidden" id="h-${barcode}" name="n_barcode[]" value="${barcode}">`);

           $('.barcode-list').append(`<li class="external_barcode list-inline-item ml-1" id="l-${barcode}">
               <a href="javascript:void(0)" class="text-danger delete" data-ope="${barcode}"><i class="fa fa-times"></i></a>
               <i id="${barcode}">${barcode}</i>
               </li>`);

           $('#new-barcode').val('');
           $('#new-barcode-label').removeClass('text-danger');
           $('#new-barcode').removeClass('invalid');
           $('#new-barcode-modal').modal('toggle');
       } else {
           $('#new-barcode').focus();
           $('#new-barcode-label').addClass('text-danger');
           $('#new-barcode').addClass('invalid');
       }
   });
   $('#add-sub-barcode-btn').on('click', function(e) {
       var unit_id = $('#sub_bar_unit_id').val();
       var barcode = $('#new-sub-barcode').val();
       var unit_value = $('#sub_bar_unit_value').val();
       var unit_price = $('#sub_bar_unit_price').val();

       if (!['', ' ', null].includes(barcode)) {
           $(`#list_bar-${unit_id}`).append(
               `<input type="hidden" id="sub-${barcode}" name="sub_unit_barcode[]" value="${barcode}">`,
               `<input type="hidden" id="sub-id-${unit_id}" name="sub_unit_badr_id[]" value="${unit_id}">`,
               `<input type="hidden" id="sub-value-${unit_value}" name="sub_bar_unit_value[]" value="${unit_value}">`,
               `<input type="hidden" id="sub-price-${unit_price}" name="sub_bar_unit_price[]" value="${unit_price}">`,
               );

           $('.card').removeClass('hidden');
           $(`#list_bar-${unit_id}`).append(`<li class="external_barcode list-inline-item ml-1" id="subbarcode-${barcode}">
               <a href="javascript:void(0)" class="text-danger sub_delete" data-ope="${barcode}"><i class="fa fa-times"></i></a>
               <i id="${barcode}">${barcode}</i>
               </li>`);

           $('#new-sub-barcode').val('');
           $('#sub_bar_unit_id').val('');
           $('#sub_bar_unit_value').val('');
           $('#sub_bar_unit_price').val('');
           $('#new-barcode-label').removeClass('text-danger');
           $('#new-sub-barcode').removeClass('is-invalid');
           $('#new-sub-barcode-modal').modal('toggle');
       } else {
           $('#new-sub-barcode').focus();
           $('#new-barcode-label').addClass('text-danger');
           $('#new-sub-barcode').addClass('is-invalid');
       }
   });
   $('#new-barcode-modal').on('hidden.bs.modal', function() {
       $('#new-barcode').val('');
       $('#new-barcode-label').removeClass('text-danger');
       $('#new-barcode').removeClass('invalid');
   });

   $(document).on('click', '.delete', function() {
       rel = $(this).data('ope');
       $('#h-' + rel).remove();
       $('#l-' + rel).remove();

       if ($(document).find('.external_barcode').length <= 0) {
           $(document).find('#newBarcode-btn').addClass('hidden');
           $(document).find('#newBarcode').addClass('hidden');
       }
   });
   $(document).on('click', '.sub_delete', function() {
       new_sub_barcode = $(this).data('ope');
       $('#sub-' + new_sub_barcode).remove();
       $('#subbarcode-' + new_sub_barcode).remove();
       if ($(document).find('.external_barcode').length <= 0) {
           $(document).find('.external_barcode').addClass('hidden');
       }
       $.ajax({
           url: "{{ route('items.update', $item->id) }}",
           method: "POST",
           data: {
               new_sub_barcode : new_sub_barcode
           },
           success: function(res) {

               console.log(res);
           }
       });

   });
   /*----------------------------------------------------------------------------------------------------------*/

   /*----------------------------------------------------------------------------------------------------------*/
   // Delete SubUnit
   $(document).on('click', '.delete-sub-unit', function() {
       let id = $(this).data('id');
       unit = $(this).data('unit');
       barcode = $(this).data('barcode');
       subbarcode = $(this).data('sub-barcode');
       $(`#${barcode}-${unit}`).remove();
       $(`#input-${id}`).val('');
       $(`#${subbarcode}-${unit}`).remove();
       $(`#input-${subbarcode}-${unit}`).val('');
       var table_length = $('#sub_units >tbody >tr').length;
       var new_table_length = $('#new_sub_units >tbody >tr').length;

       table_length == 0 && $('#sub_units').addClass('hidden'); // If table is empty hide it
       new_table_length == 0 && $('#new_sub_units').addClass('hidden'); // If table is empty hide it
   });

   $(document).on('click', '#subcat-show', function() {
       if ($('#sale_unit').val() == '') {
           // $('#sale_unit').trigger('mousedown');
           alert('{{ __('extends.choose_cat') }}');
           // $('#sale_unit').selectpicker('show');
       } else {
           var main_id = $('#sale_unit').val();
           getSubCats(main_id);
       }
   });

   $(document).on('click', '#intern-subcat-show', function() {
       if ($('#sale_unit').val() == '') {
           // $('#sale_unit').trigger('mousedown');
           alert('{{ __('extends.choose_cat') }}');
           // $('#sale_unit').selectpicker('show');
       } else if ($('#sub-cat-items').val() == '' || $('#sub-cat-items').val() == null) {
           alert('{{ __('extends.choose_sub_cat') }}');
       } else {
           var sub_id = $('#sub-cat-items').val();
           getInternCats(sub_id);
       }
   });


   $(document).on('change', '#sale_unit', function() {
       var main_id = $('#sale_unit').val();
       if (main_id != '') {
           getSubCats(main_id);
       }
   });

   $(document).on('change', '#sub-cat-items', function() {
       var sub_id = $('#sub-cat-items').val();
       if (sub_id != '' || sub_id != null) {
           getInternCats(sub_id);
       }
   });

   function getSubCats(id) {
       data = new FormData();
       data.append('main_id', id);
       data.append('_token', '{{ csrf_token() }}');
       $.ajax({
           url: "{{ route('items.ajax.get-sub-cats') }}",
           type: "post",
           data: data,
           contentType: false,
           cache: false,
           processData: false, // <==
           beforeSend: function() {
               $('#modal_client').empty().append(loader);
           },
           success: function(data) {
               $('#subcat-div').empty().append(data.data);
               $('#intern-subcat-div').removeClass('hidden');
           },
           error: function(xhr) {
               var error = '<ul class="list-unstyled mb-0 pr-0">';
               $.each(xhr.responseJSON.errors, function(index, value) {
                   error += '<li>' + value + '</li>';
               });
               error += "</ul>";
               $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(error);
           }
       });
   }

   function getInternCats(id) {
       data = new FormData();
       data.append('main_id', id);
       data.append('_token', '{{ csrf_token() }}');
       $.ajax({
           url: "{{ route('items.ajax.get-intern-sub-cats') }}",
           type: "post",
           data: data,
           contentType: false,
           cache: false,
           processData: false, // <==
           beforeSend: function() {
               $('#modal_client').empty().append(loader);
           },
           success: function(data) {
               $('#intern-subcat-div').empty().append(data.data);
               $('#intern-subcat-div').removeClass('hidden');
           },
           error: function(xhr) {
               var error = '<ul class="list-unstyled mb-0 pr-0">';
               $.each(xhr.responseJSON.errors, function(index, value) {
                   error += '<li>' + value + '</li>';
               });
               error += "</ul>";
               $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(error);
           }
       });
   }

   $(document).on('click', '#add-sub-units', function() {
       $('#modal_unit_barcode').val('');
       $('#modal_unit_price').val('');
       $('#modal_unit_value').val('');
       $('#new-sub-unit-modal').modal('show');
       $("#add-new-sub-unit-btn").attr('data-sub_units', $(this).data('sub_units'));
        setTimeout(() => {
            $('#new-sub-unit-modal').find('select').selectpicker('refresh');
        }, 200);
   });

   // Unit E-Invoice Code
   var code = $("#unit option:selected").data("unit_einvoice_code");
   $("#modal_unit_einvoice_code").val(code);

   // Add Unit E-Invoice Code When Select Unit
   $("#unit").change(function(e) {
       e.preventDefault();
       code = $("#unit option:selected").data("unit_einvoice_code");
       $("#modal_unit_einvoice_code").val(code);
   });

   function updateUnitPayPrice() {
       let pay_price = Number( $('#pay_price').val() );
       $('#new_sub_units, #sub_units').find('.unit_pay_price').each(function() {
           let cost = pay_price *  Number( $(this).data('qty'));
           $(this).text( parseFloat(cost).toFixed("{{ get_session_val('decimal_num_quant', 'integer') }}") );
       });
   }
   /********************************************************************************************************************************************/
   // New Sub Unit Modal
   $(document).on('click', '#add-new-sub-unit-btn', function() {
       var data = new FormData($('#new-sub-unit-form-modal')[0]);
       $('#show-modal-errors').addClass('hidden').empty();
       var sub_units = $(this).data('sub_units');

       // check if row is exist
       var tr = $('#sub_units >tbody >tr[class="tr-' + data.get('unit_id') + '"]');
       var input = $(`.input-${data.get('unit_id')}`);

       data.append('add_unit', true);
       data.append('_token', '{{ csrf_token() }}');

       $.ajax({
           url: "{{ route('items.ajax.get-sub-unit-row') }}",
           type: "post",
           data: data,
           contentType: false,
           cache: false,
           processData: false,
           beforeSend: function() {
               $('#modal_client').empty().append(loader);
           },
           success: function(response) {
               $('#sub_units >tbody >tr').length == 0 && $('#sub_units').removeClass('hidden');

               if (Number(sub_units) == 0) {
                   $("#new_sub_units").removeClass('hidden');
                   $('#new_sub_units >tbody').append(response.data)
                   $('#new_sub_units_inputs').append(response.n_hidden);
               } else {
                   if (tr.length == 0) {
                       $('#sub_units >tbody').append(response.data);
                       $('#sub_units_inputs').append(response.hidden);
                   } else {
                       // tr.replaceWith(response.data);
                       // input.replaceWith(response.hidden);
                       $('#messages').removeClass('hidden').empty().addClass(
                               'alert alert-danger text-center')
                           .append(`{{ __('items.sub_unit.already_exists') }}`);

                       setTimeout(function() {
                           $('#messages').addClass('hidden').empty().removeClass(
                               'alert alert-danger');
                       }, 5000);
                   }
               }

               updateUnitPayPrice();

               $('#new-sub-unit-modal').modal('toggle');
           },
           error: function(xhr) {
               var error = '<ul class="list-unstyled mb-0 pr-0">';
               $.each(xhr.responseJSON.errors, function(index, value) {
                   error += '<li>' + value + '</li>';
               });
               error += "</ul>";

               var unit = $("#unit").val();
               var unit_value = $("#modal_unit_value").val();
               var unit_price = $("#modal_unit_price").val();
               var unit_barcode = $("#modal_unit_barcode").val();

               if (['', ' ', null].includes(unit)) {
                   $("#unit").focus();
                   $("#unit").addClass('invalid');
                   $("label[for='unit']").addClass('text-danger');
               } else {
                   $("#unit").removeClass('invalid');
                   $("label[for='unit']").removeClass('text-danger');
               }

               if (['', ' ', null].includes(unit_value)) {
                   $("#modal_unit_value").focus();
                   $("#modal_unit_value").addClass('invalid');
                   $("label[for='modal_unit_value']").addClass('text-danger');
               } else {
                   $("#modal_unit_value").removeClass('invalid');
                   $("label[for='modal_unit_value']").removeClass('text-danger');
               }

               if (['', ' ', null].includes(unit_price)) {
                   $("#modal_unit_price").focus();
                   $("#modal_unit_price").addClass('invalid');
                   $("label[for='modal_unit_price']").addClass('text-danger');
               } else {
                   $("#modal_unit_price").removeClass('invalid');
                   $("label[for='modal_unit_price']").removeClass('text-danger');
               }

               if (['', ' ', null].includes(unit_barcode)) {
                   $("#modal_unit_barcode").focus();
                   $("#modal_unit_barcode").addClass('invalid');
                   $("label[for='modal_unit_barcode']").addClass('text-danger');
               } else {
                   $("#modal_unit_barcode").removeClass('invalid');
                   $("label[for='modal_unit_barcode']").removeClass('text-danger');
               }
               $('#show-modal-errors').removeClass('hidden').empty().addClass('alert alert-danger').append(error);
           }
       });
   });

   $('#new-sub-unit-modal').on('hidden.bs.modal', function() {
       $("#unit").val('');
       $("#unit").removeClass('invalid');
       $("label[for='unit']").removeClass('text-danger');

       $("#modal_unit_value").val('');
       $("#modal_unit_value").removeClass('invalid');
       $("label[for='modal_unit_value']").removeClass('text-danger');

       $("#modal_unit_price").val('');
       $("#modal_unit_price").removeClass('invalid');
       $("label[for='modal_unit_price']").removeClass('text-danger');

       $("#modal_unit_barcode").val('');
       $("#modal_unit_barcode").removeClass('invalid');
       $("label[for='modal_unit_barcode']").removeClass('text-danger');
   });
   /********************************************************************************************************************************************/

   /********************************************************************************************************************************************/
   // New Company Modal
   $('#new-company-form-modal').on('submit', function(e) {
       e.preventDefault();
       data = new FormData($('#new-company-form-modal')[0]);
       $.ajax({
           url: "{{ route('items.company.store') }}",
           type: "post",
           data: data,
           contentType: false,
           cache: false,
           processData: false, // <==
           beforeSend: function() {
               $('#modal_client').empty().append(loader);
           },
           success: function(data) {
               $(document).find('#company_id').prev('div').find('ul').append(data.list);
               $(document).find('#company_id').append(data.option);
               //$('.selectpicker').selectpicker('refresh', {noneSelectedText : "{{ trans('native.no-selected') }}"});
               $('#messages').removeClass('hidden').empty().addClass('alert alert-success').append(
                   "{{ __('extends.save_sucessfully') }}");
               $('#new-company-modal').modal('toggle');
           },
           error: function(xhr) {
               var error = '<ul class="list-unstyled mb-0">';
               $.each(xhr.responseJSON.errors, function(index, value) {
                   error += '<li>' + value + '</li>';
               });
               error += "</ul>";

               var company_name = $("#company_name").val();
               var income_ration = $("#income_ration").val();
               var sale_ratio = $("#sale_ratio").val();

               if (['', ' ', null].includes(company_name)) {
                   $("#company_name").focus();
                   $("#company_name").addClass('invalid');
                   $("label[for='company_name']").addClass('text-danger');
               } else {
                   $("#company_name").removeClass('invalid');
                   $("label[for='company_name']").removeClass('text-danger');
               }

               if (['', ' ', null].includes(income_ration)) {
                   $("#income_ration").focus();
                   $("#income_ration").addClass('invalid');
                   $("label[for='income_ration']").addClass('text-danger');
               } else {
                   $("#income_ration").removeClass('invalid');
                   $("label[for='income_ration']").removeClass('text-danger');
               }

               if (['', ' ', null].includes(sale_ratio)) {
                   $("#sale_ratio").focus();
                   $("#sale_ratio").addClass('invalid');
                   $("label[for='sale_ratio']").addClass('text-danger');
               } else {
                   $("#sale_ratio").removeClass('invalid');
                   $("label[for='sale_ratio']").removeClass('text-danger');
               }

               $('#new-company-modal').modal('toggle');
               $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(
                   error);
           }
       });
   });

   $('#new-company-modal').on('hidden.bs.modal', function() {
       $("#company_name").val('');
       $("#company_name").removeClass('invalid');
       $("label[for='company_name']").removeClass('text-danger');

       $("#income_ration").val('');
       $("#income_ration").removeClass('invalid');
       $("label[for='income_ration']").removeClass('text-danger');

       $("#sale_ratio").val('');
       $("#sale_ratio").removeClass('invalid');
       $("label[for='sale_ratio']").removeClass('text-danger');
   });
   /********************************************************************************************************************************************/

   /********************************************************************************************************************************************/
   // New Item Code Modal
   $('#new-color-form-modal').on('submit', function(e) {
       e.preventDefault();
       data = new FormData($('#new-color-form-modal')[0]);
       $.ajax({
           url: '{{ route('items.color.store') }}',
           type: "post",
           data: data,
           contentType: false,
           cache: false,
           processData: false, // <==
           beforeSend: function() {
               $('#modal_client').empty().append(loader);
           },
           success: function(data) {
               $(document).find('#card_company_id').prev('div').find('ul').append(data.list);
               $(document).find('#card_company_id').append(data.option);
               // $('.selectpicker').selectpicker('refresh', {noneSelectedText : "{{ trans('native.no-selected') }}"});
               $('#messages').removeClass('hidden').empty().addClass('alert alert-success').append(
                   "{{ __('extends.save_sucessfully') }}");
               $('#new-color-modal').modal('toggle');
           },
           error: function(xhr) {
               var error = '<ul class="list-unstyled mb-0 pr-0">';
               $.each(xhr.responseJSON.errors, function(index, value) {
                   error += '<li>' + value + '</li>';
               });
               error += "</ul>";

               var color_name = $("#color_name").val();
               var color_code = $("#color_code").val();

               if (['', ' ', null].includes(color_name)) {
                   $("#color_name").focus();
                   $("#color_name").addClass('invalid');
                   $("label[for='color_name']").addClass('text-danger');
               } else {
                   $("#color_name").removeClass('invalid');
                   $("label[for='color_name']").removeClass('text-danger');
               }

               if (['', ' ', null].includes(color_code)) {
                   $("#color_code").focus();
                   $("#color_code").addClass('invalid');
                   $("label[for='color_code']").addClass('text-danger');
               } else {
                   $("#color_code").removeClass('invalid');
                   $("label[for='color_code']").removeClass('text-danger');
               }

               $('#new-color-modal').modal('toggle');
               $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(
                   error);
           }
       });
   });

   $('#new-color-modal').on('hidden.bs.modal', function() {
       $("#color_name").val('');
       $("#color_name").removeClass('invalid');
       $("label[for='color_name']").removeClass('text-danger');

       $("#color_code").val('');
       $("#color_code").removeClass('invalid');
       $("label[for='color_code']").removeClass('text-danger');
   });
   /********************************************************************************************************************************************/

   /********************************************************************************************************************************************/
   // New Item Size Modal
   $('#new-size-form-modal').on('submit', function(e) {
       e.preventDefault();
       var data = new FormData($('#new-size-form-modal')[0]);
       $.ajax({
           url: '{{ route('items.size.store') }}',
           type: "post",
           data: data,
           contentType: false,
           cache: false,
           processData: false, // <==
           beforeSend: function() {
               $('#modal_client').empty().append(loader);
           },
           success: function(data) {
               $(document).find('#size_id').prev('div').find('ul').append(data.list);
               $(document).find('#size_id').append(data.option);
               // $('.selectpicker').selectpicker('refresh', {noneSelectedText : "{{ trans('native.no-selected') }}"});
               $('#messages').removeClass('hidden').empty().addClass('alert alert-success').append(
                   "{{ __('extends.save_sucessfully') }}");
               $('#new-size-modal').modal('toggle');
           },
           error: function(xhr) {
               var error = '<ul class="alert alert-danger">';
               $.each(xhr.responseJSON.errors, function(index, value) {
                   error += '<li>' + value + '</li>';
               });
               error += "</ul>";

               var size_name = $("#size_name").val();
               var size_code = $("#size_code").val();

               if (['', ' ', null].includes(size_name)) {
                   $("#size_name").focus();
                   $("#size_name").addClass('invalid');
                   $("label[for='size_name']").addClass('text-danger');
               } else {
                   $("#size_name").removeClass('invalid');
                   $("label[for='size_name']").removeClass('text-danger');
               }

               if (['', ' ', null].includes(size_code)) {
                   $("#size_code").focus();
                   $("#size_code").addClass('invalid');
                   $("label[for='size_code']").addClass('text-danger');
               } else {
                   $("#size_code").removeClass('invalid');
                   $("label[for='size_code']").removeClass('text-danger');
               }

               $('#new-size-modal').modal('toggle');
               $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(
                   error);
           }
       });
   });

   $('#new-size-modal').on('hidden.bs.modal', function() {
       $("#size_name").val('');
       $("#size_name").removeClass('invalid');
       $("label[for='size_name']").removeClass('text-danger');

       $("#size_code").val('');
       $("#size_code").removeClass('invalid');
       $("label[for='size_code']").removeClass('text-danger');
   });
   /********************************************************************************************************************************************/

   $(document).on('submit', '#new_item_form', function(e) {
       if ($('#item_name').val() == '' || $('#item_name').val() == null) {
           e.preventDefault();
           $('#item_name').focus();
           $('#item_name:focus').addClass('invalid');
       } else {
           $('#item_name').removeClass('invalid');
       }

       if ($('#supplier_id').val() == '' || $('#supplier_id').val() == null) {
           e.preventDefault();
           $('select#supplier_id').focus();
           $('select#supplier_id').closest('.bootstrap-select').find('button').addClass('invalid');
       } else {
           $('select#supplier_id').closest('.bootstrap-select').find('button').removeClass('invalid');
       }

       if ($('#quantity').val() == '' || $('#quantity').val() == null) {
           e.preventDefault();
           $('#quantity').focus();
           $('#quantity').addClass('invalid');
       } else {
           $('#quantity').removeClass('invalid');
       }

       if ($('#sale_unit').val() == '' || $('#sale_unit').val() == null) {
           e.preventDefault();
           $('#sale_unit').focus().select();
           $('select#sale_unit').closest('.bootstrap-select').find('button').addClass('invalid');
       } else {
           $('select#sale_unit').closest('.bootstrap-select').find('button').removeClass('invalid');
       }

       if ($('#unit_id').val() == '' || $('#unit_id').val() == null) {
           e.preventDefault();
           $('#unit_id').focus().select();
           $('select#unit_id').closest('.bootstrap-select').find('button').addClass('invalid');
       } else {
           $('select#unit_id').closest('.bootstrap-select').find('button').removeClass('invalid');
       }

       if ($('#pay_price').val() == '' || $('#pay_price').val() == null) {
           e.preventDefault();
           $('#pay_price').focus();
           $('#pay_price').addClass('invalid');
       } else {
           $('#pay_price').removeClass('invalid');
       }

       if ($('#sale_price').val() == '' || $('#sale_price').val() == null) {
           e.preventDefault();
           $('#sale_price').focus();
           $('#sale_price').addClass('invalid');
       } else {
           $('#sale_price').removeClass('invalid');
       }

       $('#pay_price').change();
   });

   $('#pay_price').change(function(e) {
       e.preventDefault();
       if ($(this).val() == '' || $(this).val() == ' ' || $(this).val() == null) {
           $('#pay_price').val(0);
       }

       updateUnitPayPrice();
   });

   $('#sale_price').focusout(function() {
       $('#lowest_price').val($(this).val());
       if ($(this).val() == '' || $(this).val() == ' ' || $(this).val() == null || $(this).val() < 0) {
           $(this).val(0);
       }
   });

   $('#pay_price').focusout(function() {
       if ($(this).val() == '' || $(this).val() == ' ' || $(this).val() == null || $(this).val() < 0) {
           $(this).val(0);
       }
   });

   $('#lowest_price').focusout(function() {
       if ($(this).val() == '' || $(this).val() == ' ' || $(this).val() == null || $(this).val() < 0) {
           $(this).val(0);
       }
   });

   $('#updateSubUnitElementModal').on('show.bs.modal', function(e) {
       hashedID = $(e.relatedTarget).data('href');
       $(document).find('#edit-sub-unit-btn').attr('data-related', $(e.relatedTarget).data('tr'));
       var data = {
           id: hashedID,
           unit_id: $("#unit_id option:selected").val(),
       };

       $.ajax({
           url: "{{ route('items.edit-sub-unit') }}",
           type: "get",
           data: data,
           contentType: false,
           cache: false,
           processData: true,
           beforeSend: function() {
               $('.updateElementWell').empty().prepend(loader);
           },
           success: function(response) {
               if (response.status == true) {
                   $('.updateElementWell').empty().prepend(response.data);
                   $('.selectpicker').selectpicker('refresh', {
                       noneSelectedText: "{{ trans('native.no-selected') }}"
                   });
               } else {
                   $('.updateElementWell').empty().prepend(
                       `<div class="alert alert-danger"><p class="text-center">${response.message}</p></div>`
                   );
               }
           },
           error: function(response) {
               $('.updateElementWell').empty().prepend(`
                   <div class="alert alert-danger">
                       <p class="text-center">{{ __('extends.something_wrong') }}</p>
                   </div>
               `);
           }
       });
   });

   $('#updateSubUnitElementModal').on('shown.bs.modal', function(e) {
       var einvoice_settings = {{ $settings->einvoice }};
       var unit_name = $('#edit_unit').find('option:selected').data('unit_name');

       if (einvoice_settings == 1) {
           var einvoice_code = $('#edit_unit').find('option:selected').data('unit_einvoice_code');
       }

       $(document).on('change', '#edit_unit', function() {
           unit_name = $('#edit_unit').find('option:selected').data('unit_name');

           if (einvoice_settings == 1) {
               einvoice_code = $(this).find('option:selected').data('unit_einvoice_code');
           }

           $(document).find('#modal_edit_unit_einvoice_code').val(einvoice_code);
       });

       $('#edit-sub-unit-form-modal').on('submit', function(e) {
           e.preventDefault();
           var data = new FormData($('#edit-sub-unit-form-modal')[0]);
           data.append('sub_id', hashedID);
           data.append('item_id', {{ $item->id }});

           $.ajax({
               url: "{{ route('items.update-sub_unit') }}",
               type: "post",
               data: data,
               contentType: false,
               cache: false,
               processData: false, // <==
               beforeSend: function() {
                   $('.updateElementWell').empty().prepend(loader);
               },
               success: function(response) {
                   console.log(response);
                   if (response.status == true) {
                       $('#messages').removeClass('hidden').empty().addClass(
                           'alert alert-success text-center').append(response.message);

                       var tr = $('#edit-sub-unit-btn').data('related');
                       var new_tr = `${data.get('unit_barcode')}-${data.get('unit_id')}`;
                       var new_row = `<tr id="${new_tr}" class="tr-${data.get('unit_id')}">
                           <td class="text-center" id="${new_tr}-name">${unit_name}</td>`;

                       if (einvoice_settings == 1) {
                           new_row +=
                               `<td class="text-center" id="${new_tr}-code">${data.get('unit_einvoice_code')}</td>`;
                       }

                       new_row += `<td class="text-center" id="${new_tr}-unit_value">${data.get('unit_value')}</td>
                           <td class="text-center" id="${new_tr}-price">${data.get('unit_price')}</td>
                           <td class="text-center" id="${new_tr}-barcode">${data.get('unit_barcode')}</td>
                           <td class="text-center">
                               <a href="javascript:void(0)" class="btn btn-success btn-xs" style="padding: 5px 16px 3px 15px;"
                                   data-toggle="modal" data-tr="${new_tr}" data-target="#updateSubUnitElementModal"
                                   data-href="${data.get('current_id')}">
                                   <i class="fa fa-edit"></i>
                               </a>

                               <a href="javascript:void(0)" class="btn btn-danger btn-xs delete-sub-unit"
                                   data-id="${data.get('id')}"
                                   data-barcode="${data.get('unit_barcode')}" data-unit="${data.get('unit_id')}">
                                   <i class="fa fa-trash"></i>
                               </a>
                           </td>
                       </tr>`;

                       $(`#${response.row_id}`).replaceWith(response.data);

                       var newValue =
                           `${data.get('current_id')}-${data.get('unit_barcode')}`;

                       if (einvoice_settings == 1) {
                           newValue += `-${data.get('unit_einvoice_code')}`;
                       }

                       newValue +=
                           `-${data.get('unit_id')}-${data.get('unit_price')}-${data.get('unit_type')}-${data.get('unit_value')}-${data.get('screenhidden') ?? 0}`;

                       $(`#input-${tr}`).replaceWith(
                           `<input type="hidden" name="sub_units[]" class="input-${data.get('unit_id')}" id="input-${new_tr}" value="${newValue}">`
                       );

                       $(this).data('related',
                           `${data.get('unit_barcode')}-${data.get('unit_id')}`);
                       $('#updateSubUnitElementModal').modal('toggle');

                       $('#sub_units_inputs').find(`#${response.input_id}`).val(response.value);

                       setTimeout(function() {
                           $('#messages').addClass('hidden').empty().removeClass(
                               'alert alert-success');
                       }, 3000);
                   } else {
                       $('#messages').removeClass('hidden').empty().addClass(
                               'alert alert-danger text-center')
                           .append(data.message);

                       setTimeout(function() {
                           $('#messages').addClass('hidden').empty().removeClass(
                               'alert alert-danger');
                       }, 3000);
                   }

                   $('#updateSubUnitElementModal').modal('toggle');
               },
               error: function(xhr) {
                   if (xhr.status == 422) {
                       var error = '<ul class="list-unstyled mb-0 pr-0">';
                       $.each(xhr.responseJSON.errors, function(index, value) {
                           error += '<li>' + value + '</li>';
                       });
                       error += "</ul>";
                       $('#messages').removeClass('hidden').empty().addClass(
                           'alert alert-danger text-center').append(
                           error);
                   }
                   setTimeout(function() {
                       $('#messages').addClass('hidden').empty().removeClass(
                           'alert alert-danger');
                   }, 3000);
                   $('#updateSubUnitElementModal').modal('toggle');
               }
           });
       });
   });


   /********************************************************************************************************************************************/
   // New Shelf Modal
   $(document).on('click', '#new-shelf-btn', function(e) {
       e.preventDefault();
       $('#new-shelf-modal').modal('show');
   });

   $('#new-shelf-form-modal').on('submit', function(e) {
       e.preventDefault();
       var data = new FormData($('#new-shelf-form-modal')[0]);
       $.ajax({
           url: "{{ route('items.shelf.store') }}",
           type: "post",
           data: data,
           contentType: false,
           cache: false,
           processData: false,
           beforeSend: function() {
               $('#modal_client').empty().append(loader);
           },
           success: function(data) {
               $(document).find('#shelf').prev('div').find('ul').append(data.list);
               $(document).find('#shelf').append(data.option);
               //$('.selectpicker').selectpicker('refresh', {noneSelectedText : "{{ trans('native.no-selected') }}"});
               $('#messages').removeClass('hidden').empty().addClass('alert alert-success').append(
                   "{{ __('extends.save_sucessfully') }}");
               $('#new-shelf-modal').modal('toggle');
           },
           error: function(xhr) {
               var error = '<ul class="list-unstyled mb-0 pr-0">';
               $.each(xhr.responseJSON.errors, function(index, value) {
                   error += '<li>' + value + '</li>';
               });
               error += "</ul>";

               var shelf_name = input["name='shelf_name'"].val();
               if (['', ' ', null].includes(shelf_name)) {
                   $("input[name='shelf_name']").focus();
                   $('label[for="shelf_name"]').addClass('text-danger');
                   $("input[name='shelf_name']").addClass('invalid');
               } else {
                   $("input[name='shelf_name']").removeClass('invalid');
                   $('label[for="shelf_name"]').removeClass('text-danger');
               }

               $('#new-shelf-modal').modal('toggle');
               $('#messages').removeClass('hidden').empty().addClass('alert alert-danger').append(
                   error);
           }
       });
   });

   $('#new-shelf-modal').on('hidden.bs.modal', function() {
       $("input[name='shelf_name']").val('');
       $("input[name='shelf_name']").removeClass('invalid');
       $('label[for="shelf_name"]').removeClass('text-danger');
   });
   /********************************************************************************************************************************************/
</script>
<script>
   $('.selectpicker').selectpicker({
       size: 8,
       noneSelectedText: '{{ trans('native.no-selected') }}'
   });
</script>
<script>
   var loadFile = function(event) {
       var output = document.getElementById('output');
       output.src = URL.createObjectURL(event.target.files[0]);
   };

   $(function() {
       var store_id = parseFloat('{{ $store_id }}');
       if (store_id > 0) {
           $('#stores').val(store_id).trigger('change');
           $('#stores').trigger('change');
       };
   })
</script>
<script src="{{ asset('tickets/js/jscolor.js') }}"></script>
@stop
