@extends('master::layouts.master')
@section('content')

<div class="card card-light mb-1_5 p-3">

<div class="container-fluid">
   <div class="row">
      <div class="col-md-12 mb-3">
         <h1 class="text-info d-inline-block">{{ __('units.title') }}</h1>
         <button class="btn btn-primary float-left mt-2" data-toggle="modal" data-target="#create_modal"
            id="add_unit_btn"><span class="glyphicon glyphicon-plus" data-toggle="tooltip" data-placement="top"
            title="Add Unit"></span>
         {{ __('units.add_unit') }}</button>
      </div>
      <div class="table-responsive">
         <table class="col-md-12 table zui-table zui-table-rounded table-striped table-hover bg-white">
            <thead>
               <tr>
                  <th scope="col">#</th>
                  <th scope="col">{{ __('units.unit_name') }}</th>
                  @if ($settings->einvoice == 1)
                  <th scope="col">{{ __('units.unit_einvoice_code') }}</th>
                  @endif
                  <th scope="col">{{ __('extends.action') }}</th>
               </tr>
            </thead>
            <tbody>
               @foreach ($units as $unit)
               <tr class="text-center" id="tr-{{ $unit->id }}" data-iterator="{{ $loop->iteration }}">
                  <td>{{ $loop->iteration }}</td>
                  <td class="{{ 'name-' . $unit->id }}">{{ $unit->name }}</td>
                  @if ($settings->einvoice == 1)
                  <td class="{{ 'einvoice-' . $unit->id }}">
                     {{ $unit->unit_einvoice_code ?? '-----' }}
                  </td>
                  @endif
                  <td>
                     <button class="btn btn-success edit-btn"
                        data-unit_id="{{ $unit->id }}">{{ __('extends.edit') }}
                     </button>
                     <button type="button" class="btn btn-danger delete-unit"
                        data-unit_id="{{ $unit->id }}">{{ __('extends.delete') }}</button>
                  </td>
               </tr>
               @endforeach
            </tbody>
         </table>
      </div>
   </div>
</div>
</div>
<div class="modal fade" id="create_modal" tabindex="-1" role="dialog" aria-labelledby="myModalLabel"
   aria-hidden="true">
   <div class="modal-dialog">
      <div class="modal-content">
         <div class="modal-header">
            <h4 class="modal-title" id="myModalLabel">{{ __('units.add_unit') }}</h4>
            <button type="button" class="close" data-dismiss="modal">&times;</button>
         </div>
         <form role="form" action="{{ route('units.store') }}" method="post" id="add_unit">
            @csrf
            <div class="modal-body pb-0" id="modal-loader-u">
               <div class="form-group">
                  <label class="control-label" for="name">{{ __('units.unit_name') }}</label>
                  <input class="form-control" name="name" id="name">
               </div>
               @if ($settings->einvoice == 1)
               <div class="form-group">
                  <label class="control-label"
                     for="unit_einvoice_code">{{ __('units.unit_einvoice_code') }}</label>
                  <select class="form-control selectpicker" name="unit_einvoice_code" id="unit_einvoice_code"
                     title="{{ __('units.select_unit_einvoice_code') }}">
                     @foreach (config('einvoice.unitcodes') as $unit)
                     <option value="{{ $unit['code'] }}">{{ $unit['code'] }} - {{ $unit['desc_ar'] != '' ? $unit['desc_ar'] : $unit['desc_en'] }}</option>
                     @endforeach
                  </select>
               </div>
               @endif
            </div>
            <div class="modal-footer d-flex justify-content-end">
               <button class="btn btn-primary" type="submit">{{ __('extends.save') }}</button>
               <button type="button" class="btn btn-secondary mr-1"
                  data-dismiss="modal">{{ __('extends.close') }}
               </button>
            </div>
         </form>
      </div>
   </div>
</div>
@stop
@section('js-scripts')
<script>
   $(document).ready(function() {
       $("#add_unit_btn").click(function(e) {
           e.preventDefault();
           $("#name").val('');
           $("#unit_einvoice_code").val('');
       });
   
       $(document).on('click', '.edit-btn', function() {
           var id = $(this).data('unit_id');
           $("#name").val($(".name-" + id).text());
           $("#unit_einvoice_code").val($(".einvoice-" + id).text());
           $(document).find("#add_unit").prepend(
               `<input type="hidden" name="id" id="unit_id" value="${id}">`);
           $('#create_modal').modal('show');
       });
   
       $('#create_modal').on('hidden.bs.modal', function() {
           $("#unit_id").remove();
       });
   
       // $("#add_unit").submit(function(e) {
       //     e.preventDefault();
       //     var data = new FormData(this);
       //     var id = data.get('id');
       //     var name = data.get('name');
       //     var unit_einvoice_code = data.get('unit_einvoice_code') ?? null;
       //     var method = id ? 'PUT' : 'POST';
   
       //     $.ajax({
       //         type: method,
       //         url: "{{ route('units.store') }}",
       //         data: $(this).serialize(),
       //         dataType: "json",
       //         success: function(response) {
       //             if (response.status == true) {
       //                 $('#success_messages').removeClass('hidden').html(
       //                     '{{ __('extends.save_sucessfully') }}');
       //                 $('#create_modal').modal('toggle');
       //                 $(document).find('.name-' + id).html(name);
       //                 $(document).find('.einvoice-' + id).html(unit_einvoice_code);
       //                 var iterator = $('table tr:last').data('iterator');
       //                 isNaN(iterator) ? iterator = 1 : iterator++;
   
       //                 setTimeout(function() {
       //                     $('#success_messages').empty().addClass('hidden');
       //                 }, 5000);
   
       //                 if (method == 'POST') {
       //                     var tr = `<tr class="text-center" id="tr-${response.unit.id}" data-iterator="${iterator}">
   //                         <td>${iterator}</td>
   //                         <td class="name-${response.unit.id}">${response.unit.name}</td>`;
   
       //                     if ({{ $settings->einvoice }} == 1) {
       //                         tr +=
       //                             `<td class="einvoice-${response.unit.id}">${response.unit.unit_einvoice_code}</td>`;
       //                     }
   
       //                     tr += `<td>
   //                         <button class="btn btn-success edit-btn" data-unit_id="${response.unit.id}">{{ __('extends.edit') }}
   //                         </button>
   //                         <button type="button" class="btn btn-danger delete-unit" data-unit_id="${response.unit.id}">{{ __('extends.delete') }}</button>
   //                     </td>
   //                     </tr>`;
   
       //                     $('tbody').append(tr);
       //                 }
       //             }
       //         },
       //         error: function(response) {
       //             var error = '<ul class="list-unstyled mb-0 pr-0">';
       //             $.each(response.responseJSON.errors, function(index, value) {
       //                 error += '<li>' + value + '</li>';
       //             });
       //             error += "</ul>";
       //             $('#create_modal').modal('toggle');
       //             $('#error_messages').removeClass('hidden').empty().append(error);
       //             setTimeout(function() {
       //                 $('#error_messages').empty().addClass('hidden');
       //             }, 5000);
       //         }
       //     });
       // });
   
       $(document).on('click', '.delete-unit', function(e) {
           e.preventDefault();
           var id = $(this).data('unit_id');
   
           if (confirm("{{ __('extends.confirm_delete') }}")) {
               $.ajax({
                   type: 'POST',
                   url: "{{ route('units.destroy') }}",
                   data: {
                       '_token': $('input[name=_token]').val(),
                       'id': id
                   },
                   success: function(response) {
                       if (response.status == true) {
                           $('#tr-' + id).fadeTo('slow', 0.7, function() {
                               $(this).remove();
                           });
                           $('#success_messages').removeClass('hidden').html(response.msg);
                           setTimeout(function() {
                               $('#success_messages').empty().addClass('hidden');
                           }, 5000);
                       } else {
                           $('#error_messages').removeClass('hidden').html(response.msg);
                           setTimeout(function() {
                               $('#error_messages').empty().addClass('hidden');
                           }, 5000);
                       }
                   }
               });
           }
       });
   });
</script>
@stop