<?php

use Modules\Item\Entities\Item;
use Modules\Item\Http\Controllers\MultiPriceController;

Route::group(['middleware' => ['web', 'auth']], function () {

    Route::get('units', 'UnitController@index')->name('units.index');
    Route::match(['post', 'put'], 'units/store', 'UnitController@store')->name('units.store');
    Route::post('units/destroy', 'UnitController@destroy')->name('units.destroy');
    Route::resource('colors', 'ColorsController');

    Route::group(['prefix' => 'items-group', 'as' => 'items-group.'], function () {

        Route::match(['get'], '/create', array('as' => 'create', 'uses' => 'ItemGroupController@create'));
        Route::match(['post'], '/store', array('as' => 'store', 'uses' => 'ItemGroupController@store'));
        Route::match(['get'], '/index', array('as' => 'index', 'uses' => 'ItemGroupController@index'));
        Route::match(['get'], '/show/{id}', array('as' => 'show', 'uses' => 'ItemGroupController@show'));
    });
    Route::group(['prefix' => 'items', 'as' => 'items.'], function () {
        Route::match(['get'], '/', array('as' => 'index', 'uses' => 'ItemController@index'));

        Route::match(['get'], '/show', function () {
            return redirect()->route('items.index');
        });

        Route::get('sync/vats', 'ItemController@syncItemVats');

        Route::match(['get'],  '/show/{id}', array('as' => 'show',    'uses' => 'ItemController@show'));
        Route::match(['get'],  '/edit/{id}', array('as' => 'edit',    'uses' => 'ItemEditController@edit'));
        Route::match(['get'],  '/create',    array('as' => 'create',  'uses' => 'ItemController@create'));
        Route::match(['get'],  '/excel',    array('as' => 'excel',  'uses' => 'ItemController@excel'));
        Route::match(['post'],  '/upload-prices', array('as' => 'prices-upload',  'uses' => 'ItemController@importItemExcel'));
        Route::match(['post'], '/update/{id}', array('as' => 'update',  'uses' => 'ItemEditController@update'));
        Route::match(['post'], '/store',     array('as' => 'store',   'uses' => 'ItemController@store'));
        Route::match(['post'], '/delete',    array('as' => 'destroy', 'uses' => 'ItemController@destroy'));

        // Item E-Invoice Code.
        Route::group(['prefix' => 'einvoice', 'as' => 'einvoice.'], function () {
            Route::match(['get', 'post'], '/egscode/{id}', array('as' => 'egscode', 'uses' => 'ItemEinvoiceController@egsCode'));
        });

        //manufacturing categories
        Route::match(['get', 'post'], '/manufacturing-cats/{item}', array('as' => 'manufacturing-cats', 'uses' => 'ItemController@manufacturingcats'));
        Route::match(['delete'], '/delete-manufacturing-cats', array('as' => 'delete-manufacturing-cats', 'uses' => 'ItemController@deleteManufacturingCats'));

        //item-group

        Route::match(['post'], '/store-supplier', array('as' => 'supplier.store', 'uses' => 'ItemController@storeSupplier'));
        Route::match(['post'], '/store-company', array('as' => 'company.store', 'uses' => 'ItemController@storeCompany'));
        Route::match(['post'], '/store-color', array('as' => 'color.store', 'uses' => 'ItemController@storeColor'));
        Route::match(['post'], '/store-size', array('as' => 'size.store', 'uses' => 'ItemController@storeSize'));
        Route::match(['post'], '/store-shelf', array('as' => 'shelf.store', 'uses' => 'ItemController@storeShelf'));
        Route::match(['post'], '/store-card-company', array('as' => 'card-company.store', 'uses' => 'ItemController@storeCardCompany'));
        Route::match(['post'], '/ajax-get-sub-cats', array('as' => 'ajax.get-sub-cats', 'uses' => 'ItemController@ajaxSubCats'));
        Route::match(['post'], '/ajax-get-intern-sub-cats', array('as' => 'ajax.get-intern-sub-cats', 'uses' => 'ItemController@ajaxInternSubCats'));
        Route::match(['post'], '/ajax-get-sub-units-row', array('as' => 'ajax.get-sub-unit-row', 'uses' => 'ItemController@ajaxSubUnits'));

        Route::match(['get'], '/ajax-edit-sub-unit', array('as' => 'edit-sub-unit', 'uses' => 'ItemEditController@editSubUnit'));
        Route::match(['post'], '/ajax-update-sub-unit', array('as' => 'update-sub_unit', 'uses' => 'ItemEditController@updateSubUnit'));

        Route::get('control-item-images', ['uses' => 'ItemController@itemImages', 'as' => 'images.index']);
        Route::post('submit-item-images', ['uses' => 'ItemController@postItemImages', 'as' => 'images.store']);
        Route::post('update-iem-image', ['uses' => 'ItemController@updateItemImage', 'as' => 'images.update']);
        Route::post('delete-item-image', ['uses' => 'ItemController@deleteItemImage', 'as' => 'images.delete']);

        Route::match(['get'], '/items-report', [
            'as' => 'items-price-report',
            'uses' => 'ItemsReportController@index'
        ]);
    });

    // Barcode Routes
    Route::group(['prefix' => 'barcode', 'as' => 'barcode.'], function () {
        Route::match(['get', 'post'], '/settings', array('as' => 'settings', 'uses' => 'BarcodeController@settings'));
        Route::match(['get', 'post'], '/printBarcode', array('as' => 'printBarcode', 'uses' => 'BarcodeController@printBarcode'));
        Route::get('/install', function () {
            $x = shell_exec('composer require milon/barcode');
            return $x == '' ? 'Barcode Package Installed' : $x;
        });
    });

    // Route::get('site-sliders', ['uses' => 'SliderController@index', 'as' => 'site-sliders.index']);

    // Route::post('delete-slider', [
    //     'as' => 'site-sliders.delete',
    //     'uses' => 'SliderController@destroy'
    // ]);

    // Route::post('store-slider', [
    //     'as' => 'site-sliders.store',
    //     'uses' => 'SliderController@store'
    // ]);


    // Route::post('ajax-items-prices-report-content', ['as' => 'items.ajax-items-prices-report-content', 'uses' => 'ItemsReportController@AjaxItemsReportFilter']);


    Route::get('update-items2', function () {
        $items = Item::where('shop_id', get_session_val('serial_id', 'integer'))->chunk(200, function ($itemsChunk) {
            foreach ($itemsChunk as $item) {
                if ($item->supplier_id > 0) {
                    $m_s = \DB::table('multi_supplier')->where([
                        'shop_id' => get_session_val('serial_id', 'integer'),
                        'item_id' => $item->id,
                        'supplier_id' => $item->supplier_id
                    ])->first();
                    if (is_null($m_s)) {
                        $m_s = \DB::table('multi_supplier')->insert([
                            'shop_id' => get_session_val('serial_id', 'integer'),
                            'item_id' => $item->id,
                            'supplier_id' => $item->supplier_id
                        ]);
                    }
                }
            }
        });
    });

    Route::group(['prefix' => 'multiPrices', 'as' => 'multiPrices.'], function(){
        Route::get('/', [MultiPriceController::class, 'index'])->name('all');
        Route::post('/store', [MultiPriceController::class, 'store'])->name('store');
        Route::post('/update', [MultiPriceController::class, 'update'])->name('update');
        Route::post('/delete', [MultiPriceController::class, 'destroy'])->name('delete');
    });

});
