<?php

namespace Modules\Maintenance\Entities;

use App\Traits\RoundingAttributesTrait;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;

class Order extends Model
{
    use RoundingAttributesTrait;
    
    public $timestamps = false;
    protected $guarded = ['id'];
    protected $table = 'requests';

    static public function getAvailableStatus()
    {
        return MaintenanceOrderStatus::where('shop_id', get_session_val('serial_id', 'integer'))->orderBy('view_order')->get();
    }

    static public function getCurrentStatusName($target_status)
    {
        return MaintenanceOrderStatus::where([ ['status', $target_status], ['shop_id', get_session_val('serial_id', 'integer')]])->first()->name ?? '';
    }
    
    public function details()
    {
        return $this->hasMany(OrderDetail::class, 'request_id');
    }

    public function client()
    {
        return $this->belongsTo(Client::class)
        ->withDefault([
            'client_name' => '---',
            'mobile1' => '---',
            'tele' => '---',
            'address' => '---'
        ])->select(['id', 'client_name', 'mobile1', 'tele', 'address']);
    }

    public function employee()
    {
        return $this->belongsTo(User::class, 'sales_man')->withDefault(['name' => '---'])->select(['id', 'name']);
    }

    public function category()
    {
        return $this->belongsTo(ItemType::class, 'category_id')->withDefault(['name' => '---'])->select(['id', 'name', 'category_id']);
    }

    public function getBrandAttribute()
    {
        $id = $this->category()->first()->category_id ?? 0;
        $brand = ItemType::where([
                'shop_id' => auth()->user()->shop_id,
                'id' => $id
            ])->first()->name ?? '---';
        return $brand;
    }

    public function store()
    {
        return $this->belongsTo(Store::class)->withDefault(['store_name' => '---'])->select(['id', 'store_name']);
    }

    public function components()
    {
        return $this->belongsToMany(Component::class, 'component_request', 'request_id')->withPivot('status');
    }

    public function records()
    {
        return $this->hasMany(OrderNote::class, 'request_id');
    }

    public function getWarranty($id = null)
    {
        $warranty = [
            '4' => 'ضمان تشغيل',
            '1' => 'ضمان اسبوع',
            '2' => 'ضمان شهر',
            '5' => 'ضمان ثلاثة شهر',
            '3' => 'ضمان ستة أشهر',
            '6' => 'ضمان سنة'
        ];
        if (is_null($id)) {
            return $warranty;
        }
        return $warranty[$this->warranty]??'';
    }

    public function adds()
    {
        return $this->hasMany(BillAddHistory::class, 'request_id');
    }

    public function bill()
    {
        return $this->belongsTo(Invoice::class, 'bill_id')->select('id', 'invoice_date');
    }

    public function inWarranty()
    {
        if ($this->bill()->count()) {
            $sale_date = Carbon::parse($this->bill->sale_date);
            $today = today()->format('Y-m-d');
            if ($this->warranty == 1) {
                $expire_date = $sale_date->addWeek();
            } elseif ($this->warranty == 2) {
                $expire_date = $sale_date->addMonth();
            } elseif ($this->warranty == 3) {
                $expire_date = $sale_date->addMonths(6);
            } elseif ($this->warranty == 4) {
                $expire_date = $sale_date->addDay();
            } elseif ($this->warranty == 5) {
                $expire_date = $sale_date->addMonths(3);
            } elseif ($this->warranty == 6) {
                $expire_date = $sale_date->addMonths(12);
            }
            $expire_date = $expire_date->format('Y-m-d');
            if ($today <= $expire_date) {
                return true;
            }
            return false;
        }
        return false;
    }

    public function logs()
    {
        return $this->hasMany(Log::class, 'bill_id')->where('type', 'orders');
    }

    public function point()
    {
        return $this->belongsTo(SalePoint::class, 'sale_point_id')->withDefault(['point_name' => '---']);
    }

    public function network()
    {
        return $this->belongsTo(SalePoint::class, 'network_id')->withDefault(['point_name' => '---']);
    }
}
