<?php

namespace Modules\Maintenance\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Maintenance\Entities\Log;
use Modules\Maintenance\Http\Requests\StoreRequest;
use Modules\Maintenance\Entities\Item;
use Modules\Maintenance\Entities\User;
use Modules\Maintenance\Entities\Order;
use Modules\Maintenance\Entities\Store;
use Modules\Maintenance\Entities\Client;
use Illuminate\Support\Facades\Validator;
use Modules\Maintenance\Entities\BillAdd;
use Modules\Maintenance\Entities\Employee;
use Modules\Maintenance\Entities\ItemType;
use Modules\Maintenance\Entities\Component;
use Modules\Maintenance\Entities\OrderNote;
use Modules\Maintenance\Entities\SalePoint;
use Modules\Maintenance\Entities\StoreItem;
use Modules\Maintenance\Entities\OrderDetail;
use Modules\Maintenance\Entities\BillAddHistory;
use Modules\Maintenance\Entities\ClientTransaction;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Carbon\Carbon;
use Modules\Invoices\Entities\InvoiceDetails;
use Modules\Invoices\Entities\InvoiceDetailsVats;
use Modules\Invoices\Entities\Invoices;
use Modules\Invoices\Entities\ItemVats;
use Modules\Maintenance\Entities\BadrRequest;

class MaintenanceController extends Controller
{
    public function fix()
    {
        $orders = Order::where('shop_id', auth()->user()->shop_id)->whereNotNull('no_bill')->select('id', 'no_bill', 'bill_id')->get();
        foreach ($orders as $order) {
            $bill = Invoices::where([
                'shop_id' => auth()->user()->shop_id,
                'id' => $order->bill_id
            ])->select('id', 'request_id')->first();

            if ($bill) {
                $bill->request_id = $order->id;
                $bill->save();
            }
        }
        return 'done';
    }

    public function enquiry(Request $request)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $user_sale_point = SalePoint::whereId(auth()->user()->sale_point)->first();
        $user_store = $user_sale_point->store_id;
        $stores = Store::where('shop_id', $shop_id)
            ->when(auth()->user()->change_store == 0, function ($q) use ($user_store) {
                $q->where('id', $user_store);
            })
            ->get();
        $status = Order::getAvailableStatus();
        $orders = Order::where('shop_id', $shop_id)
            ->whereDate('created_at', '>=', $request->from ?? today())
            ->whereDate('created_at', '<=', $request->to ?? today())
            ->when(auth()->user()->change_store == 0, function ($q) use ($user_store) {
                $q->where('store_id', $user_store);
            })
            ->select('id', 'status', 'store_id')->get();
        $type =  $type = get_session_val('date_type', 'integer');
        return view('maintenance::enquiry', compact('stores', 'orders', 'type', 'user_store', 'status'));
    }

    public function index(Request $request)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $clients = Client::where('shop_id', $shop_id)->whereIn('client_role', [0, 2])->select('id', 'client_name')->get();
        $user_store = SalePoint::whereId(auth()->user()->sale_point)->select('store_id')->first();
        $users = Employee::select('users.id', 'user_name')
            ->where([
                'users.shop_id' => $shop_id,
                'is_employee' => 0,

            ])->whereHas('point')->get();
        $status = Order::getAvailableStatus();
        $categories = ItemType::where(['shop_id' => $shop_id, 'category_id' => '0'])->select('id', 'name')->get();
        $stores = Store::where('shop_id', $shop_id);
        if (auth()->user()->change_store == 0) {
            $stores = $stores->where('id', $user_store);
        }
        $stores = $stores->get();

        $orders = Order::where([
            'shop_id' => $shop_id,
            'is_deleted' => 0
        ])
            ->when($request->filled('order_no'), function ($order_no) use ($request) {
                $order_no->where('order_no', $request->order_no);
            })
            ->when($request->filled('bill_no'), function ($bill_no) use ($request) {
                $bill_no->where('no_bill', $request->bill_no);
            })
            ->when($request->filled('serial'), function ($serial) use ($request) {
                $serial->where('serial', $request->serial);
            })
            ->when(!in_array($request->status, ['-1', '', null]), function ($status) use ($request) {
                $status->where('status', $request->status);
            })->when($request->filled('from'), function ($from) use ($request) {
                $from->whereDate('created_at', '>=', $request->from);
            })
            ->when($request->filled('to'), function ($to) use ($request) {
                $to->whereDate('created_at', '<=', $request->to);
            })
            ->when(!in_array($request->client_id, ['-1', '', null]), function ($client) use ($request) {
                $client->where('client_id', $request->client_id);
            })
            ->when(!in_array($request->modal, ['-1', '', null]), function ($modal) use ($request) {
                $modal->where('category_id', $request->modal);
            })
            ->when(!in_array($request->brand, ['-1', '', null]), function ($brand) use ($request) {
                $brand->whereHas('category', function ($query) use ($request) {
                    $query->where('category_id', $request->brand);
                });
            })
            ->when($request->filled('mobile'), function ($mobile) use ($request) {
                $mobile->whereHas('client', function ($client) use ($request) {
                    $client->where('mobile1', $request->mobile);
                });
            })
            ->when(!in_array($request->user_id, ['-1', '', null]), function ($user) use ($request) {
                $user->where('user_id', $request->user_id);
            })
            ->select('id', 'status', 'client_id', 'order_no', 'net', 'sales_man', 'category_id', 'profit', 'created_at', 'store_id')
            ->with('client', 'category', 'store')->orderBy('id', 'desc')->get();

        if (auth()->user()->change_store == 0) {
            $orders = $orders->where('store_id', $user_store);
        } else {
            if ($request->has('store_id') && !in_array($request->store_id, ['-1', '', null])) {
                $orders = $orders->where('store_id', $request->store_id);
            }
        }
        $subs = collect();
        $type = get_session_val('date_type', 'integer');
        return view('maintenance::index', compact('clients', 'users', 'categories', 'status', 'orders', 'type', 'subs', 'user_store', 'stores'));
    }


    public function create()
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $clients = Client::where('shop_id', $shop_id)->whereIn('client_role', [0, 2])->select('id', 'client_name', 'tele', 'address', 'mobile1')->get();
        $categories = ItemType::where([
            'shop_id' => $shop_id,
            'category_id' => '0',
            //            'maintenance_use' => 1
        ])->select('id', 'name')->get();
        $components = Component::select('id', 'name')->where('shop_id', $shop_id)->get();
        $type =  $type = get_session_val('date_type', 'integer');
        $user_store = SalePoint::whereId(auth()->user()->sale_point)->select('store_id')->first();
        $stores = Store::where('shop_id', $shop_id);
        if (auth()->user()->change_store == 0) {
            $stores = $stores->where('id', $user_store);
        }
        $stores = $stores->get();
        return view('maintenance::create', compact('clients', 'categories', 'components', 'type', 'user_store', 'stores'));
    }

    public function store(StoreRequest $request)
    {
        $transaction = DB::transaction(function () use ($request) {
            $shop_id  = get_session_val('serial_id', 'integer');
            $order_no = $this->getBillNumber('order', now(), $request->store_id);
            $order = Order::create([
                'shop_id' => $shop_id,
                'client_id' => $request->client_id,
                'store_id' => $request->store_id,
                'status' => 0,
                'user_id' => Auth::id(),
                'type' => 0,
                'category_id' => $request->modal,
                'damage' => $request->damage,
                'serial' => $request->serial,
                'delivery_date' => $request->delivery_date,
                'created_at' => now()
            ]);
            if ($request->file('image')) {
                $image = $request->file('image');
                $name = $order->id . '_' . time() . '.' . $image->getClientOriginalExtension();
                if (!file_exists('upload/' . $shop_id)) {
                    mkdir('upload/' . $shop_id, 0755, true);
                }
                $main_path = 'upload/' . $shop_id;

                $image->move($main_path, $name);
                $order->image = $main_path . '/' . $name;
            }
            $order->order_no = $order_no;
            $order->save();
            $components = [];
            foreach ($request->components as $id => $status) {
                $components[$id] = [
                    'status' => $status,
                    'shop_id' => $shop_id
                ];
            }
            $order->components()->attach($components);
            $client = Client::where([
                'shop_id' => $shop_id,
                'id' => $request->client_id
            ])->first();
            $client->update([
                'mobile1' => $request->mobile,
                'tele' => $request->phone ?? ''
            ]);

            $user = Employee::select('id', 'user_name')
                ->where([
                    'shop_id' => $shop_id,
                    'is_employee' => 0,
                    'id' => Auth::id()
                ])->first();
            Log::create($order->id, $user->id, 'قام المستخدم ' . $user->user_name . ' بانشاء طلب رقم ' . $order->order_no . ' بتاريخ ' . now());
            return [
                'status' => true,
                'order_id' => $order->id
            ];
        });
        if ($transaction && $transaction['status'] == true) {
            session()->flash('success', 'تم حفظ الطلب بنجاح');
            session()->put('order_id', $transaction['order_id']);
            return redirect(route('maintenance.index'));
        }
        session()->flash('error', 'خطأ أثناء الحفظ');
        return back();
    }

    public function edit($id)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $order = Order::with('details.item', 'client', 'category', 'store', 'components', 'records', 'adds.vat', 'bill', 'logs', 'point', 'network')->findOrFail($id);

        // $user_store = SalePoint::whereId(auth()->user()->sale_point)->first();
        $status = Order::getAvailableStatus();

        $user_store = SalePoint::find(auth()->user()->sale_point)->store_id;
        $points = SalePoint::select('id', 'point_name')
            ->where([
                'shop_id' => $shop_id,
                'point_type' => 1
            ])
            ->when(auth()->user()->point_change == 0, function ($q) use ($user_store) {
                $q->where('store_id', $user_store);
            })
            ->where('point_type', 1)
            ->get();
        $stores = Store::where('shop_id', $shop_id);
        if (auth()->user()->change_store == 0) {
            $stores = $stores->where('id', $user_store);
        }
        // $item_name = Item::where([
        //     'shop_id' => $shop_id
        // ])->get();
        // dd($item_name);
        $stores = $stores->get();
        $shop = \DB::table('badr_shop')->where('serial_id', $shop_id)->first();
        $decimal_value = get_session_val('decimal_num_price', 'integer');
        $sale_details = get_session_val('sale_details', 'integer');
        $currency = get_session_val('currency', 'integer');
        $bill_adds = get_session_val('bill_adds', 'integer');
        $network_payment = get_session_val('netowrk_payment', 'integer');
        $networks = collect();
        if ($network_payment == 1) {
            $networks = SalePoint::select('id', 'point_name')
                ->where([
                    'shop_id' => $shop_id,
                    'point_type' => 2
                ])
                ->when(auth()->user()->point_change == 0, function ($q) use ($user_store) {
                    $q->where('store_id', $user_store);
                })
                ->get();
        }
        $adds = BillAdd::select('id', 'Addition_name', 'check_addition', 'addition_value')->where('shop_id', $shop_id)->whereIn('add_role', [0, 1])->get();
        return view('maintenance::edit', compact('order', 'points', 'user_store', 'stores', 'decimal_value', 'status', 'sale_details', 'currency', 'bill_adds', 'adds', 'network_payment', 'networks'));
    }
    public function update(Request $request)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $request->validate([
            'order_id' => 'required|numeric|exists:requests,id',
            //            'employee' => 'required|numeric|exists:users,id',
            'warranty' => 'required|numeric|in:1,2,3,4,5,6',
            // 'add_user' => 'required|numeric|exists:users,id'
        ]);
        $order_id = $request->order_id;
        $user_id = Auth::id();
        $user_name = Employee::where([
            'id' => $user_id,
            'shop_id' => $shop_id,
            'is_employee' => 0
        ])->first()->user_name;
        $order = Order::where([
            'shop_id' => $shop_id,
            'id' => $order_id
        ])->first();
        $order->warranty = $request->warranty;
        $order->user_id = $user_id;
        //        $order->sales_man = $request->employee;
        $order->save();
        if ($request->has('engineers_notes')) {
            foreach ($request->engineers_notes as $id => $notes) {
                $new_note = new OrderNote();
                $new_note->request_id = $order->id;
                $new_note->employee_id = $id;
                $new_note->notes = $notes;
                $new_note->shop_id = $shop_id;
                $new_note->save();
                Log::create($order->id, $user_id, 'قام المستخدم ' . $user_name . ' بتسجيل ملاحظات (' . $notes . ') على الطلب رقم ' . $order->order_no . ' بتاريخ ' . now());
            }
        }
        if ($request->has('deleted_notes')) {
            $order->records()->whereIn('id', $request->deleted_notes)->delete();
            Log::create($order->id, $user_id, 'قام المستخدم ' . $user_name . ' بحذف ملاحظات من الطلب رقم ' . $order->order_no . ' بتاريخ ' . now());
        }
        if ($request->has('deleted_items')) {
            foreach ($request->deleted_items as $detail_id) {
                $detail = OrderDetail::where([
                    'id' => $detail_id,
                    'request_id' => $order_id,
                    'shop_id' => $shop_id
                ])->select('id', 'item_id')->with('item')->first();
                if ($detail) {
                    Log::create($order->id, $user_id, 'قام المستخدم ' . $user_name . ' بحذف الصنف ' . $detail->item->item_name . ' من الطلب رقم ' . $order->order_no . ' بتاريخ ' . now());
                    $detail->delete();
                }
            }
        }


        if ($request->has('items')) {
            foreach ($request->items as $item_id => $item) {
                $exp = explode('-', $item);
                $qty = $exp[0];
                $sale = $exp[1];
                $pay = $exp[2];
                $fee = 0;
                OrderDetail::create([
                    'request_id' => $order->id,
                    'shop_id' => $shop_id,
                    'item_id' => $item_id,
                    'price' => $sale,
                    'pay_price' => $pay,
                    'quantity' => $qty,
                    'item_fee' => $fee
                ]);
                $item_name = Item::where([
                    'id' => $item_id,
                    'shop_id' => $shop_id
                ])->first()->item_name;

                Log::create($order->id, $user_id, 'قام المستخدم ' . $user_name . ' بإضافة الصنف ' . $item_name . ' إلى الطلب رقم ' . $order->order_no . ' بتاريخ ' . now());
            }
        }
        if ($request->has('discount')) {
            $order->total = $request->order_total;
            $order->net = $request->order_net;
            $order->profit = $request->profit;
            $order->discount = $request->discount;
            $order->discount_type = $request->discount_type;
            $order->advance = $request->advance;
            $order->payment_type = $request->pay_method;
            $order->total_fees = 0;
            $order->sale_point_id = $request->point_id;
            $order->network_id = $request->network_id ?? 0;
            $order->network_payment = $request->network_payment ?? 0;
            $order->network_only = $request->network_only ?? 0;
            $order->save();
        }
        $shop = get_session_val('serial_id', 'integer');
        $adds = get_session_val('bill_adds', 'integer');
        if ($adds == '0' && $request->has('vat') && is_array($request->vat) && sizeof($request->vat) > 0) {
            $vats = BillAdd::where('shop_id', $shop_id)->whereIn('add_role', [0, 2])->select('id', 'addition_value', 'check_addition')->get();
            foreach ($request->vat as $vat_id => $vat_value) {
                if ($vat_value > 0) {
                    $vat = $vats->find($vat_id);
                    if ($vat) {
                        BillAddHistory::updateOrCreate([
                            'bill_id' => 0,
                            'type' => 1,
                            'request_id' => $order->id,
                            'addition_id' => $vat_id,
                            'shop_id' => $shop_id
                        ], [
                            'addition_value' => $vat_value,
                            'bill_add' => $vat->addition_value,
                            'add_type' => $vat->check_addition
                        ]);
                    }
                }
            }
        }
        if ($request->has('status')) {
            $status = $request->status;
            if ($status != 4) {
                if ($status != $order->status) {
                    Log::create($order->id, $user_id, 'قام المستخدم ' . $user_name . ' بتغيير حالة الطلب رقم ' . $order->order_no . ' من ' . Order::getCurrentStatusName($order->status) . ' إلى ' . Order::getCurrentStatusName($status) . ' بتاريخ ' . now());
                }
                $order->status = $status;
                $order->save();
                session()->flash('success', 'تم التعديل بنجاح');
            } else {
                $old_status = $order->status;
                $transaction = $this->convertOrderToBill($order, $request);
                if ($transaction) {
                    session()->flash('success', 'تم إنشاء فاتوره بنجاح');
                    session()->put('order_id', $order->id);
                    Log::create($order->id, $user_id, 'قام المستخدم ' . $user_name . ' بتغيير حالة الطلب رقم ' . $order->order_no . ' من ' . Order::getCurrentStatusName($old_status) . ' إلى ' . Order::getCurrentStatusName($order->status) . ' بتاريخ ' . now());
                } else {
                    session()->flash('success', 'خطأ أثناء إنشاء الفاتوره');
                }
            }
        }
        return back();
    }
    public function convertOrderToBill($order, $point_id, $request = [])
    {
        $transaction = DB::transaction(function () use ($order, $request) {
            $shop_id = auth()->user()->shop_id;
            $client = Client::where([
                'shop_id' => $shop_id,
                'id' => $order->client_id
            ])->first();
            $shop = \DB::table('badr_shop')->where('serial_id', $shop_id)->first();
            $network = $shop->netowrk_payment;
            $bill_payment = $order->advance;
            $point_id = $order->sale_point_id;
            if ($network == 1) {
                if ($order->network_only == 1) { //check paid from network only
                    $point_id = $order->network_id;
                    $bill_payment = $order->network_payment;
                } else {
                    $bill_payment = $order->advance + $order->network_payment;
                }
            }
            $all_net = $order->net + 0;
            if ($order->payment_type == 0) {
                $client->balance += $all_net - $bill_payment;
            }

            $bill_no = $this->getBillNumber('bill', now(), $order->store_id);
            $invoice = Invoices::create([
                'invoice_date' => now(),
                'transaction_date' => now(),
                'add_user' => $order->user_id,
                'client_id' => $client->id,
                'invoice_total_disc' => $order->discount,
                'invoice_total_disc_type' => $order->discount_type,
                'store_id' => $order->store_id,
                'bill_no' => $bill_no,
                'shop_id' => $shop_id,
                'bill_total' => $order->total,
                'net_price' => $order->net,
                'fee' => 0,
                'payment' => $bill_payment,
                'bil_payment' => $bill_payment,
                'rest' => $all_net - $bill_payment,
                'balance' => $client->balance,
                'request_id' => $order->id,
                'invoice_type' => 1,
                'invoice_total_disc_status' => 0,
                'reference_image' => $order->image
            ]);

            $details = OrderDetail::where([
                'shop_id' => $shop_id,
                'request_id' => $order->id
            ])->get();
            foreach ($details as $detail) {
                $item = Item::where([
                    'shop_id' => $shop_id,
                    'id' => $detail->item_id
                ])->first();
                if ($order->store_id == 0) {
                    $old_qty = $item->quantity;
                    $qty = $detail->quantity;
                    $new_qty = $old_qty - $qty;
                    $item->update([
                        'quantity' => $new_qty,
                        'shop_id' => $shop_id,
                    ]);
                } else {
                    $store = StoreItem::where([
                        'item_id' => $item->id,
                        'store_id' => $order->store_id,
                        'shop_id' => $shop_id
                    ])->first();
                    $old_qty = $store->store_quant;
                    $qty = $detail->quantity;
                    $new_qty = $old_qty - $qty;
                    $store->update([
                        'store_quant' => $new_qty
                    ]);
                }

                $pay_vats_weight = 0;

                if ($item->purchase_vat_state != 0)
                {
                    $item_vats = ItemVats::join('bills_add', 'bills_add.id', 'item_vat.vat_id')
                    ->where('item_id', $detail->item_id)
                    ->whereIn('item_vat.vat_use', [0, 1])
                    ->selectRaw('item_vat.*, bills_add.check_addition as main_vat_type, bills_add.addition_value as vat_value')
                    ->get();

                    foreach($item_vats as $item_vat)
                    {
                        $pay_vats_weight += $this->calculateVatWeight($item->vat_state, $detail->quantity, $detail->price, $item_vat->main_vat_type, $item_vat->vat_type, $item_vat->vat_value);

                    }
                }

                InvoiceDetails::create([
                    'invoice_id' => $invoice->id,
                    'shop_id' => get_session_val('serial_id', 'Integer'),
                    'item_id' => $detail->item_id,
                    'quantity' => $detail->quantity,
                    'quantity_after' =>  $new_qty,
                    'unit_id' => $item->unit_id,
                    'store_id' => $order->store_id,
                    'add_user' => $order->user_id,
                    'bolla' => 0,
                    'sale_price' => $detail->price,
                    'basic_sale_price' => $item->sale_price,
                    'total_price' => $detail->quantity * $detail->price,
                    'safe_point_id' => $point_id,
                    'pay_price' => $detail->pay_price,
                    'basic_pay_price' => $detail->pay_price,
                    'av_pay_price' => $detail->pay_price,
                    'ac_pay_price' => $detail->pay_price,
                    'pay_vats_weight' => $pay_vats_weight
                ]);

                if ($item->vat_state != 0)
                {

                    $item_vats = ItemVats::join('bills_add', 'bills_add.id', 'item_vat.vat_id')
                    ->where('item_id', $detail->item_id)
                    ->whereIn('item_vat.vat_use', [0, 1])
                    ->selectRaw('item_vat.*, bills_add.check_addition as main_vat_type, bills_add.addition_value as vat_value')
                    ->get();

                    foreach($item_vats as $item_vat)
                    {
                        InvoiceDetailsVats::create([
                            'invoice_id' => $invoice->id,
                            'item_id' => $detail->item_id,
                            'unit_id' => $item->unit_id,
                            'vat_id' => $item_vat->vat_id,
                            'vat_type' => $item_vat->main_vat_type,
                            'vat_operation' => $item_vat->vat_type,
                            'vat_value' => $item_vat->vat_value,
                            'vat_weight' => $this->calculateVatWeight($item->vat_state, $detail->quantity, $detail->price, $item_vat->main_vat_type, $item_vat->vat_type, $item_vat->vat_value),
                            'shop_id' => get_session_val('serial_id', 'integer')
                        ]);
                    }
                }
                
            }
            $safe = SalePoint::where([
                'shop_id' => $shop_id,
                'id' => $point_id
            ])->first();
            if ($bill_payment != 0) {
                if ($network == 1) {
                    if ($order->network_only == 1) { //check paid from network only
                        $safe->update([
                            'money_point' => $safe->money_point + $bill_payment,
                        ]);
                        $this->clientTransaction( $invoice->client_id, $bill_payment, 1, 1, now(),  $invoice->id, $client->balance, $safe,  $invoice->net, $order->user_id);
                    } else {
                        if ($order->advance != 0) {
                            $safe->update([
                                'money_point' => $safe->money_point + $order->advance,
                            ]);
                            $this->clientTransaction( $invoice->client_id, $order->advance, 1, 1, now(),  $invoice->id, $client->balance, $safe,  $invoice->net, $order->user_id);
                        }
                        if ($order->network_payment != 0) {
                            $network_safe = SalePoint::where('id', $order->network_id)->where('shop_id', $shop_id)->first();
                            $network_safe->update([
                                'money_point' => $safe->money_point + $order->network_payment,
                            ]);
                            $this->clientTransaction( $invoice->client_id, $order->network_payment, 1, 1, now(),  $invoice->id, $client->balance, $network_safe,  $invoice->net, $order->user_id);
                        }
                    }
                } else {
                    $safe->update([
                        'money_point' => $safe->money_point + $bill_payment,
                    ]);
                    $this->clientTransaction( $invoice->client_id, $bill_payment, 1, 1, now(),  $invoice->id, $client->balance, $safe,  $invoice->net, $order->user_id);
                }
            } else {
                $this->clientTransaction( $invoice->client_id, 0, 1, 0, now(),  $invoice->id, $client->balance, $safe,  $invoice->net, $order->user_id);
            }
            $order->no_bill = $bill_no;
            $order->bill_id =  $invoice->id;
            $order->status = 4;
            $order->save();

            return true;
        });
        return $transaction;
    }

    public function clientTransaction($client_id, $paid, $type, $effect, $date, $bill_id, $client_balance, $safe, $net, $user_d)
    {
        ClientTransaction::create([
            'shop_id' => $safe->shop_id,
            'client_id' => $client_id,
            'user_id' => $user_d,
            'amount' => $paid,
            'type' => $type,
            'effect' => $effect,
            'pay_day' => $date,
            'date_time' => $date,
            'bill_id' => $bill_id,
            'balance' => $client_balance,
            'safe_point_id' => $safe->id,
            'safe_balance' => $safe->money_point,
            'bill_net_total' => $net,
            'safe_type' => $safe->point_type,
        ]);
    }

    public function print($id)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $settings = DB::table('bill_setting')->select('vat_number', 'bill_header', 'message_bill')->where('shop_id', $shop_id)->first();
        $header = '';
        $footer = '';
        if ($settings) {
            $header = $settings->bill_header;
            $footer = $settings->message_bill;
        }
        $shop = get_session_val('serial_id', 'integer');
        $currency = get_session_val('currency', 'integer');
        $bill_adds = get_session_val('bill_adds', 'integer');
        $order = Order::with('details.item', 'client', 'category', 'components', 'records', 'adds')->findOrFail($id);
        if ($order->store_id > 0) {
            $store_setting = Store::where([
                'shop_id' => $shop_id,
                'id' => $order->store_id
            ])->first();
            if ($store_setting) {
                $header = $store_setting->bill_header;
                $footer = $store_setting->message_bill;
            }
        }
        $status = Order::getAvailableStatus();
        $bill = collect();
        $qr_code = '';
        if (!is_null($order->no_bill)) {
            // $bill = SaleProcess::where([
            //     // 'shop_id' => $shop_id,
            //     'bill_no' => $order->no_bill,
            //     // 'client_id' => $order->client_id,
            //     'image' => $order->image,
            // ])->with('details.item', 'adds')->first();

            // !here
            $bill = Invoices::where([
                ['invoice_type' , 1],
                ['shop_id' , $shop_id],
                ['client_id' , $order->client_id],
                ['reference_image', $order->image],
                ['bill_no' , $order->no_bill]
            ])->with(['InvoiceDetails.Item', 'vats'])->first();

            // $qr_code = file_get_contents(route('generate-QR-code', [
            //     'bill_id' => $bill->id,
            //     'code' => $shop_id,
            //     'bill_type' => 'sale'
            // ]));

            // print-barcode

            // $qr_code = file_get_contents(route('maintenance.print-barcode', ['id' => $order->id, 'embedded' => true]));
            $qr_code = '';
        }

        // dd($bill->vats->sum('vat_weight'));
        return view('maintenance::print', compact('qr_code', 'settings', 'header', 'status', 'footer', 'currency', 'order', 'bill', 'bill_adds'));
    }

    public function relatedCategories(Request $request)
    {
        $category_id = $request->id;
        $categories = ItemType::where([
            'shop_id' => auth()->user()->shop_id,
            'category_id' => $category_id
        ])->select('id', 'name')->get();
        $option = '';
        if (count($categories)) {
            foreach ($categories as $category) {
                $option .= '<option value="' . $category->id . '">' . $category->name . '</option>';
            }
        }
        return \response()->json([
            'status' => true,
            'option' => $option
        ]);
    }

    public function search(Request $request)
    {
        if (!is_null($request->search)) {

            $items = Item::where(function ($q) use ($request) {
                        $q->where('item_name', 'like', '%' . $request->search . '%')
                            ->orWhere('about', 'like', '%' . $request->search . '%')
                            ->orWhere('barcode', $request->search);
                    })
                    ->join('store_items', function ($q) use ($request) {
                        $q->on('store_items.item_id', '=', 'items.id');
                        $q->where('store_items.store_id', $request->store_id);
                    })
                    ->leftJoin('item_vat', function($q){
                        $q->on('item_vat.item_id', 'items.id');
                        $q->whereIn('item_vat.vat_use', [0, 1]);
                    })
                    ->leftJoin('bills_add', 'bills_add.id', 'item_vat.vat_id')
                    ->where('items.shop_id', auth()->user()->shop_id)
                    ->select(DB::raw('
                        items.id , items.pay_price, items.sale_price, items.unit_id,
                        items.require_serial, items.item_name, items.about, 
                        items.vat_state,
                        store_items.store_quant AS quantity,
                        GROUP_CONCAT(item_vat.vat_id) AS vats,
                        GROUP_CONCAT(bills_add.addition_value) AS vats_values,
                        GROUP_CONCAT(bills_add.add_role) AS vats_usage_status,
                        GROUP_CONCAT(bills_add.check_addition) AS vats_types,
                        GROUP_CONCAT(item_vat.vat_type) AS vats_opers ,
                        GROUP_CONCAT(item_vat.vat_use) AS vats_usage ,
                        items.purchase_vat_state
                    '))
                    ->groupBy('items.id')
                    ->orderByRaw('items.item_name + 0')
                    ->get();



            $html = view('maintenance::items', compact('items'))->render();
            if (count($items)) {
                return response()->json([
                    'status' => true,
                    'body' => $html,
                ]);
            }
            return response()->json([
                'status' => false,
                'body' => '<span class="col-md-offset-4" style="color: red">' . trans('native.no_result') . '</span>',
            ]);
        }
        return response()->json([
            'status' => false,
            'body' => '',
        ]);
    }

    public function newClient(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'name' => 'required|string',
            'mobile' => 'required|numeric|unique:clients,mobile1',
            'phone' => 'nullable|numeric|unique:clients,tele',
            'address' => 'nullable|string'
        ], [
            'mobile.unique' => 'رقم الموبايل مسجل من قبل',
            'phone.unique' => 'رقم التليفون الارضى مسجل من قبل',
        ]);
        if ($validator->fails()) {
            $errors = $validator->errors();
            $error_data = '<ul class="list-unstyled text-center alert alert-danger">';
            foreach ($errors->all() as $error) {
                $error_data .= '<li>' . $error . '</li>';
            }
            $error_data .= '</ul>';
            return response()->json([
                'status' => false,
                'body' => $error_data
            ]);
        }
        $transaction = DB::transaction(function () use ($request) {
            $client = Client::create([
                'client_name' => $request->name,
                'mobile1' => $request->mobile,
                'tele' => $request->phone,
                'address' => $request->address,
                'add_date' => now(),
                'addition_data' => '',
                'city_id' => 0,
                'country_id' => 0,
                'shop_id' => auth()->user()->shop_id
            ]);
            ClientTransaction::create([
                'date_time' => now(),
                'client_id' => $client->id,
                'amount' => 0,
                'type' => 4,
                'pay_day' => now(),
                'balance' => 0,
                'user_id' => auth()->id(),
                'shop_id' => auth()->user()->shop_id
            ]);
            $option = '<option selected value="' . $client->id . '" data-address="' . $client->address . '"
                data-mobile="' . $client->mobile1 . '" data-phone="' . $client->tele . '">' . $client->client_name . '</option>';
            return [
                'status' => true,
                'option' => $option,
                'client_id' => $client->id
            ];
        });
        if ($transaction['status']) {
            return \response()->json([
                'status' => true,
                'body' => $transaction['option'],
                'id' => $transaction['client_id']
            ]);
        }
        return \response()->json([
            'status' => false,
            'body' => 'حدث خطأ اثناء الحفظ'
        ]);
    }

    public function barcode($id, Request $request)
    {
        $order = Order::with('client')->findOrFail($id);
        $logo = get_session_val('logo_path', 'integer');

        return view('maintenance::barcode', compact('order', 'logo'));
    }

    public function destroy($id)
    {
        $order = Order::findOrFail($id);
        if (is_null($order->no_bill)) {
            $order->details->each->delete();
            $order->records->each->delete();
            $order->delete();
            $type = 'success';
            $message = 'تم الحذف بنجاح';
        } else {
            $order->is_deleted = 0;
            $order->save();
            $type = 'error';
            $message = 'لا يمكن حذف الطلب لارتباطه بفاتوره';
        }
        session()->flash($type, $message);
        return back();
    }

    public function checkQuantity(Request $request)
    {
        $select = 'quantity';
        if ($request->store_id > 0) {
            $select = 'store_quant as quantity';
        }
        $items = Item::select('items.id as item_id', 'item_name', $select)
            ->when($request->store_id > 0, function ($join) {
                $join->join('store_items', 'items.id', '=', 'store_items.item_id');
            })
            ->where('items.shop_id', auth()->user()->shop_id)->whereIn('items.id', $request->ids)->get();
        if (count($items) && count($items) == count($request->ids)) {
            $message = '';
            foreach ($items as $item) {
                if ($request->details[$item->item_id] > $item->quantity) {
                    $message .= '<li>لا يمكن اتمام الطلب الصنف  ' . $item->item_name . ' الكمية المطلوبة : ' . quant_decimal($request->details[$item->item_id]) . ' أكبر من الكمية الحالية : ' . quant_decimal($item->quantity) . '</li>';
                }
            }
            return response()->json([
                'status' => $message != '' ? true : false,
                'message' => $message
            ]);
        }
        return response()->json([
            'status' => false,
            'message' => 'الأصناف غير مسجلة'
        ]);
    }

    public function returnNotes(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required|numeric|exists:requests,id',
            'status' => 'required|numeric|in:0,1,2,3,4,5,6,7,8,9',
            'return_notes' => 'nullable|string',
            'add_user' => 'required|numeric|exists:users,id'
        ]);
        if ($validator->fails()) {
            $errors = $validator->errors();
            $error_data = '<ul class="list-unstyled text-center alert alert-danger">';
            foreach ($errors->all() as $error) {
                $error_data .= '<li>' . $error . '</li>';
            }
            $error_data .= '</ul>';
            return response()->json([
                'status' => false,
                'message' => $error_data
            ]);
        }
        $order = Order::where([
            'shop_id' => auth()->user()->shop_id,
            'id' => $request->id
        ])->first();
        $user_id = Auth::id();
        $user_name = Employee::where([
            'id' => $user_id,
            'shop_id' => auth()->user()->shop_id,
            'is_employee' => 0
        ])->first()->name;
        if ($request->status != $order->status) {
            Log::create($order->id, $user_id, 'قام المستخدم ' . $user_name . ' بتغيير حالة الطلب رقم ' . $order->order_no . ' من ' . Order::getCurrentStatusName($order->status) . ' إلى ' . Order::getCurrentStatusName($request->status) . ' بتاريخ ' . now());
        }
        if ($request->register_notes == 1 && !is_null($request->return_notes)) {
            Log::create($order->id, $user_id, 'قام المستخدم ' . $user_name . ' بتسجيل ملاحظات مرتجع صيانة على الطلب رقم ' . $order->order_no . ' بتاريخ ' . now());
        }
        $order->update([
            'guarantee' => $request->register_notes,
            'notes' => $request->return_notes,
            'status' => $request->status
        ]);
        return response()->json([
            'status' => true,
            'message' => 'تم الحفظ بنجاح'
        ]);
    }

    public function checkPassword(Request $request)
    {

        $user = User::where('user_name', $request->user_name)->where('shop_id', get_session_val('serial_id', 'integer'))->first();


        if ($user) {
            if (\Illuminate\Support\Facades\Hash::check($request->password, $user->password)) {
                return response()->json([
                    'status' => true,
                    'add_user' => $user->id,
                    'message' => 'تم التأكيد',
                ]);
            }
            return response()->json([
                'status' => false,
                'message' => 'كلمة المرور غير صحيحة',
            ]);
        }
        return response()->json([
            'status' => false,
            'message' => ' المستخدم غير موجود'
        ]);
    }

    public function getClients(Request $request)
    {
        $client_id = null;

        if ($request->embedded)
        {
            $client_id = Cache::get('embedded_client_' . Auth::id()) ?? null;
            Cache::forget('embedded_client_' . Auth::id());
        }

        $clients = Client::where([
            'shop_id' => get_session_val('serial_id', 'integer')
        ])->whereIn('client_role', [0, 2])->get();

        return response()->json([
            'clients' => $clients,
            'selected' => $client_id
        ]);

    }

    private function getBillNumber($type = "bill", $date = null, $store_id = null)
    { 

        $num = 0;
        if (is_null($date)) {
            $date = date('Y-m-d');
        }

        $late_hours = get_session_val('huro_late', 'integer');
        $date = $this->getRequiredDate($late_hours, $date);
        $num_type = get_session_val('billno_show', 'integer');
        $no_bill_type = get_session_val('no_bill_type', 'integer');

        if ($type == "bill") {
            $bill_no_ = Invoices::where([ ['shop_id' , get_session_val('serial_id', 'integer')], ['invoice_type' , 1]])->orderBy('id', 'DESC');
            if ($no_bill_type == 1 && !is_null($store_id)) {
                $bill_no_ = $bill_no_->where('store_id', $store_id);
            }
            if ($num_type == 2) { ## every day
                $bill_no_ = $bill_no_->whereRaw("DATE(invoice_date) = '" . $date . "' ");
            } elseif ($num_type == 3) { ## every month
                $bill_no_ = $bill_no_->whereRaw("YEAR(invoice_date) = YEAR('" . $date . "') AND  MONTH(invoice_date) = MONTH('" . $date . "')");
            } elseif ($num_type == 4) { ## every year
                $bill_no_ = $bill_no_->whereRaw("YEAR(invoice_date) = YEAR('" . $date . "')");
            }
            $bill_no_ = $bill_no_->first();
            $bill_no = 1;
            if ($bill_no_) {
                $bill_no_m = $bill_no_->bill_no;
                $bill_no = $bill_no_m + 1;
            }
            $num = $bill_no;
        }

        if ($type == "order") {

            $bill_no_ = BadrRequest::where('shop_id', get_session_val('serial_id', 'integer'))->orderBy('id', 'DESC');
            if ($no_bill_type == 1 && !is_null($store_id)) {
                $bill_no_ = $bill_no_->where('store_id', $store_id);
            }
            if ($num_type == 2) { ## every day
                $bill_no_ = $bill_no_->whereRaw("DATE(created_at) = '" . $date . "' ");
            } elseif ($num_type == 3) { ## every month
                $bill_no_ = $bill_no_->whereRaw("YEAR(created_at) = YEAR('" . $date . "') AND  MONTH(created_at) = MONTH('" . $date . "')");
            } elseif ($num_type == 4) { ## every year
                $bill_no_ = $bill_no_->whereRaw("YEAR(created_at) = YEAR('" . $date . "')");
            }
            $bill_no_ = $bill_no_->first();
            $bill_no = 1;
            if ($bill_no_) {
                $bill_no_m = $bill_no_->order_no;
                $bill_no = $bill_no_m + 1;
            }
            $num = $bill_no;
        }
        return $num;
    }

    private function getRequiredDate($late_hours, $date)
    {
        if (date('H') >= $late_hours) {
            $date = Carbon::parse($date)->format('Y-m-d');
        } else {
            $date = Carbon::parse($date)->subDays(1)->format('Y-m-d');
        }
        return $date;
    }

    private function calculateVatWeight($vat_state, $quantity, $price, $vat_type, $vat_operation, $vat_value)
    {
        $vat_weight = 0;
        $item_total = $quantity * $price;

        if ($vat_state == 1)
        {

            if ($vat_type == 0)
            {
                $total_plus_vat =  ( ($vat_operation == 0 ? 1 : -1) * ($item_total * (1 + (($vat_value) / 100))));
                $vat_weight =  (( $total_plus_vat - $item_total * ($vat_operation == 0 ? 1 : -1)));
            }
            else
            {
                $vat_weight = ($vat_operation == 0 ? 1 : -1) * (($vat_value) * $quantity);
            }

        }
        else if ($vat_state == 2)
        {
            if ($vat_type == 0)
            {
                $total_minus_vat = ($item_total / (1 + (($vat_value) / 100)));
                $vat_weight = (( $item_total - $total_minus_vat ));

            }
            else
            {
                $total_minus_vat = ($item_total - (($vat_value) * $quantity));
                $vat_weight = (($vat_value) * $quantity);

            }
        }

        return $vat_weight;
    }

}
