<?php

namespace Modules\Maintenance\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;
use Modules\Maintenance\Entities\Order;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Modules\Maintenance\Entities\MaintenanceOrderStatus;

class StatusController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index()
    {
        $requests_status = Order::getAvailableStatus();
        return view('maintenance::status.index', compact('requests_status'));
    }


    /**
     * Store a newly created resource in storage.
     * @param  Request $request
     * @return Response
     */
    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => ['required', 'string', Rule::unique('maintenance_order_statuses', 'name')->where('shop_id', get_session_val('serial_id', 'integer'))] ,
            'order' => 'numeric'
        ]);

        if ($validator->fails()) {
            return [
                'msg' => $this->getValidationMsgFormated($validator),
                'status' => false
            ];
        }

        $last_vals = DB::table('maintenance_order_statuses')->selectRaw('max(status) AS max_status, max(view_order) AS max_order')->first();


        $new_status = MaintenanceOrderStatus::create([
            'name' => $request->name,
            'status' => $last_vals->max_status + 1,
            'view_order' => $last_vals->max_order + 1,
            'shop_id' => get_session_val('serial_id', 'integer'),
        ]);

        return [
            'msg' => 'تم الحفظ بنجاح',
            'status' => true,
            'data' => $new_status
        ];
    }

    /**
     * Update the specified resource in storage.
     * @param  Request $request
     * @return Response
     */
    public function update(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'id' => ['required', 'numeric', Rule::exists('maintenance_order_statuses', 'id')->where('shop_id', get_session_val('serial_id', 'integer'))],
            'name' => ['required', 'string', Rule::unique('maintenance_order_statuses', 'name')->where('shop_id', get_session_val('serial_id', 'integer'))->whereNot('id', $request->id)]
        ]);

        if ($validator->fails()) {
            return [
                'msg' => $this->getValidationMsgFormated($validator),
                'status' => false
            ];
        }

        DB::table('maintenance_order_statuses')->where('id', $request->id)->update([
            'name' => $request->name
        ]);

        return [
            'msg' => 'تم التعديل بنجاح',
            'status' => true
        ];
    }

    /**
     * Remove the specified resource from storage.
     * @return Response
     */
    public function destroy(Request $request)
    {
        $deleted = false;
        $msg = 'هذة الحالة غير موجودة';

        $validator = Validator::make($request->all(), [
            'id' => ['required', 'numeric', Rule::exists('maintenance_order_statuses', 'id')->where('shop_id', get_session_val('serial_id', 'integer'))]
        ]);

        if ($validator->fails()) {
            return [
                'msg' => $this->getValidationMsgFormated($validator),
                'status' => false
            ];
        }

        $targetStatus = MaintenanceOrderStatus::find($request->id);

        if ($targetStatus->hold) {
            $msg = 'لا يمكن حذف هذه الحالة';
        }else {
            $targetStatus->delete();
            $deleted = true;
            $msg = 'تم الحذف بنجاح';
        }

        return [
            'status' => $deleted,
            'msg' => $msg
        ];

    }

    protected function getValidationMsgFormated($validator)
    {
        $errors = $validator->errors();
        $msgs = '';
        foreach ($errors->all() as $error) {
            $msgs .= '<li class="my-4 mr-2">' . $error . '</li>';
        }

        return $msgs;
    }

    public function reArrangeStatus(Request $request)
    {
        $new_order = $request->new_order;

        for ($i=0; $i < count($new_order); $i++) {
            MaintenanceOrderStatus::where('id', $new_order[$i])->update([
                'view_order' => ($i + 1)
            ]);
        }

        return [
            'status' => true,
            'msg' => 'تم تعديل الترتيب بنجاح'
        ];

    }
}
