@extends('master::layouts.master')

@section('content')
    <div class="container">
        <div class="row">
            <div class="col-md-12">
                <h2 class="title-page text-center">تقرير المستخدمين</h2>
                <form class="hidden-print">
                    <div class="col-md-3">
                        <div class="form-group">
                            <label for="from">{{trans('native.user')}} : </label>
                            <select name="user_id" id="user_id" class="form-control selectpicker" data-live-search="true">
                                <option @if ('-1' == request('user_id')) selected @endif value="-1">الكل</option>
                                @foreach($users as $user)
                                    <option @if ($user->id == request('user_id')) selected @endif value="{{ $user->id }}">{{ $user->user_name }}</option>
                                @endforeach
                            </select>
                        </div>
                    </div>
                    <div class="col-md-3">
                        <div class="form-group">
                            <label for="from">{{trans('native.from')}} : </label>
                            <input type="text" name="from" id="from" class="form-control date"
                                   value="{{ request('from')??today()->format('Y-m-d') }}" readonly required>
                        </div>
                    </div>
                    <div class="col-md-3">
                        <div class="form-group">
                            <label for="to">{{trans('native.to')}} : </label>
                            <input type="text" name="to" id="to" class="form-control date"
                                   value="{{ request('to')??today()->format('Y-m-d') }}" readonly required>
                        </div>
                    </div>
                    <button type="submit" style="margin-top: 1.9em" class="btn btn-success"><i
                                class="fa fa-search"></i>{{trans('native.search')}}</button>
                </form>
                <table class="table table-bordered table-responsive table-striped table-hover table-heading">
                    <thead class="bg-primary">
                    <tr>
                        <th scope="col">نوع العملية</th>
                        <th scope="col">العدد والتفاصيل</th>
                        <th scope="col">الاجمالى</th>
                        <th scope="col">الصافى</th>
                    </tr>
                    </thead>
                    <tbody id="bills-content">
                    @if (request()->has(['from', 'to', 'user_id']))
                        <tr class="success">
                            <td>عدد عمليات المبيعات</td>
                            <td>
                                <a target="_blank" rel="noopener noreferrer" href="{{ route('invoices.index') }}?dateFrom={{ request('from') }}&user={{ request('user_id') }}">
                                    {{ $sales->sales_count??0 }}
                                </a>
                            </td>
                            <td>{{ price_decimal($sales->sales_total??0) }}</td>
                            <td>{{ price_decimal($sales->sales_net??0) }}</td>
                        </tr>
                        @foreach(\Modules\Maintenance\Entities\Order::getAvailableStatus() as $order_status)
                            <tr @if ($order_status->status</tr> == 8) class="success" @endif>
                                <td>{{ $order_status->name }}</td>
                                <td>
                                    <a target="_blank" rel="noopener noreferrer" href="{{ route('maintenance.index') }}?from={{ request('from') }}&to={{ request('to') }}&status={{ $order_status->status }}&user_id={{ request('user_id') }}">
                                        {{ $orders->where('status', $order_status->status)->count() }}
                                    </a>
                                </td>
                                <td>{{ price_decimal($orders->where('status', $order_status->status)->sum('net')) }}</td>
                                <td>{{ price_decimal($orders->where('status', $order_status->status)->sum('total')) }}</td>
                            </tr>
                        @endforeach
                    @endif
                    </tbody>
                </table>
            </div>
        </div>
    </div>
@stop

@section('css-scripts')
    <link href="{{ asset('tickets/css/jquery-ui.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/bootstrap-datetimepicker-standalone.css') }}" rel="stylesheet">
    <link href="{{ asset('css/calenders/datepicker3.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/jquery.ptTimeSelect.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/jquery.timepicker.css') }}" rel="stylesheet">
    <link href="{{ asset('tickets/css/bootstrap-select.min.css') }}" rel="stylesheet">
@stop

@section('js-scripts')
    <script src="{{ asset('tickets/js/jquery-ui.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.ummalqura-ar.js') }}"></script>
    <script src="{{ asset('tickets/jquery.timepicker.js') }}"></script>
    <script src="{{ asset('tickets/bootstrap-datepicker.js') }}"></script>
    <script src="{{ asset('tickets/datepair.js') }}"></script>
    <script src="{{ asset('tickets/js/bootstrap-select.min.js') }}"></script>
    <script src="{{url('js/jquery.table2excel.js') }}"></script>
    <script>
        $(function () {
            $('.selectpicker').selectpicker({
                style: 'btn-default',
                size: 8
            });

            if ('{{ get_session_val('date_type', 'Integer') }}' ==  '2') {
                $(".date").calendarsPicker({
                    calendar: $.calendars.instance('gregorian'),
                    dateFormat: 'Y-mm-dd',
                    // defaultDate: "0",
                    // selectDefaultDate: true
                });
            } else {
                $(".date").calendarsPicker({
                    calendar: $.calendars.instance('ummalqura'),
                    dateFormat: 'Y-mm-dd',
                    // defaultDate: "0",
                    // selectDefaultDate: true
                });
            }
        });

        $(".excel_btn").click(function () {
            $("table").table2excel({
                exclude: ".noExl", // exclude CSS class
                name: "Worksheet Name",
                filename: "اجمالى مبيعات المستخدمين" //do not include extension
            });
        });
    </script>
@stop
