<?php

namespace Modules\Manufacture\Entities;

use App\Traits\RoundingAttributesTrait;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Modules\Maintenance\Entities\BillAdd;

class Item extends Model
{
    use RoundingAttributesTrait;
    protected $guarded = ['id'];
    protected $table = 'items';
    public $timestamps = false;

    public function unit()
    {
        return $this->belongsTo(Unit::class, 'unit_id');
    }

    public function getQuantityAttribute($value)
    {
        return round_quantity($value, true);
    }

    public function getPayPriceAttribute($value)
    {
        return round_quantity($value, true);
    }

    public function getSalePriceAttribute($value)
    {
        return round_quantity($value, true);
    }

    public function collection()
    {
        return $this->hasMany(ItemCollection::class, 'main_item');
    }

    public function items()
    {
        return $this->belongsToMany(Item::class, 'items_collection', 'main_item', 'id')->withPivot('quant', 'main_result');
    }

    public function orders()
    {
        return $this->hasMany(Order::class, 'item_id');
    }

    public function details()
    {
        return $this->hasMany(OrderDetails::class, 'item_id');
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('shop', function (Builder $builder) {
            $builder->where('items.shop_id', (auth()->user()->shop_id ?? 0));
        });
    }
}
