<?php

namespace Modules\Manufacture\Entities;

use App\Traits\RoundingAttributesTrait;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Modules\Maintenance\Entities\BillAdd;

class ItemCollection extends Model
{
    use RoundingAttributesTrait;
    protected $guarded = ['id'];
    protected $table = 'items_collection';
    public $timestamps = false;

    public function getQuantAttribute($value)
    {
        return round_quantity($value, true);
    }

    public function getMainResultAttribute($value)
    {
        return round_quantity($value, true);
    }

    public function item()
    {
        return $this->belongsTo(Item::class, 'item_id')->select('item_name', 'id', 'details', 'pay_price');
    }

    public function main()
    {
        return $this->belongsTo(Item::class, 'main_item')->select('item_name', 'id', 'details', 'pay_price');
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('shop', function (Builder $builder) {
            $builder->where('shop_id', (auth()->user()->shop_id ?? 0));
        });

        static::creating(function ($model) {
            $model->shop_id  = auth()->user()->shop_id;
            $model->add_date = now();
        });
    }
}
