<?php

namespace Modules\Manufacture\Entities;

use App\Traits\RoundingAttributesTrait;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Modules\Maintenance\Entities\BillAdd;

class Order extends Model
{
    use RoundingAttributesTrait;
    protected $guarded = ['id'];
    protected $table = 'manufacturing_order';
    public $timestamps = false;

    public function getAddDateAttribute($value)
    {
        return Carbon::parse($value)->format('Y-m-d');
    }

    public function scopeShop($query)
    {
        return $query->where('shop_id', auth()->user()->shop_id);
    }

    public function details()
    {
        return $this->hasMany(OrderDetails::class, 'order_id', 'order_id')->with('item', 'productionStore');
    }

    public function item()
    {
        return $this->belongsTo(Item::class, 'item_id', 'id')->select('id', 'item_name');
    }

    public function store()
    {
        return $this->belongsTo(Store::class)->select('id', 'store_name');
    }

    public function user()
    {
        return $this->belongsTo(User::class, 'add_user', 'id')->select('id', 'name');
    }

    public function scopeDateFilter($query)
    {
        return $query->when(request()->from_date && !request()->to_date, function ($query) {
            $query->whereDate('add_date', request()->from_date);
        })
            ->when(!request()->from_date && request()->to_date, function ($query) {
                $query->whereDate('add_date', request()->to_date);
            })
            ->when(request()->from_date && request()->to_date, function ($query) {
                $query->whereBetween('add_date', [Carbon::parse(request()->from_date)->format('Y-m-d 00:00:00'), Carbon::parse(request()->to_date)->format('Y-m-d 23:59:59')]);
            });
    }

    protected static function boot()
    {
        parent::boot();

        static::creating(function ($model) {
            $model->shop_id  = auth()->user()->shop_id;
            $model->order_id = Order::max('order_id') + 1;
            $model->add_user = auth()->id();
            $model->add_date = now();
        });
    }
}
