<?php

namespace Modules\Manufacture\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Manufacture\Entities\Item;
use Modules\Manufacture\Entities\ItemCollection;
use Modules\Manufacture\Entities\Order;
use Modules\Manufacture\Entities\OrderDetails;
use Modules\Manufacture\Entities\Store;
use Modules\Manufacture\Http\Requests\ManufactureStoreRequest;
use Modules\Manufacture\Http\Requests\ManufactureUpdateRequest;
use Modules\Manufacture\Http\Services\ManufactureInitService;
use Modules\Manufacture\Http\Services\ManufactureStoreService;
use Modules\Manufacture\Http\Services\ManufactureUpdateService;

class ManufactureController extends Controller
{
    public function index()
    {
        return view('manufacture::index');
    }

    public function loadData(Request $request)
    {
        $orders = Order::with('details', 'store', 'user')->dateFilter()->shop()
                        ->when(request()->item_id, function ($query) {
                            $query->where('item_id', request()->item_id);
                        })
                        ->when(request()->order_id, function ($query) {
                            $query->where('order_id', request()->order_id);
                        })->orderBy('id', 'DESC')->paginate(15);
        return view('manufacture::includes.list-orders', compact('orders'));
    }

    public function init ()
    {
        $list_items = Item::select('id', 'item_name', 'details')->whereHas('collection')->with('collection.item')->get();
        return view('manufacture::init', compact('list_items'));
    }

    public function saveInit (ManufactureStoreRequest $request)
    {
        $check = ItemCollection::where('main_item', $request->item_id)->exists();
        if ($check) {
            return response()->json(['message' => 'هذا المنتج مسجل من قبل'], 500);
        }

        if ( ManufactureInitService::init($request->items, $request->item_id, $request->order_qty) ) {
            session()->flash('success', 'تم الحفظ بنجاح');
            return response()->json(['reload' => true ], 200);
        } else {
            return response()->json(['message' => 'هناك خطأ ما يرجى المحاولة مرة أخرى'], 500);
        }
    }

    public function create()
    {
        $stors = Store::pluck('store_name', 'id')->toArray();
        return view('manufacture::create', compact('stors'));
    }

    public function store(ManufactureStoreRequest $request, ManufactureStoreService $service)
    {
        if ( $service->store($request) ) {
            session()->flash('success', 'تم الحفظ بنجاح');
            return response()->json(['reload' => true], 200);
        } else {
            return response()->json(['message' => 'هناك خطأ ما يرجى المحاولة مرة أخرى'], 500);
        }
    }

    public function materials()
    {
        $order_qty = request()->order_qty ?? 1;
        $collections = ItemCollection::select('item_id', 'quant')
            ->with('item:id,item_name,quantity,pay_price')
            ->whereHas('item')->where('main_item', request()->id)
            ->get();

        return view('manufacture::includes.show-materials', compact('collections', 'order_qty'));
    }

    public function getProducts()
    {
        return Item::select('id', 'item_name', 'quantity', 'unit_id', 'details')
                    ->with('unit:id,name')
                    ->where('manufacturing', request()->get('manufacturing'))
                    ->where('item_name', 'like', '%' . request()->search . '%')
                    ->get()->toArray();
    }

    public function edit($id)
    {
        $item = Item::where('id', $id)->with('collection.item')->first();
        return view('manufacture::edit', compact('item'));
    }

    public function update(ManufactureUpdateRequest $request, $id)
    {
        if ( ManufactureUpdateService::update($request->items, $id) ) {
            return back()->with('success', 'تم التعديل بنجاح');
        } else {
            return back()->with('error', 'هناك خطأ ما يرجى المحاولة مرة أخرى');
        }
    }

    public function destroy($id)
    {
        $rows = ItemCollection::where('main_item', $id)->delete();
        return response()->json(['message' => 'تم الحذف بنجاح'], 200);
    }

    public function destroyOrder($order_id)
    {
        $rows = Order::where('order_id', $order_id)->delete();
        return response()->json(['message' => 'تم الحذف بنجاح'], 200);
    }
}
