<?php

namespace Modules\Manufacture\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Manufacture\Entities\Item;
use Modules\Manufacture\Entities\ItemCollection;
use Modules\Manufacture\Entities\Order;
use Modules\Manufacture\Entities\OrderDetails;
use Modules\Manufacture\Entities\Store;
use Modules\Manufacture\Http\Requests\ManufactureStoreRequest;
use Modules\Manufacture\Http\Requests\ManufactureUpdateRequest;
use Modules\Manufacture\Http\Services\ManufactureInitService;
use Modules\Manufacture\Http\Services\ManufactureStoreService;
use Modules\Manufacture\Http\Services\ManufactureUpdateService;

class ManufactureReportController extends Controller
{
    public function index()
    {
        return view('manufacture::reports.index', ['items' => []]);
    }

    public function list()
    {
        $items = Item::select('items.id', 'items.item_name', 'items.sale_price', 'items.pay_price')->with([
                'orders' => function($query) {
                    $query->select('item_id', 'quantity')->dateFilter();
                },
            ])
            ->when(request()->item_id, function ($query) {
                $query->where('id', request()->item_id);
            })
            ->whereHas('orders', function ($query) {
                $query->dateFilter();
            })->get();

        return response()->json([
            'view' => view('manufacture::reports.table', compact('items'))->render(),
            'start_date' => request()->from_date,
            'end_date' => request()->to_date,
        ], 200);
    }
}
