<?php

namespace Modules\Manufacture\Http\Services;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Modules\EInvoice\Entities\Invoice;
use Modules\Manufacture\Entities\InvoiceDetails;
use Modules\Manufacture\Entities\Item;
use Modules\Manufacture\Entities\ItemCollection;
use Modules\Manufacture\Entities\ItemTransaction;
use Modules\Manufacture\Entities\Order;
use Modules\Manufacture\Entities\OrderDetails;
use Modules\Manufacture\Entities\Store;
use Modules\Manufacture\Entities\StoreItem;

class ManufactureStoreService {

    public array $data = [];
    public array $transactions = [];
    public float $cost = 0;
    public Order $order;
    public Store $store;
    public Store $production;
    public Item $mainItem;

    public function createFromBill(int $invoice_id, array $collection_data = [])
    {
        if (get_session_val('manufacturing', 'Integer') !== 2) return ;

        $invoice = Invoice::select('id', 'store_id')->find($invoice_id);
        if (empty($invoice)) return ;
        $invoice_details = InvoiceDetails::select('id', 'item_id', 'quantity')->where('invoice_id', $invoice_id)->get();

        foreach ($invoice_details as $detail) {
            $data ['item_id'] = $detail->item_id;
            $data ['store_id'] = $invoice->store_id;
            $data ['production_store'] = $invoice->store_id;
            $data ['order_qty'] = $detail->quantity;
            $data ['items'] = [];

            if (empty($collection_data)) {
                $collection = ItemCollection::select('item_id', 'quant')->where('main_item', $detail->item_id)->pluck('quant', 'item_id')->toArray();
            } else {
                $collection = $collection_data[$detail->id];
                // $set_init = [];
                // foreach ($collection as $collection_item_id => $collection_item_quant) {
                //     $set_init [] = [
                //         'id' => $collection_item_id,
                //         'qty' => $collection_item_quant,
                //     ];
                // }

                // ManufactureInitService::init($set_init, $detail->item_id, $detail->quantity);
            }

            foreach ($collection as $item_id => $quant) {
                $data ['items'][$item_id] = [
                    'id' => $item_id,
                    'qty' => $quant,
                ];
            }

            $this->store((object) $data, $invoice_id);
        }
    }
    public function store($request, $invoice_id = null)
    {
        try {
            DB::beginTransaction();
                $this->mainItem = Item::find($request->item_id);
                $this->store = Store::find($request->store_id);
                $this->production = Store::find($request->production_store);

                $this->createOrder($request, $invoice_id);
                $this->createOrderDetails($request);
                $this->syncQty($request);
            DB::commit();
            return true;
        } catch (\Exception $e) {
            // dd($e);
            DB::rollBack();
            return false;
        }
    }

    public function createOrder($request, $invoice_id = null)
    {
        $this->order = Order::create([
            'store_id' => $request->production_store,
            'item_id'  => $request->item_id,
            'quantity' => $request->order_qty,
            'bill_id'  => $invoice_id,
        ]);

        return $this;
    }

    public function createOrderDetails($request)
    {
        $items = Item::select('id', 'item_name', 'pay_price', 'sale_price', 'quantity')->whereIn('id', array_keys($request->items))->get();
        foreach ($items as $item) {
            $item_qty = (float) ( $request->items[$item->id]['qty'] * $this->order->quantity );
            $this->cost += (float) $item->pay_price * $item_qty;
            $this->data [] = [
                'order_id' => $this->order->order_id,
                'item_id'  => $item->id,
                'quantity' => $item_qty,
                'pay_price' => $item->pay_price,
                'store_id'  => $request->store_id,
                'shop_id'  => auth()->user()->shop_id,
            ];

            $old_item_qty = (float) ( $item->quantity ?? 0 );
            $this->transactions[] = [
                'type' => 50,
                'item_name' => $item->item_name,
                'item_id' => $item->id,
                'price' => $item->pay_price,
                'pay_price' => $item->sale_price,
                'quantity' => $old_item_qty,
                'new_quantity' => (float) $old_item_qty +  $item_qty,
                'old_quantity' => $item_qty,
                'store_id' => $request->store_id,
                'store_from' => $request->production_store,
                'order_no' => $this->order->order_id,
                'tansaction_date'  => now(),
                'action_date'  => now(),
                'user_id' => auth()->id(),
            ];

            StoreItem::where('item_id', $item->id)->where('store_id', $request->store_id)->decrement('store_quant', $item_qty);
        }

        OrderDetails::insert($this->data);
        ItemTransaction::insert($this->transactions);
        return $this;
    }

    public function syncQty($request)
    {
        $update_data['quantity'] = (float) $this->mainItem->quantity + (float) $request->order_qty;
        
        if (isset($request->update_cost) && $request->update_cost)
            $update_data['pay_price'] = $this->cost / $request->order_qty;

        $this->mainItem->update($update_data);

        if(isset($this->store->man_id) && $this->store->man_id !== 0) {
            StoreItem::where('item_id', $this->mainItem->id)->where('store_id', $this->store->id)->increment('store_quant', $item_qty);
        }

        if (isset($request->update_init) && $request->update_init) {
            foreach ($this->data as $info)
                ItemCollection::where('main_item', $request->item_id)->where('item_id', $info['item_id'])->update(['quant' => $info['quantity']]);
        }
        return $this;
    }
}
