@extends('master::layouts.master')

@section('content')
    <div class="container-fluid" style="margin: 10px 10px 0 10px">
        <div id="show-message"></div>
        <div class="row">
            <div class="col-md-12 well">
                <h1 class="title-page mb-5"> {{ __('manufacture::trans.manufacture_order') }}</h1>

                <form method="post" action="{{ route('manufacture.store') }}" id="submit-form">
                    @csrf
                    <div class="form_1 autocomplete-target" style="display: flex;">
                        <div class="form-group col-md-3">
                            <input type="text" class="form-control autocomplete" data-manufacturing="1" placeholder="{{ __('manufacture::trans.select_product') }}">
                            <input type="hidden" name="item_id" id="item_id">
                            <div class="load-items" style="width: 100%; position: absolute; z-index: 10; background-color: #fff"></div>
                            <p id="item_qty"> الكمية الحالية </p>
                            <button data-href="{{ route('manufacture.load.materials') }}" id="show-materials-btn" class="btn btn-info">عرض المواد الخام</button>
                        </div>

                        <div class="form-group col-md-2"></div>
                        <div class="form-group col-md-3">
                            <p class="text-center"> <b> ناتج عملية التصنيع للمرة الواحدة </b> </p>
                            <div>
                                <label class="font-weight-bold"> عدد عمليات التصنيع </label>
                                <input type="number" min="1" name="order_qty" id="order_qty" class="form-control" value="1">
                                <span class="text-warning"> ناتج عملية التصنيع للمرة الواحدة 1.00 </span>
                            </div>
                        </div>
                    </div>

                    <div>
                        <table class="table table-striped table-bordered table-hover table-heading">
                            <thead class="bg-info text-white">
                            <th>الصنف</th>
                            <th>الوصف</th>
                            <th>الكمية الحالية</th>
                            <th>المطلوب</th>
                            <th>تكلفة</th>
                            </thead>
                            <tbody id="load-materials"></tbody>
                            <tfoot>
                                <tr>
                                    <td class="font-weight-bold">مخزن صرف الخام</td>
                                    <td class="font-weight-bold">
                                        <select class="form-control" name="store_id" id="store_id">
                                            @foreach($stors as $id => $name)
                                                <option value="{{ $id }}">{{ $name }}</option>
                                            @endforeach
                                        </select>
                                    </td>
                                    <td class="font-weight-bold"></td>
                                    <td class="font-weight-bold">مخزن الإنتاج</td>
                                    <td class="font-weight-bold">
                                        <select class="form-control" name="production_store">
                                            @foreach($stors as $id => $name)
                                                <option value="{{ $id }}">{{ $name }}</option>
                                            @endforeach
                                        </select>
                                    </td>
                                </tr>
                            </tfoot>
                        </table>

                        <div>
                            <div class="row">
                                <div class="col-md-2">
                                    <label class="font-weight-bold" style="cursor: pointer">
                                        <input type="checkbox" name="update_init" checked value="1">
                                        تحديث تعريف التصنيع
                                    </label>
                                </div>

                                <div class="col-md-2">

                                    <label class="font-weight-bold" style="cursor: pointer">
                                        <input type="checkbox" name="update_cost" checked value="1">
                                        تحديث تكلفه الصنف
                                    </label>
                                </div>

                                <div class="col-md-3">
                                    <button type="submit" class="btn btn-info"> تسجيل عملية التصنيع </button>
                                </div>
                            </div>
                        </div>
                    </div>
                </form>
            </div>
        </div>
        @endsection

        @section('js-scripts')
            @include('manufacture::js.search')
            @include('manufacture::js.submit-form-function')
            <script>
                $(function () {
                    $('body').on('click', '#show-materials-btn', function (e) {
                        e.preventDefault();
                        let store_id = $('#store_id').val();
                        let item_id = $('#item_id').val();
                        let qty = $('#order_qty').val();
                        if (store_id == '' || item_id == '' || qty == '') {
                            alert('الرجاء ادخال البيانات');
                            return ;
                        }
                        $('#load-materials').slideUp(function (){ $(this).empty(); });
                        $.ajax({
                            url: $(this).data('href'),
                            type: 'get',
                            data: {id: item_id, order_qty: qty},
                            success: function (response) {
                                $('#load-materials').slideDown(function (){ $(this).append(response) });
                            }
                        });
                    });

                    $('body').on('change', '#order_qty, .material-qty', function () { calcTotal(); });

                    function calcTotal() {
                        let order_qty = Number($('#order_qty').val());
                        let total_cost = 0;
                        $.each( $('#load-materials tr.material-row-details'), function () {
                            let item_qty = Number($(this).find('.material-qty').val());
                            let price = Number($(this).find('.material-qty').data('price'));
                            let cost = item_qty * price;
                            $(this).find('.cost-row').val(cost.toFixed(2));
                            total_cost += cost;
                        });

                        $('#load-materials').find('#total-cost-price').val(total_cost.toFixed(2));
                        $('#load-materials').find('#unit-cost-price').val((total_cost / order_qty).toFixed(2));
                    }

                    $('body').on('click', '.select-item-name', function () {
                        $('#load-items').hide();
                        $('body').find('.autocomplete').val($(this).text());
                        $('body').find('#item_id').val($(this).data('item-id'));
                        $('body').find('#item_qty').text('الكمية الحالية - ' + ($(this).data('item-qty') || 0));
                    });
                });
            </script>
@endsection
