<?php

namespace Modules\Master\Entities;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;

class Instalment extends Model
{
    protected $guarded = [];
    protected $table = 'installments_list';
    public $timestamps = false;
    protected $appends = ['days'];

    public function getDaysAttribute()
    {
        return Carbon::parse($this->installment_date)->diffInDays( now() );
    }

    public function client()
    {
        return $this->belongsTo(Client::class, 'client_id')->select('id', 'client_name as name');
    }

    public function invoice()
    {
        return $this->belongsTo(Invoice::class, 'bill_id')->select('id', 'bill_no', 'client_id');
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('shop', function (Builder $builder) {
            $builder->where('shop_id', auth()->user()->shop_id)->where('installment_date', '<', date('Y-m-d'))->where(['installment_pay' => false]);
        });
    }
}
