<?php

namespace Modules\Master\Http\Controllers;

use App\Models\Item;
use App\Services\Einvoice\Einvoice;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\View;
use Modules\Invoices\Entities\StoreItem;
use Modules\SalePoints\Entities\SalePoint;
use Illuminate\Support\Facades\Auth;
use Modules\Dairy\Entities\Invoice;
use Modules\Invoices\Entities\Invoices;
use Modules\Invoices\Http\Controllers\SalesController;
use Modules\Master\Entities\Alert;
use Modules\Master\Entities\Instalment;
use Modules\Settings\Entities\Badrshop;
use Modules\Store\Entities\Store;

class MasterController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index()
    {
        if (request()->has('test')) {
                $invoice = new Einvoice();
                return $invoice::submitDocuments(Invoice::where('client_id', '>', 0)->first()->id, '');
		}




        $not_submit_einvoices = ['sale' => 0, 'back' => 0];
        if (get_session_val('einvoice', 'integer')) {
            $sales_invoices = clone $back_invoices = Invoices::where('shop_id', get_session_val('serial_id', 'integer'))->whereIn('submissionId', ['', ' ', NULL]);
            $not_submit_einvoices['back'] = $back_invoices->where('invoice_type', 3)->count();
            $not_submit_einvoices['sale'] = $sales_invoices->where('invoice_type', 1)->count();
        }

        $items_expired_soon = Item::join('store_items', 'items.id', 'store_items.item_id')
            ->join('stores', 'stores.id', 'store_items.store_id')
            ->where('items.shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_items.store_quant', '>', 0)
            ->where('items.expire_date', '<', DB::raw('DATE_ADD(CURDATE(), INTERVAL ' . get_session_val('expire_notification', 'Integer') . ' DAY)'))
            ->where('items.expire_date', '!=', '0000-00-00')
            ->whereNotNull('items.expire_date')
            ->count();

        $items_reached_minimum = Item::join('store_items', 'items.id', 'store_items.item_id')
            ->join('units', 'units.id', 'items.unit_id')
            ->join('items_type', 'items.sale_unit', 'items_type.id')
            ->join('stores', 'stores.id', 'store_items.store_id')
            ->where('items.shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_items.store_quant', '<=', DB::raw('items.min_quantity'))
            ->count();

        $point = Auth::user()->sale_point;
        $sale_points = collect();
        if (Auth::user()->show_points == 2) {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->get();
        }
        elseif (Auth::user()->show_points == 3) {
            $store_id = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->first()->store_id;
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_id', $store_id)
            ->get();

        }
        else {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
            ->when(!get_session_val('netowrk_payment', 'Integer'), function($query) {
                $query->where('point_type', 1);
            })->get();
        }

        // if (authorized('can_display_points'))
        // {
        //     $sale_points = SalePoint::where([
        //         ['shop_id',  get_session_val('serial_id', 'Integer')]
        //     ])->when(!get_session_val('netowrk_payment', 'Integer'), function($query) {
        //         $query->where('point_type', 1);
        //     })->limit(3)->get();
        // }
//dd(1231);

        $instalments = Instalment::with('client', 'invoice.client')->limit(10)->get();
        $alerts      = Alert::with('added')->limit(10)->get();
        $shop        = Badrshop::where('serial_id', get_session_val('serial_id', 'Integer'))->first();
        return view('master::index', compact('items_reached_minimum', 'items_expired_soon', 'sale_points','shop', 'not_submit_einvoices', 'instalments', 'alerts') );
    }

    public function unAuthorized($permission)
    {
        return View('master::unAuthorized', compact('permission'));
    }
}
