<?php

namespace Modules\Notification\Http\Controllers;

use App\Mail\ReplyMessage;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Controllers\Controller;
use GuzzleHttp\Psr7\Message;
use Illuminate\Support\Facades\Mail;
use Modules\Notification\Entities\Messages;
use Modules\Notification\Http\Requests\ReplyRequest;

class ContactController extends Controller
{

    public function index()
    {

        $shop_id  = get_session_val('serial_id', 'integer');
        $messages = Messages::where(['type' => 0, 'shop_id' => $shop_id])->orderBy('created_at', 'DESC')->paginate(10);
        return view('notification::contact/inbox', compact('messages'));
    }

    public function outbox()
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $messages = Messages::where(['type' => 1, 'shop_id' => $shop_id])->orderBy('created_at', 'DESC')->paginate(10);
        return view('notification::contact/outbox', compact('messages'));
    }

    public function deleteMessage(Request $request)
    {
        $message = Messages::find($request->request_id);
        if ($message) {
            $message->delete();
            return redirect()->back()->with(['success' => 'تم الحذف']);
        } else {
            return redirect()->back()->with(['error' => "خطأ في عملية الحذف"]);
        }
    }

    public function show($id)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $message = Messages::find($id);
        if ($message) {

            if ($message->type == 0) {

                $replies = Messages::where(['ticket_id' => $message->id, 'shop_id' => $shop_id])->get();
                $original = [];
            } else {
                $replies = collect();
                $original = Messages::find($message->ticket_id);
            }

            return view('notification::contact/show', compact('message', 'replies', 'original'));
        } else {
            abort(404);
        }
    }

    public function reply($id)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $message = Messages::where(['id' => $id, 'shop_id' => $shop_id])->where('type', 0)->first();
        if (is_null($message)) {
            return back();
        }
        return view('notification::contact/reply', compact('message'));
    }

    public function postReply(ReplyRequest $request, $id)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $message = Messages::where(['id' => $id, 'shop_id' => $shop_id])->where('type', 0)->first();
        /**
         * `client_id`, `ticket_reason_id`, `ticket_type_id`,
         * `ticket_status_id`, `description`, `created_at`, `
         * updated_at`, `employee_id`, `shop_id`, `email`, `title`,
         * `type`, `phone`, `address`, `name`
         */
        $reply = new Messages();
        $reply->client_id = 0;
        $reply->ticket_reason_id = 0;
        $reply->ticket_type_id = 0;
        $reply->ticket_status_id = 0;
        $reply->description = $request->description;
        $reply->employee_id = 0;
        $reply->shop_id = $shop_id;
        $reply->email = $request->email;
        $reply->title = $request->title;
        $reply->type = 1;
        $reply->phone = $message->phone;
        $reply->name = $message->name;
        $reply->ticket_id = $message->id;
        $reply->save();

        $header = "From:info@$_SERVER[HTTP_HOST] \r\n";
        $header .= "Cc:info@$_SERVER[HTTP_HOST] \r\n";
        $header .= "MIME-Version: 1.0\r\n";
        $header .= "Content-type: text/html\r\n";

        $email = $request->email;
        $subject = $request->title;
        $msg = $request->description;
        mail($email, $subject, $msg, $header);

        session()->flash('message-sent', 'تم الرد على الرسالة بنجاح');
        return redirect()->route('messages.outbox');
    }
}
