<?php

namespace Modules\Perish\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Http\Controllers\Controller;
use App\Models\systemLog;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Perish\Entities\Item;
use Modules\Perish\Entities\ItemTransaction;
use Modules\Perish\Entities\SalePoint;
use Modules\Perish\Entities\Store;
use Modules\Perish\Entities\StoreItem;
use Modules\Perish\Entities\Unit;
use Modules\Invoices\Entities\InvoiceDetails;
use Modules\Invoices\Entities\Invoices;
use Modules\Perish\Entities\ItemSerial ;
use Modules\WooCommerce\Entities\WooCommerceSecret;


class NewPerishController extends Controller
{

    public function index()
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        if (auth()->user()->change_store == 0) {
            $stores = Store::where('shop_id', $shop_id)
                ->where('id', auth()->user()->store_id)
                ->get();

        } else {
            $stores = Store::where('shop_id', $shop_id)->get();
        }
        $show_item_serial =get_session_val('show_item_serial', 'integer') ;
        $decimal_num_price =get_session_val('decimal_num_price', 'integer') ;

        return view('perish::new-index', compact('stores', 'show_item_serial', 'decimal_num_price'));
    }

    public function storeItems(Request $request)
    {
        $store_id = $request->store_id;
        $shop_id  = get_session_val('serial_id', 'integer');
        // $store_item = StoreItem::where('store_id', $store_id)->where('shop_id', get_session_val('serial_id', 'integer'))->first();
        $data = "";

            $items = StoreItem::join('items', 'items.id', 'store_items.item_id')
                ->join('stores', 'stores.id', 'store_items.store_id')
                ->join('units','items.unit_id',  'units.id')
                ->where('items.shop_id', $shop_id)
                ->where('store_items.store_quant', '>', 0)
                ->where('store_items.store_id', $store_id)
                ->selectRaw('items.id,units.name as unit_name, items.unit_id, items.require_serial, store_items.item_id, store_items.store_quant as quantity, items.pay_price as price, items.item_name')
                ->get();





        $options = '<option selected data-qty="0" data-price="0" data-unit="--" value="0">---</option>';
        $lists = '<li data-original-index="0">
                            <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                            <span class="text">---</span>
                            <span class="glyphicon glyphicon-ok check-mark"></span>
                            </a>
                            </li>';
        if (count($items) > 0) {
            foreach ($items as $item) {

                $item->quantity = round_money($item->quantity);

                $options .= '<option data-qty="' . $item->quantity . '"data-price="' . round_money($item->price) . '" data-unit="' . $item->unit_name . '" data-serial="' . $item->require_serial . '" value="' . $item->id . '">' . $item->item_name . '</option>';
                $lists .= '<li data-original-index="' . $item->id . '">
                            <a tabindex="0" class="" data-tokens="null" role="option" aria-disabled="false" aria-selected="false">
                            <span class="text"> ' . $item->item_name . ' </span>
                            <span class="glyphicon glyphicon-ok check-mark"></span>
                            </a>
                            </li>';
            }

        }

        return response()->json([
            'status' => true,
            'option' => $options,
            'list' => $lists,
        ]);
    }

    public function update(Request $request)
    {
        $data = $request->quantity;

        foreach ($data as $item_id => $ex_val) {
            if ($ex_val > 0) {
                DB::transaction(function () use($item_id, $ex_val, $request){
                $item = StoreItem::where('item_id', $item_id)->where('store_id', $request->store_id)->first();
                $i = Item::findOrFail($item_id);
                $current = $item->store_quant;
                $item->store_quant = $current - $ex_val;
                $item->save();
                $qty_ = $item->store_quant;

                $invoice = Invoices::create([
                    'invoice_date' => date('Y-m-d h:i:s'),
                    'transaction_date' => date('Y-m-d h:i:s'),
                    'add_user' => auth()->user()->id,
                    'store_id' =>  $request->store_id,
                    'notes' => '',
                    'shop_id' => get_session_val('serial_id', 'Integer'),
                    'invoice_type' => 18,
                    'invoice_total_disc' => 0,
                    'invoice_total_disc_type' => 1,
                    'invoice_total_disc_status' => 1,
                    'store_from' =>  $request->store_id,
                    'client_id' => $i->supplier_id
                ]);

                InvoiceDetails::create([
                   'invoice_id' => $invoice->id,
                   'shop_id' => get_session_val('serial_id', 'Integer'),
                   'item_id' => $item_id,
                   'quantity' => $ex_val,
                   'quantity_after' => $qty_,
                   'unit_id' => $i->unit_id,
                   'store_id' => $request->store_id,
                   'pay_price' =>  $i->pay_price,
                   'basic_pay_price' =>  $i->pay_price,
                   'sale_price' =>  $i->sale_price,
                   'basic_sale_price' =>  $i->sale_price,
                ]);

                $show_item_serial = get_session_val('show_item_serial', 'integer');
                if ($show_item_serial == '1' && is_array($request->serials) && sizeof($request->serials) > 0) {
                    foreach ($request->serials as $item_id => $serial) {
                        $s = explode(',', $serial);
                        foreach ($s as $serial_number) {
                            $find = ItemSerial::where([
                                'shop_id' => get_session_val('serial_id', 'integer'),
                                'item_id' => $item_id,
                                'serial' => $serial_number,
                                'store_id' => $request->store_id
                            ])->delete();
                        }
                    }
                }

                });
            }
        }
        $this->createLog(Carbon::now(), "تم اهلاك منتجات ");
        return response()->json([
            'status' => true
        ]);
    }

    public function adminSearch()
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $type = get_session_val('date_type', 'integer');
        $user_sale_point = SalePoint::whereId(auth()->user()->sale_point)->first();
        $user_store = $user_sale_point->store_id;
        $stores = Store::where('shop_id',$shop_id);
        if (auth()->user()->change_store == 0) {
            $stores = $stores->where('id', $user_store);
        }
        $stores = $stores->get();
        return view('perish::report', compact('stores', 'type', 'user_store'));
    }

    public function adminPostSearch(Request $request)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $this->validate($request, [
            'store_id' => 'required',
            'trans_from' => 'required',
            'trans_to' => 'required',
        ]);

        $store_id = $request->store_id;
        $from = $request->trans_from;

        $from = Carbon::parse($from)->format('Y-m-d');

        $to = $request->trans_to;

        $to = Carbon::parse($to)->format('Y-m-d');
        if ($store_id == 0) {
            $store_name = 'المخزن الرئيسي';
        } else {
            $store_ = Store::find($store_id);
            $store_name = $store_->store_name;
        }


        $history = ItemTransaction::where('shop_id', $shop_id)
            ->where('type', 62)
            ->where('store_id', $store_id)
            ->where(function ($q) use ($from) {
                $q->whereDate('tansaction_date', '>=', $from);
            })
            ->where(function ($q) use ($to) {
                $q->whereDate('tansaction_date', '<=', $to);
            })
            ->get();

        $total_qty = $history->sum('quantity');

        return view('perish::report-result', compact('history', 'store_name', 'from', 'to', 'total_qty'));

//        $data = adminSearchTable($issues);

//        return response()->json($data);

    }

    public function searchBarcode(Request $request)
    {
        $shop_id  = get_session_val('serial_id', 'integer');
        $barcode = $request->barcode;
        $store_id = $request->store_id;
        $item = DB::table('items')->where([
            'barcode' => $barcode,
            'items.shop_id' => $shop_id
        ])
        ->join('store_items', 'items.id', '=', 'store_items.item_id')
        ->where('store_items.store_id', $store_id)
        ->select('items.id')
        ->first();

        if ($item) {
            return $item->id;
        }
        return -1;
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}
