<?php

namespace Modules\SalePoints\Entities;

use App\Traits\RoundingAttributesTrait;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;

class SalePoint extends Model
{
    use RoundingAttributesTrait;
    
    public $timestamps = false;
    protected $guarded = ['id'];
    protected $table = 'sale_points';
    protected $fillable = [
        'point_name',
        'money_point',
        'point_type',
        'store_id',
        'currency_id',
        'currency_text',
        'shop_id'
    ];

    public function store()
    {
        return $this->belongsTo(Store::class, 'store_id', 'id')->select('id', 'store_name')->withDefault(['id' => null, 'store_name' => '']);
    }

    public function currency()
    {
        return $this->belongsTo(Currency::class, 'currency_id', 'id')->select('id', 'name')->withDefault(['id' => null, 'name' => get_session_val('currency', 'string'), 'code' => get_session_val('currency', 'string')]);
    }

    public function getCurrentPointMoneyAttribute()
    {
        return $this->money_point . ' ' . $this->currency->code;
    }

    public function getPointTypeNameAttribute()
    {
        return $this->point_type == 1 ? 'الخزن /نقاط البيع' : 'دفع الشبكه';
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('perShop', function (Builder $builder) {
            $builder->where('sale_points.shop_id', auth()->user()->shop_id)->orderBy('id', 'DESC');
        });
    }
}
