<?php

namespace Modules\SalePoints\Http\Controllers;

use App\Models\systemLog;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Modules\SalePoints\Entities\{ClientTransaction, SalePoint, Store, Currency};
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\DB;

class SalePointsController extends Controller
{
    public function index()
    {
        $currencies = Currency::select('id', 'is_main', 'code', 'name')->get();
        $main_currency = Currency::main('code');
        $stores = Store::query()->pluck('store_name', 'id');

        $salePoints = SalePoint::select('id',  'point_name', 'money_point', 'point_type', 'store_id', 'currency_id')
                                ->when(Auth::user()->show_points == 2, function($query) {
                                    $query->where('sale_points.id', auth()->user()->sale_point);
                                })
                                ->when(Auth::user()->show_points == 3, function($query) {
                                    $store_id = SalePoint::select('store_id')->where('id', auth()->user()->sale_point)->first()->store_id ?? 0;
                                    $query->where('sale_points.store_id', $store_id);
                                })
                                ->with('store', 'currency')->get();

        return view('salepoints::index', compact('salePoints', 'stores', 'currencies', 'main_currency'));
    }


    public function getSalePointsTable()
    {
        $currencies = Currency::where('shop_id',  get_session_val('serial_id', 'Integer'))
            ->select('id', 'name')
            ->get();

        $stores = Store::where('shop_id',  get_session_val('serial_id', 'Integer'))
            ->select('id', 'store_name')
            ->get();

        $salePoints = SalePoint::where('sale_points.shop_id',  get_session_val('serial_id', 'Integer'))
            ->leftjoin('stores', 'stores.id', '=', 'sale_points.store_id')
            ->select('sale_points.id',  'sale_points.point_name', 'sale_points.money_point', 'sale_points.point_type',  'stores.store_name')
            ->get();

        return view('salepoints::partials.salepoints_table', compact('salePoints', 'stores', 'currencies'))->render();
    }


    public function create()
    {
        return view('salepoints::create');
    }


    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'point_name' => 'required',
            'money_point' => 'required',
            'point_type' => 'required',
            'store_id' => 'required',
            'currency_id' => 'nullable',
            'currency_text' => 'nullable'
        ]);

        if ($validator->fails()) {
            $alert = [
                'class' => 'alert-danger',
                'title' => 'تنبيه!',
                'message' => 'حدث خطأ ما ، برجاء المحاولة مرة أخرى'
            ];
            return view('salepoints::partials.alert', compact('alert'))->render();
        }

        DB::transaction(function () use ($request) {
            $salePointData = [
                'point_name' => $request->point_name,
                'money_point' => $request->money_point,
                'point_type' => $request->point_type,
                'store_id' => $request->store_id,
                'currency_id' => $request->currency_id ?? 0,
                'currency_text' => Currency::select('code')->find($request->currency_id)->code ?? get_session_val('currency', 'string'),
                'shop_id' => get_session_val('serial_id', 'Integer')
            ];
            $salepoint = SalePoint::create($salePointData);

            $this->createClientTransaction(
                4,
                1,
                1,
                $request->money_point,
                $request->money_point,
                $salepoint->id,
                '',
                ''
            );
        });
        $this->createLog(Carbon::now(), " تم انشاء خزينه ".$request->point_name);
        $alert = [
            'class' => 'alert-primary',
            'title' => 'تنبيه!',
            'message' => 'تم الحفظ بنجاح.'
        ];
        return view('salepoints::partials.alert', compact('alert'))->render();
    }

    public function update(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'point_name' => 'required',
            'money_point' => 'required',
            'point_type' => 'required',
            'currency_id' => 'required',
            'currency_text' => 'required'
        ]);

        if ($validator->fails()) {
            $alert = [
                'class' => 'alert-danger',
                'title' => 'تنبيه!',
                'message' => 'حدث خطأ ما ، برجاء المحاولة مرة أخرى'
            ];
            return view('salepoints::partials.alert', compact('alert'))->render();
        }

        DB::transaction(function () use ($request) {
            if ($request->old_balance != $request->money_point) {

                if ($request->old_balance > $request->money_point) {
                    $amount = $request->old_balance - $request->money_point;
                    $effect = "0";
                } else {
                    $amount = $request->money_point - $request->old_balance;
                    $effect = "1";
                }

                $salePointData = [
                    'point_name' => $request->point_name,
                    'money_point' => $request->money_point,
                    'point_type' => $request->point_type,
                    'currency_id' => $request->currency_id,
                    'currency_text' => $request->currency_text,
                    'shop_id' => get_session_val('serial_id', 'Integer')
                ];

                $this->createClientTransaction(
                    8,
                    $effect,
                    1,
                    $amount,
                    $request->money_point,
                    $request->id,
                    '',
                    ''
                );
            } else {
                $salePointData = [
                    'point_name' => $request->point_name,
                    'point_type' => $request->point_type,
                    'currency_id' => $request->currency_id,
                    'currency_text' => $request->currency_text,
                    'shop_id' => get_session_val('serial_id', 'Integer')
                ];
            }
            SalePoint::where([
                'id' => $request->id,
                'shop_id' => get_session_val('serial_id', 'Integer')
            ])
                ->update($salePointData);
        });

        $alert = [
            'class' => 'alert-primary',
            'title' => 'تنبيه!',
            'message' => 'تم التعديل بنجاح.'
        ];
        return view('salepoints::partials.alert', compact('alert'))->render();
    }


    public function destroy(Request $request)
    {
        $isDeleted = 0;
        DB::transaction(function () use ($request, &$isDeleted) {
            $salePointTransactions = ClientTransaction::where('shop_id',  get_session_val('serial_id', 'Integer'))
                ->where('safe_point_id', $request->id)
                ->whereNotIn('type', ['4','11, 22, 30, 31, 32, 33, 34, 63, 64'])
                ->orWhere('effected_point', $request->id)
                ->count();

            if ($salePointTransactions  == 0) {
                $salePoint = SalePoint::find($request->id);
                $salePoint->delete();
                $isDeleted = 1;
                $this->createLog(Carbon::now(), " تم حذف خزينه ".$salePoint->point_name);
            }
        });

        $alert = [
            'class' => $isDeleted ? 'alert-primary' : 'alert-danger',
            'title' => 'تنبيه!',
            'message' =>  $isDeleted ? 'تم الحذف بنجاح.' : 'لا يمكن مسح الخزنه لا يوجد عليها عمليات.'
        ];
        return view('salepoints::partials.alert', compact('alert'))->render();
    }


    public function getSalepointModalEditBody(Request $request)
    {
        $salePoint = SalePoint::find($request->id);
        $stores = Store::where('shop_id',  get_session_val('serial_id', 'Integer'))
            ->select('id', 'store_name')
            ->get();
        $currencies = Currency::where('shop_id',  get_session_val('serial_id', 'Integer'))
            ->select('id', 'name')
            ->get();
        return view('salepoints::partials.editModal.salepoints_edit_modal_body', compact('salePoint', 'stores', 'currencies'))->render();
    }


    public function getSalepointModalTransferBody(Request $request)
    {
        $salePoint = SalePoint::where([
            'sale_points.shop_id' => get_session_val('serial_id', 'Integer'),
            'sale_points.id' => $request->id
        ])
            ->leftjoin('stores', 'stores.id', '=', 'sale_points.store_id')
            ->where('sale_points.shop_id',  get_session_val('serial_id', 'Integer'))
            ->select('sale_points.id', 'sale_points.point_name', 'sale_points.money_point', 'stores.store_name', 'stores.id as store_id')->first();


        $allSalePoints = SalePoint::where([
            ['shop_id',  get_session_val('serial_id', 'Integer')],
            ['id', '!=', [$salePoint->id]]
        ])
            ->select('id', 'point_name', 'money_point')
            ->get();

        return view('salepoints::partials.transferModal.salepoints_transfer_modal_body', compact('salePoint', 'allSalePoints'))->render();
    }


    public function transferMoney(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'salepoint_id' => 'required',
            'point_to' => 'required',
            'transfer_amount' => 'required|numeric|min:0'
        ]);

        if ($validator->fails()) {
            return $validator->errors();
        }

        if (get_session_val('negative_money', 'Integer') == 0) {
            $safe_money = SalePoint::where([
                ['shop_id',  get_session_val('serial_id', 'Integer')],
                ['id',  $request->salepoint_id]
            ])->first()->money_point;

            if ($safe_money < 0 || $safe_money - $request->transfer_amount < 0) {
                return response()->json([
                    'response_status' => -1
                ]);
            }
        }

        DB::transaction(function () use ($request) {
            $fromNewBalance = $this->updateSalePoint($request->salepoint_id, $request->transfer_amount, 0);
            $this->createClientTransaction(
                11,
                0,
                '',
                $request->transfer_amount,
                $fromNewBalance,
                $request->salepoint_id,
                $request->point_to,
                $request->transfer_notes
            );

            $toNewBalance = $this->updateSalePoint($request->point_to, $request->transfer_amount, 1);
            $this->createClientTransaction(
                11,
                1,
                '',
                $request->transfer_amount,
                $toNewBalance,
                $request->point_to,
                $request->salepoint_id,
                $request->transfer_notes
            );
            $salepoint_from = SalePoint::where([
                ['shop_id',  get_session_val('serial_id', 'Integer')],
                ['id',  $request->salepoint_id]
            ])->first();
            $salepoint_to = SalePoint::where([
                ['shop_id',  get_session_val('serial_id', 'Integer')],
                ['id',  $request->point_to]
            ])->first();
            $this->createLog(Carbon::now(),"تم التحويل من خزينه " . $salepoint_from->point_name. " الي " . $salepoint_to->point_name);
        });

        return response()->json([
            'response_status' => 1
        ]);
    }


    public function updateSalePoint($salePointId, $amount, $effect)
    {
        $salePoint = SalePoint::find($salePointId);
        if ($effect == 1) {
            $salePoint->money_point += $amount;
        } else {
            $salePoint->money_point -= $amount;
        }
        $salePoint->save();
        return $salePoint->money_point;
    }


    public function createClientTransaction(
        $type,
        $effect,
        $safe_type,
        $amount,
        $safe_balance,
        $safe_point_id,
        $effected_point,
        $notes
    ) {
        $clientTransactionData = [
            'type' => $type,
            'effect' => $effect,
            'safe_type' => $safe_type,
            'amount' => $amount,
            'safe_balance' => $safe_balance,
            'safe_point_id' => $safe_point_id,
            'date_time' => now(),
            'pay_day' => now(),
            'effected_point' => $effected_point,
            'notes' => $notes,
            'user_id' => get_session_val('user_id', 'Integer'),
            'shop_id' => get_session_val('serial_id', 'Integer'),
        ];
        ClientTransaction::create($clientTransactionData);
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}
