<?php

namespace Modules\SalesReport\Entities;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;

class ClientTransaction extends Model
{
    protected $table = 'client_transaction';
    protected $guarded = ['id'];
    public $timestamps = false;

    public function collector()
    {
        return $this->belongsTo(Employee::class, 'collector_id', 'id')->select('id', 'name', 'collecting_ratio')
                    ->where('collecting', true)->withDefault(['name' => '', 'collecting_ratio' => 0]);
    }

    public function getCollectedRatio()
    {
        return ($this->amount_collected / $this->amount) * 100;
    }

    public function client()
    {
        return $this->belongsTo(Client::class, 'client_id', 'id')->select('id', 'client_name as name')->whereIn('client_role', [0,2]);
    }

    public function invoice()
    {
        return $this->belongsTo(Invoice::class, 'bill_id', 'id')->select('id', 'bill_no')->where('shop_id', auth()->user()->shop_id);
    }

    public function scopeFilter($query)
    {
        return $query->when(request()->get('mandob_id'), function($query) {
                        $query->where('collector_id', request()->get('mandob_id'));
                    })->when(request()->get('from'), function($query) {
                        $query->where('pay_day', '>=', request()->get('from'));
                    })->when(request()->get('to'), function($query) {
                        $query->where('pay_day', '<=', request()->get('to'));
                    });
    }

    public function getNotes()
    {
        return $this->invoice ? "سداد مرتبط بفاتورة رقم <a target='_blank' href='".route('invoices.show', $this->bill_id)."'>{$this->invoice?->bill_no}</a>"  : '';
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('shop', function (Builder $builder) {
            $builder->where('shop_id', auth()->user()->shop_id)->whereIn('type', [2])->orderBy('date_time', 'DESC');
        });
    }
}
