<?php

namespace Modules\SalesReport\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Invoices\Entities\Invoices;

class CategoryProfitController extends Controller
{
    public function __invoke()
    {
        $item_units = DB::table('items_type')
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->selectRaw('id, name')
            ->get();
        $suppliers = DB::table('clients')
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereIn('client_role', [1, 2])
            ->selectRaw('id, client_name')
            ->get();

        if (request()->has('unit') && request()->has('from') && request()->has('to')) {
            $items = $this->filter();
        }

        return view('salesreport::category_profit.unit-items', get_defined_vars());
    }

    public function filter()
    {
        $from = request('from');
        $to =request('to');

        if (! $from)
        {
            $from = date('Y-m-d');
        }

        if (! $to)
        {
            $to = date('Y-m-d');
        }

        $query = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
        ->leftJoin('invoices_details_vats', function($join) {
            $join->on('invoices_details_vats.item_id', '=', 'invoices_details.item_id')
                    ->on('invoices_details_vats.invoice_id', '=', 'invoices_details.invoice_id');
        })
        ->leftJoin('items', 'items.id', 'invoices_details.item_id')
        ->leftJoin('items_type', 'items_type.id', 'items.sale_unit')
        ->when( !in_array(request('unit'), ['', -1, null]), function($q){
            $q->where('items.sale_unit', request('unit'));
        })
        ->when( !in_array(request('supplier'), ['', -1, null]), function($q){
            $q->where('items.supplier_id', request('supplier'));
        })
        ->when( ! in_array(request('store'), ['', -1, null]), function($q){
            $q->where('invoices_.store_id' , request('store'));
        })
        ->where('invoices_.shop_id' , get_session_val('serial_id', 'integer'))
        ->whereDate('invoices_.transaction_date', '>=', $from)
        ->whereDate('invoices_.transaction_date', '<=', $to)
        ->selectRaw("
            SUM(invoices_details_vats.vat_weight) as vat_weight,
            items.vat_state,
            items.sale_unit AS category_id,
            items_type.name AS category_name,
            IfNull( sum(
                (
                    (invoices_details.price_without_vat * invoices_details.quantity) + ifNull(
                        (
                            select sum(vat_weight) from invoices_details_vats
                            where invoices_details_vats.invoice_id = invoices_details.invoice_id
                            AND invoices_details_vats.unit_id = invoices_details.unit_id
                            AND invoices_details_vats.item_id = invoices_details.item_id
                        ),
                        0
                    )
                ) *
            CASE
                WHEN invoices_.invoice_type = 1 THEN 1
                ELSE 0
                END
            ), 0) AS total_sale_price,

            IfNull( sum(
                ( (invoices_details.pay_price * invoices_details.quantity) + invoices_details.pay_vats_weight ) *
            CASE
                WHEN invoices_.invoice_type = 1 THEN 1
                ELSE 0
                END
            ), 0) AS total_sale_pay_price,

            IfNull( sum(
                (
                    (invoices_details.price_without_vat * invoices_details.quantity) + ifNull(
                        (
                            select sum(vat_weight) from invoices_details_vats
                            where invoices_details_vats.invoice_id = invoices_details.invoice_id
                            AND invoices_details_vats.unit_id = invoices_details.unit_id
                            AND invoices_details_vats.item_id = invoices_details.item_id
                        ),
                        0
                    )
                ) *
            CASE
                WHEN invoices_.invoice_type = 3 THEN 1
                ELSE 0
                END
            ), 0) AS total_return_sale_price,

            IfNull( sum(
                ( (invoices_details.pay_price * invoices_details.quantity) + invoices_details.pay_vats_weight ) *
            CASE
                WHEN invoices_.invoice_type = 3 THEN 1
                ELSE 0
                END
            ), 0) AS total_return_sale_pay_price,

            IfNull( sum(
                invoices_details.pay_price * invoices_details.quantity *
            CASE
                WHEN invoices_.invoice_type = 18 THEN 1
                WHEN invoices_.invoice_type = 23 THEN -1
                ELSE 0
                END
            ), 0) AS total_perish_price,

            items.item_name
        ")
        ->groupBy("items.sale_unit")
        ->get();
        // ->toSql();
        // dd($query);
        return $query;
    }
}
