<?php

namespace Modules\SalesReport\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Clients\Entities\Client;
use Modules\Debentures\Entities\SalePoint;
use Modules\Invoices\Entities\Invoices;

class InstalledInvoicesController extends Controller
{

    public function sales(Request $request)
    {
        $clients = Client::where(['shop_id' => get_session_val('serial_id', 'integer')] )->whereIn('client_role', [0, 2])->orderBy('client_name')->get(['id', 'client_name']);

        $invoices = $this->getInvoices($request, 'sales');

        $point = Auth::user()->sale_point;
        if (Auth::user()->point_change == 0) {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->get();
        } else {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        }

        return view('salesreport::installed_invoices.sales', compact('clients', 'invoices', 'sale_points'));
    }

    public function purchases(Request $request)
    {
        $clients = Client::where(['shop_id' => get_session_val('serial_id', 'integer')] )->whereIn('client_role', [1, 2])->orderBy('client_name')->get(['id', 'client_name']);

        $invoices = $this->getInvoices($request, 'purchases');

        $point = Auth::user()->sale_point;
        if (Auth::user()->point_change == 0) {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->get();
        } else {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        }

        return view('salesreport::installed_invoices.purchases', compact('clients', 'invoices', 'sale_points'));
    }

    protected function getInvoices(Request $request, string $type = 'purchases')
    {
        $dates['start'] = $request->start_date ? $request->start_date : Carbon::today()->format('Y-m-d');
        $dates['end'] = $request->end_date ? $request->end_date : Carbon::today()->format('Y-m-d');

        $invoices = Invoices::where('shop_id', get_session_val('serial_id', 'integer'))->ofType($type)->betweenTwoDates($dates); // ->with('client');
        $invoices = $request->invoice_no ? $invoices->where('bill_no',$request->invoice_no) : $invoices;
        $invoices = $request->client_id > -1 ? $invoices->byClient($request->client_id) : $invoices;
        $invoices = $invoices->HaveRest();
        $invoices = $invoices->with('client')->withCount('installments')->paginate(20);
        return $invoices;
    }

    // public function makePayment($type, Request $request)
    // {
    //     $invoice = Invoices::with('client')->find($request->invoice_id);
    //     // dd($invoice);
    //     if ($request->all()) {
    //         $x = DB::transaction(function () use ($request) {
    //             $client = $invoice->client;
    //             $old_balance = $client->balance;

    //             $mandob = $client->client_type != false
    //                         ? Employee::select('id', 'collecting_ratio')->find($client->collector_id)
    //                         : new Employee();

    //             $total_payment = $request->payment;
    //             $discount = $request->discount;

    //             $payment = $total_payment - $discount;

    //             $client->balance = $client->balance - $payment;
    //             $client->save();


    //             $sale_point = SalePoint::findOrFail($request->sale_point);

    //             $old_safe_balance = $sale_point->money_point;

    //             $sale_point->money_point = $sale_point->money_point + $payment;
    //             $sale_point->save();

    //             $new_safe_balance = $sale_point->money_point;

    //             $bill_no = 0;
    //             if ($request->has('for_bill') && $request->bill_id > 0) {
    //                 $bill = Invoices::where([
    //                     'shop_id' => \auth()->user()->shop_id,
    //                     'bill_no' => $request->bill_no,
    //                     'client_id' => $client->id
    //                 ])->whereDate('invoice_date', $request->bill_date)->find($request->bill_id);

    //                 if ($bill) {
    //                     $bill_no = $request->bill_id;
    //                     Log::create(Carbon::now(), 'تعديل  الرصيد علي العميل    ' . $client->client_name  . ' من خلال تسجيل ايصال استلام   ' .
    //                         ' من فرق تعديل الصافي والمدفوع [ ( ' . price_decimal($bill->net_price) . ' - ' . price_decimal($bill->payment) . ' ) => (' . price_decimal($bill->net_price) . ' - ' . price_decimal($bill->payment + $payment)  . ') ]' .
    //                         ' وتعديل  رصيد الخزنة من فرق المدفوع '  . ' [ ' . price_decimal($bill->payment) . ' => ' . price_decimal($bill->payment + $payment) .
    //                         ' ] من فاتورة مبيعات '  . $bill->id . '[ ' . $bill->bill_no . ' ] ' .  ' بتاريخ  ' . Carbon::now() . ' ' . date('H:i:s'), $bill->id, 'sales');
    //                     $bill->payment += $payment;
    //                     $bill->rest = $bill->bill_total - $bill->payment;
    //                     // $bill->bil_payment += $payment;
    //                     $bill->save();

    //                     // if ($bill->payment >= $bill->net_price) {
    //                     //     $bill->pay_stat = 1;
    //                     //     $bill->save();
    //                     // }
    //                 }
    //             }


    //             $transaction = new ClientTransaction();
    //             $transaction->collector_id = $mandob->id;
    //             $transaction->amount_collected = $payment * ($mandob->collecting_ratio / 100);
    //             $transaction->date_time = Carbon::now();
    //             $transaction->client_id = $request->client;
    //             $transaction->amount = $payment;
    //             $transaction->type = 2;
    //             $transaction->effect = 1;
    //             $transaction->bill_id = $bill_no;
    //             $transaction->pay_day = $request->date_day;
    //             $transaction->balance = $old_balance - $payment;
    //             $transaction->notes = $request->notes;
    //             $transaction->safe_type = $request->sale_point;
    //             $transaction->safe_balance = $new_safe_balance;
    //             $transaction->safe_point_id = $request->sale_point;
    //             $transaction->user_id = Auth::user()->id;
    //             $transaction->shop_id = get_session_val('serial_id', 'integer');
    //             $transaction->save();


    //             if ($discount > 0) {

    //                 $old_balance = $client->balance;
    //                 $client->balance = $client->balance - $payment;
    //                 $client->save();

    //                 $transaction = new ClientTransaction();
    //                 $transaction->collector_id = $collector_id;
    //                 $transaction->date_time = Carbon::now();
    //                 $transaction->client_id = $request->client;
    //                 $transaction->amount = $discount;
    //                 $transaction->type = 60;
    //                 $transaction->effect = 1;
    //                 $transaction->bill_id = $bill_no;
    //                 $transaction->pay_day = $request->date_day;
    //                 $transaction->balance = $old_balance - $discount;
    //                 $transaction->notes = $request->notes;
    //                 $transaction->safe_type = null;
    //                 $transaction->safe_balance = null;
    //                 $transaction->safe_point_id = null;
    //                 $transaction->user_id = Auth::user()->id;
    //                 $transaction->shop_id = get_session_val('serial_id', 'integer');
    //                 $transaction->save();

    //                 $payment = $discount;

    //                 Log::create(Carbon::now(), 'تعديل  الرصيد علي العميل    ' . $client->client_name  .
    //                     ' بتسجيل رصيد معدوم بقيمة  ' . $discount . ' بتاريخ  ' . Carbon::now() . ' ' . date('H:i:s'), 0, '');
    //             }

    //             return $transaction->id;
    //         });
    //         if ($x > 0) {

    //             $url = route('debentures.print', $x);

    //             return back()->with(['success' => 'تم الحفظ بنجاح', 'printed_url' => $url]);
    //         } else {
    //             return back()->with(['error' => 'حدث خطأ اثناء الحفظ']);
    //         }
    //     }
    // }
}

