<?php

namespace Modules\SalesReport\Http\Controllers;

use App\Models\Item;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Invoices\Entities\Invoices;

class ItemProfitController extends Controller
{
    public function __invoke()
    {
        if (request()->has('item') && request()->has('from') && request()->has('to')) {
            $item = $this->profitFilter();
            $item ??= Item::findOrFail(request('item'));        // if no filtered item make item = request('item')
        }
        return view('salesreport::item_profit.profit', get_defined_vars());
    }

    public function profitFilter()
    {
        $from = request('from');
        $to =request('to');

        if (! $from)
        {
            $from = date('Y-m-d');
        }

        if (! $to)
        {
            $to = date('Y-m-d');
        }

        $query = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                ->leftJoin('items', 'items.id', 'invoices_details.item_id')
                ->where([
                    ['invoices_details.item_id' , request('item')],
                    ['invoices_.shop_id' , get_session_val('serial_id', 'integer')]
                ])
                ->whereDate('invoices_.transaction_date', '>=', $from)
                ->whereDate('invoices_.transaction_date', '<=', $to)
                ->when( ! in_array(request('store'), ['', -1, null]), function($q){
                    $q->where('invoices_.store_id' , request('store'));
                })
                ->selectRaw("
                
                    invoices_.id, 
                    
                    IfNull( sum(
                        (
                            (invoices_details.item_price_with_discount * invoices_details.quantity) + ifNull(
                                (
                                    select sum(vat_weight) from invoices_details_vats
                                    where invoices_details_vats.invoice_id = invoices_details.invoice_id  
                                    AND invoices_details_vats.unit_id = invoices_details.unit_id 
                                    AND invoices_details_vats.item_id = invoices_details.item_id 
                                ),
                                0
                            )
                        ) * 
                    CASE
                        WHEN invoices_.invoice_type = 1 THEN 1
                        ELSE 0
                        END
                    ), 0) AS total_sale_price,
                    
                    IfNull( sum(
                        ( (invoices_details.pay_price * invoices_details.quantity) + invoices_details.pay_vats_weight ) * 
                    CASE
                        WHEN invoices_.invoice_type = 1 THEN 1
                        ELSE 0
                        END
                    ), 0) AS total_sale_pay_price,
                    
                    IfNull( sum(
                        (
                            (invoices_details.item_price_with_discount * invoices_details.quantity) + ifNull(
                                (
                                    select sum(vat_weight) from invoices_details_vats
                                    where invoices_details_vats.invoice_id = invoices_details.invoice_id  
                                    AND invoices_details_vats.unit_id = invoices_details.unit_id 
                                    AND invoices_details_vats.item_id = invoices_details.item_id 
                                ),
                                0
                            )
                        ) * 
                    CASE
                        WHEN invoices_.invoice_type = 3 THEN 1
                        ELSE 0
                        END
                    ), 0) AS total_return_sale_price,
                    
                    IfNull( sum(
                        ( (invoices_details.pay_price * invoices_details.quantity) + invoices_details.pay_vats_weight ) * 
                    CASE
                        WHEN invoices_.invoice_type = 3 THEN 1
                        ELSE 0
                        END
                    ), 0) AS total_return_sale_pay_price,

                    IfNull( sum(
                        invoices_details.pay_price * invoices_details.quantity *  
                    CASE
                        WHEN invoices_.invoice_type = 18 THEN 1
                        WHEN invoices_.invoice_type = 23 THEN -1
                        ELSE 0
                        END
                    ), 0) AS total_perish_price,

                    items.item_name
                
                ")
                ->first();
        
        return $query;
    }
}
