<?php

namespace Modules\SalesReport\Http\Controllers;

use COM;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Facades\Excel;
use App\Exports\ItemTransactionReport;
use Modules\SalesReport\Entities\Item;
use Illuminate\Support\Facades\Storage;
use Modules\SalesReport\Entities\Store;
use Modules\SalesReport\Entities\Client;
use Modules\SalesReport\Entities\Factory;
use Modules\SalesReport\Entities\Invoice;
use function PHPUnit\Framework\returnSelf;
use Modules\SalesReport\Entities\BadrShop;
use Modules\SalesReport\Entities\ItemType;
use Modules\SalesReport\Entities\Supplier;
use Modules\SalesReport\Entities\PriceList;
use Modules\Invoices\Entities\Invoices;
use Modules\SalesReport\Entities\SalePoint;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Database\Eloquent\Collection;
use Modules\SalesReport\Entities\InvoiceDetail;
use Modules\SalesReport\Entities\Unit;
use Illuminate\Pagination\Paginator;
use App\DataTables\InvoiceDataTable;
use App\Models\User;
use Modules\BadrRequest\Entities\ItemUnit;
use Modules\Item\Http\Controllers\ItemController;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx\Rels;

class ItemReportController extends Controller
{
    /**
     * Item Transaction Report
     * @return Response
     */
    public function itemTransaction(Request $request)
    {
        $user = User::Find(auth()->user()->id);
        $groupIds = $user->groups->pluck('id');
        $shop_id            = get_session_val('serial_id', 'integer');
        $items              = Item::where('shop_id', $shop_id)->get();
        $clients = clientGroups()->get();
        $stores             = Store::where('shop_id', $shop_id)->get();
        $settings           = BadrShop::where('serial_id', $shop_id)->first();
        $decimal_num_quant  = $settings->decimal_num_quant;
        $decimal_num_price  = $settings->decimal_num_price;
        $details_link       = "";
        $item_total_price   = 0;
        $total_price_income = 0;
        $total_qant_income  = 0;
        $total_qant_sale    = 0;
        $total_price_sale   = 0;
        $client_name        = "";
        $results            = Invoice::join('invoices_details', 'invoices_details.invoice_id', '=', 'invoices_.id')
            ->join('items', 'items.id', '=', 'invoices_details.item_id')
            ->leftJoin('invoices_details_vats', function($join) {
                $join->on('invoices_details_vats.item_id', '=', 'invoices_details.item_id');
                $join->on('invoices_details_vats.invoice_id', '=', 'invoices_details.invoice_id');
            })
            ->leftJoin('clients as suppliers', 'suppliers.id', '=', 'invoices_.add_user')
            ->join('clients', function ($join) use ($groupIds) {
                if (auth()->user()->clients_group == 0) {
                    $join->on('clients.id', '=', 'invoices_.client_id');
                } else {
                    $join->on('clients.id', '=', 'invoices_.client_id');
                    $join->whereIn('clients.group_id', $groupIds);
                }
            })
            ->leftJoin('stores as storeFrom', 'storeFrom.id', '=', 'invoices_.store_from')
            ->leftJoin('stores as storeTo', 'storeTo.id', '=', 'invoices_.store_to')
            ->leftJoin('stores as mainStore', 'mainStore.id', '=', 'invoices_.store_id')
            ->leftJoin('users', 'users.id', 'invoices_.add_user')
            ->leftJoin('items_unit', function ($q) {
                $q->on('items_unit.item_id', 'invoices_details.item_id');
                $q->on('items_unit.unit_id', 'invoices_details.unit_id');
            })
            ->where('invoices_.shop_id', $shop_id)
            ->orderBy('invoices_details.id', 'asc')
            ->selectRaw('DISTINCT invoices_.id ,invoices_.invoice_type as type ,invoices_.store_id ,invoices_.bill_no ,invoices_.transaction_date,
                users.name As user_name, order_no,
                mainStore.store_name,
                invoices_.store_from as sf, invoices_.store_to as st , suppliers.client_name as supplier_name, suppliers.id as supplier_id,
                clients.client_name, clients.id as client_id, items.item_name, items.vat_state, invoices_.invoice_date, invoices_details.invoice_id,
                SUM( round(invoices_details.quantity * if (items_unit.id > 0, items_unit.unit_value, 1)  ,' . $decimal_num_quant . ') ) as quantity ,
                round(invoices_details.quantity_after,' . $decimal_num_quant . ') as new_quantity,
                round((invoices_details.quantity + invoices_details.quantity_after) ,' . $decimal_num_quant . ') as old_quantity,
                if (FALSE, SUM(invoices_details_vats.vat_weight), 0) AS vat_weight,
                SUM( round(invoices_details.price_without_vat * invoices_details.quantity ,' . $decimal_num_price . ') ) as price')
            ->groupBy('invoices_details.invoice_id');

        if ($request->has('item_id') && !in_array($request->item_id, ['', ' ', null])) {
            $results = ($request->item_id != -1) ? $results->where('invoices_details.item_id', $request->item_id) : $results;
        }

        if ($request->has('operation_list') && !in_array($request->operation_list, ['', ' ', null])) {
            $results = ($request->operation_list != -1) ? $results->where('invoices_.invoice_type', $request->operation_list) : $results;
        }

        if ($request->has('client_id') && !in_array($request->client_id, ['', ' ', null])) {
            $results = ($request->client_id != -1) ? $results->where('invoices_.client_id', $request->client_id) : $results;
        }

        if ($request->has('store_list') && !in_array($request->store_list, ['', ' ', null])) {
            $results = ($request->store_list != -1) ? $results->where('invoices_.store_id', $request->store_list) : $results;
        }

        if ($request->has('date_start') && !in_array($request->date_start, ['', ' ', null])) {
            $results->where(function ($query) use ($request) {
                $query->whereDate('invoices_.transaction_date', '>=', $request->date_start);
                //->orWhereDate('invoices_.invoice_date', '>=', $request->date_start);
            });
        }

        if ($request->has('date_end') && !in_array($request->date_end, ['', ' ', null])) {
            $results->where(function ($query) use ($request) {
                $query->whereDate('invoices_.transaction_date', '<=', $request->date_end);
                //->orWhereDate('invoices_.invoice_date', '<=', $request->date_end);
            });
        }

        $results = !empty($request->all()) ? $results->get() : collect();

        foreach ($results as $key => $value) {
            switch ($value->type) {
                case '0': // مشتريات
                    $total_qant_income  += $value->quantity;
                    $bill_no = $value->bill_no;
                    $item_total_price    = $value->price;
                    $total_price_income += $item_total_price;
                    $client_name         = $value->supplier_name;
                    $value->type = 3;
                    break;
                case '1': // مبيعات
                    $total_qant_sale  += $value->quantity;
                    $item_total_price  = $value->price;
                    $bill_no = $value->bill_no;
                    $total_price_sale += $value->price;
                    $client_name       = $value->client_name;
                    $value->type = 1;
                    break;
                case '2': // مرتجع مشتريات
                    $total_qant_income  -= $value->quantity;
                    $item_total_price    = $value->price;
                    $total_price_income -= $item_total_price;
                    $bill_no = $value->bill_no;
                    $client_name         = $value->client_name;
                    $value->type = 5;
                    break;
                case '3': // مرتجع مبيعات
                    $total_qant_sale  -= $value->quantity;
                    $item_total_price  = $value->price;
                    $total_price_sale -= $item_total_price;
                    $bill_no = $value->bill_no;
                    $client_name       = $value->client_name;
                    $value->type = 0;
                    break;
                default:
                    $type_id = "";
                    $bill_no = "";
                    break;
            }

            // Store Transaction
            if ($value->type == 10) {
                $store_name1 = Store::find($value->sf)->store_name ?? ' ';
                $store_name2 = Store::find($value->st)->store_name ?? ' ';

                if ($value->old_quantity < $value->new_quantity) {
                    $store_trans = __('reports.convert_to_store');
                    $store_to    = __('reports.from');
                } else {
                    $store_trans = __('reports.convert_from_store');
                    $store_to    = __('reports.to');
                }

                $details_link = $store_trans . ' <strong> ' . $store_name1 . '  </strong>  ' . $store_to . '  <strong>  ' . $store_name2 . ' رقم إذن الصرف ' . $value->order_no . ' </strong> ';
            } else if ($value->type == 7) {
                $details_link = __('native.new_item_qty');
            } else if ($value->type == 9) {
                $details_link = __('native.edit_amount');
            } else if ($value->type == 18) {
                $details_link = __('perish.perish-add');
            } else if ($value->type == 23) {
                $details_link = __('perish.replace');
            } else {
                $data = [
                    'invoice_no' => $value->bill_no,
                    'invoice_id' => $value->id,
                ];
                $value->invoice_no = $value->bill_no;
                $details_link = itemReport($value, getBillNo(collect($data)));
            }
            $results[$key]->details_link = $details_link;
        }

        return view('salesreport::item_report.item_transaction', compact(
            'items',
            'clients',
            'stores',
            'item_total_price',
            'results',
            'details_link',
            'total_qant_sale',
            'total_price_sale',
            'total_qant_income',
            'total_price_income',
        ));
    }
    /**
     * Item Inventory Report
     * @return Response
     */
    public function ItemInventory(Request $request)
    {
        $shop             = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $suppliers        = Client::whereIn('client_role', [1, 2])->select('id', 'client_name as supplier_name')->where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $user_sale_point  = SalePoint::whereId(auth()->user()->sale_point)->first();
        $categories       = ItemType::select('id', 'name')->where('shop_id', get_session_val('serial_id', 'integer'))->orderBy('name')->get();
        $prices_list      = PriceList::select('id', 'list_name')->where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $stores           = Store::select('id', 'store_name')->where('shop_id', get_session_val('serial_id', 'integer'));
        $today            = Carbon::now();
        $color_size       = $shop->color_size;
        $multi_price      = $shop->multi_price;
        $item_factory     = $shop->item_factory;
        $item_code        = $shop->item_code;
        $show_item_serial = $shop->show_item_serial;
        $user_store       = $user_sale_point->store_id;
        $factories        = $item_factory == 1 ? Factory::where('shop_id', auth()->user()->shop_id)->select('id', 'name')->get() : collect();
        $stores           = auth()->user()->change_store == 0 ? $stores->where('id', $user_store)->get() : $stores->get();
        $total_pay        = 0;
        $total_sale       = 0;
        $net              = 0;
        $total_qty        = 0;
        $items            = collect();

        if (auth()->user()->change_store == 1) {
            if ($request->store_id > '0') {
                $items = $this->getAllStoresItems($request, $shop, $user_store);
            } elseif ($request->store_id < '0' || $request->all() == []) {
                $items = $this->getAllStoresItems($request, $shop, $user_store);
                $all_count  = $items->count();
            }
        } else {
            $items = $this->getAllStoresItems($request, $shop, $user_store);
        }

        $items = $items->sortBy(function ($value) use ($request) {
            return $request->order != '' ? $value[$request->order] : $value['item_name'];
        });

        return view('salesreport::item_report.item_inventory', compact(
            'show_item_serial',
            'suppliers',
            'stores',
            'categories',
            'items',
            'today',
            'total_pay',
            'total_sale',
            'net',
            'total_qty',
            'color_size',
            'prices_list',
            'multi_price',
            'user_store',
            'factories',
            'item_factory',
            'item_code'
        ));
    }

    protected function getMainStoreItems($request, $shop)
    {
        $append = 'items.add_user';
        $items = Item::where([
            'items.shop_id' => get_session_val('serial_id', 'integer'),
            'available' => 1,
            'is_service' => 0
        ])->leftJoin('items_type', 'items.sale_unit', '=', 'items_type.id');
        if ($shop->color_size == 1) {
            $items = $items->leftJoin('sizes', 'items.size_id', '=', 'sizes.id');
            $append = 'sizes.size_name';
        }
        $factory_name = \DB::raw(' null as factory_name');
        if ($shop->item_factory == 1) {
            $factory_name = 'factories.name as factory_name';
        }
        $serials_count = \DB::raw(' null as serials_count');
        if ($shop->show_item_serial == 1) {
            $items = $items->leftJoin('item_serials', 'items.id', '=', 'item_serials.item_id');
            $serials_count = \DB::raw(' COUNT(item_serials.id) as serials_count');
        }

        $items = $items->select(
            \DB::raw('0 as store_name'),
            'items_type.name as type_name',
            'items.sale_unit',
            'items.quantity',
            'items.item_name',
            'items.about',
            'items.barcode',
            'items.sale_price',
            'items.pay_price',
            'items.require_serial',
            'items.id',
            'items.sale_code',
            'items.purchase_code',
            $append,
            $factory_name,
            $serials_count
        );


        $items = $this->filter($items, $request, $shop);
        return $items->groupBy('items.id')->get();
    }

    protected function getAllStoresItems($request, $shop, $user_store)
    {
        $append = 'items.add_user';
        $items = Item::where([
            'items.shop_id' => get_session_val('serial_id', 'integer'),
            'available'     => 1
        ])->where(function ($query) {
            $query->where('is_service', 0)->orWhere('is_service', null);
        })
            ->leftJoin('items_type', 'items.sale_unit', '=', 'items_type.id')
            ->join('store_items', function ($q) use ($request, $user_store) {
                $q->on('items.id', 'store_items.item_id');
                if ($request->store_id > 0) {
                    $q->where('store_items.store_id', $request->store_id);
                    $q->where('store_items.shop_id', '=', get_session_val('serial_id', 'integer'));
                } else { // when page load for thr first time when change store == 0
                    if (auth()->user()->change_store == 0) {
                        $q->where('store_items.store_id', $user_store);
                        $q->where('store_items.shop_id', '=', get_session_val('serial_id', 'integer'));
                    }
                }
            })->join('stores', function ($q) use ($request) {
                $q->on('store_items.store_id', 'stores.id');
                $q->where('stores.shop_id', '=', get_session_val('serial_id', 'integer'));
            })->leftJoin('items_unit', 'items_unit.item_id', '=', 'items.id')
            ->leftjoin('units', 'units.id', '=', 'items_unit.unit_id');


        if ($shop->color_size == 1) {
            $items = $items->leftJoin('sizes', 'items.size_id', '=', 'sizes.id');
            $append = 'sizes.size_name';
        }
        $factory_name = \DB::raw(' null as factory_name');
        if ($shop->item_factory == 1) {
            $factory_name = 'factories.name as factory_name';
        }
        $items = $items->select(
            'items.sale_unit',
            'items_type.name as type_name',
            'stores.store_name',
            'store_items.store_quant as quantity',
            'items.item_name',
            'items.about',
            'items.barcode',
            'items.sale_price',
            'items.pay_price',
            'items.id',
            'items.sale_code',
            'items.purchase_code',
            $append,
            $factory_name
        );
        $items = $this->filter($items, $request, $shop);
        return $items->groupBy('store_items.id')->get();
    }

    public function filter($items, $request, $shop)
    {
        if ($request->filled('supplier_id') && $request->supplier_id != '-1') {
            $items->join('clients', function ($q) use ($request) {
                $q->on('items.supplier_id', 'clients.id');
                $q->where('clients.id', $request->supplier_id);
            });
            // $items->join('multi_supplier', function ($q) use ($request) {
            //     $q->on('items.id', '=', 'multi_supplier.item_id');
            //     $q->where('multi_supplier.supplier_id', $request->supplier_id);
            // });
        } else {
            $items->leftJoin('clients', function ($q) use ($request) {
                $q->on('items.supplier_id', 'clients.id');
            });
            // $items->leftJoin('multi_supplier', function ($q) use ($request) {
            //     $q->on('items.id', '=', 'multi_supplier.item_id');
            // });
        }

        if ($request->filled('category_id') && $request->category_id != '-1') {
            $category = ItemType::where([
                'shop_id'     => get_session_val('serial_id', 'integer'),
                'category_id' => $request->category_id
            ])->pluck('id')->toArray();
            array_push($category, $request->category_id);
            $items = $items->whereIn('items.sale_unit', $category);
        }

        if ($request->filled('item_name')) {
            $items = $items->where(function ($w) use ($request) {
                $w->where('items.item_name', 'like', '%' . $request->item_name . '%')
                    ->orWhere('items.about', 'like', '%' . $request->item_name . '%');
            });
        }

        if ($shop->multi_price == 1) {
            if ($request->filled('prices') && $request->prices != '-1') {
                $items = $items->join('items_prices', function ($q) use ($request) {
                    $q->on('items.id', '=', 'items_prices.item_id');
                    $q->where('items_prices.list_id', $request->prices);
                });
            } else {
                $items = $items->leftJoin('items_prices', 'items.id', '=', 'items_prices.item_id');
            }
        }

        if ($shop->item_factory == 1) {
            if ($request->filled('factory') && $request->factory != '-1') {
                $items = $items->join('factories', function ($q) use ($request) {
                    $q->on('items.factory_id', '=', 'factories.id');
                    $q->where('items.factory_id', $request->factory);
                });
            } else {
                $items = $items->leftJoin('factories', 'items.factory_id', '=', 'factories.id');
            }
        }

        return $items;
    }

    /**
     * This Method For Get Remaining Items Report
     *
     * Request $request
     * @return view
     */
    public function remaining_items_report(Request $request)
    {
        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first(['decimal_num_price', 'decimal_num_quant']);
        $rows     = Invoices::join('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
            ->join('items', 'items.id', 'invoices_details.item_id')
            ->join('clients as supplier', 'supplier.id', 'invoices_.client_id')
            ->where([
                ['invoices_.invoice_type', 0],
                ['invoices_.shop_id', get_session_val('serial_id', 'integer')]
            ])
            ->selectRaw('distinct invoices_.id , invoices_.bill_no , items.item_name , supplier.client_name as supplier_name,
                         truncate(invoices_details.pay_price,' . $settings->decimal_num_price . ') as payprice,
                         truncate(invoices_details.quantity,' . $settings->decimal_num_quant . ') as quant_at_bill,
                         truncate(invoices_details.not_sold,' . $settings->decimal_num_quant . ') as remain_quantity,
                         invoices_details.expire_date , invoices_.transaction_date as incoming_date');

        if ($request->has('search') && !in_array($request->search, [null, '', ' '])) {
            $rows->where(function ($q) use ($request) {
                $q->where('items.item_name', 'like', '%' . $request->search . '%')
                    ->orWhere('supplier.client_name', 'like', '%' . $request->search . '%')
                    ->orWhere('invoices_.bill_no', 'like', '%' . $request->search . '%');
            });
        }

        if ($request->has('date_from') && !in_array($request->date_from, [null, '', ' '])) {
            $rows->where(function ($q) use ($request) {
                $q->where(function ($t) use ($request) {
                    $t->whereDate('invoices_.transaction_date', '>=', $request->date_from)
                        ->whereDate('invoices_.transaction_date', '!=', '0000-00-00');
                });

                $q->orWhere(function ($e) use ($request) {
                    $e->whereDate('invoices_details.expire_date', '>=', $request->date_from)
                        ->whereDate('invoices_details.expire_date', '!=', '0000-00-00');
                });
            });
        }

        if ($request->has('date_to') && !in_array($request->date_to, [null, '', ' '])) {
            $rows->where(function ($q) use ($request) {
                $q->where(function ($t) use ($request) {
                    $t->whereDate('invoices_.transaction_date', '<=', $request->date_to)
                        ->whereDate('invoices_.transaction_date', '!=', '0000-00-00');
                });

                $q->orWhere(function ($e) use ($request) {
                    $e->whereDate('invoices_details.expire_date', '<=', $request->date_to)
                        ->whereDate('invoices_details.expire_date', '!=', '0000-00-00');
                });
            });
        }

        $rows = $rows->orderBy('invoices_.transaction_date', 'DESC')->get();

        return view('salesreport::item_report.remaining_items', compact('rows'));
    }

    /**
     * This Method For Get Expired Items Report
     *
     * Request $request
     * @return view
     */
    public function expired_items_report(Request $request)
    {
        $settings = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first(['decimal_num_quant', 'expire_notification']);
        $rows     = Item::join('store_items', 'items.id', 'store_items.item_id')
            ->join('stores', 'stores.id', 'store_items.store_id')
            ->where('items.shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_items.store_quant', '>', 0)
            ->where('items.expire_date', '<', DB::raw('DATE_ADD(CURDATE(), INTERVAL ' . $settings->expire_notification . ' DAY)'))
            ->where('items.expire_date', '!=', '0000-00-00')
            ->whereNotNull('items.expire_date')
            ->selectRaw('items.item_name , round(store_items.store_quant,' . $settings->decimal_num_quant . ') as quantity , items.expire_date');

        if ($request->has('search') && !in_array($request->search, [null, '', ' '])) {
            $rows->where('items.item_name', 'like', '%' . $request->search . '%');
        }

        if ($request->has('date_from') && !in_array($request->date_from, [null, '', ' '])) {
            $rows->where(function ($q) use ($request) {
                $q->orWhere(function ($e) use ($request) {
                    $e->whereDate('items.expire_date', '>=', $request->date_from)
                        ->whereDate('items.expire_date', '!=', '0000-00-00');
                });
            });
        }

        if ($request->has('date_to') && !in_array($request->date_to, [null, '', ' '])) {
            $rows->where(function ($q) use ($request) {
                $q->orWhere(function ($e) use ($request) {
                    $e->whereDate('items.expire_date', '<=', $request->date_to)
                        ->whereDate('items.expire_date', '!=', '0000-00-00');
                });
            });
        }

        $rows = $rows->orderBy('items.expire_date', 'DESC')->get();

        return view('salesreport::item_report.expired_items', compact('rows'));
    }

    /**
     * This Method For Get Min Quantity Items Report
     *
     * Request $request
     * @return view
     */
    public function min_quantity_items_report(Request $request)
    {
        $settings   = BadrShop::where('serial_id', get_session_val('serial_id', 'integer'))->first(['decimal_num_quant', 'decimal_num_price']);
        $stores     = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $categories = ItemType::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $units      = Unit::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $rows       = Item::where('items.shop_id', get_session_val('serial_id', 'integer'))
            ->join('store_items', 'items.id', 'store_items.item_id')
            ->join('units', 'units.id', 'items.unit_id')
            ->join('items_type', 'items.sale_unit', 'items_type.id')
            ->join('stores', 'stores.id', 'store_items.store_id')
            ->where('items_type.shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_items.store_quant', '<=', DB::raw('items.min_quantity'))
            ->selectRaw('items.id as item_id ,items.item_name ,items_type.name as type_name ,units.name as unit_name,
                            units.id as unit_id ,items.sale_unit as cat_name ,items.min_quantity ,stores.store_name,
                            truncate(store_items.store_quant,' . $settings->decimal_num_quant . ') as quantity,
                            truncate(items.pay_price,' . $settings->decimal_num_price . ') as pay_price');

        if ($request->has('store') && !in_array($request->store, [null, -1, '-1', '', ' '])) {
            $rows->where('store_items.store_id', $request->store);
        }

        if ($request->has('category') && !in_array($request->category, [null, -1, '-1', '', ' '])) {
            $request->request->add(['cat_id' => $request->get('category')]);
            $catIds = (new ItemController())->getCategoriesId();
            $rows->whereIn('items_type.id', $catIds);
        }

        if ($request->has('unit') && !in_array($request->unit, [null, -1, '-1', '', ' '])) {
            $rows->where('units.id', $request->unit);
        }

        if ($request->has('search') && !in_array($request->search, [null, -1, '-1', '', ' '])) {
            $rows->where('items.shop_id', get_session_val('serial_id', 'integer'))
                ->where('items_type.shop_id', get_session_val('serial_id', 'integer'))
                ->where(function ($query) use ($request) {
                    $query->where('items.item_name', 'like', '%' . $request->search . '%')
                        ->orwhere('items_type.name', 'like', '%' . $request->search . '%');
                });
        }

        $rows = $rows->orderBy('items.id', 'DESC')->get();

        return view('salesreport::item_report.min_quantity_items', compact('stores', 'categories', 'units', 'rows'));
    }

    public function itemUnitQtyValReport(Request $request)
    {
        $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $units  = Unit::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $decimal_num_quant = get_session_val('decimal_num_quant', 'integer');
        $items = Item::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $store_id = $request->store;
        $items = Item::join('store_items', function ($q) use ($request) {
            $q->on('store_items.item_id', 'items.id')
                ->where('store_items.store_id', $request->store);
        })->where([
            ['items.shop_id', get_session_val('serial_id', 'integer')],
            ['items.available', 1],
            ['items.is_service', 0]
        ])->select(
            'items.item_name',
            'items.id',
        )->groupBy('items.id')->get();
        $results = Item::join('store_items', function ($q) use ($store_id, $decimal_num_quant, $request) {
            if ($request->item != 0) {
                $q->where('items.id', $request->item)
                    ->where('store_items.item_id', $request->item)
                    ->where('store_items.store_id', $store_id);
            } else {
                $q->on('store_items.item_id', 'items.id')
                    ->where('store_items.store_id', $store_id);
            }
        })->leftJoin('items_unit', 'items_unit.item_id', '=', 'items.id')
            ->leftjoin('units', 'units.id', '=', 'items_unit.unit_id')
            ->where([
                ['items.shop_id', get_session_val('serial_id', 'integer')],
                ['items.available', 1],
                ['items.is_service', 0]
            ])->select(
                'items.item_name',
                'items.unit_id',
                'items.id',
                'store_items.store_quant as quantity',
                'items_unit.unit_value',
                'units.name',
                DB::raw("group_concat(TRUNCATE((store_items.store_quant /items_unit.unit_value),$decimal_num_quant) ,units.name  SEPARATOR ' ) ,(   ') as unit_group"),

            )->groupBy('items.id')->get();

        return view('salesreport::item_report.item_unit_qty_report', compact('stores', 'results', 'items'));
    }

    public function getItems(Request $request)
    {
        $items = Item::join('store_items', function ($q) use ($request) {
            $q->on('store_items.item_id', 'items.id')
                ->where('store_items.store_id', $request->store);
        })->where([
            ['items.shop_id', get_session_val('serial_id', 'integer')],
            ['items.available', 1],
            ['items.is_service', 0]
        ])->select(
            'items.item_name',
            'items.id',
        )->groupBy('items.id')->get();
        if (count($items) > 0) {
            return response()->json($items);
        }
    }
}
