<?php

namespace Modules\SalesReport\Http\Controllers;

use App\Models\User;
use Carbon\Carbon;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\View;
use Modules\Invoices\Entities\ClientTransaction;
use Modules\Invoices\Entities\SalePoint;
use Modules\Invoices\Entities\Store;

class SalePointReportController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index()
    {
        return view('salesreport::index');
    }

    /**
     * Show the form for creating a new resource.
     * @return Renderable
     */
    public function create()
    {
        return view('salesreport::create');
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Renderable
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function show($id)
    {
        return view('salesreport::show');
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function edit($id)
    {
        return view('salesreport::edit');
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Renderable
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy($id)
    {
        //
    }

    public function transactionsReport(Request $request)
    {
        // dd(Carbon::createFromDate($request->start_date)->subDay());
        $user = User::Find(auth()->user()->id);
        $groupIds = $user->groups->pluck('id');
        $transactions = collect();

        if ($request->safe_point_id) {
            $transactions = ClientTransaction::leftJoin('invoices_', 'invoices_.id', 'client_transaction.bill_id')
                ->leftJoin('spending_item', 'spending_item.id', 'client_transaction.spend_id')
                ->leftJoin('imports_item', 'imports_item.id', 'client_transaction.import_id')
                ->leftJoin('sale_points AS s_P_1', DB::raw('s_P_1.id'), 'client_transaction.effected_point')
                ->leftJoin('sale_points AS s_P_2', DB::raw('s_P_2.id'), 'client_transaction.safe_point_id')
                ->leftJoin('users', 'users.id', 'client_transaction.user_id')
                // ->leftJoin('clients', 'clients.id', 'client_transaction.client_id')
                //يتاح للمستخدم العملاء الموجودين في المجموعه الخاصه بيه فقط
                ->leftJoin('clients', function ($join) use ($groupIds) {
                    if (auth()->user()->clients_group == 0) {
                        $join->on('clients.id', '=', 'client_transaction.client_id');
                    } else {
                        $join->on('clients.id', '=', 'client_transaction.client_id');
                        $join->whereIn('clients.group_id', $groupIds);
                    }
                })
                ->select(
                    DB::raw('
                            client_transaction.amount,
                            client_transaction.type,
                            client_transaction.pay_day,
                            client_transaction.type,
                            client_transaction.date_time,
                            client_transaction.safe_balance,
                            client_transaction.safe_point_id,
                            client_transaction.notes,
                            client_transaction.effect,
                            client_transaction.bill_id,
                            users.user_name,
                            clients.client_name,
                            clients.id AS client_id,
                            invoices_.bill_no as invoice_no,
                            invoices_.store_id as invoice_store,
                            invoices_.id AS invoice_id,
                            invoices_.network_notes AS network_notes,
                            invoices_.notes AS invoices_notes,
                            spending_item.name AS spending_item,
                            imports_item.name AS import_item,
                            s_P_1.point_name AS point_name1,
                            s_P_2.point_name AS point_name2
                        ')
                )
                ->where([
                    ['client_transaction.shop_id', get_session_val('serial_id', 'Integer')],
                    ['client_transaction.safe_point_id', $request->safe_point_id],
                    ['client_transaction.amount', '>', 0]
                ])
                ->when($request->store_id, function ($q) use ($request) {
                    if ($request->store_id > 0) {
                        $q->where('invoices_.store_id', $request->store_id);
                    }
                })
                ->when($request->start_date, function ($q) use ($request) {
                    $q->whereDate('client_transaction.date_time', '>=', $request->start_date);
                })
                ->when($request->end_date, function ($q) use ($request) {
                    $q->whereDate('client_transaction.date_time', '<=', $request->end_date);
                })
                ->orderBy('client_transaction.id')
                ->get();
        }

        $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get(['id', 'store_name']);
        $point = Auth::user()->sale_point;
        if (Auth::user()->show_points == 1 ||Auth::user()->show_points == 2) {
            $points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->get();
        }elseif (Auth::user()->show_points == 3) {
            $store_id = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->first()->store_id;
            $points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_id', $store_id)
          ->get();

        } else {
            $points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        }
        return view('salesreport::point_transactions_report', compact('transactions', 'points', 'stores'));
    }

}
