<?php

namespace Modules\SalesReport\Http\Controllers;

use App\Models\User;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Invoices\Entities\BadrShop;
use Modules\Invoices\Entities\Client;
use Modules\Invoices\Entities\Invoices;
use Modules\Invoices\Entities\ItemType;
use Modules\Invoices\Entities\SalePoint;
use Modules\Invoices\Entities\Store;
use Modules\User\Entities\ClientGroup;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Auth;
use Modules\Clients\Entities\ClientsGroups;
use Modules\SalesReport\Entities\Income;
use Modules\SalesReport\Entities\Spending;
use Modules\SalesReport\Entities\IncomeItem;
use Modules\SalesReport\Entities\SpendingItem;
use PhpOffice\PhpSpreadsheet\Calculation\TextData\Replace;

class SalesReportsController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index()
    {
        return view('salesreport::index');
    }

    /**
     * Show the form for creating a new resource.
     * @return Response
     */
    public function create()
    {
        return view('salesreport::create');
    }

    /**
     * store_id a newly created resource in storage.
     * @param  Request $request
     * @return Response
     */
    public function store_id(Request $request)
    {
    }

    /**
     * Show the specified resource.
     * @return Response
     */
    public function show()
    {
        return view('salesreport::show');
    }

    /**
     * Show the form for editing the specified resource.
     * @return Response
     */
    public function edit()
    {
        return view('salesreport::edit');
    }

    /**
     * Update the specified resource in storage.
     * @param  Request $request
     * @return Response
     */
    public function update(Request $request)
    {
    }

    /**
     * Remove the specified resource from storage.
     * @return Response
     */
    public function destroy()
    {
    }

    public function gross_profit_report(Request $request)
    {
        // ***********************************************************************************
        //يتاح للمستخدم الموردين او العملاء الموجودين في المجموعه الخاصه بيه فقط
        $clients = clientGroups()->whereIn('client_role', [0, 2])->orderBy('client_name')->get(['id', 'group_id', 'client_name']);
        // ***************************************************************************************
        // عرض المجمعات المتاحه للمستخدم فقط
        $clients_groups = groupList();
        $this->prepareNetSalesViews($request);
        $user = User::Find(auth()->user()->id);
        $net_sales = DB::table('view_net_sales')->get();
        $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get(['id', 'store_name']);
        $safe_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))->get(['id', 'point_name']);
        $this->dropViews();
        // view data
        return view('salesreport::gross_profit_report', compact('net_sales', 'stores', 'safe_points', 'clients', 'clients_groups', 'user'));
    }


    public function net_profit_report(Request $request)
    {
        if (auth()->user()->clients_group == 0) {
            $request->merge([
                'client' => -1,
                'store' => -1,
                'client_group' => -1
            ]);
        } else {
            $request->merge([
                'client' => -2,
                'store' => -1,
                'client_group' => -2
            ]);
        }
        $this->prepareNetSalesViews($request);
        $start_date = $request->start_date ? $request->start_date : Carbon::today()->format('Y-m-d');
        $end_date = $request->end_date ? $request->end_date : Carbon::today()->format('Y-m-d');
        $view_spend_prepare = DB::Statement("
            CREATE  OR REPLACE  VIEW view_spend_prepare as
                select
                    DATE_FORMAT(spend_date, '%Y/%m/%d' ) as spend_date , SUM(amount) as amount
                from spending
                where
                    shop_id=" . get_session_val('serial_id', 'Integer') . "  and DATE(spend_date) between '" . $start_date . "' and '" . $end_date . "' " .
            " GROUP BY DATE(spend_date)
                UNION  ALL
            select
                DATE_FORMAT(emp_transaction.spend_date, '%Y/%m/%d' ) as spend_date ,SUM(emp_transaction.amount) as amount
            from
                emp_transaction ,client_transaction
            where
                emp_transaction.shop_id=" . get_session_val('serial_id', 'Integer') .
            "  and emp_transaction.type in(42,43) and DATE(emp_transaction.spend_date) between '" . $start_date . "' and '" . $end_date . "'
                and emp_transaction.client_transaction=client_transaction.id and client_transaction.shop_id=" . get_session_val('serial_id', 'Integer') . "
            GROUP BY DATE(spend_date)
        ");

        $view_spend = DB::statement("
            CREATE  OR REPLACE  VIEW view_spend as
            select
                SUM(amount) as amount ,spend_date from view_spend_prepare as spending
            GROUP BY  DATE(spend_date)
            order by spend_date
        ");

        $view_imports = DB::statement("
            CREATE  OR REPLACE  VIEW view_imports as
            select
                DATE_FORMAT(import_date, '%Y/%m/%d' ) as import_date ,SUM(amount) as total_import
            from imports
            where
                shop_id=" . get_session_val('serial_id', 'integer') . "  and DATE(import_date) between '" .  $start_date . "' and '" . $end_date . "' " .
            " GROUP BY DATE(import_date)
        ");

        $view_destroyed_items = DB::statement("
            CREATE  OR REPLACE  VIEW view_destroyed_items as
            select
                DATE_FORMAT(transaction_date, '%Y/%m/%d' ) as action_date ,

                SUM(
                    pay_price * quantity
                    *
                    CASE
                        WHEN invoices_.invoice_type = 18 THEN 1
                    ELSE -1
                    END
                )
             as total_detroyed
            from  invoices_
            LEFT JOIN invoices_details ON invoices_details.invoice_id = invoices_.id
            where
                invoices_.shop_id=" . get_session_val('serial_id', 'integer') . "  and invoice_type in (18, 23) and DATE(transaction_date) between '" .  $start_date . "' and '" . $request->end_date .
            "' GROUP BY DATE(transaction_date)
        ");

        $view_total_earn = DB::statement("
            CREATE  OR REPLACE  VIEW view_total_earn as
            select
                ifnull(sale_date,date_back) as sale_date ,((ifnull(netprice,0) - ifnull(netpayprice,0)) - (ifnull(net_back,0) - ifnull(back_pay_price,0))) as total_earn
            from view_net_sales order by sale_date
        ");

        $view_sales_spend = DB::statement("
            CREATE  OR REPLACE  VIEW view_sales_spend as
            select
                spend_date , amount  ,view_total_earn.sale_date ,view_total_earn.total_earn,import_date,total_import ,action_date,total_detroyed
            from  view_spend
            left join  view_total_earn on  spend_date=view_total_earn.sale_date
            left join  view_imports on  spend_date=view_imports.import_date
            left join  view_destroyed_items on action_date=spend_date
            UNION
            select
                spend_date , amount ,view_total_earn.sale_date ,view_total_earn.total_earn , import_date,total_import ,action_date,total_detroyed
            from view_total_earn
            left join  view_spend on  spend_date=view_total_earn.sale_date
            left join  view_imports on  import_date=view_total_earn.sale_date
            left join  view_destroyed_items on action_date=view_total_earn.sale_date
            UNION
            select
                spend_date , amount ,view_total_earn.sale_date ,view_total_earn.total_earn , import_date,total_import ,action_date,total_detroyed
            from  view_imports
            left join  view_spend on  view_imports.import_date=spend_date
            left join  view_total_earn on  import_date=view_total_earn.sale_date
            left join  view_destroyed_items on action_date=import_date
            UNION
            select  spend_date , amount ,view_total_earn.sale_date ,view_total_earn.total_earn , import_date,total_import ,action_date,total_detroyed
            from  view_destroyed_items
            left join  view_spend on  action_date=spend_date
            left join  view_total_earn on  action_date=view_total_earn.sale_date
            left join  view_imports on action_date=view_imports.import_date
        ");

        // viewed details
        $net_sales = DB::select(Db::raw("
            select
                ifnull(sale_date,ifnull(spend_date,ifnull(import_date,action_date))) as sale_date ,
                total_earn,amount as spend,total_import ,total_detroyed
            from view_sales_spend
            order by sale_date;
        "));

        $this->dropViews();

        // view data
        return view('salesreport::net_profit_report', compact('net_sales'));
    }
    private function  prepareNetSalesViews(Request $request)
    {
        // prepare filters
        $store_filter = $client_filter = $client_group_join = $client_group_filter = $store_filter_with_alias = $client_filter_with_alias = $client_group_filter_with_alias = '';
        $start_date = $request->start_date ? $request->start_date : Carbon::today()->format('Y-m-d');
        $end_date = $request->end_date ? $request->end_date : Carbon::today()->format('Y-m-d');
        if ($request->store != -1) {
            $store_filter = ' AND invoices_.store_id = ' . intval($request->store);
            $store_filter_with_alias = ' AND i.store_id = ' . intval($request->store);
        }
        if ($request->client != -1 && $request->client != -2) {
            $client_filter = ' AND invoices_.client_id = ' . intval($request->client);
            $client_filter_with_alias = ' AND i.client_id = ' . intval($request->client);
        }
        if ($request->client == -2) {
            $clients = clientGroups()->pluck('id');
            $client_filter = ' AND invoices_.client_id in ( ' . $clients . ')  ';
            $client_filter = str_replace([' [', ']'], '', $client_filter);
            $client_filter_with_alias = ' AND i.client_id in ( ' . $clients . ') ';
            $client_filter_with_alias = str_replace([' [', ']'], '', $client_filter_with_alias);
        }
        if ($request->client_group == -2) {
            $user = User::Find(auth()->user()->id);
            $groupIds = $user->groups->pluck('id');
            $client_group_join = ' clients, ';
            $client_group_filter = ' AND clients.group_id in ( ' . $groupIds . ') AND clients.id = invoices_.client_id';
            $client_group_filter = str_replace([' [', ']'], '', $client_group_filter);
            $client_group_filter_with_alias = ' AND clients.group_id in ( ' . $groupIds . ') AND clients.id = i.client_id';
            $client_group_filter_with_alias = str_replace([' [', ']'], '', $client_group_filter_with_alias);
        }
        if ($request->client_group != -1 && $request->client_group != -2) {
            $client_group_join = ' clients, ';
            $client_group_filter = ' AND clients.group_id = ' . intval($request->client_group) . ' AND clients.id = invoices_.client_id';
            $client_group_filter_with_alias = ' AND clients.group_id = ' . intval($request->client_group) . ' AND clients.id = i.client_id';
        }
        // prepare pay price
        $pay_price = 'pay_price';
        if (get_session_val('Profits_Calc_method', 'integer') == 2) {
            $pay_price = 'av_pay_price';
        } else if (get_session_val('Profits_Calc_method', 'integer') == 3) {
            $pay_price = 'ac_pay_price';
        }
        // create & prepare views to get values
        $view_sale = DB::Statement("
                            CREATE OR REPLACE VIEW view_sale AS
                            SELECT
                            DATE_FORMAT(invoice_date, '%Y/%m/%d') AS sale_date ,
                            SUM(
                                if (
                                    items.vat_state = 1,
                                        TRUNCATE (i_d.item_price_with_discount, " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                                            +
                                            (  select ifnull( sum(invoices_details_vats.vat_weight), 0)
                                                        from invoices_details_vats
                                                        left join invoices_details
                                                        on invoices_details_vats.invoice_id = invoices_details.invoice_id
                                                        AND invoices_details_vats.shop_id =  " . get_session_val('serial_id', 'Integer') . " AND invoices_details_vats.item_id = invoices_details.item_id
                                                        left join invoices_ on invoices_.id = invoices_details.invoice_id
                                                        where invoices_details.item_id = items.id and invoices_details.invoice_id = invoices_.id AND invoices_.invoice_type = 1
                                                        AND invoices_details_vats.invoice_id = i.id AND invoices_details.id = i_d.id AND i_d.unit_id = invoices_details_vats.unit_id

                                            )
                                            ,
                                TRUNCATE
                                    (i_d.item_price_with_discount, " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                                )
                            ) AS netprice,
                            SUM(
                                if (
                                    items.vat_state = 2,
                                TRUNCATE
                                    (i_d.item_price_with_discount, " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                                    -
                                    (   select ifnull( sum(invoices_details_vats.vat_weight), 0)
                                        from invoices_details_vats
                                        left join invoices_details
                                        on invoices_details_vats.invoice_id = invoices_details.invoice_id
                                        AND invoices_details_vats.shop_id =  " . get_session_val('serial_id', 'Integer') . " AND invoices_details_vats.item_id = invoices_details.item_id
                                        left join invoices_ on invoices_.id = invoices_details.invoice_id
                                        where invoices_details.item_id = items.id  and invoices_details.invoice_id = invoices_.id AND invoices_.invoice_type = 1
                                        AND invoices_details_vats.invoice_id = i.id  AND invoices_details.id = i_d.id  AND i_d.unit_id = invoices_details_vats.unit_id
                                    )
                                    ,
                                TRUNCATE
                                    (i_d.item_price_with_discount, " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                                )
                            ) AS netprice_withoutvat,
                            SUM(
                                if (
                                    items.purchase_vat_state = 1,
                                        TRUNCATE (i_d." . $pay_price . ", " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                                            +
                                            i_d.pay_vats_weight * i_d.quantity
                                            ,

                                TRUNCATE
                                    (i_d." . $pay_price . ", " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                                )
                            ) AS netpayprice,
                            SUM(
                                if (
                                    items.purchase_vat_state = 2,
                                TRUNCATE
                                    (i_d." . $pay_price . ", " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                                    -
                                    (
                                        i_d.pay_vats_weight * i_d.quantity
                                    )
                                    ,
                                TRUNCATE
                                    (i_d." . $pay_price . ", " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                                )
                            ) AS netpayprice_withoutvat,
                            SUM(i_d.quantity) AS quantity_sales
                        FROM
                            invoices_ i,
                            " . $client_group_join . "
                            items
                        INNER JOIN invoices_details i_d
                        WHERE
                            items.id = i_d.item_id AND i.shop_id =  " . get_session_val('serial_id', 'Integer') . " AND i.id = i_d.invoice_id AND
                            DATE(invoice_date) BETWEEN '" . $start_date . "' AND '" . $end_date . "' " . $store_filter_with_alias . $client_filter_with_alias . $client_group_filter_with_alias . " AND i.invoice_type = 1
                        GROUP BY
                            DATE(sale_date)
                        ORDER BY
                            sale_date;
        ");

        $view_back = DB::Statement("
                    CREATE OR REPLACE VIEW view_back  AS
                    SELECT
                    DATE_FORMAT(invoice_date, '%Y/%m/%d') AS date_back ,
                    SUM(
                        if (
                            items.vat_state = 1,
                                TRUNCATE (i_d.item_price_with_discount, " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                                    +
                                    (  select ifnull( sum(invoices_details_vats.vat_weight), 0)
                                                from invoices_details_vats
                                                left join invoices_details
                                                on invoices_details_vats.invoice_id = invoices_details.invoice_id
                                                AND invoices_details_vats.shop_id =  " . get_session_val('serial_id', 'Integer') . " AND invoices_details_vats.item_id = invoices_details.item_id
                                                left join invoices_ on invoices_.id = invoices_details.invoice_id
                                                where invoices_details.item_id = items.id and invoices_details.invoice_id = invoices_.id AND invoices_.invoice_type = 3
                                                AND invoices_details_vats.invoice_id = i.id AND invoices_details.id = i_d.id AND i_d.unit_id = invoices_details_vats.unit_id
                                    )
                                    ,
                        TRUNCATE
                            (i_d.item_price_with_discount, " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                        )
                    ) AS net_back,
                    SUM(
                        if (
                            items.vat_state = 2,
                        TRUNCATE
                            (i_d.item_price_with_discount, " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                            -
                            (   select ifnull( sum(invoices_details_vats.vat_weight), 0)
                                from invoices_details_vats
                                left join invoices_details
                                on invoices_details_vats.invoice_id = invoices_details.invoice_id
                                AND invoices_details_vats.shop_id =  " . get_session_val('serial_id', 'Integer') . " AND invoices_details_vats.item_id = invoices_details.item_id
                                left join invoices_ on invoices_.id = invoices_details.invoice_id
                                where invoices_details.item_id = items.id  and invoices_details.invoice_id = invoices_.id AND invoices_.invoice_type = 1
                                AND invoices_details_vats.invoice_id = i.id  AND invoices_details.id = i_d.id AND i_d.unit_id = invoices_details_vats.unit_id
                            )
                            ,
                        TRUNCATE
                            (i_d.item_price_with_discount, " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                        )
                    ) AS net_back_without_vat,
                    SUM(
                        if (
                            items.purchase_vat_state = 1,
                                TRUNCATE (i_d." . $pay_price . ", " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                                    +
                                    i_d.pay_vats_weight * i_d.quantity
                                    ,

                        TRUNCATE
                            (i_d." . $pay_price . ", " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                        )
                    ) AS back_pay_price,
                    SUM(
                        if (
                            items.purchase_vat_state = 2,
                        TRUNCATE
                            (i_d." . $pay_price . ", " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                            -
                            (
                                i_d.pay_vats_weight * i_d.quantity
                            )
                            ,
                        TRUNCATE
                            (i_d." . $pay_price . ", " . get_session_val('decimal_num_price', 'Integer') . ") * i_d.quantity
                        )
                    ) AS back_pay_price_without_vat,
                    SUM(i_d.quantity) AS quantity_back
                FROM
                    invoices_ i,
                    " . $client_group_join . "
                    items
                INNER JOIN invoices_details i_d
                WHERE
                    items.id = i_d.item_id AND i.shop_id =  " . get_session_val('serial_id', 'Integer') . " AND i.id = i_d.invoice_id AND
                    DATE(invoice_date) BETWEEN '" . $start_date . "' AND '" . $end_date . "' " . $store_filter_with_alias . $client_filter_with_alias . $client_group_filter_with_alias . " AND i.invoice_type = 3
                GROUP BY
                    DATE(date_back)
                ORDER BY
                    date_back;

        ");

        $view_net_sales = DB::Statement("
                CREATE OR REPLACE VIEW view_net_sales AS
                SELECT
                    sale_date,
                    netprice,
                    netprice_withoutvat,
                    netpayprice,
                    netpayprice_withoutvat,
                    view_sale.quantity_sales,
                    date_back,
                    net_back,
                    net_back_without_vat,
                    back_pay_price,
                    back_pay_price_without_vat,
                    quantity_back
                FROM
                    view_sale
                LEFT JOIN view_back ON sale_date = date_back
                UNION
                SELECT
                    sale_date,
                    netprice,
                    netprice_withoutvat,
                    netpayprice,
                    netpayprice_withoutvat,
                    view_sale.quantity_sales,
                    date_back,
                    net_back,
                    net_back_without_vat,
                    back_pay_price,
                    back_pay_price_without_vat,
                    quantity_back
                FROM
                    view_back
                LEFT JOIN view_sale ON date_back = sale_date
        ");
    }

    private function dropViews()
    {
        DB::statement("
            DROP VIEW IF EXISTS
                view_back ,view_net_sales,view_back_details, view_sale, view_sales_spend,
                view_sales_total_spend,view_sale_pay, view_spend, view_destroyed_items,
                view_total_earn ,view_imports ,view_spend_prepare
        ");
    }

    public function items_sales_dusring_period_report(Request $request)
    {
        $user = User::Find(auth()->user()->id);
        $groupIds = $user->groups->pluck('id');
        if ($request->start_date) {
            $sales = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                ->leftJoin('items', 'items.id', 'invoices_details.item_id')
                ->leftJoin('invoices_details_vats', function($join) {
                    $join->on('invoices_details_vats.item_id', '=', 'invoices_details.item_id')
                            ->on('invoices_details_vats.invoice_id', '=', 'invoices_details.invoice_id');
                })
                ->leftJoin('items_unit', function ($q) {
                    $q->on('items_unit.unit_id', 'invoices_details.unit_id');
                    $q->on('items_unit.item_id', 'invoices_details.item_id');
                    $q->where('items_unit.unit_type', 0);
                })->Join('clients', function ($join) use ($groupIds) {
                    if (auth()->user()->clients_group == 0) {
                        $join->on('clients.id', '=', 'invoices_.client_id');
                    } else {
                        $join->on('clients.id', '=', 'invoices_.client_id');
                        $join->whereIn('clients.group_id', $groupIds);
                    }
                })
                ->when($request->category_id, function ($q) use ($request) {
                    if ($request->category_id > 0) {
                        $q->where('items.sale_unit', $request->category_id);
                    }
                })
                ->when($request->client_id, function ($q) use ($request) {
                    if ($request->client_id > 0) {
                        $q->where('invoices_.client_id', $request->client_id);
                    }
                })
                ->when($request->user_id, function ($q) use ($request) {
                    if ($request->user_id > 0) {
                        $q->where('invoices_.add_user', $request->user_id);
                    }
                })
                ->when($request->store_id, function ($q) use ($request) {
                    if ($request->store_id > 0) {
                        $q->where('invoices_.store_id', $request->store_id);
                    }
                })
                ->when($request->start_date, function ($q) use ($request) {
                    $q->whereDate('invoices_.transaction_date', '>=', $request->start_date);
                })
                ->when($request->end_date, function ($q) use ($request) {
                    $q->whereDate('invoices_.transaction_date', '<=', $request->end_date);
                })
                ->where([
                    ['invoices_.invoice_type', 1],
                    ['invoices_.shop_id', get_session_val('serial_id', 'Integer')]
                ])
                ->select(DB::raw('
                SUM(invoices_details.quantity * if (items_unit.id > 0, items_unit.unit_value, 1 )) AS quantity,
                SUM(invoices_details.price_without_vat * invoices_details.quantity) AS price,
                items.item_name,
                items.about,
                items.id,
                items.vat_state,
                SUM(invoices_details_vats.vat_weight) AS vat_weight
            '))
                ->groupBy('items.id')
                ->get();
        } else {
            $sales = collect();
        }

        $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get(['id', 'store_name']);
        $categories = ItemType::where('shop_id', get_session_val('serial_id', 'integer'))->get(['id', 'name']);
        $users = User::where('shop_id', get_session_val('serial_id', 'integer'))->get(['id', 'user_name']);
        $clients = clientGroups()->whereIn('client_role', [0, 2])->orderBy('client_name')->get(['id', 'group_id', 'client_name']);
        $selectedUser = $users->find($request->user_id);
        $selectedClient = $clients->find($request->client_id);
        $selectedStore = $stores->find($request->store_id);
        $selectedCategory = $categories->find($request->category_id);
        return view('salesreport::items_sales_dusring_period_report', compact('sales', 'stores', 'categories', 'users', 'clients', 'selectedUser', 'selectedClient', 'selectedStore', 'selectedCategory'));
    }

    public function spend_report(Request $request)
    {

        $from = null;
        $to = null;
        $section = null;
        $safe = null;
        $user = null;
        $Spend = null;
        $spends_item = SpendingItem::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $sale_points = SalePoint::where('sale_points.shop_id', get_session_val('serial_id', 'integer'));
        $point = Auth::user()->sale_point;
        if (Auth::user()->show_points == 1 ||Auth::user()->show_points == 2)  {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id' ,'integer'))
                ->where('id', $point)
                ->get();
        } elseif (Auth::user()->show_points == 3) {
            $store_id = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->first()->store_id;
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_id', $store_id)
            ->get();

        }
        else {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id' ,'integer'))->get();
        }

        $users = \DB::table('users')->where('shop_id', get_session_val('serial_id', 'integer'))->get();
        //  dd($spends_item);
        // if ($request->has('section')){
        $section = $request->section;
        $safe = $request->safe;
        $user = $request->user;
        $from = Carbon::parse($request->from)->toDateTimeString();
        $to = Carbon::parse($request->to)->endOfDay()->toDateTimeString(); // to get the last day in the specific period
        $Spend = Spending::where('spending.shop_id', get_session_val('serial_id', 'integer'))
        ->whereDate('spending.spend_date', '>=', $from)
        ->whereDate('spending.spend_date', '<=', $to)
            ->join('spending_item', 'spending_item.id', 'spending.spend_id')
            ->join('users', 'users.id', 'spending.add_user')
            ->join('sale_points', 'sale_points.id', 'spending.sale_point');
        if ($request->section != 0) {
            $Spend = $Spend->where('spend_id', $request->section);
        }
        if ($request->safe != 0) {
            $Spend = $Spend->where('spending.sale_point', $request->safe);
        }
        if ($request->user != 0) {
            $Spend = $Spend->where('spending.add_user', $request->user);
        }
        $Spend = $Spend->select('spending.spend_date', 'spending.amount', 'spending.note', 'users.name as username', 'sale_points.point_name', 'spending_item.name as spend_name')
            ->get();
        //  dd($Spend);
        // $section =$Spend[0]->spend_name;
        return view('salesreport::spend_report', ['spends_item' => $spends_item, 'Spends' => $Spend, 'from' => $request->from, 'to' => $request->to, 'section' => $section, 'section2' => $section, 'Safe' => $safe, 'Safe2' => $request->safe, 'User2' => $request->user, 'User' => $user,  'sale_points' => $sale_points, 'users' => $users]);
        // }

    }


    public function incom_report(Request $request)
    {
        //  dd($request);
        $from = null;
        $to = null;
        $section = null;
        $safe = null;
        $user = null;
        $imports_item = IncomeItem::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $point = Auth::user()->sale_point;
        if (Auth::user()->show_points == 1 ||Auth::user()->show_points == 2)  {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id' ,'integer'))
                ->where('id', $point)
                ->get();
        } elseif (Auth::user()->show_points == 3) {
            $store_id = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->first()->store_id;
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_id', $store_id)
            ->get();

        }
        else {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id' ,'integer'))->get();
        }
        $users = \DB::table('users')->where('shop_id', get_session_val('serial_id', 'integer'))->get();
        $Income = null;
        // dd($imports_item);
        if ($request->has('section')) {
            $section = $request->section;
            $from = Carbon::parse($request->from)->toDateTimeString();
            $to = Carbon::parse($request->to)->endOfDay()->toDateTimeString(); // to get the last day in the specific period
            $safe = $request->safe;
            $user = $request->user;

            $Income = Income::where('imports.shop_id', get_session_val('serial_id', 'integer'))
            ->whereDate('imports.import_date', '>=', $from)
            ->whereDate('imports.import_date', '<=', $to)
                ->join('imports_item', 'imports_item.id', 'imports.import_id')
                ->join('users', 'users.id', 'imports.add_user')
                ->join('sale_points', 'sale_points.id', 'imports.sale_point');
            if ($request->section != 0) {
                $Income = $Income->where('import_id', $request->section);
            }
            if ($request->safe != 0) {
                $Income = $Income->where('imports.sale_point', $request->safe);
            }
            if ($request->user != 0) {
                $Income = $Income->where('imports.add_user', $request->user);
            }
            $Income = $Income->select('imports.import_date', 'imports.amount', 'imports.note', 'users.name as username', 'sale_points.point_name', 'imports_item.name as import_name')
                ->get();
            //  $section =$Income[0]->import_name;
            //    dd($Income[0]->import_name);
            return view('salesreport::incom_report', ['imports_item' => $imports_item, 'incomes' => $Income, 'from' => $request->from, 'to' => $request->to, 'section' => $section, 'section2' => $request->section, 'Safe' => $safe, 'Safe2' => $request->safe, 'User2' => $request->user, 'User' => $user,  'sale_points' => $sale_points, 'users' => $users]);
        }
        return view('salesreport::incom_report', ['imports_item' => $imports_item, 'incomes' => $Income, 'from' => $request->from, 'to' => $request->to, 'section' => $section, 'section2' => $section, 'Safe' => $safe, 'Safe2' => $safe, 'User2' => $user, 'User' => $user,  'sale_points' => $sale_points, 'users' => $users]);
    }


    public function salesImportsTotal(Request $request)
    {
        $dates['start'] = $request->start_date ? $request->start_date : Carbon::today()->format('Y-m-d');
        $dates['end']   = $request->end_date ? $request->end_date : Carbon::today()->format('Y-m-d');

        $invoices = Invoices::where('shop_id', auth()->user()->shop_id)->BetweenTwoDates($dates)->where('invoice_type', 1)
                            ->selectRaw('DATE(transaction_date) as date, SUM(net_price) as total_invoices, SUM(payment) as total_payment')
                            ->groupBy(DB::raw("DATE(transaction_date)"))
                            ->paginate(15);
                            
        return view('salesreport::sales-imoprts-total', compact('invoices'));
    }

    public function installmentsAdd(Request $request)
    {
        $dates['start'] = $request->start_date ? $request->start_date : Carbon::today()->format('Y-m-d');
        $dates['end']   = $request->end_date ? $request->end_date : Carbon::today()->format('Y-m-d');

        $invoices = Invoices::where('shop_id', get_session_val('serial_id', 'integer'))
                    ->BetweenTwoDates($dates)
                    ->where('invoice_type', 1)
                    ->where('insatllments_add', '>', 0)
                    ->selectRaw('invoice_date as date, bill_no, bill_total, insatllments_add')
                    ->paginate(15);
        return view('salesreport::installments-add', compact('invoices'));
    }
}
