<?php

namespace Modules\SalesReport\Http\Controllers;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Modules\ClientInstallments\Entities\ClientTransaction;
use Modules\SalesReport\Entities\Invoice;
use Modules\SalesReport\Entities\SalePoint;
use Modules\SalesReport\Http\Constants\TransactionType;
use Modules\SalesReport\Http\Services\TransactionProcess;

class TrialBalanceController extends Controller
{
    public array $clauses = [];
    public array $totals  = [];

    public function __construct()
    {
        $this->totals = [
            'total_balance_balance_trial_balances_creditor' => 0,
            'total_balance_balance_trial_balances_debit'    => 0,
            'revision_balance_in_totals_creditor'           => 0,
            'revision_balance_in_totals_debit'              => 0,
        ];
    }

    public function index()
    {
        $this->setAllClauses();
        return view('salesreport::trial-balance.index', ['clauses' => $this->clauses, 'totals' => $this->totals]);
    }

    public function filter(Request $request)
    {
        $this->setAllClauses();
        $this->getQuery($request);
        $this->setBalanceTotals();
        return view('salesreport::trial-balance.result', ['clauses' => $this->clauses, 'totals' => $this->totals]);
    }

    protected function getQuery(Request $request)
    {
        $rows = ClientTransaction::select('pay_day', 'balance', 'amount', 'effect', 'type', 'bill_net_total', 'safe_point_id', 'bill_id')->where('shop_id', get_session_val('serial_id', 'Integer'))
                                    ->whereNull('is_deleted')
                                    ->whereNotIn('type', [4, 8])
                                    ->with('salePoint:id,point_name as name', 'invoice:id,rest')
                                    ->when(! empty($request->get('start_date')), function ($query) use($request) {
                                        $query->whereDate('pay_day', '>=', $request->start_date);
                                    })
                                    ->when(! empty($request->get('end_date')), function ($query) use($request) {
                                        $query->whereDate('pay_day', '<=', $request->end_date);
                                    })
                                    ->get()->groupBy('type');
        $check_invoices = [];
        foreach ($rows as $type => $transactions) {
            new TransactionProcess($type, $transactions, $this->clauses, $check_invoices);
        }
    }

    protected function setBalanceTotals()
    {
        foreach ($this->clauses as $name => $details) {
            $num = $details ['balance_balance_trial_balances'] ['debit'] - $details ['balance_balance_trial_balances'] ['creditor'];
            $column = $num > 0 ? 'debit' : 'creditor';
            $this->clauses[$name]['revision_balance_in_totals'][$column] = abs($num);
            $this->calcTotal($this->clauses[$name]);
        }
    }

    protected function calcTotal($details)
    {
        $this->totals ['total_balance_balance_trial_balances_creditor'] += $details['balance_balance_trial_balances']['creditor'];
        $this->totals ['total_balance_balance_trial_balances_debit']    += $details['balance_balance_trial_balances']['debit'];
        $this->totals ['revision_balance_in_totals_creditor']           += $details['revision_balance_in_totals']['creditor'];
        $this->totals ['revision_balance_in_totals_debit']              += $details['revision_balance_in_totals']['debit'];
    }

    protected function setAllClauses()
    {
        foreach ($this->getStaticClauses() as $name) {
            $this->clauses [$name] = [
                'balance_balance_trial_balances' => [ // ميزان المراجعه بالارصده
                    'debit' => 0,
                    'creditor' => 0,
                ],
                'revision_balance_in_totals' => [ // ميزان المراجعه بالمجاميع
                    'debit' => 0,
                    'creditor' => 0,
                ],
            ];
        }
    }

    protected function getStaticClauses()
    {
        $banks = SalePoint::select('point_name as name', 'point_type')->where('shop_id', get_session_val('serial_id', 'Integer'))->orderBy('point_type', 'ASC')->pluck('name')->toArray();
        return array_merge($banks, [
            TransactionType::CASH_SALES,
            TransactionType::DEFFERRED_SALES,
            TransactionType::RETURNS_SALES,
            TransactionType::CASH_PURCHASES,
            TransactionType::DEFFERRED_PURCHASES,
            TransactionType::RETURNS_PURCHASES,
            TransactionType::GENERAL_REVENUE,
            TransactionType::GENERAL_EXPENSES,
            TransactionType::SUPPLIERS,
            TransactionType::CLIENTS,
            // TransactionType::INSTALLMENTS,
        ]);
    }
}
