<?php

namespace Modules\SalesReport\Http\Controllers;

use App\Models\User;
use App\Services\Einvoice\Invoice\Invoice;
use Carbon\Carbon;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\BadrRequest\Entities\Invoices;
use Modules\Commissions\Entities\CommissionPercent;
use Modules\Item\Entities\Item;

class UserSalesController extends Controller
{

    public function totalSales(Request $request)
    {
        $dates['start'] = $request->start_date ? $request->start_date : Carbon::today()->format('Y-m-d');
        $dates['end'] = $request->end_date ? $request->end_date : Carbon::today()->format('Y-m-d');

        $sales = $this->getDataGroupedBy('users.id', $request->user_id, $dates);
        // $allItems = $this->getDataGroupedBy('invoices_details.item_id', $request->user_id, $dates);

        $sales = $sales->each(function ($sale) use($dates){
            $allItems = $this->getDataGroupedBy('invoices_details.item_id', $sale->add_user, $dates);

            // calc totals
            $sale->achieved_percent = 0;
            if($sale->goal){
                $sale->achieved_percent = ($sale->total_sale / $sale->goal);
            }
            $sale->total_profit = ($sale->total_sale - $sale->total_purchase);

            if($sale->total_purchase == 0){
                $sale->profit_percent = 100;
            }else{
                $sale->profit_percent = $sale->total_profit / $sale->total_purchase;
            }
            
            // total commission
            $sale->comission = 0;
            $commissions = CommissionPercent::where('shop_id',get_session_val('serial_id', 'Integer'))->get();

            // foreach item calc commission and increment total 
            foreach($allItems as $item){
                    if($sale->user_name == $item->user_name){
                        $item->profit = ($item->total_sale - $item->total_purchase);
                        if($item->total_purchase == 0){
                            $item->profit_percent = 100;
                        }else{
                            $item->profit_percent = $item->profit / $item->total_purchase;
                        }
                        $comission_percent = $this->getComission($item->profit_percent, $commissions);
                        $sale->comission += $comission_percent * $item->total_sale;
                }
            }

            $sale->net_profit = $sale->total_profit - $sale->comission;
            $sale->comission_percent = round_money($sale->comission / $sale->total_sale);
        });

        $users = User::where('shop_id', get_session_val('serial_id', 'integer'))->get(['id', 'user_name']);
        $items = Item::where('shop_id', get_session_val('serial_id', 'integer'))->get();

        return view('salesreport::users.users_total_sales_report', compact('users','sales','items'));

    }

    public function userSales(Request $request)
    {
        $start_date = $request->start_date ? $request->start_date : Carbon::today()->format('Y-m-d');
        $end_date = $request->end_date ? $request->end_date : Carbon::today()->format('Y-m-d');
        $sales = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                            ->leftJoin('items', 'items.id', 'invoices_details.item_id')
                            ->leftJoin('items_unit', function($q){
                                $q->on('items_unit.unit_id', 'invoices_details.unit_id');
                                $q->on('items_unit.item_id', 'invoices_details.item_id');
                                $q->where('items_unit.unit_type', 0);
                            })
                            ->leftJoin('users','users.id','invoices_.add_user')
                            ->when( $request->user_id > 0, function( $q ) use($request) {
                                $q->where( 'invoices_.add_user', $request->user_id );
                            })
                            ->when( $request->item_id > 0 , function( $q ) use($request) {
                                $q->where( 'invoices_details.item_id', $request->item_id );
                            })
                            ->whereDate( 'invoices_.transaction_date', '>=', $start_date )
                            ->whereDate( 'invoices_.transaction_date', '<=', $end_date )
                            ->with('referenceBills')
                            ->where([
                                ['invoices_.invoice_type', 1],
                                ['invoices_.shop_id', get_session_val('serial_id', 'Integer')]
                            ])
                            ->select(DB::raw('
                                invoices_.id,
                                SUM(invoices_details.quantity * if (items_unit.id > 0, items_unit.unit_value, 1 )) AS quantity,
                                SUM(invoices_details.total_price) AS sale_price,
                                invoices_details.pay_price,
                                SUM(invoices_details.pay_price * invoices_details.quantity) AS purchase_price,
                                invoices_details.unit_id,
                                items.item_name as item_name,
                                items.id as item_id,
                                users.name as user_name,
                                users.id as user_id
                            '))
                            ->groupBy('items.id')
                            ->paginate(20);
        
        $commissions = CommissionPercent::where('shop_id',get_session_val('serial_id', 'Integer'))->get();

        $sales->getCollection()->map(function ($sale, $index) use ($commissions, $sales) {
            foreach ($sale->referenceBills as $referenceBill) {
                $sale->quantity -= $referenceBill->InvoiceDetails->where('unit_id', $sale->unit_id)->where('item_id', $sale->item_id)->sum('quantity');
                $sale->sale_price -= $referenceBill->InvoiceDetails->where('unit_id', $sale->unit_id)->where('item_id', $sale->item_id)->sum('total_price');
                $sale->purchase_price = $sale->pay_price * $sale->quantity;
            }

            if ($sale->quantity <= 0) $sales->forget($index);

            $sale->profit = ($sale->sale_price - $sale->purchase_price);
            if($sale->purchase_price == 0){
                $sale->profit_percent = 100;
            }else{
                $sale->profit_percent = $sale->profit / $sale->purchase_price;
            }
            $sale->comission_percent = $this->getComission($sale->profit_percent, $commissions);
            $sale->comission = $sale->comission_percent * $sale->sale_price;
            return $sale;
        });

        $users = User::where('shop_id', get_session_val('serial_id', 'integer'))->get(['id', 'user_name']);
        $items = Item::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        return view('salesreport::users.user_sales_report', compact('users','sales','items'));
    }

    private function getComission($profitPercent)
    {
        $commissions = CommissionPercent::where('shop_id',get_session_val('serial_id', 'Integer'))->get();
        $commission = $commissions->first(function($comm) use($profitPercent) {
            $profitPercent = number_format($profitPercent * 100, 2);
            return ($comm->from <= $profitPercent && $comm->to >= $profitPercent);
        });
        return $commission ? $commission->percent / 100 : 0;
    }

    private function getDataGroupedBy($grouping_column, $user_id, $dates)
    {
        return Invoices::join('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
            ->join('users','users.id','invoices_.add_user')
            ->when( $user_id , function( $q ) use($user_id) {
                if ($user_id > 0)
                {
                    $q->where( 'invoices_.add_user', $user_id );
                }
            })
            ->whereDate( 'invoices_.transaction_date', '>=', $dates['start'] )
            ->whereDate( 'invoices_.transaction_date', '<=', $dates['end'] )
            ->where([
                ['invoices_.invoice_type', 1],
                ['invoices_.shop_id', get_session_val('serial_id', 'Integer')]
            ])
            ->selectRaw('
                invoices_.transaction_date,
                invoices_.add_user,
                SUM(invoices_details.total_price) AS total_sale,
                SUM(invoices_details.pay_price * invoices_details.quantity) AS total_purchase,
                users.goal as goal,
                users.name as user_name
            ')
            ->groupBy($grouping_column)
            ->get();
    }



}
