<?php

namespace Modules\SalesReport\Http\Services;

use Modules\SalesReport\Http\Constants\TransactionType;
use Illuminate\Database\Eloquent\Collection ;

class TransactionProcess
{
    public function __construct(protected int $type, protected Collection $transactions, public array &$clauses, public array &$check_invoices)
    {
        $this->handle();
    }

    public function handle()
    {
        switch ($this->type) {
            case 0:
                $this->salesReturn();
                break;
            case 1:
                $this->billSale();
                break;
            case 2:
                $this->clientPaymentProcess();
                break;
            case 3:
                $this->billPurchases();
                break;
            case 5:
                $this->purchasesReturned();
                break;
            case 6:
                $this->clientPayToCompany();
                break;
            case 7:
                $this->companyReturnCash();
                break;
            case 9:
                $this->clientCashReturn();
                break;
            case 24:
                $this->revenueRecording();
                break;
            case 10:
            case 43:
                $this->expenseRecording();
                break;
            case 42:
                $this->sulfateEmployees();
                break;
        }
    }

    protected function sulfateEmployees() // سلفة
    {
        foreach ($this->transactions as $row) {
            $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['creditor'] += $row->amount;
            $this->clauses [TransactionType::GENERAL_EXPENSES] ['balance_balance_trial_balances'] ['debit'] += $row->amount;
        }
    }

    protected function salesReturn() // مردود مبيعات
    {
        foreach ($this->transactions as $row) {
            if ($row->effect) {
                $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['creditor'] += $row->bill_net_total;
            } else {
                $this->clauses [TransactionType::CLIENTS] ['balance_balance_trial_balances'] ['creditor'] += $row->bill_net_total - $row->amount;
                $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['creditor'] += $row->amount;
            }
            $this->clauses [TransactionType::RETURNS_SALES] ['balance_balance_trial_balances'] ['debit'] += $row->bill_net_total;
        }
    }

    protected function billSale() // فاتورة بيع
    {
        foreach ($this->transactions as $row) {
            $this->clauses [TransactionType::CASH_SALES] ['balance_balance_trial_balances'] ['debit'] += $row->amount;
            $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['creditor'] += $row->amount;

            if (! in_array($row->bill_id, $this->check_invoices)) {
                array_push($this->check_invoices, $row->bill_id);
                $this->clauses [TransactionType::DEFFERRED_SALES] ['balance_balance_trial_balances'] ['debit'] += $row->invoice->rest;
                $this->clauses [TransactionType::CLIENTS] ['balance_balance_trial_balances'] ['creditor']  += $row->invoice->rest;
            }
        }
    }

    protected function clientPaymentProcess() // عملية سداد من العميل
    {
        foreach ($this->transactions as $row) {
            $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['debit'] += $row->amount;
            $this->clauses [TransactionType::CLIENTS] ['balance_balance_trial_balances'] ['creditor'] += $row->amount;
        }
    }

    protected function billPurchases() // فاتورة مشتريات
    {
        foreach ($this->transactions as $row) {
            $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['debit'] += $row->amount;
            $this->clauses [TransactionType::CASH_PURCHASES] ['balance_balance_trial_balances'] ['creditor'] += $row->amount;

            if (! in_array($row->bill_id, $this->check_invoices)) {
                array_push($this->check_invoices, $row->bill_id);
                $this->clauses [TransactionType::DEFFERRED_PURCHASES] ['balance_balance_trial_balances'] ['debit'] += $row->invoice->rest;
                $this->clauses [TransactionType::SUPPLIERS] ['balance_balance_trial_balances'] ['creditor']  += $row->invoice->rest;
            }
        }
    }

    protected function purchasesReturned() // مردود مشتريات
    {
        foreach ($this->transactions as $row) {
            if ($row->effect) {
                $this->clauses [TransactionType::SUPPLIERS] ['balance_balance_trial_balances'] ['debit'] += $row->bill_net_total - $row->amount;
                $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['debit'] += $row->amount;
            } else {
                $this->clauses [TransactionType::SUPPLIERS] ['balance_balance_trial_balances'] ['debit'] += $row->bill_net_total;
            }
            $this->clauses [TransactionType::RETURNS_PURCHASES] ['balance_balance_trial_balances'] ['creditor'] += $row->bill_net_total;
        }
    }
    protected function companyReturnCash() // مردود نقدي من الشركة
    {
        foreach ($this->transactions as $row) {
            $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['debit'] += $row->amount;
            $this->clauses [TransactionType::RETURNS_PURCHASES] ['balance_balance_trial_balances'] ['creditor'] += $row->amount;
        }
    }

    protected function clientPayToCompany() // دفع للشركة
    {
        foreach ($this->transactions as $row) {
            $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['debit'] += $row->amount;
            $this->clauses [TransactionType::CLIENTS] ['balance_balance_trial_balances'] ['creditor'] += $row->amount;
        }
    }

    protected function clientCashReturn() // مردود نقدي للعميل
    {
        foreach ($this->transactions as $row) {
            $this->clauses [TransactionType::CLIENTS] ['balance_balance_trial_balances'] ['debit'] += $row->amount;
            $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['creditor'] += $row->amount;
        }
    }

    protected function expenseRecording() // تسجيل مصروفات
    {
        foreach ($this->transactions as $row) {
            $this->clauses [TransactionType::GENERAL_EXPENSES] ['balance_balance_trial_balances'] ['debit'] += $row->amount;
            $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['creditor'] += $row->amount;
        }
    }

    protected function revenueRecording() // تسجيل إيرادات
    {
        foreach ($this->transactions as $row) {
            $this->clauses [$row->salePoint->name] ['balance_balance_trial_balances'] ['debit'] += $row->amount;
            $this->clauses [TransactionType::GENERAL_REVENUE] ['balance_balance_trial_balances'] ['creditor']  += $row->amount;
        }
    }

    protected function supplierInstallments() // أقساط للمورد
    {
        foreach ($this->transactions as $row) {
            $this->clauses [TransactionType::SUPPLIERS] ['balance_balance_trial_balances'] ['debit'] += $row->amount;
            $this->clauses [TransactionType::INSTALLMENTS] ['balance_balance_trial_balances'] ['creditor']    += $row->amount;
        }
    }

    protected function clientInstallments() // أقساط للعميل
    {
        foreach ($this->transactions as $row) {
            $this->clauses [TransactionType::INSTALLMENTS] ['balance_balance_trial_balances'] ['debit']    += $row->amount;
            $this->clauses [TransactionType::CLIENTS] ['balance_balance_trial_balances'] ['creditor'] += $row->amount;
        }
    }
}
