@extends('master::layouts.master')

@section('css-scripts')
    <link href="{{ asset('css/calenders/jquery.calendars.picker.css') }}" rel="stylesheet">

    <style>
        .load {
            cursor: wait;
            position: relative;
        }

        .load::before {
            font-family: "Font Awesome 5 Free";
            content: "\f021";
            position: absolute;
            left: 50%;
            top: 50%;
            transform: translate(-50%, -50%);
            font-size: 19px;
            font-weight: bolder;
            color: #000;
            z-index: 1029;
            animation: load .9s linear infinite;
        }

        .load::after {
            content: "";
            display: block;
            position: absolute;
            top: 0;
            left: 0;
            bottom: 0;
            right: 0;
            background-color: rgb(255, 255, 255);
            opacity: 0.8;
            z-index: 1028;
            height: 100%;
            transition: all .5s ease-in-out;
        }

        @keyframes load {
            from { transform: rotate(0deg); }
            to { transform: rotate(360deg); }
        }

    </style>
@endsection

@section('content')
    <main class="global-main ">
        <div class="container">
            <div class="content-center ">
                <form id="filter-form" method="post" action="{{ route('salesreports.trial-balance.filter') }}" class="py-3 px-5" style="background-color: #e8eaef">
                    @csrf
                    <div class="row">
                        <div class="col-md-2"></div>
                        <div class="col-md-3">
                            <label>بداية الفترة</label>
                            <input type="text" id="invoice_date" name="start_date" class="form-control input-sm input-light date is-datepicker" value="{{ request()->start_date ?? now()->firstOfMonth()->format('Y-m-d') }}">
                            <span> <i class="fa fa-calendar-alt calendar-icon"></i> </span>
                        </div>

                        <div class="col-md-3">
                            <label>نهاية الفترة </label>
                            <input type="text" id="invoice_date" name="end_date" class="form-control input-sm input-light date is-datepicker" value="{{ request()->end_date ?? now()->lastOfMonth()->format('Y-m-d') }}">
                            <span> <i class="fa fa-calendar-alt calendar-icon"></i> </span>
                        </div>

                        <div class="col-md-2 d-flex align-items-end justify-content-end">
                            <button type="submit" class="btn btn-sm btn-primary" style="margin-top: 15px;"> <i class="fa fa-filter fa-xs"></i> فلتر </button>
                        </div>
                    </div>
                </form>

                <div id="load-data">
                    @include('salesreport::trial-balance.result')
                </div>
            </div>

        </div>
    </main>
@stop

@section('js-scripts')
    <script src="{{ asset('js/calenders/jquery.calendars.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.plus.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.plugin.js') }}"></script>
    <script src="{{ asset('js/calenders/jquery.calendars.picker.js') }}"></script>

    <script>
        $(function () {
            $(".date").calendarsPicker({ dateFormat: 'Y-mm-dd' }).on('keydown keypress keyup change', function (e) { e.preventDefault(); });

            $('#filter-form').on('submit', function (e) {
                e.preventDefault();
                let form = $(this);
                form.parent().addClass('load');
                $.ajax({
                    url: form.attr('action'),
                    type: form.attr('method'),
                    data: form.serialize(),
                    success: function (response) {
                        $('#load-data').empty().append(response);
                    },
                    complete: function () {
                        form.parent().removeClass('load');
                    },
                });
            });

            $('#filter-form').submit();
        });
    </script>
@endsection
