<?php

namespace Modules\Settings\Http\Controllers;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;
use Modules\Settings\Entities\InvoiceSetting;
use Modules\Settings\Http\Requests\PrintSettingRequest;

class PrintSettingController extends Controller
{
    public function index()
    {
        $this->checkColumns();
        $invoice_setting = InvoiceSetting::first();
        $html = $this->setInputsData();
        return view('settings::print.index', compact('invoice_setting', 'html'));
    }

    public function update(PrintSettingRequest $request)
    {
        InvoiceSetting::updateOrCreate(['shop_id' => auth()->user()->shop_id], $request->validated());
        return back()->with('success', 'تم تحديث الاعدادات');
    }

    protected function setInputsData()
    {
        return [
            trans('storeInfo.title.items_settings') => [
                trans("storeInfo.inputs.show_item_barcode") => 'show_item_barcode',
                trans("storeInfo.inputs.show_item_name")    => 'show_item_name',
                trans("storeInfo.inputs.show_about")        => 'show_about',
                trans("storeInfo.inputs.show_item_serial")  => 'show_item_serial',
                trans("storeInfo.inputs.show_unites")       => 'show_unites',
            ],
            trans('storeInfo.title.client_settings') => [
                trans("storeInfo.inputs.show_client_balance") => 'show_client_balance',
                trans("storeInfo.inputs.show_client_name")    => 'show_client_name',
                trans("storeInfo.inputs.show_client_address") => 'show_client_address',
                trans("storeInfo.inputs.show_phone")          => 'show_phone',
                trans("storeInfo.inputs.show_client_code")    => 'show_client_code',
            ],
            trans('storeInfo.title.invoice_settings') => [
                trans("storeInfo.inputs.bill_type")         => 'bill_type',
                trans("storeInfo.inputs.show_invoice_rest") => 'show_invoice_rest',
                trans("storeInfo.inputs.show_signature")    => 'show_signature',
                trans("storeInfo.inputs.show_payment_type") => 'show_payment_type',
                trans("storeInfo.inputs.qr_code")           => 'qr_code',
                trans("storeInfo.inputs.show_bill_header")  => 'show_bill_header',
                trans("storeInfo.inputs.show_bill_message") => 'show_bill_message',
                trans("storeInfo.inputs.show_logo")         => 'show_logo',
            ],
        ];
    }

    protected function checkColumns()
    {
        if (! Schema::hasColumn( (new InvoiceSetting)->getTable(), 'show_client_code') ) {
            DB::statement("ALTER TABLE `bill_setting` ADD `show_client_code` BOOLEAN NOT NULL DEFAULT FALSE;");
        }
    }
}
