<?php

namespace Modules\Settings\Http\Controllers;

use App\Models\User;
use App\Models\Zone;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Settings\Entities\Badrshop;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Mcamara\LaravelLocalization\Facades\LaravelLocalization;
use Modules\Clients\Entities\Client;
use Modules\Clients\Entities\Country;
use Modules\Category\Entities\Unit;
use Modules\Clients\Entities\ClientTransaction;
use Modules\Employee\Entities\Employee;
use Modules\Invoices\Entities\ItemType;
use Modules\User\Entities\Log;
use App\Models\systemLog;
use Illuminate\Support\Carbon;
class SettingsController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }
    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index()
    {
        if (! authorized('can_display_settings'))
        {
            return unAuthorized(trans('users.show_settings'));
        }

        $badrshop = Badrshop::where('serial_id', get_session_val('serial_id' ,'integer'))->with('getCountry')->get()->first();
        $countries = Country::orderBy('name_ar', 'asc')->get(['id', 'iso_alpha2', 'name_ar', 'currency_code']);

        return view('settings::index' , ['badrshop'=>$badrshop, 'countries'=>$countries]);
    }

    /**
     * Show the form for creating a new resource.
     * @return Renderable
     */
    public function create()
    {
        return view('settings::create');
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Renderable
     */
    public function save(Request $request){
        try {
            DB::transaction(function () use ($request) {
                Badrshop::where('serial_id', get_session_val('serial_id', 'integer'))
                ->update([
                    'multi_currency' => $request->multi_currency ?? false,
                    'show_client_balance' => $request->show_client_balance ?? false,
                    'enter_quant' => $request->enter_quant,
                    'sale_details' => $request->sale_details,
                    'negative_money' => $request->sale_details_safe,
                    'sale_details_safe' => $request->sale_details_safe,
                    'select_client' => $request->select_client,
                    'company_ratio' => $request->company_ratio,
                    'decimal_num_price' => $request->decimal_num_price,
                    'decimal_num_quant' => $request->decimal_num_quant,
                    'client_last_saleprice' => $request->client_last_saleprice,
                    'repeat_client_name' => $request->repeat_client_name,
                    'check_lowest_price' => $request->check_lowest_price,
                    'multi_price' => $request->multi_price,
                    'units_sup'   => $request->multi_units,
                    'insatllments' => $request->insatllments,
                    'netowrk_payment' => $request->netowrk_payment,
                    'incom_total_filed' => $request->incom_total_filed,
                    'income_saleprice' => $request->income_saleprice,
                    'pay_price_update' => $request->pay_price_update,
                    'Profits_Calc_method' => $request->Profits_Calc_method,
                    'show_expire_date' => $request->show_expire_date,
                    'expire_notification' =>$request->expire_notification,
                    'einvoice' => $request->apply_vats == 1 && $request->country == 63 ? $request->einvoice : 0,
                    'use_parts_cash_proforma_invoice' => $request->apply_vats ? $request->use_parts_cash_proforma_invoice : 0,
                    'submit_einvoice' => $request->submit_einvoice,
                    'billno_show' => $request->billno_show,
                    'apply_vats' => $request->apply_vats,
                    'bill_size' => $request->bill_size,
                    'auto_print' => $request->auto_print,
                    'invoice_default_payment_type' => $request->invoice_default_payment_type,
                    'screen_type' => $request->screen_type,
                    'show_input_keys' => $request->show_input_keys,
                    'printing_shop_info_src' => $request->printing_shop_info_src,
                    'barcode_reader' => $request->barcode_reader,
                    'zone' => $request->zone,
                    'country' => $request->country,
                    'currency' => $request->currency,
                    'manufacturing' => $request->manufacturing,
                    'definite_while_selling' => $request->definite_while_selling,
                    'home_delivery' => $request->home_delivery,
                    'delivery_fee_type' => $request->delivery_fee_type,
                    'add_delivery_fees_to_the_cash_drawer' => $request->add_delivery_fees_to_the_cash_drawer,
                    'show_delivery_charges_in_the_invoice' => $request->show_delivery_charges_in_the_invoice,
                    'confirm_store_transfer'=>$request->confirm_store_transfer,
                    'telephone_required'=>$request->telephone_required,
                    'client_address_required'=>$request->client_address_required ?? false,
                    'client_limit'=>$request->client_limit,
                    'pending_bills' =>$request->pending_bills,
                    'enable_client_sales' =>$request->enable_client_sales ?? false,
                    'module' =>$request->module ?? 1,
                    'enable_commission_rates' =>$request->enable_commission_rates ?? false,
                    'enable_show_stores_web' =>$request->enable_show_stores_web ?? false,
                   // 'ecomme' =>$request->ecomme ?? false,
                    'maintenance_module' =>$request->maintenance_module ?? false,
                    'show_item_image_in_press_sale' =>$request->show_item_image_in_press_sale ?? false,
                    'activate_maintenance_mode' =>$request->activate_maintenance_mode ?? false,
                    'maintenance_message'       => $request->maintenance_message ?? ''
                ]);

                setTimeZone();
            });
            $badrshop = Badrshop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
            set_session_arr_val($badrshop, $badrshop->serial_id);
            $request->session()->put('locale', getCountryCode($request->country));
            $this->createLog(Carbon::now(),"تم التعديل علي صلاحيات البرنامج");
            return redirect()->route('master.home')->with('success', 'تم التعديل بنجاح');
            // return redirect()->route('settings.index')->with('success', 'edits updated!');
        } catch (Exception $ex) {
            return $ex->getMessage();
        }
    }


    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function show($id)
    {
        return view('settings::show');
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function edit($id)
    {
        return view('settings::edit');
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Renderable
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy($id)
    {
        //
    }

    public function countryZones(Request $request)
    {
        $data = Zone::where('country_code', $request->id)->get();
        return response()->json(
            [
                'status' => true,
                'data' =>  $data
            ]);
    }

    public function resetAccount(Request $request)
    {
        try {
            if (! Hash::check($request->password, Auth::user()->password))
            {
                return redirect()->back()->with('error', 'خطأ في الرقم السري');
            }
        } catch (\Throwable $th) {
            return redirect()->back()->with('error', 'حدث خطأ ما ... تأكد من ادخال الرقم السري ثم حاول مجددا');
        }
        $escaped_tables = [
            'general_einvoice_settings',
            'charge_cities',
            'countries',
            'failed_jobs',
            'regions',
            'sms_direct_messages',
            'sms_providers',
            'sms_provider_additional_params',
            'sms_provider_messages',
            'sms_provider_track_activity',
            'sms_status',
            'ticket_replies',
            'zone',
            'migrations',
            'suppliers_groups',
            'project_details',
            "badr_shop",
            "sale_points",
            "gift_history",
            "sms_history",
            "sms_setting",
            "items",
            "items_collection",
            "items_colors",
            "items_group",
            "items_group_types",
            "items_prices",
            "items_sizes",
            "stores",
            "store_items",
            "prices_list",
            "items_type",
            "units",
            "items_unit",
            "supplier",
            "item_images",
            "multi_supplier",
            "barcodesetting",
            "bill_setting",
            "bills_add",
            "regions",
            "charge_cities",
            'app_clients',
            'clients',
            'password_resets',
            'notifications',
            "einvoices",
            "einvoice_details",
            "einvoice_detail_vats",
            "einvoice_settings",
            "einvoice_stages",
            "address",
            "cache",
            "cache_locks",
            "personal_access_tokens",
            "sessions",
            "item_vat",
            "oauth_access_tokens",
            "oauth_auth_codes",
            "oauth_clients",
            "oauth_personal_access_clients",
            "oauth_refresh_tokens",
            "google_cities",
            "clients",
            "clients_groups",
            "users",
            "spending",
            "imports",
        ];

        try {
            DB::beginTransaction();
                $this->emptyDbTables($escaped_tables);
                $this->createClientTransactions();
            DB::commit();

            $this->logEvent('تصفير العمليات');

        } catch (\Throwable $th) {
            return redirect()->back()->with('error', 'حدث خطأ ما');
        }

        return redirect()->route('master.home')->with('success', 'تم حذف العمليات بنجاح');
    }

    protected function createClientTransactions()
    {
        $trans = [];
        $clients = Client::select('id', 'balance')->where('shop_id', auth()->user()->shop_id)->pluck('balance', 'id');
        foreach ($clients as $id => $balance) {
            $trans [] = [
                'shop_id' => auth()->user()->shop_id,
                'date_time' => date('Y-m-d H:i:s'),
                'client_id' => $id,
                'amount' => 0,
                'type' => 92, // تصفير عمليات
                'pay_day' => date('Y-m-d H:i:s'),
                'balance' => $balance,
                'user_id' => auth()->id()
            ];
        }
        ClientTransaction::insert($trans);
    }

    public function deleteAccount(Request $request)
    {
        try {
            if (! Hash::check($request->password, Auth::user()->password))
            {
                return redirect()->back()->with('error', 'خطأ في الرقم السري');
            }
        } catch (\Throwable $th) {
            return redirect()->back()->with('error', 'حدث خطأ ما ... تأكد من ادخال الرقم السري ثم حاول مجددا');
        }

        $escaped_tables = [
            'general_einvoice_settings',
            'charge_cities',
            'countries',
            'failed_jobs',
            'regions',
            'sms_direct_messages',
            'sms_providers',
            'sms_provider_additional_params',
            'sms_provider_messages',
            'sms_provider_track_activity',
            'sms_status',
            'ticket_replies',
            'zone',
            'migrations',
            'suppliers_groups',
            'project_details',
            "badr_shop",
            // "users",
            "sale_points",
            "gift_history",
            "sms_history",
            "sms_setting",
            "stores",
            "barcodesetting",
            "bill_setting",
            "regions",
            "charge_cities",
            'app_clients',
            'password_resets',
            'notifications',
            "einvoices",
            "einvoice_details",
            "einvoice_detail_vats",
            "einvoice_settings",
            "einvoice_stages",
            "address",
            "cache",
            "cache_locks",
            "personal_access_tokens",
            "sessions",
            "oauth_access_tokens",
            "oauth_auth_codes",
            "oauth_clients",
            "oauth_personal_access_clients",
            "oauth_refresh_tokens",
            "google_cities"
        ];

        try {
            $this->emptyDbTables($escaped_tables);

            $update_safes = "UPDATE sale_points SET money_point = 0 WHERE shop_id = " . get_session_val('serial_id', 'integer');
            DB::statement($update_safes);

            Client::create([
                'client_name' => trans('default.default-model', ['model' => trans('default.client-supplier')]),
                'shop_id' => get_session_val('serial_id', 'integer'),
                'client_role' => 2
            ]);

            ItemType::create([
                'name' => trans('default.default-model', ['model' => trans('default.category')]),
                'shop_id' =>get_session_val('serial_id', 'integer')
            ]);


            Unit::create([
                'name' => trans('default.default-model', ['model' => trans('default.unit')]),
                'shop_id' =>get_session_val('serial_id', 'integer')
            ]);

            $this->logEvent('حذف الحساب و انشائه من جديد');

        } catch (\Throwable $th) {
            return redirect()->back()->with('error', 'حدث خطأ ما');
        }

        return redirect()->route('master.home')->with('success', 'تم حذف الحساب و انشائة من جديد بنجاح');
    }

    private function getDbTablesNames()
    {
        return DB::select(DB::raw("SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE = 'BASE TABLE' AND TABLE_SCHEMA='". DB::connection()->getDatabaseName() ."'"));
    }

    private function getDbTableColumns($table)
    {
        return DB::select(DB::raw("SELECT `COLUMN_NAME` FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `COLUMN_NAME` = 'shop_id' AND `TABLE_SCHEMA`='".DB::connection()->getDatabaseName()."' AND `TABLE_NAME`='$table';"));
    }

    private function emptyDbTables($escaped_tables)
    {
        $all_table_names = $this->getDbTablesNames();
        foreach ($all_table_names as $table_name) {
            if (! in_array($table_name->TABLE_NAME, $escaped_tables))
            {
                if($table_name->TABLE_NAME == 'users'){
                    Employee::where(['shop_id' => auth()->user()->shop_id, 'is_employee' => 1])->delete();
                    continue;
                }
                $per_shop = '';
                if (count($this->getDbTableColumns($table_name->TABLE_NAME))) {
                    $per_shop = "WHERE shop_id = " . get_session_val('serial_id', 'integer');
                }
                DB::statement("DELETE FROM `{$table_name->TABLE_NAME}`  $per_shop");
            }
        }
    }

    private function logEvent($msg)
    {
        Log::create([
            'user_id' => Auth::user()->id,
            'date_event' => date('Y-m-d H:i:s'),
            'event' => $msg,
            'shop_id' => get_session_val('serial_id', 'integer'),
            'type' => -1
        ]);
    }
}
