<?php

namespace Modules\SpendsAndIncoms\Http\Controllers;

use App\Http\Controllers\Controller;
use Exception;
use Illuminate\Support\Facades\DB;
use Modules\SpendsAndIncoms\Entities\Income;
use Modules\SpendsAndIncoms\Entities\IncomeItem;
use Modules\SpendsAndIncoms\Entities\SpendingItem;
use Modules\SpendsAndIncoms\Entities\User;
use Modules\SpendsAndIncoms\Http\Requests\RevenuesRequest;

class RevenuesController extends Controller
{
    public function index()
    {
        $rows = IncomeItem::where('shop_id' ,get_session_val('serial_id' ,'integer'))->with('users')->get();
        return view('spendsandincoms::revenues.index' , compact('rows'));
    }

    public function create()
    {
        $users = User::select('id', 'user_name')->where('is_employee', 0)->where('shop_id', auth()->user()->shop_id)->pluck('user_name', 'id')->toArray();
        $title = trans(('native.a_new_item'));
        $selected_users = [];
        return view('spendsandincoms::revenues.form' , compact('users', 'title', 'selected_users'));
    }

    public function store(RevenuesRequest $request)
    {
        $row = $this->saveData($request->validated());
        if ($row instanceof IncomeItem) {
            session()->flash('success', 'تم انشاء ايراد جديد بنجاح');
            return response()->json([], 200);
        }
        return response()->json(['error' => $row->getMessage()], 500);
    }

    public function edit($id)
    {
        $row = IncomeItem::where('shop_id' ,get_session_val('serial_id' ,'integer'))->findOrFail($id);
        $users = User::select('id', 'user_name')->where('is_employee', 0)->where('shop_id', auth()->user()->shop_id)->pluck('user_name', 'id')->toArray();
        $title = 'تعديل بيانات مصروف';
        $selected_users = $row->users()->pluck("id")->toArray();
        return view('spendsandincoms::revenues.form' , compact('users', 'title', 'selected_users', 'row'));
    }

    public function update(RevenuesRequest $request, $id)
    {
        $row = $this->saveData($request->validated(), $id);
        if ($row instanceof IncomeItem) {
            session()->flash('success', 'تم تعديل الايراد بنجاح');
            return response()->json([], 200);
        }
        return response()->json(['error' => $row->getMessage()], 500);
    }

    public function destroy($id)
    {
        if ( Income::where('shop_id' ,get_session_val('serial_id' ,'integer'))->where('import_id' , $id)->count() > 0 ) {
            return response()->json(['message' => 'لا يمكن حذف الايراد لارتباطه ببيانات اخري'], 404);
        }
        IncomeItem::where('shop_id' ,get_session_val('serial_id' ,'integer'))->where('id', $id)->delete();
        return response()->json(['message' => 'تم حذف الايراد بنجاح'], 200);
    }

    protected function saveData(array $validated, $id = null)
    {
        try {
            DB::beginTransaction();
                $row = IncomeItem::updateOrCreate(['id' => $id, 'shop_id' => auth()->user()->shop_id], ['name' => $validated['name']]);
                if ( isset($validated['users']) && is_array( $validated['users'] ) )
                    $row->users()->sync( $validated['users'] );
            DB::commit();
            return $row;
        } catch(Exception $e) {
            DB::rollBack();
            return $e;
        }
    }
}
