<?php

namespace Modules\StageMaintenance\Entities;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Modules\StageMaintenance\Traits\isEnabledScope;

class Finishing extends Model
{
    use HasFactory;
    use isEnabledScope;

    protected $fillable = [
        'name',
        'is_enabled',
        'order',
    ];

    protected static function newFactory()
    {
        return \Modules\StageMaintenance\Database\factories\FinishingFactory::new();
    }

    public function getPreviousFinishingAttribute()
    {
        if($this->order){
            return Finishing::where('order', '<', $this->order)->orderBy('order', 'DESC')->first();
        }
    }
    public function updateNextFinishingOrder()
    {
        self::where('is_enabled', 0)->update([
            'order' => null
        ]);
        if($this->is_enabled){
            $nextFinishing = Finishing::where('order', ">=" ,$this->order)
                ->enabled()
                ->where('id', '!=', $this->id)
                ->orderBy('order')
                ->first();
            if($nextFinishing){
                $nextFinishing->update([
                    'order' => $this->order + 1
                ]);
                $nextFinishing->updateNextFinishingOrder();
            }
        }
        return $this;
    }

    public static function updateAllNextFinishingOrder()
    {
        $firstFinishing = Finishing::OrderBy('order')->enabled()->first();
        if(!$firstFinishing) return;
        $firstFinishing->update(['order' => 1]);
        $firstFinishing->updateNextFinishingOrder();
    }

}
