<?php

namespace Modules\StageMaintenance\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\StageMaintenance\Entities\Finishing;
use Modules\StageMaintenance\Http\Requests\FinishingRequest;
use Modules\StageMaintenance\Transformers\FinishingResource;

class FinishingController extends Controller
{
    public function create(Request $request)
    {
        $finishings = Finishing::when($request->filled('name'), function ($q) use ($request){
            $q->where('name', 'like', "%{$request->name}%");
        })
        ->orderBy('order')
        ->get();
        return view('stagemaintenance::finishing.create', compact('finishings'));
    }

    public function store(FinishingRequest $request)
    {
        $data = $request->validated();
        $finishing = Finishing::create($data);
        if($finishing->is_enabled){
            $finishing->updateNextFinishingOrder();
        }
        $finishings = Finishing::orderBy('order')->get();

        return response()->json([
            'success' => true,
            'data'    => FinishingResource::collection($finishings),
            'message' => 'تم اضافة المرحلة بنجاح',
        ]);
    }

    public function show(Finishing $finishing)
    {
        return response()->json([
            'success' => true,
            'data'    => $finishing,
        ]);
    }

    public function update(FinishingRequest $request,Finishing $finishing)
    {
        $data = $request->validated();
        $finishing->update($data);
        if($finishing->is_enabled) {
            $finishing->updateNextFinishingOrder();
        } else {
            $finishing->update([
                'order' => null
            ]);
            Finishing::updateAllNextFinishingOrder();
        }
        $finishings = Finishing::orderBy('order')->get();
        return response()->json([
            'success' => true,
            'data'    => FinishingResource::collection($finishings),
            'message' => 'تم تعديل المرحلة بنجاح',
        ]);
    }

    public function destroy(Finishing $finishing)
    {
        $finishing->delete();
        if($finishing->is_enabled){
            Finishing::updateAllNextFinishingOrder();
        }
        $finishings = Finishing::orderBy('order')->get();
        return response()->json([
            'success' => true,
            'data'    => FinishingResource::collection($finishings),
            'message' => 'تم حذف المرحلة بنجاح',
        ]);
    }
}
