<?php

namespace Modules\StageMaintenance\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\StageMaintenance\Entities\Machine;
use Modules\StageMaintenance\Http\Requests\MachineRequest;

class MachineController extends Controller
{
    /**
     * Show the form for creating a new resource.
     * @return Renderable
     */
    public function create(Request $request)
    {
        $machines = Machine::when($request->filled('name'), function ($q) use ($request){
            $q->where('name', 'like', "%{$request->name}%");
        })->get();
        return view('stagemaintenance::machine.create', compact('machines'));
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Renderable
     */
    public function store(MachineRequest $request)
    {
        $data = $request->validated();
        $machine = Machine::create($data);
        return response()->json([
            'success' => true,
            'data'    => $machine,
            'message' => 'تم اضافة الماكينة بنجاح',
        ]);
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function show(Machine $machine)
    {
        return response()->json([
            'success' => true,
            'data'    => $machine,
        ]);
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Renderable
     */
    public function update(MachineRequest $request, Machine $machine)
    {
        $data = $request->validated();
        $machine->update($data);
        return response()->json([
            'success' => true,
            'data'    => $machine,
            'message' => 'تم تعديل الماكينة بنجاح',
        ]);
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy(Machine $machine)
    {
        $machine->delete();
        return response()->json([
            'success' => true,
            'message' => 'تم حذف الماكينة بنجاح',
        ]);
    }
}
