<?php

namespace Modules\StageMaintenance\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\StageMaintenance\Entities\Shift;
use Modules\StageMaintenance\Http\Requests\ShiftRequest;

class ShiftController extends Controller
{
    public function create(Request $request)
    {
        $shifts = Shift::when($request->filled('name'), function ($q) use ($request){
            $q->where('name', 'like', "%{$request->name}%");
        })->get();
        return view('stagemaintenance::shifts.create', compact('shifts'));
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Renderable
     */
    public function store(ShiftRequest $request)
    {
        $data = $request->validated();
        $shift = Shift::create($data);
        return response()->json([
            'success' => true,
            'data'    => $shift,
            'message' => 'تم اضافة الوردية بنجاح',
        ]);
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function show(Shift $shift)
    {
        return response()->json([
            'success' => true,
            'data'    => $shift,
        ]);
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Renderable
     */
    public function update(ShiftRequest $request,Shift $shift)
    {
        $data = $request->validated();
        $shift->update($data);
        return response()->json([
            'success' => true,
            'data'    => $shift,
            'message' => 'تم تعديل الوردية بنجاح',
        ]);
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy(Shift $shift)
    {
        $shift->delete();
        return response()->json([
            'success' => true,
            'message' => 'تم حذف الوردية بنجاح',
        ]);
    }
}
