<?php

namespace Modules\StageMaintenance\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\StageMaintenance\Entities\Worker;
use Modules\StageMaintenance\Http\Requests\WorkerRequest;

class WorkerController extends Controller
{
    public function create(Request $request)
    {
        $workers = Worker::when($request->filled('name'), function ($q) use ($request){
            $q->where('name', 'like', "%{$request->name}%");
        })->get();
        return view('stagemaintenance::workers.create', compact('workers'));
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Renderable
     */
    public function store(WorkerRequest $request)
    {
        $data = $request->validated();
        $worker = Worker::create($data);
        return response()->json([
            'success' => true,
            'data'    => $worker,
            'message' => 'تم اضافة العامل بنجاح',
        ]);
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function show(Worker $worker)
    {
        return response()->json([
            'success' => true,
            'data'    => $worker,
        ]);
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Renderable
     */
    public function update(WorkerRequest $request,Worker $worker)
    {
        $data = $request->validated();
        $worker->update($data);
        return response()->json([
            'success' => true,
            'data'    => $worker,
            'message' => 'تم تعديل العامل بنجاح',
        ]);
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy(Worker $worker)
    {
        $worker->delete();
        return response()->json([
            'success' => true,
            'message' => 'تم حذف العامل بنجاح',
        ]);
    }
}
