<?php

namespace Modules\StageMaintenance\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;

class ManufacturingRequest extends FormRequest
{
    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
        $base = [
            'start_date' => 'required|date',
            'item_id' => 'required|exists:items,id',
            'color' => 'required',
            'machine_id' => 'required|exists:machines,id',
            'quantity' => 'required|numeric|min:1',
            'worker_id' => 'required|exists:workers,id',
            'shift_id' => 'required|exists:shifts,id',
            'materials' => 'required|array',
            'materials.*.material_id' => 'required|exists:items,id',
            'materials.*.material_quantity' => 'required|numeric|min:1',
        ];
        return match (request()->method()){
            "POST" => $base,
            "PUT", "PATCH" => $base,
        };
    }

    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    public function attributes()
    {
        return [
            'start_date' => __('stagemaintenance::validation.start_date'),
            'item_id' => __('stagemaintenance::validation.item_id'),
            'color' => __('stagemaintenance::validation.color'),
            'machine_id' => __('stagemaintenance::validation.machine_id'),
            'quantity' => __('stagemaintenance::validation.quantity'),
            'worker_id' => __('stagemaintenance::validation.worker_id'),
            'shift_id' => __('stagemaintenance::validation.shift_id'),
            'materials' => __('stagemaintenance::validation.materials'),
            'material_id' => __('stagemaintenance::validation.material_id'),
            'material_quantity' => __('stagemaintenance::validation.material_quantity'),
        ];
    }
}
