@push('js')
<script>
    $.ajaxSetup({
        headers: {
            'X-CSRF-TOKEN': "{{ csrf_token() }}"
        }
    })

    $(document).ajaxSuccess(function(event,xhr,options){
        if(!xhr.responseJSON) return;
        if (xhr.responseJSON.success && xhr.responseJSON.message) {
            swalSuccess(xhr.responseJSON.message);
        } else if(xhr.responseJSON.message) {
            swalError(xhr.responseJSON.message);
        }
    });

    $(document).ajaxError(function(event,xhr,options){
        if(!xhr.responseJSON) return;
        if (xhr.responseJSON.errors) {
            let errors = xhr.responseJSON.errors;
            let allErrors = [];
            for(i in errors){
                for(j in errors[i]){
                    allErrors.push(errors[i][j]);
                }
            }
            swalError(allErrors.join('<br>'));
        }
    });

    const Toast = Swal.mixin({
        toast: true,
        position: 'top-start',
        showConfirmButton: false,
        timer: 2000,
    })

    function swalSuccess(msg) {
        Toast.fire({
            icon: 'success',
            title: 'Success!',
            text: msg
        })
    }

    function swalError(msg) {
        Toast.fire({
            icon: 'error',
            title: 'Error!',
            html: msg
        })
    }
    function swalConfirm(callback, msg = "@lang('messages.wont_revert')") {
        Swal.fire({
            title: '@lang('messages.are_you_sure')',
            text: msg,
            icon: 'warning',
            showCancelButton: true,
            confirmButtonColor: '#3085d6',
            cancelButtonColor: '#d33',
            confirmButtonText: '@lang('messages.yes_do_it')',
            cancelButtonText: '@lang('messages.cancel')'
        }).then((result) => {
            if (result.isConfirmed) {
                callback();
            }
        })
    }

    $(document).on('submit', '.ajax-form', function(e){
        e.preventDefault();
        let form = $(this);
        let url = form.attr('action');
        let formData = new FormData(this);
        let formMethod = form.find('[name="_method"]').val() ?? null;
        let data = form.serializeArray().filter((item) => item.name !== '_token');
        let successCallback = form.attr('success');
        let inputs = form.find('input,textarea,select,button');
        inputs.attr('disabled', 'disabled');
        $.ajax({
            method: "POST",
            data: formData,
            url,
            processData: false,
            contentType: false,
            success: res => {
                if(!res.success) return
                if(successCallback)
                    window[successCallback](data, res)
                if(formMethod) {
                    form.closest('.modal').modal('hide');
                    if(formMethod.toLowerCase() == 'delete')
                        $(`#${form.attr('row')}`).remove();
                }
                form[0].reset();
            },
            complete: () => {
                inputs.removeAttr('disabled');
            }
        })
    })

    $(document).on('click', '.update-modal-btn', function (e) {
        e.preventDefault();
        let modal = $($(this).data('target'));
        let url = $(this).data('url');
        let dataUrl = $(this).data('dataurl');
        let form = modal.find('form');
        form.attr('action', url);
        form[0].reset();
        form.find('[type="checkbox"]').removeAttr('checked');
        let inputs = form.find('input,textarea,select,button, checkbox');
        inputs.attr('disabled', 'disabled');
        $.ajax({
            url: dataUrl,
            method: 'GET',
            success: function (res) {
                if(res.success){
                    let data = res.data;
                    for(i in data){
                        form.find(`input[name="${i}"]:not([type="checkbox"])`).val(data[i]);
                        form.find(`input[name="${i}"][type="checkbox"]`).prop('checked',Boolean(data[i])).val('1');
                        form.find(`textarea[name="${i}"]`).text(data[i]);
                        form.find(`select[name="${i}"] option`).removeAttr('selected');
                        form.find(`select[name="${i}"] option[value="${data[i]}"]`).attr('selected', 'selected');
                    }
                }
            }
        }).done(() => {
            inputs.removeAttr('disabled').trigger('change');
        });
    });


    $(document).on('click', '.delete-modal-btn', function (e) {
        e.preventDefault();
        let row = $($(this).closest('tr'));
        let modal = $($(this).data('target'));
        let url = $(this).data('url');
        let form = modal.find('form');
        form.attr('action', url);
        form.attr('row', row.attr('id'));
    });
</script>
@endpush
