<?php

namespace Modules\Store\Http\Controllers;

use App\Services\Uploaders\StoreImgUploader;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;

use Illuminate\Http\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use Intervention\Image\Facades\Image;
use Modules\Store\Entities\Item;
use Modules\Store\Entities\Shelf;
use Modules\Store\Entities\Store;
use Modules\Store\Entities\StoreItem;
use Modules\User\Entities\Log;

class StoreController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index()
    {
       $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->paginate(20);
        return view('store::index', compact('stores'));
    }

    /**
     * Show the form for creating a new resource.
     * @return Response
     */
    public function create()
    {
        return view('store::create');
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|unique_shop_year:stores,store_name,shop_id,' . get_session_val('serial_id', 'integer'),
        ]);


        DB::transaction(function () use ($request) {
            $store = new Store();
            $store->store_name = $request->name;
            $store->tax_number = $request->tax_number;
            $store->address = $request->address;
            $store->bill_header = $request->bill_header;
            $store->message_bill = $request->message_bill;

            if ($request->has('img')) {
                if ($request->file('img') != null) {
                    $name_one = 'upload/' . get_session_val('serial_id', 'integer') . '/stores/';
                    $destinationPath = '../' . $name_one;

                    File::isDirectory($destinationPath) or File::makeDirectory($destinationPath, 755, true, true);

                    $fileExt = $request->file('img')->getClientOriginalExtension();
                    $file_name = $name_one . date("Y-m-d") . '_' . time() . '.' . $fileExt;
                    $request->file('img')->move($destinationPath, $file_name);
                }
            }
            $store->img = isset($file_name) ? $file_name : "";

            $store->shop_id = get_session_val('serial_id', 'integer');
            $store->save();

            $shelf = new Shelf();
            $shelf->store_id = $store->id;
            $shelf->shelf_name = ' افتراضي';
            $shelf->add_user = auth()->user()->id;
            $shelf->shop_id = get_session_val('serial_id', 'integer');
            $shelf->save();

            Log::create(
            [ 'date_event' => date('Y-m-d H:i:s'),
                'event' => trans('native.add_the_store') . $store->store_name,
                'user_id' => Auth::id(),
                'shop_id' => get_session_val('serial_id', 'integer')
            ]);

        });

        return redirect()->route('stores.index')->with(['success' => trans('native.correct_save')]);
    }

    /**
     * Show the specified resource.
     * @return Response
     */
    public function show($id)
    {

        $img = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->logo_path;

        $store = Store::where('shop_id', get_session_val('serial_id', 'integer'))->findOrFail($id);

        $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get()->except($id);

        $items = StoreItem::where('store_id', $id)
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->where(function ($q) {
                $ids = Item::where('shop_id', get_session_val('serial_id', 'integer'))->pluck('id');
                $ids = json_decode(json_encode($ids));
                $q->whereIn('item_id', $ids);
            })
//            ->where('store_quant', '>', '0')
            ->select('id', 'item_id', 'store_quant', 'store_id')
            ->get();


        $items->chunk(20)->each(function ($items_ch) {
            $items_ch->each(function ($item) {
                $name = Item::where('shop_id', get_session_val('serial_id', 'integer'))->find($item->item_id);
                $item->item_name = $name->item_name ?? '---';
                $item->price = $name->sale_price ?? '---';
            });
        });


        return view('store::show', compact('items', 'store', 'img', 'stores'));
    }

    /**
     * Show the form for editing the specified resource.
     * @return Response
     */
    public function edit(Request $request)
    {
        $store = Store::find($request->id);
        $settings = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();


        $data = "";
        if ($store) {
            $data .= '

               <input type="hidden" name="_token" value="' . csrf_token() . '">
               <input type="hidden" name="store_id" value="' . $store->id . '">
                <div class="form-group">
                    <label for="name">' . trans('native.store_name') . '</label>
                    <input class="form-control" name="name" id="name" value="' . $store->store_name . '">
                </div>
                  <div class="form-group">
                    <label for="name">' . trans('native.tax_number') . '</label>
                    <input class="form-control" name="tax_number" id="tax_number" value="' . $store->tax_number . '">
                 </div>
                  <div class="form-group">
                    <label for="name">' . trans('native.address') . '</label>
                    <input class="form-control" name="address" id="address" value="' . $store->address . '">
                </div>
                        <div class="form-group">
                            <label> بيانات اعلي الفاتوره :</label>
                            <div class="row">
                                <div class="col-md-10">
                                    <textarea name="bill_header" class="form-control" id="bill_header" cols="30" rows="10">' . $settings->bill_header . '</textarea>
                                </div>
                            </div>
                        </div>
                        <div class="form-group">
                            <label> بيانات اسفل الفاتوره :</label>
                            <div class="row">
                                <div class="col-md-10">
                                    <textarea name="message_bill" class="form-control" id="message_bill" cols="30" rows="10">' . $settings->message_bill . '</textarea>
                                </div>
                            </div>
                        </div>
                ';
            if ($store->img == '' or is_null($store->img)) {
                $data .= '<div class="form-group">
                            <label for="image">' . trans('native.image') . '</label>
                            <input type="file" class="form-control" name="img" id="img">
                        </div>';
            } else {
                $data .= '<div class="row">
                    <div class="col-md-4">
                        <div class="form-group">
                            <label for="image">' . trans('native.image') . '</label>
                            <input type="file" class="form-control" name="img" id="img">
                        </div>
                        </div>
                    <div class="col-md-8">
                        <img height="60" src="' . $store->getImg() . '">
                    </div>
                </div>';

            }

        } else {
            $data .= '<p class="alert alert-danger">' . trans('native.Error_save') . '</p>';
        }


        return response()->json([
            'data' => $data
        ]);
    }


    public function editPage($id)
    {
        $store = Store::find($id);
        return view('store::edit', compact('store'));
    }


    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @return Response
     */
    public function update(Request $request)
    {
        /*
        * needs: path, file_name + extension, year, month
        */

        $request->validate(['name' => 'required']);

        $check = Store::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('id', '!=', $request->store_id)
            ->pluck('store_name');
        $check = json_decode(json_encode($check));

        $store = Store::find($request->store_id);

        if (in_array($request->name, $check)) {
            return back()->with(['error' => trans('store.repeated_name')]);
        } else {

            $store->store_name = $request->name;
            $store->tax_number = $request->tax_number;
            $store->address = $request->address;
            $store->bill_header = $request->bill_header;
            $store->message_bill = $request->message_bill;


            $uploader     =  new StoreImgUploader($request, 'img');
            $result  =  $uploader->checkAndUpload();

            if ($result['status'])
            {
                $uploader->clean($store->img);
                $store->img = $result['path'];
            }

            $store->save();

            Log::create(
                [ 'date_event' => date('Y-m-d H:i:s'),
                    'event' => trans('native.edit_the_store') . $store->store_name,
                    'user_id' => Auth::id(),
                    'shop_id' => get_session_val('serial_id', 'integer')
            ]);

            return redirect()->route('stores.index')->with(['success' => trans('native.correct_edit')]);

        }
    }

    /**
     * Remove the specified resource from storage.
     * @return Response
     */
    public function destroy(Request $request)
    {
        $store = Store::find($request->store_id);

        $check = StoreItem::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_id', $store->id)
            ->count();

        if ($check > 0) {
            return back()->with(['error' => trans('native.store_canot_delete')]);
        } else {
            $store_name = $store->store_name;
            $link = '../' . $store->img;
            File::delete($link);
            $store->delete();

            Log::create(
                [ 'date_event' => date('Y-m-d H:i:s'),
                    'event' => trans('native.delete_the_store') . $store_name,
                    'user_id' => Auth::id(),
                    'shop_id' => get_session_val('serial_id', 'integer')
            ]);

            return back()->with(['success' => trans('native.correct_delete')]);
        }
    }

}

