<?php

namespace Modules\Store\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\systemLog;
use Exception;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Invoices\Entities\InvoiceDetails;
use Modules\Invoices\Entities\Invoices;
use Modules\Item\Entities\BadrShop;
use Modules\Store\Entities\Alert;
use Modules\Store\Entities\ClientTransaction;
use Modules\Store\Entities\Item;
use Modules\Store\Entities\ItemSerial;
use Modules\Store\Entities\SalePoint;
use Modules\Store\Entities\Store;
use Modules\Store\Entities\StoreItem;
use Modules\Store\Entities\StoreOrder;
use Modules\Store\Entities\StoreTransferItem;
use Modules\User\Entities\Log;
use Modules\Item\Entities\ItemUnit;

class StoreTransformController extends Controller
{
    public function CheckSerial(Request $request)
    {
        $check = ItemSerial::where([
            'shop_id' => get_session_val('serial_id', 'integer'),
            'item_id' => $request->item_id,
            'serial' => $request->serial,
            'store_id' => $request->store_id
        ])->first();
        if ($check) {
            if ($check->store_id == $request->store_id) {
                if ($check->sold == 0) {
                    return \response()->json([
                        'status' => '1',
                        'msg' => ''
                    ]);
                }
                return \response()->json([
                    'status' => '1',
                    'msg' => trans('native.serial_sold_cannot_transform')
                ]);
            }
            return \response()->json([
                'status' => '1',
                'msg' => trans('native.serial_not_exist_in_the_selected_store')
            ]);
        } else {
            return \response()->json([
                'status' => '0',
                'msg' => trans('native.serial_not_exist')
            ]);
        }
    }

    public function get_store_name($store_id)
    {
        if ($store_id == '0') {
            $store_name = trans('native.main_store');
        } else {
            $store_name = Store::where('shop_id', get_session_val('serial_id', 'integer'))
                ->find($store_id)->store_name ?? 'NAN';
        }

        return $store_name;
    }

    public function transform(Request $request)
    {
        if (Auth::user()->change_store == 0) {
            $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', auth()->user()->store_id)
                ->get();
        } else {
            $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        }

        if ($request->has('from_store') && !in_array($request->from_store, [null, ''])) {

            $select1 = 'items.quantity as quant_1';
            $select2 = ', items.quantity as quant_2';

            if ($request->from_store > 0) {
                $select1 = 'st1.store_quant as quant_1';
            }

            if ($request->to_store > 0) {
                $select2 = ', st2.store_quant as quant_2';
            }

            $items = Item::with(['mainUnit', 'subUnits.unitName'])
                ->selectRaw('items.id, items.id as item_id, item_name, about, items.unit_id,
                sale_price, pay_price, barcode, require_serial, ' . $select1 . $select2);

            if ($request->from_store > 0) {
                $items = $items->leftJoin('store_items as st1', function ($join) use ($request) {
                    $join->on('items.id', '=', 'st1.item_id')
                        ->where('st1.store_id', '=', $request->from_store);
                })

                    ->leftJoin('stores as s1', function ($join) use ($request) {
                        $join->on('s1.id', '=', 'st1.store_id')
                            ->where('s1.id', '=', $request->from_store);
                    });
            }

            if ($request->to_store > 0) {
                $items = $items->leftJoin('store_items as st2', function ($join) use ($request) {
                    $join->on('items.id', '=', 'st2.item_id')
                        ->where('st2.store_id', '=', $request->to_store);
                })
                    ->leftJoin('stores as s2', function ($join) use ($request) {
                        $join->on('s2.id', '=', 'st2.store_id')
                            ->where('s2.id', '=', $request->to_store);
                    });
            }

            $items = $items->where('items.shop_id', Auth()->user()->shop_id)->groupBy('items.id')->get();
        } else {
            $items = collect();
        }
        $shop = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $show_item_serial = $shop->show_item_serial;

        return view('store::transform', compact('stores', 'request', 'items', 'show_item_serial'));
    }

    public function doTransform(Request $request)
    {
        $shop = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $confirm_store_transfer = $shop->confirm_store_transfer;

        $items = $request->item_qty;
        if ($items && is_array($items) && sizeof($items) > 0) {
            try {
                DB::transaction(function () use ($request, $items, $shop, $confirm_store_transfer) {
                    $current_store_id = $request->from_store;
                    # store where we transform items to
                    $store_id = $request->to_store;

                    $order_l = StoreOrder::where('shop_id', get_session_val('serial_id', 'integer'))
                        ->orderBy('id', 'desc')->first();

                    if ($order_l) {
                        $order_no = $order_l->order_no + 1;
                    } else {
                        $order_no = 1;
                    }

                    $time_now = $request->date;
                    $time = date('H:i:s');
                    $time_now = $time_now . ' ' . $time;
                    $time_now = Carbon::parse($time_now)->format('Y-m-d H:i:s');

                    #create new order
                    $order = new StoreOrder();
                    $order->order_no = $order_no;
                    $order->store_from = $current_store_id;
                    $order->store_to = $store_id;
                    $order->date_order = $time_now;
                    $order->add_user = auth()->user()->id;
                    $order->shop_id = get_session_val('serial_id', 'integer');

                    if ($confirm_store_transfer == 1) {
                        $order->confirm = 1;
                        $order->confirm_by = auth()->user()->id;
                    } else {
                        $order->confirm = 0;
                        $order->confirm_by = 0;
                    }
                    $order->save();

                    session()->put('order_no', $order->order_no);
                    session()->put('order_id', $order->id);

                    if ($confirm_store_transfer == 1) {
                        $this->updateOrder($items, $store_id, $request->date, $order, $shop, $current_store_id, $request->serials);
                    } else {
                        foreach ($items as $item_id => $qty) {
                            [$item_id, $unit_id] = explode('-', $item_id);

                            $s_item = Item::where('shop_id', get_session_val('serial_id', 'integer'))
                                ->where('id', $item_id)
                                ->select('id', 'item_name', 'quantity', 'pay_price', 'sale_price')
                                ->first();

                            $unit_value = 1;
                            if ($s_item->unit_id != $unit_id) {
                                $unit_value = ItemUnit::where([
                                    'item_id' => $item_id,
                                    'unit_id' => $unit_id
                                ])->first()->unit_value ?? 1;
                            }

                            $orderDetails = new StoreTransferItem();
                            $orderDetails->item_id = $item_id;
                            // $orderDetails->quantity = $qty * $unit_value;
                            $orderDetails->quantity = $qty;
                            $orderDetails->order_id = $order->id;
                            $orderDetails->unit_id = $unit_id;
                            $orderDetails->shop_id = get_session_val('serial_id', 'integer');
                            $orderDetails->save();
                        }
                    }
                });
            } catch(Exception $e) {
                return back()->with(['error' => trans('native.Error_save')]);
            }

            $id = session('order_id');
            $no = session('order_no');

            session()->forget('order_no');
            session()->forget('order_id');


            if ($confirm_store_transfer == 1) {
                $msg = trans('native.The transfer was successful and the exchange number is');
            } else {
                $msg = trans('native.The transfer request was successful and the exchange number is');
            }

            return redirect()->route('store.transform-item', $id)->with(['success' => $msg . $no]);
        } else {
            return back()->with(['error' => trans('native.please_select_item')]);
        }
    }

    public function confirmTransformRequest($id)
    {
        $shop = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $order = StoreOrder::where('shop_id', get_session_val('serial_id', 'integer'))->with('user')->findOrFail($id);

        if ($order->confirm == 0) { //غير مؤكده

            DB::transaction(function () use ($shop, $order) {
                $createdAt = Carbon::parse(Carbon::now());
                $date = $createdAt->format('Y-m-d');
                $store_id = $order->store_to;
                $current_store_id = $order->store_from;
                $serials = null;
                $getItems = StoreTransferItem::where('shop_id', get_session_val('serial_id', 'integer'))
                    ->where('order_id', $order->id)
                    ->orderBy('item_id')
                    ->groupBy('item_id')
                    ->get();


                $items = array();
                foreach ($getItems as $item) {
                    $items[$item->item_id . '-' . $item->unit_id] = $item->quantity;
                }
                // start transfer items
                $this->updateOrder($items, $store_id, $date, $order, $shop, $current_store_id, $serials);

                // update store order from UNCONFIRMED to CONFIRMED
                $order->confirm = 1;
                $order->confirm_by = auth()->user()->id;
                $order->date_confirm = Carbon::now();
                $order->save();
            }); // end transaction
            return back()->with(['success' => trans('native.The transfer was successful and the exchange number is') . $order->order_no]);
        } else { // تم تاكيده بالفعل
            return back()->with(['success' => trans('native.The transfer request is already confirmed and the exchange number is') . $order->order_no]);
        }
    }

    public function declineTransformRequest($id)
    {
        $order = StoreOrder::where('shop_id', get_session_val('serial_id', 'integer'))->with('user')->find($id);

        // if there is no orders found / deleted
        if (!$order) {
            return redirect()->route('stores.transform-list');
        }

        DB::transaction(function () use ($order) {
            // delete order items
            StoreTransferItem::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('order_id', $order->id)
                ->delete();

            // delete the order
            $order->delete();
        }); //end transaction

        $msg = trans('native.Transfer request has been declined No.');
        return redirect()->route('stores.transform-list')->with(['success' => $msg . $order->order_no]);
    }

    public function updateOrder($items, $store_id, $date, $order, $shop, $current_store_id, $serials)
    {

        $store_name1 = Store::where('shop_id', get_session_val('serial_id', 'integer'))->find($current_store_id)->store_name ?? '---';

        $store_name = Store::where('shop_id', get_session_val('serial_id', 'integer'))->find($store_id)->store_name ?? '---';

        $this->createAlert(auth()->user()->id, $order->order_no, $store_name1, $store_name, $date);

        $event = 'تحويل اصناف من المخزن ' . $store_name1 . ' الي ' . $store_name;

        Log::create(
            [
                'date_event' =>  Carbon::now(),
                'event' => $event,
                'user_id' => auth()->id(),
              'shop_id' => get_session_val('serial_id', 'integer'),
            ]
        );

        $total_amount = 0;
        foreach ($items as $item_id => $qty) {
            [$item_id, $unit_id] = explode('-', $item_id);
            $s_item = Item::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $item_id)
                ->select('id', 'item_name', 'quantity', 'pay_price', 'sale_price')
                ->first();

            $unit_value = 1;
            if ($s_item->unit_id != $unit_id) {
                $unit_value = ItemUnit::where([
                    'item_id' => $item_id,
                    'unit_id' => $unit_id
                ])->first()->unit_value ?? 1;
            }

            //كميه الاصناف اكبر من المراد نقلها
            if ($s_item && $qty > 0) {
                $current_store_item = StoreItem::where('shop_id', get_session_val('serial_id', 'integer'))
                    ->where('item_id', $item_id)
                    ->where('store_id', $current_store_id)
                    ->first();
                $old_qty_currnt = $current_store_item->store_quant ?? 0;
                $new_store = Store::where('shop_id', get_session_val('serial_id', 'integer'))->find($store_id);

                if ($new_store && $current_store_item) {
                    $new_store_item = StoreItem::where('store_id', $store_id)
                        ->where('shop_id', get_session_val('serial_id', 'integer'))
                        ->where('item_id', $s_item->id)
                        ->select('id', 'item_id', 'store_quant')
                        ->first();

                    if ($new_store_item) {
                        $old_qty = $new_store_item->store_quant;
                        $new_store_item->increment('store_quant', $qty * $unit_value);
                        $new_store_item->save();
                    } else {
                        $old_qty = 0;
                        $new_store_item = new StoreItem();
                        $new_store_item->shop_id = get_session_val('serial_id', 'integer');
                        $new_store_item->item_id = $s_item->id;
                        $new_store_item->store_id = $store_id;
                        $new_store_item->store_quant = $qty * $unit_value;
                        $new_store_item->add_user = Auth::id();
                        $new_store_item->save();
                    }

                    $current_store_item->decrement('store_quant', $qty * $unit_value);
                    $current_store_item->save();

                    $this->itemTransaction(
                        $store_id,
                        $current_store_id,
                        $date,
                        $item_id,
                        $s_item->item_name,
                        $current_store_item->store_quant,
                        $old_qty_currnt,
                        $qty,
                        $order->order_no,
                        $unit_id,
                        $current_store_id
                    );

                    # transform from store_id to main store  =>  qty from current
                    $this->itemTransaction(
                        $store_id,
                        $current_store_id,
                        $date,
                        $item_id,
                        $s_item->item_name,
                        $new_store_item->store_quant,
                        $old_qty,
                        $qty,
                        $order->order_no,
                        $unit_id,
                        $store_id
                    );
                }

                $chec = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->trans_price;
                if ($chec == '0') {
                    $amount = $s_item->pay_price * $qty;
                } else {
                    $amount = $s_item->sale_price * $qty;
                }
                $total_amount += $amount;
            }
        }
        $show_item_serial = $shop->show_item_serial;
        if ($show_item_serial == '1' && is_array($serials) && sizeof($serials) > 0) {
            foreach ($serials as $item_id => $serial) {
                $s = explode(',', $serial);
                foreach ($s as $serial_number) {
                    $find = ItemSerial::where([
                        'shop_id' => get_session_val('serial_id', 'integer'),
                        'item_id' => $item_id,
                        'serial' => $serial_number,
                        'store_id' => $current_store_id
                    ])->update([
                        'store_id' => $store_id
                    ]);
                }
            }

        }
    }

    public function transformAll(Request $request)
    {
        try {
            DB::transaction(function () use ($request) {

                $current_store = Store::where('shop_id', get_session_val('serial_id', 'integer'))->findOrFail($request->current);

                #current store items
                $s_items = StoreItem::where('store_id', $current_store->id)
                    ->where('shop_id', get_session_val('serial_id', 'integer'))
                    ->where(function ($q) {
                        $ids = Item::where('shop_id', get_session_val('serial_id', 'integer'))->pluck('id');
                        $ids = json_decode(json_encode($ids));
                        $q->whereIn('item_id', $ids);
                    })
                    ->select('id', 'item_id', 'store_quant')
                    ->get();

                # store where we transform items to
                $store_id = $request->store_id;

                $order_l = StoreOrder::where('shop_id', get_session_val('serial_id', 'integer'))
                    ->orderBy('id', 'desc')->first();

                if ($order_l) {
                    $order_no = $order_l->order_no + 1;
                } else {
                    $order_no = 1;
                }
                $time_now = $request->date;
                $time = date('H:i:s');
                $time_now = $time_now . ' ' . $time;
                $time_now = Carbon::parse($time_now)->format('Y-m-d H:i:s');

                #create new order
                $order = new StoreOrder();
                $order->order_no = $order_no;
                $order->store_from = $current_store->id;
                $order->store_to = $store_id;
                $order->date_order = $time_now;
                $order->add_user = auth()->user()->id;
                $order->shop_id = get_session_val('serial_id', 'integer');
                $order->confirm = 1;
                $order->confirm_by = auth()->user()->id;
                $order->save();

                session()->put('order_no', $order->order_no);
                session()->put('order_id', $order->id);

                $store_name1 = $current_store->store_name;

                $store_name = Store::where('shop_id', get_session_val('serial_id', 'integer'))->find($store_id)->store_name ?? '---';

                $from_users = DB::table('users')
                    ->where('shop_id', get_session_val('serial_id', 'integer'))
                    ->where('store_id', $current_store->id)
                    ->get();
                $to_users = DB::table('users')
                    ->where('shop_id', get_session_val('serial_id', 'integer'))
                    ->where('store_id', $store_id)
                    ->get();

                $this->createAlert(auth()->user()->id, $order_no, $store_name1, $store_name, $request->date);

                $total_amount = 0;

                #loop through current store items to start transform.
                $s_items->chunk(20)->each(function ($items_ch) use ($store_id, $request, $current_store, $order_no, $from_users, $to_users, $order, &$total_amount) {
                    $items_ch->each(function ($item) use ($store_id, $request, $current_store, $order_no, $from_users, $to_users, $order, &$total_amount) {
                        $old_qty_currnt = $item->store_quant;

                        $my_item = Item::where('shop_id', get_session_val('serial_id', 'integer'))->find($item->item_id);


                        if ($my_item && $item->store_quant > 0) {

                            $trans_qty = $item->store_quant;

                            $new_store = Store::where('shop_id', get_session_val('serial_id', 'integer'))->find($store_id);

                            if ($new_store) {
                                $new_store_item = StoreItem::where('store_id', $store_id)
                                    ->where('shop_id', get_session_val('serial_id', 'integer'))
                                    ->where('item_id', $item->item_id)
                                    ->select('id', 'item_id', 'store_quant')
                                    ->first();

                                if ($new_store_item) {
                                    $old_qty = $new_store_item->store_quant;
                                    $new_store_item->increment('store_quant', $item->store_quant);
                                    $new_store_item->save();
                                } else {
                                    $old_qty = 0;
                                    $new_store_item = new StoreItem();
                                    $new_store_item->shop_id = get_session_val('serial_id', 'integer');
                                    $new_store_item->item_id = $item->item_id;
                                    $new_store_item->store_id = $store_id;
                                    $new_store_item->store_quant = $item->store_quant;
                                    $new_store_item->save();
                                }

                                $item->decrement('store_quant', $item->store_quant);
                                $item->save();

                                # transform from current_store to store_id  =>  qty from current
                                $this->itemTransaction(
                                    $store_id,
                                    $request->current,
                                    $request->date,
                                    $item->item_id,
                                    $my_item->item_name,
                                    $item->store_quant,
                                    $old_qty_currnt,
                                    $old_qty_currnt,
                                    $order_no,
                                    $my_item->unit_id,
                                    $request->current
                                );
                                # transform from store_id to current_store  =>  qty from store_id
                                $this->itemTransaction(
                                    $store_id,
                                    $request->current,
                                    $request->date,
                                    $item->item_id,
                                    $my_item->item_name,
                                    $new_store_item->store_quant,
                                    $old_qty,
                                    $old_qty_currnt,
                                    $order_no,
                                    $my_item->unit_id,
                                    $store_id
                                );
                            }

                            $chec = DB::table('badr_shop')
                                ->where('serial_id', get_session_val('serial_id', 'integer'))
                                ->first()->trans_price;
                            if ($chec == '0') {
                                $amount = $my_item->pay_price * $trans_qty;
                            } else {
                                $amount = $my_item->sale_price * $trans_qty;
                            }
                            $total_amount += $amount;
                        }
                    });
                });
                $event = 'تحويل كل الاصناف من المخزن ' . $store_name1 . ' الي ' . $store_name;

                Log::create(
                    [
                        'date_event' =>  Carbon::now(),
                        'event' => $event,
                        'user_id' => auth()->id(),
                      'shop_id' => get_session_val('serial_id', 'integer'),
                    ]
                );
                // $this->clientTransaction($request->date, $total_amount, $order->id);
            });

            $id = session('order_id');
            $no = session('order_no');
            session()->forget('order_no');
            session()->forget('order_id');
            return redirect()->route('store.transform-item', $id)->with(['success' => 'تم التحويل بنجاح ورقم الصرف هو: ' . $no]);
        } catch (\Exception $e) {
            return back()->with(['error' => trans('native.An_error_occurred_during_the_conversion')]);
        }
    }

    public function transformList(Request $request)
    {
        $records = StoreOrder::where('shop_id', get_session_val('serial_id', 'integer'));

        if ($request->has('order_no') && !in_array($request->order_no, ['', null])) {
            $records = $records->where('order_no', $request->order_no);
        }
        if ($request->has('store_id') && !in_array($request->store_id, ['', 'x', null])) {
            $records = $records->where(function ($q) use ($request) {
                $q->where('store_from', $request->store_id);
                $q->orWhere('store_to', $request->store_id);
            });
        }

        // filter confirmed or not confirmed
        if ($request->has('store_confirm') && !in_array($request->store_confirm, ['', 'x', null])) {
            $records = $records->where(function ($q) use ($request) {
                $q->where('confirm', $request->store_confirm);
            });
        }

        if ($request->has('from_date') && !in_array($request->from_date, ['', null])) {
            $records = $records->whereDate('date_order', '>=', $request->from_date);
        }
        if ($request->has('to_date') && !in_array($request->to_date, ['', null])) {
            $records = $records->whereDate('date_order', '<=', $request->to_date);
        }
        $records = $records
            ->orderBy('order_no', 'DESC')
            ->paginate(30);

        $records->each(function ($record) {
            $record->store_from = $this->get_store_name($record->store_from);
            $record->store_to = $this->get_store_name($record->store_to);
        });

        if (Auth::user()->change_store == 0) {
            $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', auth()->user()->store_id)
                ->get();
        } else {
            $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        }
        return view('store::transform-list', compact('records', 'stores'));
    }

    public function transformListItem($id)
    {
        $img = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->logo_path;

        $order = StoreOrder::where('shop_id', get_session_val('serial_id', 'integer'))->with('user')->findOrFail($id);
        $order->store_from_name = $this->get_store_name($order->store_from);
        $order->store_to_name = $this->get_store_name($order->store_to);

        if ($order->confirm == 1) {
            $records = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                ->leftJoin('units', 'invoices_details.unit_id', 'units.id')
                ->where('invoices_.shop_id', get_session_val('serial_id', 'integer'))
                ->where('invoices_.order_no', $order->order_no)
                ->where('invoices_.invoice_type', 10)
                ->where('invoices_.store_from', $order->store_from)
                ->orderBy('item_id')
                ->groupBy('item_id')
                ->select('invoices_details.*', 'units.name as unit')
                ->get();

            $records->each(function ($record) {
                $item = Item::selectRaw('pay_price, barcode, item_name')->where('id', $record->item_id)
                    ->where('shop_id', get_session_val('serial_id', 'integer'))->first();

                if ($item) {

                    $unit_value = 1;

                    if ($record->unit_id != $item->unit_id) {
                        $unit_value = ItemUnit::where([
                            'item_id' => $record->item_id,
                            'unit_id' => $record->unit_id
                        ])->first()->unit_value ?? 1;
                    }

                    if ($record->pay_price == 0) {
                        $record->pay_price = $item->pay_price * $unit_value;
                    }

                    $record->barcode = $item->barcode;
                    $record->item_name = $item->item_name;
                }
            });
        } else {
            $records = StoreTransferItem::where('store_transfer_items.shop_id', get_session_val('serial_id', 'integer'))
                            ->leftJoin('units', 'store_transfer_items.unit_id', '=', 'units.id')
                            ->where('store_transfer_items.order_id', $order->id)
                            ->select('units.name as unit', 'store_transfer_items.*')
                            ->orderBy('store_transfer_items.item_id')
                            ->groupBy('store_transfer_items.item_id')
                            ->get();

            $records->each(function ($record) {
                $item = Item::selectRaw('pay_price, barcode, item_name')->where('id', $record->item_id)
                            ->where('shop_id', get_session_val('serial_id', 'integer'))->first();
                if ($item) {

                    $unit_value = 1;
                    if ($record->unit_id != $item->unit_id) {
                        $unit_value = ItemUnit::where([
                            'item_id' => $record->item_id,
                            'unit_id' => $record->unit_id
                        ])->first()->unit_value ?? 1;
                    }

                    if ($record->pay_price == 0) {
                        $record->pay_price = $item->pay_price * $unit_value;
                    }
                    $record->barcode = $item->barcode;

                    $record->item_name = $item->item_name;
                }
            });
        }

        return view('store::transform-item', compact('order', 'records', 'img'));
    }

    public function item_transform_report(Request $request)
    {
        # 0 => pay_price, 1 => sale_price
        $chec = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->trans_price;
        $img = DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->logo_path;
        $stores = Store::where('shop_id', get_session_val('serial_id', 'integer'))->get();
        if ($request->all() != []) {
            $data = ClientTransaction::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where(function ($q) {
                    $q->where('effect', '3');
                    $q->orWhere('type', 11);
                });
            if ($request->has('from_date') && !in_array($request->from_date, ['', null])) {
                $data = $data->whereDate('pay_day', '>=', $request->from_date);
            }
            if ($request->has('to_date') && !in_array($request->to_date, ['', null])) {
                $data = $data->whereDate('pay_day', '<=', $request->to_date);
            }

            $data = $data->get();

            $store_id = $request->store_id;

            $data->chunk(20)->each(function ($data_chunked) use ($store_id) {
                $data_chunked->each(function ($item) use ($store_id) {
                    if ($item->type == 11) {
                        $sale_point_id = $item->safe_point_id;
                        $sale_point = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                            ->find($sale_point_id);
                        $point_store_id = $sale_point->store_id ?? 0;

                        $item->store_from = $this->get_store_name($point_store_id);
                    } else {
                        $order_id = $item->store_order_id;
                        $order = StoreOrder::where('shop_id', get_session_val('serial_id', 'integer'))
                            ->find($order_id);

                        $store_name_from = $this->get_store_name($order->store_from);
                        $store_name_to = $this->get_store_name($order->store_to);
                        if ($order->store_from == $store_id) {
                            $store_name = $store_name_from;
                        } else {
                            $store_name = $store_name_to;
                        }
                        $item->store_name = $store_name;
                        $item->from_store = $order->store_from;
                        $item->to_store = $order->store_to;
                        $item->order_no = $order->order_no;
                    }
                });
            });

            if ($request->has('store_id') && !in_array($request->store_id, ['', null, 'x'])) {
                $results = collect();
                $data->chunk(20)->each(function ($data_chunked) use ($store_id, $results) {
                    $data_chunked->each(function ($item) use ($store_id, $results) {
                        if ($item->type == 11) {
                            $sale_point_id = $item->safe_point_id;
                            $sale_point = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                                ->find($sale_point_id);
                            $point_store_id = $sale_point->store_id ?? 0;

                            if ($store_id == $point_store_id) {
                                $results->push($item);
                            }
                        } else {
                            $order_id = $item->store_order_id;
                            $order = StoreOrder::where('shop_id', get_session_val('serial_id', 'integer'))
                                ->find($order_id);

                            if ($order->store_from == $store_id || $order->store_to == $store_id) {
                                $results->push($item);
                            }
                        }
                    });
                });
                if ($request->store_id == '0') {
                    $store_name = 'المخزن الرئيسي';
                } else {
                    $store_name = Store::where('shop_id', get_session_val('serial_id', 'integer'))
                        ->find($request->store_id)->store_name ?? 'NAN';
                }
            } else {
                $store_name = trans('native.all_branches');
                $results = $data;
            }
        } else {
            $results = collect();
            $store_name = '';
        }

        return view('store::item_transform_report', compact('results', 'store_name', 'stores', 'img'));
    }

    protected function itemTransaction($store_id, $current_store, $date, $item_id, $item_name, $new_qty, $old_qty, $qty, $order_no, $unit_id, $effected_store_id)
    {
        $item = Item::where('shop_id', get_session_val('serial_id', 'integer'))->find($item_id);

        $invoice = Invoices::create([
            'invoice_date' => date('Y-m-d h:i:s'),
            'transaction_date' => date('Y-m-d h:i:s'),
            'add_user' => auth()->user()->id,
            'client_id' => 0,
            'store_id' => $effected_store_id,
            'notes' => '',
            'shop_id' => get_session_val('serial_id', 'Integer'),
            'invoice_type' => 10,
            'invoice_total_disc' => 0,
            'invoice_total_disc_type' => 1,
            'invoice_total_disc_status' => 1,
            'store_from' => $current_store,
            'store_to' => $store_id,
            'order_no' => $order_no,
        ]);

        InvoiceDetails::create([
            'invoice_id' => $invoice->id,
            'shop_id' => get_session_val('serial_id', 'Integer'),
            'item_id' => $item->id,
            'quantity' => $qty,
            'quantity_after' => $new_qty,
            'unit_id' => $unit_id,
            'store_id' => $effected_store_id,
        ]);
    }

    protected function clientTransaction($date, $amount, $order_no)
    {
        $datetime = Carbon::parse($date)->format('Y-m-d H:i:s');
        $date = Carbon::parse($date)->format('Y-m-d');

        $clientTransaction = new ClientTransaction();
        $clientTransaction->shop_id = get_session_val('serial_id', 'integer');
        $clientTransaction->date_time = $datetime;
        $clientTransaction->client_id = 0;
        $clientTransaction->amount = $amount;
        $clientTransaction->type = 3;
        $clientTransaction->effect = 3; # 3 => no effect for safe, just store
        $clientTransaction->pay_day = $date;
        $clientTransaction->balance = 0; # new
        $clientTransaction->bill_id = 0;
        $clientTransaction->bill_net_total = 0;
        $clientTransaction->safe_type = 1;
        $clientTransaction->safe_balance = 0;
        $clientTransaction->safe_point_id = 0;

        $clientTransaction->user_id = auth()->id();

        $clientTransaction->sale_back_id = 0;
        $clientTransaction->problem_id = 0;
        $clientTransaction->supplier_id = 0;
        $clientTransaction->salary_month = 0;

        $clientTransaction->OutTransactionID = 0;
        $clientTransaction->spend_id = 0;

        $clientTransaction->store_order_id = $order_no;

        $clientTransaction->save();
    }

    public function createAlert($user_id, $order_no, $store_name, $store_name2, $date)
    {
        $alert = new Alert();
        $alert->add_user = $user_id;
        $alert->shop_id = get_session_val('serial_id', 'integer');
        $alert->alert_name = trans('native.exchange_of_stores');
        $para1 = 'تم تحويل اصناف من المخزن: ' . $store_name . ' الي المخزن: ' . $store_name2 . ' برقم الصرف: ' . $order_no;
        $alert->para = $para1;
        $alert->date = $date;
        $alert->save();
    }

    public function getItemData(Request $request)
    {
        $barcode = $request->barcode;
        $from_store = $request->from_store;
        $to_store = $request->to_store;

        $store_item_qtys = Item::leftJoin('items_unit', 'items_unit.item_id', 'items.id')
            ->leftJoin('store_items AS from_store', function ($q) use ($from_store) {
                $q->on('from_store.item_id', '=', 'items.id');
                $q->where('from_store.store_id', $from_store);
            })
            ->leftJoin('store_items AS to_store', function ($q) use ($to_store) {
                $q->on('to_store.item_id', '=', 'items.id');
                $q->where('to_store.store_id', $to_store);
            })
            ->selectRaw('items.id, items.item_name, items.sale_price, items.pay_price, items.about, items.barcode, items.require_serial AS serial,
            case
                when items_unit.item_id > 0 and items_unit.unit_type = 0 and items_unit.unit_barcode = ' . $barcode . ' then items_unit.unit_id
            else items.unit_id
            end AS unit_id,
            from_store.store_quant AS qty_from, to_store.store_quant AS qty_to
        ')
            ->where('items.shop_id', get_session_val('serial_id', 'integer'))
            ->where(function ($q) use ($barcode) {
                $q->where('items.barcode', $barcode);
                $q->orWhere('items_unit.unit_barcode', $barcode);
            })
            ->first();

        return $this->return_data($store_item_qtys);
    }


    public function return_data($item)
    {
        if (is_null($item)) {
            # no item at all
            $response = response()->json([
                'status' => false,
                'msg' => trans('native.item_recognition_error')
            ]);
        } else {
            $item->sale_price = price_decimal($item->sale_price);
            $item->pay_price = price_decimal($item->pay_price);
            $item->qty_from = quant_decimal($item->qty_from);
            $item->qty_to = quant_decimal($item->qty_to);
            #main barcode unit found
            $response = response()->json([
                'status' => true,
                'data' => $item
            ]);
        }
        return $response;
    }


    public function editRecord($id, Request $request)
    {
        $order = StoreOrder::where('shop_id', get_session_val('serial_id', 'integer'))->with('user')->findOrFail($id);
        $order->store_from_name = $this->get_store_name($order->store_from);
        $order->store_to_name = $this->get_store_name($order->store_to);
        $img = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first()->logo_path;
        $shop = \DB::table('badr_shop')->where('serial_id', get_session_val('serial_id', 'integer'))->first();
        $show_item_serial = $shop->show_item_serial;
        // items
        $from_store = $order->store_from;
        $to_store = $order->store_to;

        $select1 = 'items.quantity as quant_1';
        $select2 = ', items.quantity as quant_2';

        if ($from_store > 0) {
            $select1 = 'st1.store_quant as quant_1';
        }

        if ($to_store > 0) {
            $select2 = ', st2.store_quant as quant_2';
        }

        $items = Item::with(['mainUnit', 'subUnits.unitName'])
            ->selectRaw('items.id, items.id as item_id, item_name, about, items.unit_id, sale_price, pay_price, barcode, require_serial, ' . $select1 . $select2);


        if ($from_store > 0) {
            $items = $items->leftJoin('store_items as st1', function ($join) use ($from_store) {
                $join->on('items.id', '=', 'st1.item_id')
                    ->where('st1.store_id', '=', $from_store);
            })

                ->leftJoin('stores as s1', function ($join) use ($from_store) {
                    $join->on('s1.id', '=', 'st1.store_id')
                        ->where('s1.id', '=', $from_store);
                });
        }

        if ($to_store > 0) {
            $items = $items->leftJoin('store_items as st2', function ($join) use ($to_store) {
                $join->on('items.id', '=', 'st2.item_id')
                    ->where('st2.store_id', '=', $to_store);
            })
                ->leftJoin('stores as s2', function ($join) use ($to_store) {
                    $join->on('s2.id', '=', 'st2.store_id')
                        ->where('s2.id', '=', $to_store);
                });
        }

        $items = $items->where('items.shop_id', \auth()->user()->shop_id)->groupBy('items.id')->get();
        //--------------------

        $records = StoreTransferItem::where('store_transfer_items.shop_id', get_session_val('serial_id', 'integer'))
            ->leftJoin('units', 'store_transfer_items.unit_id', '=', 'units.id')
            ->leftJoin('items', 'store_transfer_items.item_id', '=', 'items.id')
            ->leftJoin('items_unit', function($query) {
                $query->on('items_unit.item_id', '=', 'store_transfer_items.item_id')
                        ->on('items_unit.unit_id', '=', 'store_transfer_items.unit_id');
            })
            ->leftJoin('store_items', function ($q) use ($order) {
                $q->on('store_items.item_id', 'items.id')
                    ->where('store_items.store_id', $order->store_from);
            })
            ->where('order_id', $id)
            ->orderBy('item_id')
            ->groupBy('item_id')
            ->select(
                'store_transfer_items.*',
                'units.name',
                'items.item_name',
                'items.pay_price',
                'items.pay_price',
                'items.pay_price',
                DB::raw('if(items_unit.unit_value > 0, items_unit.unit_value, 1) as unit_quantity'),
                DB::raw('if(items_unit.unit_value > 0, store_transfer_items.quantity / items_unit.unit_value, store_transfer_items.quantity) as order_quantity'),
                DB::raw('if(store_items.id > 0, store_items.store_quant, items.quantity) as item_quantity')
            )
            ->get();
        // return $records;
        return view('store::edit_transform_records', compact('order', 'img', 'show_item_serial', 'items', 'records'));
    }
    public function updateRecord(Request $request)
    {
         $order = StoreOrder::where('shop_id', get_session_val('serial_id', 'integer'))->with('user')->findOrFail($request->order_id);
         try {
             $row = DB::transaction(function () use ($request, $order) {
                 if ($request->quantity) {
                     foreach ($request->quantity as $id => $qty) {
                         $orderDetails = StoreTransferItem::where(['id' => $id, 'shop_id' => get_session_val('serial_id', 'integer')]);
                         $orderDetails->update([
                             'quantity' => $qty
                         ]);
                     }
                 }

                 if ($request->item_qty) {
                     foreach ($request->item_qty as $item_id => $qty) {
                         [$item_id, $unit_id] = explode('-', $item_id);

                         $s_item = Item::where('shop_id', get_session_val('serial_id', 'integer'))
                             ->where('id', $item_id)
                             ->select('id', 'item_name', 'quantity', 'pay_price', 'sale_price')
                             ->first();

                         $unit_value = 1;
                         if ($s_item->unit_id != $unit_id) {
                             $unit_value = ItemUnit::where([
                                 'item_id' => $item_id,
                                 'unit_id' => $unit_id
                             ])->first()->unit_value ?? 1;
                         }

                         $orderDetails = new StoreTransferItem();
                         $orderDetails->item_id = $item_id;
                         $orderDetails->quantity = $qty;
                         $orderDetails->order_id = $order->id;
                         $orderDetails->unit_id = $unit_id;
                         $orderDetails->shop_id = get_session_val('serial_id', 'integer');
                         $orderDetails->save();
                     }
                 }
                 return true;
             });
         } catch(Exception $e) {
             dd($e);
         }
        $msg = trans('native.The transfer was successful updated');
        return redirect()->route('store.transform-item', $request->order_id)->with(['success' => $msg]);
    }

}
