<?php

namespace Modules\SupplierInstallments\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Modules\SupplierInstallments\Entities\Client;
use Modules\SupplierInstallments\Entities\Helper;
use Modules\SupplierInstallments\Entities\Instalment;

class SupplierInstallmentReportController extends Controller
{


    public function suppliersReport()
    {
        $suppliers = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereIn('client_role', [1, 2])
            ->select('id', 'client_name', 'balance')->get();

        return view('supplierinstallments::suppliers-report', compact('suppliers'));
    }

    public function suppliersReportResult(Request $request)
    {

        $this->validate($request, [
            'supplier_id' => 'required',
            'period' => 'required',
        ]);

        $today = $request->today;

        $today_date = Carbon::parse($today)->format('Y-m-d');

        $p = $request->period;
        $period = '--';
        if ($p == '1'){
            $period = 'شهر';
        }elseif ($p == '3'){
            $period = '3 شهور';
        }elseif ($p == '6'){
            $period = '6 شهور';
        }elseif ($p == '12'){
            $period = 'سنة';
        }

        $supplier_id = $request->supplier_id;

        if ($supplier_id == '0'){
            $suppliers = Client::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('balance', '!=', 0)
                ->whereIn('client_role', [1, 2])
                ->pluck('id');
            $supplier_name = 'الكل';
        }else {
            $suppliers = Client::where('shop_id', get_session_val('serial_id', 'integer'))
                ->whereIn('client_role', [1, 2])
                ->where('id', $request->supplier_id)
                ->pluck('id');
            $supplier_name = Client::find($request->supplier_id)->client_name;
        }

        $needed_suppliers = Client::whereIn('id', $suppliers)
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->select('id', 'client_name', 'balance')
            ->get();

        foreach ($needed_suppliers as $needed_supplier){
            $delaies = Instalment::where('supplier_id', $needed_supplier->id)
                ->where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('installment_pay', 0)
                ->where('type', 1)
                ->whereDate('installment_date', '<', $today_date)
                ->sum('installment_amount');
            $needed_supplier->delaies = $delaies;

            $needed_supplier->data = Helper::getSupplierColumnsData($p, $today, $needed_supplier->id);

        }

        $columns = Helper::getColumns($p, $today);
        $subColumns = Helper::getSubColumns($p);
//        dd($columns);
        return view('supplierinstallments::suppliers-report-result', compact( 'subColumns', 'supplier_name', 'columns', 'needed_suppliers',  'period', 'p'));
    }


}
