<?php

namespace Modules\SupplierInstallments\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Contracts\View\View;
use Illuminate\Http\Response;
use Illuminate\Support\Carbon;
use Modules\SupplierInstallments\Entities\ClientTransaction;
use Modules\SupplierInstallments\Entities\Instalment;
use Modules\SupplierInstallments\Entities\SalePoint;
use Modules\SupplierInstallments\Entities\Supplier;
use Auth;
use Modules\Invoices\Entities\Invoices;
use Modules\SupplierInstallments\Entities\Client;

class SupplierInstallmentsController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index(Request $request)
    {
        $suppliers = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereIn('client_role', [1, 2])
            ->get();

        $instalments = Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('type', 1)
            ->when($request->supplier > 0, function($q) use($request){
                $q->where('supplier_id', $request->supplier);
            })
            ->when($request->bill > 0, function($q) use($request){
                $q->where('bill_id', $request->bill);
            })
            ->orderBy(\DB::raw('installment_date'))
            ->get();

        return view('supplierinstallments::index', compact('suppliers', 'instalments'));
    }

    /**
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function delete($id)
    {
        $instalment = Instalment::where('id', $id)->where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('type', 1)
            ->first();
        if ($instalment) {
            $instalment->delete();
            session()->flash('success', 'تمت عملية حذف القسط بنجاح');
            return back();
        }
        return back();
    }

    /**
     * This method return the instalment that requested by the user
     * @param Request $request
     * @return null|string|void
     */
    public function getByTypeAjax(Request $request)
    {
        $date = $request->today;
        $date = Carbon::parse($date)->format('Y-m-d');
        $type = $request->type;
        $supplier = $request->supplier;
        $instalments = null;
        $instalments = $this->getInstalmentsBasedOnType($type, $supplier, $date);
        $instalments = $this->formatInstalments($instalments);
        return $instalments;
    }

    /**
     * This method return the outstanding instalments for both all suppliers
     * or a certain one based on the $supplier parameter value
     * if the $supplier value equal zero then get all records otherwise get the target one
     * @param $supplier
     * @return mixed
     */
    private function getLateInstalment($supplier, $date) {
        return Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where(function($q) use($supplier){
                if ($supplier != 0) {
                    $q->where('supplier_id', $supplier);
                }
            })
            ->where('type', 1)
            ->where('installment_pay', 0)
            ->whereDate('installment_date', '<', $date)
            ->orderBy(\DB::raw('installment_date'))
            ->get();
    }

    /**
     * This method return the paid instalment for both all suppliers
     * or a certain one based on the $supplier parameter value
     * if the $supplier value equal zero then get all records otherwise get the target one
     * @param $supplier
     * @return mixed
     */
    private function getPaidInstalment($supplier)
    {
        return Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where(function($q) use($supplier){
                if ($supplier != 0) {
                    $q->where('supplier_id', $supplier);
                }
            })
            ->where('type', 1)
            ->where('installment_pay', 1)
            ->orderBy(\DB::raw('installment_date'))
            ->get();
    }
    /**
     * This method return the all instalments for both all suppliers
     * or a certain one based on the $supplier parameter value
     * if the $supplier value equal zero then get all records otherwise get the target one
     * @param $supplier
     * @return mixed
     */
    private function getAllInstallment($supplier)
    {
        return Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where(function($q) use($supplier){
                if ($supplier != 0) {
                    $q->where('supplier_id', $supplier);
                }
            })
            ->where('type', 1)
            ->orderBy(\DB::raw('installment_date'))
            ->get();
    }
    /**
     * This method will decide the type of the instalments whether it (paid, outstanding, all)
     * in addition determine does the request for all suppliers or a specific one
     * then return the result
     * @param $supplier
     * @param $type
     * @return mixed
     */
    private function getInstalmentsBasedOnType($type, $supplier, $date)
    {
        $instalments = null;
        switch ($type) {
            case "late":
               $instalments = $this->getLateInstalment($supplier, $date);
                break;
            case "paid":
                $instalments = $this->getPaidInstalment($supplier);
                break;
            case "all" :
                $instalments = $this->getAllInstallment($supplier);
                break;
            default:
                return;
        }
        return $instalments;
    }

    /**
     * This method receive a collection of instalments then looping through each of them-
     * and style it in html table before returning the result
     * @param $instalments
     * @return string
     */
    private function formatInstalments($instalments)
    {
        $output = '';
        if (sizeof($instalments)) {
            $loop =0;
            foreach($instalments as $instalment) {
                $loop++;
                $class = $instalment->installment_pay ==1 ? 'class="bg-success text-white"' : '';

                $output .= '  <tr '.$class.'>
                            <td>'.$loop.'</td>
                            <td>'.$instalment->optionalSupplier()->client_name.'</td>

                            <td>'.$instalment->installment_date.'</td>
                            <td>'.$instalment->pay_date.'</td>
                            <td>'.$instalment->installment_amount.'</td>
                            <td>'.$instalment->notes.'</td>';

                             if ($instalment->installment_pay == 1) {

                    $output .= '<td></td></tr>';
                } else {

                    // $output .= '    <td><a href="' . route('supplierinstallment.payingTheDebt', $instalment->id) . '" class="btn btn-circle btn-success"><i class="fa fa-money-bill-alt"></i></a></td>
                    //         <td><a href="' . route('supplierinstallment.delete', $instalment->id) . '" class="btn btn-circle btn-danger"><i class="fa fa-minus-circle"></i></a></td>
                    //     </tr>';

                    $output .= '    <td><a href="' . route('supplierinstallment.payingTheDebt', $instalment->id) . '" class="btn btn-circle btn-success"><i class="fa fa-money-bill-alt"></i></a></td>
                    </tr>';
                }

            }
        } else {
            $output = 'لا يوجد بيانات';
        }
        return $output;
    }



    public function getSupplierBalanceAjax(Request $request)
    {
        $supplierObject = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereIn('client_role', [1, 2])
            ->where('id', $request->supplier)
            ->first();
        $scheduled = Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('type', 1)
            ->where('installment_pay',0)
            ->where('supplier_id', $supplierObject->id)
            ->sum('installment_amount');
        if ($supplierObject) {
            return $this->formatSupplierBalance($supplierObject, $scheduled);
        }
        return 'لا يوجد بيانات';
    }

    /**
     * Formatting the supplier balance;
     * @param $supplierObject
     * @param $scheduled
     * @return string
     */
    private function formatSupplierBalance($supplierObject, $scheduled)
    {
        $unScheduled = abs($supplierObject->balance) - $scheduled;

        $output = "<tr>";
        $output.= "<td>".$supplierObject->client_name."</td>";
        $output.= "<td>".$supplierObject->tel or $supplierObject->mobile."</td>";
        $output.= "<td>".round_money($supplierObject->balance)."</td>";
        $output.= "<td>".round_money($scheduled)."</td>";
        $output.= "<td>".round_money($unScheduled)."</td>";
        $output.= "<td><a class='btn btn-success' href='".route('supplierinstallment.rescheduleDebt', $supplierObject->id)."'>جدولة</a></td>";
        return $output;
    }

    /**
     * Get supplier by id and return the rescheduling page.
     * @param $id
     * @return View
     */
    public function rescheduleDebt($id)
    {
        $price_decimal = get_session_val('decimal_num_price', 'integer');

        $supplier = Client::where('id', $id)
            ->whereIn('client_role', [1, 2])
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->first();

        $scheduled = Instalment::where('supplier_id', $id)
            ->where('installment_pay', 0)
            ->where('type', 1)
            ->where('shop_id', get_session_val('serial_id', 'integer'))
            ->sum('installment_amount');

        if ($scheduled >= abs($supplier->balance)) {
            session()->flash('error', ' المديونيات الخاصة بالمورد '.$supplier->client_name.' تم جدولتها مسبقاً ');
            return back();
        }
        $balance = $supplier->balance + $scheduled;

        return view('supplierinstallments::reschedule', compact('supplier', 'price_decimal', 'balance','scheduled'));
    }

    /**
     * Insert the instalments dates and amount
     */
    public function storeRescheduleDebt(Request $request)
    {
        $request->validate([
            'supplier_id' => 'required',
            'installment_amount' => 'required',
            'installment_date.*' => 'required'
        ]);
        if (is_array($request->installment_date) && sizeof($request->installment_date)){
            foreach($request->installment_date as $key=>  $instalment_date) {
                $instalmentAmountCheck = $this->makeSureInstalmentsLessThanBalance($request->supplier_id);
                if ($instalmentAmountCheck === false) {
                    session()->flash('error', 'عفوا يجب أن لا يتخطى إجمالي المبلغ المجدول مبلغ الدين');
                    return redirect()->route('supplierinstallment.index');
                }

                Instalment::create([
                    'client_id' => 0,
                    'installment_amount' => $request->installment_amount[$key],
                    'installment_date' => Carbon::parse($instalment_date)->format('Y-m-d'),
                    'bill_id' => 0,
                    'installment_pay' => 0,
                    'notes' => $request->notes or '-',
                    'shop_id' => get_session_val('serial_id', 'integer'),
                    'type' => 1,
                    'supplier_id' => $request->supplier_id
                ]);

            }

            session()->flash('success', 'تم عملية الجدولة بنجاح');
            return redirect()->route('supplierinstallment.index');
        }

        return back();
    }


    /**
     * @param $id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function payingTheDebtView($id)
    {
        $instalment = Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->with('supplier:id,client_name')
            ->where('type', 1)
            ->where('id', $id)->first();
        if (!$instalment->supplier) return back()->with('error', 'بيانات هذا المورد غير موجوده');
        $salePoints = $this->allowedStores();
        return view('supplierinstallments::paying-debt', compact('instalment', 'salePoints'));
    }

    /**
     * Paying the debt of instalment to the supplier
     * @param $id
     * @return mixed
     */
    public function payingTheDebt(Request $request, $id)    // done
    {

        $xxx = $request->installment_amount;

        $instalment = Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('type', 1)
            ->where('id', $id)->first();

        $today = $this->formatDate($request->today);

        $paidMoney = $request->installment_amount;

        $s_point = SalePoint::find($request->sale_point);
        $n_s_balance = $s_point->money_point - $xxx;

        $supplier = Client::find($instalment->supplier_id);
        if ($supplier) {
            $supplier->balance += $xxx;
            $supplier->save();
        }

        $s_point->money_point = $n_s_balance;
        $s_point->save();

        if ($paidMoney > $instalment->installment_amount) {
            \DB::beginTransaction();
                $allInstalments = $this->getUnpaidInstalmentsForSupplier($instalment->supplier_id);
    
                foreach($allInstalments as $oneInstalment) {
                    if ($paidMoney > $oneInstalment->installment_amount) {
                        $oneInstalment->installment_pay = 1;
                        $oneInstalment->pay_date = now();
                        $oneInstalment->notes = 'تم دفع مبلغ ' . $paidMoney . ' بدلا من مبلغ '. $oneInstalment->installment_amount  . $request->notes;
                        $oneInstalment->save();
    
                    }
    
                    if ($paidMoney == $oneInstalment->installment_amount) {
                        $oneInstalment->installment_pay = 1;
                        $oneInstalment->pay_date = now();
                        $oneInstalment->notes = 'تم سداد القسط كاملا';
                        $oneInstalment->save();
    
                    }
                    
                    if ($paidMoney < $oneInstalment->installment_amount && $paidMoney > 0) {
                        $oneInstalment->installment_amount = $oneInstalment->installment_amount - $paidMoney;
                        $oneInstalment->notes =  ' تم ترحيل مبلغ ' . $paidMoney . ' زائد من القسط السابق وخصمه من هذا القسط';
                        $oneInstalment->save();
                    }
    
                    $paidMoney = $paidMoney - $oneInstalment->installment_amount;
                }
                
                $this->doTransaction(
                    $request->sale_point,$instalment->supplier_id,
                    $xxx, $supplier->balance,$today,$n_s_balance
                );
    
                // here
                $this->updateInvoicePayment($instalment->bill_id, $xxx);
            \DB::commit();

            session()->flash('success', 'تمت عملية السداد بنجاح');
            return redirect()->route('supplierinstallment.index');
        }

        if ($paidMoney < $instalment->installment_amount) {

            // $instalment->pay_date = $request->today;
            $instalment->installment_amount = $instalment->installment_amount - $paidMoney;
            $instalment->notes = $request->notes ? $request->notes : '' . ' '. "تم دفع $paidMoney جزء من القسط بتاريخ $request->today";
            $instalment->save();

            $this->doTransaction(
                $request->sale_point,$instalment->supplier_id,
                $paidMoney, $supplier->balance,$today,$n_s_balance
            );

            $this->updateInvoicePayment($instalment->bill_id, $paidMoney);

            session()->flash('success', 'تمت عملية سداد جزء من القسط بنجاح');
            return redirect()->route('supplierinstallment.index');

        }

        if ($instalment) {
            \DB::beginTransaction();
                $instalment->installment_pay = 1;
                $instalment->pay_date = now();
                $instalment->save();
    
                $this->doTransaction(
                    $request->sale_point,$instalment->supplier_id,
                    $instalment->installment_amount, $supplier->balance,$today,$n_s_balance
                );
    
                $this->updateInvoicePayment($instalment->bill_id, $instalment->installment_amount);
            \DB::commit();

            session()->flash('success', 'تمت عملية السداد بنجاح');
            return redirect()->route('supplierinstallment.index');
        }

        return back();
    }


    /**
     * Receive the supplier id and the amount of instalment
     * Get the supplier by it's id
     * Subtract the instalment amount from the supplier balance
     * Save the record
     * @param $supplierId
     * @param $instalmentAmount
     * @return bool|\Illuminate\Http\RedirectResponse
     */

    private function subtractInstalmentAmountFromSupplierBalance($supplierId, $instalmentAmount)
    {
        $supplier = Client::where('shop_id', get_session_val('serial_id', 'integer'))
            ->whereIn('client_role', [1, 2])
            ->where('id', $supplierId)->first();
        if ($supplier) {
            $supplier->balance = $supplier->balance + $instalmentAmount;
            $supplier->save();
            return $supplier;
        }
        return false;
    }

    /**
     * @param $salePointId
     * @param $instalmentAmount
     * @return bool
     */
    private function subtractInstalmentAmountFromSalePoint($salePointId, $instalmentAmount) {
        $salePoint = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('id', $salePointId)
            ->first();
        $salePoint->money_point = $salePoint->money_point - $instalmentAmount;
        if ($salePoint->save()) {
            return $salePoint;
        }
        return false;
    }


    /**
     * Check if the amount of the instalments is less than supplier balance or not
     * @param $supplierId
     * @return bool
     */
    private function makeSureInstalmentsLessThanBalance($supplierId)
    {
        $supplier = $this->getSupplier($supplierId);
        $totalAmountOfInstalment = $this->totalAmountOfInstalments($supplierId);
        if ($totalAmountOfInstalment > $supplier->balance) {
            return true;
        }
        return false;
    }

    /**
     * Get the total amount of the instalments for one supplier
     * @param $supplierId
     * @return float
     */
    private function totalAmountOfInstalments($supplierId)
    {
        $totalAmountOfInstalemnt = Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('supplier_id', $supplierId)
            ->where('type', 1)
            ->where('installment_pay', 0)->sum('installment_amount');
        return $totalAmountOfInstalemnt;
    }


    /**
     * Get supplier by its id
     * @param $supplierId
     * @return object $supplier
     */
    private function getSupplier($supplierId)
    {
        $supplier = Client::where('shop_id', get_session_val('serial_id', 'integer'))
                    ->whereIn('client_role', [1, 2])
                    ->where('id', $supplierId)->first();
        return $supplier;
    }


    /**
     * Get unpaid instalments for a specific supplier
     * @param $supplierId
     * @return mixed
     */
    private function getUnpaidInstalmentsForSupplier($supplierId)
    {
        return Instalment::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('installment_pay', 0)
            ->where('type', 1)
            ->where('supplier_id', $supplierId)
            ->orderBy(\DB::raw('installment_date'))
            ->get();
    }

    private function doTransaction($salePoint, $supplierId, $paidMoney, $supplierBalance, $date, $moneyPoint){

        $clientTransaction = new ClientTransaction();
        $clientTransaction->shop_id = get_session_val('serial_id', 'integer');
        $clientTransaction->date_time = $date;
        $clientTransaction->client_id = $supplierId;
        $clientTransaction->amount = $paidMoney;
        $clientTransaction->type = 6;
        $clientTransaction->effect = 0;  #     1 للايداع وصفر للسحب
        $clientTransaction->pay_day = $date;
        $clientTransaction->balance = $supplierBalance; # new not old?
        $clientTransaction->bill_id = 0;
        $clientTransaction->bill_net_total = 0;

        $clientTransaction->safe_type = 1;

        $clientTransaction->safe_balance = $moneyPoint;
        $clientTransaction->safe_point_id = $salePoint;
        $clientTransaction->user_id = \Auth::id();

        $clientTransaction->sale_back_id = 0;
        $clientTransaction->problem_id = 0;
        $clientTransaction->supplier_id = $supplierId;
        $clientTransaction->salary_month = 0;
        $clientTransaction->OutTransactionID = 0;
        $clientTransaction->spend_id = 0;
        $clientTransaction->notes = request()->get('notes');

        $clientTransaction->save();

    }

    private function formatDate($date)
    {
        $time = date('H:i:s');
        $date_time = $date.' '.$time;

        $date = Carbon::parse($date_time)->format('Y-m-d H:i:s');
        return $date;
    }

    private function allowedStores()
    {
        $point = Auth::user()->sale_point;
        if (Auth::user()->show_points == 1 ||Auth::user()->show_points == 2)  {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id' ,'integer'))
                ->where('id', $point)
                ->get();
        } elseif (Auth::user()->show_points == 3) {
            $store_id = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('id', $point)
                ->first()->store_id;
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'))
            ->where('store_id', $store_id)
            ->get();

        }
        else {
            $sale_points = SalePoint::where('shop_id', get_session_val('serial_id' ,'integer'))->get();
        }
        return $sale_points;
    }

    protected function updateInvoicePayment($invoice_id, $paid)
    {
        $invoice = Invoices::find($invoice_id);

        if (! is_null($invoice) && $paid > 0) {
            if ($invoice->payment < $invoice->net_price) {
                $amount = $paid;
                if ($invoice->net_price - $invoice->payment >= $paid) {
                    $amount = $paid;
                } else {
                    $amount = $invoice->net_price - $invoice->payment;
                }
                
                $invoice->payment += $amount;
                $invoice->rest -= $amount;

                $invoice->save();
            }
        }
    }
}
