<?php

namespace Modules\transformer\Http\Controllers;

use App\Models\BillAddHistory;
use App\Models\BillImage;
use App\Models\Car;
use App\Models\CardNumber;
use App\Models\ClientGlasses;
use App\Models\ClientImage;
use App\Models\ClientLocation;
use App\Models\ConvertMoney;
use App\Models\DetailQuantity;
use App\Models\DiscountCard;
use App\Models\Favorite;
use App\Models\GiftHistory;
use App\Models\Instalment;
use App\Models\Item;
use App\Models\ItemRate;
use App\Models\LogSystem;
use App\Models\ManufacturingOrder;
use App\Models\OfferProcess;
use App\Models\OutstoreSetting;
use App\Models\Request as clientRequest;
use App\Models\SmsHistory;
use App\Models\SubscriptionDate;
use App\Models\SupplierOrder;
use App\Models\TypesCookingSales;
use App\Models\Visit;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;
use Modules\Clients\Entities\Client;
use Modules\Invoices\Entities\ClientTransaction;
use Modules\Invoices\Entities\InvoiceDetails;
use Modules\Invoices\Entities\Invoices;
use Modules\Invoices\Entities\ItemSerial;
use Modules\Invoices\Entities\Store;
use Modules\Invoices\Entities\StoreItem;
use Modules\Item\Entities\MultiSupplier;
use PhpParser\Node\Stmt\TryCatch;
use Illuminate\Support\Facades\Schema;
use Modules\Invoices\Entities\InvoiceDetailsVats;

use function PHPUnit\Framework\throwException;

class transformerController extends Controller
{
    private  $chunk = 100, $null_errors = [], $default_cols_vals = [];
    private $file_path = '../storage/logs/transformer.txt';

    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index()
    {

        $start_time = Date('H:i:s');

        $this->prepareLogFile();
        $this->transformUneffectedTables();
        $this->transformTables();

        $end_time = Date('H:i:s');

        return view('transformer::index', compact('start_time', 'end_time'));
    }

    private function transformTables()
    {
        $this->transformClients(); 
        $this->transformSuppliers();

        $this->transformItems();

        $this->transformPurchaseInvoices();
        $this->transformReturnedPurchaseInvoices();
        $this->transformSalesInvoices();
        $this->transformReturnedSalesInvoices();

        $this->transformClientsTransactions();
        $this->transformItemsTransactions();

        $this->transformBillImages();
        $this->transformCardNumbers();
        $this->transformCars();
        $this->transformChargeOrderBills();
        $this->transformClientGlasses();
        $this->transformClientImages();

        $this->transformClientGroups();
        $this->transformBillsAddHistory();
        $this->transformClientLocations();
        $this->transformConvertMoney();
        $this->transformDetailQuantities();

        $this->transformDiscountCard();
        $this->transformFavorites();
        $this->transformGiftHistory();
        $this->transformInstallmentsList();

        $this->transformItemRate();
        $this->transformItemSerials();

        $this->transformLogSystem();
        $this->transformManufacturingOrder();
        $this->transformMultiSupplier();
        $this->transformOfferProcess();

        $this->transformOutstoreorders();
        $this->transformOutstoreSetting();

        $this->transformRequests();
        $this->transformSmsHistory();

        $this->transformSubscriptionDates();
        $this->transformSupplierOrders();
        $this->transformTypesCookingSales();

        $this->transformVisits();
    }

    private function transformUneffectedTables()
    {
        $all_table_names = DB::connection('old_db')->select(DB::raw("SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE = 'BASE TABLE' AND TABLE_SCHEMA='". DB::connection('old_db')->getDatabaseName() ."'"));        

        $effected_tables = [
            'clients', 
            'supplier', 
            'items',
            'incoming_bill',
            'incoming_details',
            'incoming_bill_return',
            'incoming_details_return',
            'sale_process',
            'sale_details',
            'sale_back',
            'sale_back_invoice',
            'items_transaction',
            'client_transaction',
            'bill_images',
            'card_numbers',
            'client_glasses',
            'client_images',
            'client_locations',
            'convert_money',
            'detail_quantities',
            'discount_cards',
            'favorites',
            'gift_history',
            'installments_list',
            'item_rates',
            'item_serials',
            'log_system',
            'manufacturing_order',
            'multi_supplier',
            'offer_process',
            'outstoreorders',
            'outstoresetting',
            'requests',
            'sms_history',
            'subscription_dates',
            'supplier_orders',
            'types_cooking_sales',
            'visits',
            'bills_add_history',
            'clients_groups'
        ];

        $escaped_tables = [
            'migrations',
            'project_details',
            "cache",
            "cache_locks",
            "sessions"
        ];

        $id_is_different = [
            'badr_shop' => 'serial_id',
            'barcodesetting'=> 'shop_id',
            'bill_setting' => 'shop_id',
            'sms_setting' => 'shop_id',
            'zone' => 'zone_id',
            'relative_shops' => 'main_id'
        ];

        $no_shop_id = [
            'suppliers_groups',
            'personal_access_tokens',
            'address',
            'charge_cities',
            'countries',
            'failed_jobs',
            'regions',
            'sms_direct_messages',
            'sms_providers',
            'sms_provider_additional_params',
            'sms_provider_messages',
            'sms_provider_track_activity',
            'sms_status',
            'ticket_replies',
            'zone'
        ];

        foreach ($all_table_names as $key => $table_name) {
            if ( !in_array($table_name->TABLE_NAME, $effected_tables) & !in_array($table_name->TABLE_NAME, $escaped_tables))
            {
                $id = $id_is_different[$table_name->TABLE_NAME]  ?? 'id';
                $shop_id = $table_name->TABLE_NAME == 'badr_shop' ? 'serial_id' : 'shop_id';
                $this->transformTable($table_name->TABLE_NAME, $id, $shop_id , in_array($table_name->TABLE_NAME, $no_shop_id));
            }
        }
    }

    private function transformTable($table_name, $id = 'id', $shop_id = 'shop_id', $no_shop_id = false)
    {

        try {
            $max_min_ids = DB::connection('old_db')->table($table_name)
            ->when(!$no_shop_id, function($q) use($shop_id){
                $q->where($shop_id ,get_session_val('serial_id', 'Integer'));
            })
            ->select(DB::raw('MAX('. $id .') AS max_id, MIN('. $id .') AS min_id'))->first();
        } catch (\Throwable $th) {
            $this->transformerError($table_name, null, $th->getMessage());
            return;
        }

        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;


        $current_selection = DB::connection('old_db')->table($table_name)
        ->where([ [$id, '<=', ($max_id)], [$id, '>=', ($temp_max_id)] ])
        ->when(!$no_shop_id, function($q) use($shop_id){
            $q->where($shop_id ,get_session_val('serial_id', 'Integer'));
        })
        ->orderBy($id)
        ->chunk( $this->chunk, function($group) use($table_name, $id){
            foreach($group as $data){
                try {
                    $assigned_data = (array)$data;
                    if ($table_name == 'users')
                    {
                        $assigned_data['password'] = Hash::make(123456789);
                    }
                    DB::table($table_name)->insert($assigned_data);
                } catch (\Throwable $th) {
                    $this->transformerError($table_name, $assigned_data[$id], $th->getMessage());
                    continue;
                }
            }
        });
    }

    private function transformClients()
    {


        // step 1
        $max_min_ids = DB::connection('old_db')->table('clients')->where('shop_id' ,get_session_val('serial_id', 'Integer'))
        ->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;

        $current_selection = DB::connection('old_db')->table('clients')
        ->where([ ['id', '<=', ($max_id)], ['id', '>=', ($temp_max_id)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
        ->orderBy('id')
        ->chunk( $this->chunk, function($group){
            foreach($group as $data){
                try {
                    $new_client = new Client();
                    $assigned_data = (array)$data;
                    $assigned_data['old_id'] = $data->id;
                    unset($assigned_data['id']);
                    $new_client->fill($assigned_data);
                    $new_client->save();
                } catch (\Throwable $th) {
                    $this->transformerError('clients', $data->id, $th->getMessage());
                    continue;
                }
            }
        });

        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('clients')
            ->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                try {
                    $new_client = new Client();
                    $assigned_data = (array)$data;
                    $assigned_data['old_id'] = $data->id;
                    unset($assigned_data['id']);
                    $new_client->fill($assigned_data);
                    $new_client->save();
                } catch (\Throwable $th) {
                    $this->transformerError('clients', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }

        if (array_key_exists('clients', $this->null_errors))
        {
            $this->transformNullClients();
        }
    }   

    private function transformNullClients()
    {

        $null_ids = $this->null_errors['clients'];
        $current_selection = DB::connection('old_db')->table('clients')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $new_client = new Client();
                    $assigned_data = (array)$data;
                    $assigned_data['old_id'] = $data->id;
                    unset($assigned_data['id']);

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue('clients', $null_col);
                    }

                    $new_client->fill($assigned_data);
                    $new_client->save();

                    $inserted = true;
                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError('clients', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }
    }

    private function transformSuppliers()
    {
        // step 1
        $max_min_ids = DB::connection('old_db')->table('supplier')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        
        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('supplier')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                try {
                    DB::table('clients')->insert([
                        'client_name' => $data->supplier_name,
                        'tele' => $data->tel,
                        'address' => $data->address,
                        'supplier_person' => $data->supplier_person,
                        'mobile1' => $data->mobile,
                        'balance' => $data->balance,
                        'devices_company' => $data->devices_company,
                        'add_user' => $data->add_user ?? 0,
                        'add_date' => $data->add_date,
                        'edit_user' => $data->edit_user ?? 0,
                        'edit_date' => $data->edit_date,
                        'shop_id' => $data->shop_id,
                        'client_tax_number' => $data->tax_number,
                        'currency_id' => $data->currency_id,
                        'has_vat' => $data->has_vat ?? 0,
                        'mobile_id' => $data->mobile_id,
                        'client_role' => 1,
                        'old_id' => $data->id
                    ]);

                } catch (\Throwable $th) {
                    $this->transformerError('supplier', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }

        if (array_key_exists('supplier', $this->null_errors))
        {
            $this->transformNullSuppliers();
        }
    }   

    private function transformNullSuppliers()
    {

        $null_ids = $this->null_errors['supplier'];
        $current_selection = DB::connection('old_db')->table('supplier')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {
                    $assigned_data = [
                        'client_name' => $data->supplier_name,
                        'tele' => $data->tel,
                        'address' => $data->address,
                        'supplier_person' => $data->supplier_person,
                        'mobile1' => $data->mobile,
                        'balance' => $data->balance,
                        'devices_company' => $data->devices_company,
                        'add_user' => $data->add_user ?? 0,
                        'add_date' => $data->add_date,
                        'edit_user' => $data->edit_user ?? 0,
                        'edit_date' => $data->edit_date,
                        'shop_id' => $data->shop_id,
                        'client_tax_number' => $data->tax_number,
                        'currency_id' => $data->currency_id,
                        'has_vat' => $data->has_vat ?? 0,
                        'mobile_id' => $data->mobile_id,
                        'client_role' => 1,
                        'old_id' => $data->id
                    ];

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue('supplier', $null_col);
                    }


                    DB::table('clients')->insert($assigned_data);

                    $inserted = true;
                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError('supplier', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }

    }   

    private function transformItems()   // needs supplier transformation
    {
        // step 1
        $max_min_ids = DB::connection('old_db')->table('items')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();

        if (is_null($max_min_ids->max_id))
        {
            return;
        }
        else {
            try {
                $new_store = new Store();
                $new_store->store_name = 'المخزن الافتراضي';
                $new_store->shop_id = get_session_val('serial_id', 'Integer');
                $new_store->save();
            } catch (\Throwable $th) {
                $this->transformerError('stores', null , $th->getMessage());
                return;
            }
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('items')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                try {

                    DB::transaction(function () use($data, $new_store) {

                        $new_item = new Item();
                        $assigned_data = (array)$data;
                        if ($data->supplier_id > 0)
                        {
                            $new_client_id = ($data->supplier_id > 0) ? $this->getClientNewId(1, $data->supplier_id, $data, 'items') : $data->supplier_id;
                            $assigned_data['supplier_id'] = $new_client_id;
                        }
                        $new_item->fill($assigned_data);
                        $new_item->quantity = 0;
                        $new_item->save();

                        $new_store_item = new StoreItem();
                        $new_store_item->item_id = $new_item->id;
                        $new_store_item->store_id = $new_store->id;
                        $new_store_item->store_quant = $data->quantity ?? 0;
                        $new_store_item->save();

                    });

                } catch (\Throwable $th) {
                    $this->transformerError('items', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }

        if (array_key_exists('items', $this->null_errors))
        {
            $this->transformNullItems($new_store->id);
        }
    }

    private function transformNullItems($new_store_id)
    {
        
        $null_ids = $this->null_errors['items'];
        $current_selection = DB::connection('old_db')->table('items')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    DB::transaction(function () use($data, $new_store_id, $temp_null_cols, &$inserted) {

                        $new_item = new Item();
                        $assigned_data = (array)$data;
                        if ($data->supplier_id > 0)
                        {
                            $new_client_id = ($data->supplier_id > 0) ? $this->getClientNewId(1, $data->supplier_id, $data, 'items') : $data->supplier_id;
                            $assigned_data['supplier_id'] = $new_client_id;
                        }

                        // loop on null cols
                        foreach ($temp_null_cols as $null_col) {
                            $assigned_data[$null_col] = $this->getDefaultValue('items', $null_col);
                        }

                        $new_item->fill($assigned_data);
                        $new_item->quantity = 0;
                        $new_item->save();

                        $new_store_item = new StoreItem();
                        $new_store_item->item_id = $new_item->id;
                        $new_store_item->store_id = $new_store_id;
                        $new_store_item->store_quant = $data->quantity ?? 0;
                        $new_store_item->save();

                        $inserted = true;
                    });

                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError('items', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }

    }   

    private function transformPurchaseInvoices()
    {
        $discount_vals = [];

        // step 1
        $max_min_ids = DB::connection('old_db')->table('incoming_bill')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        if (is_null($max_min_ids->max_id))
        {
            return;
        }
        
        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('incoming_bill')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();
            
            foreach ($current_selection as $data) {
                 
                try {
                    DB::table('invoices')->insert([
                        'invoice_date' => $data->incoming_date,
                        'bill_total' => $data->total_bill,
                        'add_user' => $data->add_user,
                        'client_id' =>  ($data->supplier_id > 0) ? $this->getClientNewId(1, $data->supplier_id, $data, 'incoming_bill') : $data->supplier_id,
                        'net_price' => $data->net_price,
                        'payment' => $data->payment,
                        'bill_payment' => $data->payment,
                        'rest' => $data->net_price - $data->payment,
                        'store_id' => $data->store_id,
                        'bill_no' => $data->incom_id,
                        'notes' => $data->notes,
                        'shop_id' => $data->shop_id,
                        'balance' => $data->supplier_balance,
                        'currency_id' => $data->currency_id,
                        'currency_value' => $data->currency_value,
                        'invoice_type' => 0,
                        'transaction_date' => $data->add_date,
                        'reference_image' => $data->reference_image,
                        'effected_bill_id' => $data->supplier_bill_no,
                        'old_invoice_id' => $data->id
                    ]);

                    if (!is_null($data->discount) & floatval($data->discount) != 0) {
                        /* 
                            0 => %, 1 => $
                        */
                        $discount_vals[$data->id] = 
                        [
                            $this->getItemDiscount($data->discount, $data->discount_type, 0, $data->id)
                            +
                            $this->getItemDiscount($data->vat_discount_value, $data->vat_discount_type, 0, $data->id), 
                            $data->discount_type
                        ];
                    }
                } catch (\Throwable $th) {
                    $this->transformerError('incoming_bill', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }

        // step 2
        
        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_min_ids = DB::connection('old_db')->table('incoming_details')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        $max_id = !is_null($max_min_ids) ? ($max_min_ids->max_id ?? 0) : 0;
        $temp_max_id = !is_null($max_min_ids) ? ($max_min_ids->min_id ?? 0) : 0;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('incoming_details')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                $invoice_id = Invoices::where([
                    ['shop_id' , get_session_val('serial_id', 'Integer')],
                    ['old_invoice_id', $data->bill_id],
                    ['invoice_type', 0]
                ])->limit(1)->value('id');

                try {

                    if ($invoice_id)
                    {
                        DB::table('invoices_details')->insert([
                            'invoice_id' => $invoice_id,
                            'item_id' => $data->items_id,
                            'shop_id' => $data->shop_id,
                            'quantity' => $data->quantity,
                            'unit_id' => $data->unit,
                            'pay_price' => $data->price,
                            'sale_price' => $data->sale_price ?? 0,
                            'total_price' => $data->total,
                            'basic_sale_price' => $data->sale_price ?? 0,
                            'basic_pay_price' => $data->basic_price_in ?? $data->price,
                            'expire_date' => $data->expire_date,
                            'main_qty' => $data->main_qty,
                            'disc_qty' => $data->disc_qty,
                            'glass_height' => $data->glass_height,
                            'glass_width' => $data->glass_width,
                            'glass_number' => $data->glass_number,
                            'item_about' => $data->about_item,
                            'item_notes' => $data->notes,
                            'not_sold' => 0,
                            'discount' => key_exists($data->bill_id, $discount_vals) ? $discount_vals[$data->bill_id][0] : 0,
                            'discount_type' => key_exists($data->bill_id, $discount_vals) ? $discount_vals[$data->bill_id][1] : 0,
                            'ac_pay_price' => 0,
                            'av_pay_price' => 0,
                            'pay_vats_weight' => 0,
                        ]);
    
                    }
                    else {
                        $this->transformerError('incoming_details', $data->id, 'invoice with id ( ' . $data->bill_id. ' ) not exists');
                    }

                } catch (\Throwable $th) {
                    $this->transformerError('incoming_details', $data->id, $th->getMessage());
                    continue;
                }
                try {

                    if ($invoice_id)
                    {
                        $add_details = DB::connection('old_db')->table('bills_add_history')->where( [
                            ['bill_id', $data->bill_id],
                            ['type', 0],
                            ['shop_id' , get_session_val('serial_id', 'Integer')]
                            ])->first();
                        $item = DB::connection('old_db')->table('items')->where([ ['id', $data->items_id], ['shop_id' ,get_session_val('serial_id', 'Integer')]] )->first();
                        
                        if (! is_null($add_details) & ! is_null($item))
                        {
                            DB::table('invoices_details_vats')->insert([
                                'invoice_id' => $invoice_id, 
                                'item_id' => $data->items_id,
                                'vat_id' => $add_details->addition_id,
                                'vat_type' => $add_details->add_type,
                                'vat_value' => $add_details->bill_add,
                                'vat_weight' => $this->getItemVats($item->purchase_vat_state, $add_details->bill_add, $add_details->add_type, $data->price),
                                'vat_operation' => 0,
                                'shop_id' => $data->shop_id,
                            ]);
                        }
                        else if (is_null($item))
                        {
                            $this->transformerError('incoming_details', $data->id, 'item with id ( '. $data->items_id .' ) is deleted ');
                        }
                    }

                } catch (\Throwable $th) {
                    $this->transformerError('incoming_details', $data->id, $th->getMessage());
                    continue;
                }


            }
            $loop++;
        }

        
        if (array_key_exists('incoming_bill', $this->null_errors) || array_key_exists('incoming_details', $this->null_errors))
        {
            $this->transformNullPurchaseInvoices();
        }
    }   

    private function transformNullPurchaseInvoices()
    {
        $discount_vals = [];

        // first step    
        $null_ids = $this->null_errors['incoming_bill'];
        $current_selection = DB::connection('old_db')->table('incoming_bill')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $assigned_data = [
                        'invoice_date' => $data->incoming_date,
                        'bill_total' => $data->total_bill,
                        'add_user' => $data->add_user,
                        'client_id' =>  ($data->supplier_id > 0) ? $this->getClientNewId(1, $data->supplier_id, $data, 'incoming_bill') : $data->supplier_id,
                        'net_price' => $data->net_price,
                        'payment' => $data->payment,
                        'bill_payment' => $data->payment,
                        'rest' => $data->net_price - $data->payment,
                        'store_id' => $data->store_id,
                        'bill_no' => $data->incom_id,
                        'notes' => $data->notes,
                        'shop_id' => $data->shop_id,
                        'balance' => $data->supplier_balance,
                        'currency_id' => $data->currency_id,
                        'currency_value' => $data->currency_value,
                        'invoice_type' => 0,
                        'transaction_date' => $data->add_date,
                        'reference_image' => $data->reference_image,
                        'effected_bill_id' => $data->supplier_bill_no,
                        'old_invoice_id' => $data->id
                    ];

                    if (!is_null($data->discount) & floatval($data->discount) != 0) {
                        $discount_vals[$data->id] = 
                        [
                            $this->getItemDiscount($data->discount, $data->discount_type, 0, $data->id)
                            +
                            $this->getItemDiscount($data->vat_discount_value, $data->vat_discount_type, 0, $data->id), 
                            $data->discount_type
                        ];
                    }

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue('invoices', $null_col);
                    }

                    DB::table('invoices')->insert($assigned_data);
                    $inserted = true;

                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError('incoming_bill', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }

        // second step    
        $null_ids = $this->null_errors['incoming_details'];
        $current_selection = DB::connection('old_db')->table('incoming_details')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $invoice_id = Invoices::where([
                        ['shop_id' , get_session_val('serial_id', 'Integer')],
                        ['old_invoice_id', $data->bill_id],
                        ['invoice_type', 0]
                    ])->limit(1)->value('id');

                    $invoice_details = InvoiceDetails::where([
                        ['shop_id' , get_session_val('serial_id', 'Integer')],
                        ['invoice_id', $invoice_id],
                        ['item_id', $data->items_id]
                    ])->first();

                    if (! $invoice_details)
                    {
                        $assigned_data = [
                                'invoice_id' => $invoice_id,
                                'item_id' => $data->items_id,
                                'shop_id' => $data->shop_id,
                                'quantity' => $data->quantity,
                                'unit_id' => $data->unit,
                                'pay_price' => $data->price,
                                'sale_price' => $data->sale_price ?? 0,
                                'total_price' => $data->total,
                                'basic_sale_price' => $data->sale_price ?? 0,
                                'basic_pay_price' => $data->basic_price_in ?? $data->price,
                                'expire_date' => $data->expire_date,
                                'main_qty' => $data->main_qty,
                                'disc_qty' => $data->disc_qty,
                                'glass_height' => $data->glass_height,
                                'glass_width' => $data->glass_width,
                                'glass_number' => $data->glass_number,
                                'item_about' => $data->about_item,
                                'item_notes' => $data->notes,
                                'not_sold' => 0,
                                'discount' => key_exists($data->bill_id, $discount_vals) ? $discount_vals[$data->bill_id][0] : 0,
                                'discount_type' => key_exists($data->bill_id, $discount_vals) ? $discount_vals[$data->bill_id][1] : 0,
                                'ac_pay_price' => 0,
                                'av_pay_price' => 0,
                                'pay_vats_weight' => 0,
                        ];

                        // loop on null cols
                        foreach ($temp_null_cols as $table_name => $null_col) {
                            if ($table_name == 'invoice_details')
                            {
                                $assigned_data[$null_col] = $this->getDefaultValue('invoice_details', $null_col);
                            }
                        }

                        DB::table('invoices_details')->insert($assigned_data);
                    }


                    $invoice_details_vat = InvoiceDetailsVats::where([
                        ['shop_id' , get_session_val('serial_id', 'Integer')],
                        ['invoice_id', $invoice_id],
                        ['item_id', $data->items_id]
                    ])->first();

                    if (! $invoice_details_vat)
                    {
                        $add_details = DB::connection('old_db')->table('bills_add_history')->where( [
                            ['bill_id', $data->bill_id],
                            ['type', 0],
                            ['shop_id' , get_session_val('serial_id', 'Integer')]
                            ])->first();
                        $item = DB::connection('old_db')->table('items')->where([ ['id', $data->items_id], ['shop_id' ,get_session_val('serial_id', 'Integer')]] )->first();

                        if (! is_null($add_details) & ! is_null($item))
                        {
                            $assigned_data = [
                                'invoice_id' => $invoice_id, 
                                'item_id' => $data->items_id,
                                'vat_id' => $add_details->addition_id,
                                'vat_type' => $add_details->add_type,
                                'vat_value' => $add_details->bill_add,
                                'vat_weight' => $this->getItemVats($item->purchase_vat_state, $add_details->bill_add, $add_details->add_type, $data->price),
                                'vat_operation' => 0,
                                'shop_id' => $data->shop_id,
                            ];

                            // loop on null cols
                            foreach ($temp_null_cols as $table_name => $null_col) {
                                if ($table_name == 'invoices_details_vats')
                                {
                                    $assigned_data[$null_col] = $this->getDefaultValue('invoices_details_vats', $null_col);
                                }
                            }

                            DB::table('invoices_details_vats')->insert($assigned_data);
                        }
                    }

                    $inserted = true;

                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    $table_name = Str::between($th->getMessage(), "insert into `", "` (`");

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols[$table_name], $check_result[1]);
                    }
                    else {
                        $this->transformerError('incoming_details', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }

    }   

    private function transformReturnedPurchaseInvoices()
    {
        $discount_vals = [];

        // step 1
        $max_min_ids = DB::connection('old_db')->table('incoming_bill_return')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        
        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('incoming_bill_return')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();
            foreach ($current_selection as $data) {
                 
                try {
                    DB::table('invoices')->insert([
                        'invoice_date' => $data->incoming_date,
                        'bill_total' => $data->total_bill,
                        'add_user' => $data->add_user,
                        'client_id' => ($data->supplier_id > 0) ? $this->getClientNewId(1, $data->supplier_id, $data, 'incoming_bill_return') : $data->supplier_id,
                        'net_price' => $data->net_price,
                        'payment' => $data->payment,
                        'bill_payment' => $data->payment,
                        'rest' => $data->net_price - $data->payment,
                        'store_id' => $data->store_id,
                        'bill_no' => $data->incom_id,
                        'notes' => $data->notes,
                        'shop_id' => $data->shop_id,
                        'balance' => $data->supplier_balance,
                        'invoice_type' => 2,
                        'transaction_date' => $data->add_date,
                        'effected_bill_id' => $data->supplier_bill_no,
                        'old_invoice_id' => $data->id
                    ]);

                    if (!is_null($data->discount) & floatval($data->discount) != 0) {
                        /* 
                            0 => %, 1 => $
                        */
                        $discount_vals[$data->id] = [$this->getItemDiscount($data->discount, $data->discount_type, 2, $data->id), $data->discount_type];
                    }
                } catch (\Throwable $th) {
                    $this->transformerError('incoming_bill_return', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }

        // step 2
        
        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_min_ids = DB::connection('old_db')->table('incoming_details_return')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        $max_id = !is_null($max_min_ids) ? ($max_min_ids->max_id ?? 0) : 0;
        $temp_max_id = !is_null($max_min_ids) ? ($max_min_ids->min_id ?? 0) : 0;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('incoming_details_return')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                $invoice_id = Invoices::where([
                    ['shop_id' , get_session_val('serial_id', 'Integer')],
                    ['old_invoice_id', $data->bill_id],
                    ['invoice_type', 2]
                ])->limit(1)->value('id');

                try {

                    if ($invoice_id)
                    {
                        DB::table('invoices_details')->insert([
                            'invoice_id' => $invoice_id,
                            'item_id' => $data->items_id,
                            'shop_id' => $data->shop_id,
                            'quantity' => $data->quantity,
                            'unit_id' => $data->unit,
                            'pay_price' => $data->price,
                            'sale_price' => $data->sale_price ?? 0,
                            'total_price' => $data->total,
                            'basic_sale_price' => $data->sale_price ?? 0,
                            'basic_pay_price' => $data->basic_price_in ?? $data->price,
                            'expire_date' => $data->expire_date,
                            'not_sold' => 0,
                            'discount' => key_exists($data->bill_id, $discount_vals) ? $discount_vals[$data->bill_id][0] : 0,
                            'discount_type' => key_exists($data->bill_id, $discount_vals) ? $discount_vals[$data->bill_id][1] : 0,
                            'ac_pay_price' => 0,
                            'av_pay_price' => 0,
                            'pay_vats_weight' => 0,
                        ]);
    
                    }
                    else {
                        $this->transformerError('incoming_details_return', $data->id, 'invoice with old_invoice_id ( ' . $data->bill_id. ' ) not exists');
                    }


                } catch (\Throwable $th) {
                    $this->transformerError('incoming_details_return', $data->id, $th->getMessage());
                    continue;
                }
                try {
                    if ($invoice_id)
                    {
                        $add_details = DB::connection('old_db')->table('bills_add_history')->where( [
                            ['bill_id', $data->bill_id],
                            ['type', 5],
                            ['shop_id' , get_session_val('serial_id', 'Integer')]
                            ])->first();
                        $item = DB::connection('old_db')->table('items')->where('id', $data->items_id)->first();
                        
                        if (! is_null($add_details) & ! is_null($item))
                        {
                            DB::table('invoices_details_vats')->insert([
                                'invoice_id' => $invoice_id, 
                                'item_id' => $data->items_id,
                                'vat_id' => $add_details->addition_id,
                                'vat_type' => $add_details->add_type,
                                'vat_value' => $add_details->bill_add,
                                'vat_weight' => $this->getItemVats($item->purchase_vat_state, $add_details->bill_add, $add_details->add_type, $data->price),
                                'vat_operation' => 0,
                                'shop_id' => $data->shop_id,
                            ]);
                        }
                        else if (is_null($item))
                        {
                            $this->transformerError('incoming_details_return', $data->id, 'item with id ( '. $data->items_id .' ) is deleted ');
                        }
                    }

                } catch (\Throwable $th) {
                    $this->transformerError('incoming_details_return', $data->id, $th->getMessage());
                    continue;
                }

            }
            $loop++;
        }

        if (array_key_exists('incoming_bill_return', $this->null_errors) || array_key_exists('incoming_details_return', $this->null_errors))
        {
            $this->transformNullReturnedPurchaseInvoices();
        }
    }

    private function transformNullReturnedPurchaseInvoices()
    {
        $discount_vals = [];

        // first step    
        $null_ids = $this->null_errors['incoming_bill_return'];
        $current_selection = DB::connection('old_db')->table('incoming_bill_return')->where('shop_id', get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while (!$inserted) {
                try {

                    $assigned_data = [
                        'invoice_date' => $data->incoming_date,
                        'bill_total' => $data->total_bill,
                        'add_user' => $data->add_user,
                        'client_id' => ($data->supplier_id > 0) ? $this->getClientNewId(1, $data->supplier_id, $data, 'incoming_bill_return') : $data->supplier_id,
                        'net_price' => $data->net_price,
                        'payment' => $data->payment,
                        'bill_payment' => $data->payment,
                        'rest' => $data->net_price - $data->payment,
                        'store_id' => $data->store_id,
                        'bill_no' => $data->incom_id,
                        'notes' => $data->notes,
                        'shop_id' => $data->shop_id,
                        'balance' => $data->supplier_balance,
                        'invoice_type' => 2,
                        'transaction_date' => $data->add_date,
                        'effected_bill_id' => $data->supplier_bill_no,
                        'old_invoice_id' => $data->id
                    ];

                    if (!is_null($data->discount) & floatval($data->discount) != 0) {

                        $discount_vals[$data->id] = [$this->getItemDiscount($data->discount, $data->discount_type, 2, $data->id), $data->discount_type];
                    }

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue('invoices', $null_col);
                    }

                    DB::table('invoices')->insert($assigned_data);
                    $inserted = true;
                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true) {
                        array_push($temp_null_cols, $check_result[1]);
                    } else {
                        $this->transformerError('incoming_bill_return', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }

        // second step    
        $null_ids = $this->null_errors['incoming_details_return'];
        $current_selection = DB::connection('old_db')->table('incoming_details_return')->where('shop_id', get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while (!$inserted) {
                try {

                    $invoice_id = Invoices::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['old_invoice_id', $data->bill_id],
                        ['invoice_type', 2]
                    ])->limit(1)->value('id');

                    $invoice_details = InvoiceDetails::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoice_id', $invoice_id],
                        ['item_id', $data->items_id]
                    ])->first();

                    if (!$invoice_details) {
                        $assigned_data = [
                            'invoice_id' => $invoice_id,
                            'item_id' => $data->items_id,
                            'shop_id' => $data->shop_id,
                            'quantity' => $data->quantity,
                            'unit_id' => $data->unit,
                            'pay_price' => $data->price,
                            'sale_price' => $data->sale_price ?? 0,
                            'total_price' => $data->total,
                            'basic_sale_price' => $data->sale_price ?? 0,
                            'basic_pay_price' => $data->basic_price_in ?? $data->price,
                            'expire_date' => $data->expire_date,
                            'not_sold' => 0,
                            'discount' => key_exists($data->bill_id, $discount_vals) ? $discount_vals[$data->bill_id][0] : 0,
                            'discount_type' => key_exists($data->bill_id, $discount_vals) ? $discount_vals[$data->bill_id][1] : 0,
                            'ac_pay_price' => 0,
                            'av_pay_price' => 0,
                            'pay_vats_weight' => 0,
                        ];

                        // loop on null cols
                        foreach ($temp_null_cols as $table_name => $null_col) {
                            if ($table_name == 'invoice_details') {
                                $assigned_data[$null_col] = $this->getDefaultValue('invoice_details', $null_col);
                            }
                        }

                        DB::table('invoices_details')->insert($assigned_data);
                    }

                    $invoice_details_vat = InvoiceDetailsVats::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoice_id', $invoice_id],
                        ['item_id', $data->items_id]
                    ])->first();

                    if (!$invoice_details_vat) {
                        $add_details = DB::connection('old_db')->table('bills_add_history')->where([
                            ['bill_id', $data->bill_id],
                            ['type', 5],
                            ['shop_id', get_session_val('serial_id', 'Integer')]
                        ])->first();
                        $item = DB::connection('old_db')->table('items')->where([['id', $data->items_id], ['shop_id', get_session_val('serial_id', 'Integer')]])->first();

                        if (!is_null($add_details) & !is_null($item)) 
                        {
                            $assigned_data = [
                                'invoice_id' => $invoice_id,
                                'item_id' => $data->items_id,
                                'vat_id' => $add_details->addition_id,
                                'vat_type' => $add_details->add_type,
                                'vat_value' => $add_details->bill_add,
                                'vat_weight' => $this->getItemVats($item->purchase_vat_state, $add_details->bill_add, $add_details->add_type, $data->price),
                                'vat_operation' => 0,
                                'shop_id' => $data->shop_id,
                            ];

                            // loop on null cols
                            foreach ($temp_null_cols as $table_name => $null_col) {
                                if ($table_name == 'invoices_details_vats') {
                                    $assigned_data[$null_col] = $this->getDefaultValue('invoices_details_vats', $null_col);
                                }
                            }

                            DB::table('invoices_details_vats')->insert($assigned_data);
                        }
                    }

                    $inserted = true;
                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    $table_name = Str::between($th->getMessage(), "insert into `", "` (`");

                    if ($check_result[0] == true) {
                        array_push($temp_null_cols[$table_name], $check_result[1]);
                    } else {
                        $this->transformerError('incoming_details_return', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }
    }

    private function transformSalesInvoices()
    {
        $discount_vals = [];

        // step 1
        $max_min_ids = DB::connection('old_db')->table('sale_process')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        
        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('sale_process')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();
            
            foreach ($current_selection as $data) {
                 
                try {
                    DB::table('invoices')->insert([
                        'transaction_date' => $data->date_process,
                        'invoice_date' => $data->sale_date,
                        'bill_total' => $data->total_price,
                        'add_user' => $data->add_user,
                        'client_id' => $this->getClientNewId(0, $data->client_id, $data, 'sale_process'),
                        'net_price' => $data->net_price,
                        'payment' => $data->payment,
                        'bill_payment' => $data->bil_payment,
                        'rest' => $data->rest,
                        'store_id' => $data->store,
                        'boll_per' => $data->boll_per,
                        'sales_man' => $data->sales_man,
                        'bill_no' => $data->bill_no,
                        'notes' => $data->notes,
                        'shop_id' => $data->shop_id,
                        'balance' => $data->balance ?? $data->clientOldBalance + $data->payment,
                        'pay_date' => $data->pay_date,
                        'pay_process' => $data->pay_process,
                        'notify_time' => $data->notify_time,
                        'notify_alert' => $data->notify_alert,
                        'cooking' => $data->cooking,
                        'insatllments_add' => $data->insatllments_add,
                        'StoreTransport' => $data->StoreTransport,
                        'StoreServices' => $data->StoreServices,
                        'car_id' => $data->car_id,
                        'fee' => $data->fee,
                        'status' => $data->status,
                        'done_date' => $data->done_date,
                        'due_date' => $data->due_date,
                        'notes_export' => $data->notes_export,
                        'notes_export2' => $data->notes_export2,
                        'show_bill' => $data->show_bill,
                        'coupon_id' => $data->coupon_id,
                        'fee_status' => $data->fee_status,
                        'server_status' => $data->server_status,
                        'server_bill_no' => $data->server_bill_no,
                        'city_id' => $data->city_id,
                        'bill_source' => $data->bill_source,
                        'bill_service_id' => $data->bill_service_id,
                        'delivery_man_id' => $data->delivery_man_id,
                        'delivery_date' => $data->delivery_date,
                        'bill_service_value' => $data->bill_service_value,
                        'delivery_option_id' => $data->delivery_option_id,
                        'delivery_period_id' => $data->delivery_period_id,
                        'order_ready' => $data->order_ready,
                        'action_type' => $data->action_type,
                        'charge_company_id' => $data->charge_company_id,
                        'request_id' => $data->request_id,
                        'is_commission_paid' => $data->is_commission_paid ?? 0,
                        'network_notes' => $data->network_notes,
                        'invoice_type' => 1,
                        'effected_bill_id' => $data->local_bill_no,
                        'old_invoice_id' => $data->id 
                    ]);

                    if (!is_null($data->discount) & floatval($data->discount) != 0) {
                        /* 
                            0 => %, 1 => $
                        */
                        $discount_vals[$data->id] = 
                        [
                            $this->getItemDiscount($data->discount, $data->discount_type, 1, $data->id)
                            +
                            $this->getItemDiscount($data->vat_discount_value, $data->vat_discount_type, 1, $data->id), 
                            $data->discount_type
                        ];
                    }
                } catch (\Throwable $th) {
                    $this->transformerError('sale_process', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }

        // step 2
        
        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_min_ids = DB::connection('old_db')->table('sale_details')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        $max_id = !is_null($max_min_ids) ? ($max_min_ids->max_id ?? 0) : 0;
        $temp_max_id = !is_null($max_min_ids) ? ($max_min_ids->min_id ?? 0) : 0;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('sale_details')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                $invoice_id = Invoices::where([
                    ['shop_id' , get_session_val('serial_id', 'Integer')],
                    ['old_invoice_id', $data->sale_id],
                    ['invoice_type', 1]
                ])->limit(1)->value('id');

                try {

                    if ($invoice_id)
                    {
                        DB::table('invoices_details')->insert([
                            'invoice_id' => $invoice_id,
                            'item_id' => $data->items_id,
                            'shop_id' => $data->shop_id,
                            'quantity' => $data->quantity,
                            'unit_id' => $data->unit,
                            'quantity_after' => $data->quant_after,
                            'pay_price' => $data->pay_price ?? 0,
                            'sale_price' => $data->price,
                            'total_price' => $data->total_price,
                            'basic_sale_price' => $data->basic_price ?? 0,
                            'basic_pay_price' => $data->pay_price ?? $data->price,
                            'glass_height' => $data->glass_height,
                            'glass_width' => $data->glass_width,
                            'glass_number' => $data->glass_number,
                            'item_about' => $data->about,
                            'item_notes' => $data->item_notes,
                            'not_sold' => 0,
                            'discount' => key_exists($data->sale_id, $discount_vals) ? $discount_vals[$data->sale_id][0] : 0,
                            'discount_type' => key_exists($data->sale_id, $discount_vals) ? $discount_vals[$data->sale_id][1] : 0,
                            'ac_pay_price' => $data->ac_pay_price,
                            'av_pay_price' => $data->av_pay_price,
                            'pay_vats_weight' => 0,
                            'sale_ratio' => $data->sale_ratio,
                            'ratio_type' => $data->ratio_type,
                            'bolla' => $data->bolla,
                            'sales_man' => $data->sale_man,
                            'safe_point_id' => $data->sale_point,
                            'store_id' =>$data->store_id,
                            'isservice' => $data->isservice,
                            'notify_alert' => $data->notify_alert,
                            'notify_time' => $data->notify_time,
                            'record_type' => $data->record_type,
                            'label_from' => $data->label_from,
                            'label_to' => $data->label_to,
                            'pricePercentage' => $data->pricePercentage,
                            'item_serial' => $data->item_serial,
                            'add_user' => $data->add_user,
                            'duration_type' => $data->duration_type,
                            'duration' => $data->duration,
                            'from_date' => $data->from_date,
                            'to_date' => $data->to_date,
                            'color_id' => $data->color_id,
                            'size_id' => $data->size_id,
                            'cards' => $data->cards,
                            'spare_id' => $data->spare_id,
                            'car_id' => $data->car_id,
                            'sales_row_id' => $data->sales_row_id,
                            'offer_id' => $data->offer_id,
                            'offer_text' => $data->offer_text,
                            'offer_type' => $data->offer_type,
                            'main_qty' => $data->main_qty,
                            'disc_qty' => $data->disc_qty,
                        ]);
    
                    }
                    else {
                        $this->transformerError('sale_details', $data->id, 'invoice with id ( ' . $data->sale_id. ' ) not exists');
                    }

                } catch (\Throwable $th) {
                    $this->transformerError('sale_details', $data->id, $th->getMessage());
                    continue;
                }
                try {

                    if ($invoice_id)
                    {
                        $add_details = DB::connection('old_db')->table('bills_add_history')->where( [
                            ['bill_id', $data->sale_id],
                            ['type', 1],
                            ['shop_id' , get_session_val('serial_id', 'Integer')]
                            ])->first();
                        $item = DB::connection('old_db')->table('items')->where([ ['id', $data->items_id], ['shop_id' ,get_session_val('serial_id', 'Integer')]] )->first();
                        
                        if (! is_null($add_details) & ! is_null($item))
                        {
                            DB::table('invoices_details_vats')->insert([
                                'invoice_id' => $invoice_id, 
                                'item_id' => $data->items_id,
                                'vat_id' => $add_details->addition_id,
                                'vat_type' => $add_details->add_type,
                                'vat_value' => $add_details->bill_add,
                                'vat_weight' => $this->getItemVats($data->item_vat_state, $add_details->bill_add, $add_details->add_type, $data->price),
                                'vat_operation' => 0,
                                'shop_id' => $data->shop_id,
                            ]);
                        }
                        else if (is_null($item))
                        {
                            $this->transformerError('sale_details', $data->id, 'item with id ( '. $data->items_id .' ) is deleted ');
                        }
                    }

                } catch (\Throwable $th) {
                    $this->transformerError('sale_details', $data->id, $th->getMessage());
                    continue;
                }


            }
            $loop++;
        }

        if (array_key_exists('sale_process', $this->null_errors) || array_key_exists('sale_details', $this->null_errors))
        {
            $this->transformNullSalesInvoices();
        }
    }

    private function transformNullSalesInvoices()
    {
        $discount_vals = [];

        // first step    
        $null_ids = $this->null_errors['sale_process'];
        $current_selection = DB::connection('old_db')->table('sale_process')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $assigned_data = [
                        'transaction_date' => $data->date_process,
                        'invoice_date' => $data->sale_date,
                        'bill_total' => $data->total_price,
                        'add_user' => $data->add_user,
                        'client_id' => $this->getClientNewId(0, $data->client_id, $data, 'sale_process'),
                        'net_price' => $data->net_price,
                        'payment' => $data->payment,
                        'bill_payment' => $data->bil_payment,
                        'rest' => $data->rest,
                        'store_id' => $data->store,
                        'boll_per' => $data->boll_per,
                        'sales_man' => $data->sales_man,
                        'bill_no' => $data->bill_no,
                        'notes' => $data->notes,
                        'shop_id' => $data->shop_id,
                        'balance' => $data->balance ?? $data->clientOldBalance + $data->payment,
                        'pay_date' => $data->pay_date,
                        'pay_process' => $data->pay_process,
                        'notify_time' => $data->notify_time,
                        'notify_alert' => $data->notify_alert,
                        'cooking' => $data->cooking,
                        'insatllments_add' => $data->insatllments_add,
                        'StoreTransport' => $data->StoreTransport,
                        'StoreServices' => $data->StoreServices,
                        'car_id' => $data->car_id,
                        'fee' => $data->fee,
                        'status' => $data->status,
                        'done_date' => $data->done_date,
                        'due_date' => $data->due_date,
                        'notes_export' => $data->notes_export,
                        'notes_export2' => $data->notes_export2,
                        'show_bill' => $data->show_bill,
                        'coupon_id' => $data->coupon_id,
                        'fee_status' => $data->fee_status,
                        'server_status' => $data->server_status,
                        'server_bill_no' => $data->server_bill_no,
                        'city_id' => $data->city_id,
                        'bill_source' => $data->bill_source,
                        'bill_service_id' => $data->bill_service_id,
                        'delivery_man_id' => $data->delivery_man_id,
                        'delivery_date' => $data->delivery_date,
                        'bill_service_value' => $data->bill_service_value,
                        'delivery_option_id' => $data->delivery_option_id,
                        'delivery_period_id' => $data->delivery_period_id,
                        'order_ready' => $data->order_ready,
                        'action_type' => $data->action_type,
                        'charge_company_id' => $data->charge_company_id,
                        'request_id' => $data->request_id,
                        'is_commission_paid' => $data->is_commission_paid ?? 0,
                        'network_notes' => $data->network_notes,
                        'invoice_type' => 1,
                        'effected_bill_id' => $data->local_bill_no,
                        'old_invoice_id' => $data->id 
                    ];

                    if (!is_null($data->discount) & floatval($data->discount) != 0) {
                        $discount_vals[$data->id] = 
                        [
                            $this->getItemDiscount($data->discount, $data->discount_type, 1, $data->id)
                            +
                            $this->getItemDiscount($data->vat_discount_value, $data->vat_discount_type, 1, $data->id), 
                            $data->discount_type
                        ];
                    }

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue('invoices', $null_col);
                    }

                    DB::table('invoices')->insert($assigned_data);
                    $inserted = true;

                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError('sale_process', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }

        // second step    
        $null_ids = $this->null_errors['sale_details'];
        $current_selection = DB::connection('old_db')->table('sale_details')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $invoice_id = Invoices::where([
                        ['shop_id' , get_session_val('serial_id', 'Integer')],
                        ['old_invoice_id', $data->bill_id],
                        ['invoice_type', 1]
                    ])->limit(1)->value('id');

                    $invoice_details = InvoiceDetails::where([
                        ['shop_id' , get_session_val('serial_id', 'Integer')],
                        ['invoice_id', $invoice_id],
                        ['item_id', $data->items_id]
                    ])->first();

                    if (! $invoice_details)
                    {
                        $assigned_data = [
                            'invoice_id' => $invoice_id,
                            'item_id' => $data->items_id,
                            'shop_id' => $data->shop_id,
                            'quantity' => $data->quantity,
                            'unit_id' => $data->unit,
                            'quantity_after' => $data->quant_after,
                            'pay_price' => $data->pay_price ?? 0,
                            'sale_price' => $data->price,
                            'total_price' => $data->total_price,
                            'basic_sale_price' => $data->basic_price ?? 0,
                            'basic_pay_price' => $data->pay_price ?? $data->price,
                            'glass_height' => $data->glass_height,
                            'glass_width' => $data->glass_width,
                            'glass_number' => $data->glass_number,
                            'item_about' => $data->about,
                            'item_notes' => $data->item_notes,
                            'not_sold' => 0,
                            'discount' => key_exists($data->sale_id, $discount_vals) ? $discount_vals[$data->sale_id][0] : 0,
                            'discount_type' => key_exists($data->sale_id, $discount_vals) ? $discount_vals[$data->sale_id][1] : 0,
                            'ac_pay_price' => $data->ac_pay_price,
                            'av_pay_price' => $data->av_pay_price,
                            'pay_vats_weight' => 0,
                            'sale_ratio' => $data->sale_ratio,
                            'ratio_type' => $data->ratio_type,
                            'bolla' => $data->bolla,
                            'sales_man' => $data->sale_man,
                            'safe_point_id' => $data->sale_point,
                            'store_id' =>$data->store_id,
                            'isservice' => $data->isservice,
                            'notify_alert' => $data->notify_alert,
                            'notify_time' => $data->notify_time,
                            'record_type' => $data->record_type,
                            'label_from' => $data->label_from,
                            'label_to' => $data->label_to,
                            'pricePercentage' => $data->pricePercentage,
                            'item_serial' => $data->item_serial,
                            'add_user' => $data->add_user,
                            'duration_type' => $data->duration_type,
                            'duration' => $data->duration,
                            'from_date' => $data->from_date,
                            'to_date' => $data->to_date,
                            'color_id' => $data->color_id,
                            'size_id' => $data->size_id,
                            'cards' => $data->cards,
                            'spare_id' => $data->spare_id,
                            'car_id' => $data->car_id,
                            'sales_row_id' => $data->sales_row_id,
                            'offer_id' => $data->offer_id,
                            'offer_text' => $data->offer_text,
                            'offer_type' => $data->offer_type,
                            'main_qty' => $data->main_qty,
                            'disc_qty' => $data->disc_qty,
                        ];

                        // loop on null cols
                        foreach ($temp_null_cols as $table_name => $null_col) {
                            if ($table_name == 'invoice_details')
                            {
                                $assigned_data[$null_col] = $this->getDefaultValue('invoice_details', $null_col);
                            }
                        }

                        DB::table('invoices_details')->insert($assigned_data);
                    }


                    $invoice_details_vat = InvoiceDetailsVats::where([
                        ['shop_id' , get_session_val('serial_id', 'Integer')],
                        ['invoice_id', $invoice_id],
                        ['item_id', $data->items_id]
                    ])->first();

                    if (! $invoice_details_vat)
                    {
                        $add_details = DB::connection('old_db')->table('bills_add_history')->where( [
                            ['bill_id', $data->bill_id],
                            ['type', 1],
                            ['shop_id' , get_session_val('serial_id', 'Integer')]
                            ])->first();
                        $item = DB::connection('old_db')->table('items')->where([ ['id', $data->items_id], ['shop_id' ,get_session_val('serial_id', 'Integer')]] )->first();

                        if (! is_null($add_details) & ! is_null($item))
                        {
                            $assigned_data = [
                                'invoice_id' => $invoice_id, 
                                'item_id' => $data->items_id,
                                'vat_id' => $add_details->addition_id,
                                'vat_type' => $add_details->add_type,
                                'vat_value' => $add_details->bill_add,
                                'vat_weight' => $this->getItemVats($data->item_vat_state, $add_details->bill_add, $add_details->add_type, $data->price),
                                'vat_operation' => 0,
                                'shop_id' => $data->shop_id,
                            ];

                            // loop on null cols
                            foreach ($temp_null_cols as $table_name => $null_col) {
                                if ($table_name == 'invoices_details_vats')
                                {
                                    $assigned_data[$null_col] = $this->getDefaultValue('invoices_details_vats', $null_col);
                                }
                            }

                            DB::table('invoices_details_vats')->insert($assigned_data);
                        }
                    }

                    $inserted = true;

                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    $table_name = Str::between($th->getMessage(), "insert into `", "` (`");

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols[$table_name], $check_result[1]);
                    }
                    else {
                        $this->transformerError('sale_details', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }
    }

    private function transformReturnedSalesInvoices()
    {
        $discount_vals = [];

        // step 1
        $max_min_ids = DB::connection('old_db')->table('sale_back_invoice')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        
        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('sale_back_invoice')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                try {
                    DB::table('invoices')->insert([
                        'invoice_date' => $data->sale_date, 
                        'bill_total' => $data->total_price,
                        'add_user' => $data->add_user,
                        'client_id' => ($data->client_id > 0) ? $this->getClientNewId(0, $data->client_id, $data, 'sale_back_invoice') : $data->client_id,
                        'net_price' => $data->net_price,
                        'payment' => $data->payment,
                        'bill_payment' => $data->bil_payment,
                        'rest' => $data->rest,
                        'store_id' => $data->store,
                        'bill_no' => $data->bill_no,
                        'notes' => $data->notes,
                        'shop_id' => $data->shop_id,
                        'balance' => $data->balance ?? $data->clientOldBalance + $data->payment,
                        'invoice_type' => 3,
                        'transaction_date' => $data->date_process,
                        'effected_bill_id' => $data->sale_id,
                        'boll_per' => $data->boll_per,
                        'sales_man' => $data->sales_man,
                        'pay_date' => $data->pay_date,
                        'pay_process' => $data->pay_process,
                        'notify_time' => $data->notify_time,
                        'notify_alert' => $data->notify_alert,
                        'cooking' => $data->cooking,
                        'insatllments_add' => $data->insatllments_add,
                        'old_invoice_id' => $data->id
                    ]);

                    if (!is_null($data->discount) & floatval($data->discount) != 0) {
                        /* 
                            0 => %, 1 => $
                        */
                        $discount_vals[$data->id] = [
                            $this->getItemDiscount($data->discount, $data->discount_type, 3, $data->id)
                            , 
                            $data->discount_type
                        ];
                    }
                } catch (\Throwable $th) {
                    $this->transformerError('sale_back_invoice', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }
        
        // step 2
        
        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_min_ids = DB::connection('old_db')->table('sale_back')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        $max_id = !is_null($max_min_ids) ? ($max_min_ids->max_id ?? 0) : 0;
        $temp_max_id = !is_null($max_min_ids) ? ($max_min_ids->min_id ?? 0) : 0;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }
        
        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('sale_back')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                $invoice_id = Invoices::where([
                    ['shop_id' , get_session_val('serial_id', 'Integer')],
                    ['old_invoice_id', $data->back_id],
                    ['invoice_type', 3]
                ])->limit(1)->value('id');

                try {

                    if ($invoice_id)
                    {   
                        DB::table('invoices_details')->insert([
                            'invoice_id' => $invoice_id,
                            'item_id' => $data->items_id,
                            'shop_id' => $data->shop_id,
                            'quantity' => $data->quantity,
                            'unit_id' => $data->unit,
                            'pay_price' => $data->pay_price ?? 0,
                            'sale_price' => $data->price ,
                            'total_price' => $data->price * $data->quantity,
                            'basic_sale_price' => $data->basic_price ?? 0,
                            'basic_pay_price' => $data->pay_price ?? 0,
                            'safe_point_id' => $data->sale_point,
                            'add_user' => $data->add_user,
                            'item_serial' => $data->item_serial,
                            'not_sold' => 0,
                            'discount' => key_exists($data->back_id, $discount_vals) ? $discount_vals[$data->back_id][0] : 0,
                            'discount_type' => key_exists($data->back_id, $discount_vals) ? $discount_vals[$data->back_id][1] : 0,
                            'ac_pay_price' => 0,
                            'av_pay_price' => 0,
                            'pay_vats_weight' => 0,
                        ]);
    
                    }
                    else {
                        $this->transformerError('sale_back', $data->id, 'invoice with old_invoice_id ( ' . $data->back_id. ' ) not exists');
                    }

                } catch (\Throwable $th) {
                    $this->transformerError('sale_back', $data->id, $th->getMessage());
                    continue;
                }

                try {
                    if ($invoice_id)
                    {
                        $add_details = DB::connection('old_db')->table('bills_add_history')->where( [
                            ['bill_id', $data->back_id],
                            ['type', 3],
                            ['shop_id' , get_session_val('serial_id', 'Integer')]
                            ])->first();
                        $item = DB::connection('old_db')->table('items')->where('id', $data->items_id)->first();
                        
                        if (! is_null($add_details) & ! is_null($item))
                        {
                            DB::table('invoices_details_vats')->insert([
                                'invoice_id' => $invoice_id, 
                                'item_id' => $data->items_id,
                                'vat_id' => $add_details->addition_id,
                                'vat_type' => $add_details->add_type,
                                'vat_value' => $add_details->bill_add,
                                'vat_weight' => $this->getItemVats($data->item_vat_state, $add_details->bill_add, $add_details->add_type, $data->price),
                                'vat_operation' => 0,
                                'shop_id' => $data->shop_id,
                            ]);
                        }
                        else {
                            if (is_null($add_details))
                            {
                                $this->transformerError('sale_back', $data->id, 'no bill_add_history record for this bill ');
                            }
                            if (is_null($item))
                            {
                                $this->transformerError('sale_back', $data->id, 'item with id ( '. $data->items_id .' ) is deleted ');
                            }
                        }
                    }


                } catch (\Throwable $th) {
                    $this->transformerError('sale_back', $data->id, $th->getMessage());
                    continue;
                }


            }
            $loop++;
        }

        if (array_key_exists('sale_back_invoice', $this->null_errors) || array_key_exists('sale_back', $this->null_errors))
        {
            $this->transformNullReturnedSalesInvoices();
        }
    }

    private function transformNullReturnedSalesInvoices()
    {
        $discount_vals = [];

        // first step    
        $null_ids = $this->null_errors['sale_back_invoice'];
        $current_selection = DB::connection('old_db')->table('sale_back_invoice')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $assigned_data = [
                        'invoice_date' => $data->sale_date, 
                        'bill_total' => $data->total_price,
                        'add_user' => $data->add_user,
                        'client_id' => ($data->client_id > 0) ? $this->getClientNewId(0, $data->client_id, $data, 'sale_back_invoice') : $data->client_id,
                        'net_price' => $data->net_price,
                        'payment' => $data->payment,
                        'bill_payment' => $data->bil_payment,
                        'rest' => $data->rest,
                        'store_id' => $data->store,
                        'bill_no' => $data->bill_no,
                        'notes' => $data->notes,
                        'shop_id' => $data->shop_id,
                        'balance' => $data->balance ?? $data->clientOldBalance + $data->payment,
                        'invoice_type' => 3,
                        'transaction_date' => $data->date_process,
                        'effected_bill_id' => $data->sale_id,
                        'boll_per' => $data->boll_per,
                        'sales_man' => $data->sales_man,
                        'pay_date' => $data->pay_date,
                        'pay_process' => $data->pay_process,
                        'notify_time' => $data->notify_time,
                        'notify_alert' => $data->notify_alert,
                        'cooking' => $data->cooking,
                        'insatllments_add' => $data->insatllments_add,
                        'old_invoice_id' => $data->id
                    ];

                    if (!is_null($data->discount) & floatval($data->discount) != 0) {
                        
                        $discount_vals[$data->id] = [
                            $this->getItemDiscount($data->discount, $data->discount_type, 3, $data->id)
                            , 
                            $data->discount_type
                        ];
                    }

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue('invoices', $null_col);
                    }

                    DB::table('invoices')->insert($assigned_data);
                    $inserted = true;

                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError('sale_back_invoice', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }

        // second step    
        $null_ids = $this->null_errors['sale_back'];
        $current_selection = DB::connection('old_db')->table('sale_back')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $invoice_id = Invoices::where([
                        ['shop_id' , get_session_val('serial_id', 'Integer')],
                        ['old_invoice_id', $data->bill_id],
                        ['invoice_type', 3]
                    ])->limit(1)->value('id');

                    $invoice_details = InvoiceDetails::where([
                        ['shop_id' , get_session_val('serial_id', 'Integer')],
                        ['invoice_id', $invoice_id],
                        ['item_id', $data->items_id]
                    ])->first();

                    if (! $invoice_details)
                    {
                        $assigned_data = [
                            'invoice_id' => $invoice_id,
                            'item_id' => $data->items_id,
                            'shop_id' => $data->shop_id,
                            'quantity' => $data->quantity,
                            'unit_id' => $data->unit,
                            'pay_price' => $data->pay_price ?? 0,
                            'sale_price' => $data->price ,
                            'total_price' => $data->price * $data->quantity,
                            'basic_sale_price' => $data->basic_price ?? 0,
                            'basic_pay_price' => $data->pay_price ?? 0,
                            'safe_point_id' => $data->sale_point,
                            'add_user' => $data->add_user,
                            'item_serial' => $data->item_serial,
                            'not_sold' => 0,
                            'discount' => key_exists($data->back_id, $discount_vals) ? $discount_vals[$data->back_id][0] : 0,
                            'discount_type' => key_exists($data->back_id, $discount_vals) ? $discount_vals[$data->back_id][1] : 0,
                            'ac_pay_price' => 0,
                            'av_pay_price' => 0,
                            'pay_vats_weight' => 0,
                        ];

                        // loop on null cols
                        foreach ($temp_null_cols as $table_name => $null_col) {
                            if ($table_name == 'invoice_details')
                            {
                                $assigned_data[$null_col] = $this->getDefaultValue('invoice_details', $null_col);
                            }
                        }

                        DB::table('invoices_details')->insert($assigned_data);
                    }


                    $invoice_details_vat = InvoiceDetailsVats::where([
                        ['shop_id' , get_session_val('serial_id', 'Integer')],
                        ['invoice_id', $invoice_id],
                        ['item_id', $data->items_id]
                    ])->first();

                    if (! $invoice_details_vat)
                    {
                        $add_details = DB::connection('old_db')->table('bills_add_history')->where( [
                            ['bill_id', $data->bill_id],
                            ['type', 3],
                            ['shop_id' , get_session_val('serial_id', 'Integer')]
                            ])->first();
                        $item = DB::connection('old_db')->table('items')->where([ ['id', $data->items_id], ['shop_id' ,get_session_val('serial_id', 'Integer')]] )->first();

                        if (! is_null($add_details) & ! is_null($item))
                        {
                            $assigned_data = [
                                'invoice_id' => $invoice_id, 
                                'item_id' => $data->items_id,
                                'vat_id' => $add_details->addition_id,
                                'vat_type' => $add_details->add_type,
                                'vat_value' => $add_details->bill_add,
                                'vat_weight' => $this->getItemVats($data->item_vat_state, $add_details->bill_add, $add_details->add_type, $data->price),
                                'vat_operation' => 0,
                                'shop_id' => $data->shop_id,
                            ];

                            // loop on null cols
                            foreach ($temp_null_cols as $table_name => $null_col) {
                                if ($table_name == 'invoices_details_vats')
                                {
                                    $assigned_data[$null_col] = $this->getDefaultValue('invoices_details_vats', $null_col);
                                }
                            }

                            DB::table('invoices_details_vats')->insert($assigned_data);
                        }
                    }

                    $inserted = true;

                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    $table_name = Str::between($th->getMessage(), "insert into `", "` (`");

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols[$table_name], $check_result[1]);
                    }
                    else {
                        $this->transformerError('sale_back', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }

    } 

    private function transformItemsTransactions()
    {
        // step 1
        $max_min_ids = DB::connection('old_db')->table('items_transaction')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();

        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('items_transaction')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                try {

                    DB::transaction(function () use($data){

                        $new_invoice = new Invoices();
                        $new_invoice_details = new InvoiceDetails();

                        $new_invoice->invoice_date = $data->action_date;
                        $new_invoice->add_user = $data->user_id;

                        if (!is_null($data->supplier_id) & $data->supplier_id > 0)
                        {
                            $new_invoice->client_id = ($data->supplier_id > 0) ? $this->getClientNewId(1, $data->supplier_id, $data, 'items_transaction') : $data->supplier_id;
                        }
                        
                        $new_invoice->store_id = $data->store_id;
                        $new_invoice->invoice_type = $this->getItemTransactionNewType($data->type);
                        $new_invoice->transaction_date = $data->tansaction_date;

                        if ( in_array($data->type, array(3, 32, 33)) )   // purchases
                        {
                            $new_invoice->effected_bill_id = ($this->getInvoiceNewId($data->type, $data->incom_id, 0))[1];

                            $new_invoice_details->pay_price = $data->price;
                            $new_invoice_details->basic_pay_price = $data->price ?? 0;
                        }
                        if ( in_array($data->type, array(1, 30, 31)) )   // sales
                        {
                            $new_invoice->effected_bill_id = ($this->getInvoiceNewId($data->type, $data->sale_id, 0))[1];

                            $new_invoice_details->sale_price = $data->price;
                            $new_invoice_details->basic_sale_price = $data->price ?? 0;

                            $new_invoice_details->pay_price = $data->pay_price;
                            $new_invoice_details->basic_pay_price = $data->pay_price ?? 0;
                        }
                        if ( $data->type == 5 )   // returned purchases
                        {
                            $new_invoice->effected_bill_id = ($this->getInvoiceNewId($data->type, 0, $data->incom_return_id))[1];

                            $new_invoice_details->pay_price = $data->price;
                            $new_invoice_details->basic_pay_price = $data->price ?? 0;

                            $new_invoice_details->sale_price = $data->price;
                            $new_invoice_details->basic_sale_price = $data->price ?? 0;

                            $new_invoice_details->pay_price = $data->pay_price;
                            $new_invoice_details->basic_pay_price = $data->pay_price ?? 0;
                        }
                        if ( in_array($data->type, array(0, 63, 64)) )   // returned sales
                        {
                            $new_invoice->effected_bill_id = ($this->getInvoiceNewId($data->type, 0, $data->back_id))[1];
                        }
                        if ( $data->type == 12)  // stores items transformation 
                        {
                            $new_invoice->effected_bill_id = $data->order_no;

                            $new_invoice_details->from_store = $data->store_from;
                            $new_invoice_details->to_store = $data->store_to;
                        }
                        if ( in_array($data->type, array(50, 51)) )   // manufacturing
                        {
                            $new_invoice->effected_bill_id = $data->manuf_order;
                        }
                        if ( in_array($data->type, array(100, 101)) )   // project
                        {
                            $new_invoice->effected_bill_id = $data->project_id;
                        }

                        $new_invoice->save();
                        
                        // invoice details info
                        $new_invoice_details->invoice_id = $new_invoice->id;
                        $new_invoice_details->shop_id = $data->shop_id;
                        $new_invoice->item_id = $data->item_id;
                        $new_invoice_details->store_id = $data->store_id;
                        $new_invoice_details->quantity = $data->quantity;
                        $new_invoice_details->quantity_after = $data->new_quantity;
                        $new_invoice_details->add_user = $data->user_id;
                        $new_invoice_details->replaced = $data->replaced;

                        $new_invoice_details->save();
                    });
                    
                } catch (\Throwable $th) {
                    $this->transformerError('items_transaction', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }

        if (array_key_exists('items_transaction', $this->null_errors))
        {
            $this->transformNullItemsTransactions();
        }
    }

    private function transformNullItemsTransactions()
    {

        $null_ids = $this->null_errors['items_transaction'];
        $current_selection = DB::connection('old_db')->table('items_transaction')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    DB::transaction(function () use($data, &$inserted, $temp_null_cols){

                        $new_invoice = new Invoices();
                        $new_invoice_details = new InvoiceDetails();

                        $new_invoice->invoice_date = $data->action_date;
                        $new_invoice->add_user = $data->user_id;

                        if (!is_null($data->supplier_id) & $data->supplier_id > 0)
                        {
                            $new_invoice->client_id = ($data->supplier_id > 0) ? $this->getClientNewId(1, $data->supplier_id, $data, 'items_transaction') : $data->supplier_id;
                        }
                        
                        $new_invoice->store_id = $data->store_id;
                        $new_invoice->invoice_type = $this->getItemTransactionNewType($data->type);
                        $new_invoice->transaction_date = $data->tansaction_date;

                        if ( in_array($data->type, array(3, 32, 33)) )   // purchases
                        {
                            $new_invoice->effected_bill_id = ($this->getInvoiceNewId($data->type, $data->incom_id, 0))[1];

                            $new_invoice_details->pay_price = $data->price;
                            $new_invoice_details->basic_pay_price = $data->price ?? 0;
                        }
                        if ( in_array($data->type, array(1, 30, 31)) )   // sales
                        {
                            $new_invoice->effected_bill_id = ($this->getInvoiceNewId($data->type, $data->sale_id, 0))[1];

                            $new_invoice_details->sale_price = $data->price;
                            $new_invoice_details->basic_sale_price = $data->price ?? 0;

                            $new_invoice_details->pay_price = $data->pay_price;
                            $new_invoice_details->basic_pay_price = $data->pay_price ?? 0;
                        }
                        if ( $data->type == 5 )   // returned purchases
                        {
                            $new_invoice->effected_bill_id = ($this->getInvoiceNewId($data->type, 0, $data->incom_return_id))[1];

                            $new_invoice_details->pay_price = $data->price;
                            $new_invoice_details->basic_pay_price = $data->price ?? 0;

                            $new_invoice_details->sale_price = $data->price;
                            $new_invoice_details->basic_sale_price = $data->price ?? 0;

                            $new_invoice_details->pay_price = $data->pay_price;
                            $new_invoice_details->basic_pay_price = $data->pay_price ?? 0;
                        }
                        if ( in_array($data->type, array(0, 63, 64)) )   // returned sales
                        {
                            $new_invoice->effected_bill_id = ($this->getInvoiceNewId($data->type, 0, $data->back_id))[1];
                        }
                        if ( $data->type == 12)  // stores items transformation 
                        {
                            $new_invoice->effected_bill_id = $data->order_no;

                            $new_invoice_details->from_store = $data->store_from;
                            $new_invoice_details->to_store = $data->store_to;
                        }
                        if ( in_array($data->type, array(50, 51)) )   // manufacturing
                        {
                            $new_invoice->effected_bill_id = $data->manuf_order;
                        }
                        if ( in_array($data->type, array(100, 101)) )   // project
                        {
                            $new_invoice->effected_bill_id = $data->project_id;
                        }

                        $new_invoice_details->shop_id = $data->shop_id;
                        $new_invoice->item_id = $data->item_id;
                        $new_invoice_details->store_id = $data->store_id;
                        $new_invoice_details->quantity = $data->quantity;
                        $new_invoice_details->quantity_after = $data->new_quantity;
                        $new_invoice_details->add_user = $data->user_id;
                        $new_invoice_details->replaced = $data->replaced;

                        foreach ($temp_null_cols as $table_name => $null_col) {
                            if ($table_name == 'invoices')
                            {
                                $new_invoice[$null_col] = $this->getDefaultValue('invoices', $null_col);
                            }
                            else if ($table_name == 'invoices_details')
                            {
                                $new_invoice_details[$null_col] = $this->getDefaultValue('invoices_details', $null_col);
                            }
                        }

                        $new_invoice->save();
                        
                        $new_invoice_details->invoice_id = $new_invoice->id;
                        $new_invoice_details->save();
                        
                        $inserted = true;
                    });

                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    $table_name = Str::between($th->getMessage(), "insert into `", "` (`");
                    
                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols[$table_name], $check_result[1]);
                    }
                    else {
                        $this->transformerError('items_transaction', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }
    }

    private function transformClientsTransactions()
    {
        // step 1
        $max_min_ids = DB::connection('old_db')->table('client_transaction')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        
        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('client_transaction')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();
            
            foreach ($current_selection as $data) {
                 
                try {
                    $new_client_transaction = new ClientTransaction();
                    $assigned_data = (array)$data;

                    $bill_id_status = !(is_null($data->bill_id)) & ($data->bill_id > 0);
                    $sale_back_id_status = !(is_null($data->sale_back_id)) & ($data->sale_back_id > 0);

                    if ( $bill_id_status || $sale_back_id_status)
                    {
                        if ($data->type == 5)
                        {
                            [$column_name, $new_id] = $this->getInvoiceNewId(5, $data->bill_id, $data->bill_id);
                            $column_name = 'bill_id';
                        }
                        else {
                            [$column_name, $new_id] = $this->getInvoiceNewId($data->type, $data->bill_id, $data->sale_back_id);
                        }

                        if ($new_id)
                        {
                            if ($column_name == 'bill_id')
                            {
                                $assigned_data['bill_id'] = $new_id;
                            }
                            else {
                                $assigned_data['sale_back_id'] = $new_id;
                            }                    

                        }
                        else {
                            $this->transformerError('client_transaction', $data->id, 'Invoice with old_id ( '. $data->bill_id .' or '. $data->sale_back_id .' ) and transaction_type = '. $data->type .' is deleted ');
                            continue;
                        }
                    }

                    if (!(is_null($data->client_id)) & ($data->client_id > 0))
                    {
                        $assigned_data['client_id'] = $this->getClientNewId(0, $data->client_id, $data, 'client_transaction');
                    }

                    if (!(is_null($data->supplier_id)) & ($data->supplier_id > 0))
                    {
                        $assigned_data['supplier_id'] = ($data->supplier_id > 0) ? $this->getClientNewId(1, $data->supplier_id, $data, 'client_transaction') : $data->supplier_id;
                    }

                    unset($assigned_data['id']);    // can be removed
                    $new_client_transaction->fill($assigned_data);
                    $new_client_transaction->save();

                } catch (\Throwable $th) {
                    $this->transformerError('client_transaction', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }

        if (array_key_exists('client_transaction', $this->null_errors))
        {
            $this->transformNullClientsTransactions();
        }
    }

    private function transformNullClientsTransactions()
    {
        $null_ids = $this->null_errors['client_transaction'];
        $current_selection = DB::connection('old_db')->table('client_transaction')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $new_client_transaction = new ClientTransaction();
                    $assigned_data = (array)$data;

                    $bill_id_status = !(is_null($data->bill_id)) & ($data->bill_id > 0);
                    $sale_back_id_status = !(is_null($data->sale_back_id)) & ($data->sale_back_id > 0);

                    if ( $bill_id_status || $sale_back_id_status)
                    {
                        if ($data->type == 5)
                        {
                            [$column_name, $new_id] = $this->getInvoiceNewId(5, $data->bill_id, $data->bill_id);
                            $column_name = 'bill_id';
                        }
                        else {
                            [$column_name, $new_id] = $this->getInvoiceNewId($data->type, $data->bill_id, $data->sale_back_id);
                        }

                        if ($column_name == 'bill_id')
                        {
                            $assigned_data['bill_id'] = $new_id;
                        }
                        else {
                            $assigned_data['sale_back_id'] = $new_id;
                        }                    
                    }

                    if (!(is_null($data->client_id)) & ($data->client_id > 0))
                    {
                        $assigned_data['client_id'] = $this->getClientNewId(0, $data->client_id, $data, 'client_transaction');
                    }

                    if (!(is_null($data->supplier_id)) & ($data->supplier_id > 0))
                    {
                        $assigned_data['supplier_id'] = ($data->supplier_id > 0) ? $this->getClientNewId(1, $data->supplier_id, $data, 'client_transaction') : $data->supplier_id;
                    }

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue('client_transaction', $null_col);
                    }

                    unset($assigned_data['id']);
                    $new_client_transaction->fill($assigned_data);
                    $new_client_transaction->save();

                    $inserted = true;
                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError('client_transaction', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }
    }

    private function transformBillsAddHistory()
    {
        // step 1
        $max_min_ids = DB::connection('old_db')->table('bills_add_history')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();

        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('bills_add_history')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                try {
                    
                    if ($data->bill_id)
                    {
                        continue;
                    }

                    $new_bill_add_history = new BillAddHistory();
                    $assigned_data = (array)$data;
                    // unset($assigned_data['id']);
                    $new_bill_add_history->fill($assigned_data);
                    $new_bill_add_history->save();

                } catch (\Throwable $th) {
                    $this->transformerError('bills_add_history', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }
        
        if (array_key_exists('bills_add_history', $this->null_errors))
        {
            $this->transformNullBillsAddHistory();
        }
    }
    
    private function transformNullBillsAddHistory()
    {
        $null_ids = $this->null_errors['bills_add_history'];
        $current_selection = DB::connection('old_db')->table('bills_add_history')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $new_bill_add_history = new BillAddHistory();
                    $assigned_data = (array)$data;

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue('bills_add_history', $null_col);
                    }

                    $new_bill_add_history->fill($assigned_data);
                    $new_bill_add_history->save();

                    $inserted = true;
                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError('bills_add_history', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }
    }

    private function transformBillImages()  // needs sale_process transforming 
    {
        $this->transformNeedsClientAndInvoice('bill_images', null, 1, 'bill_id');
    }

    private function transformCardNumbers() // needs sale_process, incoming_bill transforming
    {
        // step 1
        $max_min_ids = DB::connection('old_db')->table('card_numbers')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();

        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('card_numbers')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                try {
                    $new_card_number = new CardNumber();
                    $assigned_data = (array)$data;

                    $new_sale_invoice_id = $this->getInvoiceNewId(1, $data->sale_id, 0)[1];
                    $new_purchase_invoice_id = $this->getInvoiceNewId(3, $data->income_id, 0)[1];

                    $escape = false;

                    if (is_null($new_sale_invoice_id))
                    {
                        $this->transformerError('card_numbers', $data->id, 'Invoice with old_id ( '. $data->sale_id .' ) and transaction_type = 1 is deleted ');
                        $escape = true;
                    }

                    if (is_null($new_purchase_invoice_id))
                    {
                        $this->transformerError('card_numbers', $data->id, 'Invoice with old_id ( '. $data->income_id .' ) and transaction_type = 3 is deleted ');
                        $escape = true;
                    }

                    if ($escape)
                    {
                        continue;
                    }

                    $assigned_data['sale_id'] = $new_sale_invoice_id;
                    $assigned_data['income_id'] = $new_purchase_invoice_id;

                    // unset($assigned_data['id']);
                    $new_card_number->fill($assigned_data);
                    $new_card_number->save();
                } catch (\Throwable $th) {
                    $this->transformerError('card_numbers', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }

        if (array_key_exists('card_numbers', $this->null_errors))
        {
            $this->transformNullCardNumbers();
        }
    }

    private function transformNullCardNumbers() 
    {
        $null_ids = $this->null_errors['card_numbers'];
        $current_selection = DB::connection('old_db')->table('card_numbers')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $new_card_number = new CardNumber();
                    $assigned_data = (array)$data;

                    $new_sale_invoice_id = $this->getInvoiceNewId(1, $data->sale_id, 0)[1];
                    $new_purchase_invoice_id = $this->getInvoiceNewId(3, $data->income_id, 0)[1];

                    $assigned_data['sale_id'] = $new_sale_invoice_id;
                    $assigned_data['income_id'] = $new_purchase_invoice_id;

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue('card_numbers', $null_col);
                    }

                    $new_card_number->fill($assigned_data);
                    $new_card_number->save();

                    $inserted = true;
                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError('card_numbers', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }
    }

    private function transformCars()    // needs clients transforming
    {
        $this->transformNeedsClientAndInvoice('cars', 0);
    }

    private function transformChargeOrderBills()    // needs sale_process transforming
    {
        $this->transformNeedsClientAndInvoice('charge_order_bills', null, 1, 'bill_id');
    }

    private function transformClientGlasses()    // needs clients transforming
    {
        $this->transformNeedsClientAndInvoice('client_glasses', 0);
    }

    private function transformClientImages()    // needs clients transforming
    {
        $this->transformNeedsClientAndInvoice('client_images', 0);
    }

    private function transformClientLocations()    // needs clients transforming
    {
        $this->transformNeedsClientAndInvoice('client_locations', 0);
    }

    private function transformConvertMoney()    // needs supplier transforming
    {
        $this->transformNeedsClientAndInvoice('convert_money', 1);
    }

    private function transformDetailQuantities()    // needs incoming_bill transforming
    {
        $this->transformNeedsClientAndInvoice('detail_quantities', null, 3, 'bill_id');
    }

    private function transformDiscountCard()    // needs sale_process transforming
    {
        // step 1
        $max_min_ids = DB::connection('old_db')->table('discount_cards')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();

        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('discount_cards')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                try {
                    $new_discount_card = new DiscountCard();
                    $assigned_data = (array)$data;
                    
                    $new_bill_used_id = $this->getInvoiceNewId(1, $data->bill_id_used, 0)[1];
                    $new_bill_sell_id = $this->getInvoiceNewId(1, $data->bill_id_sell, 0)[1];

                    $escape = false;

                    if (is_null($new_bill_used_id))
                    {
                        $this->transformerError('discount_cards', $data->id, 'Invoice with old_id ( '. $data->bill_id_used .' ) and transaction_type = 1 is deleted ');
                        $escape = true;
                    }

                    if (is_null($new_bill_sell_id))
                    {
                        $this->transformerError('discount_cards', $data->id, 'Invoice with old_id ( '. $data->bill_id_sell .' ) and transaction_type = 1 is deleted ');
                        $escape = true;
                    }

                    if ($escape)
                    {
                        continue;
                    }

                    // unset($assigned_data['id']);
                    $assigned_data['bill_id_used'] = $new_bill_used_id;
                    $assigned_data['bill_id_sell'] = $new_bill_sell_id;

                    $new_discount_card->fill($assigned_data);
                    $new_discount_card->save();
                } catch (\Throwable $th) {
                    $this->transformerError('discount_cards', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }
        
        if (array_key_exists('discount_cards', $this->null_errors))
        {
            $this->transformNullDiscountCard();
        }
    }

    private function transformNullDiscountCard()
    {
        $null_ids = $this->null_errors['discount_cards'];
        $current_selection = DB::connection('old_db')->table('discount_cards')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $new_discount_card = new DiscountCard();
                    $assigned_data = (array)$data;
                    
                    $new_bill_used_id = $this->getInvoiceNewId(1, $data->bill_id_used, 0)[1];
                    $new_bill_sell_id = $this->getInvoiceNewId(1, $data->bill_id_sell, 0)[1];

                    $assigned_data['bill_id_used'] = $new_bill_used_id;
                    $assigned_data['bill_id_sell'] = $new_bill_sell_id;

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue('discount_cards', $null_col);
                    }

                    $new_discount_card->fill($assigned_data);
                    $new_discount_card->save();

                    $inserted = true;
                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError('discount_cards', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }
        
    }

    private function transformFavorites()   // needs clients transforming
    {
        $this->transformNeedsClientAndInvoice('favorites', 0);
    }

    private function transformGiftHistory()     // needs clients, sale_process transforming
    {
        $this->transformNeedsClientAndInvoice('gift_history', 0, 1, 'bill_id');
    }
    
    private function transformInstallmentsList()    // needs clients, sale_process transforming
    {
        $this->transformNeedsClientAndInvoice('installments_list', 0, 1, 'bill_id');
    }

    private function transformItemRate()   // needs clients transforming
    {
        $this->transformNeedsClientAndInvoice('item_rates', 0);
    }

    private function transformItemSerials()    // needs sale_process, incoming_bill transforming
    {
        // step 1
        $max_min_ids = DB::connection('old_db')->table('item_serials')->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX(id) AS max_id, MIN(id) AS min_id'))->first();
        
        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);
        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table('item_serials')->where([ ['id', '<=', ($loop *  $this->chunk)], ['id', '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {
                 
                try {
                    $new_item_serial = new ItemSerial();
                    $assigned_data = (array)$data;
                    
                    if ($data->sale_id)
                    {
                        $new_sale_id = $this->getInvoiceNewId(1, $data->sale_id, 0)[1];
                    }

                    if ($data->income_id)
                    {
                        $new_income_id = $this->getInvoiceNewId(3, $data->income_id, 0)[1];
                    }


                    $escape = false;

                    if (is_null($new_sale_id))
                    {
                        $this->transformerError('item_serials', $data->id, 'Invoice with old_id ( '. $data->sale_id .' ) and transaction_type = 1 is deleted ');
                        $escape = true;
                    }

                    if (is_null($new_income_id))
                    {
                        $this->transformerError('item_serials', $data->id, 'Invoice with old_id ( '. $data->income_id .' ) and transaction_type = 3 is deleted ');
                        $escape = true;
                    }

                    if ($escape)
                    {
                        continue;
                    }

                    // unset($assigned_data['id']);
                    $assigned_data['sale_id'] = $new_sale_id;
                    $assigned_data['income_id'] = $new_income_id;

                    $new_item_serial->fill($assigned_data);
                    $new_item_serial->save();
                } catch (\Throwable $th) {
                    $this->transformerError('item_serials', $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }
        
        if (array_key_exists('item_serials', $this->null_errors))
        {
            $this->transformNullItemSerials();
        }
    }

    private function transformNullItemSerials()
    {

        $null_ids = $this->null_errors['item_serials'];
        $current_selection = DB::connection('old_db')->table('item_serials')->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn('id', array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {

                    $new_item_serial = new ItemSerial();
                    $assigned_data = (array)$data;
                    
                    if ($data->sale_id)
                    {
                        $new_sale_id = $this->getInvoiceNewId(1, $data->sale_id, 0)[1];
                    }

                    if ($data->income_id)
                    {
                        $new_income_id = $this->getInvoiceNewId(3, $data->income_id, 0)[1];
                    }

                    // unset($assigned_data['id']);
                    $assigned_data['sale_id'] = $new_sale_id;
                    $assigned_data['income_id'] = $new_income_id;

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue('item_serials', $null_col);
                    }

                    $new_item_serial->fill($assigned_data);
                    $new_item_serial->save();

                    $inserted = true;
                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError('item_serials', $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }
    }

    private function transformLogSystem()    // needs sale_process, sale_back_invoice, incoming_bill, incoming_bill_return, transforming
    {
        $this->transformNeedsClientAndInvoice('log_system', null, -1, 'bill_id');
    }

    private function transformManufacturingOrder()    // needs sale_process transforming
    {
        $this->transformNeedsClientAndInvoice('manufacturing_order', null, 1, 'bill_id');
    }

    private function transformMultiSupplier()   // needs supplier transforming
    {
        $this->transformNeedsClientAndInvoice('multi_supplier', 1);
    }

    private function transformOfferProcess()   // needs client transforming
    {
        $this->transformNeedsClientAndInvoice('offer_process', 0);
    }

    private function transformOutstoreorders()   // needs client, sale_process transforming
    {
        $this->transformNeedsClientAndInvoice('outstoreorders', 0, 1, 'bill_id');
    }

    private function transformOutstoreSetting()   // needs supplier transforming
    {
        $this->transformNeedsClientAndInvoice('outstoresetting', 1, null, null, 'shop_id');
    }

    private function transformRequests()   // needs client, sale_process transforming
    {
        $this->transformNeedsClientAndInvoice('requests', 0, 1, 'bill_id');
    }

    private function transformSmsHistory() // needs client transforming
    {
        $this->transformNeedsClientAndInvoice('sms_history', 0);
    }

    private function transformSubscriptionDates()   // needs client, sale_process transforming
    {
        $this->transformNeedsClientAndInvoice('subscription_dates', 0, 1, 'sale_id');
    }

    private function transformSupplierOrders() // needs supplier, incoming_bill transforming
    {
        $this->transformNeedsClientAndInvoice('supplier_orders', 1, 3, 'income_id');
    }

    private function transformTypesCookingSales()    // needs sale_process transforming
    {
        $this->transformNeedsClientAndInvoice('types_cooking_sales', null, 1, 'sale_details_id');
    }

    private function transformVisits() // needs client transforming
    {
        $this->transformNeedsClientAndInvoice('visits', 0);
    }

    private function transformClientGroups()    // needs clients transforming
    {
        $this->transformNeedsClientAndInvoice('clients_groups', 0);
    }

    private function transformNeedsClientAndInvoice($table_name, $client_role = null, $invoice_trans_no = null, $invoice_foreign_name = null, $id = 'id')
    {
        try {
            $max_min_ids = DB::connection('old_db')->table($table_name)->where('shop_id' ,get_session_val('serial_id', 'Integer'))->select(DB::raw('MAX('. $id .') AS max_id, MIN('. $id .') AS min_id'))->first();
        } catch (\Throwable $th) {
            $this->transformerError($table_name, null, $th->getMessage());
            return;
        }

        if (is_null($max_min_ids->max_id))
        {
            return;
        }

        $max_id = $max_min_ids->max_id;
        $temp_max_id = $max_min_ids->min_id;
        $loop = intval($temp_max_id /  $this->chunk);

        if (! $loop)
        {
            $loop = 1;
        }

        while (($max_id +  $this->chunk) >= ($loop *  $this->chunk)) {
            $current_selection = DB::connection('old_db')->table($table_name)->where([ [$id, '<=', ($loop *  $this->chunk)], [$id, '>', (($loop - 1) *  $this->chunk)], ['shop_id' , get_session_val('serial_id', 'Integer')]])
                ->limit( $this->chunk)->get();

            foreach ($current_selection as $data) {

                try {
                    $assigned_data = (array)$data;

                    if (! is_null($client_role))
                    {
                        $client_foreign_name = $client_role == 1 ? 'supplier_id' : 'client_id';
                        
                        if ($assigned_data[$client_foreign_name])
                        {
                            $assigned_data[$client_foreign_name] = ($assigned_data[$client_foreign_name] > 0) ? $this->getClientNewId($client_role, $assigned_data[$client_foreign_name], $data, $table_name) : $assigned_data[$client_foreign_name];
                        }
                    }

                    if ($invoice_trans_no)
                    {
                        if ($assigned_data[$invoice_foreign_name])
                        {

                            if ($invoice_trans_no == -1)
                            {
                                $invoice_trans_no = $this->getLogTransactionNo($data->type);
                            }

                            $new_invoice_id = $this->getInvoiceNewId($invoice_trans_no, $assigned_data[$invoice_foreign_name], $assigned_data[$invoice_foreign_name])[1];

                            if (is_null($new_invoice_id))
                            {
                                $this->transformerError($table_name, $data->id, 'Invoice with old_id ( '. $assigned_data[$invoice_foreign_name] .' ) and transaction_type = '. $invoice_trans_no .' is deleted ');
                                continue;
                            }

                            $assigned_data[$invoice_foreign_name] = $new_invoice_id;
                        }

                    }

                    DB::connection()->table($table_name)->insert($assigned_data);

                } catch (\Throwable $th) {
                    $this->transformerError($table_name, $data->id, $th->getMessage());
                    continue;
                }
            }
            $loop++;
        }

        // restore null errors
        if (array_key_exists($table_name, $this->null_errors))
        {
            $this->transformNullRecordNeedsClientAndInvoice($table_name, $client_role, $invoice_trans_no, $invoice_foreign_name, $id );
        }
    }

    private function transformNullRecordNeedsClientAndInvoice($table_name, $client_role = null, $invoice_trans_no = null, $invoice_foreign_name = null, $id = 'id')
    {
        $null_ids = $this->null_errors[$table_name];

        $current_selection = DB::connection('old_db')->table($table_name)->where('shop_id' , get_session_val('serial_id', 'Integer'))->whereIn($id, array_keys($null_ids))->get();

        foreach ($current_selection as $data) {

            $temp_null_cols = [$null_ids[$data->id]];
            $inserted = false;

            while(! $inserted)
            {
                try {
                    $assigned_data = (array)$data;
    
                    if (! is_null($client_role))
                    {
                        $client_foreign_name = $client_role == 1 ? 'supplier_id' : 'client_id';
                        
                        if ($assigned_data[$client_foreign_name])
                        {
                            $assigned_data[$client_foreign_name] = ($assigned_data[$client_foreign_name] > 0) ? $this->getClientNewId($client_role, $assigned_data[$client_foreign_name], $data, $table_name) : $assigned_data[$client_foreign_name];
                        }
                    }
    
                    if ($invoice_trans_no)
                    {
                        if ($assigned_data[$invoice_foreign_name])
                        {
    
                            if ($invoice_trans_no == -1)
                            {
                                $invoice_trans_no = $this->getLogTransactionNo($data->type);
                            }
    
                            $assigned_data[$invoice_foreign_name] = $this->getInvoiceNewId($invoice_trans_no, $assigned_data[$invoice_foreign_name], $assigned_data[$invoice_foreign_name])[1];
                        }
    
                    }

                    // loop on null cols
                    foreach ($temp_null_cols as $null_col) {
                        $assigned_data[$null_col] = $this->getDefaultValue($table_name, $null_col);
                    }
                    
                    DB::connection()->table($table_name)->insert($assigned_data);
                    $inserted = true;
                } catch (\Throwable $th) {
                    $check_result = $this->isNullException($th->getMessage());

                    if ($check_result[0] == true)
                    {
                        array_push($temp_null_cols, $check_result[1]);
                    }
                    else {
                        $this->transformerError($table_name, $data->id, $th->getMessage());
                        $inserted = true;
                    }
                }
            }
        }

    }

    private function getItemDiscount($invoice_total_discount, $invoice_discount_type, $invoice_type, $invoice_id)
    {
        $table_name = $column_name = null;
        switch ($invoice_type) {
            case 0:
                $table_name = 'incoming_details';
                $column_name = 'bill_id';
                break;
            case 1:
                $table_name = 'sale_details';
                $column_name = 'sale_id';
                break;
            case 2:
                $table_name = 'incoming_details_return';
                $column_name = 'bill_id';
                break;
            default:
                $table_name = 'sale_back';
                $column_name = 'back_id';
                break;
        }

        if ($invoice_discount_type == 1) {
            $total_qty_regardless_unit = DB::connection('old_db')->table($table_name)->where($column_name, $invoice_id)->sum('quantity');
            if (! $total_qty_regardless_unit)
                $total_qty_regardless_unit = 1;
            return $invoice_total_discount / $total_qty_regardless_unit;
        }

        return $invoice_total_discount;
    }

    private function getItemVats($vat_state, $vat_value, $vat_type, $item_price)
    {
        if ($vat_state == 1)
        {
            if ($vat_type == 0)
            {
                return ( $item_price * ( 1 + (floatval($vat_value) / 100)) ) - $item_price;
            }
            else {
                return $vat_value;
            }            
        }
        else if ($vat_state == 2)
        {
            if ($vat_type == 0)
            {
                return $item_price - ( $item_price / ( 1 + (floatval($vat_value) / 100)) );
            }
            else {
                return $item_price - $vat_value;
            }  
        }
        else {
            return 0;
        }

    }

    private function transformerError($table_name, $id, $msg)
    {
        if (strpos( $msg, 'cannot be null'))
        {
            $this->null_errors[$table_name][$id] = Str::between($msg, "1048 Column '", "' cannot be null");
        }

        $transformer_file = fopen($this->file_path, 'a');

        $error_details = json_encode([
            'table' => $table_name,
            'id' => $id,
            'msg' => $msg
        ]);

        fwrite($transformer_file, $error_details);
        fwrite($transformer_file, '|||');
        fclose($transformer_file);
    }

    private function getClientNewId($client_role, $old_id, $data, $table_name)
    {
        $client_id = Client::where([
            ['shop_id', get_session_val('serial_id', 'Integer')],
            ['client_role', $client_role],
            ['old_id', $old_id]
        ])->limit(1)->value('id') ?? -1;

        $role_type = $client_role == 0 ? 'client' : 'supplier';

        if ($client_id == -1)
        {
            $this->transformerError($table_name, $data->id, $role_type . ' with old_id ( ' . $old_id. ' ) not exists');
        }

        return $client_id;
    }

    private function getInvoiceNewId($transaction_type, $bill_id, $returned_bill_id)
    {
        $column_name = 'bill_id';
        $new_id = null;

        if ( (is_null($bill_id) || $bill_id == 0) & (is_null($returned_bill_id) || $returned_bill_id == 0) )
        {
            return [$column_name, $new_id];
        }

        switch ($transaction_type) {
            // purchases
            case '3':    
            case '6':    
            case '7':
            case '32':    
            case '33':
            case '70':       
                $new_id = Invoices::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoice_type', 0],
                        ['old_invoice_id', $bill_id]
                ])->limit(1)->value('id');
                break;

            // sales
            case '1':    
            case '2':    
            case '9':
            case '30':    
            case '31':
            case '34':    
            case '99':       
                $new_id = Invoices::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoice_type', 1],
                        ['old_invoice_id', $bill_id]
                ])->limit(1)->value('id');
                break;

            // returned purchases
            case '5':        
                $new_id = Invoices::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoice_type', 2],
                        ['old_invoice_id', $returned_bill_id]
                ])->limit(1)->value('id');
                $column_name = 'sale_back_id';
                break;
            
            // returned sales
            // cases [0, 63, 64]
            default:
                $new_id = Invoices::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoice_type', 3],
                        ['old_invoice_id', $returned_bill_id]
                ])->limit(1)->value('id');
                $column_name = 'sale_back_id';
                break;
        }

        return [$column_name, $new_id];
    }

    private function getItemTransactionNewType($old_type)
    {
        $types = [
            '0' => 4,       // returned sales, increase quantity
            '1' => 5,       // sales, decrease quantity
            '3' => 6,       // purchases, increase quantity
            '4' => 7,       // new item
            '5' => 8,       // returned purchases, decrease quantity
            '8' => 9,       // change quantity in item edit
            '12' => 10,     // stores transformation
            '22' => 11,     // item deleted
            '30' => 12,     // sales, increase quantity
            '31' => 13,     // sales, decrease quantity
            '32' => 14,     // purchases, increase quantity
            '33' => 15,     // purchases, decrease quantity
            '50' => 16,     // manufacturing
            '51' => 17,     // manufacturing
            '62' => 18,     // perish
            '63' => 19,     // returned sales, decrease quantity
            '64' => 20,     // returned sales, decrease quantity
            '100' => 21,    // project
            '101' => 22,    // project
            '88' => 23,     // perish with replaced = 1
        ];

        return $types[$old_type] ?? null;
    }

    private function getLogTransactionNo($string_type)
    {
        $int_types = [
            'purchase' => 3,
            'sales' => 1,
            'offer-sales' => 1,
            'back_purchase' => 5,
            'back_sales' => 0,
        ];

        return $int_types[$string_type] ?? -1;
    }

    private function getDefaultValue($table_name, $column_name)
    {

        if (isset($this->default_cols_vals[$table_name][$column_name]))
        {
            return $this->default_cols_vals[$table_name][$column_name];
        }

        $db_name = DB::connection()->getDatabaseName();
        $result = DB::select(DB::raw("
                SELECT Column_Default, DATA_TYPE FROM Information_Schema.Columns 
                WHERE Table_Schema = '". $db_name ."'
                AND Table_Name = '". $table_name ."'
                AND Column_Name = '". $column_name ."'
            "));
        if ($result)
        {

            if ($result[0]->Column_Default == 'NULL' || $result[0]->Column_Default == null)
            {

                if ($result[0]->DATA_TYPE == 'NULL')
                {
                    $this->default_cols_vals[$table_name][$column_name] = null;
                    return null;
                }
                else {
                    switch ($result[0]->DATA_TYPE) {
                        case 'int':
                        case 'tinyint':
                        case 'decimal':
                        case 'double':
                            $this->default_cols_vals[$table_name][$column_name] = 0;
                            return 0;
                        case 'text':
                        case 'varchar':
                            $this->default_cols_vals[$table_name][$column_name] = ' ';
                            return ' ';
                        case 'date':
                            $this->default_cols_vals[$table_name][$column_name] = date('Y-m-d');
                            return date('Y-m-d');
                        case 'timestamp':
                        case 'datetime':
                            $this->default_cols_vals[$table_name][$column_name] = date('Y-m-d') . ' ' . date('H:i:s');
                            return date('Y-m-d') . ' ' . date('H:i:s');
                        default:
                            $this->default_cols_vals[$table_name][$column_name] = null;
                            return null;
                    }
                }
            }
            else {
                $this->default_cols_vals[$table_name][$column_name] = $result[0]->Column_Default;
                return $result[0]->Column_Default;
            }
        }
    }

    private function isNullException($original_msg)
    {
        $start = "SQLSTATE[23000]: Integrity constraint violation: 1048 Column '";
        $col_end = "' cannot be null";
        $pos = strpos($original_msg, $col_end);

        if ($pos === false)
        {
            return [false];
        }
        else {
            $col_name = Str::between($original_msg, $start, $col_end);
            return [true, $col_name];
        }
    }

    public function prepareLogFile()
    {
        $myfile = fopen($this->file_path, "w");
        fclose($myfile);
    }

    public function showErrors()
    {
        $log_errors = [];

        $handler = fopen($this->file_path, 'r');
        if ($handler) {
            while (($line = fgets($handler)) !== false) {
                $splited = explode('|||', $line);
                foreach($splited as $error)
                {
                    if ($error)
                        array_push($log_errors, json_decode($error));
                }
            }
            fclose($handler);
        }
        return view('transformer::errors', compact('log_errors'));
    }

}
