<?php

namespace Modules\User\Http\Controllers;

use App\Models\systemLog;
use App\Models\User;
use App\Services\BCMS\EmployeeService;
use Carbon\Carbon;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Http;
use Modules\Invoices\Entities\ClientGroup;
use Modules\Store\Entities\Store;
use Modules\User\Entities\Category;
use Modules\User\Entities\CategoryUser;
use Modules\User\Entities\SalePoint;
use Modules\User\Entities\Log;

use Modules\User\Entities\UserGroup;
use Modules\User\Http\Requests\CreateUserRequest;
use Modules\User\Http\Requests\UpdateUserRequest;


class UserController extends Controller
{
    private $permissions = [];
    private $employeeService;
    public function __construct(EmployeeService $employeeService)
    {
        $this->permissions = get_permissions();
        $this->employeeService = $employeeService;
    }

    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    public function index()
    {
        if (!(Auth::user())[$this->permissions['can_manage_users']]) {
            return unAuthorized(trans('users.user_management'));
        }
        $users = User::leftJoin('sale_points', 'sale_points.id', 'users.sale_point')
            ->where([
                ['users.shop_id', get_session_val('serial_id', 'Integer')],
                //   ['users.login', 1]
            ])
            ->get(['users.id', 'users.user_name', 'users.name', 'sale_points.point_name', 'users.login']);

        return view('user::index', compact('users'));
    }


    /**
     * Show the form for creating a new resource.
     * @return Renderable
     */
    public function create()
    {
        $groups = ClientGroup::where(['shop_id'=> get_session_val('serial_id', 'Integer')])->get(['id', 'name']);
        $points = SalePoint::where('shop_id', get_session_val('serial_id', 'Integer'))->get(['id', 'point_name']);
        $stores = Store::where('shop_id', get_session_val('serial_id', 'Integer'))->get(['id', 'store_name']);
        $categories = Category::where('shop_id', get_session_val('serial_id', 'Integer'))->get(['id', 'name']);
        $user_categories = [];
        $route = Route('users.save');
        return view('user::create', compact('points', 'groups', 'route', 'stores', 'categories', 'user_categories'));
    }

    /**
     * Store a newly created resource in storage.
     * @param CreateUserRequest $request
     * @return Renderable
     */
    public function store(CreateUserRequest $request)
    {
        $new_user = new User();
        foreach ($this->permissions as $key => $value) {
            if (isset($request[$key])) {
                $new_user[$value] = ($request[$key] == 'on' ? 1 : $request[$key]);
            } else {
                $new_user[$value] = 0;
            }
        }
        $new_user->shop_id = get_session_val('serial_id', 'Integer');
        unset($new_user->clients_group);
        if ($new_user->save()) {
            $this->storeUserGroup($request, $new_user, get_session_val('serial_id', 'Integer'));
            $this->updateUserCategories($new_user, $request->input('categories', []));
            $this->createLog(Carbon::now(), "تم اضافه   المستخدم".$request->user_name);
            if(get_session_val('bcms', 'Integer')) {
                $this->employeeService->create([
                    'full_name' => $new_user->name,
                    'user_name' => $new_user->user_name,
                    'password' => $request->get('user_password'),
                    'badr_user_id' => $new_user->id,
                ]);
            }
            return redirect(route('users.all'))->with(['success' => trans('users.user_add_successfully')]);
        } else {
            return redirect()->back()->with(['error' => trans('users.error_while_saving')]);
        }
    }
    /**
     * Show the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function show($id)
    {
        return view('user::show');
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function edit($id)
    {
        $groups = ClientGroup::where(['shop_id'=> get_session_val('serial_id', 'Integer')])->get(['id', 'name']);
        $points = SalePoint::where('shop_id', get_session_val('serial_id', 'Integer'))->get(['id', 'point_name']);
        $stores = Store::where('shop_id', get_session_val('serial_id', 'Integer'))->get(['id', 'store_name']);
        $categories = Category::where('shop_id', get_session_val('serial_id', 'Integer'))->get(['id', 'name']);
        $route = Route('users.update', $id);
        $user = User::findOrFail($id);
        $user_categories = CategoryUser::select('category_id')->where(['user_id' => $id, 'shop_id' => get_session_val('serial_id', 'Integer')])->pluck('category_id')->toArray();
        $groupIds = $user->groups->pluck('id');
        return view('user::edit', compact('user', 'route', 'points', 'groups','groupIds', 'stores', 'categories', 'user_categories'));
    }

    /**
     * Update the specified resource in storage.
     * @param UserRequest $request
     * @param int $id
     * @return Renderable
     */
    public function update(UpdateUserRequest $request, $id)
    {
        $existing_user = User::findOrFail($id);
        foreach ($this->permissions as $key => $value) {
            if ($value == 'password' && blank($request->get('user_password'))) continue;

            if (isset($request[$key])) {
                $existing_user[$value] = ($request[$key] == 'on' ? 1 : $request[$key]);
            } else {
                $existing_user[$value] = 0;
            }
        }

        if ($existing_user->save()) {
            $this->updateUserGroup($request, $id, get_session_val('serial_id', 'Integer'));
            $this->updateUserCategories($existing_user, $request->input('categories', []));
            $this->createLog(Carbon::now(), "تم التعديل علي المستخدم".$existing_user->user_name);
            if(get_session_val('bcms', 'Integer')) {
                $this->employeeService->update([
                    'full_name' => $existing_user->name,
                    'user_name' => $existing_user->user_name,
                    'password' => $request->get('user_password'),
                ], $existing_user->id);
            }
            return redirect(route('users.all'))->with(['success' => trans('users.user_update_successfully')]);
        } else {
            return redirect()->back()->with(['error' => trans('users.eror_while_updating')]);
        }
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy(Request $request)
    {
        $id = $request->id ?? null;
        $user = User::findOrFail($id);
        $user[$this->permissions['can_login']] = 0;
        if(get_session_val('bcms', 'Integer')) {
            $this->employeeService->destroy($user->id);
        }
        if ($user->save()) {
            $this->createLog(Carbon::now(), "تم حذف  المستخدم".$user->user_name);
            return response([
                'msg' => trans('users.deleted_successfully'),
                'status' => 1
            ]);
        } else {
            return response([
                'msg' => trans('users.error_while_deleting'),
                'status' => 0
            ]);
        }
    }

    public function activity(Request $request, $id)
    {
        $user = User::findOrFail($id);
        $user_log = Log::where([
            ['shop_id', get_session_val('serial_id', 'Integer')],
            ['user_id', $id]
        ])
            ->when($request->start_date, function ($q) use ($request) {
                $q->whereDate('date_event', '>=', $request->start_date);
            })
            ->when($request->end_date, function ($q) use ($request) {
                $q->whereDate('date_event', '<=', $request->end_date);
            })
            ->orderBy('date_event', 'Desc')
            ->select(['id', 'date_event', 'event'])
            ->paginate(25);
        return view('user::activity', compact('user_log', 'user'));
    }
    private function storeUserGroup($request, $new_user, $shop_id)
    {
        $user = User::where(['user_name' => $new_user->user_name, 'shop_id' => $shop_id])->first();
        if ($request->user_group == 0) {
            $user->clients_group = $request->user_group;
            $user->save();
        } else {
            $user->clients_group = 1;
            $user->save();
            foreach ($request->user_group as $userGroup) {
                $userGroupStore = UserGroup::create([
                    'group_id' => $userGroup,
                    'user_id' => $user->id,
                    'shop_id' => $shop_id,
                    'add_user' => auth()->user()->id,
                ]);
                $userGroupStore->save();
            }
        }
    }
    private function updateUserGroup($request, $id, $shop_id)
    {
        $user = User::where(['id' => $id, 'shop_id' => $shop_id])->first();
        $user_groups = UserGroup::where(['shop_id' => $shop_id, 'add_user' => auth()->user()->id, 'user_id' => $id])->get();
        foreach ($request->user_group as $userGroup) {
            if ($userGroup == 0) {
                $user->clients_group = $request->user_group;
                $user->save();
                $user_groups->each->delete();
            } else {
                $user->clients_group = 1;
                $user->save();
                if ($user_groups) {
                    $user_groups->each->delete();
                }
                $userGroupStore = UserGroup::create([
                    'group_id' => $userGroup,
                    'user_id' => $id,
                    'shop_id' => $shop_id,
                    'add_user' => auth()->user()->id,
                ]);
                $userGroupStore->save();
            }
        }
    }

    private function updateUserCategories(User $user, $categories = [])
    {
        $rows = [];
        foreach ($categories as $category) {
            $rows[$category] = ['shop_id' => $user->shop_id];
        }
        $user->categories()->sync($rows);
    }
  
    public function usersGoals()
    {
        $usersWithGoals = User::where('shop_id',get_session_val('serial_id', 'Integer'))->whereNotNull('goal')->where('goal','>',0)->get();
        $users = User::where('shop_id',get_session_val('serial_id', 'Integer'))->get();
        return view('user::goals', compact('users','usersWithGoals'));
    }

    public function storeGoal(Request $request)
    {
        $request->validate([
            'user_id' => 'required|exists:users,id' ,
            'goal'   => 'required|numeric',
        ]);

        User::where('id', $request->user_id)->where('shop_id',get_session_val('serial_id', 'Integer'))->update([
            'goal'   => $request->goal,
        ]);
        $this->createLog(Carbon::now(),"تم اضافه اهداف مستخدم");


        return redirect()->back()->with('success', 'تم اضافة الهدف بنجاح');
    }

    public function updateGoal(Request $request)
    {
        $request->validate([
            'user_id' => 'required|exists:users,id' ,
            'goal'   => 'required|numeric',
        ]);

        if(User::where('id', $request->user_id)->where('shop_id',get_session_val('serial_id', 'Integer'))->update(['goal' => $request->goal,]))
        {
            session()->flash('success','تم التعديل بنجاح');
        }else{
            session()->flash('error','حدث خطأ ما في التعديل');
        }
        $this->createLog(Carbon::now(),"تم تعديل اهداف مستخدم");

        return true;
    }

    public function getUserGoalsTable()
    {
        $users = User::where('shop_id',get_session_val('serial_id', 'Integer'))->whereNotNull('goal')->where('goal','>',0)->get();
        return view('user::partials.user-goals-table-body', compact('users'))->render();
    }

    public function getUserDetails(User $user){
        // return $user;
        return view('user::partials.editModal.edit-goal-modal-body', compact('user'))->render();
    }

    public function destroyGoal(Request $request)
    {
        // return $request;
        $id = $request->user_id ?? null;
        $user = User::where('shop_id',get_session_val('serial_id', 'Integer'))->where('id',$id)->first();

        if ($user->update(['goal' => 0]))
        {
            session()->flash('success','تم الحذف بنجاح');
        }else{
            session()->flash('error','حدث خطأ ما في الحذف');
        }
        $this->createLog(Carbon::now(),"تم حذف اهداف مستخدم");

        return true;
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}
