<?php

namespace Modules\VersionUpdating\Http\Controllers;

use Exception;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use PDOException;
use ZipArchive;

class VersionController extends Controller
{
    private $conn;
    private $shop_id;
    private $newVersion;

    private $oldConn;
    private $oldDbName;
    private $oldVersion;

    private $newFolderPath;

    private $log_text= "";
    private $step=1;

    private $update_link = "https://albadrsales.com/update/start.php";
    private $update_status = 0;

    public function __construct()
    {
        $this->oldConn = DB::getPdo();
        $this->oldDbName = env('DB_DATABASE');
    }
    
    public function index()
    {
        $this->init();
        $server_output = $this->makeRemoteCall();;
        $currentVersion = config('config.version');
        $newVersion = $server_output['version'] ?? '';
        $anyUpdates = false;
        if($currentVersion !== $newVersion){
            $anyUpdates = true;
        }
        return view('versionupdating::index', compact('currentVersion', 'newVersion', 'anyUpdates' ));
    }

    public function update()
    {
        $this->init();
        
        $server_output = $this->makeRemoteCall();

        if(! $server_output)
        { 
            echo "NO Updates Now";
            exit();
        }

        $this->newVersion = $server_output['version']; 
        $folder_name = $server_output['folder_name']; 
        $remote_file_url = $server_output['file_path'];
        $this->newFolderPath = dirname(dirname(getcwd())) . '/' . $folder_name;


        // Not needed step
        // $this->newFolderPath = $this->createNewVersionFolder($folder_name);
        // if(!$this->newFolderPath){
        //     exit;
        // }


        $newArchivedVersionFile = $this->getRemoteVersion($remote_file_url);
        
        $isExtracted = $this->extractArchive($newArchivedVersionFile);
        
        if(!$isExtracted){
            $this->wrong("Error in extracting archive");
            $this->sendLog();
            exit;
        }
        
        
        if( ! $this->deleteArchiveFile($newArchivedVersionFile) ){
            $this->wrong("Error in deleteing archive file");
            $this->sendLog();
        }

        $newDbName = $folder_name;
        $this->createNewDB($newDbName);
        $this->createDbStructure($newDbName);
        $this->transferData($newDbName);
        $this->movePublicFolder();
        $this->redirectToNewVersion();
        
    }

    private function init()
    {
        ini_set('memory_limit', '512M');
        ini_set('max_execution_time', '0');
        header( 'Content-type: text/html; charset=utf-8' );
        ob_end_flush();

        ob_start();

        // $shop = DB::table('badr_shop')->select('serial_id')->first();
        
        // if(! $shop){
        //     $this->wrong("check Database Error 104");
        //     $this->sendLog();
        //     exit();
        // }

        $this->shop_id = auth()->user()->shop_id;
        $this->oldVersion = env('APP_VERSION');
        return true;
        
    }

    private function makeRemoteCall()
    {
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $this->update_link);
        curl_setopt($ch, CURLOPT_POST, 1);
        // in real life you should use something like:
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query(array('id' => $this->shop_id, 'cv' => $this->oldVersion)));

        // receive server response ...
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);
        curl_close ($ch);

        return json_decode($response, true);
    }

    // private function createNewVersionFolder($folder_name)
    // {
    //     //creart folder for newversion
    //     $current_path= getcwd(); // curren path
    //     $up_path= dirname(dirname($current_path,1)); // get upper path
    //     $new_folder= $up_path."/".$folder_name;

    //     if (!file_exists($new_folder))/* Check folder exists or not */
    //     {
    //         if (!mkdir($new_folder, 0777)) {
    //             $this->wrong( "Failed to create directory");
    //             exit;
    //         }
            
    //         $this->great( "Folder  ". $new_folder ." Created <br />"); /* Success Message */
            
    //         ob_flush();
    //         flush();
    //     }else{
    //         $message="Folder ".$new_folder." exist </br>";
	//         $this->wrong($message);
    //         $this->sendLog();
    //         return false;
    //     }

    //     return $new_folder;
    // }

    private function getRemoteVersion($remote_file_url)
    {
        // $local_file = $this->newFolderPath."/".'files.zip';
        $local_file = dirname(dirname(getcwd())).'/files.zip';
        //download files
        set_time_limit(0); //Unlimited max execution time	
        
        //strat copy files 
        $this->great("Start Getting New Version ..<br />","remote ".$remote_file_url." copy to local " . $local_file ." ");

        $remote = fopen($remote_file_url, 'r');
        $local = fopen($local_file, 'w');

        $headers = get_headers($remote_file_url, 1);
        $filesize = $headers['Content-Length'];
        // $steps_downlod = $filesize/2048;
        
        $read_bytes = 0;
        $oldp=0;
        $os = array(5,10, 20,30,40,50,60,70,80,85,90,95,98);
        echo "<br/> <div class='alert alert-success' role='alert' style='clear: both;'> download %[.. "; 

        // $downloadlog="";

        while(!feof($remote)) 
        {
            try{
                $buffer = fread($remote, 2048);
                fwrite($local, $buffer);
            }catch(Exception $e){
                $msg = "error in read/write operation from server";  // $e->getMessage();
                $this->wrong($msg);
                $this->sendLog();
            }

            $read_bytes += 2048;

            $progress = min(100, 100 * $read_bytes / $filesize);

                $p=round($progress);
                
                if(in_array($p, $os) &&  $oldp != $p	)
                {
                    echo   $p .".. "; 
                    $oldp=$p;
                }
            
            
            
            ob_flush();
            flush();
        
        }
        echo " ] 100% </div><br /> " .
        fclose($remote);
        fclose($local);

        
        $this->great("  success to copy $remote_file_url  <br />" ,$oldp);

        //ob_flush();
        
        ob_flush();
        flush();

        return $local_file;
    }

    private function extractArchive($newFile)
    {
        //extract files to new folder
        $zip = new ZipArchive();
        $res = $zip->open($newFile);
        if ($res === TRUE) 
        {
            $this->great("Start to Extra file  $newFile  <br />");
            $zip->extractTo(dirname(dirname(getcwd())));
            $zip->close();
            $this->great('extract file !<br />');
            return true;
        }else{
            return false;
        }
    }

    private function deleteArchiveFile($newFile)
    {
        if (unlink($newFile)) {
            $this->great('Archived file deleted !<br />');
            return true;
        } else {
            return false;
        }    
    }

    private function createNewDB($dbName)
    {
        //creat db structure 
        try { 
            $this->conn = new \PDO('mysql:host=127.0.0.1', env("DB_USERNAME", "root"), env("DB_PASSWORD","") ); // Check connection
        } catch (\PDOException $e) {
            $this->wrong ("Connection failed: " . $e->getMessage());
            $this->sendLog();
            exit;
        }

        // Create database
        try{
            $sql="CREATE DATABASE IF NOT EXISTS ".$dbName."  DEFAULT CHARACTER SET utf8 DEFAULT COLLATE utf8_general_ci;"; 
            $this->conn->exec($sql);
            $this->great("Database created successfully  <br>");
            ob_flush();
            flush();
        }catch (\PDOException $e) {
            $this->wrong ("Creating new db failed: " . $e->getMessage());
            $this->sendLog();
            exit;
        }
    }
    
    private function createDbStructure($newDbName)
    {

        $queries = "SET sql_mode = ''; 
                SET sql_mode = 'NO_AUTO_VALUE_ON_ZERO'; 
                SET time_zone = '+00:00'; 
                SET SESSION group_concat_max_len = 1000000;";
            
        try{
            $this->conn = new \PDO("mysql:host=127.0.0.1;dbname=$newDbName", env("DB_USERNAME", "root"), env("DB_PASSWORD","") );
            $this->conn->exec($queries);

            //read databse structre file
            $dbStructure = file_get_contents($this->newFolderPath.'/modules/installer/stubs/structure.stub', true);

            $stmt = $this->conn->query($dbStructure);

            do {
                // Fetch the next result set (if any)
                $stmt->nextRowset();
            } while ($stmt->nextRowset());
        
        }catch(PDOException $e){
            $this->wrong('error in creating the db structure :: ' . $e->getMessage());
            $this->sendLog();
            exit;
        }

        $this->great("Database Structure Created <br>");
    
        ob_flush();
        flush();
        return true;
    }

    private function transferData($newDbName)
    {
        // get all table names
        $tables = array();
        
        $queries = "set names 'utf8';
                        SET sql_mode = 'NO_AUTO_VALUE_ON_ZERO';  
                        SET SESSION group_concat_max_len = 1000000;";

        try{
            $this->oldConn->exec($queries);
            $stmt = $this->oldConn->query("SHOW full TABLES where Table_Type != 'VIEW';");
            $tables = $stmt->fetchAll(\PDO::FETCH_COLUMN);
        }catch(\PDOException $e){
            $this->wrong('Error in getting the tables');
            $this->sendLog();
            exit;
        }
        
        
        // Transferring data
        foreach($tables as $table)
        {            
            try{
                $row = $this->oldConn->query("SELECT GROUP_CONCAT(CONCAT('`',COLUMN_NAME,'`')) FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = '".$this->oldDbName."' AND TABLE_NAME = '".$table."'")->fetch();
                $query = "INSERT INTO $newDbName.$table (".$row[0].") SELECT ".$row[0]." FROM ".$this->oldDbName.".".$table.";";
                $this->oldConn->exec($query);
                
            }catch(Exception $e){
                $this->wrong('<br/> Error in transferring data from old table ('. $table . ') :: ' . $e->getMessage() . '<br/>');
                $this->sendLog();
            }

            $this->great("<br />Complete get data from old table ".$table." <br /> ");
            
            ob_flush();
            flush();
            usleep( 500 );  //.5 second
        }
        return true;
    }
    
    private function movePublicFolder()
    {
        $this->great( " Start copy uploads folder  <br>");
        
        $source = public_path();
        $destination = $this->newFolderPath;

        File::copyDirectory($source, $destination);
        
        $this->great( " end  copy Upload folder  <br>");
        
    }

    private function redirectToNewVersion()
    {
        //redirect link to new version
        // creat htaccess to direct to new version
        $this->great("Start Redirect to New Version <br>");
        rename(public_path('index.php'), public_path('old-index.php')) ;

        $redirect_code = '<?php header("Location: '.$this->newFolderPath.'/public/index.php");?>';

        // file_put_contents(public_path('index.php'), $redirect_code."\r\n".file_get_contents(public_path('index.php')));
        file_put_contents(public_path('index.php'), $redirect_code);
        
        $this->great("Update Old to redirect To New <br>");
        $this->update_status=1;
        return true;
    }

    private function sendLog()
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->update_link);
        curl_setopt($ch, CURLOPT_POST, 1);
        // in real life you should use something like:
        curl_setopt($ch, CURLOPT_POSTFIELDS,
        http_build_query(array('id' => $this->shop_id,
                                'cv' => $this->oldVersion ,
                                'nv'=>$this->newVersion,
                                'log'=>$this->log_text,
                                'status'=>$this->update_status)));

        // receive server response ...
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        // $server_output=json_decode(curl_exec($ch),true);
        curl_exec($ch);
        curl_close ($ch);
    }


    private function wrong($x,$add_data="")
    {
        echo  '<div class="alert alert-danger" role="alert"> Step ' . $this->step.' : '.$x.'</div>';
        $this->log_text .= $x.$add_data ;
        $this->step +=1;
        ob_flush();
        flush();
    }

    private function great($x,$add_data="")
    {
        echo  '<div class="alert alert-success" role="alert"> Step ' . $this->step.' :  '.$x.'</div>';
        $this->log_text .= $x.$add_data ;
        $this->step +=1;
        ob_flush();
        flush();
    }
}
