<?php

namespace App\Http\Controllers\Auth;

use App\Exceptions\RegisteringException;
use App\Http\Controllers\Controller;
use App\Models\BarcodeSetting;
use App\Models\SalesMan;
use App\Providers\RouteServiceProvider;
use App\Models\User;
use Illuminate\Foundation\Auth\RegistersUsers;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Modules\Category\Entities\Unit;
use Modules\Clients\Entities\Client;
use Modules\Invoices\Entities\ClientTransaction;
use Modules\Invoices\Entities\InvoiceSettings;
use Modules\Invoices\Entities\ItemType;
use Modules\Item\Entities\Company;
use Modules\SalePoints\Entities\SalePoint;
use Modules\Settings\Entities\Badrshop;
use Modules\Item\Entities\Store;

class RegisterController extends Controller
{
    protected $temp_user, $temp_data, $temp_active_id, $temp_serial_id, $temp_package, $temp_add, $temp_plan;
    // $active_id = $data[0];
    // $serial_id = $data[1];
    // $package = $data[2]; //multi store or not 
    // $add = $data[3]; //users numbers
    // $plan = $data[4]; //plan numbers
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

    use RegistersUsers;

    /**
     * Where to redirect users after registration.
     *
     * @var string
     */
    protected $redirectTo = RouteServiceProvider::HOME;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest');
    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array  $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {        
        return Validator::make($data, [
            'store_name' => ['required', 'string', 'max:255'],
            'manager_name' => ['required', 'string', 'max:255'],
            'store_serial' => ['required', 'string', 'max:255'],
            'manager_user_name' => ['required', 'string', 'max:255'],
            'user_password' => ['required', 'string', 'max:100', 'min:8'],
            'user_pasword_confirm' => ['required', 'string', 'min:8', 'max:100', 'same:user_password'],
            'active_code' => ['required_if:activation_type,1', 'string', 'max:50', 'nullable' ],
        ]);
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array  $data
     * @return \App\Models\User
     */
    protected function create(array $data)
    {
        $this->temp_data = $data;

        $this->validateMap()->createDb()->createTables()->createDefaults();

        createMaintenanceOrderDefaultStatuses($this->temp_serial_id);
        
        return $this->temp_user;
    }

    protected function validateMap()
    {
        if ( $this->temp_data['activation_type'] == 1)
        {
            $this->validateStoreOffline();
        }
        else {
            $this->validateStoreOnline();
        }
        return $this;
    }

    protected function validateStoreOffline()
    {
        $response_details = explode(':', $this->temp_data['active_code']);
        try {
            $this->temp_active_id = $response_details[0];
            $this->temp_serial_id = $response_details[1];
        } catch (\Throwable $th) {
            abort(404);
        }
        $this->temp_add = $response_details[2] ?? 1;
        $this->temp_plan = $response_details[3] ?? 0;
    }

    protected function validateStoreOnline()
    {

        $store_serial = $this->temp_data['store_serial']; 
        $response = Http::get('http://albadrsales.com/bpmclient/install.php?&pac=' . getPcMac() . '&active=' . $store_serial);
        $response_details = explode(":", Str::between($response, '("', '")'));

        throw_if (count($response_details) < 5, RegisteringException::class, $response_details[0]);

        $this->prepareDataForDefaults($response_details);
        return $this;
    }

    protected function prepareDataForDefaults(array $response_details)
    {
        [$this->temp_active_id, $this->temp_serial_id, $this->temp_package, $this->temp_add, $this->temp_plan] = $response_details;
    }

    protected function createDb($connection_user_name = 'root', $connection_password = '')
    {
        $DB_name = env('DB_DATABASE', 'almustas_bsdrforsales');
        $connection_user_name = env('DB_USERNAME', 'root');
        $connection_password = env('DB_PASSWORD', '');
        $con =  mysqli_connect('localhost', $connection_user_name, $connection_password, $DB_name);
        mysqli_query($con, "set names 'utf8';");

        $query = "CREATE DATABASE IF NOT EXISTS $DB_name  DEFAULT CHARACTER SET utf8 DEFAULT COLLATE utf8_general_ci;";

        $result = mysqli_query($con, $query);

        return $this;
    }

    protected function createTables()
    {
        Artisan::call('migrate');
        return $this;
    }

    protected function createDefaults()
    {
        DB::transaction(function () {

            Badrshop::create([
                'plan' => $this->temp_plan,
                'shop_name' => $this->temp_data['store_name'],
                'u_no' => $this->temp_add,
                'telephone' => $this->temp_data['store_serial'],
                'ower_name' => $this->temp_data['manager_name'],
                'serial_id' => $this->temp_serial_id,
                'active_id' => $this->temp_active_id,
                'module' => 1,
                'zone' => 'Africa/Cairo',
                'style' => 1,
                'date_type' => 2,
                'online' => 0,
                'Regist_purchases' => 3,
                'active_date' => date('Y-m-d H:i:s'),
                'bill_no_record' => 12,
                'currency' => trans('native.pound'),
                'notify' => 0,
                'run_date' => now()->addDays(5)
            ]);
            
            \Modules\EInvoice\Entities\Settings::create([
                'shop_id' => $request->shop_id,
                'company_name' => $request->store_name,
                'active_id' => ' ',
                'branch_id' => 0,
                'country_id' => ' ',
                'currency_id' => trans('native.pound'),
                'registration_id' => ' ',
                'client_id' => ' ',
                'client_secret' => ' ',
            ]);

            $client = Client::create([
                'client_name' => trans('default.default-model', ['model' => trans('default.client-supplier')]),
                'shop_id' => $this->temp_serial_id,
                'client_role' => 2
            ]);

            $store = Store::create([
                'store_name' => trans('default.default-model', ['model' => trans('default.store')]),
                'shop_id' => $this->temp_serial_id
            ]);

            $safe_point = SalePoint::create([
                'point_name' => trans('default.default-model', ['model' => trans('default.safe_point')]),
                'shop_id' => $this->temp_serial_id,
                'store_id' => $store->id
            ]);

            SalesMan::create([
                'sales_name' => trans('default.default-model', ['model' => trans('default.sales_man')]),
                'shop_id' => $this->temp_serial_id
            ]);

            ItemType::create([
                'name' => trans('default.default-model', ['model' => trans('default.category')]),
                'shop_id' => $this->temp_serial_id
            ]);

            InvoiceSettings::create([
                'shop_id' => $this->temp_serial_id
            ]);

            BarcodeSetting::create([
                'shop_id' => $this->temp_serial_id
            ]);

            Unit::create([
                'name' => trans('default.default-model', ['model' => trans('default.unit')]),
                'shop_id' => $this->temp_serial_id
            ]);

            Company::create([
                'company_name' => trans('default.default-model', ['model' => trans('default.company')]),
                'shop_id' => $this->temp_serial_id
            ]);

            $this->temp_user = user::create([
                'name' => $this->temp_data['manager_name'],
                'email' => $this->temp_data['manager_user_name'],
                'password' => $this->temp_data['user_password'],
                'shop_id' => $this->temp_serial_id,
                'user_name' => $this->temp_data['manager_user_name'],
                'users' => 1,
                'setting' => 1,
                'sale_point' => 1
            ]);

            ClientTransaction::create(
                [
                    'date_time' => date("Y-m-d H:i:s"),
                    'client_id' => $client->id,
                    'amount' => 0,
                    'type' => 4,
                    'effect' => 2,
                    'pay_day' => date("Y-m-d H:i:s"),
                    'balance' => 0,
                    'user_id' => $this->temp_user->id,
                    'shop_id' => $this->temp_serial_id
                ],
                [
                    'date_time' => date("Y-m-d H:i:s"),
                    'amount' => 0,
                    'type' => 4,
                    'effect' => 2,
                    'pay_day' => date("Y-m-d H:i:s"),
                    'user_id' => $this->temp_user->id,
                    'shop_id' => $this->temp_serial_id,
                    'safe_type' => 1,
                    'safe_balance' => 0,
                    'safe_point_id' => $safe_point->id
                ],
                [
                    'date_time' => date("Y-m-d H:i:s"),
                    'amount' => 0,
                    'type' => 4,
                    'effect' => 1,
                    'pay_day' => date("Y-m-d H:i:s"),
                    'user_id' => $this->temp_user->id,
                    'shop_id' => $this->temp_serial_id,
                    'safe_type' => 2,
                    'safe_balance' => 0,
                ]
            );
            
            createDefaultInvoice();
        });

        set_session($this->temp_user);

        return $this;
    }

}
