<?php

namespace App\Http\Controllers;

use App\Http\Requests\NewAccountRequest;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Controller;
use App\Models\BarcodeSetting;
use App\Models\SalesMan;
use App\Models\User;
use Modules\Category\Entities\Unit;
use Modules\Clients\Entities\Client;
use Modules\Invoices\Entities\ClientTransaction;
use Modules\Invoices\Entities\InvoiceSettings;
use Modules\Invoices\Entities\ItemType;
use Modules\Item\Entities\Company;
use Modules\SalePoints\Entities\SalePoint;
use Modules\Settings\Entities\Badrshop;
use Modules\Item\Entities\Store;

class HomeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth')->except(['createAccount', 'new_account']);
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        return view('home');
    }

    public function new_account(NewAccountRequest $request)
    {
        DB::transaction(function () use($request) {

            Badrshop::create([
                'plan' => 1,
                'shop_name' => $request->store_name,
                'u_no' => 20,
                'telephone' => 123456789,
                'ower_name' => $request->user_name,
                'serial_id' => $request->shop_id,
                'active_id' => ' ',
                'module' => 1,
                'zone' => 'Africa/Cairo',
                'style' => 1,
                'date_type' => 2,
                'online' => 0,
                'Regist_purchases' => 3,
                'active_date' => date('Y-m-d H:i:s'),
                'bill_no_record' => 12,
                'currency' => trans('native.pound'),
                'notify' => 0
            ]);

            \Modules\EInvoice\Entities\Settings::create([
                'shop_id' => $request->shop_id,
                'company_name' => $request->store_name,
                'active_id' => ' ',
                'branch_id' => 0,
                'country_id' => ' ',
                'currency_id' => trans('native.pound'),
                'registration_id' => ' ',
                'client_id' => ' ',
                'client_secret' => ' ',
            ]);

            $client = Client::create([
                'client_name' => trans('default.default-model', ['model' => trans('default.client-supplier')]),
                'shop_id' => $request->shop_id,
                'client_role' => 2
            ]);

            $store = Store::create([
                'store_name' => trans('default.default-model', ['model' => trans('default.store')]),
                'shop_id' =>  $request->shop_id,
            ]);

            $safe_point = SalePoint::create([
                'point_name' => trans('default.default-model', ['model' => trans('default.safe_point')]),
                'shop_id' =>  $request->shop_id,
                'store_id' => $store->id
            ]);

            SalesMan::create([
                'sales_name' => trans('default.default-model', ['model' => trans('default.sales_man')]),
                'shop_id' => $request->shop_id
            ]);

            ItemType::create([
                'name' => trans('default.default-model', ['model' => trans('default.category')]),
                'shop_id' => $request->shop_id
            ]);

            InvoiceSettings::create([
                'shop_id' => $request->shop_id
            ]);

            BarcodeSetting::create([
                'shop_id' => $request->shop_id
            ]);

            Unit::create([
                'name' => trans('default.default-model', ['model' => trans('default.unit')]),
                'shop_id' => $request->shop_id
            ]);

            Company::create([
                'company_name' => trans('default.default-model', ['model' => trans('default.company')]),
                'shop_id' => $request->shop_id
            ]);

            $user = User::create([
                'name' => $request->user_name,
                'email' => $request->user_name,
                'password' => $request->password,
                'shop_id' => $request->shop_id,
                'user_name' => $request->user_name,
                'users' => 1,
                'setting' => 1,
                'sale_point' => $safe_point->id
            ]);

            ClientTransaction::create(
                [
                    'date_time' => date("Y-m-d H:i:s"),
                    'client_id' => $client->id,
                    'amount' => 0,
                    'type' => 4,
                    'effect' => 2,
                    'pay_day' => date("Y-m-d H:i:s"),
                    'balance' => 0,
                    'user_id' => $user->id,
                    'shop_id' => $request->shop_id
                ],
                [
                    'date_time' => date("Y-m-d H:i:s"),
                    'amount' => 0,
                    'type' => 4,
                    'effect' => 2,
                    'pay_day' => date("Y-m-d H:i:s"),
                    'user_id' => $user->id,
                    'shop_id' => $request->shop_id,
                    'safe_type' => 1,
                    'safe_balance' => 0,
                    'safe_point_id' => $safe_point->id
                ],
                [
                    'date_time' => date("Y-m-d H:i:s"),
                    'amount' => 0,
                    'type' => 4,
                    'effect' => 1,
                    'pay_day' => date("Y-m-d H:i:s"),
                    'user_id' => $user->id,
                    'shop_id' => $request->shop_id,
                    'safe_type' => 2,
                    'safe_balance' => 0,
                ]
            );

            createMaintenanceOrderDefaultStatuses($request->shop_id);
        });

        return redirect()->route('login')->with([ 'success' => 'تم انشاء المتجر بنجاح' ])->withInput(['user_name', $request->user_name]);

    }

    public function createAccount()
    {
        return view('auth.new_account');
    }
}
