<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;
use Illuminate\Support\Facades\View;
use Modules\Item\Entities\BadrShop;

class CheckShopIsActive
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure(\Illuminate\Http\Request): (\Illuminate\Http\Response|\Illuminate\Http\RedirectResponse)  $next
     * @return \Illuminate\Http\Response|\Illuminate\Http\RedirectResponse
     */
    public function handle(Request $request, Closure $next)
    {
        if ($request->is('api/*')) {
            return $next($request);
        }

        if (auth()->check() || $request->is('*/api')) {
            $this->isAuth();
            return $next($request);
        } else {
            if(in_array(last(explode('/', request()->getUri())), ['login', 'installer', 'new_account']) && in_array(request()->method(), ['POST', 'GET'])) {
                return $next($request);
            } else {
                return redirect()->route('login');
            }
        }
    }

    protected function isAuth()
    {
        $shop = BadrShop::select('serial_id', 'active_id', 'telephone', 'online', 'run_date', 'module')->where('serial_id', auth()->user()->shop_id)->first();
        if ($shop && ! in_array(request()->root(), ['https://host.albadrsystems.net', 'https://albadrsystems.com', 'https://badrforsales-test.souqalbadr.top'])) {
            abort_if($shop->checkOnlineIsNotActive(), 420, 'تم إنتهاء مدة إشتراكك');
            abort_if($shop->checkOfflineIsNotActive(), 421, 'اتصل بالدعم الفنى كود المشكله 103');
        }

        if ($shop->module == 2 && app()->getLocale() == 'ar') {
            app()->setLocale('ar-milk');
        }

        View::composer('*', function ($view) {

            if(! Cache::has('is_live') ) {
                $is_live = Schema::hasTable('einvoice_settings')
                ? DB::table('einvoice_settings')->select('is_live')->where('shop_id', auth()->user()->shop_id)->first()?->is_live
                : null;
                Cache::set('is_live', $is_live);
            } else {
                $is_live = Cache::get('is_live');
            }

            $view->with('einvoice_mode', $is_live);
        });
    }
}
