<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Http\Request;
use Modules\Invoices\Entities\Invoices;

class InvoiceTypeFilter
{
    private $permission = null;
    private $permission_translated = null;
    private $action = null;

    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure(\Illuminate\Http\Request): (\Illuminate\Http\Response|\Illuminate\Http\RedirectResponse)  $next
     * @return \Illuminate\Http\Response|\Illuminate\Http\RedirectResponse
     */
    public function handle(Request $request, Closure $next)
    {
        $url_details = explode('/', $request->route()->uri);
        if (count($url_details) > 2)
        {
            $this->action = $url_details[2];
        }

        $invoice_type = $request->route()->parameter('invoice_type') ?? '';

        if (in_array(strtolower($invoice_type),Invoices::getAvailableTypes()))
        {
            if ($this->check_auth($invoice_type))
                return $next($request);
            return unAuthorized($this->permission_translated);
        }
        else
            return redirect()->route('invoices.unsupported_type', $invoice_type);
    }

    private function check_auth($invoice_type)
    {

        if ($this->action == 'new')
        {
            if (in_array($invoice_type, ['sales', 'sales offers']))
            {
                $this->permission = 'can_sale';
                $this->permission_translated = trans('users.sale_point');
                return authorized($this->permission);
            }
            else if ($invoice_type == 'returned sales')
            {
                $this->permission = 'can_return_sales';
                $this->permission_translated = trans('users.record_returns');
                return authorized($this->permission);
            }
            else if ($invoice_type == 'purchases')
            {
                $this->permission = 'can_add_purchases_invoices';
                $this->permission_translated = trans('users.add_purchases');
                return authorized($this->permission);
            }
            else
            {
                $this->permission = 'can_add_returned_purchases';
                $this->permission_translated = trans('users.add_payoff_purchases');
                return authorized($this->permission);
            }
        }
        else if (is_null($this->action))   // $this->action == 'index'</else>
        {
            if ( in_array($invoice_type, ['sales', 'sales offers', 'returned sales']))
            {
                $this->permission = 'can_display_sales';
                $this->permission_translated = trans('users.read_daily_sales');
                return authorized($this->permission);
            }
            else
            {
                $this->permission = 'can_display_purchases';
                $this->permission_translated = trans('users.show_purchases');
                return authorized($this->permission);
            }
        }
        return true;
    }
}
