<?php

namespace App\Imports;

use App\Models\systemLog;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\ToModel;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;
use Illuminate\Validation\Rule;

use Maatwebsite\Excel\Concerns\Importable;
use Maatwebsite\Excel\Concerns\WithBatchInserts;
use Maatwebsite\Excel\Concerns\WithValidation;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use Maatwebsite\Excel\Concerns\SkipsErrors;
use Maatwebsite\Excel\Concerns\SkipsOnError;
use Maatwebsite\Excel\Concerns\SkipsFailures;
use Maatwebsite\Excel\Concerns\SkipsOnFailure;
use Illuminate\Support\Facades\Validator;
use Maatwebsite\Excel\Concerns\SkipsEmptyRows;
use Maatwebsite\Excel\Validators\Failure;
use Modules\Excel\Entities\Item;
use Modules\Excel\Entities\ItemUnit;
use Modules\Excel\Entities\Unit;

class ItemUnitImport implements
    ToCollection,
    WithValidation,
    WithHeadingRow,
    SkipsOnError,
    WithBatchInserts
{
    use Importable, SkipsErrors;
    public function collection(Collection $rows)
    {
        if ($rows->count() == 0) {
            return redirect()->route('excel.units-excel')->with('error', 'الملف فارغ');
        }
        foreach ($rows as $row) {
            $item_name = $row['item_name'];
            $unit_name = $row['unit_name'];
            $unit_barcode = $row['unit_barcode'];
            $unit_price =   $row['unit_price'];
            $unit_value =   $row['unit_value'];
            $item = Item::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('item_name', $item_name)->first();
            $unit = Unit::where('shop_id', get_session_val('serial_id', 'integer'))
                ->where('name', $unit_name)->first();
            $item_units = ItemUnit::where([
                'shop_id' => get_session_val('serial_id', 'integer')
            ])->get();
            foreach ($item_units as $ItemUnit) {
                if ($row['unit_barcode'] == $ItemUnit->unit_barcode) {
                    return redirect()->route('excel.units-excel')->with('error', ' باركود الوحده' . '  ' . $row['unit_name'] . '  ' . 'مسجل من قبل'  . ' ');
                }
            }
            if ($unit_barcode != null) {
                if (strpos($unit_barcode, '.')) {
                    return redirect()->back()->withErrors(['error' => 'صيغه باركود الوحده غير صحيحه للصنف ' . '  ' . $item_name . '']);
                }
            }
            if (!preg_match("~^[a-z0-9٠-٩\-+/,()/'\s\p{Arabic}]{1,60}$~iu", $unit_name)) {
                return redirect()->back()->withErrors(['صيغه اسم الوحده غير صحيحه للصنف ' . '  ' . $item_name . '']);
            }
            if ($unit  != null) {
                if ($item->unit_id == $unit->id) {

                    return redirect()->back()->withErrors(['error' => 'لا يمكن تسجيل الوحده الفرعيه بنفس الوحده الاساسيه للصنف ' . '  ' . $item_name . '']);
                }
            }
            if ($unit != null) {
                $item_unit = ItemUnit::where([
                    'item_id' => $item->id,
                    'unit_id' => $unit->id,
                    'shop_id' => get_session_val('serial_id', 'integer')
                ])->first();
                if ($item_unit) {
                    $existed_barcode = $item_unit->unit_barcode;
                    if ($existed_barcode == $unit_barcode) {
                        $item_unit->unit_value =  $unit_value;
                        $item_unit->unit_price =   $unit_price;
                        $item_unit->save();
                    } else {
                        $item_unit->unit_barcode = $unit_barcode;
                        $item_unit->unit_value =  $unit_value;
                        $item_unit->unit_price =   $unit_price;
                        $item_unit->save();
                    }
                } else {
                    $item_unit = new ItemUnit();
                    $item_unit->item_id = $item->id;
                    $item_unit->unit_id = $unit->id;
                    $item_unit->unit_value =  $unit_value;
                    $item_unit->unit_price = $row['unit_price'];
                    $item_unit->unit_barcode = $unit_barcode;
                    $item_unit->shop_id =  get_session_val('serial_id', 'integer');
                    $item_unit->save();
                }
            } else {

                $new_unit = new Unit();
                $new_unit->name = $row['unit_name'];
                $new_unit->shop_id = get_session_val('serial_id', 'integer');
                $new_unit->save();
                $unit = Unit::where('shop_id', get_session_val('serial_id', 'integer'))
                    ->where('name', $row['unit_name'])->first();
                $item_unit = new ItemUnit();
                $item_unit->item_id = $item->id;
                $item_unit->unit_id = $unit->id;
                $item_unit->unit_value =  $unit_value;
                $item_unit->unit_price =   $unit_price;
                $item_unit->unit_barcode = $unit_barcode;
                $item_unit->shop_id =  get_session_val('serial_id', 'integer');
                $item_unit->save();
            }
        }
        $this->createLog(Carbon::now(),"تم اضافه وحدات فرعيه بالاكسل");
        return redirect()->route('excel.units-excel')->with(['success' => trans('native.process_success')]);
    }
    public function headingRow(): int
    {
        return 1;
    }
    public function rules(): array
    {


        return [
            '*.item_name' => [
                'required',
                'exists:items,item_name',
            ],
            '*.unit_value' => [
                'required',
                'numeric',
            ],
            '*.unit_price' => [
                'required',
                'numeric',
            ],
            '*.unit_name' => [
                'required',
                'string',

            ],

            '*.unit_barcode' => [
                'nullable',
                'numeric',
                'unique:items,barcode',
                Rule::unique('items_unit')->where(function ($query) {
                    return $query->where('shop_id', get_session_val('serial_id', 'integer'));
                }),

            ],

        ];
    }
    public function customValidationMessages(): array
    {
        return [
            'item_name.required'               => __('items.item_name.required'),
            'unit_barcode.numeric' => 'باركود الوحده يجب ان يكون ارقام فقط',
            'unit_barcode.unique' => 'باركود الوحده  مستخدم من قبل',
            'item_name.exists'                 => __('items.item_name.exists'),
            'unit_barcode.required'            => __('items.unit_barcode_required'),
            'unit_value.required'                 => __('items.unit_value_required'),
            'unit_value.numeric'                 => __('items.unit_value_numeric'),

            'unit_price.required' => __('items.unit_price_required'),
            'unit_price.numeric' => __('items.unit_price_numeric'),
            'unit_name.required'   => __('units.unit_name_required'),
            'unit_name.alpha_num'   => 'صيغه اسم الوحده غير صحيحه',
        ];
    }
    public function batchSize(): int
    {
        return 1000;
    }

    public function chunkSize(): int
    {
        return 1000;
    }
    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }
}
