<?php

namespace App\Models;

use App\Traits\RoundingAttributesTrait;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\Hash;
use Laravel\Sanctum\HasApiTokens;
use Modules\Api\Entities\Request;
use Modules\Clients\Entities\ClientsGroups;
use Modules\User\Entities\Category;
use Modules\User\Entities\CategoryUser;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable, RoundingAttributesTrait;

    protected $table = 'users';
    public $timestamps = false;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    // protected $fillable = [
    //     'user_name',
    //     'password',
    // ];

    protected $guarded = [];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    // protected function password(): Attribute
    // {
    //     return Attribute::make(
    //         get: fn ($value) => ($value),
    //         set: fn ($value) => Hash::make($value),
    //     );
    // }

    protected function SetPasswordAttribute($value)
    {
        $this->attributes['password'] = Hash::make($value);
    }

    protected function setEmailAttribute($value)
    {
        $this->attributes['email'] = strtolower($value);
    }

    protected function setUserNameAttribute($value)
    {
        $this->attributes['user_name'] = strtolower($value);

    }


    public function salePoint()
    {
        return $this->belongsTo(SalePoint::class , 'sale_point', 'id');
    }

    public function categories()
    {
        return $this->belongsToMany(Category::class, CategoryUser::class);
    }

	public function  getUserSalePoints(){
        $sale_points = SalePoint::where('shop_id', get_session_val('serial_id', 'integer'));
		$point = $this->sale_point;
		//dd($point);
		switch($this->show_points){
             case 1:
		     case 2:
               $safe_points = (clone $sale_points)->where('id', $point)->get();
			break;
			case 3:
			  $store_id = (clone $sale_points)->where('id', $point)->first()->store_id;
              $safe_points = (clone $sale_points)->where('store_id', $store_id)->get();
           break;
		   default:
		       $safe_points = (clone $sale_points)->get();
		   break;

		}
	 return $safe_points;	
   }

    public function getLastSubDay()
    {
        if (! auth()->check()) return 0;
        return now()->subDay()->diffInDays(Carbon::parse(get_session_val('run_date', 'string')), false);
    }
    public function groups()
    {
        return $this->belongsToMany(ClientsGroups::class, 'users_groups', 'user_id', 'group_id');
    }

    public  function createLog($date, $event)
    {
        $log = new systemLog();
        $log->user_id = auth()->id();
        $log->shop_id = get_session_val('serial_id', 'integer');
        $log->date_event = $date;
        $log->event = $event;
        $log->save();
    }

}
