<?php

namespace App\Observers;

use Modules\Invoices\Entities\BillAdd;
use Modules\Invoices\Entities\InvoiceDetails;
use Modules\Invoices\Entities\Invoices;
use Modules\Invoices\Entities\Item;

class InvoiceDetailsObserver
{
    public function creating(InvoiceDetails $details)
    {
        $details->price_without_vat = $details->item_price_with_discount;
    }

    public function created(InvoiceDetails $details)
    {
        if ($details->price_without_vat != $details->item_price_with_discount) {
            $details->update(['price_without_vat' => $details->item_price_with_discount]);
        }

        $invoice_type = Invoices::select('invoice_type')->where('shop_id', $details->shop_id)->find($details->invoice_id)->invoice_type ?? 1;
        $item = Item::select('vat_state', 'purchase_vat_state', 'id')->where('shop_id', $details->shop_id)->find($details->item_id);

        $item->price = $details->price_without_vat;
        $vat_state = in_array($invoice_type, [0, 2]) ? $item->purchase_vat_state : $item->vat_state;

        $amount = $item->price;
        if ($vat_state == 2 && get_session_val('apply_vats', 'integer') == 1) {
            $amount = $this->caclAmount($invoice_type, $item);
        }
        $details->update(['price_without_vat' => $amount]);
    }

    protected function caclAmount($invoice_type, $item)
    {
        $vat_use = in_array($invoice_type, [0, 2]) ? [0, 2] : [0, 1];
        $adds = BillAdd::select('addition_value', 'check_addition', 'id')
                        ->where(['shop_id' => auth()->user()->shop_id, 'add_role' => false])
                        ->whereHas('itemVats', function($query) use($item, $vat_use) {
                            $query->where('item_id', $item->id)->whereIn('vat_use', $vat_use);
                        })->get();

        $vat_value = $this->calcVats($adds, $item->price);
        return $item->price - $vat_value;
    }

    protected function calcVats($adds, $price)
    {
        $vat_value = 0;
        foreach ($adds as $add) {
            if ( $add->check_addition ) {
                $vat_value += $add->addition_value;
            } else {
                $vat_value += $price - ($price / ( 1 + ($add->addition_value / 100) ));
            }
        }

        return $vat_value;
    }

    protected function calcDiscount($details, $price, $vats)
    {
        if ($details->discount == 0) return $price;
        return $details->discount_type
                ? $price - $details->discount
                : $price - ( ($price - $vats) * ( $details->discount / 100 ) );
    }
}
