<?php

namespace App\Services\Einvoice\Interfaces;

use Illuminate\Support\Collection;

interface InitiateDocument
{
    /**
     * This Method to set Document to be submitted.
     *
     * @param mixed $bill
     * @param string $cert_token
     * @return $this
     */
    public function setDocument(mixed $bill, string $cert_token): self;

    /**
     * This Method to set Issuer to be submitted.
     *
     * @param $settings
     * @return array
     */
    public function setIssuer($settings): array;

    /**
     * This Method to set Receiver to be submitted.
     *
     * @param $buyer
     * @param $invoice
     * @return array
     */
    public function setReceiver($buyer, $invoice): array;

    /**
     * This Method is used to validate the parameters of the invoice.
     *
     * @param $invoice
     * @param $merchant
     * @param $buyer
     * @param $vats
     * @return void
     */
    public function validateInvoice($invoice, $settings, $buyer, $vats): void;

    /**
     * This Method to get the invoice details.
     *
     * @param $invoiceDetails
     * @param $invoice
     * @param $totalSalesAmount
     * @param $totalItemsDiscountAmount
     * @param $settings
     * @param $totalDiscount
     * @param $invoiceLines
     * @param $taxes
     * @return array
     */
    public function invoiceDetails($invoiceDetails, $invoice, $totalSalesAmount, $totalItemsDiscountAmount, $settings, $totalDiscount, $invoiceLines, $taxes): array;

    /**
     * This Method to get the item vats.
     *
     * @param $item_
     * @param $vats
     * @param $taxableItems
     * @param $totalTaxableFees
     * @param $taxes
     * @param $net_price
     * @param $quantity
     * @return numeric-string $totalItemTax
     */
    public function invoiceVats($item_, $vats, $taxableItems, $totalTaxableFees, $taxes, $net_price, $quantity): string;

    /**
     * This Method is used to calculate the tax totals.
     *
     * @param $taxes
     * @return array $taxTotals
     */
    public function invoiceTaxTotal($taxes): array;

    /**
     * This Method is used to calculate totals.
     *
     * @param $item_price
     * @param $quantity
     * @param $discount
     * @param $totalItemTax
     * @return array
     */
    public function invoiceTotal($item_price, $quantity, $discount, $totalItemTax): array;
}
