<?php

namespace App\Services\Einvoice\Interfaces;

use Illuminate\Http\Response;
use Illuminate\Support\Collection;

interface UpdateDocument
{
//    /**
//     * Set the document in e-invoice format and log it in e-invoice stage table.
//     *
//     * @param mixed $bill
//     * @return array
//     */
//    public function initEinvoiceDocument($bill): array;

    /**
     * Initialize e-invoice Stage to submit to e-invoice service.
     *
     * @param $bill
     * @return $this
     */
    public function initEinvoiceStage($bill): self;

    /**
     * Get Stages From E-invoice Stage Table.
     *
     * @param $einvoiceLogs
     * @return $this
     */
    public function getEinvoiceStage($einvoiceLogs): self;

    /**
     * Get the submitted document data from e-invoice stage table.
     *
     * @param $bills
     * @return array
     */
    public function getEinvoiceStageDocument($bills): array;

    /**
     * Submit the documents to eInvoicing solution or retry to submit if failed.
     *
     * @param $bills
     * @return array | Response
     */
    public function submitEinvoice($bills): Response|array;

    /**
     * Update the database with the submitted document.
     *
     * @param $data
     * @param $invoices
     * @param $operationLogs
     * @return response
     */
    public function updateEinvoice($data, $invoices, $operationLogs): Response;

    /**
     * Taxpayer can submit one or more signed documents to eInvoicing solution using this Method.
     *
     * @param mixed $bills
     * @param string $cert_token
     * @return Response
     *
     * This Method Can Submit (Invoices, Credit Notes, Debit Notes).
     */
    public function submitDocuments($bills, $cert_token): Response;

    /**
     * Get the e-invoice status from log e-invoice stage table.
     *
     * @param $operationLogs
     * @param $invoices
     * @param $index
     * @param $data
     * @param $document
     * @return void
     */
    public function getEinvoiceStatus($operationLogs, $invoices, $index, $data, $document): void;

    /**
     * Get the invoice document from einvoice stage table.
     *
     * @param $operationLogs
     * @param $invoices
     * @param $index
     * @param $document
     * @return array
     */
    public function getEinvoiceDocument($operationLogs, $invoices, $index, $document): array;

    /**
     * Get the invoice details from einvoice stage table.
     *
     * @param $operationLogs
     * @param $invoices
     * @param $index
     * @param $document
     * @return array
     */
    public function getEinvoiceDetails($operationLogs, $invoices, $index, $document): array;

    /**
     * Check log einvoice stage table for invoice.
     * If the invoice is already exist in database then update the invoice.
     * If the invoice is not exist in database then create the invoice.
     *
     * @param $operationLogs
     * @param $invoices
     * @param $index
     * @param $details
     * @param $einvoice
     */
    public function saveEinvoice($operationLogs, $invoices, $index, $details, $einvoice);

    /**
     * Check log einvoice stage table for invoice.
     * If the invoice detials is already exist in database then update the invoice.
     * If the invoice details is not exist in database then create the invoice.
     *
     * @param $operationLogs
     * @param $invoices
     * @param $index
     * @param $details
     * @param $e_invoice
     */
    public function saveInvoiceDetail($operationLogs, $invoices, $index, $details, $e_invoice);

    /**
     * Check log e-invoice stage table for invoice.
     * If the invoice details vats is already exist in database then update the invoice.
     * If the invoice details vats is not exist in database then create the invoice.
     *
     * @param $operationLogs
     * @param $invoices
     * @param $index
     * @param $details
     * @param $e_invoice
     * @return void
     */
    public function saveInvoiceVat($operationLogs, $invoices, $index, $details, $e_invoice, $invoice_detail): void;

    /**
     * Check if items vats are added successfully or if there is no vats.
     *
     * @param $invoice_vat
     * @param $e_invoice
     * @param $invoices
     * @param $details
     * @param $index
     * @param $vats
     * @return void
     */
    public function checkInvoiceVat($invoice_vat, $e_invoice, $invoices, $details, $index, $vats): void;

    /**
     * This Method Update the Invoice in database.
     *
     * @param $data
     * @param $invoices
     * @return Response
     */
    public function updateEinvoiceTables($data, $invoices, $operationLogs): Response;
}
