<?php

namespace App\Services\Einvoice\ReceiptNew;

use App\Models\BranchAddressSetting;
use App\Models\BusinessSetting;
use App\Models\Invoice;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Str;
use Modules\Branch\Entities\Branch;
use Modules\Invoices\Entities\Invoices;

class InvoiceFormatter
{
    private  static  $instance;
    private array $businessSettings = [];
    private array $branchAddress = [];
    private function __construct()
    {
        //
    }

    public static function make()
    {
        return static::$instance = new InvoiceFormatter();
    }

    public function initialize(array $address)
    {
        $this->businessSettings = [];
        $this->branchAddress = [];
        if (!count($this->businessSettings)) {
            $this->businessSettings = BusinessSettingsService::make()->get();
        }
        if (!count($this->branchAddress)) {
            // $this->branchAddress = BranchAddressService::make()->get();
            $this->branchAddress = $address;
        }

        return $this;
    }

    public function format(Invoices $invoice, string $deviceID = "", Branch $branch)
    {
        $items = $invoice->items()->where('branch_id', $branch->id)->get();
        return [
                'header' => $this->formatHeader($invoice->closing_date, $invoice->ticket_id, ""),
                'documentType' => $this->formatDocument(),
                'seller' => $this->formatSeller($deviceID, $branch),
                'buyer' => [
                    'type' => 'P'
                ],
                'itemData' => InvoiceItemFormatter::make()->handle($items, $this->businessSettings),
                // 'totalSales' => number_format($invoice->sub_total, 2, '.', ''), //subtotal
                // 'totalSales' => round($invoice->sub_total, 1), //subtotal
                'totalSales' => round($items->sum('total'), 1), //subtotal
                'totalCommercialDiscount' => round(0, 1), //total discounts,
                // 'taxTotals' => $this->formatTax($invoice->tax),
                // 'taxTotals' => $this->formatTax($invoice->total * 0.14),
                'paymentMethod' => $this->businessSettings['paymentMethod'],
                // 'netAmount' => number_format($invoice->sub_total, 2, '.', ''), //subtotal
                // 'netAmount' => round($invoice->sub_total, 1), //subtotal
                'netAmount' => round($items->sum('total'), 1), //subtotal
                // 'feesAmount' => number_format($invoice->fees, 2, '.', ''), //service charge,
                'feesAmount' => round(0, 1),
                // 'totalAmount' => number_format($invoice->total, 2, '.', '') //total price
                // 'totalAmount' => round($invoice->total, 1) //total price
                'totalAmount' => round($items->sum('total'), 1) //total price
            ];
    }
    public function formatSeller($terminalId, Branch $branch)
    {
        return [
            'rin' => $this->businessSettings['rin'],
            'companyTradeName' => $this->businessSettings['companyTradeName'],
            // 'branchCode' => $this->businessSettings['branchCode'],
            'branchCode' => $branch->branch_code,
            'branchAddress' => $this->formatBranchAddress(),
            'deviceSerialNumber' => $terminalId, //Terminal ID
            // 'activityCode' => $this->businessSettings['activityCode']
            'activityCode' => $branch->activity_code
        ];
    }
    public function formatBranchAddress()
    {
        return $this->branchAddress;
    }
    public function formatTax($taxAmount)
    {
        //get business settings taxType
        return [
            'taxType' => $this->businessSettings['taxType'],
            'amount' => $taxAmount //total tax
        ];
    }
    public function formatDocument()
    {
        return [
            'receiptType' => $this->businessSettings['receiptType'],
            'typeVersion' => $this->businessSettings['typeVersion']
        ];
    }
    public function formatHeader(string $date, string $receiptNumber, string $previous = "")
    {

        return [
            'dateTimeIssued'    => Carbon::parse($date)->toIso8601ZuluString(), //closing Date
            'receiptNumber'     => $receiptNumber,
            'uuid'              => '',
            // 'previousUUID'      => !empty($previous) ? $this->generateUUID($previous) : "",
            'previousUUID'      =>  "",
            'currency'          => 'EGP',
            "exchangeRate"      => 0.0,
            // 'orderdeliveryMode' => $this->businessSettings['Order Delivery Mode'],
        ];
    }
    public function generateUUID(string $id)
    {
        // return Str::uuid();
        // return hash('sha256', $id);
        return hash('sha256', Str::uuid());
    }
    public static function flattenJson(array $data)
    {

        $str = "";
        foreach ($data as $key => $value) {
            $str .= "\"" . Str::upper($key) . "\"";
            if (is_array($value)) {

                if (is_array($value[\array_key_first($value)]) && is_array($value[\array_key_last($value)]))
                    $str .= static::flattenArray($key, $value);
                else
                    $str .= static::flattenJson($value);
            } else {
                if (gettype($value) == 'double') {
                    $str .= '"' . number_format($value, 1, '.', '') . '"';
                } else
                    $str .= "\"" . $value . "\"";
            }
        }
        return $str;
    }
    public static function flattenArray(string $key, array $values)
    {
        $str = "";
        foreach ($values as $value) {
            $str .= "\"" . Str::upper($key) . "\"";
            if (is_array($value)) {
                $str .= static::flattenJson($value);
            } else {
                if (gettype($value) == 'double') {
                    $str .= '"' . number_format($value, 1, '.', '') . '"';
                } else
                    $str .= "\"" . $value . "\"";
            }
        }
        return $str;
    }
}
