<?php
    namespace App\Services;

    use App\Services\Updaters\InvoiceNegSafe;
    use Modules\Invoices\Entities\Invoices;
    use Modules\Invoices\Http\Requests\NewInvoiceRequest;

    abstract class Invoice{

        private $request, $invoice;
        protected $type, $safe, $client;
        protected $invoiceTotalDisc;

        public function __construct(NewInvoiceRequest $request)
        {
            $this->request = $request;
            $this->checkPermissions();
        }

        private function checkPayment()
        {
            if (! authorized('can_change_payment'))  // permission check
            {
                if ($this->request->pay_method == 1)
                {
                    $this->request->paid = $this->request->net;
                }
                else {
                    $this->request->paid = 0;
                }
            }
        }

        private function checkDate()
        {
            if (! authorized('can_change_sale_invoice_date'))  // permission check
            {
                $this->request->invoice_date = (date('Y-m-d'). ' ' . date('H:i:s'));
            }
            else {
                if (!$this->request->invoice_date)
                {
                    $this->request->invoice_date = date('Y-m-d');
                }
                $this->request->invoice_date .= ' ' . date('H:i:s');
            }
        }

        private function checkPermissions()
        {
            $this->checkDate();
            $this->checkPayment();
        }

        public function getRequest()
        {
            return $this->request;
        }

        public function create()
        {
            return $this->createInvoice()->MapInvoiceNo();
        }

        private function createInvoice()
        {
            $this->invoice = Invoices::createNew($this, $this->client, $this->safe);

            return $this;
        }

        public function getType()
        {
            return $this->type;
        }

        public function getInvoice()
        {
            return $this->invoice;
        }

        public function getClient()
        {
            return $this->client;
        }

        public function getSafe()
        {
            return $this->safe;
        }

        public function getCashePayment()
        {
            return $this->cash_paid;
        }

        public function getNetworkPayment()
        {
            return $this->network_paid;
        }

        private function MapInvoiceNo()
        {
            $generate_by = get_session_val('billno_show', 'integer');
            $depend_on = get_session_val('no_bill_type', 'integer');

            if ($generate_by == 1 && $this->request->bill_no)
            {
                $this->validateInvoiceNo($depend_on);
            }
            else {
                $this->getInvoiceNo($depend_on, $generate_by);
            }
            $this->invoice->save();
            return $this;
        }

        private function validateInvoiceNo($depend_on)
        {
            $this->invoice->bill_no = validate_bill_no($this->type, $this->request->bill_no, $depend_on, $this->request->store_id, $this->invoice->id);
        }

        private function getInvoiceNo($depend_on, $generate_by)
        {
            $this->invoice->bill_no = get_invoice_no($this->type, $generate_by ,$this->request->invoice_date, '', $depend_on , $this->request->store_id, $this->invoice->id );
        }

        public function setTotalDisc($total_disc)
        {
            $this->invoiceTotalDisc += $total_disc;
        }

        public function explodeValue($value, $offset) {
            $period = explode('-', $value);
            return $period[$offset] ?? 0;
        }

        public function updateTotalDisc()
        {
            $this->invoice->update([
                'invoice_total_disc' =>  get_session_val('company_ratio', 'integer') == 1 ? $this->invoiceTotalDisc :
                    ( get_session_val('company_ratio', 'integer') == 2 ?
                        ($this->request->invoice_discount_type == 1 ? $this->request->invoice_discount :
                        ($this->request->invoice_discount > 0 ? ($this->request->invoice_total - $this->request->calculated_total) : 0)) : 0),
            ]);
        }

    }
?>
