<?php

namespace App\Services\Location;

class GeoLocationController
{
    const _URL = "http://www.geoplugin.net/json.gp?ip=";

    public static function index()
    {
        try {
            return self::display()->Results;
        } catch (\Exception $e) {
            return self::error($e);
        }
    }
    public static function take($data)
    {
        return new GeoLocation($data);
    }

    public static function display()
    {

        return self::take([])->getMyIP()->discover();
    }

    public static function error($e): string
    {
        return $e->getMessage() . ' on line: ' . $e->getLine() . ' in file: ' . $e->getFile() . ' with code: ' . $e->getCode();
    }

    public static function getMyIP($data): object
    {
        /*
        if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
            $address = $_SERVER['HTTP_CLIENT_IP'];
        } else if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $address = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $address = $_SERVER['REMOTE_ADDR'];
        }
        */
        // $address = file_get_contents("https://ipecho.net/plain") ?? '127.0.0.1';

        $address = request()->ip();
        // dd($address);

        $data['address'] = $address;
        return (object)$data;
    }

    public static function discover($data)
    {
        return self::fetch($data);
    }

    public static function fetch($data)
    {
        $results = self::call($data);

        return self::arrange($data, $results);
    }

    public static function arrange($data, $results)
    {

        $address = $data->address;
        if(gettype($results) == 'object'){

            if(isset($results->geoplugin_status) && ! in_array($results->geoplugin_status, range(200,210)) ){
                $country = 'invalid_ip';
            }else{
                $country = $results->geoplugin_countryName;
                    $address = $results->geoplugin_request;
            }
        }else{
            $country = 'invalid_calling';
            $results = (object)[];
        }
        $data->country = $country ?? '';
        $data->address = $address ?? '';

        $data->geolocation = $results;
        return $data;
    }

    public static function call($data)
    {
        // dd($data);
        
        try {
            $url = self::_URL . $data->address;
            // $url = self::_URL;
            
            // dd($url);
            $curl = curl_init();

            curl_setopt_array($curl, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => [],
                CURLOPT_HTTPHEADER => array(
                    'Content-Type: application/json'
                ),
            ));
            $response = curl_exec($curl);

            $res = json_decode($response);

            if ($response != false && $res && isset($res->status)) {
                $res = curl_errno($curl);
            }
            curl_close($curl);
            return (object)$res;
        } catch (\Exception $e) {
            return (object)['error' => self::error($e)];
        }
    }

}
