<?php
    namespace App\Services\Updaters;

    use App\Services\InvoiceDetail;
    use Modules\Invoices\Entities\StoreItem;
    use Modules\Invoices\Entities\ItemUnit;

    abstract class InvoiceStoreItemUpdater{
        protected $item_id, $store_id, $store_item, $item_unit_id, $invoice_detail, $new_qty, $old_qty;
        public $item;

        public function __construct(int $item_id,int $store_id, int $item_unit_id, InvoiceDetail $invoice_detail)
        {
            $this->item_id = $item_id;
            $this->store_id = $store_id;
            $this->item_unit_id = $item_unit_id;
            $this->invoice_detail = $invoice_detail;
            $this->getOrCreate();
            $this->old_qty = $this->store_item->store_quant;
            $this->item = \DB::table('items')->find($item_id);
        }

        private function getOrCreate()
        {
            $store_item = StoreItem::where('item_id', $this->item_id)->where('store_id', $this->store_id)->first();
            if (is_null($store_item))
            {
                $this->intializeNewStoreItem();
            }
            else {
                $this->store_item = $store_item;
            }
        }

        private function intializeNewStoreItem()
        {
            $this->store_item = StoreItem::createNew($this);
        }

        protected function updateQty()
        {
            $this->store_item->update([
                'store_quant' => $this->new_qty,
                'edit_user' => auth()->user()->id,
                'edit_date' => date('Y-m-d H:i:s')
            ]);
        }

        protected function getItemUnitValue()
        {
            $item_unit = ItemUnit::where([
                'shop_id' => get_session_val('serial_id', 'Integer'),
                'item_id' => $this->item_id,
                'unit_id' => $this->invoice_detail->input_details[2]
            ])->first();

            return floatVal($item_unit->unit_value ?? 1);
        }

        public function getNewQty()
        {
            return $this->store_item->store_quant;
        }

        public function getItemId()
        {
            return $this->item_id;
        }
        
        public function getStoreId()
        {
            return $this->store_id;
        }
        
        abstract protected function setNewQty();
        abstract public function update();
    }
?>