<?php
    namespace App\Services\Uploaders;

    use Illuminate\Http\Request;
    use Illuminate\Support\Facades\Storage;

    abstract class ImgUploader{

        protected const ROOT = 'uploads';
        protected $shop_id, $request, $key;

        public function __construct(Request $request, string $key)
        {
            $this->shop_id = get_session_val('serial_id', 'integer');
            $this->request = $request;
            $this->key = $key;
        }

        static protected function getCurrentYear()
        {
            return date('Y');
        }

        static protected function getCurrentMonth()
        {
            return (int) date('m');
        }

        static protected function getPathSperator()
        {
            return '/';
        }

        protected function getPath()
        {
            return self::ROOT . self::getPathSperator() .  $this->shop_id . self::getPathSperator() . 
                   self::getCurrentYear() . self::getPathSperator() . self::getCurrentMonth() . self::getPathSperator(). $this->getDir();
        }

        protected function getUniqueName()
        {
            return date("Y-m-d") . '_' . time();
        }

        protected function getExtension()
        {
            return $this->request->file( $this->key )->getClientOriginalExtension();
        }

        protected function getNewNameWithExtension()
        {
            return $this->getUniqueName() . '.' . $this->getExtension();
        }

        static public function clean($path)
        {
            if($path && file_exists($path))
            {
                unlink($path);
            }
        }

        public function isValid()
        {
            if ($this->request->has( $this->key )) {
                return true;
            }
            return false;
        }

        protected function upload()
        {
            $path = $this->getPath();
            $this->request->file($this->key)->move(public_path($path), $this->getNewNameWithExtension());
            return $path . self::getPathSperator() . $this->getNewNameWithExtension();
        }

        public function checkAndUpload()
        {
            if ($this->isValid())
            {
                return [ 'status' =>  true, 'path' =>  $this->upload()];
            }
            
            return [ 'status' =>  false, 'path' => ''];
        }
        
        // protected to force child class to specify its root dir ( forggetten not allowed here)
        abstract protected function getDir();

    }
?>