<?php

namespace App\Traits\Einvoice;

use Illuminate\Support\Str;
use Ixudra\Curl\Facades\Curl;
use App\Traits\Einvoice\AuthTrait;

trait NotificationTrait
{
    use AuthTrait;

    /**
     * This Method allow eInvoicing to verify during registration of the system notifications that API is accessible from the eInvoicing solution.
     *
     * @param  string $rin
     * @return string $response
     */
    public function ping($rin)
    {
        // Login to the Taxpayer API
        $this->token = in_array($this->token, [null, '']) ? $this->loginTaxpayer() : $this->token;

        // Send a request to the Taxpayer API
        $response = Curl::to(self::$apiBaseUrl . '/ping')
            ->withData(['rin' => $rin])
            ->withHeader('Accept-Language', app()->getLocale())
            ->withBearer($this->token)
            ->returnResponseObject()
            ->asJson(true)
            ->put();

        $data = $response->content;

        // Data from the response
        return response($data, $response->status);
    }

    /**
     * This Method allows ERP system to query for previously received notifications.
     *
     * @param queryParams
     * @return notifications
     */
    public function getNotification($query)
    {
        // Login to the Taxpayer API.
        $this->token = in_array($this->token, [null, '']) ? $this->loginTaxpayer() : $this->token;

        $url = in_array($query, [' ', '', null])
            ? self::$apiBaseUrl . '/api/v1.0/notifications/taxpayer'
            : self::$apiBaseUrl . '/api/v1.0/notifications/taxpayer?' . $query;

        // Send a request to the Taxpayer API.
        $response = Curl::to($url)
            ->withBearer($this->token)
            ->withContentType('application/json')
            ->returnResponseObject()
            ->asJson(true)
            ->get();

        $data = $response->content;

        // Data from the response
        return $data;
    }

    /**
     * This Method enables ERP system to receive notifications on document related events - validation, issuance, rejection and cancellation.
     *
     * @param document
     * @param count
     * @return notifications
     */
    public function receiveDocumentNotifications($document, $count = 1)
    {
        // Login to the Taxpayer API
        $this->token = in_array($this->token, [null, '']) ? $this->loginTaxpayer() : $this->token;

        $body = [
            "deliveryId" => Str::random(10),
            "type"       => "document",
            "count"      => $count,
            "message"    => [
                "type"           => $document['type'],
                "uuid"           => $document['uuid'],
                "longId"         => $document['longId'],
                "status"         => $document['status'],
                "internalId"     => $document['internalId'],
                "submissionUUID" => $document['submissionUUID'],
            ]
        ];

        // Send a request to the Taxpayer API.
        $response = Curl::to(self::$apiBaseUrl . '/notifications/documents')
            ->withData($body)
            ->withBearer($this->token)
            ->withHeader('Accept-Language', app()->getLocale())
            ->withContentType('application/json')
            ->returnResponseObject()
            ->asJson(true)
            ->put();

        $data = $response->content;

        // Data from the response
        return $data;
    }

    /**
     * This Method allows ERP system to receive notification when document package is ready for download.
     *
     * @param count
     * @return notifications
     */
    public function receiveDownloadReadyNotification($count = 1)
    {
        // Login to the Taxpayer API
        $this->token = in_array($this->token, [null, '']) ? $this->loginTaxpayer() : $this->token;

        $body = [
            "deliveryId" => Str::random(10),
            "type"       => "document-package-ready",
            "count"      => $count,
            "message"    => [
                "type"      => "document-package-ready",
                "packageId" => Str::random(10),
            ]
        ];

        // Send a request to the Taxpayer API.
        $response = Curl::to(self::$apiBaseUrl . '/notifications/documentpackages')
            ->withData($body)
            ->withBearer($this->token)
            ->withHeader('Accept-Language', app()->getLocale())
            ->withContentType('application/json')
            ->returnResponseObject()
            ->asJson(true)
            ->put();

        $data = $response->content;

        // Data from the response
        return $data;
    }
}
