<?php

namespace App\Traits\Einvoice;

use App\Traits\Einvoice\ReceiptDocumentTrait;
use Ixudra\Curl\Facades\Curl;

trait ReceiptTrait
{
    use ReceiptDocumentTrait;

    /**
     * This Method allows Taxpayer can submit one or more signed documents to eReceipt solution using this API.
     *
     * @param $bills
     * @return $data
     */
    public function submitReceipt($bills)
    {
        // Login to POS API
        $this->posToken = in_array($this->posToken, [null, '']) ? $this->authenticatePOS() : $this->posToken;

        // Set Document to be submitted
        $receipts = $this->setReceipt($bills);

        $body = [
            "receipts" => $receipts->values()->toArray(),
        ];

        // Send a request to the POS API
        $response = Curl::to(self::$apiBaseUrl . '/api/v1/receiptsubmissions')
            ->withData($body)
            ->withBearer($this->posToken)
            ->withContentType('application/json')
            ->returnResponseObject()
            ->asJson(true)
            ->post();

        $data = $response->content;

        // Update the invoice in database.
        if (isset($data['acceptedDocuments'])) {
            foreach ($data['acceptedDocuments'] as $index => $document) {
                // Update the invoice in database.
                $invoice  = Invoice::where('id', $receipts->keys()[$index])->first();
                $invoice->update([
                    'submissionId'  => $data['submissionId'],
                    'uuid'          => $document['uuid'],
                    'longId'        => $document['longId'],
                    'internalId'    => $document['internalId'],
                    'hashKey'       => $document['hashKey'],
                    'einvoice_type' => 3,
                ]);
            }
        }

        // Data from the response
        return $data;
    }

    /**
     * This Method allows Taxpayer can submit one or more signed documents to eReceipt solution using this API.
     *
     * @param $bills
     * @return $data
     */
    public function submitReturnReceipt($bills)
    {
        // Login to POS API
        $this->posToken = in_array($this->posToken, [null, '']) ? $this->authenticatePOS() : $this->posToken;

        // Set Document to be submitted
        $receipts = $this->setReceipt($bills);

        $body = [
            "receipts" => $receipts
        ];

        // Send a request to the POS API
        $response = Curl::to(self::$apiBaseUrl . '/api/v1/receiptsubmissions')
            ->withData($body)
            ->withBearer($this->posToken)
            ->withContentType('application/json')
            ->returnResponseObject()
            ->asJson(true)
            ->post();

        $data = $response->content;

        // Data from the response
        return $response->status == 200
            ? $data
            : $data['error'];
    }
}
