<?php

namespace App\Traits;

use Carbon\Carbon;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Lang;
use Laravel\Passport\HasApiTokens;
use phpDocumentor\Reflection\Types\False_;


trait HelperTrait
{
    use HasApiTokens;
    /**
     *  Store incoming photo
     * //@return photo
     * @var $folder
     * @var $photo
     */

    public function maxStore($shop_id)
    {

        $items = DB::table('store_items')
            ->where('store_items.shop_id', '=', $shop_id)
            ->select('store_items.item_id', 'store_items.store_id', 'store_items.shop_id', 'store_items.store_quant')
            ->join('stores', 'stores.id', '=', 'store_items.store_id')
            ->join('request_settings', 'request_settings.shop_id', '=', 'store_items.shop_id')
            ->select('request_settings.max_items as max_item', 'request_settings.ignore_quantity', 'store_items.item_id', 'store_items.store_id', 'store_items.store_quant', 'stores.store_name')
            ->get();
        return $items;
    }

    public static function sendNote($tokens = [], $contents, $data = [])
    {
        //FCM api URL
        $url = 'https://fcm.googleapis.com/fcm/send';
        //api_key available in Firebase Console -> Project Settings -> CLOUD MESSAGING -> Server key
        // $server_key = 'AIzaSyB54Fa7ZfuRjL-yl87y2CBRCxygBgjZIbY';
        $server_key = 'AAAAEQc33f0:APA91bEi416xh-Ab8fr-EBkVUeE1ecgrMxdHMAewaNLXWsMlp1ukC18dmkeyiCMR3fodpNpNLsD-alGd9xsFU3H6n9inmck--WmTiecHYb1btRJVYifL0FSKAkQk_0Ku_EUoTywJ3XYe';

        //header with content_type api key
        $headers = array(
            'Content-Type:application/json; charset=utf-8',
            'Authorization:key=' . $server_key
        );

        $notification = [
            'body' => $contents,
            'click_action' => 'MAIN_ACTIVITY',
            'sound' => true,
        ];

        $fields = [
            'registration_ids' => $tokens, //multple token array
            //        'to'        => $tokens, //single token
            'notification' => $notification,
            'data' => $data
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($fields));
        $result = curl_exec($ch);
        curl_close($ch);

        return $result;
    }


    public function notifyByFirebase($title, $body, $tokens, $data = [], $type = null, $sound = 'default')        // paramete 5 =>>>> $type
    {

        $registrationIDs = $tokens;

        // 'vibrate' available in GCM, but not in FCM
        $fcmMsg = array(
            'body' => $body,
            'title' => $title,
            'sound' => $sound,
            'click_action' => 'MAIN_ACTIVITY',
            'color' => "#203E78"
        );
        // I haven't figured 'color' out yet.
        // On one phone 'color' was the background color behind the actual app icon.  (ie Samsung Galaxy S5)
        // On another phone, it was the color of the app icon. (ie: LG K20 Plush)

        // 'to' => $singleID ;      // expecting a single ID
        // 'registration_ids' => $registrationIDs ;     // expects an array of ids
        // 'priority' => 'high' ; // options are normal and high, if not set, defaults to high.
        $fcmFields = array(
            'registration_ids' => $registrationIDs,
            'priority' => 'high',
            'notification' => $fcmMsg,
            'data' => $data
        );
        //dd(env('FIREBASE_API_ACCESS_KEY'));
        $server_key = 'AAAAEQc33f0:APA91bEi416xh-Ab8fr-EBkVUeE1ecgrMxdHMAewaNLXWsMlp1ukC18dmkeyiCMR3fodpNpNLsD-alGd9xsFU3H6n9inmck--WmTiecHYb1btRJVYifL0FSKAkQk_0Ku_EUoTywJ3XYe';


        $headers = array(
            'Authorization:key=' . $server_key,
            'Content-Type: application/json; charset=utf-8'
        );

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://fcm.googleapis.com/fcm/send');
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($fcmFields));
        $result = curl_exec($ch);
        info($result);
        curl_close($ch);
        return $result;
    }

    /**
     * @param $status
     *
     * @param $message
     * @param array $data
     * @param null $resource
     * @return \Illuminate\Http\JsonResponse
     */
    // public function responseJson($status, $message, $data =null, $resource = null)
    // {
    //     if ($resource) {
    //         return $resource->additional([
    //             'status' => $status,
    //             'message' => $message,
    //             'additional_data' => $data,
    //             'lang' => Lang::getLocale(),
    //             'snap_availability' => auth()->guard('api')->user()->snap_availability ?? false
    //         ]);
    //     }

    //     if ($status == 3){
    //         $response = [
    //             'status' => $status,
    //             'message' => $message,
    //             'errors' => $data,
    //             'lang' => Lang::getLocale(),
    //             'snap_availability' => auth()->guard('api')->user()->snap_availability ?? false
    //         ];
    //     }else {

    //         $response = [

    //             'status' => $status,
    //             'message' => $message,
    //             'data' => $data,
    //             'lang' => Lang::getLocale(),
    //             'snap_availability' => auth()->guard('api')->user()->snap_availability ?? false
    //         ];
    //     }




    //     return response()->json($response);
    // }

    
    public function returnErrors($validator)
    {
        $errors = $validator->errors();
        $error_data = [];
        foreach ($errors->all() as $error) {
            array_push($error_data, $error);
        }
        $data = $error_data;
        return $data;
    }
    public function responseJson($status, $message = null, $data = null, $token = null, $status_number)
    {
        return response()->json([
            'status' => $status,
            'message' => $message,
            'data' => $data,
            'token' => $token
        ], $status_number);
    }

    function saveImage($image, $folder)
    {
        if (! $image) return '';
        $day = Carbon::today()->day;
        $year = Carbon::today()->year;
        $month = Carbon::today()->month;
        $destinationFolder = $folder . '/' . $year . '/' . $month . '/' . $day . '/';
        $destinationPath = public_path($destinationFolder);
        File::isDirectory($destinationFolder) or File::makeDirectory($destinationPath, 0755, true, true);
        $file_extension = strtolower($image->getClientOriginalExtension());
        $file_name = strtolower($image->getClientOriginalName());
        $file_name = $destinationFolder .time() . rand(1111, 9999) . '.' . $file_extension;
        $image->move($destinationPath, $file_name);
        return $file_name;
    }

    function updateImage($image, $folder, $old_image)
    {
        if ($old_image) {
            if (file_exists(public_path() . '/' . $old_image)) {
                unlink(public_path() . '/' . $old_image);
            }
        }
        $day = Carbon::today()->day;
        $year = Carbon::today()->year;
        $month = Carbon::today()->month;
        $destinationFolder = $folder . '/' . $year . '/' . $month . '/' . $day . '/';
        $destinationPath = public_path($destinationFolder);
        File::isDirectory($destinationPath) or File::makeDirectory($destinationPath, 0755, true, true);
        $file_extension = strtolower($image->getClientOriginalExtension());
        $file_name = $destinationFolder .time() . rand(1111, 9999) . '.' . $file_extension;
        $image->move($destinationPath, $file_name);
        return $file_name;
    }

    /**
     *  Delete Image
     * @param string $image
     * @param string $image_folder
     * @return void
     */
    function deleteImage($image, $image_folder)
    {

        // Find the path for this image
        $image_path = public_path() . '/' . $image_folder . '/' . $image;

        // Delete image
        File::delete($image_path);
    }


    // Soft delete for products
    protected function paginated($items, $perPage)
    {
        $pageStart = request('page', 1);
        $offSet = ($pageStart * $perPage) - $perPage;
        $itemsForCurrentPage = $items->slice($offSet, $perPage);

        return new LengthAwarePaginator(
            $itemsForCurrentPage,
            $items->count(),
            $perPage,
            Paginator::resolveCurrentPage(),
            ['path' => Paginator::resolveCurrentPath()]
        );
    }

    function paginateTheResult($items, $perPage)
    {
        $pageStart = request('page', 1);
        $offSet = ($pageStart * $perPage) - $perPage;
        $itemsForCurrentPage = $items->slice($offSet, $perPage);

        $itemsForCurrentPage = $itemsForCurrentPage->values();

        return new LengthAwarePaginator(
            $itemsForCurrentPage,
            $items->count(),
            $perPage,
            Paginator::resolveCurrentPage(),
            ['path' => Paginator::resolveCurrentPath()]
        );
    }


    function generateRandomString($length = 3)
    {
        $characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[random_int(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    public function setNotificationLocal($client)
    {
        app()->setLocale($client->lang);
    }
}
