<?php

namespace App\helpers;

use Ixudra\Curl\Facades\Curl;
use App\Traits\Einvoice\ReceiptTrait;
use App\Traits\Einvoice\ItemCodeTrait;
use App\Traits\Einvoice\NotificationTrait;
use App\Traits\Einvoice\Invoice\InvoiceTrait;

class Einvoice
{
    use InvoiceTrait, ReceiptTrait, NotificationTrait, ItemCodeTrait;

    /**
     * This Method allows taxpayers to retrieve information on previous document package requests, their status.
     *
     * @param number $pageNo
     * @param number $pageSize
     * @return $data
     */
    public function getPackageRequests($pageSize = 20, $pageNo = 1)
    {
        // Login to the Taxpayer API.
        $this->loginTaxpayer();

        // Send a request to the Taxpayer API.
        $response = Curl::to(self::$apiBaseUrl . '/api/v1/documentPackages/requests?pageSize=' . $pageSize . '&pageNo=' . $pageNo)
            ->withBearer($this->token)
            ->withContentType('application/json')
            ->returnResponseObject()
            ->asJson(true)
            ->get();

        $data = $response->content;

        // Data from the response
        return response($data, $response->status);
    }

    /**
     * This Method allows taxpayer systems to download previously requested document packages as XML or JSON files (compressed).
     *
     * @param string $rid (Unique ID of the package)
     * @return $data
     */
    public function getDocumentPackage($rid)
    {
        // Login to the Taxpayer API.
        $this->loginTaxpayer();

        // Send a request to the Taxpayer API.
        $response = Curl::to(self::$apiBaseUrl . '/api/v1/documentPackages/' . $rid)
            ->withBearer($this->token)
            ->returnResponseObject()
            ->get();

        $data = $response->content;

        // Data from the response
        return response($data, $response->status);
    }

    /**
     * This Method allows taxpayer system to request larger packages of previously sent or received documents to be prepared for download.
     *
     * @param array $data
     * @return $data
     */
    public function requestDocumentPackage($data)
    {
        // Login to the Taxpayer API.
        $this->loginTaxpayer();

        // Send a request to the Taxpayer API.
        $response = Curl::to(self::$apiBaseUrl . '/api/v1/documentPackages/requests')
            ->withData($data)
            ->withBearer($this->token)
            ->withContentType('application/json')
            ->returnResponseObject()
            ->asJson(true)
            ->post();

        $data = $response->content;

        // Data from the response
        return response($data, $response->status);
    }
}
