<?php

use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Modules\Invoices\Entities\Invoices;
use Illuminate\Support\Facades\DB;
use Modules\ClientInstallments\Entities\Instalment;
use Modules\Invoices\Entities\InvoiceDetails;
use Modules\Invoices\Entities\Item;
use Modules\Invoices\Entities\ItemUnit;

    function get_invoice_no($invoice_type = 1, $generate_by = 0, $invoice_date = '', $eg_vat = '', $depend_on = 0, $store_id = 0, $except_id = 0) :int
    {
        /*
            invoice_type [
                'sales' => 1,
                'returned sales' => 3
            ]
        */
        /*
            generate_by [
                'auto' => 0,
                'manule' => 1,
                'day' => 2,
                'month' => 3,
                'year' => 4,
            ]
        */

        $invoice_date = Carbon::parse($invoice_date);
        $last_no = Invoices::where([
                        ['shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoices_.invoice_type', $invoice_type]
                    ])
                    ->when(is_integer($except_id), function($q) use($except_id){
                        $q->where('invoices_.id', '<', $except_id);
                    })
                    ->when($generate_by == 2, function($q) use($invoice_date){
                        $q->whereDate('invoices_.invoice_date', $invoice_date->format('Y-m-d'));
                    })
                    ->when($generate_by == 3, function($q) use($invoice_date){
                        $q->where(DB::raw('YEAR(invoices_.invoice_date)'), $invoice_date->format('Y'));
                        $q->where(DB::raw('Month(invoices_.invoice_date)'), $invoice_date->format('m'));
                    })
                    ->when($generate_by == 4, function($q) use($invoice_date){
                        $q->where(DB::raw('YEAR(invoices_.invoice_date)'), $invoice_date->format('Y'));
                    })
                    ->when($depend_on == 1, function($q) use($store_id){
                        $q->where('invoices_.store_id', $store_id);
                    })
                    ->select(['bill_no'])
                    ->orderBy('invoices_.id', 'DESC')
                    ->limit(1)
                    ->first();

        $invoice_no = ($last_no->bill_no ?? 0) + 1;

        if ($generate_by == 1) {
            $invoice_no = request()->get('bill_no');
            $check = Invoices::select('bill_no')->where(['shop_id' => get_session_val('serial_id', 'Integer'), 'invoices_.invoice_type' => $invoice_type])
                            ->when($depend_on == 1, fn($q) => $q->where('invoices_.store_id', $store_id) )
                            ->where('invoices_.id', '!=', $except_id)
                            ->where('invoices_.bill_no', $invoice_no)
                            ->first();
            if ($check || blank($invoice_no)) {
                $invoice_no = Invoices::where(['shop_id' => get_session_val('serial_id', 'Integer'), 'invoices_.invoice_type' => $invoice_type])->max('bill_no') + 1;
            }
        }

        // $cash_temp_key = get_session_val('serial_id', 'Integer') . '_max_invoice_id_' . 'temp';

        // if ($invoice_no <= Cache::get( $cash_temp_key))
        // {
        //     if ($invoice_no <= Cache::get( $cash_temp_key))
        //     {
        //         $invoice_no = Cache::get( $cash_temp_key) + 1;
        //     }
        // }

        // Cache::put( $cash_temp_key, $invoice_no );

        return $invoice_no;
    }

    function validate_bill_no($invoice_type, $invoice_no, $depend_on, $store_id, $except_id = 0) :int
    {
        $no_exists = Invoices::where([
            ['shop_id', get_session_val('serial_id', 'Integer')],
            ['invoices_.invoice_type', $invoice_type],
            ['invoices_.bill_no', $invoice_no]
        ])
        ->when($except_id, function($q) use($except_id){
            $q->where('invoices_.id', '!=', $except_id);
        })
        ->when($depend_on == 1, function($q) use($store_id){
            $q->where('invoices_.store_id', $store_id);
        })
        ->select(['id'])
        ->limit(1)->first();
        
        return ( !is_null($no_exists) ? get_next_invoice_no($invoice_type, $depend_on, $store_id) : $invoice_no);
    }

    function get_next_invoice_no($invoice_type, $depend_on, $store_id)  :int
    {
        $last_bill_no = Invoices::where([
            ['shop_id', get_session_val('serial_id', 'Integer')],
            ['invoices_.invoice_type', $invoice_type],
        ])
        ->when($depend_on == 1, function($q) use($store_id){
            $q->where('invoices_.store_id', $store_id);
        })
        ->select(DB::raw('MAX(bill_no) AS last_bill_no'))
        ->first();

        return ($last_bill_no->last_bill_no ?? 0) + 1;
    }

    function update_purchase_invoice_sold_qty($invoice_type, $item_id, $quantity, $unit_id, $client_id, $returned_pay_price) :float
    {
        $actual_pay_price = 0.0;

        $unit_value = ItemUnit::where([
            ['shop_id', get_session_val('serial_id', 'Integer')],
            ['unit_id', $unit_id],
            ['item_id', $item_id]
        ])->limit(1)->value('unit_value') ?? 1;

        $quantity *= $unit_value;

        if ($invoice_type == 'sales')
        {
            // decrease quantity
            $set_qty = $quantity;
            DB::transaction(function () use($item_id, $set_qty, &$actual_pay_price){
                $temp_set_qty = 0;

                $start_qty = Invoices::leftJoin('invoices_details', 'invoices_details.invoice_id', 'invoices_.id')
                            ->leftJoin('items_unit', function($q){
                                $q->on('items_unit.id', 'invoices_details.unit_id');
                                $q->where('items_unit.item_id', 'invoices_details.item_id');
                            })
                            ->where([
                                ['invoices_.invoice_type', 7],
                                ['invoices_details.item_id', $item_id],
                                ['invoices_details.not_sold', '>', '0'],
                                ['invoices_.shop_id', get_session_val('serial_id', 'Integer')]
                                ])
                                ->select(DB::raw('if (items_unit.id > 0, items_unit.unit_value, 1) AS unit_value, invoices_details.id'))
                            ->first();

                if ($start_qty)
                {
                    $updated_invoice_detasils = InvoiceDetails::find($start_qty->id);
                    $temp_unit_value = $start_qty->unit_value;

                    if ($set_qty <= ($updated_invoice_detasils->not_sold * $temp_unit_value))
                    {
                        // update and finish
                        $updated_invoice_detasils->not_sold = $set_qty;
                        $updated_invoice_detasils->save();
                        $temp_set_qty = $set_qty;
                        $set_qty = 0;
                    }
                    else {

                        // update & decrease set_qty
                        $temp_set_qty = ($updated_invoice_detasils->not_sold);
                        $set_qty -= $temp_set_qty;
                        $updated_invoice_detasils->not_sold = 0;
                        $updated_invoice_detasils->save();
                    }

                    $actual_pay_price += ($updated_invoice_detasils->pay_price/ $temp_unit_value) * $temp_set_qty;
                }

                // then decrease it from purchases invoices
                while($set_qty > 0)
                {
                    $invoice_details = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                    ->leftJoin('items_unit', function($q){
                        $q->on('items_unit.id', 'invoices_details.unit_id');
                        $q->where('items_unit.item_id', 'invoices_details.item_id');
                    })
                    ->where([
                        ['invoices_.shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoices_.invoice_type', 0],
                        ['invoices_details.item_id', $item_id],
                        ['invoices_details.not_sold', '>', 0]
                    ])->select(DB::raw('
                        invoices_details.quantity * if (items_unit.id > 0, items_unit.unit_value, 1) AS quantity, invoices_details.id, if (items_unit.id > 0, items_unit.unit_value, 1) AS unit_value
                    '))
                    ->first();


                    if ($invoice_details)
                    {
                        $invoice_details_updated = InvoiceDetails::find($invoice_details->id);
                        $temp_unit_value = $invoice_details->unit_value ?? 1;

                        if ($set_qty == ($invoice_details_updated->not_sold * $temp_unit_value))
                        {
                            // update and finish
                            $invoice_details_updated->not_sold = 0;
                            $invoice_details_updated->save();
                            $temp_set_qty = $set_qty;
                            $set_qty = 0;
                        }
                        else if ($set_qty - ($invoice_details_updated->not_sold * $temp_unit_value) > 0) {
                            // update & decrease set_qty
                            $temp_set_qty = ($invoice_details_updated->not_sold);
                            $set_qty -= $temp_set_qty;
                            $invoice_details_updated->not_sold = 0;
                            $invoice_details_updated->save();
                        }
                        else {
                            // update
                            $invoice_details_updated->not_sold -= $set_qty;
                            $invoice_details_updated->save();
                            $temp_set_qty = $set_qty ;
                            $set_qty = 0;
                        }
                        $actual_pay_price += ($invoice_details_updated->pay_price/ $temp_unit_value) * $temp_set_qty;
                    }
                    else {
                        $last_pay_price = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                            ->leftJoin('items_unit', function($q){
                                $q->on('items_unit.id', 'invoices_details.unit_id');
                                $q->where('items_unit.item_id', 'invoices_details.item_id');
                            })
                            ->where([
                                ['invoices_.shop_id', get_session_val('serial_id', 'Integer')],
                                ['invoices_.invoice_type', 0],
                                ['invoices_details.item_id', $item_id]
                            ])->select(DB::raw('
                                invoices_details.pay_price / if (items_unit.id > 0, items_unit.unit_value, 1) AS last_pay_price
                            '))
                            ->orderBy('invoices_.id', 'DESC')
                            ->limit(1)
                            ->value('last_pay_price')
                        ?? (
                            Item::leftJoin('items_unit', 'items_unit.id', 'items.unit_id')
                            ->where([
                                ['items.shop_id', get_session_val('serial_id', 'Integer')],
                                ['items.id', $item_id]
                            ])->select(DB::raw('
                                items.pay_price / if (items_unit.id > 0, items_unit.unit_value, 1) AS last_pay_price
                            '))
                            ->limit(1)
                            ->value('last_pay_price')
                                ?? 0
                        );

                        $actual_pay_price += ($last_pay_price) * $set_qty;
                        $set_qty = 0;
                    }

                }
            });

        }
        else if ($invoice_type == 'returned purchases')
        {
            /**
             * prioriry: purchase invoice with same client & price, item with same pay_price, invoice with same client, item pay_price
             * target:   decrease quantity & return purchase price
             */

            $unset_qty = $quantity;
            DB::transaction(function () use($item_id, $unset_qty, &$actual_pay_price, $client_id, $returned_pay_price){

                while( $unset_qty > 0 )
                {
                    $invoice_details = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                    ->leftJoin('items_unit', [
                        ['items_unit.id', 'invoices_details.unit_id'],
                        ['items_unit.item_id', 'invoices_details.item_id']
                    ])
                    ->where([
                        ['invoices_.shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoices_.client_id', $client_id],
                        ['invoices_details.item_id', $item_id],
                        ['invoices_details.pay_price', $returned_pay_price],
                        ['invoices_details.not_sold', '>', 0]
                    ])->select(DB::raw('
                        invoices_details.quantity * if (items_unit.id > 0, items_unit.unit_value, 1) AS quantity, if (items_unit.id > 0, items_unit.unit_value, 1) AS unit_value, invoices_details.id
                    '))->first();

                    $set_qty = 0;

                    if ($invoice_details)    // if i find invoice with same client & price
                    {
                        $updated_invoice_detasils = InvoiceDetails::find($invoice_details->id);
                        $updated_unit_value = $invoice_details->unit_value ?? 1;

                        if ($unset_qty <= ($updated_invoice_detasils->not_sold * $updated_unit_value))
                        {
                            $updated_invoice_detasils->not_sold -= ( $unset_qty / $updated_unit_value );
                            $set_qty = $unset_qty;
                            $unset_qty = 0;
                        }
                        else {   // not sold < target to be sit
                            $set_qty = ($updated_invoice_detasils->not_sold * $updated_unit_value);
                            $updated_invoice_detasils->not_sold = 0;
                            $unset_qty -= $set_qty;
                        }

                        $updated_invoice_detasils->save();
                        $actual_pay_price += ($updated_invoice_detasils->pay_price/ $updated_unit_value) * $set_qty;
                    }
                    else {
                        $invoice_details = Invoices::leftJoin('invoices_details', 'invoices_details.invoice_id', 'invoices_.id')
                        ->leftJoin('items_unit', function($q){
                            $q->on('items_unit.id', 'invoices_details.unit_id');
                            $q->where('items_unit.item_id', 'invoices_details.item_id');
                        })
                        ->where([
                            ['invoices_.invoice_type', 7],
                            ['invoices_details.item_id', $item_id],
                            ['invoices_details.not_sold', '>', '0'],
                            ['invoices_details.pay_price', $returned_pay_price],
                            ['invoices_.shop_id', get_session_val('serial_id', 'Integer')]
                            ])
                            ->select(DB::raw('invoices_details.not_sold * if (items_unit.id > 0, items_unit.unit_value, 1) AS not_sold, if (items_unit.id > 0, items_unit.unit_value, 1) AS unit_value, invoices_details.id'))
                        ->first();

                        if ($invoice_details)
                        {
                            $updated_invoice_details = InvoiceDetails::find($item_id);
                            $updated_unit_value = $invoice_details->unit_value;

                            if ($unset_qty <= ($updated_invoice_details->not_sold * $updated_unit_value))
                            {
                                $updated_invoice_details->not_sold -= ( $unset_qty / $updated_unit_value );
                                $set_qty = $unset_qty;
                                $unset_qty = 0;
                            }
                            else {   // not sold < target to be sit
                                $set_qty = ($updated_invoice_details->not_sold * $updated_unit_value);
                                $updated_invoice_details->not_sold = 0;
                                $unset_qty -= $set_qty;
                            }

                            $updated_invoice_details->save();
                            $actual_pay_price += ($updated_invoice_details->pay_price/ $updated_unit_value) * $set_qty;
                        }
                        else {
                            $invoice_details = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                            ->leftJoin('items_unit', [
                                ['items_unit.id', 'invoices_details.unit_id'],
                                ['items_unit.item_id', 'invoices_details.item_id']
                            ])
                            ->where([
                                ['invoices_.shop_id', get_session_val('serial_id', 'Integer')],
                                ['invoices_.client_id', $client_id],
                                ['invoices_details.item_id', $item_id],
                                ['invoices_details.not_sold', '>', 0]
                            ])->select(DB::raw('
                                invoices_details.quantity * if (items_unit.id > 0, items_unit.unit_value, 1) AS quantity, if (items_unit.id > 0, items_unit.unit_value, 1) AS unit_value, invoices_details.id
                            '))->first();

                            $set_qty = 0;

                            if ($invoice_details)    // if i find invoice with same client
                            {
                                $updated_invoice_detasils = InvoiceDetails::find($invoice_details->id);
                                $updated_unit_value = $invoice_details->unit_value ?? 1;

                                if ($unset_qty <= ($updated_invoice_detasils->not_sold * $updated_unit_value))
                                {
                                    $updated_invoice_detasils->not_sold -= ( $unset_qty / $updated_unit_value );
                                    $set_qty = $unset_qty;
                                    $unset_qty = 0;
                                }
                                else {   // not sold < target to be sit
                                    $set_qty = ($updated_invoice_detasils->not_sold * $updated_unit_value);
                                    $updated_invoice_detasils->not_sold = 0;
                                    $unset_qty -= $set_qty;
                                }

                                $updated_invoice_detasils->save();
                                $actual_pay_price += ($updated_invoice_detasils->pay_price/ $updated_unit_value) * $set_qty;
                            }
                            else {
                                $item_details = Item::leftJoin('items_unit', [
                                    ['items_unit.id', 'items.unit_id'],
                                    ['items_unit.item_id', 'items.id']
                                    ])->where([
                                        ['items.shop_id', get_session_val('serial_id', 'Integer')],
                                        ['items.id', $item_id],
                                    ])
                                    ->select(DB::raw('items.pay_price, if (items_unit.id > 0, items_unit.unit_value, 1) AS unit_value'))
                                    ->first();

                                if ($item_details)
                                {
                                    $updated_unit_value = $item_details->unit_value;
                                    $set_qty = $unset_qty;
                                    $actual_pay_price += ($item_details->pay_price/ $updated_unit_value) * $set_qty;
                                }
                                $unset_qty = 0;
                            }
                        }
                    }
                }

            });
        }
        else {   // returned sales
            // increase quantity
            $set_qty = $quantity;
            DB::transaction(function () use($item_id, $set_qty, &$actual_pay_price, $client_id, $returned_pay_price){
                $temp_set_qty = 0;

                $sold_invoice_details = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                ->leftJoin('items_unit', function($q){
                    $q->on('items_unit.id', 'invoices_details.unit_id');
                    $q->where('items_unit.item_id', 'invoices_details.item_id');
                })
                ->where([
                    ['invoices_.shop_id', get_session_val('serial_id', 'Integer')],
                    ['invoices_.invoice_type', 1],
                    ['invoices_details.item_id', $item_id],
                    ['invoices_.client_id', $client_id]
                ])->select(DB::raw('
                    invoices_details.quantity * if (items_unit.id > 0, items_unit.unit_value, 1) AS quantity, invoices_details.id, items_unit.unit_value,
                    ac_pay_price
                '))
                ->orderBy('invoices_.id', 'DESC')
                ->first();

                if ($sold_invoice_details)
                {
                    $invoice_details = Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                    ->leftJoin('items_unit', function($q){
                        $q->on('items_unit.id', 'invoices_details.unit_id');
                        $q->where('items_unit.item_id', 'invoices_details.item_id');
                    })
                    ->where([
                        ['invoices_.shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoices_.invoice_type', 0],
                        ['invoices_details.item_id', $item_id],
                        ['invoices_details.pay_price', $sold_invoice_details->ac_pay_price],
                        ['invoices_details.not_sold', '<', 'invoices_details.quantity']
                    ])->select(DB::raw('
                        invoices_details.quantity * if (items_unit.id > 0, items_unit.unit_value, 1) AS quantity, invoices_details.id, if (items_unit.id > 0, items_unit.unit_value, 1) AS unit_value,
                        ac_pay_price
                    '))
                    ->orderBy('invoices_.id', 'DESC')
                    ->first();

                    if ($invoice_details)
                    {
                        $invoice_details_updated = InvoiceDetails::find($invoice_details->id);
                        $temp_unit_value = $invoice_details->unit_value ?? 1;

                        if ($set_qty <= ( ($invoice_details_updated->quantity - $invoice_details_updated->not_sold) * $temp_unit_value))
                        {
                            // update and finish
                            $invoice_details_updated->not_sold += $set_qty;
                            $invoice_details_updated->save();
                            $temp_set_qty = $set_qty;
                            $set_qty = 0;
                        }
                        else {

                            // update & decrease set_qty
                            $temp_set_qty = ($invoice_details_updated->not_sold);
                            $set_qty -= $temp_set_qty;
                            $invoice_details_updated->not_sold = 0;
                            $invoice_details_updated->save();
                        }

                        $actual_pay_price += ($invoice_details_updated->pay_price/ $temp_unit_value) * $temp_set_qty;
                    }

                }

                // then decrease it from purchases invoices
                while($set_qty > 0)
                {
                    $invoice_details = Invoices::leftJoin('invoices_details', [
                        ['invoices_.id', 'invoices_details.invoice_id'],
                        ['invoices_details.not_sold', '<', 'invoices_details.quantity']
                    ])
                    ->leftJoin('items_unit', function($q){
                        $q->on('items_unit.id', 'invoices_details.unit_id');
                        $q->where('items_unit.item_id', 'invoices_details.item_id');
                    })
                    ->where([
                        ['invoices_.shop_id', get_session_val('serial_id', 'Integer')],
                        ['invoices_.invoice_type', 0],
                        ['invoices_details.item_id', $item_id]
                    ])->select(DB::raw('
                        invoices_details.quantity * if (items_unit.id > 0, items_unit.unit_value, 1) AS quantity, invoices_details.id, if (items_unit.id > 0, items_unit.unit_value, 1) AS unit_value
                    '))
                    ->first();

                    if ($invoice_details)
                    {
                        $invoice_details_updated = InvoiceDetails::find($invoice_details->id);
                        $temp_unit_value = $invoice_details->unit_value ?? 1;

                        if ($set_qty == (($invoice_details_updated->quantity - $invoice_details_updated->not_sold) * $temp_unit_value))
                        {
                            // update and finish
                            $invoice_details_updated->not_sold += $set_qty;
                            $invoice_details_updated->save();
                            $temp_set_qty = $set_qty;
                            $set_qty = 0;
                        }
                        else if ($set_qty - (($invoice_details_updated->quantity - $invoice_details_updated->not_sold) * $temp_unit_value) > 0) {

                            // update & decrease set_qty
                            $temp_set_qty = ($invoice_details_updated->quantity - $invoice_details_updated->not_sold);
                            $set_qty -= $temp_set_qty;
                            $invoice_details_updated->not_sold += $temp_set_qty;
                            $invoice_details_updated->save();
                        }
                        else {
                            // update
                            $invoice_details_updated->not_sold += $set_qty;
                            $invoice_details_updated->save();
                            $temp_set_qty = $set_qty ;
                            $set_qty = 0;
                        }
                        $actual_pay_price += ($invoice_details_updated->pay_price/ $temp_unit_value) * $temp_set_qty;
                    }
                    else {
                        $last_pay_price = Invoices::leftJoin('invoices_details', 'invoices_details.invoice_id', 'invoices_.id')
                                                ->leftJoin('items_unit', function($q){
                                                    $q->on('items_unit.id', 'invoices_details.unit_id');
                                                    $q->where('items_unit.item_id', 'invoices_details.item_id');
                                                })
                                                ->where([
                                                    ['invoices_.invoice_type', 7],
                                                    ['invoices_details.item_id', $item_id],
                                                    ['invoices_details.not_sold', '>', '0'],
                                                    ['invoices_details.pay_price', $returned_pay_price],
                                                    ['invoices_.shop_id', get_session_val('serial_id', 'Integer')]
                                                    ])
                                                ->select(DB::raw('
                                                invoices_details.pay_price / if (items_unit.id > 0, items_unit.unit_value, 1) AS last_pay_price
                                                '))
                                                ->limit(1)
                                                ->value('last_pay_price')
                                                ?? (
                                                        Invoices::leftJoin('invoices_details', 'invoices_.id', 'invoices_details.invoice_id')
                                                        ->leftJoin('items_unit', function($q){
                                                            $q->on('items_unit.id', 'invoices_details.unit_id');
                                                            $q->where('items_unit.item_id', 'invoices_details.item_id');
                                                        })
                                                        ->where([
                                                            ['invoices_.shop_id', get_session_val('serial_id', 'Integer')],
                                                            ['invoices_.invoice_type', 0],
                                                            ['invoices_details.item_id', $item_id]
                                                        ])->select(DB::raw('
                                                            invoices_details.pay_price / if (items_unit.id > 0, items_unit.unit_value, 1) AS last_pay_price
                                                        '))
                                                        ->orderBy('invoices_.id', 'DESC')
                                                        ->limit(1)
                                                        ->value('last_pay_price')
                                                        ??
                                                        Item::leftJoin('items_unit', 'items_unit.id', 'items.unit_id')
                                                        ->where([
                                                            ['items.shop_id', get_session_val('serial_id', 'Integer')],
                                                            ['items.id', $item_id]
                                                        ])->select(DB::raw('
                                                            items.pay_price / if (items_unit.id > 0, items_unit.unit_value, 1) AS last_pay_price
                                                        '))
                                                        ->limit(1)
                                                        ->value('last_pay_price')
                                                );

                        $actual_pay_price += ($last_pay_price) * $set_qty;
                        $set_qty = 0;
                    }

                }

            });
        }

        if ($quantity == 0 ) $quantity = 1;
        return ( ($actual_pay_price * $unit_value) / $quantity);   // unit price
    }

    function is_installed($invoice)
    {
        return Instalment::where(['shop_id'=> get_session_val('serial_id', 'integer'),'bill_id' => "$invoice->id"])->exists();
    }


    function maxInvoiceNumber($invoice_type, $store_id)
    {
        $today = today();
        $generate_by = get_session_val('billno_show', 'integer');
        $depend_on = get_session_val('no_bill_type', 'integer');
        $last_no = Invoices::select('bill_no')
                            ->where([ 'shop_id' => get_session_val('serial_id', 'Integer'), 'invoices_.invoice_type' => convertInvoiceTypeNameToNumber($invoice_type)])
                            ->when($generate_by == 2, function($q) use($today){
                                $q->whereDate('invoices_.invoice_date', $today->format('Y-m-d'));
                            })
                            ->when($generate_by == 3, function($q) use($today){
                                $q->where(DB::raw('YEAR(invoices_.invoice_date)'), $today->format('Y'));
                                $q->where(DB::raw('Month(invoices_.invoice_date)'), $today->format('m'));
                            })
                            ->when($generate_by == 4, function($q) use($today){
                                $q->where(DB::raw('YEAR(invoices_.invoice_date)'), $today->format('Y'));
                            })
                            ->when($depend_on == 1, function($q) use($store_id){
                                $q->where('invoices_.store_id', $store_id);
                            })->max('bill_no') ?? 0;
                            
        // if ($generate_by == 1) {
        //     $last_no = $last_no->orderBy('invoices_.id', 'DESC')->first()->bill_no ?? 0;
        // } else {
        //     $last_no = $last_no->max('bill_no') ?? 0;
        // }

        return $last_no + 1;
    }

    function convertInvoiceTypeNameToNumber($type)
    {
        $types = [
            'purchases' => 0,
            'sales' => 1,
            'purchases back' => 2,
            'sales back' => 3,
            'sales offers' => 4
        ];
        return $types[$type] ?? 1;
    }