<?php

use App\Models\User;
use App\Models\Zone;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Modules\Settings\Entities\Badrshop;
use Modules\Clients\Entities\Client;
use Modules\Invoices\Entities\InvoiceDetails;
use Modules\Invoices\Entities\Item;
use Modules\Invoices\Entities\Invoices;
use Modules\Item\Entities\Unit;
use Illuminate\Pagination\LengthAwarePaginator;
use Modules\Maintenance\Entities\MaintenanceOrderStatus;
use Modules\Clients\Entities\Country;

    function set_session($user = null)  :void
    {
        if (is_null($user))
        {
            $user = User::find(Auth::user()->id);
        }

        $user_id = $user->id;
        $shop_id = $user->shop_id;
        $user_permissions = 'user' . '_' . $user_id;

        Cache::put($shop_id, array(BadrShop::where('serial_id', $shop_id)->first())[0] );
        Cache::put($user_permissions, array(User::where('id', $user_id)->first())[0]);
    }

    function set_session_val($key, $value, $gate = null, $type = null)  :bool
    {
        if (! $gate)
        {
            if (Auth::user())
            {
                $gate = Auth::user()->shop_id;
            }
            else {
                Auth::logout();
            }
        }
        else if ($type)
        {
            $gate = $type . '_' . $gate;
        }

        if (Cache::has($gate) & isset(Cache::get($gate)[$key]))
        {
            $stored_val = Cache::get($gate);
            $stored_val[$key] = $value;
            return Cache::put($gate, $stored_val);
        }
        return false;
    }

    function set_session_arr_val($values, $gate, $type = null)   :bool
    {
        if ($type)
        {
            $gate = $type . '_' . $gate;
        }

        return Cache::put($gate, $values);
    }

    /*
     * $expected_return_value used in array & object cases
     */
    function get_session_val($key, $expected_return_type, $expected_return_key = null, $expected_return_value = null, $gate = null, $type = null)
    {
        if ($type)
        {
            $gate = $type . '_' . $gate;
        }
        else {
            if (Auth::user())
            {
                $gate = Auth::user()->shop_id;
            }
            else {
                Auth::logout();
            }
        }
        if (Cache::has($gate) & isset(Cache::get($gate)[$key]))
        {
            return Cache::get($gate)[$key];
        }
        return replace_null_with_expected($expected_return_type, $expected_return_key, $expected_return_value);
    }


    function replace_null_with_expected($expected_return_type, $expected_return_key = null, $expected_return_value = null)
    {
        switch (strtolower($expected_return_type)) {
            case 'string':
                return '';
                break;
            case 'integer':
                return 0;
                break;
            case 'double':
                return 0.0;
                break;
            case 'array':
                return [$expected_return_key => $expected_return_value];
                break;
            case 'object':
                return ((object)[$expected_return_key => $expected_return_value]);
                break;
            default:
                return null;
                break;
        }
    }






    function getSubCatOptions($id)
    {
        $item = \Modules\Item\Entities\Category::find($id);
        $subs = $item->subCategory();
        $options = '';
        if ($subs->count() > 0) {
            foreach ($subs as $sub) {
                $options .= '<option value="' . $sub->id . '" data-parent="' . $id . '">' . $sub->name . '</option>';
                $options .= getSubCatOptions($sub->id);
            }
        }
        return $options;
    }

    function getSubCatOptionsWithSelected($cat_id, $sale_unit)
    {
        $item = \Modules\Item\Entities\Category::find($cat_id);
        $subs = $item->subCategory();
        $options = '';
        if ($subs->count() > 0) {
            foreach ($subs as $sub) {
                if ($sub->id == $sale_unit) {
                    $options .= '<option selected value="' . $sub->id . '" data-parent="' . $cat_id . '">' . $sub->name . '</option>';
                } else {
                    $options .= '<option value="' . $sub->id . '" data-parent="' . $cat_id . '">' . $sub->name . '</option>';
                }
                $options .= getSubCatOptionsWithSelected($sub->id, $sale_unit);
            }
        }
        return $options;
    }

    /**
     * Custom Pagination.
     *
     * @param documents
     * @param perPage
     * @return Response
     */
function paginate($documents, $perPage, $totalCount = null)
{
    if ($totalCount == null && !empty($documents)) {
        $pageStart               = request('page', 1);
        $offSet                  = ($pageStart * $perPage) - $perPage;
        $documentsForCurrentPage = $documents->slice($offSet, $perPage);
        $totalCount              = $documents->count();
    } else {
        $documentsForCurrentPage = $documents;
    }

    return new LengthAwarePaginator(
        $documentsForCurrentPage,
        $totalCount,
        $perPage,
        Paginator::resolveCurrentPage(),
        ['path' => Paginator::resolveCurrentPath()]
    );
}

function setTimeZone() :void
{
    $zone = 'Africa/Cairo';
    if (auth()->check()){
        if (strlen(get_session_val('zone', 'string')) > 0)
            $zone = get_session_val('zone', 'string');
    }
    date_default_timezone_set($zone);
}

function getLogoPath()
{
    if (strlen(get_session_val('logo_path', 'string')) > 0)
    {
        return asset(get_session_val('logo_path', 'string'));
    }
    return "https://pbs.twimg.com/media/EdiVb4aUcAEOmUR.png";
}

function createDefaultInvoice()
{
    $client = Client::create([
        'client_name' => trans('default.default-model', ['model' => trans('default.client-supplier')]),
        'shop_id' => 0,
        'client_role' => 2
    ]);

    $unit = Unit::create([
        'name' => trans('default.default-model', ['model' => trans('default.unit')]),
        'shop_id' => 0
    ]);

    $item = Item::create([
        'item_name' => trans('default.default-model', ['model' => trans('default.item')]),
        'sale_price' => 200,
        'pay_price' => 200,
        'shop_id' => 0
    ]);

    $invoice = Invoices::create([
        'invoice_date' => date('Y-m-d h:i:s'),
        'transaction_date' => date('Y-m-d h:i:s'),
        'bill_total' => 100,
        'add_user' => 0,
        'client_id' => $client->id,
        'net_price' => 100,
        'payment' => 50,
        'bill_payment' => 50,
        'rest' => 50,
        'store_id' => 0,
        'bill_no' => 0,
        'notes' => 'ملاحظات فاتورة',
        'shop_id' => 0,
        'balance' => 0,
        'currency_id' => 0,
        'currency_value' => 'جنية',
        'invoice_type' => 0,
        'invoice_total_disc' => 0,
        'invoice_total_disc_type' => 1,
        'invoice_total_disc_status' => 0
    ]);

    InvoiceDetails::create([
        'invoice_id' => $invoice->id,
        'shop_id' => 0,
        'item_id' => $item->id,
        'quantity' => 1,
        'unit_id' => $unit->id,
        'pay_price' => 100,
        'sale_price' => 100,
        'expire_date' => date('Y-m-d'),
        'total_price' => 100,
        'basic_sale_price' => 100,
        'basic_pay_price' => 100,
        'not_sold' => 0,
        'ac_pay_price' => 100,
        'discount' => 0,
        'discount_type' => 1,
        'pay_vats_weight' => 0
    ]);

}

function createMaintenanceOrderDefaultStatuses($shop_id)
{
    MaintenanceOrderStatus::insert([
        [
            'name' => 'جديد انتظار التشخيص',
            'hold' => 1,
            'status' => 0,
            'view_order' => 1,
            'shop_id' => $shop_id
        ],
        [
            'name' => 'لم يتم الاصلاح',
            'hold' => 0,
            'status' => 1,
            'view_order' => 2,
            'shop_id' => $shop_id
        ],
        [
            'name' => 'في انتظار العميل بعد التشخيص',
            'hold' => 0,
            'status' => 2,
            'view_order' => 3,
            'shop_id' => $shop_id
        ],
        [
            'name' => 'موافقة العميل علي الصيانة',
            'hold' => 0,
            'status' => 9,
            'view_order' => 4,
            'shop_id' => $shop_id
        ],
        [
            'name' => 'رفض العميل للصيانة',
            'hold' => 0,
            'status' => 5,
            'view_order' => 5,
            'shop_id' => $shop_id
        ],
        [
            'name' => 'في الصيانة',
            'hold' => 0,
            'status' => 3,
            'view_order' => 6,
            'shop_id' => $shop_id
        ],
        [
            'name' => 'في انتظار قطع الغيار',
            'hold' => 0,
            'status' => 7,
            'view_order' => 7,
            'shop_id' => $shop_id
        ],
        [
            'name' => 'تم الاصلاح في انتظار التسليم',
            'hold' => 0,
            'status' => 8,
            'view_order' => 8,
            'shop_id' => $shop_id
        ],
        [
            'name' => 'مرتجع تحت الضمان',
            'hold' => 0,
            'status' => 6,
            'view_order' => 9,
            'shop_id' => $shop_id
        ],
        [
            'name' => 'تم التسليم للعميل و اصدار الفاتورة',
            'hold' => 1,
            'status' => 4,
            'view_order' => 10,
            'shop_id' => $shop_id
        ]
    ]);
}

function handleLanguage(){
    $shop = Badrshop::where('serial_id', get_session_val('serial_id', 'integer'))->first();
    $country_code = $shop ? getCountryCode($shop->country) : 'ar';
    app()->setlocale($country_code);
}

function getCountryCode($country_id){
    $country_code = Country::find($country_id)->iso_alpha2;
    return strtolower($country_code);
}


function getSupportDetails()
{
    $country = \App\Services\Location\GeoLocationController::index()->country ?? 'Egypt';

    if ($country == 'Egypt' || $country == 'invalid_ip') {
        $data = [
            'video' => asset('imgs/youtube.png'),
            'support_number' => '01061660342 - 0402970833',
            'whatsapp' => 'https://wa.link/kp457n',
        ];
    } else {
        $data = [
            'video' => asset('imgs/youtube.png'),
            'support_number' => '1111111 - 2222222',
            'whatsapp' => 'https://wa.link/kp457n',
        ];
    }
    return $data;
}
