@extends('layouts.app')

@section('content')
@if (! in_array(app()->getLocale(), ['ar', 'ksa']))
    <style>
        .text-right {
            text-align: left !important;
        }
        .login-form-group>span {
            right: unset !important;
            left: 20px !important;
        }
        .login-form-group {
            padding-right: unset;
            padding-left: 60px !important;
        }
        span.toggle-password {
            left: auto !important;
            right: 30px !important;
        }
    </style>
@endif


<div class="row ">
    <div class="col-md-8 offset-md-2">
        <h1 class="h3 text-center d-block d-sm-none"> @lang('login.title') </h1>

        @if (session('success'))
            <div class="alert alert-success">
                {{ session('success') }}
            </div>
        @endif

        @if (isset($errors) && $errors->any())
            <div class="alert alert-danger">
                <ul>
                    @foreach ($errors->all() as $error)
                        <li>{{ $error }}</li>
                    @endforeach
                </ul>
            </div>
        @endif

        <div class="col-md-6 d-block d-sm-none">
            <div class="logo text-center">
                <a href="#" title="@lang('login.badr')">
                    <img src=" {{ asset('imgs/logo.png') }}" alt="@lang('login.badr')" title="@lang('login.badr')" />
                </a>
            </div>
        </div>
        <div class="login-content">
            <h1 class="h3 text-center d-none d-sm-block">@lang('login.title')</h1>
            <div class="login-content__form">
                <form method="POST" action="{{ route('login') }}">
                    @csrf
                    <div class="login-content__form_top">
                        <div class="login-form-group">
                            <span>
                                <img src=" {{ asset('imgs/login/Profile.png') }}" alt="  " title="  " />
                            </span>
                            <label for="user_name">@lang('login.user-or-email')</label>
                            <input type="text" class="form-control" name="user_name" id="user_name" placeholder="@lang('login.user-or-email')"
                                value="{{ old('user_name', env('LOGIN_NAME')) }}" autofocus required>
                        </div>

                        <div class="login-form-group">
                            <span>
                                <img src=" {{ asset('imgs/login/Group117.png') }}" alt="  " title="  " />
                            </span>
                            <label for="password-field">@lang('login.password')</label>
                            <input type="password" class="form-control" name="password" id="password" placeholder="@lang('login.password')"
                                   value="{{ env('LOGIN_PASS') }}" required>
                            <span class="fa fa-eye-slash toggle-password" id="togglePassword"></span>
                        </div>
                    </div>

                    <div class="form-group mt-4">
                        <div class="row">
                            <div class="col-md-6 text-right">
                                <div class="custom-control custom-checkbox">
                                    <input type="checkbox" class="custom-control-input-bg" id="customCheck1" name="remember">
                                    <label class="custom-control-label-bg" for="customCheck1">
                                        <strong>@lang('login.remember')</strong>
                                    </label>
                                </div>
                            </div>
                            <div class="col-md-6 text-left d-sm-block">
                                {{-- <a href="#" class="text-danger hidden d-none">
                                    <i class="fas fa-exclamation-circle text-danger" aria-hidden="true"></i>
                                    <strong>
                                        نسيت كلمه المرور
                                    </strong>
                                </a> --}}

                                @if ( function_exists('checkDatabaseConnection') && !checkDatabaseConnection() && Route::has('installer.index'))
                                    <a href="{{ route('installer.index') }}" class="text-info">
                                        <strong>
                                            @lang('login.install')
                                            <i class="fa fa-download text-info" aria-hidden="true"></i>
                                        </strong>
                                    </a>
                                @endif
                            </div>
                        </div>
                    </div>
                    <div class="form-group row mt-5">
                        <div class="col-md-6 col-12">
                            <button type="submit"
                                class="btn btn-primary d-block w-100 btn-xlg rounded-10 auth-submit">
                                <i class="fa  fa-angle-right"></i>
                                @lang('login.login')
                            </button>
                        </div>

                        <div class="col-md-6 col-12">
                            @if(Route::has('create_account'))
                                <a href="{{ route('create_account') }}" class="btn btn-secondary d-block w-100 btn-xlg rounded-10 new-account">
                                    @lang('login.create-new-test')
                                </a>
                            @endif

                            @if (Route::has('register'))
                                <a href="{{ route('create_account') }}" class="btn btn-secondary d-block w-100 btn-xlg rounded-10 new-account">
                                    @lang('login.create-new')
                                </a>
                            @endif
                        </div>
                    </div>

                    {{-- <div class="col-12  d-block  d-sm-none d-flex justify-content-center hidden">
                        <a href="#" class="text-danger">
                            <strong>
                                @lang('login.forget-password')
                                <i class="fa fa-info-circle fa-x" aria-hidden="true"></i>
                            </strong>
                        </a>
                    </div> --}}
                </form>
            </div>
        </div>
    </div>
    <div class="col-md-12 mt-5">
        <div class="row">
            @php $details = getSupportDetails(); @endphp

            <div class="col-md-3">
                <div class="explanation">
                    <a href="{{ $details['whatsapp'] }}" target="_blank" title="@lang('login.goto-whats')">
                        <img src="{{asset('imgs/whatsapp_logo.png')}}" style="width: 49px;">
                        <p>
                            {{-- <i class="fa-brands fa-whatsapp" style="color: #00b66c !important"></i>     --}}
                            @lang('login.contact-us')
                        </p>
                    </a>
                </div>
            </div>

            <div class="col-md-6">
                <div class="text-center for-contact">
                    <p>@lang('login.contact-to-support')</p>
                    <p>{{ $details['support_number'] }}</p>
                </div>
            </div>

            <div class="col-md-3">
                <div class="explanation">
                    <a href="">
                        <img src="{{ $details['video'] }}" alt="">
                        <p> @lang('login.show-video') </p>
                    </a>

                </div>
            </div>
        </div>
    </div>
</div>
@endsection
